'use strict';



;define("ghost-admin/adapters/application", ["exports", "ghost-admin/adapters/embedded-relation-adapter"], function (_exports, _embeddedRelationAdapter) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _embeddedRelationAdapter.default.extend({
    shouldBackgroundReloadRecord() {
      return false;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/base", ["exports", "ember-ajax/mixins/ajax-support", "ember-simple-auth/mixins/data-adapter-mixin", "ember-data/adapters/rest", "ghost-admin/utils/ghost-paths"], function (_exports, _ajaxSupport, _dataAdapterMixin, _rest, _ghostPaths) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _rest.default.extend(_dataAdapterMixin.default, _ajaxSupport.default, {
    host: window.location.origin,
    namespace: (0, _ghostPaths.default)().apiRoot.slice(1),
    session: Ember.inject.service(),

    shouldBackgroundReloadRecord() {
      return false;
    },

    authorize()
    /*xhr*/
    {// noop - we're using server-side session cookies
    },

    query(store, type, query) {
      let id;

      if (query.id) {
        id = query.id;
        delete query.id;
      }

      return this.ajax(this.buildURL(type.modelName, id), 'GET', {
        data: query
      });
    },

    buildURL() {
      // Ensure trailing slashes
      let url = this._super(...arguments);

      if (url.slice(-1) !== '/') {
        url += '/';
      }

      return url;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/embedded-relation-adapter", ["exports", "ghost-admin/adapters/base"], function (_exports, _base) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // EmbeddedRelationAdapter will augment the query object in calls made to
  // DS.Store#findRecord, findAll, query, and queryRecord with the correct "includes"
  // (?include=relatedType) by introspecting on the provided subclass of the DS.Model.
  // In cases where there is no query object (DS.Model#save, or simple finds) the URL
  // that is built will be augmented with ?include=... where appropriate.
  //
  // Example:
  // If a model has an embedded hasMany relation, the related type will be included:
  // roles: DS.hasMany('role', { embedded: 'always' }) => ?include=roles
  var _default = _base.default.extend({
    find(store, type, id, snapshot) {
      return this.ajax(this.buildIncludeURL(store, type.modelName, id, snapshot, 'find'), 'GET');
    },

    findRecord(store, type, id, snapshot) {
      return this.ajax(this.buildIncludeURL(store, type.modelName, id, snapshot, 'findRecord'), 'GET');
    },

    findAll(store, type, sinceToken) {
      let query, url;

      if (sinceToken) {
        query = {
          since: sinceToken
        };
      }

      url = this.buildIncludeURL(store, type.modelName, null, null, 'findAll');
      return this.ajax(url, 'GET', {
        data: query
      });
    },

    query(store, type, query) {
      return this._super(store, type, this.buildQuery(store, type.modelName, query));
    },

    queryRecord(store, type, query) {
      return this._super(store, type, this.buildQuery(store, type.modelName, query));
    },

    createRecord(store, type, snapshot) {
      return this.saveRecord(store, type, snapshot, {
        method: 'POST'
      }, 'createRecord');
    },

    updateRecord(store, type, snapshot) {
      let options = {
        method: 'PUT',
        id: Ember.get(snapshot, 'id')
      };
      return this.saveRecord(store, type, snapshot, options, 'updateRecord');
    },

    saveRecord(store, type, snapshot, options, requestType) {
      let _options = options || {};

      let url = this.buildIncludeURL(store, type.modelName, _options.id, snapshot, requestType);
      let payload = this.preparePayload(store, type, snapshot);
      return this.ajax(url, _options.method, payload);
    },

    preparePayload(store, type, snapshot) {
      let serializer = store.serializerFor(type.modelName);
      let payload = {};
      serializer.serializeIntoHash(payload, type, snapshot);
      return {
        data: payload
      };
    },

    buildIncludeURL(store, modelName, id, snapshot, requestType, query) {
      let includes = this.getEmbeddedRelations(store, modelName);
      let url = this.buildURL(modelName, id, snapshot, requestType, query);

      if (includes.length) {
        url += "?include=".concat(includes.map(Ember.String.underscore).join(','));
      }

      return url;
    },

    buildQuery(store, modelName, options) {
      let deDupe = {};
      let toInclude = this.getEmbeddedRelations(store, modelName);
      let query = options || {};

      if (toInclude.length) {
        // If this is a find by id, build a query object and attach the includes
        if (typeof options === 'string' || typeof options === 'number') {
          query = {};
          query.id = options;
          query.include = toInclude.map(Ember.String.underscore).join(',');
        } else if (typeof options === 'object' || Ember.isNone(options)) {
          // If this is a find all (no existing query object) build one and attach
          // the includes.
          // If this is a find with an existing query object then merge the includes
          // into the existing object. Existing properties and includes are preserved.
          query = query || {};
          toInclude = toInclude.concat(query.include ? query.include.split(',') : []);
          toInclude.forEach(include => {
            deDupe[include] = true;
          });
          query.include = Object.keys(deDupe).join(',');
        }
      }

      return query;
    },

    getEmbeddedRelations(store, modelName) {
      let model = store.modelFor(modelName);
      let ret = [];
      let embedded = []; // Iterate through the model's relationships and build a list
      // of those that need to be pulled in via "include" from the API

      model.eachRelationship((name, meta) => {
        if (meta.kind === 'hasMany' && Object.prototype.hasOwnProperty.call(meta.options, 'embedded') && meta.options.embedded === 'always') {
          ret.push(name);
          embedded.push([name, meta.type]);
        }
      });
      embedded.forEach((_ref) => {
        let [relName, modelName] = _ref;
        this.getEmbeddedRelations(store, modelName).forEach(name => {
          ret.push("".concat(relName, ".").concat(name));
        });
      });
      return ret;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/page", ["exports", "ghost-admin/adapters/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    // posts and pages now include everything by default
    buildIncludeURL(store, modelName, id, snapshot, requestType, query) {
      return this.buildURL(modelName, id, snapshot, requestType, query);
    },

    buildQuery(store, modelName, options) {
      return options;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/post", ["exports", "ghost-admin/adapters/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    // posts and pages now include everything by default
    buildIncludeURL(store, modelName, id, snapshot, requestType, query) {
      return this.buildURL(modelName, id, snapshot, requestType, query);
    },

    buildQuery(store, modelName, options) {
      return options;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/setting", ["exports", "ghost-admin/adapters/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    updateRecord(store, type, record) {
      let data = {};
      let serializer = store.serializerFor(type.modelName); // remove the fake id that we added onto the model.

      delete record.id; // use the SettingSerializer to transform the model back into
      // an array of settings objects like the API expects

      serializer.serializeIntoHash(data, type, record); // use the ApplicationAdapter's buildURL method but do not
      // pass in an id.

      return this.ajax(this.buildURL(type.modelName), 'PUT', {
        data
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/tag", ["exports", "ghost-admin/adapters/application", "ghost-admin/mixins/slug-url"], function (_exports, _application, _slugUrl) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend(_slugUrl.default);

  _exports.default = _default;
});
;define("ghost-admin/adapters/theme", ["exports", "ghost-admin/adapters/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    activate(model) {
      let url = "".concat(this.buildURL('theme', model.get('id')), "activate/");
      return this.ajax(url, 'PUT', {
        data: {}
      }).then(data => {
        this.store.pushPayload(data);
        return model;
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/adapters/user", ["exports", "ghost-admin/adapters/application", "ghost-admin/mixins/id-url", "ghost-admin/mixins/slug-url"], function (_exports, _application, _idUrl, _slugUrl) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend(_slugUrl.default, _idUrl.default, {
    queryRecord(store, type, query) {
      if (!query || query.id !== 'me') {
        return this._super(...arguments);
      }

      let url = this.buildURL(type.modelName, 'me', null, 'findRecord');
      return this.ajax(url, 'GET', {
        data: {
          include: 'roles'
        }
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/app", ["exports", "ghost-admin/utils/link-component", "ghost-admin/utils/route", "ghost-admin/resolver", "ghost-admin/config/environment", "ember-load-initializers"], function (_exports, _linkComponent, _route, _resolver, _environment, _emberLoadInitializers) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const App = Ember.Application.extend({
    Resolver: _resolver.default,
    modulePrefix: _environment.default.modulePrefix,
    podModulePrefix: _environment.default.podModulePrefix,
    // eslint-disable-next-line
    customEvents: {
      touchstart: null,
      touchmove: null,
      touchend: null,
      touchcancel: null
    }
  }); // TODO: remove once the validations refactor is complete
  // eslint-disable-next-line

  Ember.Debug.registerWarnHandler((message, options, next) => {
    let skip = ['ds.errors.add', 'ds.errors.remove', 'ds.errors.clear'];

    if (skip.includes(options.id)) {
      return;
    }

    next(message, options);
  });
  (0, _emberLoadInitializers.default)(App, _environment.default.modulePrefix);
  var _default = App;
  _exports.default = _default;
});
;define("ghost-admin/authenticators/cookie", ["exports", "ember-simple-auth/authenticators/base"], function (_exports, _base) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.extend({
    ajax: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    sessionEndpoint: Ember.computed('ghostPaths.apiRoot', function () {
      return "".concat(this.ghostPaths.apiRoot, "/session");
    }),
    restore: function restore() {
      return Ember.RSVP.resolve();
    },

    authenticate(identification, password) {
      const data = {
        username: identification,
        password
      };
      const options = {
        data,
        contentType: 'application/json;charset=utf-8',
        // ember-ajax will try and parse the response as JSON if not explicitly set
        dataType: 'text'
      };
      return this.ajax.post(this.sessionEndpoint, options);
    },

    invalidate() {
      // if we're invalidating because of a 401 we can end up in an infinite
      // loop if we then try to perform a DELETE /session/ request
      // TODO: find a more elegant way to handle this
      if (this.ajax.skipSessionDeletion) {
        this.ajax.skipSessionDeletion = false;
        return Ember.RSVP.resolve();
      }

      return this.ajax.del(this.sessionEndpoint);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/-lf-get-outlet-state", ["exports", "liquid-fire/components/-lf-get-outlet-state"], function (_exports, _lfGetOutletState) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _lfGetOutletState.default;
    }
  });
});
;define("ghost-admin/components/aspect-ratio-box", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ratio: '1/1',
    base: 'height',
    isResizing: true,
    _ratio: 1,

    init() {
      this._super(...arguments);

      this._onResizeHandler = () => {
        Ember.run.debounce(this, this._resize, 200);
      };
    },

    didReceiveAttrs() {
      (true && !(this.ratio.match(/\d+\/\d+/)) && Ember.assert('{{aspect-ratio-box}} requires a `ratio` property in the format `"16/9"`', this.ratio.match(/\d+\/\d+/)));
      this._ratio = this.ratio.split('/').reduce((prev, curr) => prev / curr);
    },

    didInsertElement() {
      this._resize();

      window.addEventListener('resize', this._onResizeHandler);
    },

    willDestroyElement() {
      this._super(...arguments);

      window.removeEventListener('resize', this._onResizeHandler);
    },

    _resize() {
      this.set('isResizing', true);
      Ember.run.schedule('afterRender', this, function () {
        if (this.base === 'height') {
          this.element.style.width = "".concat(this.element.clientHeight * this._ratio, "px");
        } else {
          this.element.style.height = "".concat(this.element.clientWidth * this._ratio, "px");
        }

        this.set('isResizing', false);
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/basic-dropdown", ["exports", "ember-basic-dropdown/components/basic-dropdown"], function (_exports, _basicDropdown) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _basicDropdown.default;
    }
  });
});
;define("ghost-admin/components/basic-dropdown/content-element", ["exports", "ember-basic-dropdown/components/basic-dropdown/content-element"], function (_exports, _contentElement) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _contentElement.default;
    }
  });
});
;define("ghost-admin/components/basic-dropdown/content", ["exports", "ember-basic-dropdown/components/basic-dropdown/content"], function (_exports, _content) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _content.default;
    }
  });
});
;define("ghost-admin/components/basic-dropdown/trigger", ["exports", "ember-basic-dropdown/components/basic-dropdown/trigger"], function (_exports, _trigger) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _trigger.default;
    }
  });
});
;define("ghost-admin/components/draggable-object-target", ["exports", "ember-drag-drop/components/draggable-object-target"], function (_exports, _draggableObjectTarget) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _draggableObjectTarget.default;
  _exports.default = _default;
});
;define("ghost-admin/components/draggable-object", ["exports", "ember-drag-drop/components/draggable-object"], function (_exports, _draggableObject) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _draggableObject.default;
  _exports.default = _default;
});
;define("ghost-admin/components/ember-load-remover", ["exports", "ember-load/components/ember-load-remover"], function (_exports, _emberLoadRemover) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberLoadRemover.default;
    }
  });
});
;define("ghost-admin/components/gh-activating-list-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    active: false,
    classNameBindings: ['active'],
    linkClasses: null,
    tagName: 'li',
    actions: {
      setActive(value) {
        Ember.run.schedule('afterRender', this, function () {
          this.set('active', value);
        });
      }

    },

    click() {
      this.$('a').blur();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-alert", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    notifications: Ember.inject.service(),
    classNameBindings: ['typeClass'],
    classNames: ['gh-alert'],
    tagName: 'article',
    typeClass: Ember.computed('message.type', function () {
      let type = this.get('message.type');
      let classes = '';
      let typeMapping;
      typeMapping = {
        success: 'green',
        error: 'red',
        warn: 'blue',
        info: 'blue'
      };

      if (typeMapping[type] !== undefined) {
        classes += "gh-alert-".concat(typeMapping[type]);
      }

      return classes;
    }),
    actions: {
      closeNotification() {
        this.notifications.closeNotification(this.message);
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-alerts", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    notifications: Ember.inject.service(),
    classNames: 'gh-alerts',
    tagName: 'aside',
    messages: Ember.computed.alias('notifications.alerts')
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-app", ["exports", "jquery"], function (_exports, _jquery) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    classNames: ['gh-app'],
    showSettingsMenu: false,

    didReceiveAttrs() {
      this._super(...arguments);

      let showSettingsMenu = this.showSettingsMenu;
      (0, _jquery.default)('body').toggleClass('settings-menu-expanded', showSettingsMenu);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-basic-dropdown", ["exports", "ember-basic-dropdown/components/basic-dropdown", "ember-basic-dropdown/templates/components/basic-dropdown"], function (_exports, _basicDropdown, _basicDropdown2) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _basicDropdown.default.extend({
    dropdown: Ember.inject.service(),
    layout: _basicDropdown2.default,

    didInsertElement() {
      this._super(...arguments);

      this.dropdown.on('close', this, this.close);
    },

    willDestroyElement() {
      this._super(...arguments);

      this.dropdown.off('close', this, this.close);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-blog-url", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    config: Ember.inject.service(),
    tagName: ''
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-cm-editor", ["exports", "ghost-admin/utils/bound-one-way", "ember-concurrency"], function (_exports, _boundOneWay, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const CmEditorComponent = Ember.Component.extend({
    lazyLoader: Ember.inject.service(),
    classNameBindings: ['isFocused:focus'],
    textareaClass: '',
    isFocused: false,
    // options for the editor
    autofocus: false,
    indentUnit: 4,
    lineNumbers: true,
    lineWrapping: false,
    mode: 'htmlmixed',
    theme: 'xq-light',
    _editor: null,
    // reference to CodeMirror editor
    // Allowed actions
    'focus-in': () => {},
    update: () => {},
    _value: (0, _boundOneWay.default)('value'),

    // make sure a value exists
    didReceiveAttrs() {
      if (this._value === null || undefined) {
        this.set('_value', '');
      }

      if (this.mode !== this._lastMode && this._editor) {
        this._editor.setOption('mode', this.mode);
      }

      this._lastMode = this.mode;
    },

    didInsertElement() {
      this._super(...arguments);

      this.initCodeMirror.perform();
    },

    willDestroyElement() {
      this._super(...arguments); // Ensure the editor exists before trying to destroy it. This fixes
      // an error that occurs if codemirror hasn't finished loading before
      // the component is destroyed.


      if (this._editor) {
        let editor = this._editor.getWrapperElement();

        editor.parentNode.removeChild(editor);
        this._editor = null;
      }
    },

    actions: {
      updateFromTextarea(value) {
        this.update(value);
      }

    },
    initCodeMirror: (0, _emberConcurrency.task)(function* () {
      let loader = this.lazyLoader;
      yield loader.loadScript('codemirror', 'assets/codemirror/codemirror.js');
      Ember.run.scheduleOnce('afterRender', this, this._initCodeMirror);
    }),

    _initCodeMirror() {
      let options = this.getProperties('lineNumbers', 'lineWrapping', 'indentUnit', 'mode', 'theme', 'autofocus');
      Ember.assign(options, {
        value: this._value
      });
      let textarea = this.element.querySelector('textarea');

      if (textarea && textarea === document.activeElement) {
        options.autofocus = true;
      }

      this._editor = new CodeMirror.fromTextArea(textarea, options); // by default CodeMirror will place the cursor at the beginning of the
      // content, it makes more sense for the cursor to be at the end

      if (options.autofocus) {
        this._editor.setCursor(this._editor.lineCount(), 0);
      } // events


      this._setupCodeMirrorEventHandler('focus', this, this._focus);

      this._setupCodeMirrorEventHandler('blur', this, this._blur);

      this._setupCodeMirrorEventHandler('change', this, this._update);
    },

    _setupCodeMirrorEventHandler(event, target, method) {
      let callback = Ember.run.bind(target, method);

      this._editor.on(event, callback);

      this.one('willDestroyElement', this, function () {
        this._editor.off(event, callback);
      });
    },

    _update(codeMirror, changeObj) {
      Ember.run.once(this, this.update, codeMirror.getValue(), codeMirror, changeObj);
    },

    _focus(codeMirror, event) {
      this.set('isFocused', true);
      Ember.run.once(this, this['focus-in'], codeMirror.getValue(), codeMirror, event);
    },

    _blur()
    /* codeMirror, event */
    {
      this.set('isFocused', false);
    }

  });
  CmEditorComponent.reopenClass({
    positionalParams: ['value']
  });
  var _default = CmEditorComponent;
  _exports.default = _default;
});
;define("ghost-admin/components/gh-content-cover", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ui: Ember.inject.service(),
    classNames: ['content-cover'],

    click() {
      this.ui.closeMenus();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-date-time-picker", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    settings: Ember.inject.service(),
    tagName: '',
    date: '',
    time: '',
    errors: null,
    dateErrorProperty: null,
    timeErrorProperty: null,
    _time: '',
    _previousTime: '',
    _minDate: null,
    _maxDate: null,
    blogTimezone: Ember.computed.reads('settings.activeTimezone'),
    hasError: Ember.computed.or('dateError', 'timeError'),
    timezone: Ember.computed('blogTimezone', function () {
      let blogTimezone = this.blogTimezone;
      return _moment.default.utc().tz(blogTimezone).format('z');
    }),
    dateError: Ember.computed('errors.[]', 'dateErrorProperty', function () {
      let errors = this.errors;
      let property = this.dateErrorProperty;

      if (errors && !Ember.isEmpty(errors.errorsFor(property))) {
        return errors.errorsFor(property).get('firstObject').message;
      }
    }),
    timeError: Ember.computed('errors.[]', 'timeErrorProperty', function () {
      let errors = this.errors;
      let property = this.timeErrorProperty;

      if (errors && !Ember.isEmpty(errors.errorsFor(property))) {
        return errors.errorsFor(property).get('firstObject').message;
      }
    }),

    didReceiveAttrs() {
      let date = this.date;
      let time = this.time;
      let minDate = this.minDate;
      let maxDate = this.maxDate;
      let blogTimezone = this.blogTimezone;

      if (!Ember.isBlank(date)) {
        this.set('_date', (0, _moment.default)(date));
      } else {
        this.set('_date', (0, _moment.default)().tz(blogTimezone));
      }

      if (Ember.isBlank(time)) {
        this.set('_time', this._date.format('HH:mm'));
      } else {
        this.set('_time', this.time);
      }

      this.set('_previousTime', this._time); // unless min/max date is at midnight moment will diable that day

      if (minDate === 'now') {
        this.set('_minDate', (0, _moment.default)((0, _moment.default)().format('YYYY-MM-DD')));
      } else if (!Ember.isBlank(minDate)) {
        this.set('_minDate', (0, _moment.default)((0, _moment.default)(minDate).format('YYYY-MM-DD')));
      } else {
        this.set('_minDate', null);
      }

      if (maxDate === 'now') {
        this.set('_maxDate', (0, _moment.default)((0, _moment.default)().format('YYYY-MM-DD')));
      } else if (!Ember.isBlank(maxDate)) {
        this.set('_maxDate', (0, _moment.default)((0, _moment.default)(maxDate).format('YYYY-MM-DD')));
      } else {
        this.set('_maxDate', null);
      }
    },

    actions: {
      // if date or time is set and the other property is blank set that too
      // so that we don't get "can't be blank" errors
      setDate(date) {
        if (date !== this._date) {
          this.setDate(date);

          if (Ember.isBlank(this.time)) {
            this.setTime(this._time);
          }
        }
      },

      setTime(time) {
        if (time.match(/^\d:\d\d$/)) {
          time = "0".concat(time);
        }

        if (time !== this._previousTime) {
          this.setTime(time);
          this.set('_previousTime', time);

          if (Ember.isBlank(this.date)) {
            this.setDate(this._date);
          }
        }
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-download-count", ["exports", "ghost-admin/config/environment", "ember-concurrency"], function (_exports, _environment, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ajax: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    tagName: '',
    count: '',

    didInsertElement() {
      this._poll.perform();
    },

    _poll: (0, _emberConcurrency.task)(function* () {
      let url = this.get('ghostPaths.count');
      let pattern = /(-?\d+)(\d{3})/;

      try {
        let data = yield this.ajax.request(url);
        let count = data.count.toString();

        while (pattern.test(count)) {
          count = count.replace(pattern, '$1,$2');
        }

        this.set('count', count);

        if (_environment.default.environment !== 'test') {
          yield (0, _emberConcurrency.timeout)(2000);

          this._poll.perform();
        }
      } catch (e) {// no-op - we don't want to create noise for a failed download count
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-dropdown-button", ["exports", "ghost-admin/mixins/dropdown-mixin"], function (_exports, _dropdownMixin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend(_dropdownMixin.default, {
    dropdown: Ember.inject.service(),
    tagName: 'button',
    attributeBindings: ['href', 'role'],
    role: 'button',
    // matches with the dropdown this button toggles
    dropdownName: null,

    // Notify dropdown service this dropdown should be toggled
    click(event) {
      this._super(event);

      this.dropdown.toggleDropdown(this.dropdownName, this);

      if (this.tagName === 'a') {
        return false;
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-dropdown", ["exports", "ghost-admin/mixins/dropdown-mixin"], function (_exports, _dropdownMixin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend(_dropdownMixin.default, {
    dropdown: Ember.inject.service(),
    classNames: 'dropdown',
    classNameBindings: ['fadeIn:fade-in-scale:fade-out', 'isOpen:open:closed'],
    name: null,
    closeOnClick: false,
    // Helps track the user re-opening the menu while it's fading out.
    closing: false,
    // Helps track whether the dropdown is open or closes, or in a transition to either
    isOpen: false,
    // Managed the toggle between the fade-in and fade-out classes
    fadeIn: Ember.computed('isOpen', 'closing', function () {
      return this.isOpen && !this.closing;
    }),

    didInsertElement() {
      let dropdownService = this.dropdown;

      this._super(...arguments);

      dropdownService.on('close', this, this.close);
      dropdownService.on('toggle', this, this.toggle);
    },

    willDestroyElement() {
      let dropdownService = this.dropdown;

      this._super(...arguments);

      dropdownService.off('close', this, this.close);
      dropdownService.off('toggle', this, this.toggle);
    },

    open() {
      this.set('isOpen', true);
      this.set('closing', false);
      this.set('button.isOpen', true);
    },

    close() {
      this.set('closing', true);

      if (this.button) {
        this.set('button.isOpen', false);
      }

      this.$().on('animationend webkitAnimationEnd oanimationend MSAnimationEnd', event => {
        if (event.originalEvent.animationName === 'fade-out') {
          Ember.run(this, function () {
            if (this.closing) {
              this.set('isOpen', false);
              this.set('closing', false);
            }
          });
        }
      });
    },

    // Called by the dropdown service when any dropdown button is clicked.
    toggle(options) {
      let isClosing = this.closing;
      let isOpen = this.isOpen;
      let name = this.name;
      let targetDropdownName = options.target;
      let button = this.button;

      if (name === targetDropdownName && (!isOpen || isClosing)) {
        if (!button) {
          button = options.button;
          this.set('button', button);
        }

        this.open();
      } else if (isOpen) {
        this.close();
      }
    },

    click(event) {
      this._super(event);

      if (this.closeOnClick) {
        return this.close();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-editor-post-status", ["exports", "ghost-admin/config/environment", "ember-concurrency"], function (_exports, _environment, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    post: null,
    isSaving: false,
    'data-test-editor-post-status': true,
    _isSaving: false,
    isNew: Ember.computed.reads('post.isNew'),
    isScheduled: Ember.computed.reads('post.isScheduled'),
    isPublished: Ember.computed('post.{isPublished,pastScheduledTime}', function () {
      let isPublished = this.get('post.isPublished');
      let pastScheduledTime = this.get('post.pastScheduledTime');
      return isPublished || pastScheduledTime;
    }),

    // isSaving will only be true briefly whilst the post is saving,
    // we want to ensure that the "Saving..." message is shown for at least
    // a few seconds so that it's noticeable
    didReceiveAttrs() {
      if (this.isSaving) {
        this.showSavingMessage.perform();
      }
    },

    showSavingMessage: (0, _emberConcurrency.task)(function* () {
      this.set('_isSaving', true);
      yield (0, _emberConcurrency.timeout)(_environment.default.environment === 'test' ? 0 : 3000);
      this.set('_isSaving', false);
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-editor", ["exports", "ghost-admin/components/gh-image-uploader"], function (_exports, _ghImageUploader) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ui: Ember.inject.service(),
    classNameBindings: ['isDraggedOver:-drag-over', 'isFullScreen:gh-editor-fullscreen', 'isPreview:gh-editor-preview'],
    // Internal attributes
    droppedFiles: null,
    headerClass: '',
    headerHeight: 0,
    imageExtensions: _ghImageUploader.IMAGE_EXTENSIONS,
    imageMimeTypes: _ghImageUploader.IMAGE_MIME_TYPES,
    isDraggedOver: false,
    isFullScreen: false,
    isSplitScreen: false,
    uploadedImageUrls: null,
    // Private
    _dragCounter: 0,
    _onResizeHandler: null,
    _viewActionsWidth: 190,

    init() {
      this._super(...arguments);

      this._onResizeHandler = evt => {
        Ember.run.debounce(this, this._setHeaderClass, evt, 100);
      };
    },

    didInsertElement() {
      this._super(...arguments);

      window.addEventListener('resize', this._onResizeHandler);

      this._setHeaderClass();
    },

    willDestroyElement() {
      this._super(...arguments);

      window.removeEventListener('resize', this._onResizeHandler);
    },

    actions: {
      toggleFullScreen(isFullScreen) {
        this.set('isFullScreen', isFullScreen);
        this.ui.set('isFullScreen', isFullScreen);
        Ember.run.scheduleOnce('afterRender', this, this._setHeaderClass);
      },

      togglePreview(isPreview) {
        this.set('isPreview', isPreview);
      },

      toggleSplitScreen(isSplitScreen) {
        this.set('isSplitScreen', isSplitScreen);
        Ember.run.scheduleOnce('afterRender', this, this._setHeaderClass);
      },

      uploadImages(fileList, resetInput) {
        // convert FileList to an array so that resetting the input doesn't
        // clear the file references before upload actions can be triggered
        let files = Array.from(fileList);
        this.set('droppedFiles', files);
        resetInput();
      },

      uploadComplete(uploads) {
        this.set('uploadedImageUrls', uploads.mapBy('url'));
        this.set('droppedFiles', null);
      },

      uploadCancelled() {
        this.set('droppedFiles', null);
      }

    },

    _setHeaderClass() {
      let editorTitle = this.element.querySelector('.gh-editor-title, .kg-title-input');
      let smallHeaderClass = 'gh-editor-header-small';
      let newHeaderClass = '';
      this._editorTitleElement = editorTitle;

      if (this.isSplitScreen) {
        this.set('headerClass', smallHeaderClass);
        return;
      }

      if (editorTitle) {
        let boundingRect = editorTitle.getBoundingClientRect();
        let maxRight = window.innerWidth - this._viewActionsWidth;

        if (boundingRect.right >= maxRight) {
          newHeaderClass = smallHeaderClass;
        }
      }

      if (newHeaderClass !== this.headerClass) {
        // grab height of header so that we can pass it as an offset to other
        // editor components
        Ember.run.scheduleOnce('afterRender', this, this._setHeaderHeight);
      }

      this.set('headerClass', newHeaderClass);
    },

    _setHeaderHeight() {
      if (this.headerClass && this._editorTitleElement) {
        let height = this._editorTitleElement.offsetHeight;
        return this.set('headerHeight', height);
      }

      this.set('headerHeight', 0);
    },

    // dragOver is needed so that drop works
    dragOver(event) {
      if (!event.dataTransfer) {
        return;
      } // this is needed to work around inconsistencies with dropping files
      // from Chrome's downloads bar


      if (navigator.userAgent.indexOf('Chrome') > -1) {
        let eA = event.dataTransfer.effectAllowed;
        event.dataTransfer.dropEffect = eA === 'move' || eA === 'linkMove' ? 'move' : 'copy';
      }

      event.preventDefault();
      event.stopPropagation();
    },

    // dragEnter is needed so that the drag class is correctly removed
    dragEnter(event) {
      if (!event.dataTransfer) {
        return;
      }

      event.preventDefault();
      event.stopPropagation(); // the counter technique prevents flickering of the drag class when
      // dragging across child elements

      this._dragCounter += 1;
      this.set('isDraggedOver', true);
    },

    dragLeave(event) {
      event.preventDefault();
      event.stopPropagation();
      this._dragCounter -= 1;

      if (this._dragCounter === 0) {
        this.set('isDraggedOver', false);
      }
    },

    drop(event) {
      event.preventDefault();
      event.stopPropagation();
      this._dragCounter = 0;
      this.set('isDraggedOver', false);

      if (event.dataTransfer.files) {
        this.set('droppedFiles', event.dataTransfer.files);
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-error-message", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * Renders one random error message when passed a DS.Errors object
   * and a property name. The message will be one of the ones associated with
   * that specific property. If there are no errors associated with the property,
   * nothing will be rendered.
   * @param  {DS.Errors} errors   The DS.Errors object
   * @param  {string} property    The property name
   */
  var _default = Ember.Component.extend({
    tagName: 'p',
    classNames: ['response'],
    errors: null,
    property: '',
    isVisible: Ember.computed.notEmpty('errors'),
    message: Ember.computed('errors.[]', 'property', function () {
      let property = this.property;
      let errors = this.errors;
      let messages = [];
      let index;

      if (!Ember.isEmpty(errors) && errors.get(property)) {
        errors.get(property).forEach(error => {
          messages.push(error);
        });
        index = Math.floor(Math.random() * messages.length);
        return messages[index].message;
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-feature-flag", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const FeatureFlagComponent = Ember.Component.extend({
    feature: Ember.inject.service(),
    tagName: 'label',
    classNames: 'checkbox',
    attributeBindings: ['for', 'disabled'],
    disabled: Ember.computed('_disabled', function () {
      if (this._disabled) {
        return true;
      }

      return false;
    }),
    value: Ember.computed('_flagValue', {
      get() {
        return this._flagValue;
      },

      set(key, value) {
        if (this.flag === 'members' && value === true) {
          this.set("feature.subscribers", false);
        }

        return this.set("feature.".concat(this.flag), value);
      }

    }),
    for: Ember.computed('flag', function () {
      return "labs-".concat(this.flag);
    }),
    name: Ember.computed('flag', function () {
      return "labs[".concat(this.flag, "]");
    }),

    init() {
      this._super(...arguments);

      Ember.defineProperty(this, '_flagValue', Ember.computed.readOnly("feature.".concat(this.flag)), function () {
        return this.get("feature.".concat(this.flag));
      });
    }

  });
  FeatureFlagComponent.reopenClass({
    positionalParams: ['flag', '_disabled']
  });
  var _default = FeatureFlagComponent;
  _exports.default = _default;
});
;define("ghost-admin/components/gh-file-input", ["exports", "emberx-file-input/components/x-file-input"], function (_exports, _xFileInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // TODO: remove this override and use {{x-file-input}} directly once we've
  // upgraded to emberx-file-input@1.2.0
  var _default = _xFileInput.default.extend({
    change(e) {
      let action = this.action;
      let files = this.files(e);

      if (files.length && action) {
        action(files, this.resetInput.bind(this));
      }
    },

    /**
    * Gets files from event object.
    *
    * @method
    * @private
    * @param {$.Event || Event}
    */
    files(e) {
      return (e.originalEvent || e).testingFiles || e.target.files;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-file-upload", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    _file: null,
    acceptEncoding: null,
    uploadButtonText: 'Text',
    uploadButtonDisabled: true,
    shouldResetForm: true,

    // closure actions
    onUpload() {},

    onAdd() {},

    actions: {
      upload() {
        if (!this.uploadButtonDisabled && this._file) {
          this.onUpload(this._file);
        } // Prevent double post by disabling the button.


        this.set('uploadButtonDisabled', true); // Reset form

        if (this.shouldResetForm) {
          this.$().closest('form')[0].reset();
        }
      }

    },

    change(event) {
      this.set('uploadButtonDisabled', false);
      this.onAdd();
      this._file = event.target.files[0];
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-file-uploader", ["exports", "ghost-admin/services/ajax"], function (_exports, _ajax) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const DEFAULTS = {
    accept: ['text/csv'],
    extensions: ['csv']
  };

  var _default = Ember.Component.extend({
    ajax: Ember.inject.service(),
    eventBus: Ember.inject.service(),
    notifications: Ember.inject.service(),
    tagName: 'section',
    classNames: ['gh-image-uploader'],
    classNameBindings: ['dragClass'],
    labelText: 'Select or drag-and-drop a file',
    url: null,
    paramName: 'file',
    accept: null,
    extensions: null,
    validate: null,
    file: null,
    response: null,
    dragClass: null,
    failureMessage: null,
    uploadPercentage: 0,
    // Allowed actions
    fileSelected: () => {},
    uploadStarted: () => {},
    uploadFinished: () => {},
    uploadSuccess: () => {},
    uploadFailed: () => {},
    formData: Ember.computed('file', function () {
      let paramName = this.paramName;
      let file = this.file;
      let formData = new FormData();
      formData.append(paramName, file);
      return formData;
    }),
    progressStyle: Ember.computed('uploadPercentage', function () {
      let percentage = this.uploadPercentage;
      let width = '';

      if (percentage > 0) {
        width = "".concat(percentage, "%");
      } else {
        width = '0';
      }

      return Ember.String.htmlSafe("width: ".concat(width));
    }),

    // we can optionally listen to a named event bus channel so that the upload
    // process can be triggered externally
    init() {
      this._super(...arguments);

      let listenTo = this.listenTo;
      this.accept = this.accept || DEFAULTS.accept;
      this.extensions = this.extensions || DEFAULTS.extensions;

      if (listenTo) {
        this.eventBus.subscribe("".concat(listenTo, ":upload"), this, function (file) {
          if (file) {
            this.set('file', file);
          }

          this.send('upload');
        });
      }
    },

    didReceiveAttrs() {
      this._super(...arguments);

      let accept = this.accept;
      let extensions = this.extensions;
      this._accept = !Ember.isBlank(accept) && !Ember.isArray(accept) ? accept.split(',') : accept;
      this._extensions = !Ember.isBlank(extensions) && !Ember.isArray(extensions) ? extensions.split(',') : extensions;
    },

    willDestroyElement() {
      let listenTo = this.listenTo;

      this._super(...arguments);

      if (listenTo) {
        this.eventBus.unsubscribe("".concat(listenTo, ":upload"));
      }
    },

    actions: {
      fileSelected(fileList, resetInput) {
        let [file] = Array.from(fileList);

        let validationResult = this._validate(file);

        this.set('file', file);
        this.fileSelected(file);

        if (validationResult === true) {
          Ember.run.schedule('actions', this, function () {
            this.generateRequest();

            if (resetInput) {
              resetInput();
            }
          });
        } else {
          this._uploadFailed(validationResult);

          if (resetInput) {
            resetInput();
          }
        }
      },

      upload() {
        if (this.file) {
          this.generateRequest();
        }
      },

      reset() {
        this.set('file', null);
        this.set('uploadPercentage', 0);
        this.set('failureMessage', null);
      }

    },

    dragOver(event) {
      if (!event.dataTransfer) {
        return;
      } // this is needed to work around inconsistencies with dropping files
      // from Chrome's downloads bar


      if (navigator.userAgent.indexOf('Chrome') > -1) {
        let eA = event.dataTransfer.effectAllowed;
        event.dataTransfer.dropEffect = eA === 'move' || eA === 'linkMove' ? 'move' : 'copy';
      }

      event.stopPropagation();
      event.preventDefault();
      this.set('dragClass', '-drag-over');
    },

    dragLeave(event) {
      event.preventDefault();
      this.set('dragClass', null);
    },

    drop(event) {
      event.preventDefault();
      this.set('dragClass', null);

      if (event.dataTransfer.files) {
        this.send('fileSelected', event.dataTransfer.files);
      }
    },

    generateRequest() {
      let ajax = this.ajax;
      let formData = this.formData;
      let url = this.url;
      this.uploadStarted();
      ajax.post(url, {
        data: formData,
        processData: false,
        contentType: false,
        dataType: 'text',
        xhr: () => {
          let xhr = new window.XMLHttpRequest();
          xhr.upload.addEventListener('progress', event => {
            this._uploadProgress(event);
          }, false);
          return xhr;
        }
      }).then(response => {
        this._uploadSuccess(JSON.parse(response));
      }).catch(error => {
        this._uploadFailed(error);
      }).finally(() => {
        this.uploadFinished();
      });
    },

    _uploadProgress(event) {
      if (event.lengthComputable) {
        Ember.run(() => {
          let percentage = Math.round(event.loaded / event.total * 100);
          this.set('uploadPercentage', percentage);
        });
      }
    },

    _uploadSuccess(response) {
      this.uploadSuccess(response);
      this.send('reset');
    },

    _uploadFailed(error) {
      let message;

      if ((0, _ajax.isVersionMismatchError)(error)) {
        this.notifications.showAPIError(error);
      }

      if ((0, _ajax.isUnsupportedMediaTypeError)(error)) {
        message = 'The file type you uploaded is not supported.';
      } else if ((0, _ajax.isRequestEntityTooLargeError)(error)) {
        message = 'The file you uploaded was larger than the maximum file size your server allows.';
      } else if (error.payload && error.payload.errors && !Ember.isBlank(error.payload.errors[0].message)) {
        message = Ember.String.htmlSafe(error.payload.errors[0].message);
      } else {
        message = 'Something went wrong :(';
      }

      this.set('failureMessage', message);
      this.uploadFailed(error);
    },

    _validate(file) {
      if (this.validate) {
        return this.validate(file);
      } else {
        return this._defaultValidator(file);
      }
    },

    _defaultValidator(file) {
      let [, extension] = /(?:\.([^.]+))?$/.exec(file.name);
      let extensions = this._extensions;

      if (!extension || extensions.indexOf(extension.toLowerCase()) === -1) {
        return new _ajax.UnsupportedMediaTypeError();
      }

      return true;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-form-group", ["exports", "ghost-admin/components/gh-validation-status-container"], function (_exports, _ghValidationStatusContainer) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _ghValidationStatusContainer.default.extend({
    classNames: 'form-group'
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-fullscreen-modal", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const FullScreenModalComponent = Ember.Component.extend({
    dropdown: Ember.inject.service(),
    model: null,
    modifier: null,
    modalPath: Ember.computed('modal', function () {
      return "modal-".concat(this.modal || 'unknown');
    }),
    modalClasses: Ember.computed('modifiers', function () {
      let modalClass = 'fullscreen-modal';
      let modifiers = (this.modifier || '').split(' ');
      let modalClasses = Ember.A([modalClass]);
      modifiers.forEach(modifier => {
        if (!Ember.isBlank(modifier)) {
          let className = "".concat(modalClass, "-").concat(modifier);
          modalClasses.push(className);
        }
      });
      return modalClasses.join(' ');
    }),

    didInsertElement() {
      Ember.run.schedule('afterRender', this, function () {
        this.dropdown.closeDropdowns();
      });
    },

    actions: {
      close() {
        return this.close();
      },

      confirm() {
        return this.confirm();
      },

      clickOverlay() {
        this.send('close');
      }

    },
    // Allowed actions
    close: () => Ember.RSVP.resolve(),
    confirm: () => Ember.RSVP.resolve()
  });
  FullScreenModalComponent.reopenClass({
    positionalParams: ['modal']
  });
  var _default = FullScreenModalComponent;
  _exports.default = _default;
});
;define("ghost-admin/components/gh-image-uploader-with-preview", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    allowUnsplash: false,
    actions: {
      update() {
        let action = this.update;

        if (action) {
          action(...arguments);
        }
      },

      uploadStarted() {
        let action = this.uploadStarted;

        if (action) {
          action(...arguments);
        }
      },

      uploadFinished() {
        let action = this.uploadFinished;

        if (action) {
          action(...arguments);
        }
      },

      remove() {
        let action = this.remove;

        if (action) {
          action();
        }
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-image-uploader", ["exports", "ghost-admin/utils/ghost-paths", "ghost-admin/services/ajax"], function (_exports, _ghostPaths, _ajax) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _exports.IMAGE_PARAMS = _exports.IMAGE_EXTENSIONS = _exports.IMAGE_MIME_TYPES = void 0;
  const IMAGE_MIME_TYPES = 'image/gif,image/jpg,image/jpeg,image/png,image/svg+xml';
  _exports.IMAGE_MIME_TYPES = IMAGE_MIME_TYPES;
  const IMAGE_EXTENSIONS = ['gif', 'jpg', 'jpeg', 'png', 'svg'];
  _exports.IMAGE_EXTENSIONS = IMAGE_EXTENSIONS;
  const IMAGE_PARAMS = {
    purpose: 'image'
  };
  _exports.IMAGE_PARAMS = IMAGE_PARAMS;

  var _default = Ember.Component.extend({
    ajax: Ember.inject.service(),
    config: Ember.inject.service(),
    notifications: Ember.inject.service(),
    settings: Ember.inject.service(),
    tagName: 'section',
    classNames: ['gh-image-uploader'],
    classNameBindings: ['dragClass'],
    image: null,
    text: '',
    altText: '',
    saveButton: true,
    accept: '',
    extensions: null,
    uploadUrl: null,
    paramName: 'file',
    paramsHash: null,
    resourceName: 'images',
    validate: null,
    allowUnsplash: false,
    dragClass: null,
    failureMessage: null,
    file: null,
    url: null,
    uploadPercentage: 0,
    _defaultAccept: IMAGE_MIME_TYPES,
    _defaultExtensions: IMAGE_EXTENSIONS,
    _defaultUploadUrl: '/images/upload/',
    _defaultParamsHash: IMAGE_PARAMS,
    _showUnsplash: false,
    // Allowed actions
    fileSelected: () => {},
    update: () => {},
    uploadStarted: () => {},
    uploadFinished: () => {},
    uploadSuccess: () => {},
    uploadFailed: () => {},
    // TODO: this wouldn't be necessary if the server could accept direct
    // file uploads
    formData: Ember.computed('file', function () {
      let file = this.file;
      let formData = new FormData();
      formData.append(this.paramName, file);
      Object.keys(this.paramsHash || {}).forEach(key => {
        formData.append(key, this.paramsHash[key]);
      });
      return formData;
    }),
    description: Ember.computed('text', 'altText', function () {
      let altText = this.altText;
      return this.text || (altText ? "Upload image of \"".concat(altText, "\"") : 'Upload an image');
    }),
    progressStyle: Ember.computed('uploadPercentage', function () {
      let percentage = this.uploadPercentage;
      let width = '';

      if (percentage > 0) {
        width = "".concat(percentage, "%");
      } else {
        width = '0';
      }

      return Ember.String.htmlSafe("width: ".concat(width));
    }),

    init() {
      this._super(...arguments);

      if (!this.accept) {
        this.set('accept', this._defaultAccept);
      }

      if (!this.extensions) {
        this.set('extensions', this._defaultExtensions);
      }

      if (!this.uploadUrl) {
        this.set('uploadUrl', this._defaultUploadUrl);
      }

      if (!this.paramsHash) {
        this.set('paramsHash', this._defaultParamsHash);
      }
    },

    didReceiveAttrs() {
      let image = this.image;
      this.set('url', image);
    },

    actions: {
      fileSelected(fileList, resetInput) {
        // can't use array destructuring here as FileList is not a strict
        // array and fails in Safari
        // eslint-disable-next-line ember-suave/prefer-destructuring
        let file = fileList[0];

        let validationResult = this._validate(file);

        this.set('file', file);
        this.fileSelected(file);

        if (validationResult === true) {
          Ember.run.schedule('actions', this, function () {
            this.generateRequest();

            if (resetInput) {
              resetInput();
            }
          });
        } else {
          this._uploadFailed(validationResult);

          if (resetInput) {
            resetInput();
          }
        }
      },

      addUnsplashPhoto(_ref) {
        let {
          src
        } = _ref;
        this.set('url', src);
        this.send('saveUrl');
      },

      reset() {
        this.set('file', null);
        this.set('uploadPercentage', 0);
      },

      saveUrl() {
        let url = this.url;
        this.update(url);
      }

    },

    dragOver(event) {
      if (!event.dataTransfer) {
        return;
      } // this is needed to work around inconsistencies with dropping files
      // from Chrome's downloads bar


      if (navigator.userAgent.indexOf('Chrome') > -1) {
        let eA = event.dataTransfer.effectAllowed;
        event.dataTransfer.dropEffect = eA === 'move' || eA === 'linkMove' ? 'move' : 'copy';
      }

      event.stopPropagation();
      event.preventDefault();
      this.set('dragClass', '-drag-over');
    },

    dragLeave(event) {
      event.preventDefault();
      this.set('dragClass', null);
    },

    drop(event) {
      event.preventDefault();
      this.set('dragClass', null);

      if (event.dataTransfer.files) {
        this.send('fileSelected', event.dataTransfer.files);
      }
    },

    _uploadProgress(event) {
      if (event.lengthComputable) {
        Ember.run(() => {
          let percentage = Math.round(event.loaded / event.total * 100);
          this.set('uploadPercentage', percentage);
        });
      }
    },

    _uploadSuccess(response) {
      let uploadResponse;
      let responseUrl;

      try {
        uploadResponse = JSON.parse(response);
      } catch (e) {
        if (!(e instanceof SyntaxError)) {
          throw e;
        }
      }

      if (uploadResponse) {
        let resource = Ember.get(uploadResponse, this.resourceName);

        if (resource && Ember.isArray(resource) && resource[0]) {
          responseUrl = Ember.get(resource[0], 'url');
        }
      }

      this.set('url', responseUrl);
      this.send('saveUrl');
      this.send('reset');
      this.uploadSuccess(responseUrl);
    },

    _uploadFailed(error) {
      let message;

      if ((0, _ajax.isVersionMismatchError)(error)) {
        this.notifications.showAPIError(error);
      }

      if ((0, _ajax.isUnsupportedMediaTypeError)(error)) {
        let validExtensions = this.extensions.join(', .').toUpperCase();
        validExtensions = ".".concat(validExtensions);
        message = "The image type you uploaded is not supported. Please use ".concat(validExtensions);
      } else if ((0, _ajax.isRequestEntityTooLargeError)(error)) {
        message = 'The image you uploaded was larger than the maximum file size your server allows.';
      } else if (error.payload.errors && !Ember.isBlank(error.payload.errors[0].message)) {
        message = error.payload.errors[0].message;
      } else {
        message = 'Something went wrong :(';
      }

      this.set('failureMessage', message);
      this.uploadFailed(error);
    },

    generateRequest() {
      let ajax = this.ajax;
      let formData = this.formData;
      let uploadUrl = this.uploadUrl; // CASE: we want to upload an icon and we have to POST it to a different endpoint, expecially for icons

      let url = "".concat((0, _ghostPaths.default)().apiRoot).concat(uploadUrl);
      this.uploadStarted();
      ajax.post(url, {
        data: formData,
        processData: false,
        contentType: false,
        dataType: 'text',
        xhr: () => {
          let xhr = new window.XMLHttpRequest();
          xhr.upload.addEventListener('progress', event => {
            this._uploadProgress(event);
          }, false);
          return xhr;
        }
      }).then(response => {
        this._uploadSuccess(response);
      }).catch(error => {
        this._uploadFailed(error);
      }).finally(() => {
        this.uploadFinished();
      });
    },

    _validate(file) {
      if (this.validate) {
        return this.validate(file);
      } else {
        return this._defaultValidator(file);
      }
    },

    _defaultValidator(file) {
      let extensions = this.extensions;
      let [, extension] = /(?:\.([^.]+))?$/.exec(file.name);

      if (!Ember.isArray(extensions)) {
        extensions = extensions.split(',');
      }

      if (!extension || extensions.indexOf(extension.toLowerCase()) === -1) {
        return new _ajax.UnsupportedMediaTypeError();
      }

      return true;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-infinity-loader", ["exports", "ember-infinity/components/infinity-loader", "ghost-admin/templates/components/infinity-loader"], function (_exports, _infinityLoader, _infinityLoader2) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _infinityLoader.default.extend({
    layout: _infinityLoader2.default
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-koenig-editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    // public attrs
    classNames: ['gh-koenig-editor', 'relative', 'w-100', 'vh-100', 'overflow-x-hidden', 'overflow-y-auto', 'z-0'],
    title: '',
    titlePlaceholder: '',
    body: null,
    bodyPlaceholder: '',
    bodyAutofocus: false,
    // internal properties
    _title: null,
    _editor: null,
    _mousedownY: 0,

    // closure actions
    onTitleChange() {},

    onTitleBlur() {},

    onBodyChange() {},

    onEditorCreated() {},

    onWordCountChange() {},

    actions: {
      focusTitle() {
        this._title.focus();
      },

      // triggered when a mousedown is registered on .gh-koenig-editor-pane
      trackMousedown(event) {
        this._mousedownY = event.clientY;
      },

      // triggered when a mouseup is registered on .gh-koenig-editor-pane
      focusEditor(event) {
        if (event.target.classList.contains('gh-koenig-editor-pane')) {
          let editorCanvas = this._editor.element;
          let {
            bottom
          } = editorCanvas.getBoundingClientRect(); // if a mousedown and subsequent mouseup occurs below the editor
          // canvas, focus the editor and put the cursor at the end of the
          // document

          if (this._mousedownY > bottom && event.clientY > bottom) {
            let {
              post
            } = this._editor;
            let range = post.toRange();
            let {
              tailSection
            } = range;
            event.preventDefault();

            this._editor.focus(); // we should always have a visible cursor when focusing
            // at the bottom so create an empty paragraph if last
            // section is a card


            if (tailSection.isCardSection) {
              this._editor.run(postEditor => {
                let newSection = postEditor.builder.createMarkupSection('p');
                postEditor.insertSectionAtEnd(newSection);
                tailSection = newSection;
              });
            }

            this._editor.selectRange(tailSection.tailPosition()); // ensure we're scrolled to the bottom


            this.element.scrollTop = this.element.scrollHeight;
          }
        }
      },

      /* title related actions -------------------------------------------- */
      onTitleCreated(titleElement) {
        this._title = titleElement;
      },

      onTitleChange(newTitle) {
        this.onTitleChange(newTitle);
      },

      onTitleFocusOut() {
        this.onTitleBlur();
      },

      onTitleKeydown(event) {
        let value = event.target.value;
        let selectionStart = event.target.selectionStart; // enter will always focus the editor
        // down arrow will only focus the editor when the cursor is at the
        // end of the input to preserve the default OS behaviour

        if (event.key === 'Enter' || event.key === 'Tab' || (event.key === 'ArrowDown' || event.key === 'ArrowRight') && (!value || selectionStart === value.length)) {
          event.preventDefault(); // on Enter we also want to create a blank para if necessary

          if (event.key === 'Enter') {
            this._addParaAtTop();
          }

          this._editor.focus();
        }
      },

      /* body related actions --------------------------------------------- */
      onEditorCreated(koenig) {
        this._setupEditor(koenig);

        this.onEditorCreated(koenig);
      },

      onBodyChange(newMobiledoc) {
        this.onBodyChange(newMobiledoc);
      }

    },

    /* public methods ------------------------------------------------------- */

    /* internal methods ----------------------------------------------------- */
    _setupEditor(koenig) {
      let component = this;
      this._koenig = koenig;
      this._editor = koenig.editor; // focus the title when pressing SHIFT+TAB

      this._editor.registerKeyCommand({
        str: 'SHIFT+TAB',

        run() {
          component.send('focusTitle');
          return true;
        }

      });
    },

    _addParaAtTop() {
      if (!this._editor) {
        return;
      }

      let editor = this._editor;
      let section = editor.post.toRange().head.section; // create a blank paragraph at the top of the editor unless it's already
      // a blank paragraph

      if (section.isListItem || !section.isBlank || section.text !== '') {
        editor.run(postEditor => {
          let {
            builder
          } = postEditor;
          let newPara = builder.createMarkupSection('p');
          let sections = section.isListItem ? section.parent.parent.sections : section.parent.sections;
          postEditor.insertSectionBefore(sections, newPara, section);
        });
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-loading-spinner", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: '',
    showSpinner: false,
    // ms until the loader is displayed,
    // prevents unnecessary flash of spinner
    slowLoadTimeout: 200,

    didInsertElement() {
      this.startSpinnerTimeout.perform();
    },

    startSpinnerTimeout: (0, _emberConcurrency.task)(function* () {
      yield (0, _emberConcurrency.timeout)(this.slowLoadTimeout);
      this.set('showSpinner', true);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-main", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: 'main',
    classNames: ['gh-main'],
    ariaRole: 'main',

    mouseEnter() {
      let action = this.onMouseEnter;

      if (action) {
        action();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-markdown-editor", ["exports", "ghost-admin/mixins/shortcuts", "ghost-admin/utils/ctrl-or-cmd", "ghost-admin/utils/format-markdown"], function (_exports, _shortcuts, _ctrlOrCmd, _formatMarkdown) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend(_shortcuts.default, {
    config: Ember.inject.service(),
    notifications: Ember.inject.service(),
    settings: Ember.inject.service(),
    classNames: ['gh-markdown-editor'],
    classNameBindings: ['_isFullScreen:gh-markdown-editor-full-screen', '_isSplitScreen:gh-markdown-editor-side-by-side'],
    // Public attributes
    autofocus: false,
    imageMimeTypes: null,
    isFullScreen: false,
    markdown: null,
    options: null,
    placeholder: '',
    showMarkdownHelp: false,
    uploadedImageUrls: null,
    enableSideBySide: true,
    enablePreview: true,
    enableHemingway: true,
    shortcuts: null,
    // Private
    _editor: null,
    _editorFocused: false,
    _isFullScreen: false,
    _isSplitScreen: false,
    _isHemingwayMode: false,
    _isUploading: false,
    _showUnsplash: false,
    _uploadedImageUrls: null,

    // Closure actions
    onChange() {},

    onFullScreenToggle() {},

    onImageFilesSelected() {},

    onPreviewToggle() {},

    onSplitScreenToggle() {},

    simpleMDEOptions: Ember.computed('options', function () {
      let options = this.options || {};
      let defaultOptions = {
        // use our Showdown config with sanitization for previews
        previewRender(markdown) {
          return (0, _formatMarkdown.default)(markdown);
        },

        // Ghost-specific SimpleMDE toolbar config - allows us to create a
        // bridge between SimpleMDE buttons and Ember actions
        toolbar: ['bold', 'italic', 'heading', '|', 'quote', 'unordered-list', 'ordered-list', '|', 'link', {
          name: 'image',
          action: () => {
            this._openImageFileDialog();
          },
          className: 'fa fa-picture-o',
          title: 'Upload Image(s)'
        }, '|', {
          name: 'preview',
          action: () => {
            this._togglePreview();
          },
          className: 'fa fa-eye no-disable',
          title: 'Render Preview (Ctrl-Alt-R)',
          useCtrlOnMac: true
        }, {
          name: 'side-by-side',
          action: () => {
            this.send('toggleSplitScreen');
          },
          className: 'fa fa-columns no-disable no-mobile',
          title: 'Side-by-side Preview (Ctrl-Alt-P)',
          useCtrlOnMac: true
        }, '|', {
          name: 'spellcheck',
          action: () => {
            this._toggleSpellcheck();
          },
          className: 'fa fa-check',
          title: 'Spellcheck (Ctrl-Alt-S)',
          useCtrlOnMac: true
        }, {
          name: 'hemingway',
          action: () => {
            this._toggleHemingway();
          },
          className: 'fa fa-h-square',
          title: 'Hemingway Mode (Ctrl-Alt-H)',
          useCtrlOnMac: true
        }, {
          name: 'guide',
          action: () => {
            this.send('toggleMarkdownHelp');
          },
          className: 'fa fa-question-circle',
          title: 'Markdown Guide'
        }],
        // disable shortcuts for side-by-side and fullscreen because they
        // trigger interal SimpleMDE methods that will result in broken
        // layouts
        shortcuts: {
          toggleFullScreen: null,
          togglePreview: null,
          toggleSideBySide: null,
          drawImage: null
        },
        // only include the number of words in the status bar
        status: ['words']
      };
      let toolbar = defaultOptions.toolbar;

      if (!this.enableSideBySide) {
        let sideBySide = toolbar.findBy('name', 'side-by-side');
        let index = toolbar.indexOf(sideBySide);
        toolbar.splice(index, 1);
      }

      if (!this.enablePreview) {
        let preview = toolbar.findBy('name', 'preview');
        let index = toolbar.indexOf(preview);
        toolbar.splice(index, 1);
      }

      if (!this.enableHemingway) {
        let hemingway = toolbar.findBy('name', 'hemingway');
        let index = toolbar.indexOf(hemingway);
        toolbar.splice(index, 1);
      }

      if (this.get('settings.unsplash.isActive')) {
        let image = toolbar.findBy('name', 'image');
        let index = toolbar.indexOf(image) + 1;
        toolbar.splice(index, 0, {
          name: 'unsplash',
          action: () => {
            this.send('toggleUnsplash');
          },
          className: 'fa fa-camera',
          title: 'Add Image from Unsplash'
        });
      }

      let lastItem = null;
      toolbar.forEach((item, index) => {
        if (item === '|' && item === lastItem) {
          toolbar[index] = null;
        }

        lastItem = item;
      });
      defaultOptions.toolbar = toolbar.filter(Boolean);
      return Ember.assign(defaultOptions, options);
    }),

    init() {
      this._super(...arguments);

      let shortcuts = {};
      shortcuts["".concat(_ctrlOrCmd.default, "+shift+i")] = {
        action: 'openImageFileDialog'
      };
      shortcuts['ctrl+alt+s'] = {
        action: 'toggleSpellcheck'
      };

      if (this.enablePreview) {
        shortcuts['ctrl+alt+r'] = {
          action: 'togglePreview'
        };
      }

      if (this.enableSideBySide) {
        shortcuts['ctrl+alt+p'] = {
          action: 'toggleSplitScreen'
        };
      }

      if (this.enableHemingway) {
        shortcuts['ctrl+alt+h'] = {
          action: 'toggleHemingway'
        };
      }

      this.shortcuts = shortcuts;
    },

    // extract markdown content from single markdown card
    didReceiveAttrs() {
      this._super(...arguments);

      let uploadedImageUrls = this.uploadedImageUrls;

      if (!Ember.isEmpty(uploadedImageUrls) && uploadedImageUrls !== this._uploadedImageUrls) {
        this._uploadedImageUrls = uploadedImageUrls; // must be done afterRender to avoid double modify of mobiledoc in
        // a single render

        Ember.run.scheduleOnce('afterRender', this, () => {
          this._insertImages(uploadedImageUrls);
        });
      } // focus the editor when the markdown value changes, this is necessary
      // because both the autofocus and markdown values can change without a
      // re-render, eg. navigating from edit->new


      if (this.autofocus && this._editor && this.markdown !== this._editor.value()) {
        this.send('focusEditor');
      } // use internal values to avoid updating bound values


      if (!Ember.isEmpty(this.isFullScreen)) {
        this.set('_isFullScreen', this.isFullScreen);
      }

      if (!Ember.isEmpty(this.isSplitScreen)) {
        this.set('_isSplitScreen', this.isSplitScreen);
      }

      this._updateButtonState();
    },

    didInsertElement() {
      this._super(...arguments);

      this.registerShortcuts(); // HACK: iOS will scroll the body up when activating the keyboard, this
      // causes problems in the CodeMirror based editor because iOS doesn't
      // scroll the cursor and other measurement elements which results in
      // rather unfriendly behaviour with text appearing in seemingly random
      // places and an inability to select things properly
      //
      // To get around this we use a raf loop that constantly makes sure the
      // body scrollTop is 0 when the editor is on screen

      let iOS = !!navigator.platform && /iPad|iPhone|iPod/.test(navigator.platform);

      if (iOS) {
        this._preventBodyScroll();
      }
    },

    willDestroyElement() {
      if (this._isSplitScreen) {
        this._disconnectSplitPreview();
      }

      this.removeShortcuts();

      this._super(...arguments);

      if (this._preventBodyScrollId) {
        window.cancelAnimationFrame(this._preventBodyScrollId);
      }
    },

    actions: {
      // trigger external update, any mobiledoc updates are handled there
      updateMarkdown(markdown) {
        this.onChange(markdown);
      },

      // store a reference to the simplemde editor so that we can handle
      // focusing and image uploads
      setEditor(editor) {
        this._editor = editor; // disable CodeMirror's drag/drop handling as we want to handle that
        // in the parent gh-editor component

        this._editor.codemirror.setOption('dragDrop', false); // default to spellchecker being off


        this._editor.codemirror.setOption('mode', 'gfm'); // add non-breaking space as a special char
        // eslint-disable-next-line no-control-regex


        this._editor.codemirror.setOption('specialChars', /[\u0000-\u001f\u007f-\u009f\u00ad\u061c\u200b-\u200f\u2028\u2029\ufeff\xa0]/g);

        this._updateButtonState();
      },

      // used by the title input when the TAB or ENTER keys are pressed
      focusEditor() {
        let position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'bottom';

        this._editor.codemirror.focus();

        if (position === 'bottom') {
          this._editor.codemirror.execCommand('goDocEnd');
        } else if (position === 'top') {
          this._editor.codemirror.execCommand('goDocStart');
        }

        return false;
      },

      // HACK FIXME (PLEASE):
      // - clicking toolbar buttons will cause the editor to lose focus
      // - this is painful because we often want to know if the editor has focus
      //   so that we can insert images and so on in the correct place
      // - the blur event will always fire before the button action is triggered 😞
      // - to work around this we track focus state manually and set it to false
      //   after an arbitrary period that's long enough to allow the button action
      //   to trigger first
      // - this _may_ well have unknown issues due to browser differences,
      //   variations in performance, moon cycles, sun spots, or cosmic rays
      // - here be 🐲
      // - (please let it work 🙏)
      updateFocusState(focused) {
        if (focused) {
          this._editorFocused = true;
        } else {
          Ember.run.later(this, function () {
            this._editorFocused = false;
          }, 100);
        }
      },

      openImageFileDialog() {
        let captureSelection = this._editor.codemirror.hasFocus();

        this._openImageFileDialog({
          captureSelection
        });
      },

      toggleUnsplash() {
        if (this._showUnsplash) {
          return this.toggleProperty('_showUnsplash');
        } // capture current selection before it's lost by clicking toolbar btn


        if (this._editorFocused) {
          this._imageInsertSelection = {
            anchor: this._editor.codemirror.getCursor('anchor'),
            head: this._editor.codemirror.getCursor('head')
          };
        }

        this.toggleProperty('_showUnsplash');
      },

      insertUnsplashPhoto(_ref) {
        let {
          src,
          alt,
          caption
        } = _ref;
        let image = {
          alt,
          url: src,
          credit: "<small>".concat(caption, "</small>")
        };

        this._insertImages([image]);
      },

      togglePreview() {
        this._togglePreview();
      },

      toggleFullScreen() {
        let isFullScreen = !this._isFullScreen;
        this.set('_isFullScreen', isFullScreen);

        this._updateButtonState();

        this.onFullScreenToggle(isFullScreen); // leave split screen when exiting full screen mode

        if (!isFullScreen && this._isSplitScreen) {
          this.send('toggleSplitScreen');
        }
      },

      toggleSplitScreen() {
        let isSplitScreen = !this._isSplitScreen;
        let previewButton = this._editor.toolbarElements.preview;
        this.set('_isSplitScreen', isSplitScreen);

        this._updateButtonState(); // set up the preview rendering and scroll sync
        // afterRender is needed so that necessary components have been
        // added/removed and editor pane length has settled


        if (isSplitScreen) {
          // disable the normal SimpleMDE preview if it's active
          if (this._editor.isPreviewActive()) {
            let preview = this._editor.toolbar.find(button => button.name === 'preview');

            preview.action(this._editor);
          }

          if (previewButton) {
            previewButton.classList.add('disabled');
          }

          Ember.run.scheduleOnce('afterRender', this, this._connectSplitPreview);
        } else {
          if (previewButton) {
            previewButton.classList.remove('disabled');
          }

          Ember.run.scheduleOnce('afterRender', this, this._disconnectSplitPreview);
        }

        this.onSplitScreenToggle(isSplitScreen); // go fullscreen when entering split screen mode

        this.send('toggleFullScreen');
      },

      toggleSpellcheck() {
        this._toggleSpellcheck();
      },

      toggleHemingway() {
        this._toggleHemingway();
      },

      toggleMarkdownHelp() {
        this.toggleProperty('showMarkdownHelp');
      }

    },

    _preventBodyScroll() {
      this._preventBodyScrollId = window.requestAnimationFrame(() => {
        let body = document.querySelector('body'); // only scroll the editor if the editor is active so that we don't
        // clobber scroll-to-input behaviour in the PSM

        if (document.activeElement.closest('.CodeMirror')) {
          if (body.scrollTop !== 0) {
            let editor = document.querySelector('.gh-markdown-editor'); // scroll the editor by the same amount the body has been scrolled,
            // this should keep the cursor on screen when opening the keyboard

            editor.scrollTop += body.scrollTop;
            body.scrollTop = 0;
          }
        }

        this._preventBodyScroll();
      });
    },

    _insertImages(urls) {
      let cm = this._editor.codemirror; // loop through urls and generate image markdown

      let images = urls.map(url => {
        // plain url string, so extract filename from path
        if (Ember.typeOf(url) === 'string') {
          let filename = url.split('/').pop();
          let alt = filename; // if we have a normal filename.ext, set alt to filename -ext

          if (filename.lastIndexOf('.') > 0) {
            alt = filename.slice(0, filename.lastIndexOf('.'));
          }

          return "![".concat(alt, "](").concat(url, ")"); // full url object, use attrs we're given
        } else {
          let image = "![".concat(url.alt, "](").concat(url.url, ")");

          if (url.credit) {
            image += "\n".concat(url.credit);
          }

          return image;
        }
      });
      let text = images.join('\n\n'); // clicking the image toolbar button will lose the selection so we use
      // the captured selection to re-select here

      if (this._imageInsertSelection) {
        // we want to focus but not re-position
        this.send('focusEditor', null); // re-select and clear the captured selection so drag/drop still
        // inserts at the correct place

        cm.setSelection(this._imageInsertSelection.anchor, this._imageInsertSelection.head);
        this._imageInsertSelection = null;
      } // focus editor and place cursor at end if not already focused


      if (!cm.hasFocus()) {
        this.send('focusEditor');
        text = "\n\n".concat(text, "\n\n");
      } // insert at cursor or replace selection then position cursor at end
      // of inserted text


      cm.replaceSelection(text, 'end');
    },

    // mark the split-pane/full-screen/spellcheck buttons active when they're active
    _updateButtonState() {
      if (this._editor) {
        let sideBySideButton = this._editor.toolbarElements['side-by-side'];
        let spellcheckButton = this._editor.toolbarElements.spellcheck;
        let hemingwayButton = this._editor.toolbarElements.hemingway;

        if (sideBySideButton) {
          if (this._isSplitScreen) {
            sideBySideButton.classList.add('active');
          } else {
            sideBySideButton.classList.remove('active');
          }
        }

        if (spellcheckButton) {
          if (this._editor.codemirror.getOption('mode') === 'spell-checker') {
            spellcheckButton.classList.add('active');
          } else {
            spellcheckButton.classList.remove('active');
          }
        }

        if (hemingwayButton) {
          if (this._isHemingwayMode) {
            hemingwayButton.classList.add('active');
          } else {
            hemingwayButton.classList.remove('active');
          }
        }
      }
    },

    // set up the preview auto-update and scroll sync
    _connectSplitPreview() {
      let cm = this._editor.codemirror;
      let editor = this._editor;
      let editorPane = this.element.querySelector('.gh-markdown-editor-pane');
      let previewPane = this.element.querySelector('.gh-markdown-editor-preview');
      let previewContent = this.element.querySelector('.gh-markdown-editor-preview-content');
      this._editorPane = editorPane;
      this._previewPane = previewPane;
      this._previewContent = previewContent; // from SimpleMDE -------

      let sideBySideRenderingFunction = function sideBySideRenderingFunction() {
        previewContent.innerHTML = editor.options.previewRender(editor.value(), previewContent);
      };

      cm.sideBySideRenderingFunction = sideBySideRenderingFunction;
      sideBySideRenderingFunction();
      cm.on('update', cm.sideBySideRenderingFunction); // Refresh to fix selection being off (#309)

      cm.refresh(); // ----------------------

      this._onEditorPaneScroll = this._scrollHandler.bind(this);
      editorPane.addEventListener('scroll', this._onEditorPaneScroll, false);

      this._scrollSync();
    },

    _scrollHandler() {
      if (!this._scrollSyncTicking) {
        requestAnimationFrame(this._scrollSync.bind(this));
      }

      this._scrollSyncTicking = true;
    },

    _scrollSync() {
      let editorPane = this._editorPane;
      let previewPane = this._previewPane;
      let height = editorPane.scrollHeight - editorPane.clientHeight;
      let ratio = parseFloat(editorPane.scrollTop) / height;
      let move = (previewPane.scrollHeight - previewPane.clientHeight) * ratio;
      previewPane.scrollTop = move;
      this._scrollSyncTicking = false;
    },

    _disconnectSplitPreview() {
      let cm = this._editor.codemirror;
      cm.off('update', cm.sideBySideRenderingFunction);
      cm.refresh();

      this._editorPane.removeEventListener('scroll', this._onEditorPaneScroll, false);

      delete this._previewPane;
      delete this._previewPaneContent;
      delete this._onEditorPaneScroll;
    },

    _openImageFileDialog() {
      let {
        captureSelection = true
      } = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      if (captureSelection) {
        // capture the current selection before it's lost by clicking the
        // file input button
        this._imageInsertSelection = {
          anchor: this._editor.codemirror.getCursor('anchor'),
          head: this._editor.codemirror.getCursor('head')
        };
      } // trigger the dialog via gh-file-input, when a file is selected it will
      // trigger the onImageFilesSelected closure action


      this.$('input[type="file"]').click();
    },

    // wrap SimpleMDE's built-in preview toggle so that we can trigger a closure
    // action that can apply our own classes higher up in the DOM
    _togglePreview() {
      this.onPreviewToggle(!this._editor.isPreviewActive());

      this._editor.togglePreview();
    },

    _toggleSpellcheck() {
      let cm = this._editor.codemirror;

      if (cm.getOption('mode') === 'spell-checker') {
        cm.setOption('mode', 'gfm');
      } else {
        cm.setOption('mode', 'spell-checker');
      }

      this._updateButtonState();
    },

    _toggleHemingway() {
      let cm = this._editor.codemirror;
      let extraKeys = cm.getOption('extraKeys');
      let notificationText = '';
      this._isHemingwayMode = !this._isHemingwayMode;

      if (this._isHemingwayMode) {
        notificationText = '<span class="gh-notification-title">Hemingway Mode On:</span> Write now; edit later. Backspace disabled.';

        extraKeys.Backspace = function () {};
      } else {
        notificationText = '<span class="gh-notification-title">Hemingway Mode Off:</span> Normal editing restored.';
        delete extraKeys.Backspace;
      }

      cm.setOption('extraKeys', extraKeys);

      this._updateButtonState();

      cm.focus();
      this.notifications.showNotification(Ember.String.htmlSafe(notificationText), {
        key: 'editor.hemingwaymode'
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-member-avatar", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  const stringToHslColor = function stringToHslColor(str, saturation, lightness) {
    var hash = 0;

    for (var i = 0; i < str.length; i++) {
      hash = str.charCodeAt(i) + ((hash << 5) - hash);
    }

    var h = hash % 360;
    return 'hsl(' + h + ', ' + saturation + '%, ' + lightness + '%)';
  };

  var _default = Ember.Component.extend({
    tagName: '',
    member: null,
    initialsClass: 'f6 fw3',
    backgroundStyle: Ember.computed('member.name', function () {
      let name = this.member.name;

      if (name) {
        let color = stringToHslColor(name, 55, 55);
        return Ember.String.htmlSafe("background-color: ".concat(color));
      }
    }),
    initials: Ember.computed('member.name', function () {
      let names = this.member.name.split(' ');
      let intials = [names[0][0], names[names.length - 1][0]];
      return intials.join('').toUpperCase();
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-mobile-nav-bar", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ui: Ember.inject.service(),
    tagName: 'nav',
    classNames: ['gh-mobile-nav-bar']
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-nav-menu", ["exports", "ember-basic-dropdown/utils/calculate-position"], function (_exports, _calculatePosition) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    config: Ember.inject.service(),
    feature: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    router: Ember.inject.service(),
    session: Ember.inject.service(),
    ui: Ember.inject.service(),
    tagName: 'nav',
    classNames: ['gh-nav'],
    iconStyle: '',
    showMenuExtension: Ember.computed('config.clientExtensions.menu', 'session.user.isOwner', function () {
      return this.get('config.clientExtensions.menu') && this.get('session.user.isOwner');
    }),
    showDropdownExtension: Ember.computed('config.clientExtensions.dropdown', 'session.user.isOwner', function () {
      return this.get('config.clientExtensions.dropdown') && this.get('session.user.isOwner');
    }),
    showScriptExtension: Ember.computed('config.clientExtensions.script', 'session.user.isOwner', function () {
      return this.get('config.clientExtensions.script') && this.get('session.user.isOwner');
    }),
    isIntegrationRoute: Ember.computed('router.currentRouteName', function () {
      let re = /^settings\.integration/;
      return re.test(this.router.currentRouteName);
    }),
    // HACK: {{link-to}} should be doing this automatically but there appears to
    // be a bug in Ember that's preventing it from working immediately after login
    isOnSite: Ember.computed('router.currentRouteName', function () {
      return this.router.currentRouteName === 'site';
    }),

    // the menu has a rendering issue (#8307) when the the world is reloaded
    // during an import which we have worked around by not binding the icon
    // style directly. However we still need to keep track of changing icons
    // so that we can refresh when a new icon is uploaded
    didReceiveAttrs() {
      this._setIconStyle();
    },

    actions: {
      transitionToOrRefreshSite() {
        let {
          currentRouteName
        } = this.router;

        if (currentRouteName === 'site') {
          Ember.getOwner(this).lookup("route:".concat(currentRouteName)).refresh();
        } else {
          this.router.transitionTo('site');
        }
      }

    },

    // equivalent to "left: auto; right: -20px"
    userDropdownPosition(trigger, dropdown) {
      let {
        horizontalPosition,
        verticalPosition,
        style
      } = (0, _calculatePosition.default)(...arguments);
      let {
        width: dropdownWidth
      } = dropdown.firstElementChild.getBoundingClientRect();
      style.right += dropdownWidth - 20;
      style['z-index'] = 1100;
      return {
        horizontalPosition,
        verticalPosition,
        style
      };
    },

    _setIconStyle() {
      let icon = this.icon;

      if (icon === this._icon) {
        return;
      }

      this._icon = icon;

      if (icon && icon.match(/^https?:\/\//i)) {
        this.set('iconStyle', Ember.String.htmlSafe("background-image: url(".concat(icon, ")")));
        return;
      }

      let subdirRegExp = new RegExp("^".concat(this.get('ghostPaths.subdir')));
      let blogIcon = icon ? icon : 'favicon.ico';
      let iconUrl;
      blogIcon = blogIcon.replace(subdirRegExp, '');
      iconUrl = this.get('ghostPaths.url').join(this.get('config.blogUrl'), blogIcon).replace(/\/$/, '');
      iconUrl += "?t=".concat(new Date().valueOf());
      this.set('iconStyle', Ember.String.htmlSafe("background-image: url(".concat(iconUrl, ")")));
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-navitem-url-input", ["exports", "validator"], function (_exports, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  // URI is attached to the window global as part of the
  // google-caja html-css-sanitizer-bundle
  const {
    URI
  } = window;

  let joinUrlParts = function joinUrlParts(url, path) {
    if (path[0] !== '/' && url.slice(-1) !== '/') {
      path = "/".concat(path);
    } else if (path[0] === '/' && url.slice(-1) === '/') {
      path = path.slice(1);
    }

    return url + path;
  };

  let isRelative = function isRelative(url) {
    // "protocol://", "//example.com", "scheme:", "#anchor", & invalid paths
    // should all be treated as absolute
    return !url.match(/\s/) && !_validator.default.isURL(url) && !url.match(/^(\/\/|#|[a-zA-Z0-9-]+:)/);
  };

  var _default = Ember.TextField.extend({
    classNames: 'gh-input',

    // Allowed actions
    update() {},

    clearErrors() {},

    isBaseUrl: Ember.computed('baseUrl', 'value', function () {
      return this.baseUrl === this.value;
    }),

    didReceiveAttrs() {
      this._super(...arguments); // value coming is likely to be relative but we always want to show
      // absolute urls in the input fields


      this.set('value', this._makeAbsoluteUrl(this.url));
    },

    focusIn(event) {
      this.set('hasFocus', true);

      if (this.isBaseUrl) {
        // position the cursor at the end of the input
        Ember.run.next(function (el) {
          let {
            length
          } = el.value;
          el.setSelectionRange(length, length);
        }, event.target);
      }
    },

    keyDown(event) {
      // delete the "placeholder" value all at once
      if (this.isBaseUrl && (event.keyCode === 8 || event.keyCode === 46)) {
        this.set('value', '');
        event.preventDefault();
      } // CMD-S


      if (event.keyCode === 83 && event.metaKey) {
        this.notifyUrlChanged();
      }
    },

    keyPress(event) {
      this.clearErrors(); // enter key

      if (event.keyCode === 13) {
        this.notifyUrlChanged();
      }

      return true;
    },

    focusOut() {
      this.set('hasFocus', false);
      this.notifyUrlChanged();
    },

    notifyUrlChanged() {
      let url = this.value.trim();
      let urlURI = URI.parse(url);
      let baseUrl = this.baseUrl;
      let baseURI = URI.parse(baseUrl);

      function getHost(uri) {
        let host = uri.getDomain();

        if (uri.getPort()) {
          host = "".concat(host, ":").concat(uri.getPort());
        }

        return host;
      }

      let urlHost = getHost(urlURI);
      let baseHost = getHost(baseURI); // ensure value property is trimmed

      this.set('value', url); // if we have an email address, add the mailto:

      if (_validator.default.isEmail(url)) {
        url = this.update("mailto:".concat(url));
        this.set('value', url);
        return;
      } // get our baseUrl relativity checks in order


      let isAnchorLink = url.match(/^#/);
      let isRelativeToBasePath = urlURI.getPath() && urlURI.getPath().indexOf(baseURI.getPath()) === 0; // if our path is only missing a trailing / mark it as relative

      if ("".concat(urlURI.getPath(), "/") === baseURI.getPath()) {
        isRelativeToBasePath = true;
      }

      let isOnSameHost = urlHost === baseHost || !urlHost && isRelativeToBasePath; // if relative to baseUrl, remove the base url before sending to action

      if (!isAnchorLink && isOnSameHost && isRelativeToBasePath) {
        url = url.replace(/^[a-zA-Z0-9-]+:/, '');
        url = url.replace(/^\/\//, '');
        url = url.replace(baseHost, '');
        url = url.replace(baseURI.getPath(), ''); // handle case where url path is same as baseUrl path but missing trailing slash

        if (urlURI.getPath().slice(-1) !== '/') {
          url = url.replace(baseURI.getPath().slice(0, -1), '');
        }

        if (url !== '' || !this.isNew) {
          if (!url.match(/^\//)) {
            url = "/".concat(url);
          }

          if (!url.match(/\/$/) && !url.match(/[.#?]/)) {
            url = "".concat(url, "/");
          }
        }
      } // we update with the relative URL but then transform it back to absolute
      // for the input value. This avoids problems where the underlying relative
      // value hasn't changed even though the input value has


      if (url.match(/^(\/\/|#|[a-zA-Z0-9-]+:)/) || _validator.default.isURL(url) || _validator.default.isURL("".concat(baseHost).concat(url))) {
        url = this.update(url);
        this.set('value', this._makeAbsoluteUrl(url));
      }
    },

    _makeAbsoluteUrl(url) {
      if (isRelative(url)) {
        url = joinUrlParts(this.baseUrl, url);
      }

      return url;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-navitem", ["exports", "ghost-admin/mixins/validation-state"], function (_exports, _validationState) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend(_validationState.default, {
    classNames: 'gh-blognav-item',
    classNameBindings: ['errorClass', 'navItem.isNew::gh-blognav-item--sortable'],
    new: false,

    // closure actions
    addItem() {},

    deleteItem() {},

    updateUrl() {},

    updateLabel() {},

    errors: Ember.computed.readOnly('navItem.errors'),
    errorClass: Ember.computed('hasError', function () {
      if (this.hasError) {
        return 'gh-blognav-item--error';
      }
    }),
    actions: {
      addItem() {
        this.addItem();
      },

      deleteItem(item) {
        this.deleteItem(item);
      },

      updateUrl(value) {
        return this.updateUrl(value, this.navItem);
      },

      updateLabel(value) {
        return this.updateLabel(value, this.navItem);
      },

      clearLabelErrors() {
        this.get('navItem.errors').remove('label');
      },

      clearUrlErrors() {
        this.get('navItem.errors').remove('url');
      }

    },

    keyPress(event) {
      // enter key
      if (event.keyCode === 13 && this.get('navItem.isNew')) {
        event.preventDefault();
        Ember.run.scheduleOnce('actions', this, function () {
          this.send('addItem');
        });
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-notification", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    notifications: Ember.inject.service(),
    tagName: 'article',
    classNames: ['gh-notification', 'gh-notification-passive'],
    classNameBindings: ['typeClass'],
    message: null,
    typeClass: Ember.computed('message.type', function () {
      let type = this.get('message.type');
      let classes = '';
      let typeMapping;
      typeMapping = {
        success: 'green',
        error: 'red',
        warn: 'yellow'
      };

      if (typeMapping[type] !== undefined) {
        classes += "gh-notification-".concat(typeMapping[type]);
      }

      return classes;
    }),

    didInsertElement() {
      this._super(...arguments);

      this.$().on('animationend webkitAnimationEnd oanimationend MSAnimationEnd', event => {
        if (event.originalEvent.animationName === 'fade-out') {
          this.notifications.closeNotification(this.message);
        }
      });
    },

    willDestroyElement() {
      this._super(...arguments);

      this.$().off('animationend webkitAnimationEnd oanimationend MSAnimationEnd');
    },

    actions: {
      closeNotification() {
        this.notifications.closeNotification(this.message);
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-notifications", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    notifications: Ember.inject.service(),
    tagName: 'aside',
    classNames: 'gh-notifications',
    messages: Ember.computed.alias('notifications.notifications')
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-post-settings-menu", ["exports", "ghost-admin/mixins/settings-menu-component", "ghost-admin/utils/bound-one-way", "ghost-admin/utils/format-markdown", "moment", "ember-concurrency"], function (_exports, _settingsMenuComponent, _boundOneWay, _formatMarkdown, _moment, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const PSM_ANIMATION_LENGTH = 400;

  var _default = Ember.Component.extend(_settingsMenuComponent.default, {
    store: Ember.inject.service(),
    config: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    slugGenerator: Ember.inject.service(),
    session: Ember.inject.service(),
    settings: Ember.inject.service(),
    ui: Ember.inject.service(),
    rcServices: Ember.inject.service('rc-services'),
    post: null,
    _showSettingsMenu: false,
    _showThrobbers: false,
    privateComments: false,
    canonicalUrlScratch: Ember.computed.alias('post.canonicalUrlScratch'),
    customExcerptScratch: Ember.computed.alias('post.customExcerptScratch'),
    codeinjectionFootScratch: Ember.computed.alias('post.codeinjectionFootScratch'),
    codeinjectionHeadScratch: Ember.computed.alias('post.codeinjectionHeadScratch'),
    metaDescriptionScratch: Ember.computed.alias('post.metaDescriptionScratch'),
    metaTitleScratch: Ember.computed.alias('post.metaTitleScratch'),
    ogDescriptionScratch: Ember.computed.alias('post.ogDescriptionScratch'),
    ogTitleScratch: Ember.computed.alias('post.ogTitleScratch'),
    twitterDescriptionScratch: Ember.computed.alias('post.twitterDescriptionScratch'),
    twitterTitleScratch: Ember.computed.alias('post.twitterTitleScratch'),
    rcDescriptionScratch: Ember.computed.alias('post.rcDescriptionScratch'),
    rcTitleScratch: Ember.computed.alias('post.rcTitleScratch'),
    roomNameScratch: Ember.computed.alias('post.roomNameScratch'),
    slugValue: (0, _boundOneWay.default)('post.slug'),
    allowAnnouncements: (0, _boundOneWay.default)('settings.isAnnounced'),
    allowCollaboration: (0, _boundOneWay.default)('settings.canCollaborate'),
    allowAuthorRooms: (0, _boundOneWay.default)('settings.isAuthorsRooms'),
    allowComments: (0, _boundOneWay.default)('settings.isComments'),
    announce: (0, _boundOneWay.default)('allowAnnouncements'),
    collaborate: (0, _boundOneWay.default)('post.collaborate'),
    roomName: Ember.computed.or('roomNameScratch', 'settings.roomName'),
    facebookDescription: Ember.computed.or('ogDescriptionScratch', 'customExcerptScratch', 'seoDescription'),
    facebookImage: Ember.computed.or('post.ogImage', 'post.featureImage'),
    facebookTitle: Ember.computed.or('ogTitleScratch', 'seoTitle'),
    twitterDescription: Ember.computed.or('twitterDescriptionScratch', 'customExcerptScratch', 'seoDescription'),
    twitterImage: Ember.computed.or('post.twitterImage', 'post.featureImage'),
    twitterTitle: Ember.computed.or('twitterTitleScratch', 'seoTitle'),
    rcDescription: Ember.computed.or('rcDescriptionScratch', 'customExcerptScratch', 'seoDescription'),
    rcImage: Ember.computed.or('post.rcImage', 'post.featureImage'),
    rcTitle: Ember.computed.or('rcTitleScratch', 'seoTitle'),
    seoTitle: Ember.computed('metaTitleScratch', 'post.titleScratch', function () {
      return this.metaTitleScratch || this.post.titleScratch || '(Untitled)';
    }),
    seoDescription: Ember.computed('post.scratch', 'metaDescriptionScratch', function () {
      const metaDescription = this.metaDescriptionScratch || '';
      const mobiledoc = this.get('post.scratch');
      const [markdownCard] = mobiledoc && mobiledoc.cards || [];
      const markdown = markdownCard && markdownCard[1] && markdownCard[1].markdown;
      let placeholder;

      if (metaDescription) {
        placeholder = metaDescription;
      } else {
        let div = document.createElement('div');
        div.innerHTML = (0, _formatMarkdown.default)(markdown, false); // Strip HTML

        placeholder = div.textContent; // Replace new lines and trim

        placeholder = placeholder.replace(/\n+/g, ' ').trim();
      }

      return placeholder;
    }),
    seoURL: Ember.computed('post.{slug,canonicalUrl}', 'config.blogUrl', function () {
      const blogUrl = this.get('config.blogUrl');
      const seoSlug = this.post.slug || '';
      const canonicalUrl = this.post.canonicalUrl || '';

      if (canonicalUrl) {
        if (canonicalUrl.match(/^\//)) {
          return "".concat(blogUrl).concat(canonicalUrl);
        } else {
          return canonicalUrl;
        }
      } else {
        let seoURL = "".concat(blogUrl, "/").concat(seoSlug); // only append a slash to the URL if the slug exists

        if (seoSlug) {
          seoURL += '/';
        }

        return seoURL;
      }
    }),

    didReceiveAttrs() {
      this._super(...arguments); // HACK: ugly method of working around the CSS animations so that we
      // can add throbbers only when the animation has finished
      // TODO: use liquid-fire to handle PSM slide-in and replace tabs manager
      // with something more ember-like


      if (this.showSettingsMenu && !this._showSettingsMenu) {
        this.showThrobbers.perform();
      } // fired when menu is closed


      if (!this.showSettingsMenu && this._showSettingsMenu) {
        const post = this.post;
        const errors = post.get('errors'); // reset the publish date if it has an error

        if (errors.has('publishedAtBlogDate') || errors.has('publishedAtBlogTime')) {
          post.set('publishedAtBlogTZ', post.get('publishedAtUTC'));
          post.validate({
            attribute: 'publishedAtBlog'
          });
        }

        if (errors.has('roomName')) {
          post.set('roomNameScratch', post.get('roomName'));
          post.validate({
            attribute: 'roomName'
          });
        } // remove throbbers


        this.set('_showThrobbers', false);
      }

      this._showSettingsMenu = this.showSettingsMenu;
    },

    actions: {
      showSubview(subview) {
        this._super(...arguments);

        this.set('subview', subview); // Chrome appears to have an animation bug that cancels the slide
        // transition unless there's a delay between the animation starting
        // and the throbbers being removed

        Ember.run.later(this, function () {
          this.set('_showThrobbers', false);
        }, 50);
      },

      closeSubview() {
        this._super(...arguments);

        this.set('subview', null);
        this.showThrobbers.perform();
      },

      discardEnter() {
        return false;
      },

      toggleFeatured() {
        this.toggleProperty('post.featured'); // If this is a new post.  Don't save the post.  Defer the save
        // to the user pressing the save button

        if (this.get('post.isNew')) {
          return;
        }

        this.savePost.perform().catch(error => {
          this.showError(error);
          this.post.rollbackAttributes();
        });
      },

      /**
       * triggered by user manually changing announce-setting
       */
      toggleAnnounce() {
        const post = this.post;
        const announce = this.announce;
        this.toggleProperty('announce');
        post.set('announce', !announce);
        post.set('announceChanged', true);
      },

      toggleComment() {
        const post = this.post;
        const privateComments = this.privateComments;
        this.toggleProperty('privateComments');
        post.set('discussionRoomType', privateComments ? 'c' : 'p');
      },

      toggleCollaborate() {
        const post = this.post;
        const collaborate = this.collaborate;
        this.toggleProperty('collaborate');
        post.set('collaborate', !collaborate);
      },

      togglePrivatePost() {
        this.toggleProperty('post.isPrivate');
      },

      /**
       * triggered by user manually changing room-name
       */
      validateRoom(newRoom) {
        const oldRoom = this.roomName;
        const post = this.post;
        const errMessage = 'Room does not exist'; // reset errors and validation

        post.get('errors').remove('roomName');

        if (!newRoom) {
          newRoom = oldRoom;
        }

        this.rcServices.getRoom({
          rname: newRoom
        }).then(room => {
          const {
            data: [{
              exist,
              rid
            }]
          } = room;

          if (!exist) {
            throw errMessage;
          }

          post.set('roomName', newRoom);
          post.set('roomId', rid);
        }).catch(e => {
          if (e === errMessage) {
            post.get('errors').add('roomName', errMessage);
            return;
          }

          throw e;
        });
      },

      /**
       * triggered by user manually changing slug
       */
      updateSlug(newSlug) {
        return this.updateSlug.perform(newSlug).catch(error => {
          this.showError(error);
          this.post.rollbackAttributes();
        });
      },

      setPublishedAtBlogDate(date) {
        const post = this.post;
        const dateString = (0, _moment.default)(date).format('YYYY-MM-DD');
        post.get('errors').remove('publishedAtBlogDate');

        if (post.get('isNew') || date === post.get('publishedAtBlogDate')) {
          post.validate({
            property: 'publishedAtBlog'
          });
        } else {
          post.set('publishedAtBlogDate', dateString);
          return this.savePost.perform();
        }
      },

      setPublishedAtBlogTime(time) {
        const post = this.post;
        post.get('errors').remove('publishedAtBlogDate');

        if (post.get('isNew') || time === post.get('publishedAtBlogTime')) {
          post.validate({
            property: 'publishedAtBlog'
          });
        } else {
          post.set('publishedAtBlogTime', time);
          return this.savePost.perform();
        }
      },

      setCustomExcerpt(excerpt) {
        const post = this.post;
        const currentExcerpt = post.get('customExcerpt');

        if (excerpt === currentExcerpt) {
          return;
        }

        post.set('customExcerpt', excerpt);
        return post.validate({
          property: 'customExcerpt'
        }).then(() => this.savePost.perform());
      },

      setHeaderInjection(code) {
        const post = this.post;
        const currentCode = post.get('codeinjectionHead');

        if (code === currentCode) {
          return;
        }

        post.set('codeinjectionHead', code);
        return post.validate({
          property: 'codeinjectionHead'
        }).then(() => this.savePost.perform());
      },

      setFooterInjection(code) {
        const post = this.post;
        const currentCode = post.get('codeinjectionFoot');

        if (code === currentCode) {
          return;
        }

        post.set('codeinjectionFoot', code);
        return post.validate({
          property: 'codeinjectionFoot'
        }).then(() => this.savePost.perform());
      },

      setMetaTitle(metaTitle) {
        // Grab the post and current stored meta title
        const post = this.post;
        const currentTitle = post.get('metaTitle'); // If the title entered matches the stored meta title, do nothing

        if (currentTitle === metaTitle) {
          return;
        } // If the title entered is different, set it as the new meta title


        post.set('metaTitle', metaTitle); // Make sure the meta title is valid and if so, save it into the post

        return post.validate({
          property: 'metaTitle'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setMetaDescription(metaDescription) {
        // Grab the post and current stored meta description
        const post = this.post;
        const currentDescription = post.get('metaDescription'); // If the title entered matches the stored meta title, do nothing

        if (currentDescription === metaDescription) {
          return;
        } // If the title entered is different, set it as the new meta title


        post.set('metaDescription', metaDescription); // Make sure the meta title is valid and if so, save it into the post

        return post.validate({
          property: 'metaDescription'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setCanonicalUrl(value) {
        // Grab the post and current stored meta description
        const post = this.post;
        const currentCanonicalUrl = post.canonicalUrl; // If the value entered matches the stored value, do nothing

        if (currentCanonicalUrl === value) {
          return;
        } // If the value supplied is different, set it as the new value


        post.set('canonicalUrl', value); // Make sure the value is valid and if so, save it into the post

        return post.validate({
          property: 'canonicalUrl'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setOgTitle(ogTitle) {
        // Grab the post and current stored facebook title
        const post = this.post;
        const currentTitle = post.get('ogTitle'); // If the title entered matches the stored facebook title, do nothing

        if (currentTitle === ogTitle) {
          return;
        } // If the title entered is different, set it as the new facebook title


        post.set('ogTitle', ogTitle); // Make sure the facebook title is valid and if so, save it into the post

        return post.validate({
          property: 'ogTitle'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setOgDescription(ogDescription) {
        // Grab the post and current stored facebook description
        const post = this.post;
        const currentDescription = post.get('ogDescription'); // If the title entered matches the stored facebook description, do nothing

        if (currentDescription === ogDescription) {
          return;
        } // If the description entered is different, set it as the new facebook description


        post.set('ogDescription', ogDescription); // Make sure the facebook description is valid and if so, save it into the post

        return post.validate({
          property: 'ogDescription'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setTwitterTitle(twitterTitle) {
        // Grab the post and current stored twitter title
        const post = this.post;
        const currentTitle = post.get('twitterTitle'); // If the title entered matches the stored twitter title, do nothing

        if (currentTitle === twitterTitle) {
          return;
        } // If the title entered is different, set it as the new twitter title


        post.set('twitterTitle', twitterTitle); // Make sure the twitter title is valid and if so, save it into the post

        return post.validate({
          property: 'twitterTitle'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setTwitterDescription(twitterDescription) {
        // Grab the post and current stored twitter description
        const post = this.post;
        const currentDescription = post.get('twitterDescription'); // If the description entered matches the stored twitter description, do nothing

        if (currentDescription === twitterDescription) {
          return;
        } // If the description entered is different, set it as the new twitter description


        post.set('twitterDescription', twitterDescription); // Make sure the twitter description is valid and if so, save it into the post

        return post.validate({
          property: 'twitterDescription'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setRcTitle(rcTitle) {
        // Grab the post and current stored twitter title
        const post = this.post;
        const currentTitle = post.get('rcTitle'); // If the title entered matches the stored twitter title, do nothing

        if (currentTitle === rcTitle) {
          return;
        } // If the title entered is different, set it as the new twitter title


        post.set('rcTitle', rcTitle); // Make sure the twitter title is valid and if so, save it into the post

        return post.validate({
          property: 'rcTitle'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setRcDescription(rcDescription) {
        // Grab the post and current stored twitter description
        const post = this.post;
        const currentDescription = post.get('rcDescription'); // If the description entered matches the stored twitter description, do nothing

        if (currentDescription === rcDescription) {
          return;
        } // If the description entered is different, set it as the new twitter description


        post.set('rcDescription', rcDescription); // Make sure the twitter description is valid and if so, save it into the post

        return post.validate({
          property: 'rcDescription'
        }).then(() => {
          if (post.get('isNew')) {
            return;
          }

          return this.savePost.perform();
        });
      },

      setCoverImage(image) {
        this.set('post.featureImage', image);
        this.savePostUtils();
      },

      clearCoverImage() {
        this.set('post.featureImage', '');
        this.savePostUtils();
      },

      setOgImage(image) {
        this.set('post.ogImage', image);
        this.savePostUtils();
      },

      clearOgImage() {
        this.set('post.ogImage', '');
        this.savePostUtils();
      },

      setTwitterImage(image) {
        this.set('post.twitterImage', image);
        this.savePostUtils();
      },

      clearTwitterImage() {
        this.set('post.twitterImage', '');
        this.savePostUtils();
      },

      setRcImage(image) {
        this.set('post.rcImage', image);
        this.savePostUtils();
      },

      clearRcImage() {
        this.set('post.rcImage', '');
        this.savePostUtils();
      },

      changeAuthors(newAuthors) {
        const post = this.post; // return if nothing changed

        if (newAuthors.mapBy('id').join() === post.get('authors').mapBy('id').join()) {
          return;
        }

        post.set('authors', newAuthors);
        post.validate({
          property: 'authors'
        }); // if this is a new post (never been saved before), don't try to save it

        this.savePostUtils();
      },

      deletePost() {
        if (this.deletePost) {
          this.deletePost();
        }
      }

    },
    showThrobbers: (0, _emberConcurrency.task)(function* () {
      yield (0, _emberConcurrency.timeout)(PSM_ANIMATION_LENGTH);
      this.set('_showThrobbers', true);
    }).restartable(),

    showError(error) {
      // TODO: remove null check once ValidationEngine has been removed
      if (error) {
        this.notifications.showAPIError(error);
      }
    },

    savePostUtils() {
      if (this.get('post.isNew')) {
        return;
      }

      this.savePost.perform().catch(error => {
        this.showError(error);
        this.post.rollbackAttributes();
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-posts-list-item", ["exports", "jquery"], function (_exports, _jquery) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ghostPaths: Ember.inject.service(),
    tagName: 'li',
    classNames: ['gh-posts-list-item'],
    classNameBindings: ['active'],
    post: null,
    active: false,
    currentUserId: null,

    // closure actions
    onClick() {},

    onDoubleClick() {},

    isFeatured: Ember.computed.alias('post.featured'),
    // isPage: alias('post.page'),
    isPage: Ember.computed.equal('post.displayName', 'page'),
    isDraft: Ember.computed.equal('post.status', 'draft'),
    isPublished: Ember.computed.equal('post.status', 'published'),
    isScheduled: Ember.computed.equal('post.status', 'scheduled'),
    authorNames: Ember.computed('post.authors.[]', function () {
      let authors = this.get('post.authors');
      return authors.map(author => author.get('name') || author.get('email')).join(', ');
    }),
    canEdit: Ember.computed('post.authors.[]', 'currentUserId', function () {
      let authors = this.get('post.authors');
      authors = authors.map(author => author.get('id'));
      return authors.includes(this.currentUserId);
    }),
    subText: Ember.computed('post.{excerpt,customExcerpt,metaDescription}', function () {
      let text = this.get('post.excerpt') || '';
      let customExcerpt = this.get('post.customExcerpt');
      let metaDescription = this.get('post.metaDescription');

      if (!Ember.isBlank(customExcerpt)) {
        text = customExcerpt;
      } else if (!Ember.isBlank(metaDescription)) {
        text = metaDescription;
      }

      return "".concat(text.slice(0, 80), "...");
    }),

    didReceiveAttrs() {
      if (this.active) {
        this.scrollIntoView();
      }
    },

    click() {
      this.onClick(this.post);
    },

    doubleClick() {
      this.onDoubleClick(this.post);
    },

    scrollIntoView() {
      let element = this.$();
      let offset = element.offset().top;
      let elementHeight = element.height();
      let container = (0, _jquery.default)('.content-list');
      let containerHeight = container.height();
      let currentScroll = container.scrollTop();
      let isBelowTop, isAboveBottom, isOnScreen;
      isAboveBottom = offset < containerHeight;
      isBelowTop = offset > elementHeight;
      isOnScreen = isBelowTop && isAboveBottom;

      if (!isOnScreen) {
        // Scroll so that element is centered in container
        // 40 is the amount of padding on the container
        container.clearQueue().animate({
          scrollTop: currentScroll + offset - 40 - containerHeight / 2
        });
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-profile-image", ["exports", "jquery", "blueimp-md5", "ember-ajax/request", "validator", "ember-concurrency"], function (_exports, _jquery, _blueimpMd, _request, _validator, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const ANIMATION_TIMEOUT = 1000;
  /**
   * A component to manage a user profile image. By default it just handles picture uploads,
   * but if passed a bound 'email' property it will render the user's gravatar image
   *
   * Example: {{gh-profile-image email=controllerEmailProperty setImage="controllerActionName" debounce=500}}
   *
   * @param  {int}            size              The size of the image to render
   * @param  {String}         email             Reference to a bound email object if gravatar image behavior is desired.
   * @param  {String|action}  setImage          The string name of the action on the controller to be called when an image is added.
   * @param  {int}            debounce          Period to wait after changes to email before attempting to load gravatar
   * @property  {Boolean}     hasUploadedImage  Whether or not the user has uploaded an image (whether or not to show the default image/gravatar image)
   * @property  {String}      defaultImage      String containing the background-image css property of the default user profile image
   * @property  {String}      imageBackground   String containing the background-image css property with the gravatar url
   */

  var _default = Ember.Component.extend({
    config: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    email: '',
    size: 180,
    debounce: 300,
    imageFile: null,
    hasUploadedImage: false,
    _defaultImageUrl: '',

    // closure actions
    setImage() {},

    placeholderStyle: Ember.String.htmlSafe('background-image: url()'),
    avatarStyle: Ember.String.htmlSafe('display: none'),

    init() {
      this._super(...arguments);

      let defaultImage = '/img/user-image.png';
      this._defaultImageUrl = this.get('ghostPaths.assetRoot').replace(/\/$/, '') + defaultImage;

      this._setPlaceholderImage(this._defaultImageUrl);
    },

    didReceiveAttrs() {
      this._super(...arguments);

      if (this.get('config.useGravatar')) {
        this.setGravatar.perform();
      }
    },

    actions: {
      imageSelected(fileList, resetInput) {
        // eslint-disable-next-line
        let imageFile = fileList[0];

        if (imageFile) {
          let reader = new FileReader();
          this.set('imageFile', imageFile);
          this.setImage(imageFile);
          reader.addEventListener('load', () => {
            let dataURL = reader.result;
            this.set('previewDataURL', dataURL);
          }, false);
          reader.readAsDataURL(imageFile);
        }

        resetInput();
      },

      openFileDialog(event) {
        // simulate click to open file dialog
        // using jQuery because IE11 doesn't support MouseEvent
        (0, _jquery.default)(event.target).closest('figure').find('input[type="file"]').click();
      }

    },

    dragOver(event) {
      if (!event.dataTransfer) {
        return;
      } // this is needed to work around inconsistencies with dropping files
      // from Chrome's downloads bar


      if (navigator.userAgent.indexOf('Chrome') > -1) {
        let eA = event.dataTransfer.effectAllowed;
        event.dataTransfer.dropEffect = eA === 'move' || eA === 'linkMove' ? 'move' : 'copy';
      }

      event.stopPropagation();
      event.preventDefault();
    },

    dragLeave(event) {
      event.preventDefault();
    },

    drop(event) {
      event.preventDefault();

      if (event.dataTransfer.files) {
        this.send('imageSelected', event.dataTransfer.files);
      }
    },

    setGravatar: (0, _emberConcurrency.task)(function* () {
      yield (0, _emberConcurrency.timeout)(this.debounce);
      let email = this.email;

      if (_validator.default.isEmail(email || '')) {
        let size = this.size;
        let gravatarUrl = "//www.gravatar.com/avatar/".concat((0, _blueimpMd.default)(email), "?s=").concat(size, "&d=404");

        try {
          // HEAD request is needed otherwise jquery attempts to process
          // binary data as JSON and throws an error
          yield (0, _request.default)(gravatarUrl, {
            type: 'HEAD'
          }); // gravatar exists so switch style and let browser load it

          this._setAvatarImage(gravatarUrl); // wait for fade-in animation to finish before removing placeholder


          yield (0, _emberConcurrency.timeout)(ANIMATION_TIMEOUT);

          this._setPlaceholderImage('');
        } catch (e) {
          // gravatar doesn't exist so make sure we're still showing the placeholder
          this._setPlaceholderImage(this._defaultImageUrl); // then make sure the avatar isn't visible


          this._setAvatarImage('');
        }
      }
    }).restartable(),

    _setPlaceholderImage(url) {
      this.set('placeholderStyle', Ember.String.htmlSafe("background-image: url(".concat(url, ");")));
    },

    _setAvatarImage(url) {
      let display = url ? 'block' : 'none';
      this.set('avatarStyle', Ember.String.htmlSafe("background-image: url(".concat(url, "); display: ").concat(display)));
    },

    queueFile(e, data) {
      let fileName = data.files[0].name;

      if (/\.(gif|jpe?g|png|svg?z)$/i.test(fileName)) {
        let action = this.setImage;

        if (action) {
          action(data);
        }
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-progress-bar", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: '',
    // Public attributes
    percentage: 0,
    isError: false,
    // Internal attributes
    progressStyle: '',

    didReceiveAttrs() {
      this._super(...arguments);

      let percentage = this.percentage;
      let width = percentage > 0 ? "".concat(percentage, "%") : '0';
      this.set('progressStyle', Ember.String.htmlSafe("width: ".concat(width)));
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-psm-authors-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    store: Ember.inject.service(),
    // public attrs
    selectedAuthors: null,
    tagName: '',
    triggerId: '',
    // internal attrs
    availableAuthors: null,

    // closure actions
    updateAuthors() {},

    availableAuthorNames: Ember.computed('availableAuthors.@each.name', function () {
      return this.availableAuthors.map(author => author.get('name').toLowerCase());
    }),

    init() {
      this._super(...arguments); // perform a background query to fetch all users and set `availableAuthors`
      // to a live-query that will be immediately populated with what's in the
      // store and be updated when the above query returns


      this.store.query('user', {
        limit: 'all'
      });
      this.set('availableAuthors', this.store.peekAll('user'));
    },

    actions: {
      updateAuthors(newAuthors) {
        this.updateAuthors(newAuthors);
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-psm-tags-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    store: Ember.inject.service(),
    // public attrs
    post: null,
    tagName: '',
    // internal attrs
    _availableTags: null,
    availableTags: Ember.computed.sort('_availableTags.[]', function (tagA, tagB) {
      // ignorePunctuation means the # in internal tag names is ignored
      return tagA.name.localeCompare(tagB.name, undefined, {
        ignorePunctuation: true
      });
    }),
    availableTagNames: Ember.computed('availableTags.@each.name', function () {
      return this.availableTags.map(tag => tag.name.toLowerCase());
    }),

    init() {
      this._super(...arguments); // perform a background query to fetch all users and set `availableTags`
      // to a live-query that will be immediately populated with what's in the
      // store and be updated when the above query returns


      this.store.query('tag', {
        limit: 'all'
      });
      this.set('_availableTags', this.store.peekAll('tag'));
    },

    actions: {
      matchTags(tagName, term) {
        return tagName.toLowerCase() === term.trim().toLowerCase();
      },

      hideCreateOptionOnMatchingTag(term) {
        return !this.availableTagNames.includes(term.toLowerCase());
      },

      updateTags(newTags) {
        let currentTags = this.get('post.tags'); // destroy new+unsaved tags that are no longer selected

        currentTags.forEach(function (tag) {
          if (!newTags.includes(tag) && tag.get('isNew')) {
            tag.destroyRecord();
          }
        }); // update tags

        return this.set('post.tags', newTags);
      },

      createTag(tagName) {
        let currentTags = this.get('post.tags');
        let currentTagNames = currentTags.map(tag => tag.get('name').toLowerCase());
        let tagToAdd;
        tagName = tagName.trim(); // abort if tag is already selected

        if (currentTagNames.includes(tagName.toLowerCase())) {
          return;
        } // find existing tag if there is one


        tagToAdd = this._findTagByName(tagName); // create new tag if no match

        if (!tagToAdd) {
          tagToAdd = this.store.createRecord('tag', {
            name: tagName
          }); // set to public/internal based on the tag name

          tagToAdd.updateVisibility();
        } // push tag onto post relationship


        return currentTags.pushObject(tagToAdd);
      }

    },

    // methods
    _findTagByName(name) {
      let withMatchingName = function withMatchingName(tag) {
        return tag.name.toLowerCase() === name.toLowerCase();
      };

      return this.availableTags.find(withMatchingName);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-psm-template-select", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    store: Ember.inject.service(),
    // public attributes
    tagName: '',
    post: null,
    // internal properties
    activeTheme: null,

    // closure actions
    onTemplateSelect() {},

    // computed properties
    customTemplates: Ember.computed('activeTheme.customTemplates.[]', function () {
      let templates = this.get('activeTheme.customTemplates') || [];
      let defaultTemplate = {
        filename: '',
        name: 'Default'
      };
      return Ember.isEmpty(templates) ? templates : [defaultTemplate, ...templates.sortBy('name')];
    }),
    matchedSlugTemplate: Ember.computed('post.{page,slug}', 'activeTheme.slugTemplates.[]', function () {
      let slug = this.get('post.slug');
      let type = this.post.constructor.modelName;
      let [matchedTemplate] = this.get('activeTheme.slugTemplates').filter(function (template) {
        return template.for.includes(type) && template.slug === slug;
      });
      return matchedTemplate;
    }),
    selectedTemplate: Ember.computed('post.customTemplate', 'customTemplates.[]', function () {
      let templates = this.customTemplates;
      let filename = this.get('post.customTemplate');
      return templates.findBy('filename', filename);
    }),

    // hooks
    didInsertElement() {
      this._super(...arguments);

      this.loadActiveTheme.perform();
    },

    actions: {
      selectTemplate(template) {
        this.onTemplateSelect(template.filename);
      }

    },
    // tasks
    loadActiveTheme: (0, _emberConcurrency.task)(function* () {
      let store = this.store;
      let themes = yield store.peekAll('theme');

      if (Ember.isEmpty(themes)) {
        themes = yield store.findAll('theme');
      }

      let activeTheme = themes.filterBy('active', true).get('firstObject');
      this.set('activeTheme', activeTheme);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-publishmenu-draft", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    post: null,
    saveType: null,
    // used to set minDate in datepicker
    _minDate: null,
    _publishedAtBlogTZ: null,
    'data-test-publishmenu-draft': true,

    didInsertElement() {
      this.post.set('publishedAtBlogTZ', this.get('post.publishedAtUTC'));
      this.send('setSaveType', 'publish');
    },

    actions: {
      setSaveType(type) {
        if (this.saveType !== type) {
          let hasDateError = !Ember.isEmpty(this.get('post.errors').errorsFor('publishedAtBlogDate'));
          let hasTimeError = !Ember.isEmpty(this.get('post.errors').errorsFor('publishedAtBlogTime'));

          let minDate = this._getMinDate();

          this.set('_minDate', minDate);
          this.setSaveType(type); // when publish: switch to now to avoid validation errors
          // when schedule: switch to last valid or new minimum scheduled date

          if (type === 'publish') {
            if (!hasDateError && !hasTimeError) {
              this._publishedAtBlogTZ = this.get('post.publishedAtBlogTZ');
            } else {
              this._publishedAtBlogTZ = this.get('post.publishedAtUTC');
            }

            this.post.set('publishedAtBlogTZ', this.get('post.publishedAtUTC'));
          } else {
            if (!this._publishedAtBlogTZ || (0, _moment.default)(this._publishedAtBlogTZ).isBefore(minDate)) {
              this.post.set('publishedAtBlogTZ', minDate);
            } else {
              this.post.set('publishedAtBlogTZ', this._publishedAtBlogTZ);
            }
          }

          this.post.validate();
        }
      },

      setDate(date) {
        let post = this.post;
        let dateString = (0, _moment.default)(date).format('YYYY-MM-DD');
        post.set('publishedAtBlogDate', dateString);
        return post.validate();
      },

      setTime(time) {
        let post = this.post;
        post.set('publishedAtBlogTime', time);
        return post.validate();
      }

    },

    // API only accepts dates at least 2 mins in the future, default the
    // scheduled date 5 mins in the future to avoid immediate validation errors
    _getMinDate() {
      return _moment.default.utc().add(5, 'minutes');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-publishmenu-published", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    'data-test-publishmenu-published': true,

    didInsertElement() {
      this.setSaveType('publish');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-publishmenu-scheduled", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    clock: Ember.inject.service(),
    post: null,
    saveType: null,
    isClosing: null,
    // used to set minDate in datepicker
    _minDate: null,
    'data-test-publishmenu-scheduled': true,
    timeToPublished: Ember.computed('post.publishedAtUTC', 'clock.second', function () {
      let publishedAtUTC = this.get('post.publishedAtUTC');

      if (!publishedAtUTC) {
        return null;
      }

      this.get('clock.second');
      return publishedAtUTC.toNow(true);
    }),

    didInsertElement() {
      this.set('_minDate', new Date());
      this.setSaveType('schedule');
    },

    actions: {
      setSaveType(type) {
        if (this.saveType !== type) {
          this.set('_minDate', new Date());
          this.setSaveType(type); // when draft switch to now to avoid validation errors
          // when schedule switch back to saved date to avoid unnecessary re-scheduling

          if (type === 'draft') {
            this.post.set('publishedAtBlogTZ', new Date());
          } else {
            this.post.set('publishedAtBlogTZ', this.get('post.publishedAtUTC'));
          }

          this.post.validate();
        }
      },

      setDate(date) {
        let post = this.post;
        let dateString = (0, _moment.default)(date).format('YYYY-MM-DD');
        post.set('publishedAtBlogDate', dateString);
        return post.validate();
      },

      setTime(time) {
        let post = this.post;

        if (!this.isClosing) {
          post.set('publishedAtBlogTime', time);
          return post.validate();
        }
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-publishmenu", ["exports", "jquery", "ember-concurrency"], function (_exports, _jquery, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    clock: Ember.inject.service(),
    classNames: 'gh-publishmenu',
    displayState: 'draft',
    post: null,
    postStatus: 'draft',
    saveTask: null,
    runningText: null,
    _publishedAtBlogTZ: null,
    _previousStatus: null,
    isClosing: null,
    forcePublishedMenu: Ember.computed.reads('post.pastScheduledTime'),
    postState: Ember.computed('post.{isPublished,isScheduled}', 'forcePublishedMenu', function () {
      if (this.forcePublishedMenu || this.get('post.isPublished')) {
        return 'published';
      } else if (this.get('post.isScheduled')) {
        return 'scheduled';
      } else {
        return 'draft';
      }
    }),
    triggerText: Ember.computed('postState', function () {
      let state = this.postState;

      if (state === 'published') {
        return 'Update';
      } else if (state === 'scheduled') {
        return 'Scheduled';
      } else {
        return 'Publish';
      }
    }),
    _runningText: Ember.computed('postState', 'saveType', function () {
      let saveType = this.saveType;
      let postState = this.postState;
      let runningText;

      if (postState === 'draft') {
        runningText = saveType === 'publish' ? 'Publishing' : 'Scheduling';
      }

      if (postState === 'published') {
        runningText = saveType === 'publish' ? 'Updating' : 'Unpublishing';
      }

      if (postState === 'scheduled') {
        runningText = saveType === 'schedule' ? 'Rescheduling' : 'Unscheduling';
      }

      return runningText || 'Publishing';
    }),
    buttonText: Ember.computed('postState', 'saveType', function () {
      let saveType = this.saveType;
      let postState = this.postState;
      let buttonText;

      if (postState === 'draft') {
        buttonText = saveType === 'publish' ? 'Publish' : 'Schedule';
      }

      if (postState === 'published') {
        buttonText = saveType === 'publish' ? 'Update' : 'Unpublish';
      }

      if (postState === 'scheduled') {
        buttonText = saveType === 'schedule' ? 'Reschedule' : 'Unschedule';
      }

      return buttonText || 'Publish';
    }),
    successText: Ember.computed('_previousStatus', 'postState', function () {
      let postState = this.postState;
      let previousStatus = this._previousStatus;
      let buttonText;

      if (previousStatus === 'draft') {
        buttonText = postState === 'published' ? 'Published' : 'Scheduled';
      }

      if (previousStatus === 'published') {
        buttonText = postState === 'draft' ? 'Unpublished' : 'Updated';
      }

      if (previousStatus === 'scheduled') {
        buttonText = postState === 'draft' ? 'Unscheduled' : 'Rescheduled';
      }

      return buttonText;
    }),

    didReceiveAttrs() {
      this._super(...arguments); // update the displayState based on the post status but only after a
      // save has finished to avoid swapping the menu prematurely and triggering
      // calls to `setSaveType` due to the component re-rendering
      // TODO: we should have a better way of dealing with this where we don't
      // rely on the side-effect of component rendering calling setSaveType


      let postStatus = this.postStatus;

      if (postStatus !== this._postStatus) {
        if (this.get('saveTask.isRunning')) {
          this.get('saveTask.last').then(() => {
            this.set('displayState', postStatus);
          });
        } else {
          this.set('displayState', postStatus);
        }
      }

      this._postStatus = this.postStatus;
    },

    actions: {
      setSaveType(saveType) {
        let post = this.post;
        this.set('saveType', saveType);

        if (saveType === 'draft') {
          post.set('statusScratch', 'draft');
        } else if (saveType === 'schedule') {
          post.set('statusScratch', 'scheduled');
        } else if (saveType === 'publish') {
          post.set('statusScratch', 'published');
        }
      },

      open() {
        this._cachePublishedAtBlogTZ();

        this.set('isClosing', false);
        this.get('post.errors').clear();

        if (this.onOpen) {
          this.onOpen();
        }
      },

      close(dropdown, e) {
        let post = this.post; // don't close the menu if the datepicker popup is clicked

        if (e && (0, _jquery.default)(e.target).closest('.ember-power-datepicker-content').length) {
          return false;
        } // cleanup


        this._resetPublishedAtBlogTZ();

        post.set('statusScratch', null);
        post.validate();

        if (this.onClose) {
          this.onClose();
        }

        this.set('isClosing', true);
        return true;
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      // runningText needs to be declared before the other states change during the
      // save action.
      this.set('runningText', this._runningText);
      this.set('_previousStatus', this.get('post.status'));
      this.setSaveType(this.saveType);

      try {
        // validate publishedAtBlog first to avoid an alert for displayed errors
        yield this.post.validate({
          property: 'publishedAtBlog'
        }); // actual save will show alert for other failed validations

        let post = yield this.saveTask.perform();

        this._cachePublishedAtBlogTZ();

        return post;
      } catch (error) {
        // re-throw if we don't have a validation error
        if (error) {
          throw error;
        }
      }
    }),

    _cachePublishedAtBlogTZ() {
      this._publishedAtBlogTZ = this.get('post.publishedAtBlogTZ');
    },

    // when closing the menu we reset the publishedAtBlogTZ date so that the
    // unsaved changes made to the scheduled date aren't reflected in the PSM
    _resetPublishedAtBlogTZ() {
      this.post.set('publishedAtBlogTZ', this._publishedAtBlogTZ);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-scheduled-post-countdown", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    clock: Ember.inject.service(),
    post: null,
    // countdown timer to show the time left until publish time for a scheduled post
    // starts 15 minutes before scheduled time
    countdown: Ember.computed('post.{publishedAtUTC,isScheduled}', 'clock.second', function () {
      let isScheduled = this.get('post.isScheduled');

      let publishTime = this.get('post.publishedAtUTC') || _moment.default.utc();

      let timeUntilPublished = publishTime.diff(_moment.default.utc(), 'minutes', true);
      let isPublishedSoon = timeUntilPublished > 0 && timeUntilPublished < 15; // force a recompute

      this.get('clock.second');

      if (isScheduled && isPublishedSoon) {
        return (0, _moment.default)(publishTime).fromNow();
      } else {
        return false;
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-scroll-trigger", ["exports", "ember-in-viewport"], function (_exports, _emberInViewport) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend(_emberInViewport.default, {
    enter() {},

    exit() {},

    registerElement() {},

    didInsertElement() {
      let offset = this.triggerOffset || {}; // if triggerOffset is a number we use it for all dimensions

      if (typeof offset === 'number') {
        offset = {
          top: offset,
          bottom: offset,
          left: offset,
          right: offset
        };
      }

      this.set('viewportSpy', true);
      this.set('viewportTolerance', {
        top: offset.top,
        bottom: offset.bottom,
        left: offset.left,
        right: offset.right
      });

      this._super(...arguments);

      this.registerElement(this.element);
    },

    didEnterViewport() {
      return this.enter();
    },

    didExitViewport() {
      return this.exit();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-search-input-trigger", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    actions: {
      captureMouseDown(e) {
        e.stopPropagation();
      },

      search(term) {
        // open dropdown if not open and term is present
        // close dropdown if open and term is blank
        if (Ember.isBlank(term) === this.get('select.isOpen')) {
          Ember.isBlank(term) ? this.close() : this.open(); // ensure focus isn't lost when dropdown is closed

          if (Ember.isBlank(term)) {
            this._focusInput();
          }
        }

        this.select.actions.search(term);
      },

      focusInput() {
        this._focusInput();
      },

      resetInput() {
        let input = this.element && this.element.querySelector('input');

        if (input) {
          input.value = '';
        }
      },

      handleKeydown(e) {
        let select = this.select; // TODO: remove keycode check once EPS is updated to 1.0

        if (!select.isOpen || e.keyCode === 32) {
          e.stopPropagation();
        }
      }

    },

    open() {
      this.select.actions.open();
    },

    close() {
      this.select.actions.close();
    },

    _focusInput() {
      let input = this.element && this.element.querySelector('input');

      if (input) {
        input.focus();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-search-input", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.computedGroup = computedGroup;
  _exports.default = void 0;

  function computedGroup(category) {
    return Ember.computed('content', 'currentSearch', function () {
      if (!this.currentSearch || !this.content) {
        return [];
      }

      return this.content.filter(item => {
        let search = this.currentSearch.toString().toLowerCase();
        return item.category === category && item.title.toString().toLowerCase().indexOf(search) >= 0;
      });
    });
  }

  var _default = Ember.Component.extend({
    store: Ember.inject.service('store'),
    router: Ember.inject.service('router'),
    ajax: Ember.inject.service(),
    notifications: Ember.inject.service(),
    content: null,
    contentExpiresAt: false,
    contentExpiry: 30000,
    currentSearch: '',
    selection: null,
    posts: computedGroup('Posts'),
    pages: computedGroup('Pages'),
    users: computedGroup('Users'),
    tags: computedGroup('Tags'),
    groupedContent: Ember.computed('posts', 'pages', 'users', 'tags', function () {
      let groups = [];

      if (!Ember.isEmpty(this.posts)) {
        groups.pushObject({
          groupName: 'Posts',
          options: this.posts
        });
      }

      if (!Ember.isEmpty(this.pages)) {
        groups.pushObject({
          groupName: 'Pages',
          options: this.pages
        });
      }

      if (!Ember.isEmpty(this.users)) {
        groups.pushObject({
          groupName: 'Users',
          options: this.users
        });
      }

      if (!Ember.isEmpty(this.tags)) {
        groups.pushObject({
          groupName: 'Tags',
          options: this.tags
        });
      }

      return groups;
    }),

    init() {
      this._super(...arguments);

      this.content = [];
    },

    actions: {
      openSelected(selected) {
        if (!selected) {
          return;
        }

        if (selected.category === 'Posts') {
          let id = selected.id.replace('post.', '');
          this.router.transitionTo('editor.edit', 'post', id);
        }

        if (selected.category === 'Pages') {
          let id = selected.id.replace('page.', '');
          this.router.transitionTo('editor.edit', 'page', id);
        }

        if (selected.category === 'Users') {
          let id = selected.id.replace('user.', '');
          this.router.transitionTo('staff.user', id);
        }

        if (selected.category === 'Tags') {
          let id = selected.id.replace('tag.', '');
          this.router.transitionTo('settings.tags.tag', id);
        }
      },

      onFocus() {
        this._setKeymasterScope();
      },

      onBlur() {
        this._resetKeymasterScope();
      },

      search(term) {
        return this.performSearch.perform(term);
      }

    },
    performSearch: (0, _emberConcurrency.task)(function* (term) {
      if (Ember.isBlank(term)) {
        return [];
      } // start loading immediately in the background


      this.refreshContent.perform(); // debounce searches to 200ms to avoid thrashing CPU

      yield (0, _emberConcurrency.timeout)(200); // wait for any on-going refresh to finish

      if (this.refreshContent.isRunning) {
        yield (0, _emberConcurrency.waitForProperty)(this, 'refreshContent.isIdle');
      } // set dependent CP term and re-calculate CP


      this.set('currentSearch', term);
      return this.groupedContent;
    }).restartable(),
    refreshContent: (0, _emberConcurrency.task)(function* () {
      let promises = [];
      let now = new Date();
      let contentExpiresAt = this.contentExpiresAt;

      if (contentExpiresAt > now) {
        return true;
      }

      this.set('content', []);
      promises.pushObject(this._loadPosts());
      promises.pushObject(this._loadPages());
      promises.pushObject(this._loadUsers());
      promises.pushObject(this._loadTags());

      try {
        yield Ember.RSVP.all(promises);
      } catch (error) {
        // eslint-disable-next-line
        console.error(error);
      }

      let contentExpiry = this.contentExpiry;
      this.set('contentExpiresAt', new Date(now.getTime() + contentExpiry));
    }).drop(),

    _loadPosts() {
      let store = this.store;
      let postsUrl = "".concat(store.adapterFor('post').urlForQuery({}, 'post'), "/");
      let postsQuery = {
        fields: 'id,title,page',
        limit: 'all',
        status: 'all'
      };
      let content = this.content;
      return this.ajax.request(postsUrl, {
        data: postsQuery
      }).then(posts => {
        content.pushObjects(posts.posts.map(post => ({
          id: "post.".concat(post.id),
          title: post.title,
          category: 'Posts'
        })));
      }).catch(error => {
        this.notifications.showAPIError(error, {
          key: 'search.loadPosts.error'
        });
      });
    },

    _loadPages() {
      let store = this.store;
      let pagesUrl = "".concat(store.adapterFor('page').urlForQuery({}, 'page'), "/");
      let pagesQuery = {
        fields: 'id,title,page',
        limit: 'all',
        status: 'all'
      };
      let content = this.content;
      return this.ajax.request(pagesUrl, {
        data: pagesQuery
      }).then(pages => {
        content.pushObjects(pages.pages.map(page => ({
          id: "page.".concat(page.id),
          title: page.title,
          category: 'Pages'
        })));
      }).catch(error => {
        this.notifications.showAPIError(error, {
          key: 'search.loadPosts.error'
        });
      });
    },

    _loadUsers() {
      let store = this.store;
      let usersUrl = "".concat(store.adapterFor('user').urlForQuery({}, 'user'), "/");
      let usersQuery = {
        fields: 'name,slug',
        limit: 'all'
      };
      let content = this.content;
      return this.ajax.request(usersUrl, {
        data: usersQuery
      }).then(users => {
        content.pushObjects(users.users.map(user => ({
          id: "user.".concat(user.slug),
          title: user.name,
          category: 'Users'
        })));
      }).catch(error => {
        this.notifications.showAPIError(error, {
          key: 'search.loadUsers.error'
        });
      });
    },

    _loadTags() {
      let store = this.store;
      let tagsUrl = "".concat(store.adapterFor('tag').urlForQuery({}, 'tag'), "/");
      let tagsQuery = {
        fields: 'name,slug',
        limit: 'all'
      };
      let content = this.content;
      return this.ajax.request(tagsUrl, {
        data: tagsQuery
      }).then(tags => {
        content.pushObjects(tags.tags.map(tag => ({
          id: "tag.".concat(tag.slug),
          title: tag.name,
          category: 'Tags'
        })));
      }).catch(error => {
        this.notifications.showAPIError(error, {
          key: 'search.loadTags.error'
        });
      });
    },

    _setKeymasterScope() {
      key.setScope('search-input');
    },

    _resetKeymasterScope() {
      key.setScope('default');
    },

    willDestroy() {
      this._super(...arguments);

      this._resetKeymasterScope();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-simplemde", ["exports", "ghost-admin/config/environment", "ember-concurrency"], function (_exports, _environment, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.TextArea.extend({
    lazyLoader: Ember.inject.service(),
    // Public attributes
    autofocus: false,
    options: null,
    value: null,
    placeholder: '',
    // Private
    _editor: null,

    // Closure actions
    onChange() {},

    onEditorInit() {},

    onEditorDestroy() {},

    // default SimpleMDE options, see docs for available config:
    // https://github.com/sparksuite/simplemde-markdown-editor#configuration
    defaultOptions: Ember.computed(function () {
      return {
        autofocus: this.autofocus,
        indentWithTabs: false,
        placeholder: this.placeholder,
        tabSize: 4
      };
    }),

    init() {
      this._super(...arguments);

      if (Ember.isEmpty(this.options)) {
        this.set('options', {});
      }
    },

    // update the editor when the value property changes from the outside
    didReceiveAttrs() {
      this._super(...arguments);

      if (Ember.isEmpty(this._editor)) {
        return;
      } // compare values before forcing a content reset to avoid clobbering
      // the undo behaviour


      if (this.value !== this._editor.value()) {
        let cursor = this._editor.codemirror.getDoc().getCursor();

        this._editor.value(this.value);

        this._editor.codemirror.getDoc().setCursor(cursor);
      }
    },

    // instantiate the editor with the contents of value
    didInsertElement() {
      this._super(...arguments);

      this.initSimpleMDE.perform();
    },

    willDestroyElement() {
      this._editor.toTextArea();

      delete this._editor;

      this._super(...arguments);
    },

    initSimpleMDE: (0, _emberConcurrency.task)(function* () {
      yield this.lazyLoader.loadScript('simplemde', 'assets/simplemde/simplemde.js');
      let editorOptions = Ember.assign({
        element: document.getElementById(this.elementId)
      }, this.defaultOptions, this.options); // disable spellchecker when testing so that the exterally loaded plugin
      // doesn't fail

      if (_environment.default.environment === 'test') {
        editorOptions.spellChecker = false;
      }

      this._editor = new SimpleMDE(editorOptions);

      this._editor.value(this.value || '');

      this._editor.codemirror.on('change', (instance, changeObj) => {
        // avoid a "modified x twice in a single render" error that occurs
        // when the underlying value is completely swapped out
        if (changeObj.origin !== 'setValue') {
          this.onChange(this._editor.value());
        }
      });

      this._editor.codemirror.on('focus', () => {
        this.onFocus();
      });

      this._editor.codemirror.on('blur', () => {
        this.onBlur();
      });

      if (this.autofocus) {
        this._editor.codemirror.execCommand('goDocEnd');
      }

      this.onEditorInit(this._editor);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-site-iframe", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    config: Ember.inject.service(),
    tagName: '',

    didReceiveAttrs() {
      // reset the src attribute each time the guid changes - allows for
      // a click on the navigation item to reset back to the homepage
      if (this.guid !== this._lastGuid) {
        let iframe = document.querySelector('#site-frame');

        if (iframe) {
          iframe.src = "".concat(this.config.get('blogUrl'), "/");
        }
      }

      this._lastGuid = this.guid;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-skip-link", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: 'a',
    anchor: '',
    classNames: ['sr-only', 'sr-only-focusable'],
    // Add attributes to component for href
    // href should be set to retain anchor properties
    // such as pointer cursor and text underline
    attributeBindings: ['href'],
    // Used so that upon clicking on the link
    // anchor behaviors or ignored
    href: Ember.String.htmlSafe('javascript:;'),

    click() {
      let el = document.querySelector(this.anchor);

      if (el) {
        // Scrolls to the top of main content or whatever
        // is passed to the anchor attribute
        document.body.scrollTop = el.getBoundingClientRect().top;

        let removeTabindex = function removeTabindex() {
          el.removeAttribute('tabindex');
        }; // This sets focus on the content which was skipped to
        // upon losing focus, the tabindex should be removed
        // so that normal keyboard navigation picks up from focused
        // element


        el.setAttribute('tabindex', -1);
        el.focus();
        el.addEventListener('blur', removeTabindex);
        el.addEventListener('focusout', removeTabindex);
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-tag-settings-form", ["exports", "ghost-admin/utils/bound-one-way"], function (_exports, _boundOneWay) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    Handlebars
  } = Ember;

  var _default = Ember.Component.extend({
    feature: Ember.inject.service(),
    config: Ember.inject.service(),
    mediaQueries: Ember.inject.service(),
    tag: null,
    isViewingSubview: false,
    // Allowed actions
    setProperty: () => {},
    showDeleteTagModal: () => {},
    scratchName: (0, _boundOneWay.default)('tag.name'),
    scratchSlug: (0, _boundOneWay.default)('tag.slug'),
    scratchDescription: (0, _boundOneWay.default)('tag.description'),
    scratchMetaTitle: (0, _boundOneWay.default)('tag.metaTitle'),
    scratchMetaDescription: (0, _boundOneWay.default)('tag.metaDescription'),
    isMobile: Ember.computed.reads('mediaQueries.maxWidth600'),
    title: Ember.computed('tag.isNew', function () {
      if (this.get('tag.isNew')) {
        return 'New tag';
      } else {
        return 'Tag settings';
      }
    }),
    seoTitle: Ember.computed('scratchName', 'scratchMetaTitle', function () {
      let metaTitle = this.scratchMetaTitle || '';
      metaTitle = metaTitle.length > 0 ? metaTitle : this.scratchName;

      if (metaTitle && metaTitle.length > 70) {
        metaTitle = metaTitle.substring(0, 70).trim();
        metaTitle = Handlebars.Utils.escapeExpression(metaTitle);
        metaTitle = Ember.String.htmlSafe("".concat(metaTitle, "&hellip;"));
      }

      return metaTitle;
    }),
    seoURL: Ember.computed('scratchSlug', function () {
      let blogUrl = this.get('config.blogUrl');
      let seoSlug = this.scratchSlug || '';
      let seoURL = "".concat(blogUrl, "/tag/").concat(seoSlug); // only append a slash to the URL if the slug exists

      if (seoSlug) {
        seoURL += '/';
      }

      if (seoURL.length > 70) {
        seoURL = seoURL.substring(0, 70).trim();
        seoURL = Handlebars.Utils.escapeExpression(seoURL);
        seoURL = Ember.String.htmlSafe("".concat(seoURL, "&hellip;"));
      }

      return seoURL;
    }),
    seoDescription: Ember.computed('scratchDescription', 'scratchMetaDescription', function () {
      let metaDescription = this.scratchMetaDescription || '';
      metaDescription = metaDescription.length > 0 ? metaDescription : this.scratchDescription;

      if (metaDescription && metaDescription.length > 156) {
        metaDescription = metaDescription.substring(0, 156).trim();
        metaDescription = Handlebars.Utils.escapeExpression(metaDescription);
        metaDescription = Ember.String.htmlSafe("".concat(metaDescription, "&hellip;"));
      }

      return metaDescription;
    }),

    didReceiveAttrs() {
      this._super(...arguments);

      let oldTagId = this._oldTagId;
      let newTagId = this.get('tag.id');

      if (newTagId !== oldTagId) {
        this.reset();
      }

      this._oldTagId = newTagId;
    },

    actions: {
      setProperty(property, value) {
        this.setProperty(property, value);
      },

      setCoverImage(image) {
        this.setProperty('featureImage', image);
      },

      clearCoverImage() {
        this.setProperty('featureImage', '');
      },

      openMeta() {
        this.set('isViewingSubview', true);
      },

      closeMeta() {
        this.set('isViewingSubview', false);
      },

      deleteTag() {
        this.showDeleteTagModal();
      }

    },

    reset() {
      this.set('isViewingSubview', false);

      if (this.$()) {
        this.$('.settings-menu-pane').scrollTop(0);
      }
    },

    focusIn() {
      key.setScope('tag-settings-form');
    },

    focusOut() {
      key.setScope('default');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-tag", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: ''
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-tags-management-container", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    mediaQueries: Ember.inject.service(),
    classNames: ['view-container'],
    classNameBindings: ['isMobile'],
    tags: null,
    selectedTag: null,
    isMobile: Ember.computed.reads('mediaQueries.maxWidth600'),
    isEmpty: Ember.computed.equal('tags.length', 0),
    displaySettingsPane: Ember.computed('isEmpty', 'selectedTag', 'isMobile', function () {
      let isEmpty = this.isEmpty;
      let selectedTag = this.selectedTag;
      let isMobile = this.isMobile; // always display settings pane for blank-slate on mobile

      if (isMobile && isEmpty) {
        return true;
      } // display list if no tag is selected on mobile


      if (isMobile && Ember.isBlank(selectedTag)) {
        return false;
      } // default to displaying settings pane


      return true;
    }),

    init() {
      this._super(...arguments);

      this.mediaQueries.on('change', this, this._fireMobileChangeActions);
    },

    willDestroyElement() {
      this._super(...arguments);

      this.mediaQueries.off('change', this, this._fireMobileChangeActions);
    },

    _fireMobileChangeActions(key, value) {
      if (key === 'maxWidth600') {
        let leftMobileAction = this.leftMobile;
        this.set('isMobile', value);

        if (!value && leftMobileAction) {
          leftMobileAction();
        }
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-task-button", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * Task Button works exactly like Spin button, but with one major difference:
   *
   * Instead of passing a "submitting" parameter (which is bound to the parent object),
   * you pass an ember-concurrency task. All of the "submitting" behavior is handled automatically.
   *
   * As another bonus, there's no need to handle canceling the promises when something
   * like a controller changes. Because the only task running is handled through this
   * component, all running promises will automatically be cancelled when this
   * component is removed from the DOM
   */
  const GhTaskButton = Ember.Component.extend({
    tagName: 'button',
    classNameBindings: ['isRunning:appear-disabled', 'isIdleClass', 'isRunningClass', 'isSuccessClass', 'isFailureClass'],
    attributeBindings: ['disabled', 'form', 'type', 'tabindex'],
    task: null,
    disabled: false,
    defaultClick: false,
    buttonText: 'Save',
    idleClass: '',
    runningClass: '',
    successText: 'Saved',
    successClass: 'gh-btn-green',
    failureText: 'Retry',
    failureClass: 'gh-btn-red',
    // Allowed actions
    action: () => {},
    isRunning: Ember.computed.reads('task.last.isRunning'),
    runningText: Ember.computed.reads('buttonText'),
    // hasRun is needed so that a newly rendered button does not show the last
    // state of the associated task
    hasRun: Ember.computed('task.performCount', function () {
      return this.get('task.performCount') > this._initialPerformCount;
    }),
    isIdleClass: Ember.computed('isIdle', function () {
      if (this.isIdle) {
        return this.idleClass;
      }
    }),
    isRunningClass: Ember.computed('isRunning', function () {
      if (this.isRunning) {
        return this.runningClass || this.idleClass;
      }
    }),
    isSuccess: Ember.computed('hasRun', 'isRunning', 'task.last.value', function () {
      if (!this.hasRun || this.isRunning) {
        return false;
      }

      let value = this.get('task.last.value');
      return !Ember.isBlank(value) && value !== false;
    }),
    isSuccessClass: Ember.computed('isSuccess', function () {
      if (this.isSuccess) {
        return this.successClass;
      }
    }),
    isFailure: Ember.computed('hasRun', 'isRunning', 'isSuccess', 'task.last.error', function () {
      if (!this.hasRun || this.isRunning || this.isSuccess) {
        return false;
      }

      return this.get('task.last.error') !== undefined;
    }),
    isFailureClass: Ember.computed('isFailure', function () {
      if (this.isFailure) {
        return this.failureClass;
      }
    }),
    isIdle: Ember.computed('isRunning', 'isSuccess', 'isFailure', function () {
      return !this.isRunning && !this.isSuccess && !this.isFailure;
    }),

    init() {
      this._super(...arguments);

      this._initialPerformCount = this.get('task.performCount');
    },

    click() {
      // let the default click bubble if defaultClick===true - useful when
      // you want to handle a form submit action rather than triggering a
      // task directly
      if (this.defaultClick) {
        if (!this.isRunning) {
          this._restartAnimation.perform();
        }

        return;
      } // do nothing if disabled externally


      if (this.disabled) {
        return false;
      }

      let task = this.task;
      let taskName = this.get('task.name');
      let lastTaskName = this.get('task.last.task.name'); // task-buttons are never disabled whilst running so that clicks when a
      // taskGroup is running don't get dropped BUT that means we need to check
      // here to avoid spamming actions from multiple clicks

      if (this.isRunning && taskName === lastTaskName) {
        return;
      }

      this.action();
      task.perform();

      this._restartAnimation.perform(); // prevent the click from bubbling and triggering form actions


      return false;
    },

    // mouseDown can be prevented, this is useful for situations where we want
    // to avoid on-blur events triggering before the button click
    mouseDown(event) {
      if (this.disableMouseDown) {
        event.preventDefault();
      }
    },

    // when local validation fails there's no transition from failed->running
    // so we want to restart the retry spinner animation to show something
    // has happened when the button is clicked
    _restartAnimation: (0, _emberConcurrency.task)(function* () {
      if (this.$('.retry-animated').length) {
        // eslint-disable-next-line
        let elem = this.$('.retry-animated')[0];
        elem.classList.remove('retry-animated');
        yield (0, _emberConcurrency.timeout)(10);
        elem.classList.add('retry-animated');
      }
    })
  });
  GhTaskButton.reopenClass({
    positionalParams: ['buttonText']
  });
  var _default = GhTaskButton;
  _exports.default = _default;
});
;define("ghost-admin/components/gh-text-input", ["exports", "ghost-admin/mixins/text-input"], function (_exports, _textInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.TextField.extend(_textInput.default, {
    classNames: 'gh-input'
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-textarea", ["exports", "ghost-admin/mixins/text-input"], function (_exports, _textInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.TextArea.extend(_textInput.default, {
    resizeDetector: Ember.inject.service(),
    classNames: 'gh-input',
    autoExpand: false,

    didReceiveAttrs() {
      this._super(...arguments); // trigger auto-expand any time the value changes


      if (this.autoExpand) {
        Ember.run.scheduleOnce('afterRender', this, this._autoExpand);
      }
    },

    willInsertElement() {
      this._super(...arguments); // disable the draggable resize element that browsers add to textareas


      if (this.autoExpand) {
        this.element.style.resize = 'none';
      }
    },

    didInsertElement() {
      this._super(...arguments); // set up resize handler on element insert so that we can autoexpand
      // when the element container changes size


      if (this.autoExpand) {
        Ember.run.scheduleOnce('afterRender', this, this._setupAutoExpand);
      }

      if (this.didCreateTextarea) {
        this.didCreateTextarea(this.element);
      }
    },

    willDestroyElement() {
      this._teardownAutoExpand();

      this._super(...arguments);
    },

    _autoExpand() {
      let el = this.element; // collapse the element first so that we can shrink as well as expand
      // then set the height to match the text height

      if (el) {
        el.style.height = 0;
        el.style.height = "".concat(el.scrollHeight, "px");
      }
    },

    _setupAutoExpand() {
      this._resizeCallback = Ember.run.bind(this, this._onResize);
      this.resizeDetector.setup(this.autoExpand, this._resizeCallback);

      this._autoExpand();
    },

    _onResize() {
      this._autoExpand();
    },

    _teardownAutoExpand() {
      this.resizeDetector.teardown(this.autoExpand, this._resizeCallback);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-theme-error-li", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: '',
    error: null,
    showDetails: false,
    actions: {
      toggleDetails() {
        this.toggleProperty('showDetails');
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-theme-table", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    themes: null,
    sortedThemes: Ember.computed('themes.@each.active', function () {
      let themes = this.themes.map(t => {
        let theme = {};
        let themePackage = Ember.get(t, 'package');
        theme.model = t;
        theme.name = Ember.get(t, 'name');
        theme.label = themePackage ? "".concat(themePackage.name) : theme.name;
        theme.version = themePackage ? "".concat(themePackage.version) : '1.0';
        theme.package = themePackage;
        theme.active = Ember.get(t, 'active');
        theme.isDeletable = !theme.active;
        return theme;
      });
      let duplicateThemes = [];
      themes.forEach(theme => {
        let duplicateLabels = themes.filterBy('label', theme.label);

        if (duplicateLabels.length > 1) {
          duplicateThemes.pushObject(theme);
        }
      });
      duplicateThemes.forEach(theme => {
        if (theme.name !== 'casper') {
          theme.label = "".concat(theme.label, " (").concat(theme.name, ")");
        }
      }); // "(default)" needs to be added to casper manually as it's always
      // displayed and would mess up the duplicate checking if added earlier

      let casper = themes.findBy('name', 'casper');

      if (casper) {
        casper.label = "".concat(casper.label, " (default)");
        casper.isDefault = true;
        casper.isDeletable = false;
      } // sorting manually because .sortBy('label') has a different sorting
      // algorithm to [...strings].sort()


      return themes.sort((themeA, themeB) => {
        let a = themeA.label.toLowerCase();
        let b = themeB.label.toLowerCase();

        if (a < b) {
          return -1;
        }

        if (a > b) {
          return 1;
        }

        return 0;
      });
    }).readOnly()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-timezone-select", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    clock: Ember.inject.service(),
    classNames: ['form-group', 'for-select'],
    activeTimezone: null,
    availableTimezones: null,
    // Allowed actions
    update: () => {},
    availableTimezoneNames: Ember.computed.mapBy('availableTimezones', 'name'),
    hasTimezoneOverride: Ember.computed('activeTimezone', 'availableTimezoneNames', function () {
      let activeTimezone = this.activeTimezone;
      let availableTimezoneNames = this.availableTimezoneNames;
      return !availableTimezoneNames.includes(activeTimezone);
    }),
    selectedTimezone: Ember.computed('activeTimezone', 'availableTimezones', 'hasTimezoneOverride', function () {
      let hasTimezoneOverride = this.hasTimezoneOverride;
      let activeTimezone = this.activeTimezone;
      let availableTimezones = this.availableTimezones;

      if (hasTimezoneOverride) {
        return {
          name: '',
          label: ''
        };
      }

      return availableTimezones.filterBy('name', activeTimezone).get('firstObject');
    }),
    selectableTimezones: Ember.computed('availableTimezones', 'hasTimezoneOverride', function () {
      let hasTimezoneOverride = this.hasTimezoneOverride;
      let availableTimezones = this.availableTimezones;

      if (hasTimezoneOverride) {
        return [{
          name: '',
          label: ''
        }, ...availableTimezones];
      }

      return availableTimezones;
    }),
    localTime: Ember.computed('hasTimezoneOverride', 'activeTimezone', 'selectedTimezone', 'clock.second', function () {
      let hasTimezoneOverride = this.hasTimezoneOverride;
      let timezone = hasTimezoneOverride ? this.activeTimezone : this.get('selectedTimezone.name');
      this.get('clock.second');
      return timezone ? (0, _moment.default)().tz(timezone).format('HH:mm:ss') : (0, _moment.default)().utc().format('HH:mm:ss');
    }),
    actions: {
      setTimezone(timezone) {
        this.update(timezone);
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-token-input", ["exports", "ember-power-select/utils/group-utils", "ember-concurrency"], function (_exports, _groupUtils, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    Handlebars
  } = Ember;
  const BACKSPACE = 8;
  const TAB = 9;

  var _default = Ember.Component.extend({
    // public attrs
    allowCreation: true,
    closeOnSelect: false,
    labelField: 'name',
    matcher: _groupUtils.defaultMatcher,
    searchField: 'name',
    tagName: '',
    triggerComponent: 'gh-token-input/trigger',
    optionsWithoutSelected: Ember.computed('options.[]', 'selected.[]', function () {
      return this.optionsWithoutSelectedTask.perform();
    }),
    actions: {
      handleKeydown(select, event) {
        // On backspace with empty text, remove the last token but deviate
        // from default behaviour by not updating search to match last token
        if (event.keyCode === BACKSPACE && Ember.isBlank(event.target.value)) {
          let lastSelection = select.selected[select.selected.length - 1];

          if (lastSelection) {
            this.onchange(select.selected.slice(0, -1), select);
            select.actions.search('');
            select.actions.open(event);
          } // prevent default


          return false;
        } // Tab should work the same as Enter if there's a highlighted option


        if (event.keyCode === TAB && !Ember.isBlank(event.target.value) && select.highlighted) {
          if (!select.selected || select.selected.indexOf(select.highlighted) === -1) {
            select.actions.choose(select.highlighted, event);
            return false;
          }
        } // fallback to default


        return true;
      },

      onfocus() {
        key.setScope('gh-token-input');

        if (this.onfocus) {
          this.onfocus(...arguments);
        }
      },

      onblur() {
        key.setScope('default');

        if (this.onblur) {
          this.onblur(...arguments);
        }
      }

    },
    optionsWithoutSelectedTask: (0, _emberConcurrency.task)(function* () {
      let options = yield this.options;
      let selected = yield this.selected;
      return options.filter(o => !selected.includes(o));
    }),

    shouldShowCreateOption(term, options) {
      if (!this.allowCreation) {
        return false;
      }

      if (this.showCreateWhen) {
        return this.showCreateWhen(term, options);
      } else {
        return this.hideCreateOptionOnSameTerm(term, options);
      }
    },

    hideCreateOptionOnSameTerm(term, options) {
      let searchField = this.searchField;
      let existingOption = options.findBy(searchField, term);
      return !existingOption;
    },

    addCreateOption(term, options) {
      if (this.shouldShowCreateOption(term, options)) {
        options.unshift(this.buildSuggestionForTerm(term));
      }
    },

    searchAndSuggest(term, select) {
      return this.searchAndSuggestTask.perform(term, select);
    },

    searchAndSuggestTask: (0, _emberConcurrency.task)(function* (term, select) {
      let newOptions = (yield this.optionsWithoutSelected).toArray();

      if (term.length === 0) {
        return newOptions;
      }

      let searchAction = this.search;

      if (searchAction) {
        let results = yield searchAction(term, select);

        if (results.toArray) {
          results = results.toArray();
        }

        this.addCreateOption(term, results);
        return results;
      }

      newOptions = this.filter(Ember.A(newOptions), term);
      this.addCreateOption(term, newOptions);
      return newOptions;
    }),

    selectOrCreate(selection, select, keyboardEvent) {
      // allow tokens to be created with spaces
      if (keyboardEvent && keyboardEvent.code === 'Space') {
        select.actions.search("".concat(select.searchText, " "));
        return;
      } // guard against return being pressed when nothing is selected


      if (!Ember.isArray(selection)) {
        return;
      }

      let suggestion = selection.find(option => option.__isSuggestion__);

      if (suggestion) {
        this.oncreate(suggestion.__value__, select);
      } else {
        this.onchange(selection, select);
      } // clear select search


      select.actions.search('');
    },

    filter(options, searchText) {
      let matcher;

      if (this.searchField) {
        matcher = (option, text) => this.matcher(Ember.get(option, this.searchField), text);
      } else {
        matcher = (option, text) => this.matcher(option, text);
      }

      return (0, _groupUtils.filterOptions)(options || [], searchText, matcher);
    },

    buildSuggestionForTerm(term) {
      return {
        __isSuggestion__: true,
        __value__: term,
        text: this.buildSuggestionLabel(term)
      };
    },

    buildSuggestionLabel(term) {
      let buildSuggestion = this.buildSuggestion;

      if (buildSuggestion) {
        return buildSuggestion(term);
      }

      return Ember.String.htmlSafe("Add <strong>\"".concat(Handlebars.Utils.escapeExpression(term), "\"...</strong>"));
    },

    // always select the first item in the list that isn't the "Add x" option
    defaultHighlighted(select) {
      let {
        results
      } = select;
      let option = (0, _groupUtils.advanceSelectableOption)(results, undefined, 1);

      if (results.length > 1 && option.__isSuggestion__) {
        option = (0, _groupUtils.advanceSelectableOption)(results, option, 1);
      }

      return option;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-token-input/select-multiple", ["exports", "jquery", "ember-power-select/components/power-select-multiple"], function (_exports, _jquery, _powerSelectMultiple) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const endActions = 'click.ghToken mouseup.ghToken touchend.ghToken'; // triggering focus on the search input within ESA's onfocus event breaks the
  // drag-n-drop functionality in ember-drag-drop so we watch for events that
  // could be the start of a drag and disable the default focus behaviour until
  // we get another event signalling the end of a drag

  var _default = _powerSelectMultiple.default.extend({
    tagName: 'div',
    _canFocus: true,

    willDestroyElement() {
      this._super(...arguments);

      if (this._allowFocusListener) {
        (0, _jquery.default)(window).off(endActions, this._allowFocusListener);
      }
    },

    actions: {
      optionMouseDown(event) {
        if (event.which === 1 && !event.ctrlKey) {
          this._denyFocus(event);
        }
      },

      optionTouchStart(event) {
        this._denyFocus(event);
      },

      handleFocus() {
        if (this._canFocus) {
          this._super(...arguments);
        }
      }

    },

    _denyFocus() {
      if (this._canFocus) {
        this._canFocus = false;
        this._allowFocusListener = Ember.run.bind(this, this._allowFocus);
        (0, _jquery.default)(window).on(endActions, this._allowFocusListener);
      }
    },

    _allowFocus() {
      this._canFocus = true;
      (0, _jquery.default)(window).off(endActions, this._allowFocusListener);
      this._allowFocusListener = null;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-token-input/select", ["exports", "ember-power-select/components/power-select"], function (_exports, _powerSelect) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // NOTE: This is only here because we wanted to override the `eventType` attr.
  // DO NOT add any functionality here, this will hopefully disappear after an
  // upstream PR
  var _default = _powerSelect.default.extend({});

  _exports.default = _default;
});
;define("ghost-admin/components/gh-token-input/suggested-option", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: ''
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-token-input/tag-token", ["exports", "ember-drag-drop/components/draggable-object"], function (_exports, _draggableObject) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _draggableObject.default.extend({
    attributeBindings: ['title'],
    classNames: ['tag-token'],
    classNameBindings: ['internal:tag-token--internal'],
    internal: Ember.computed.readOnly('content.isInternal'),
    primary: Ember.computed('idx', 'internal', function () {
      return !this.internal && this.idx === 0;
    }),
    title: Ember.computed('internal', function () {
      if (this.internal) {
        return "Internal tag";
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-token-input/trigger", ["exports", "ember-power-select/components/power-select-multiple/trigger"], function (_exports, _trigger) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _trigger.default.extend({
    actions: {
      chooseOption(option) {
        this.select.actions.choose(option);
      },

      handleOptionMouseDown(event) {
        if (!event.target.closest('[data-selected-index]')) {
          let action = this.get('extra.optionMouseDown');

          if (action) {
            return action(event);
          }
        }
      },

      handleOptionTouchStart(event) {
        let action = this.get('extra.optionTouchStart');

        if (action) {
          return action(event);
        }
      },

      reorderItems() {
        // ember-drag-drop's sortable-objects has two-way bindings and will
        // update EPS' selected value directly. We have to create a copy
        // after sorting in order to force the onchange action to be triggered
        let selectedCopy = this.select.selected.slice();
        this.select.actions.select(selectedCopy);
      },

      // copied directly from EPS, the default behaviour of stopping propagation
      // of keydown events prevents our shortcuts from being triggered
      onKeydown(e) {
        let {
          onKeydown,
          select
        } = this;

        if (onKeydown && onKeydown(e) === false) {
          e.stopPropagation();
          return false;
        }

        if (e.keyCode === 8) {
          e.stopPropagation();

          if (Ember.isBlank(e.target.value)) {
            let lastSelection = select.selected[select.selected.length - 1];

            if (lastSelection) {
              select.actions.select(this.buildSelection(lastSelection, select), e);

              if (typeof lastSelection === 'string') {
                select.actions.search(lastSelection);
              } else {
                let searchField = this.searchField;
                (true && !(searchField) && Ember.assert('`{{power-select-multiple}}` requires a `searchField` when the options are not strings to remove options using backspace', searchField));
                select.actions.search(Ember.get(lastSelection, searchField));
              }

              select.actions.open(e);
            }
          }
        } // Disable the propagation cancellation so that our shortcuts still work
        // } else if (e.keyCode >= 48 && e.keyCode <= 90 || e.keyCode === 32) { // Keys 0-9, a-z or SPACE
        //     e.stopPropagation();
        // }

      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-tour-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  let instancesCounter = 0;
  let triangleClassPositions = {
    'top-left': {
      attachment: 'top left',
      targetAttachment: 'bottom center',
      offset: '0 28px'
    },
    top: {
      attachment: 'top center',
      targetAttachment: 'bottom center'
    },
    'top-right': {
      attachment: 'top right',
      targetAttachment: 'bottom center',
      offset: '0 -28px'
    },
    'right-top': {
      attachment: 'top right',
      targetAttachment: 'middle left',
      offset: '28px 0'
    },
    right: {
      attachment: 'middle right',
      targetAttachment: 'middle left'
    },
    'right-bottom': {
      attachment: 'bottom right',
      targetAttachment: 'middle left',
      offset: '-28px 0'
    },
    'bottom-right': {
      attachment: 'bottom right',
      targetAttachment: 'top center',
      offset: '0 -28px'
    },
    bottom: {
      attachment: 'bottom center',
      targetAttachment: 'top center'
    },
    'bottom-left': {
      attachment: 'bottom left',
      targetAttachment: 'top center',
      offset: '0 28px'
    },
    'left-bottom': {
      attachment: 'bottom left',
      targetAttachment: 'middle right',
      offset: '-28px 0'
    },
    left: {
      attachment: 'middle left',
      targetAttachment: 'middle right'
    },
    'left-top': {
      attachment: 'top left',
      targetAttachment: 'middle right',
      offset: '28px 0'
    }
  };
  const GhTourItemComponent = Ember.Component.extend({
    mediaQueries: Ember.inject.service(),
    tour: Ember.inject.service(),
    tagName: '',
    throbberId: null,
    target: null,
    throbberAttachment: 'middle center',
    popoverTriangleClass: 'top',
    isOpen: false,
    _elementId: null,
    _throbber: null,
    _throbberElementId: null,
    _throbberElementSelector: null,
    _popoverAttachment: null,
    _popoverTargetAttachment: null,
    _popoverOffset: null,
    isMobile: Ember.computed.reads('mediaQueries.isMobile'),
    isVisible: Ember.computed('isMobile', '_throbber', function () {
      let isMobile = this.isMobile;
      let hasThrobber = !Ember.isBlank(this._throbber);
      return !isMobile && hasThrobber;
    }),

    init() {
      this._super(...arguments); // this is a tagless component so we need to generate our own elementId


      this._elementId = instancesCounter += 1;
      this._throbberElementId = "throbber-".concat(this._elementId);
      this._throbberElementSelector = "#throbber-".concat(this._elementId);
      this._handleOptOut = Ember.run.bind(this, this._remove);
      this._handleViewed = Ember.run.bind(this, this._removeIfViewed);
      this.tour.on('optOut', this._handleOptOut);
      this.tour.on('viewed', this._handleViewed);
    },

    didReceiveAttrs() {
      let throbberId = this.throbberId;
      let throbber = this.tour.activeThrobber(throbberId);
      let triangleClass = this.popoverTriangleClass;
      let popoverPositions = triangleClassPositions[triangleClass];
      this._throbber = throbber;
      this._popoverAttachment = popoverPositions.attachment;
      this._popoverTargetAttachment = popoverPositions.targetAttachment;
      this._popoverOffset = popoverPositions.offset;
    },

    willDestroyElement() {
      this.tour.off('optOut', this._handleOptOut);
      this.tour.off('viewed', this._handleViewed);

      this._super(...arguments);
    },

    actions: {
      open() {
        this.set('isOpen', true);
      },

      close() {
        this._close();
      },

      markAsViewed() {
        let throbberId = this.throbberId;
        this.tour.markThrobberAsViewed(throbberId);
        this.set('_throbber', null);

        this._close();
      },

      optOut() {
        this.tour.optOut();
        this.set('_throbber', null);

        this._close();
      }

    },

    _removeIfViewed(id) {
      if (id === this.throbberId) {
        this._remove();
      }
    },

    _remove() {
      this.set('_throbber', null);
    },

    _close() {
      this.set('isOpen', false);
    }

  });
  GhTourItemComponent.reopenClass({
    positionalParams: ['throbberId']
  });
  var _default = GhTourItemComponent;
  _exports.default = _default;
});
;define("ghost-admin/components/gh-trim-focus-input", ["exports", "ghost-admin/components/gh-text-input"], function (_exports, _ghTextInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * This doesn't override the OneWayInput component because
   * we need finer control. It borrows
   * parts from both the OneWayInput component and Ember's default
   * input component
   */
  const TrimFocusInputComponent = _ghTextInput.default.extend({
    shouldFocus: true,

    focusOut(event) {
      this._trimInput(event.target.value, event);

      this._super(...arguments);
    },

    _trimInput(value, event) {
      if (value && typeof value.trim === 'function') {
        value = value.trim();
      }

      this.element.value = value;

      this._elementValueDidChange(event);

      let inputMethod = this.input;

      if (inputMethod) {
        inputMethod(event);
      }
    }

  });

  var _default = TrimFocusInputComponent;
  _exports.default = _default;
});
;define("ghost-admin/components/gh-unsplash-photo", ["exports", "jquery"], function (_exports, _jquery) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    height: 0,
    photo: null,
    tagName: '',
    width: 1200,
    zoomed: false,

    // closure actions
    select() {},

    zoom() {},

    style: Ember.computed('zoomed', function () {
      if (this.zoomed) {
        return Ember.String.htmlSafe('width: auto; margin: 0;');
      }
    }),
    // avoid "binding style attributes" warnings
    containerStyle: Ember.computed('photo.color', 'zoomed', function () {
      let styles = [];
      let ratio = this.get('photo.ratio');
      let zoomed = this.zoomed;
      styles.push("background-color: ".concat(this.get('photo.color')));

      if (zoomed) {
        styles.push("cursor: zoom-out");
      } else {
        styles.push("padding-bottom: ".concat(ratio * 100, "%"));
      }

      return Ember.String.htmlSafe(styles.join('; '));
    }),
    imageUrl: Ember.computed('photo.urls.regular', function () {
      let url = this.get('photo.urls.regular');
      url = url.replace(/&w=1080/, '&w=1200');
      return url;
    }),

    didReceiveAttrs() {
      this._super(...arguments);

      this.set('height', this.width * this.photo.ratio);

      if (this.zoomed && !this._zoomed) {
        this._setZoomedSize();
      }

      this._zoomed = this.zoomed;

      if (this.zoomed && !this._resizeHandler) {
        this._setupResizeHandler();
      } else if (!this.zoomed && this._resizeHandler) {
        this._teardownResizeHandler();
      }
    },

    didInsertElement() {
      this._super(...arguments);

      this._hasRendered = true;

      if (this.zoomed) {
        this._setZoomedSize();
      }
    },

    willDestroyElement() {
      this._super(...arguments);

      this._teardownResizeHandler();
    },

    actions: {
      select(event) {
        event.preventDefault();
        event.stopPropagation();
        this.select(this.photo);
      },

      zoom(event) {
        let $target = (0, _jquery.default)(event.target); // only zoom when it wasn't one of the child links clicked

        if (!$target.is('a') && $target.closest('a').hasClass('gh-unsplash-photo')) {
          event.preventDefault();
          this.zoom(this.photo);
        } // don't propagate otherwise we can trigger the closeZoom action on the overlay


        event.stopPropagation();
      }

    },

    _setZoomedSize() {
      if (!this._hasRendered) {
        return false;
      }

      let a = document.querySelector("[data-unsplash-zoomed-photo=\"".concat(this.photo.id, "\"]"));
      a.style.width = '100%';
      a.style.height = '100%';
      let offsets = a.getBoundingClientRect();
      let ratio = this.photo.ratio;
      let maxHeight = {
        width: offsets.height / ratio,
        height: offsets.height
      };
      let maxWidth = {
        width: offsets.width,
        height: offsets.width * ratio
      };
      let usableSize = null;

      if (ratio <= 1) {
        usableSize = maxWidth.height > offsets.height ? maxHeight : maxWidth;
      } else {
        usableSize = maxHeight.width > offsets.width ? maxWidth : maxHeight;
      }

      a.style.width = "".concat(usableSize.width, "px");
      a.style.height = "".concat(usableSize.height, "px");
    },

    _setupResizeHandler() {
      if (this._resizeHandler) {
        return;
      }

      this._resizeHandler = Ember.run.bind(this, this._handleResize);
      window.addEventListener('resize', this._resizeHandler);
    },

    _teardownResizeHandler() {
      window.removeEventListener('resize', this._resizeHandler);
      this._resizeHandler = null;
    },

    _handleResize() {
      this._throttleResize = Ember.run.throttle(this, this._setZoomedSize, 100);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-unsplash", ["exports", "ghost-admin/mixins/shortcuts"], function (_exports, _shortcuts) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const ONE_COLUMN_WIDTH = 540;
  const TWO_COLUMN_WIDTH = 940;

  var _default = Ember.Component.extend(_shortcuts.default, {
    resizeDetector: Ember.inject.service(),
    unsplash: Ember.inject.service(),
    ui: Ember.inject.service(),
    shortcuts: null,
    tagName: '',
    zoomedPhoto: null,
    searchTerm: null,

    // closure actions
    close() {},

    select() {},

    sideNavHidden: Ember.computed.or('ui.{isFullScreen,showMobileMenu}'),

    init() {
      this._super(...arguments);

      this.shortcuts = {
        escape: {
          action: 'handleEscape',
          scope: 'all'
        }
      };
    },

    didReceiveAttrs() {
      this._super(...arguments);

      if (this.searchTerm !== this._searchTerm) {
        this.unsplash.updateSearch(this.searchTerm);
      }

      this._searchTerm = this.searchTerm;
    },

    didInsertElement() {
      this._super(...arguments);

      this._resizeCallback = Ember.run.bind(this, this._handleResize);
      this.resizeDetector.setup('[data-unsplash]', this._resizeCallback);
      this.registerShortcuts();
    },

    willDestroyElement() {
      this.resizeDetector.teardown('[data-unsplash]', this._resizeCallback);
      this.removeShortcuts();
      this.send('resetKeyScope');

      this._super(...arguments);
    },

    actions: {
      loadNextPage() {
        this.unsplash.loadNextPage();
      },

      search(term) {
        this.unsplash.updateSearch(term);
        this.send('closeZoom');
      },

      zoomPhoto(photo) {
        this.set('zoomedPhoto', photo);
      },

      closeZoom() {
        this.set('zoomedPhoto', null);
      },

      select(photo) {
        this.unsplash.triggerDownload(photo);
        let selectParams = {
          src: photo.urls.regular,
          alt: photo.description || '',
          caption: "Photo by <a href=\"".concat(photo.user.links.html, "?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit\">").concat(photo.user.name, "</a> / <a href=\"https://unsplash.com/?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit\">Unsplash</a>")
        };
        this.select(selectParams);
        this.close();
      },

      close() {
        this.close();
      },

      retry() {
        this.unsplash.retryLastRequest();
      },

      setKeyScope() {
        key.setScope('unsplash');
      },

      resetKeyScope() {
        key.setScope('default');
      },

      handleEscape() {
        if (this.zoomedPhoto) {
          return this.send('closeZoom');
        }

        this.close();
      }

    },

    _handleResize(element) {
      let width = element.clientWidth;
      let columns = 3;

      if (width <= ONE_COLUMN_WIDTH) {
        columns = 1;
      } else if (width <= TWO_COLUMN_WIDTH) {
        columns = 2;
      }

      this.unsplash.changeColumnCount(columns);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-uploader", ["exports", "ghost-admin/utils/ghost-paths", "ember-concurrency"], function (_exports, _ghostPaths, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  // TODO: this is designed to be a more re-usable/composable upload component, it
  // should be able to replace the duplicated upload logic in:
  // - gh-image-uploader
  // - gh-file-uploader
  //
  // In order to support the above components we'll need to introduce an
  // "allowMultiple" attribute so that single-image uploads don't allow multiple
  // simultaneous uploads
  const MAX_SIMULTANEOUS_UPLOADS = 2;
  /**
   * Result from a file upload
   * @typedef {Object} UploadResult
   * @property {string} fileName - file name, eg "my-image.png"
   * @property {string} url - url relative to Ghost root,eg "/content/images/2017/05/my-image.png"
   */

  const UploadTracker = Ember.Object.extend({
    file: null,
    total: 0,
    loaded: 0,

    init() {
      this._super(...arguments);

      this.total = this.file && this.file.size || 0;
    },

    update(_ref) {
      let {
        loaded,
        total
      } = _ref;
      this.total = total;
      this.loaded = loaded;
    }

  });

  var _default = Ember.Component.extend({
    ajax: Ember.inject.service(),
    tagName: '',
    // Public attributes
    accept: '',
    extensions: '',
    files: null,
    paramName: 'file',
    paramsHash: null,
    resourceName: 'images',
    uploadUrl: null,
    // Interal attributes
    errors: null,
    // [{fileName: 'x', message: 'y'}, ...]
    totalSize: 0,
    uploadedSize: 0,
    uploadPercentage: 0,
    uploadUrls: null,
    // [{filename: 'x', url: 'y'}],
    // Private
    _defaultUploadUrl: '/images/upload/',
    _files: null,
    _uploadTrackers: null,

    // Closure actions
    onCancel() {},

    onComplete() {},

    onFailed() {},

    onStart() {},

    onUploadStart() {},

    onUploadFailure() {},

    onUploadSuccess() {},

    // Optional closure actions
    // validate(file) {}
    init() {
      this._super(...arguments);

      this.set('errors', []);
      this.set('uploadUrls', []);
      this._uploadTrackers = [];

      if (!this.paramsHash) {
        this.set('paramsHash', {
          purpose: 'image'
        });
      }
    },

    didReceiveAttrs() {
      this._super(...arguments); // set up any defaults


      if (!this.uploadUrl) {
        this.set('uploadUrl', this._defaultUploadUrl);
      } // if we have new files, validate and start an upload


      let files = this.files;

      this._setFiles(files);
    },

    actions: {
      setFiles(files, resetInput) {
        this._setFiles(files);

        if (resetInput) {
          resetInput();
        }
      },

      cancel() {
        this._reset();

        this.onCancel();
      }

    },

    _setFiles(files) {
      this.set('files', files);

      if (files && files !== this._files) {
        if (this.get('_uploadFiles.isRunning')) {
          // eslint-disable-next-line
          console.error('Adding new files whilst an upload is in progress is not supported.');
        }

        this._files = files; // we cancel early if any file fails client-side validation

        if (this._validate()) {
          this._uploadFiles.perform(files);
        }
      }
    },

    _validate() {
      let files = this.files;

      let validate = this.validate || this._defaultValidator.bind(this);

      let ok = [];
      let errors = []; // NOTE: for...of loop results in a transpilation that errors in Edge,
      // once we drop IE11 support we should be able to use native for...of

      for (let i = 0; i < files.length; i += 1) {
        let file = files[i];
        let result = validate(file);

        if (result === true) {
          ok.push(file);
        } else {
          errors.push({
            fileName: file.name,
            message: result
          });
        }
      }

      if (Ember.isEmpty(errors)) {
        return true;
      }

      this.set('errors', errors);
      this.onFailed(errors);
      return false;
    },

    // we only check the file extension by default because IE doesn't always
    // expose the mime-type, we'll rely on the API for final validation
    _defaultValidator(file) {
      let extensions = this.extensions;
      let [, extension] = /(?:\.([^.]+))?$/.exec(file.name); // if extensions is falsy exit early and accept all files

      if (!extensions) {
        return true;
      }

      if (!Ember.isArray(extensions)) {
        extensions = extensions.split(',');
      }

      if (!extension || extensions.indexOf(extension.toLowerCase()) === -1) {
        let validExtensions = ".".concat(extensions.join(', .').toUpperCase());
        return "The file type you uploaded is not supported. Please use ".concat(validExtensions);
      }

      return true;
    },

    _uploadFiles: (0, _emberConcurrency.task)(function* (files) {
      let uploads = [];

      this._reset();

      this.onStart(files); // NOTE: for...of loop results in a transpilation that errors in Edge,
      // once we drop IE11 support we should be able to use native for...of

      for (let i = 0; i < files.length; i += 1) {
        let file = files[i];
        let tracker = UploadTracker.create({
          file
        });

        this._uploadTrackers.pushObject(tracker);

        uploads.push(this._uploadFile.perform(tracker, file, i));
      } // populates this.errors and this.uploadUrls


      yield (0, _emberConcurrency.all)(uploads);

      if (!Ember.isEmpty(this.errors)) {
        this.onFailed(this.errors);
      }

      this.onComplete(this.uploadUrls);
    }).drop(),
    // eslint-disable-next-line ghost/ember/order-in-components
    _uploadFile: (0, _emberConcurrency.task)(function* (tracker, file, index) {
      let ajax = this.ajax;

      let formData = this._getFormData(file);

      let url = "".concat((0, _ghostPaths.default)().apiRoot).concat(this.uploadUrl);

      try {
        this.onUploadStart(file);
        let response = yield ajax.post(url, {
          data: formData,
          processData: false,
          contentType: false,
          dataType: 'text',
          xhr: () => {
            let xhr = new window.XMLHttpRequest();
            xhr.upload.addEventListener('progress', event => {
              Ember.run(() => {
                tracker.update(event);

                this._updateProgress();
              });
            }, false);
            return xhr;
          }
        }); // force tracker progress to 100% in case we didn't get a final event,
        // eg. when using mirage

        tracker.update({
          loaded: file.size,
          total: file.size
        });

        this._updateProgress();

        let uploadResponse;
        let responseUrl;

        try {
          uploadResponse = JSON.parse(response);
        } catch (e) {
          if (!(e instanceof SyntaxError)) {
            throw e;
          }
        }

        if (uploadResponse) {
          let resource = Ember.get(uploadResponse, this.resourceName);

          if (resource && Ember.isArray(resource) && resource[0]) {
            responseUrl = Ember.get(resource[0], 'url');
          }
        }

        let result = {
          url: responseUrl,
          fileName: file.name
        };
        this.uploadUrls[index] = result;
        this.onUploadSuccess(result);
        return true;
      } catch (error) {
        // grab custom error message if present
        let message = error.payload.errors && error.payload.errors[0].message || '';
        let context = error.payload.errors && error.payload.errors[0].context || ''; // fall back to EmberData/ember-ajax default message for error type

        if (!message) {
          message = error.message;
        }

        let result = {
          message,
          context,
          fileName: file.name
        }; // TODO: check for or expose known error types?

        this.errors.pushObject(result);
        this.onUploadFailure(result);
      }
    }).maxConcurrency(MAX_SIMULTANEOUS_UPLOADS).enqueue(),

    // NOTE: this is necessary because the API doesn't accept direct file uploads
    _getFormData(file) {
      let formData = new FormData();
      formData.append(this.paramName, file, file.name);
      Object.keys(this.paramsHash || {}).forEach(key => {
        formData.append(key, this.paramsHash[key]);
      });
      return formData;
    },

    // TODO: this was needed because using CPs directly resulted in infrequent updates
    // - I think this was because updates were being wrapped up to save
    // computation but that hypothesis needs testing
    _updateProgress() {
      let trackers = this._uploadTrackers;
      let totalSize = trackers.reduce((total, tracker) => total + tracker.get('total'), 0);
      let uploadedSize = trackers.reduce((total, tracker) => total + tracker.get('loaded'), 0);
      this.set('totalSize', totalSize);
      this.set('uploadedSize', uploadedSize);

      if (totalSize === 0 || uploadedSize === 0) {
        return;
      }

      let uploadPercentage = Math.round(uploadedSize / totalSize * 100);
      this.set('uploadPercentage', uploadPercentage);
    },

    _reset() {
      this.set('errors', []);
      this.set('totalSize', 0);
      this.set('uploadedSize', 0);
      this.set('uploadPercentage', 0);
      this.set('uploadUrls', []);
      this._uploadTrackers = [];
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-url-preview", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /*
  Example usage:
  {{gh-url-preview prefix="tag" slug=theSlugValue tagName="p" classNames="description"}}
  */
  var _default = Ember.Component.extend({
    config: Ember.inject.service(),
    classNames: 'ghost-url-preview',
    prefix: null,
    slug: null,
    url: Ember.computed('slug', function () {
      // Get the blog URL and strip the scheme
      let blogUrl = this.get('config.blogUrl'); // Remove `http[s]://`

      let noSchemeBlogUrl = blogUrl.substr(blogUrl.indexOf('://') + 3); // Get the prefix and slug values

      let prefix = this.prefix ? "".concat(this.prefix, "/") : '';
      let slug = this.slug ? "".concat(this.slug, "/") : ''; // Join parts of the URL together with slashes

      let theUrl = "".concat(noSchemeBlogUrl, "/").concat(prefix).concat(slug);
      return theUrl;
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-user-active", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: '',
    user: null,
    lastLoginUTC: Ember.computed('user.lastLoginUTC', function () {
      let lastLoginUTC = this.get('user.lastLoginUTC');
      return lastLoginUTC ? (0, _moment.default)(lastLoginUTC).fromNow() : '(Never)';
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-user-list-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: ''
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-validation-status-container", ["exports", "ghost-admin/mixins/validation-state"], function (_exports, _validationState) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * Handles the CSS necessary to show a specific property state. When passed a
   * DS.Errors object and a property name, if the DS.Errors object has errors for
   * the specified property, it will change the CSS to reflect the error state
   * @param  {DS.Errors} errors   The DS.Errors object
   * @param  {string} property    Name of the property
   */
  var _default = Ember.Component.extend(_validationState.default, {
    classNameBindings: ['errorClass'],
    errorClass: Ember.computed('property', 'hasError', 'hasValidated.[]', function () {
      let hasValidated = this.hasValidated;
      let property = this.property;

      if (hasValidated && hasValidated.includes(property)) {
        return this.hasError ? 'error' : 'success';
      } else {
        return '';
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/gh-view-title", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    ui: Ember.inject.service(),
    tagName: 'h2',
    classNames: ['view-title']
  });

  _exports.default = _default;
});
;define("ghost-admin/components/illiquid-model", ["exports", "liquid-fire/components/illiquid-model"], function (_exports, _illiquidModel) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _illiquidModel.default;
    }
  });
});
;define("ghost-admin/components/infinity-loader", ["exports", "ember-infinity/components/infinity-loader", "ghost-admin/templates/components/gh-infinity-loader"], function (_exports, _infinityLoader, _ghInfinityLoader) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _infinityLoader.default.extend({
    layout: _ghInfinityLoader.default
  });

  _exports.default = _default;
});
;define("ghost-admin/components/kg-action-bar", ["exports", "koenig-editor/components/kg-action-bar"], function (_exports, _kgActionBar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _kgActionBar.default;
    }
  });
});
;define("ghost-admin/components/koenig-basic-html-input", ["exports", "koenig-editor/components/koenig-basic-html-input"], function (_exports, _koenigBasicHtmlInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigBasicHtmlInput.default;
    }
  });
});
;define("ghost-admin/components/koenig-caption-input", ["exports", "koenig-editor/components/koenig-caption-input"], function (_exports, _koenigCaptionInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCaptionInput.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-code", ["exports", "koenig-editor/components/koenig-card-code"], function (_exports, _koenigCardCode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardCode.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-embed", ["exports", "koenig-editor/components/koenig-card-embed"], function (_exports, _koenigCardEmbed) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardEmbed.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-gallery", ["exports", "koenig-editor/components/koenig-card-gallery"], function (_exports, _koenigCardGallery) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardGallery.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-hr", ["exports", "koenig-editor/components/koenig-card-hr"], function (_exports, _koenigCardHr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardHr.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-html", ["exports", "koenig-editor/components/koenig-card-html"], function (_exports, _koenigCardHtml) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardHtml.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-image", ["exports", "koenig-editor/components/koenig-card-image"], function (_exports, _koenigCardImage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardImage.default;
    }
  });
});
;define("ghost-admin/components/koenig-card-markdown", ["exports", "koenig-editor/components/koenig-card-markdown"], function (_exports, _koenigCardMarkdown) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCardMarkdown.default;
    }
  });
});
;define("ghost-admin/components/koenig-card", ["exports", "koenig-editor/components/koenig-card"], function (_exports, _koenigCard) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigCard.default;
    }
  });
});
;define("ghost-admin/components/koenig-editor", ["exports", "koenig-editor/components/koenig-editor"], function (_exports, _koenigEditor) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigEditor.default;
    }
  });
});
;define("ghost-admin/components/koenig-link-input", ["exports", "koenig-editor/components/koenig-link-input"], function (_exports, _koenigLinkInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigLinkInput.default;
    }
  });
});
;define("ghost-admin/components/koenig-link-toolbar", ["exports", "koenig-editor/components/koenig-link-toolbar"], function (_exports, _koenigLinkToolbar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigLinkToolbar.default;
    }
  });
});
;define("ghost-admin/components/koenig-menu-content", ["exports", "koenig-editor/components/koenig-menu-content"], function (_exports, _koenigMenuContent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigMenuContent.default;
    }
  });
});
;define("ghost-admin/components/koenig-plus-menu", ["exports", "koenig-editor/components/koenig-plus-menu"], function (_exports, _koenigPlusMenu) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigPlusMenu.default;
    }
  });
});
;define("ghost-admin/components/koenig-slash-menu", ["exports", "koenig-editor/components/koenig-slash-menu"], function (_exports, _koenigSlashMenu) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigSlashMenu.default;
    }
  });
});
;define("ghost-admin/components/koenig-toolbar", ["exports", "koenig-editor/components/koenig-toolbar"], function (_exports, _koenigToolbar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigToolbar.default;
    }
  });
});
;define("ghost-admin/components/liquid-append", ["exports", "liquid-wormhole/components/liquid-append"], function (_exports, _liquidAppend) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidAppend.default;
    }
  });
});
;define("ghost-admin/components/liquid-bind", ["exports", "liquid-fire/components/liquid-bind"], function (_exports, _liquidBind) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidBind.default;
    }
  });
});
;define("ghost-admin/components/liquid-child", ["exports", "liquid-fire/components/liquid-child"], function (_exports, _liquidChild) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidChild.default;
    }
  });
});
;define("ghost-admin/components/liquid-container", ["exports", "liquid-fire/components/liquid-container"], function (_exports, _liquidContainer) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidContainer.default;
    }
  });
});
;define("ghost-admin/components/liquid-destination", ["exports", "liquid-wormhole/components/liquid-destination"], function (_exports, _liquidDestination) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidDestination.default;
    }
  });
});
;define("ghost-admin/components/liquid-if", ["exports", "liquid-fire/components/liquid-if"], function (_exports, _liquidIf) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidIf.default;
    }
  });
});
;define("ghost-admin/components/liquid-measured", ["exports", "liquid-fire/components/liquid-measured"], function (_exports, _liquidMeasured) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidMeasured.default;
    }
  });
  Object.defineProperty(_exports, "measure", {
    enumerable: true,
    get: function get() {
      return _liquidMeasured.measure;
    }
  });
});
;define("ghost-admin/components/liquid-outlet", ["exports", "liquid-fire/components/liquid-outlet"], function (_exports, _liquidOutlet) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidOutlet.default;
    }
  });
});
;define("ghost-admin/components/liquid-spacer", ["exports", "liquid-fire/components/liquid-spacer"], function (_exports, _liquidSpacer) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidSpacer.default;
    }
  });
});
;define("ghost-admin/components/liquid-sync", ["exports", "liquid-fire/components/liquid-sync"], function (_exports, _liquidSync) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidSync.default;
    }
  });
});
;define("ghost-admin/components/liquid-tether", ["exports", "liquid-tether/components/liquid-tether"], function (_exports, _liquidTether) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidTether.default;
    }
  });
});
;define("ghost-admin/components/liquid-unless", ["exports", "liquid-fire/components/liquid-unless"], function (_exports, _liquidUnless) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidUnless.default;
    }
  });
});
;define("ghost-admin/components/liquid-versions", ["exports", "liquid-fire/components/liquid-versions"], function (_exports, _liquidVersions) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidVersions.default;
    }
  });
});
;define("ghost-admin/components/liquid-wormhole", ["exports", "liquid-wormhole/components/liquid-wormhole"], function (_exports, _liquidWormhole) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidWormhole.default;
    }
  });
});
;define("ghost-admin/components/modal-base", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({
    tagName: 'section',
    classNames: 'modal-content',
    _previousKeymasterScope: null,
    // Allowed Actions
    closeModal: () => {},

    didInsertElement() {
      this._super(...arguments);

      this._setupShortcuts();
    },

    willDestroyElement() {
      this._super(...arguments);

      this._removeShortcuts();
    },

    actions: {
      confirm() {
        throw new Error('You must override the "confirm" action in your modal component');
      },

      closeModal() {
        this.closeModal();
      }

    },

    _setupShortcuts() {
      Ember.run(function () {
        document.activeElement.blur();
      });
      this._previousKeymasterScope = key.getScope();
      key('enter', 'modal', () => {
        this.send('confirm');
      });
      key('escape', 'modal', () => {
        this.send('closeModal');
      });
      key.setScope('modal');
    },

    _removeShortcuts() {
      key.unbind('enter', 'modal');
      key.unbind('escape', 'modal');
      key.setScope(this._previousKeymasterScope);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-all", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    store: Ember.inject.service(),
    ajax: Ember.inject.service(),
    actions: {
      confirm() {
        this.deleteAll.perform();
      }

    },

    _deleteAll() {
      let deleteUrl = this.get('ghostPaths.url').api('db');
      return this.ajax.del(deleteUrl);
    },

    _unloadData() {
      this.store.unloadAll('post');
      this.store.unloadAll('tag');
    },

    _showSuccess() {
      this.notifications.showAlert('All content deleted from database.', {
        type: 'success',
        key: 'all-content.delete.success'
      });
    },

    _showFailure(error) {
      this.notifications.showAPIError(error, {
        key: 'all-content.delete'
      });
    },

    deleteAll: (0, _emberConcurrency.task)(function* () {
      try {
        yield this._deleteAll();

        this._unloadData();

        this._showSuccess();
      } catch (error) {
        this._showFailure(error);
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-integration", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    router: Ember.inject.service(),
    notifications: Ember.inject.service(),
    integration: Ember.computed.alias('model'),
    actions: {
      confirm() {
        this.deleteIntegration.perform();
      }

    },
    deleteIntegration: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
        this.router.transitionTo('settings.integrations');
      } catch (error) {
        this.notifications.showAPIError(error, {
          key: 'integration.delete.failed'
        });
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-member", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    notifications: Ember.inject.service(),
    member: Ember.computed.alias('model.member'),
    onSuccess: Ember.computed.alias('model.onSuccess'),
    actions: {
      confirm() {
        this.deleteMember.perform();
      }

    },

    _success() {
      // clear any previous error messages
      this.notifications.closeAlerts('post.delete'); // trigger the success action

      if (this.onSuccess) {
        this.onSuccess();
      }
    },

    _failure(error) {
      this.notifications.showAPIError(error, {
        key: 'post.delete.failed'
      });
    },

    deleteMember: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.member.destroyRecord();

        this._success();
      } catch (e) {
        this._failure(e);
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-post", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    notifications: Ember.inject.service(),
    post: Ember.computed.alias('model.post'),
    onSuccess: Ember.computed.alias('model.onSuccess'),
    actions: {
      confirm() {
        this.deletePost.perform();
      }

    },

    _deletePost() {
      let post = this.post; // definitely want to clear the data store and post of any unsaved,
      // client-generated tags

      post.updateTags();
      return post.destroyRecord();
    },

    _success() {
      // clear any previous error messages
      this.notifications.closeAlerts('post.delete'); // trigger the success action

      if (this.onSuccess) {
        this.onSuccess();
      }
    },

    _failure(error) {
      this.notifications.showAPIError(error, {
        key: 'post.delete.failed'
      });
    },

    deletePost: (0, _emberConcurrency.task)(function* () {
      try {
        yield this._deletePost();

        this._success();
      } catch (e) {
        this._failure(e);
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-subscriber", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    // Allowed actions
    confirm: () => {},
    subscriber: Ember.computed.alias('model'),
    actions: {
      confirm() {
        this.deleteSubscriber.perform();
      }

    },
    deleteSubscriber: (0, _emberConcurrency.task)(function* () {
      yield this.confirm();
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-tag", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    // Allowed actions
    confirm: () => {},
    tag: Ember.computed.alias('model'),
    postInflection: Ember.computed('tag.count.posts', function () {
      return this.get('tag.count.posts') > 1 ? 'posts' : 'post';
    }),
    actions: {
      confirm() {
        this.deleteTag.perform();
      }

    },
    deleteTag: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-theme", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    // Allowed actions
    confirm: () => {},
    theme: Ember.computed.alias('model.theme'),
    download: Ember.computed.alias('model.download'),
    actions: {
      confirm() {
        this.deleteTheme.perform();
      }

    },
    deleteTheme: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-user", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    // Allowed actions
    confirm: () => {},
    user: Ember.computed.alias('model'),
    actions: {
      confirm() {
        this.deleteUser.perform();
      }

    },
    deleteUser: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-delete-webhook", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    notifications: Ember.inject.service(),
    webhook: Ember.computed.alias('model'),
    actions: {
      confirm() {
        this.deleteWebhook.perform();
      }

    },
    deleteWebhook: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } catch (error) {
        this.notifications.showAPIError(error, {
          key: 'webhook.delete.failed'
        });
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-import-subscribers", ["exports", "ghost-admin/components/modal-base", "ghost-admin/utils/ghost-paths"], function (_exports, _modalBase, _ghostPaths) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    labelText: 'Select or drag-and-drop a CSV File',
    response: null,
    closeDisabled: false,
    // Allowed actions
    confirm: () => {},
    uploadUrl: Ember.computed(function () {
      return "".concat((0, _ghostPaths.default)().apiRoot, "/subscribers/csv/");
    }),
    actions: {
      uploadStarted() {
        this.set('closeDisabled', true);
      },

      uploadFinished() {
        this.set('closeDisabled', false);
      },

      uploadSuccess(response) {
        this.set('response', response.meta.stats); // invoke the passed in confirm action

        this.confirm();
      },

      confirm() {// noop - we don't want the enter key doing anything
      },

      closeModal() {
        if (!this.closeDisabled) {
          this._super(...arguments);
        }
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-invite-new-user", ["exports", "ghost-admin/components/modal-base", "ghost-admin/mixins/validation-engine", "ember-concurrency"], function (_exports, _modalBase, _validationEngine, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    Promise
  } = Ember.RSVP;

  var _default = _modalBase.default.extend(_validationEngine.default, {
    notifications: Ember.inject.service(),
    store: Ember.inject.service(),
    rcService: Ember.inject.service('rc-services'),
    classNames: 'modal-content invite-new-user',
    role: null,
    roles: null,
    defaultRole: null,
    validationType: 'inviteUser',

    init() {
      this._super(...arguments);
    },

    didInsertElement() {
      this._super(...arguments);

      this.fetchRoles.perform();
    },

    willDestroyElement() {
      this._super(...arguments); // TODO: this should not be needed, ValidationEngine acts as a
      // singleton and so it's errors and hasValidated state stick around


      this.errors.clear();
      this.set('hasValidated', Ember.A());
    },

    actions: {
      setRole(role) {
        this.set('role', role);
        this.errors.remove('role');
      },

      confirm() {
        this.sendInvitation.perform();
      }

    },

    validate() {
      let username = this.username; // TODO: either the validator should check the username's existence or
      // the API should return an appropriate error when attempting to save

      return new Promise((resolve, reject) => this._super().then(() => Ember.RSVP.hash({
        rc_users: this.rcService.getUser(username),
        users: this.store.findAll('user', {
          reload: true
        })
      }).then(data => {
        let existingRCUser = data.rc_users.data[0].exist;
        let existingUser = data.users.findBy('rc_username', username);

        if (existingUser || !existingRCUser) {
          this.errors.clear('username');

          if (existingUser) {
            this.errors.add('username', 'A user with that username already exists.');
          } else {
            this.errors.add('username', 'Username doesnot exist');
          } // TODO: this shouldn't be needed, ValidationEngine doesn't mark
          // properties as validated when validating an entire object


          this.hasValidated.addObject('username');
          reject();
        } else {
          resolve();
        }
      }), () => {
        // TODO: this shouldn't be needed, ValidationEngine doesn't mark
        // properties as validated when validating an entire object
        this.hasValidated.addObject('username');
        reject();
      }));
    },

    fetchRoles: (0, _emberConcurrency.task)(function* () {
      let roles = yield this.store.query('role', {
        permissions: 'assign'
      });
      let defaultRole = roles.findBy('name', 'Author');
      defaultRole = defaultRole ? defaultRole : roles.findBy('name', 'Contributor');
      this.set('roles', roles);
      this.set('defaultRole', defaultRole);

      if (!this.role) {
        this.set('role', defaultRole);
      }
    }),
    sendInvitation: (0, _emberConcurrency.task)(function* () {
      let username = this.username;
      let role = this.role;
      let notifications = this.notifications;

      try {
        yield this.validate();
        const user = yield this.rcService.addUser(username, role); // Check and notify if user is added

        if (user.invitation && user.invitation[0].message === 'User Added') {
          notifications.showNotification(user.invitation[0].message, {
            key: 'invite.send.success'
          });
        } else {
          notifications.showAlert('Unable to add user', {
            type: 'error',
            key: 'invite.send.failed'
          });
        }

        this.send('closeModal');
      } catch (error) {
        // validation will reject and cause this to be called with no error
        if (error) {
          // invite.deleteRecord();
          notifications.showAPIError(error, {
            key: 'invite.send'
          });
          this.send('closeModal');
        }
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-leave-editor", ["exports", "ghost-admin/components/modal-base"], function (_exports, _modalBase) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    actions: {
      confirm() {
        this.confirm();
        this.send('closeModal');
      }

    },
    // Allowed actions
    confirm: () => Ember.RSVP.resolve()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-leave-settings", ["exports", "ghost-admin/components/modal-base"], function (_exports, _modalBase) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    actions: {
      confirm() {
        this.confirm().finally(() => {
          if (!this.isDestroyed && !this.isDestroying) {
            this.send('closeModal');
          }
        });
      }

    },
    // Allowed actions
    confirm: () => Ember.RSVP.resolve()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-markdown-help", ["exports", "ghost-admin/components/modal-base"], function (_exports, _modalBase) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({});

  _exports.default = _default;
});
;define("ghost-admin/components/modal-new-integration", ["exports", "ghost-admin/components/modal-base", "ember-ajax/errors", "ember-concurrency"], function (_exports, _modalBase, _errors, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    router: Ember.inject.service(),

    confirm() {},

    integration: Ember.computed.alias('model'),
    actions: {
      updateName(name) {
        this.integration.set('name', name);
        this.integration.set('hasValidated', Ember.A());
        this.integration.errors.clear();
      },

      confirm() {
        return this.createIntegration.perform();
      }

    },
    createIntegration: (0, _emberConcurrency.task)(function* () {
      try {
        let integration = yield this.confirm();
        this.router.transitionTo('settings.integration', integration);
      } catch (error) {
        // TODO: server-side validation errors should be serialized
        // properly so that errors are added to model.errors automatically
        if (error && (0, _errors.isInvalidError)(error)) {
          let [firstError] = error.payload.errors;
          let {
            message
          } = firstError;

          if (message && message.match(/name/i)) {
            this.get('integration.errors').add('name', message);
            this.get('integration.hasValidated').pushObject('name');
            return;
          }
        } // bubble up to the global error handler


        if (error) {
          throw error;
        }
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-new-subscriber", ["exports", "ghost-admin/components/modal-base", "ember-ajax/errors", "ember-concurrency"], function (_exports, _modalBase, _errors, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    subscriber: Ember.computed.alias('model'),
    actions: {
      updateEmail(newEmail) {
        this.set('subscriber.email', newEmail);
        this.set('subscriber.hasValidated', Ember.A());
        this.get('subscriber.errors').clear();
      },

      confirm() {
        this.addSubscriber.perform();
      }

    },
    addSubscriber: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
        this.send('closeModal');
      } catch (error) {
        // TODO: server-side validation errors should be serialized
        // properly so that errors are added to model.errors automatically
        if (error && (0, _errors.isInvalidError)(error)) {
          let [firstError] = error.payload.errors;
          let {
            context
          } = firstError;

          if (context && context.match(/email/i)) {
            this.get('subscriber.errors').add('email', context);
            this.get('subscriber.hasValidated').pushObject('email');
            return;
          }
        } // route action so it should bubble up to the global error handler


        if (error) {
          throw error;
        }
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-suspend-user", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    // Allowed actions
    confirm: () => {},
    user: Ember.computed.alias('model'),
    actions: {
      confirm() {
        return this.suspendUser.perform();
      }

    },
    suspendUser: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-theme-warnings", ["exports", "ghost-admin/components/modal-base"], function (_exports, _modalBase) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    'data-test-theme-warnings-modal': true,
    title: Ember.computed.reads('model.title'),
    message: Ember.computed.reads('model.message'),
    warnings: Ember.computed.reads('model.warnings'),
    errors: Ember.computed.reads('model.errors'),
    fatalErrors: Ember.computed.reads('model.fatalErrors'),
    canActivate: Ember.computed.reads('model.canActivate')
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-transfer-owner", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    user: null,
    // Allowed actions
    confirm: () => {},
    actions: {
      confirm() {
        this.transferOwnership.perform();
      }

    },
    transferOwnership: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-unsuspend-user", ["exports", "ghost-admin/components/modal-base", "ember-concurrency"], function (_exports, _modalBase, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    // Allowed actions
    confirm: () => {},
    user: Ember.computed.alias('model'),
    actions: {
      confirm() {
        return this.unsuspendUser.perform();
      }

    },
    unsuspendUser: (0, _emberConcurrency.task)(function* () {
      try {
        yield this.confirm();
      } finally {
        this.send('closeModal');
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-upload-image", ["exports", "ghost-admin/components/modal-base", "ghost-admin/utils/caja-sanitizers", "ember-concurrency"], function (_exports, _modalBase, _cajaSanitizers, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    config: Ember.inject.service(),
    notifications: Ember.inject.service(),
    model: null,
    url: '',
    newUrl: '',
    _isUploading: false,
    image: Ember.computed('model.{model,imageProperty}', {
      get() {
        let imageProperty = this.get('model.imageProperty');
        return this.get("model.model.".concat(imageProperty));
      },

      set(key, value) {
        let model = this.get('model.model');
        let imageProperty = this.get('model.imageProperty');
        return model.set(imageProperty, value);
      }

    }),

    didReceiveAttrs() {
      let image = this.image;
      this.set('url', image);
      this.set('newUrl', image);
    },

    actions: {
      fileUploaded(url) {
        this.set('url', url);
        this.set('newUrl', url);
      },

      removeImage() {
        this.set('url', '');
        this.set('newUrl', '');
      },

      confirm() {
        this.uploadImage.perform();
      },

      isUploading() {
        this.toggleProperty('_isUploading');
      }

    },

    // TODO: should validation be handled in the gh-image-uploader component?
    //  pro - consistency everywhere, simplification here
    //  con - difficult if the "save" is happening externally as it does here
    //
    //  maybe it should be handled at the model level?
    //      - automatically present everywhere
    //      - file uploads should always result in valid urls so it should only
    //        affect the url input form
    keyDown() {
      this._setErrorState(false);
    },

    _setErrorState(state) {
      if (state) {
        this.$('.url').addClass('error');
      } else {
        this.$('.url').removeClass('error');
      }
    },

    _validateUrl(url) {
      if (!Ember.isEmpty(url) && !_cajaSanitizers.default.url(url)) {
        this._setErrorState(true);

        return {
          message: 'Image URI is not valid'
        };
      }

      return true;
    },

    // end validation
    uploadImage: (0, _emberConcurrency.task)(function* () {
      let model = this.get('model.model');
      let newUrl = this.newUrl;

      let result = this._validateUrl(newUrl);

      let notifications = this.notifications;

      if (result === true) {
        this.set('image', newUrl);

        try {
          yield model.save();
        } catch (e) {
          notifications.showAPIError(e, {
            key: 'image.upload'
          });
        } finally {
          this.send('closeModal');
        }
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-upload-theme", ["exports", "ghost-admin/components/modal-base", "ghost-admin/utils/ghost-paths", "ghost-admin/services/ajax"], function (_exports, _modalBase, _ghostPaths, _ajax) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const DEFAULTS = {
    accept: ['application/zip', 'application/x-zip-compressed'],
    extensions: ['zip']
  };

  var _default = _modalBase.default.extend({
    eventBus: Ember.inject.service(),
    store: Ember.inject.service(),
    accept: null,
    extensions: null,
    themes: null,
    closeDisabled: false,
    file: null,
    theme: false,
    displayOverwriteWarning: false,
    hideUploader: Ember.computed.or('theme', 'displayOverwriteWarning'),
    currentThemeNames: Ember.computed.mapBy('model.themes', 'name'),
    uploadUrl: Ember.computed(function () {
      return "".concat((0, _ghostPaths.default)().apiRoot, "/themes/upload/");
    }),
    themeName: Ember.computed('theme.{name,package.name}', function () {
      let themePackage = this.get('theme.package');
      let name = this.get('theme.name');
      return themePackage ? "".concat(themePackage.name, " - ").concat(themePackage.version) : name;
    }),
    fileThemeName: Ember.computed('file', function () {
      let file = this.file;
      return file.name.replace(/\.zip$/, '');
    }),
    canActivateTheme: Ember.computed('theme', function () {
      let theme = this.theme;
      return theme && !theme.get('active');
    }),

    init() {
      this._super(...arguments);

      this.accept = this.accept || DEFAULTS.accept;
      this.extensions = this.extensions || DEFAULTS.extensions;
    },

    actions: {
      validateTheme(file) {
        let themeName = file.name.replace(/\.zip$/, '').replace(/[^\w@.]/gi, '-').toLowerCase();
        let currentThemeNames = this.currentThemeNames;
        this.set('file', file);
        let [, extension] = /(?:\.([^.]+))?$/.exec(file.name);
        let extensions = this.extensions;

        if (!extension || extensions.indexOf(extension.toLowerCase()) === -1) {
          return new _ajax.UnsupportedMediaTypeError();
        }

        if (file.name.match(/^casper\.zip$/i)) {
          return {
            payload: {
              errors: [{
                message: 'Sorry, the default Casper theme cannot be overwritten.<br>Please rename your zip file and try again.'
              }]
            }
          };
        }

        if (!this._allowOverwrite && currentThemeNames.includes(themeName)) {
          this.set('displayOverwriteWarning', true);
          return false;
        }

        return true;
      },

      confirmOverwrite() {
        this._allowOverwrite = true;
        this.set('displayOverwriteWarning', false); // we need to schedule afterRender so that the upload component is
        // displayed again in order to subscribe/respond to the event bus

        Ember.run.schedule('afterRender', this, function () {
          this.eventBus.publish('themeUploader:upload', this.file);
        });
      },

      uploadStarted() {
        this.set('closeDisabled', true);
      },

      uploadFinished() {
        this.set('closeDisabled', false);
      },

      uploadSuccess(response) {
        this.store.pushPayload(response);
        let theme = this.store.peekRecord('theme', response.themes[0].name);
        this.set('theme', theme);

        if (Ember.get(theme, 'warnings.length') > 0) {
          this.set('validationWarnings', Ember.get(theme, 'warnings'));
        } // Ghost differentiates between errors and fatal errors
        // You can't activate a theme with fatal errors, but with errors.


        if (Ember.get(theme, 'errors.length') > 0) {
          this.set('validationErrors', Ember.get(theme, 'errors'));
        }

        this.set('hasWarningsOrErrors', this.get('validationErrors.length') || this.get('validationWarnings.length')); // invoke the passed in confirm action

        this.get('model.uploadSuccess')(theme);
      },

      uploadFailed(error) {
        if ((0, _ajax.isThemeValidationError)(error)) {
          let errors = error.payload.errors[0].details.errors;
          let fatalErrors = [];
          let normalErrors = []; // to have a proper grouping of fatal errors and none fatal, we need to check
          // our errors for the fatal property

          if (errors && errors.length > 0) {
            for (let i = 0; i < errors.length; i += 1) {
              if (errors[i].fatal) {
                fatalErrors.push(errors[i]);
              } else {
                normalErrors.push(errors[i]);
              }
            }
          }

          this.set('fatalValidationErrors', fatalErrors);
          this.set('validationErrors', normalErrors);
        }
      },

      confirm() {// noop - we don't want the enter key doing anything
      },

      activate() {
        this.get('model.activate')(this.theme);
        this.closeModal();
      },

      closeModal() {
        if (!this.closeDisabled) {
          this._super(...arguments);
        }
      },

      reset() {
        this.set('theme', null);
        this.set('validationWarnings', []);
        this.set('validationErrors', []);
        this.set('fatalValidationErrors', []);
        this.set('hasWarningsOrErrors', false);
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/components/modal-webhook-form", ["exports", "ghost-admin/components/modal-base", "ghost-admin/models/webhook", "ghost-admin/helpers/event-name", "ember-concurrency"], function (_exports, _modalBase, _webhook, _eventName, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _modalBase.default.extend({
    config: Ember.inject.service(),
    router: Ember.inject.service(),
    availableEvents: null,
    error: null,
    buttonText: 'Save',
    successText: 'Saved',

    confirm() {},

    webhook: Ember.computed.alias('model'),

    init() {
      this._super(...arguments);

      this.availableEvents = _eventName.AVAILABLE_EVENTS;
    },

    didReceiveAttrs() {
      if (this.webhook.isNew) {
        this.set('buttonText', 'Create');
        this.set('successText', 'Created');
      }
    },

    actions: {
      selectEvent(value) {
        this.webhook.set('event', value);
        this.webhook.validate({
          property: 'event'
        });
      },

      confirm() {
        this.saveWebhook.perform();
      }

    },
    saveWebhook: (0, _emberConcurrency.task)(function* () {
      this.set('error', null);

      try {
        let webhook = yield this.confirm();
        let integration = yield webhook.get('integration');
        this.router.transitionTo('settings.integration', integration);
      } catch (error) {
        // TODO: server-side validation errors should be serialized
        // properly so that errors are added to model.errors automatically
        if (error && error.payload && error.payload.errors) {
          let attrs = Array.from(_webhook.default.attributes.keys());
          error.payload.errors.forEach(error => {
            let {
              message,
              property = ''
            } = error;
            property = Ember.String.camelize(property);

            if (property && attrs.includes(property)) {
              this.webhook.errors.add(property, message);
              this.webhook.hasValidated.pushObject(property);
            } else {
              this.set('error', "Error: ".concat(message));
            }
          });
          return;
        } // bubble up to the global error handler


        if (error) {
          throw error;
        }
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/components/object-bin", ["exports", "ember-drag-drop/components/object-bin"], function (_exports, _objectBin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _objectBin.default;
  _exports.default = _default;
});
;define("ghost-admin/components/one-way-select", ["exports", "ember-one-way-select/components/one-way-select"], function (_exports, _oneWaySelect) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _oneWaySelect.default;
    }
  });
});
;define("ghost-admin/components/one-way-select/option", ["exports", "ember-one-way-select/components/one-way-select/option"], function (_exports, _option) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _option.default;
    }
  });
});
;define("ghost-admin/components/power-calendar-multiple", ["exports", "ember-power-calendar/components/power-calendar-multiple"], function (_exports, _powerCalendarMultiple) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerCalendarMultiple.default;
    }
  });
});
;define("ghost-admin/components/power-calendar-multiple/days", ["exports", "ember-power-calendar/components/power-calendar-multiple/days"], function (_exports, _days) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _days.default;
    }
  });
});
;define("ghost-admin/components/power-calendar-range", ["exports", "ember-power-calendar/components/power-calendar-range"], function (_exports, _powerCalendarRange) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerCalendarRange.default;
    }
  });
});
;define("ghost-admin/components/power-calendar-range/days", ["exports", "ember-power-calendar/components/power-calendar-range/days"], function (_exports, _days) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _days.default;
    }
  });
});
;define("ghost-admin/components/power-calendar", ["exports", "ember-power-calendar/components/power-calendar"], function (_exports, _powerCalendar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerCalendar.default;
    }
  });
});
;define("ghost-admin/components/power-calendar/days", ["exports", "ember-power-calendar/components/power-calendar/days"], function (_exports, _days) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _days.default;
    }
  });
});
;define("ghost-admin/components/power-calendar/nav", ["exports", "ember-power-calendar/components/power-calendar/nav"], function (_exports, _nav) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _nav.default;
    }
  });
});
;define("ghost-admin/components/power-datepicker", ["exports", "ember-power-datepicker/components/power-datepicker"], function (_exports, _powerDatepicker) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerDatepicker.default;
    }
  });
});
;define("ghost-admin/components/power-select-multiple", ["exports", "ember-power-select/components/power-select-multiple"], function (_exports, _powerSelectMultiple) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerSelectMultiple.default;
    }
  });
});
;define("ghost-admin/components/power-select-multiple/trigger", ["exports", "ember-power-select/components/power-select-multiple/trigger"], function (_exports, _trigger) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _trigger.default;
    }
  });
});
;define("ghost-admin/components/power-select-vertical-collection-options", ["exports", "ember-power-select/components/power-select/options", "ghost-admin/templates/components/power-select-vertical-collection-options"], function (_exports, _options, _powerSelectVerticalCollectionOptions) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _options.default.extend({
    layout: _powerSelectVerticalCollectionOptions.default
  });

  _exports.default = _default;
});
;define("ghost-admin/components/power-select", ["exports", "ember-power-select/components/power-select"], function (_exports, _powerSelect) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerSelect.default;
    }
  });
});
;define("ghost-admin/components/power-select/before-options", ["exports", "ember-power-select/components/power-select/before-options"], function (_exports, _beforeOptions) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _beforeOptions.default;
    }
  });
});
;define("ghost-admin/components/power-select/options", ["exports", "ember-power-select/components/power-select/options"], function (_exports, _options) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _options.default;
    }
  });
});
;define("ghost-admin/components/power-select/placeholder", ["exports", "ember-power-select/components/power-select/placeholder"], function (_exports, _placeholder) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _placeholder.default;
    }
  });
});
;define("ghost-admin/components/power-select/power-select-group", ["exports", "ember-power-select/components/power-select/power-select-group"], function (_exports, _powerSelectGroup) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerSelectGroup.default;
    }
  });
});
;define("ghost-admin/components/power-select/search-message", ["exports", "ember-power-select/components/power-select/search-message"], function (_exports, _searchMessage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _searchMessage.default;
    }
  });
});
;define("ghost-admin/components/power-select/trigger", ["exports", "ghost-admin/templates/components/power-select/trigger", "ember-power-select/components/power-select/trigger"], function (_exports, _trigger, _trigger2) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _trigger2.default.extend({
    layout: _trigger.default
  });

  _exports.default = _default;
});
;define("ghost-admin/components/sortable-objects", ["exports", "ember-drag-drop/components/sortable-objects"], function (_exports, _sortableObjects) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _sortableObjects.default;
  _exports.default = _default;
});
;define("ghost-admin/components/sticky-element", ["exports", "ember-sticky-element/components/sticky-element"], function (_exports, _stickyElement) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _stickyElement.default;
    }
  });
});
;define("ghost-admin/components/sticky-element/trigger", ["exports", "ember-sticky-element/components/sticky-element/trigger"], function (_exports, _trigger) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _trigger.default;
    }
  });
});
;define("ghost-admin/components/vertical-collection", ["exports", "@html-next/vertical-collection/components/vertical-collection/component"], function (_exports, _component) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _component.default;
    }
  });
});
;define("ghost-admin/components/x-file-input", ["exports", "emberx-file-input/components/x-file-input"], function (_exports, _xFileInput) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _xFileInput.default;
    }
  });
});
;define("ghost-admin/controllers/about", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  var _default = Ember.Controller.extend({
    config: Ember.inject.service(),
    upgradeStatus: Ember.inject.service(),
    copyrightYear: Ember.computed(function () {
      let date = new Date();
      return date.getFullYear();
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/application", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    dropdown: Ember.inject.service(),
    router: Ember.inject.service(),
    session: Ember.inject.service(),
    settings: Ember.inject.service(),
    ui: Ember.inject.service(),
    showNavMenu: Ember.computed('router.currentRouteName', 'session.{isAuthenticated,user.isFulfilled}', 'ui.isFullScreen', function () {
      let {
        router,
        session,
        ui
      } = this; // if we're in fullscreen mode don't show the nav menu

      if (ui.isFullScreen) {
        return false;
      } // we need to defer showing the navigation menu until the session.user
      // promise has fulfilled so that gh-user-can-admin has the correct data


      if (!session.isAuthenticated || !session.user.isFulfilled) {
        return false;
      }

      return (router.currentRouteName !== 'error404' || session.isAuthenticated) && !router.currentRouteName.match(/(signin|setup)/);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/editor", ["exports", "ghost-admin/models/post", "ghost-admin/utils/bound-one-way", "ghost-admin/config/environment", "ghost-admin/utils/isNumber", "ember-ajax/errors", "ghost-admin/services/ajax", "ember-concurrency"], function (_exports, _post, _boundOneWay, _environment, _isNumber, _errors, _ajax, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const DEFAULT_TITLE = '(Untitled)'; // time in ms to save after last content edit

  const AUTOSAVE_TIMEOUT = 3000; // time in ms to force a save if the user is continuously typing

  const TIMEDSAVE_TIMEOUT = 60000; // this array will hold properties we need to watch for this.hasDirtyAttributes

  let watchedProps = ['post.scratch', 'post.titleScratch', 'post.hasDirtyAttributes', 'post.tags.[]', 'post.isError']; // add all post model attrs to the watchedProps array, easier to do it this way
  // than remember to update every time we add a new attr

  _post.default.eachAttribute(function (name) {
    watchedProps.push("post.".concat(name));
  });

  const messageMap = {
    errors: {
      post: {
        published: {
          published: 'Update failed',
          draft: 'Saving failed',
          scheduled: 'Scheduling failed'
        },
        draft: {
          published: 'Publish failed',
          draft: 'Saving failed',
          scheduled: 'Scheduling failed'
        },
        scheduled: {
          scheduled: 'Update failed',
          draft: 'Unscheduling failed',
          published: 'Publish failed'
        }
      }
    },
    success: {
      post: {
        published: {
          published: 'Updated.',
          draft: 'Saved.',
          scheduled: 'Scheduled.'
        },
        draft: {
          published: 'Published!',
          draft: 'Saved.',
          scheduled: 'Scheduled.'
        },
        scheduled: {
          scheduled: 'Updated.',
          draft: 'Unscheduled.',
          published: 'Published!'
        }
      }
    }
  };

  var _default = Ember.Controller.extend({
    application: Ember.inject.controller(),
    feature: Ember.inject.service(),
    notifications: Ember.inject.service(),
    router: Ember.inject.service(),
    slugGenerator: Ember.inject.service(),
    session: Ember.inject.service(),
    ui: Ember.inject.service(),

    /* public properties -----------------------------------------------------*/
    infoMessage: null,
    leaveEditorTransition: null,
    shouldFocusEditor: false,
    showDeletePostModal: false,
    showLeaveEditorModal: false,
    // koenig related properties
    wordcount: null,

    /* private properties ----------------------------------------------------*/
    // set by setPost and _postSaved, used in hasDirtyAttributes
    _previousTagNames: null,

    /* computed properties ---------------------------------------------------*/
    post: Ember.computed.alias('model'),
    // store the desired post status locally without updating the model,
    // the model will only be updated when a save occurs
    willPublish: (0, _boundOneWay.default)('post.isPublished'),
    willSchedule: (0, _boundOneWay.default)('post.isScheduled'),
    // updateSlug and save should always be enqueued so that we don't run into
    // problems with concurrency, for example when Cmd-S is pressed whilst the
    // cursor is in the slug field - that would previously trigger a simultaneous
    // slug update and save resulting in ember data errors and inconsistent save
    // results
    saveTasks: (0, _emberConcurrency.taskGroup)().enqueue(),
    _tagNames: Ember.computed.mapBy('post.tags', 'name'),
    hasDirtyAttributes: Ember.computed(...watchedProps, {
      get() {
        return this._hasDirtyAttributes();
      },

      set(key, value) {
        return value;
      }

    }),
    _autosaveRunning: Ember.computed('_autosave.isRunning', '_timedSave.isRunning', function () {
      let autosave = this.get('_autosave.isRunning');
      let timedsave = this.get('_timedSave.isRunning');
      return autosave || timedsave;
    }),
    _canAutosave: Ember.computed('post.isDraft', function () {
      return _environment.default.environment !== 'test' && this.get('post.isDraft');
    }),

    /* actions ---------------------------------------------------------------*/
    actions: {
      updateScratch(mobiledoc) {
        this.set('post.scratch', mobiledoc); // save 3 seconds after last edit

        this._autosave.perform(); // force save at 60 seconds


        this._timedSave.perform();
      },

      updateTitleScratch(title) {
        this.set('post.titleScratch', title);
      },

      // updates local willPublish/Schedule values, does not get applied to
      // the post's `status` value until a save is triggered
      setSaveType(newType) {
        if (newType === 'publish') {
          this.set('willPublish', true);
          this.set('willSchedule', false);
        } else if (newType === 'draft') {
          this.set('willPublish', false);
          this.set('willSchedule', false);
        } else if (newType === 'schedule') {
          this.set('willSchedule', true);
          this.set('willPublish', false);
        }
      },

      save(options) {
        return this.save.perform(options);
      },

      // used to prevent unexpected background saves. Triggered when opening
      // publish menu, starting a manual save, and when leaving the editor
      cancelAutosave() {
        this._autosave.cancelAll();

        this._timedSave.cancelAll();
      },

      toggleLeaveEditorModal(transition) {
        let leaveTransition = this.leaveEditorTransition; // "cancel" was clicked in the "are you sure?" modal so we just
        // reset the saved transition and remove the modal

        if (!transition && this.showLeaveEditorModal) {
          this.set('leaveEditorTransition', null);
          this.set('showLeaveEditorModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveEditorTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('saveTasks.isRunning')) {
            return this.get('saveTasks.last').then(() => {
              transition.retry();
            });
          } // if an autosave is scheduled, cancel it, save then transition


          if (this._autosaveRunning) {
            this.send('cancelAutosave');
            this.autosave.cancelAll();
            return this.autosave.perform().then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveEditorModal', true);
        }
      },

      // called by the "are you sure?" modal
      leaveEditor() {
        let transition = this.leaveEditorTransition;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // perform cleanup and reset manually, ensures the transition will succeed


        this.reset();
        return transition.retry();
      },

      toggleDeletePostModal() {
        if (!this.get('post.isNew')) {
          this.toggleProperty('showDeletePostModal');
        }
      },

      setKoenigEditor(koenig) {
        this._koenig = koenig; // remove any empty cards when displaying a draft post
        // - empty cards may be left in draft posts due to autosave occuring
        //   whilst an empty card is present then the user closing the browser
        //   or refreshing the page

        if (this.post.isDraft) {
          this._koenig.cleanup();
        }
      },

      updateWordCount(counts) {
        this.set('wordCount', counts);
      }

    },

    /* Public tasks ----------------------------------------------------------*/
    // separate task for autosave so that it doesn't override a manual save
    autosave: (0, _emberConcurrency.task)(function* () {
      if (!this.get('save.isRunning')) {
        return yield this.save.perform({
          silent: true,
          backgroundSave: true
        });
      }
    }).drop(),
    // save tasks cancels autosave before running, although this cancels the
    // _xSave tasks  that will also cancel the autosave task
    save: (0, _emberConcurrency.task)(function* () {
      let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      let prevStatus = this.get('post.status');
      let isNew = this.get('post.isNew');
      let status;
      this.send('cancelAutosave');

      if (options.backgroundSave && !this.hasDirtyAttributes) {
        return;
      }

      if (options.backgroundSave) {
        // do not allow a post's status to be set to published by a background save
        status = 'draft';
      } else {
        if (this.get('post.pastScheduledTime')) {
          status = !this.willSchedule && !this.willPublish ? 'draft' : 'published';
        } else {
          if (this.willPublish && !this.get('post.isScheduled')) {
            status = 'published';
          } else if (this.willSchedule && !this.get('post.isPublished')) {
            status = 'scheduled';
          } else {
            status = 'draft';
          }
        }
      } // ensure we remove any blank cards when performing a full save


      if (!options.backgroundSave) {
        if (this._koenig) {
          this._koenig.cleanup();

          this.set('hasDirtyAttributes', true);
        }
      } // Set the properties that are indirected
      // set mobiledoc equal to what's in the editor but create a copy so that
      // nested objects/arrays don't keep references which can mean that both
      // scratch and mobiledoc get updated simultaneously


      this.set('post.mobiledoc', JSON.parse(JSON.stringify(this.post.scratch || null)));
      this.set('post.status', status); // Set a default title

      if (!this.get('post.titleScratch').trim()) {
        this.set('post.titleScratch', DEFAULT_TITLE);
      }

      this.set('post.title', this.get('post.titleScratch'));
      this.set('post.customExcerpt', this.get('post.customExcerptScratch'));
      this.set('post.footerInjection', this.get('post.footerExcerptScratch'));
      this.set('post.headerInjection', this.get('post.headerExcerptScratch'));
      this.set('post.metaTitle', this.get('post.metaTitleScratch'));
      this.set('post.metaDescription', this.get('post.metaDescriptionScratch'));
      this.set('post.ogTitle', this.get('post.ogTitleScratch'));
      this.set('post.ogDescription', this.get('post.ogDescriptionScratch'));
      this.set('post.twitterTitle', this.get('post.twitterTitleScratch'));
      this.set('post.twitterDescription', this.get('post.twitterDescriptionScratch'));

      if (!this.get('post.slug')) {
        this.saveTitle.cancelAll();
        yield this.generateSlug.perform();
      }

      try {
        let post = yield this.post.save(options);

        if (!options.silent) {
          this._showSaveNotification(prevStatus, post.get('status'), isNew ? true : false);
        }

        this.post.set('statusScratch', null); // redirect to edit route if saving a new record

        if (isNew && post.get('id')) {
          if (!this.leaveEditorTransition) {
            this.replaceRoute('editor.edit', post);
          }

          return true;
        }

        return post;
      } catch (error) {
        // re-throw if we have a general server error
        if (error && !(0, _errors.isInvalidError)(error)) {
          this.send('error', error);
          return;
        }

        this.set('post.status', prevStatus);

        if (!options.silent) {
          let errorOrMessages = error || this.get('post.errors.messages');

          this._showErrorAlert(prevStatus, this.get('post.status'), errorOrMessages); // simulate a validation error for upstream tasks


          throw undefined;
        }

        return this.post;
      }
    }).group('saveTasks'),

    /*
     * triggered by a user manually changing slug
     */
    updateSlug: (0, _emberConcurrency.task)(function* (_newSlug) {
      let slug = this.get('post.slug');
      let newSlug, serverSlug;
      newSlug = _newSlug || slug;
      newSlug = newSlug && newSlug.trim(); // Ignore unchanged slugs or candidate slugs that are empty

      if (!newSlug || slug === newSlug) {
        // reset the input to its previous state
        this.set('slugValue', slug);
        return;
      }

      serverSlug = yield this.slugGenerator.generateSlug('post', newSlug); // If after getting the sanitized and unique slug back from the API
      // we end up with a slug that matches the existing slug, abort the change

      if (serverSlug === slug) {
        return;
      } // Because the server transforms the candidate slug by stripping
      // certain characters and appending a number onto the end of slugs
      // to enforce uniqueness, there are cases where we can get back a
      // candidate slug that is a duplicate of the original except for
      // the trailing incrementor (e.g., this-is-a-slug and this-is-a-slug-2)
      // get the last token out of the slug candidate and see if it's a number


      let slugTokens = serverSlug.split('-');
      let check = Number(slugTokens.pop()); // if the candidate slug is the same as the existing slug except
      // for the incrementor then the existing slug should be used

      if ((0, _isNumber.default)(check) && check > 0) {
        if (slug === slugTokens.join('-') && serverSlug !== newSlug) {
          this.set('slugValue', slug);
          return;
        }
      }

      this.set('post.slug', serverSlug); // If this is a new post.  Don't save the post.  Defer the save
      // to the user pressing the save button

      if (this.get('post.isNew')) {
        return;
      }

      return yield this.post.save();
    }).group('saveTasks'),
    // used in the PSM so that saves are sequential and don't trigger collision
    // detection errors
    savePost: (0, _emberConcurrency.task)(function* () {
      try {
        return yield this.post.save();
      } catch (error) {
        if (error) {
          let status = this.get('post.status');

          this._showErrorAlert(status, status, error);
        }

        throw error;
      }
    }).group('saveTasks'),
    saveTitle: (0, _emberConcurrency.task)(function* () {
      let post = this.post;
      let currentTitle = post.get('title');
      let newTitle = post.get('titleScratch').trim();

      if (currentTitle && newTitle && newTitle === currentTitle) {
        return;
      } // this is necessary to force a save when the title is blank


      this.set('hasDirtyAttributes', true); // generate a slug if a post is new and doesn't have a title yet or
      // if the title is still '(Untitled)'

      if (post.get('isNew') && !currentTitle || currentTitle === DEFAULT_TITLE) {
        yield this.generateSlug.perform();
      }

      if (this.get('post.isDraft')) {
        yield this.autosave.perform();
      }

      this.ui.updateDocumentTitle();
    }),
    generateSlug: (0, _emberConcurrency.task)(function* () {
      let title = this.get('post.titleScratch'); // Only set an "untitled" slug once per post

      if (title === DEFAULT_TITLE && this.get('post.slug')) {
        return;
      }

      try {
        let slug = yield this.slugGenerator.generateSlug('post', title);

        if (!Ember.isBlank(slug)) {
          this.set('post.slug', slug);
        }
      } catch (error) {
        // Nothing to do (would be nice to log this somewhere though),
        // but a rejected promise needs to be handled here so that a resolved
        // promise is returned.
        if ((0, _ajax.isVersionMismatchError)(error)) {
          this.notifications.showAPIError(error);
        }
      }
    }).enqueue(),

    /* Public methods --------------------------------------------------------*/
    // called by the new/edit routes to change the post model
    setPost(post) {
      // don't do anything else if we're setting the same post
      if (post === this.post) {
        // set autofocus as change signal to the persistent editor on new->edit
        this.set('shouldFocusEditor', post.get('isNew'));
        return;
      } // reset everything ready for a new post


      this.reset();
      this.set('post', post); // autofocus the editor if we have a new post

      this.set('shouldFocusEditor', post.get('isNew')); // need to set scratch values because they won't be present on first
      // edit of the post
      // TODO: can these be `boundOneWay` on the model as per the other attrs?

      post.set('titleScratch', post.get('title'));
      post.set('scratch', post.get('mobiledoc'));
      this._previousTagNames = this._tagNames;

      this._attachModelHooks(); // triggered any time the admin tab is closed, we need to use a native
      // dialog here instead of our custom modal


      window.onbeforeunload = () => {
        if (this.hasDirtyAttributes) {
          return '==============================\n\n' + 'Hey there! It looks like you\'re in the middle of writing' + ' something and you haven\'t saved all of your content.' + '\n\nSave before you go!\n\n' + '==============================';
        }
      };
    },

    // called by editor route's willTransition hook, fires for editor.new->edit,
    // editor.edit->edit, or editor->any. Triggers `toggleLeaveEditorModal` action
    // which will either finish autosave then retry transition or abort and show
    // the "are you sure?" modal
    willTransition(transition) {
      let post = this.post; // exit early and allow transition if we have no post, occurs if reset
      // has already been called as in the `leaveEditor` action

      if (!post) {
        return;
      } // clean up blank cards when leaving the editor if we have a draft post
      // - blank cards could be left around due to autosave triggering whilst
      //   a blank card is present then the user attempting to leave
      // - will mark the post as dirty so it gets saved when transitioning


      if (this._koenig && post.isDraft) {
        this._koenig.cleanup();
      }

      let hasDirtyAttributes = this.hasDirtyAttributes;
      let state = post.getProperties('isDeleted', 'isSaving', 'hasDirtyAttributes', 'isNew');
      let fromNewToEdit = this.get('router.currentRouteName') === 'editor.new' && transition.targetName === 'editor.edit' && transition.intent.contexts && transition.intent.contexts[0] && transition.intent.contexts[0].id === post.get('id');
      let deletedWithoutChanges = state.isDeleted && (state.isSaving || !state.hasDirtyAttributes); // controller is dirty and we aren't in a new->edit or delete->index
      // transition so show our "are you sure you want to leave?" modal

      if (!fromNewToEdit && !deletedWithoutChanges && hasDirtyAttributes) {
        transition.abort();
        this.send('toggleLeaveEditorModal', transition);
        return;
      } // the transition is now certain to complete so cleanup and reset if
      // we're exiting the editor. new->edit keeps everything around and
      // edit->edit will call reset in the setPost method if necessary


      if (!fromNewToEdit && transition.targetName !== 'editor.edit') {
        this.reset();
      }
    },

    // called when the editor route is left or the post model is swapped
    reset() {
      let post = this.post; // make sure the save tasks aren't still running in the background
      // after leaving the edit route

      this.send('cancelAutosave');

      if (post) {
        // clear post of any unsaved, client-generated tags
        post.updateTags(); // remove new+unsaved records from the store and rollback any unsaved changes

        if (post.get('isNew')) {
          post.deleteRecord();
        } else {
          post.rollbackAttributes();
        } // remove the create/update event handlers that were added to the post


        this._detachModelHooks();
      }

      this._previousTagNames = [];
      this.set('post', null);
      this.set('hasDirtyAttributes', false);
      this.set('shouldFocusEditor', false);
      this.set('leaveEditorTransition', null);
      this.set('infoMessage', null);
      this.set('wordCount', null); // remove the onbeforeunload handler as it's only relevant whilst on
      // the editor route

      window.onbeforeunload = null;
    },

    /* Private tasks ---------------------------------------------------------*/
    // save 3 seconds after the last edit
    _autosave: (0, _emberConcurrency.task)(function* () {
      if (!this._canAutosave) {
        return;
      } // force an instant save on first body edit for new posts


      if (this.get('post.isNew')) {
        return this.autosave.perform();
      }

      yield (0, _emberConcurrency.timeout)(AUTOSAVE_TIMEOUT);
      this.autosave.perform();
    }).restartable(),
    // save at 60 seconds even if the user doesn't stop typing
    _timedSave: (0, _emberConcurrency.task)(function* () {
      if (!this._canAutosave) {
        return;
      }

      while (_environment.default.environment !== 'test' && true) {
        yield (0, _emberConcurrency.timeout)(TIMEDSAVE_TIMEOUT);
        this.autosave.perform();
      }
    }).drop(),

    /* Private methods -------------------------------------------------------*/
    _hasDirtyAttributes() {
      let post = this.post;

      if (!post) {
        return false;
      } // if the Adapter failed to save the post isError will be true
      // and we should consider the post still dirty.


      if (post.get('isError')) {
        return true;
      } // post.tags is an array so hasDirtyAttributes doesn't pick up
      // changes unless the array ref is changed


      let currentTags = this.getWithDefault('_tagNames', []).join('');
      let previousTags = this.getWithDefault('_previousTagNames', []).join('');

      if (currentTags !== previousTags) {
        return true;
      } // titleScratch isn't an attr so needs a manual dirty check


      if (this.titleScratch !== this.title) {
        return true;
      } // scratch isn't an attr so needs a manual dirty check


      let mobiledoc = post.get('mobiledoc');
      let scratch = post.get('scratch'); // additional guard in case we are trying to compare null with undefined

      if (scratch || mobiledoc) {
        let mobiledocJSON = JSON.stringify(mobiledoc);
        let scratchJSON = JSON.stringify(scratch);

        if (scratchJSON !== mobiledocJSON) {
          return true;
        }
      } // new+unsaved posts always return `hasDirtyAttributes: true`
      // so we need a manual check to see if any


      if (post.get('isNew')) {
        let changedAttributes = Object.keys(post.changedAttributes());
        return changedAttributes.length ? true : false;
      } // we've covered all the non-tracked cases we care about so fall
      // back on Ember Data's default dirty attribute checks


      return post.get('hasDirtyAttributes');
    },

    // post.save() is called in multiple places, rather than remembering to
    // add a .then in every instance we use model hooks to update our local
    // values used for `hasDirtyAttributes`
    _attachModelHooks() {
      let post = this.post;

      if (post) {
        post.on('didCreate', this, this._postSaved);
        post.on('didUpdate', this, this._postSaved);
      }
    },

    _detachModelHooks() {
      let post = this.post;

      if (post) {
        post.off('didCreate', this, this._postSaved);
        post.off('didUpdate', this, this._postSaved);
      }
    },

    _postSaved() {
      let post = this.post; // remove any unsaved tags
      // NOTE: `updateTags` changes `hasDirtyAttributes => true`.
      // For a saved post it would otherwise be false.

      post.updateTags();
      this._previousTagNames = this._tagNames; // if the two "scratch" properties (title and content) match the post,
      // then it's ok to set hasDirtyAttributes to false
      // TODO: why is this necessary?

      let titlesMatch = post.get('titleScratch') === post.get('title');
      let bodiesMatch = JSON.stringify(post.get('scratch')) === JSON.stringify(post.get('mobiledoc'));

      if (titlesMatch && bodiesMatch) {
        this.set('hasDirtyAttributes', false);
      }
    },

    _showSaveNotification(prevStatus, status, delay) {
      let message = messageMap.success.post[prevStatus][status];
      let notifications = this.notifications;
      let type, path;

      if (status === 'published') {
        type = this.get('post.page') ? 'Page' : 'Post';
        path = this.get('post.url');
      } else {
        type = 'Preview';
        path = this.get('post.previewUrl');
      }

      message += "&nbsp;<a href=\"".concat(path, "\" target=\"_blank\">View ").concat(type, "</a>");
      notifications.showNotification(message.htmlSafe(), {
        delayed: delay
      });
    },

    _showErrorAlert(prevStatus, status, error, delay) {
      let message = messageMap.errors.post[prevStatus][status];
      let notifications = this.notifications;
      let errorMessage;

      function isString(str) {
        /* global toString */
        return toString.call(str) === '[object String]';
      }

      if (error && isString(error)) {
        errorMessage = error;
      } else if (error && Ember.isArray(error)) {
        // This is here because validation errors are returned as an array
        // TODO: remove this once validations are fixed
        errorMessage = error[0];
      } else if (error && error.payload && error.payload.errors && error.payload.errors[0].message) {
        return this.notifications.showAPIError(error, {
          key: 'post.save'
        });
      } else {
        errorMessage = 'Unknown Error';
      }

      message += ": ".concat(errorMessage);
      message = Ember.String.htmlSafe(message);
      notifications.showAlert(message, {
        type: 'error',
        delayed: delay,
        key: 'post.save'
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/error", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    stack: false,
    error: Ember.computed.readOnly('model'),
    code: Ember.computed('error.status', function () {
      return this.get('error.status') > 200 ? this.get('error.status') : 500;
    }),
    message: Ember.computed('error.statusText', function () {
      if (this.code === 404) {
        return 'Page not found';
      }

      return this.get('error.statusText') !== 'error' ? this.get('error.statusText') : 'Internal Server Error';
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/member", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    members: Ember.inject.controller(),
    store: Ember.inject.service(),
    router: Ember.inject.service(),
    member: Ember.computed.alias('model'),
    subscription: Ember.computed('member.subscriptions', function () {
      let subscriptions = this.member.get('subscriptions');

      if (!subscriptions) {
        return {
          amount: '...',
          status: '...',
          plan: '...'
        };
      }

      let subscription = subscriptions[0] || {};
      return {
        amount: subscription.amount ? subscription.amount / 100 : 0,
        status: subscription.status || '-',
        plan: subscription.plan || 'Free'
      };
    }),
    actions: {
      finaliseDeletion() {
        // decrememnt the total member count manually so there's no flash
        // when transitioning back to the members list
        if (this.members.meta) {
          this.members.decrementProperty('meta.pagination.total');
        }

        this.router.transitionTo('members');
      }

    },
    fetchMember: (0, _emberConcurrency.task)(function* (memberId) {
      yield this.store.findRecord('member', memberId, {
        reload: true
      }).then(data => {
        this.set('member', data);
      });
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/members", ["exports", "moment", "ember-concurrency"], function (_exports, _moment, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  var _default = Ember.Controller.extend({
    store: Ember.inject.service(),
    meta: null,
    members: null,
    searchText: '',

    init() {
      this._super(...arguments);

      this.set('members', this.store.peekAll('member'));
    },

    filteredMembers: Ember.computed('members.@each.{name,email}', 'searchText', function () {
      let {
        members,
        searchText
      } = this;
      searchText = searchText.toLowerCase();
      let filtered = members.filter(member => {
        if (!searchText) {
          return true;
        }

        let {
          name,
          email
        } = member;
        return name.toLowerCase().indexOf(searchText) >= 0 || email.toLowerCase().indexOf(searchText) >= 0;
      });
      return filtered;
    }),
    fetchMembers: (0, _emberConcurrency.task)(function* () {
      let newFetchDate = new Date();
      let results;

      if (this._hasFetchedAll) {
        // fetch any records modified since last fetch
        results = yield this.store.query('member', {
          limit: 'all',
          filter: "updated_at:>='".concat(_moment.default.utc(this._lastFetchDate).format('YYYY-MM-DD HH:mm:ss'), "'")
        });
      } else {
        // fetch all records
        results = yield this.store.query('member', {
          limit: 'all'
        });
        this._hasFetchedAll = true;
      }

      this.set('meta', results.meta);
      this._lastFetchDate = newFetchDate;
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/pages-loading", ["exports", "ghost-admin/controllers/posts-loading"], function (_exports, _postsLoading) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  var _default = _postsLoading.default.extend({
    postsController: Ember.inject.controller('pages')
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/pages", ["exports", "ghost-admin/controllers/posts"], function (_exports, _posts) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const TYPES = [{
    name: 'All pages',
    value: null
  }, {
    name: 'Draft pages',
    value: 'draft'
  }, {
    name: 'Published pages',
    value: 'published'
  }, {
    name: 'Scheduled pages',
    value: 'scheduled'
  }, {
    name: 'Featured pages',
    value: 'featured'
  }];
  const ORDERS = [{
    name: 'Newest',
    value: null
  }, {
    name: 'Oldest',
    value: 'published_at asc'
  }, {
    name: 'Recently updated',
    value: 'updated_at desc'
  }];
  /* eslint-disable ghost/ember/alias-model-in-controller */

  var _default = _posts.default.extend({
    init() {
      this._super(...arguments);

      this.availableTypes = TYPES;
      this.availableOrders = ORDERS;
    },

    actions: {
      openEditor(page) {
        this.transitionToRoute('editor.edit', 'page', page.get('id'));
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/posts-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  var _default = Ember.Controller.extend({
    postsController: Ember.inject.controller('posts'),
    session: Ember.inject.service(),
    availableTypes: Ember.computed.readOnly('postsController.availableTypes'),
    selectedType: Ember.computed.readOnly('postsController.selectedType'),
    availableTags: Ember.computed.readOnly('postsController.availableTags'),
    selectedTag: Ember.computed.readOnly('postsController.selectedTag'),
    availableOrders: Ember.computed.readOnly('postsController.availableOrders'),
    selectedOrder: Ember.computed.readOnly('postsController.selectedOrder')
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/posts", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const TYPES = [{
    name: 'All posts',
    value: null
  }, {
    name: 'Draft posts',
    value: 'draft'
  }, {
    name: 'Published posts',
    value: 'published'
  }, {
    name: 'Scheduled posts',
    value: 'scheduled'
  }, {
    name: 'Featured posts',
    value: 'featured'
  }];
  const ORDERS = [{
    name: 'Newest',
    value: null
  }, {
    name: 'Oldest',
    value: 'published_at asc'
  }, {
    name: 'Recently updated',
    value: 'updated_at desc'
  }];

  var _default = Ember.Controller.extend({
    session: Ember.inject.service(),
    store: Ember.inject.service(),
    queryParams: ['type', 'tag', 'order'],
    type: null,
    tag: null,
    order: null,
    _hasLoadedTags: false,
    _hasLoadedAuthors: false,
    availableTypes: null,
    availableOrders: null,

    init() {
      this._super(...arguments);

      this.availableTypes = TYPES;
      this.availableOrders = ORDERS;
    },

    postsInfinityModel: Ember.computed.alias('model'),
    showingAll: Ember.computed('type', 'tag', function () {
      let {
        type,
        tag
      } = this.getProperties(['type', 'tag']);
      return !type && !tag;
    }),
    selectedType: Ember.computed('type', function () {
      let types = this.get('availableTypes');
      return types.findBy('value', this.get('type'));
    }),
    selectedOrder: Ember.computed('order', function () {
      let orders = this.get('availableOrders');
      return orders.findBy('value', this.get('order'));
    }),
    _availableTags: Ember.computed(function () {
      return this.get('store').peekAll('tag');
    }),
    availableTags: Ember.computed('_availableTags.[]', function () {
      let tags = this.get('_availableTags').filter(tag => tag.get('id') !== null).sort((tagA, tagB) => tagA.name.localeCompare(tagB.name, undefined, {
        ignorePunctuation: true
      }));
      let options = tags.toArray();
      options.unshiftObject({
        name: 'All tags',
        slug: null
      });
      return options;
    }),
    selectedTag: Ember.computed('tag', '_availableTags.[]', function () {
      let tag = this.get('tag');
      let tags = this.get('availableTags');
      return tags.findBy('slug', tag);
    }),
    currentUserId: Ember.computed('session.user.id', function () {
      return this.get('session.user.id');
    }),
    actions: {
      changeType(type) {
        this.set('type', Ember.get(type, 'value'));
      },

      changeAuthor(author) {
        this.set('author', Ember.get(author, 'slug'));
      },

      changeTag(tag) {
        this.set('tag', Ember.get(tag, 'slug'));
      },

      changeOrder(order) {
        this.set('order', Ember.get(order, 'value'));
      },

      openEditor(post) {
        this.transitionToRoute('editor.edit', 'post', post.get('id'));
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/code-injection", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    notifications: Ember.inject.service(),
    settings: Ember.inject.service(),
    actions: {
      save() {
        this.save.perform();
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('save.isRunning')) {
            return this.get('save.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;
        let settings = this.settings;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on settings props


        settings.rollbackAttributes();
        return transition.retry();
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      let notifications = this.notifications;

      try {
        return yield this.settings.save();
      } catch (error) {
        notifications.showAPIError(error, {
          key: 'code-injection.save'
        });
        throw error;
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/design", ["exports", "jquery", "ghost-admin/models/navigation-item", "ghost-admin/services/ajax", "ember-concurrency"], function (_exports, _jquery, _navigationItem, _ajax, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  var _default = Ember.Controller.extend({
    config: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    session: Ember.inject.service(),
    settings: Ember.inject.service(),
    dirtyAttributes: false,
    newNavItem: null,
    themes: null,
    themeToDelete: null,

    init() {
      this._super(...arguments);

      this.set('newNavItem', _navigationItem.default.create({
        isNew: true
      }));
    },

    showDeleteThemeModal: Ember.computed.notEmpty('themeToDelete'),
    blogUrl: Ember.computed('config.blogUrl', function () {
      let url = this.get('config.blogUrl');
      return url.slice(-1) !== '/' ? "".concat(url, "/") : url;
    }),
    actions: {
      save() {
        this.save.perform();
      },

      addNavItem() {
        let newNavItem = this.newNavItem; // If the url sent through is blank (user never edited the url)

        if (newNavItem.get('url') === '') {
          newNavItem.set('url', '/');
        }

        return newNavItem.validate().then(() => {
          this.addNewNavItem();
        });
      },

      deleteNavItem(item) {
        if (!item) {
          return;
        }

        let navItems = this.get('settings.navigation');
        navItems.removeObject(item);
        this.set('dirtyAttributes', true);
      },

      updateLabel(label, navItem) {
        if (!navItem) {
          return;
        }

        navItem.set('label', label);
        this.set('dirtyAttributes', true);
      },

      updateUrl(url, navItem) {
        if (!navItem) {
          return;
        }

        navItem.set('url', url);
        this.set('dirtyAttributes', true);
        return url;
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('save.isRunning')) {
            return this.get('save.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;
        let settings = this.settings;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on settings props


        settings.rollbackAttributes();
        this.set('dirtyAttributes', false);
        return transition.retry();
      },

      activateTheme(theme) {
        return theme.activate().then(theme => {
          if (!Ember.isEmpty(theme.get('warnings'))) {
            this.set('themeWarnings', theme.get('warnings'));
            this.set('showThemeWarningsModal', true);
          }

          if (!Ember.isEmpty(theme.get('errors'))) {
            this.set('themeErrors', theme.get('errors'));
            this.set('showThemeWarningsModal', true);
          }
        }).catch(error => {
          if ((0, _ajax.isThemeValidationError)(error)) {
            let errors = error.payload.errors[0].details.errors;
            let fatalErrors = [];
            let normalErrors = []; // to have a proper grouping of fatal errors and none fatal, we need to check
            // our errors for the fatal property

            if (errors.length > 0) {
              for (let i = 0; i < errors.length; i += 1) {
                if (errors[i].fatal) {
                  fatalErrors.push(errors[i]);
                } else {
                  normalErrors.push(errors[i]);
                }
              }
            }

            this.set('themeErrors', normalErrors);
            this.set('themeFatalErrors', fatalErrors);
            this.set('showThemeErrorsModal', true);
            return;
          }

          throw error;
        });
      },

      downloadTheme(theme) {
        let downloadURL = "".concat(this.get('ghostPaths.apiRoot'), "/themes/").concat(theme.name, "/download/");
        let iframe = (0, _jquery.default)('#iframeDownload');

        if (iframe.length === 0) {
          iframe = (0, _jquery.default)('<iframe>', {
            id: 'iframeDownload'
          }).hide().appendTo('body');
        }

        iframe.attr('src', downloadURL);
      },

      deleteTheme(theme) {
        if (theme) {
          return this.set('themeToDelete', theme);
        }

        return this._deleteTheme();
      },

      hideDeleteThemeModal() {
        this.set('themeToDelete', null);
      },

      hideThemeWarningsModal() {
        this.set('themeWarnings', null);
        this.set('themeErrors', null);
        this.set('themeFatalErrors', null);
        this.set('showThemeWarningsModal', false);
        this.set('showThemeErrorsModal', false);
      },

      reset() {
        this.set('newNavItem', _navigationItem.default.create({
          isNew: true
        }));
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      let navItems = this.get('settings.navigation');
      let newNavItem = this.newNavItem;
      let notifications = this.notifications;
      let validationPromises = [];

      if (!newNavItem.get('isBlank')) {
        validationPromises.pushObject(this.send('addNavItem'));
      }

      navItems.map(item => {
        validationPromises.pushObject(item.validate());
      });

      try {
        yield Ember.RSVP.all(validationPromises);
        this.set('dirtyAttributes', false);
        return yield this.settings.save();
      } catch (error) {
        if (error) {
          notifications.showAPIError(error);
          throw error;
        }
      }
    }),

    addNewNavItem() {
      let navItems = this.get('settings.navigation');
      let newNavItem = this.newNavItem;
      newNavItem.set('isNew', false);
      navItems.pushObject(newNavItem);
      this.set('dirtyAttributes', true);
      this.set('newNavItem', _navigationItem.default.create({
        isNew: true
      }));
      (0, _jquery.default)('.gh-blognav-line:last input:first').focus();
    },

    _deleteTheme() {
      let theme = this.store.peekRecord('theme', this.themeToDelete.name);

      if (!theme) {
        return;
      }

      return theme.destroyRecord().then(() => {
        // HACK: this is a private method, we need to unload from the store
        // here so that uploading another theme with the same "id" doesn't
        // attempt to update the deleted record
        theme.unloadRecord();
      }).catch(error => {
        this.notifications.showAPIError(error);
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/general", ["exports", "jquery", "ghost-admin/utils/random-password", "validator", "ghost-admin/components/gh-image-uploader", "ember-concurrency"], function (_exports, _jquery, _randomPassword, _validator, _ghImageUploader, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  const ICON_EXTENSIONS = ['ico', 'png'];

  var _default = Ember.Controller.extend({
    config: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    session: Ember.inject.service(),
    settings: Ember.inject.service(),
    ui: Ember.inject.service(),
    rcServices: Ember.inject.service('rc_services'),
    availableTimezones: null,
    iconExtensions: null,
    iconMimeTypes: 'image/png,image/x-icon',
    imageExtensions: _ghImageUploader.IMAGE_EXTENSIONS,
    imageMimeTypes: _ghImageUploader.IMAGE_MIME_TYPES,
    _scratchRoom: null,
    _scratchFacebook: null,
    _scratchTwitter: null,

    init() {
      this._super(...arguments);

      this.iconExtensions = ICON_EXTENSIONS;
      this._scratchRoom = this.get('settings.roomName');
    },

    privateRSSUrl: Ember.computed('config.blogUrl', 'settings.publicHash', function () {
      let blogUrl = this.get('config.blogUrl');
      let publicHash = this.get('settings.publicHash');
      return "".concat(blogUrl, "/").concat(publicHash, "/rss");
    }),
    actions: {
      save() {
        this.save.perform();
      },

      setTimezone(timezone) {
        this.set('settings.activeTimezone', timezone.name);
      },

      removeImage(image) {
        // setting `null` here will error as the server treats it as "null"
        this.settings.set(image, '');
      },

      /**
       * Opens a file selection dialog - Triggered by "Upload Image" buttons,
       * searches for the hidden file input within the .gh-setting element
       * containing the clicked button then simulates a click
       * @param  {MouseEvent} event - MouseEvent fired by the button click
       */
      triggerFileDialog(event) {
        // simulate click to open file dialog
        // using jQuery because IE11 doesn't support MouseEvent
        (0, _jquery.default)(event.target).closest('.gh-setting-action').find('input[type="file"]').click();
      },

      /**
       * Fired after an image upload completes
       * @param  {string} property - Property name to be set on `this.settings`
       * @param  {UploadResult[]} results - Array of UploadResult objects
       * @return {string} The URL that was set on `this.settings.property`
       */
      imageUploaded(property, results) {
        if (results[0]) {
          return this.settings.set(property, results[0].url);
        }
      },

      toggleIsPrivate(isPrivate) {
        let settings = this.settings;
        settings.set('isPrivate', isPrivate);
        settings.get('errors').remove('password');
        let changedAttrs = settings.changedAttributes(); // set a new random password when isPrivate is enabled

        if (isPrivate && changedAttrs.isPrivate) {
          settings.set('password', (0, _randomPassword.default)()); // reset the password when isPrivate is disabled
        } else if (changedAttrs.password) {
          settings.set('password', changedAttrs.password[0]);
        }
      },

      toggleIsAnnounced(isAnnounced) {
        let settings = this.settings;
        settings.set('isAnnounced', isAnnounced); // Change isAuthorsRooms to false when isAnnounce is disabled

        if (!isAnnounced) {
          settings.set('isAuthorsRooms', false);
          settings.set('canCollaborate', false);
        }
      },

      toggleIsAuthorsRooms(isAuthorsRooms) {
        let settings = this.settings;
        settings.set('isAuthorsRooms', isAuthorsRooms);
      },

      toggleIsComments(isComments) {
        let settings = this.settings;
        settings.set('isComments', isComments);
      },

      toggleInviteOnly(inviteOnly) {
        let settings = this.settings;
        settings.set('inviteOnly', inviteOnly);
      },

      toggleCanCollaborate(canCollaborate) {
        let settings = this.settings; // canCollaborate and inviteOnly can be set true at the same time.

        if (canCollaborate) {
          settings.set('inviteOnly', false);
        }

        settings.set('canCollaborate', canCollaborate);
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('save.isRunning')) {
            return this.get('save.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;
        let settings = this.settings;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on settings props


        settings.rollbackAttributes();
        return transition.retry();
      },

      validateRoom() {
        let newRoom = this._scratchRoom;
        let oldRoom = this.get('settings.roomName');
        let errMessage = 'Room does not exist'; // reset errors and validation

        this.get('settings.errors').remove('room');
        this.get('settings.hasValidated').removeObject('room');

        if (!newRoom) {
          newRoom = oldRoom;
        }

        this.rcServices.getRoom({
          rname: newRoom
        }).then(room => {
          const existingRCRoom = room.data[0].exist && room.data[0].roomname === newRoom;

          if (!existingRCRoom) {
            throw errMessage;
          }

          Ember.run.schedule('afterRender', this, function () {
            this.set('settings.roomName', newRoom);
            this.set('settings.roomId', room.data[0].rid);
          });
        }).catch(e => {
          if (e === errMessage) {
            this.get('settings.errors').add('room', errMessage);
            return;
          }

          throw e;
        }).finally(() => {
          this.get('settings.hasValidated').pushObject('room');
        });
      },

      validateFacebookUrl() {
        let newUrl = this._scratchFacebook;
        let oldUrl = this.get('settings.facebook');
        let errMessage = ''; // reset errors and validation

        this.get('settings.errors').remove('facebook');
        this.get('settings.hasValidated').removeObject('facebook');

        if (newUrl === '') {
          // Clear out the Facebook url
          this.set('settings.facebook', '');
          return;
        } // _scratchFacebook will be null unless the user has input something


        if (!newUrl) {
          newUrl = oldUrl;
        }

        try {
          // strip any facebook URLs out
          newUrl = newUrl.replace(/(https?:\/\/)?(www\.)?facebook\.com/i, ''); // don't allow any non-facebook urls

          if (newUrl.match(/^(http|\/\/)/i)) {
            throw 'invalid url';
          } // strip leading / if we have one then concat to full facebook URL


          newUrl = newUrl.replace(/^\//, '');
          newUrl = "https://www.facebook.com/".concat(newUrl); // don't allow URL if it's not valid

          if (!_validator.default.isURL(newUrl)) {
            throw 'invalid url';
          }

          this.set('settings.facebook', '');
          Ember.run.schedule('afterRender', this, function () {
            this.set('settings.facebook', newUrl);
          });
        } catch (e) {
          if (e === 'invalid url') {
            errMessage = 'The URL must be in a format like ' + 'https://www.facebook.com/yourPage';
            this.get('settings.errors').add('facebook', errMessage);
            return;
          }

          throw e;
        } finally {
          this.get('settings.hasValidated').pushObject('facebook');
        }
      },

      validateTwitterUrl() {
        let newUrl = this._scratchTwitter;
        let oldUrl = this.get('settings.twitter');
        let errMessage = ''; // reset errors and validation

        this.get('settings.errors').remove('twitter');
        this.get('settings.hasValidated').removeObject('twitter');

        if (newUrl === '') {
          // Clear out the Twitter url
          this.set('settings.twitter', '');
          return;
        } // _scratchTwitter will be null unless the user has input something


        if (!newUrl) {
          newUrl = oldUrl;
        }

        if (newUrl.match(/(?:twitter\.com\/)(\S+)/) || newUrl.match(/([a-z\d.]+)/i)) {
          let username = [];

          if (newUrl.match(/(?:twitter\.com\/)(\S+)/)) {
            [, username] = newUrl.match(/(?:twitter\.com\/)(\S+)/);
          } else {
            [username] = newUrl.match(/([^/]+)\/?$/mi);
          } // check if username starts with http or www and show error if so


          if (username.match(/^(http|www)|(\/)/) || !username.match(/^[a-z\d._]{1,15}$/mi)) {
            errMessage = !username.match(/^[a-z\d._]{1,15}$/mi) ? 'Your Username is not a valid Twitter Username' : 'The URL must be in a format like https://twitter.com/yourUsername';
            this.get('settings.errors').add('twitter', errMessage);
            this.get('settings.hasValidated').pushObject('twitter');
            return;
          }

          newUrl = "https://twitter.com/".concat(username);
          this.get('settings.hasValidated').pushObject('twitter');
          this.set('settings.twitter', '');
          Ember.run.schedule('afterRender', this, function () {
            this.set('settings.twitter', newUrl);
          });
        } else {
          errMessage = 'The URL must be in a format like ' + 'https://twitter.com/yourUsername';
          this.get('settings.errors').add('twitter', errMessage);
          this.get('settings.hasValidated').pushObject('twitter');
          return;
        }
      }

    },

    _deleteTheme() {
      let theme = this.store.peekRecord('theme', this.themeToDelete.name);

      if (!theme) {
        return;
      }

      return theme.destroyRecord().catch(error => {
        this.notifications.showAPIError(error);
      });
    },

    save: (0, _emberConcurrency.task)(function* () {
      let notifications = this.notifications;
      let config = this.config;

      try {
        let settings = yield this.settings.save();
        config.set('blogTitle', settings.get('title')); // this forces the document title to recompute after a blog title change

        this.ui.updateDocumentTitle();
        return settings;
      } catch (error) {
        if (error) {
          notifications.showAPIError(error, {
            key: 'settings.save'
          });
        }

        throw error;
      }
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integration", ["exports", "ghost-admin/utils/copy-text-to-clipboard", "ghost-admin/components/gh-image-uploader", "ember-concurrency"], function (_exports, _copyTextToClipboard, _ghImageUploader, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    config: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    imageExtensions: _ghImageUploader.IMAGE_EXTENSIONS,
    imageMimeTypes: _ghImageUploader.IMAGE_MIME_TYPES,
    integration: Ember.computed.alias('model'),
    apiUrl: Ember.computed(function () {
      let origin = window.location.origin;
      let subdir = this.ghostPaths.subdir;
      let url = this.ghostPaths.url.join(origin, subdir);
      return url.replace(/\/$/, '');
    }),
    allWebhooks: Ember.computed(function () {
      return this.store.peekAll('webhook');
    }),
    filteredWebhooks: Ember.computed('integration.id', 'allWebhooks.@each.{isNew,isDeleted}', function () {
      return this.allWebhooks.filter(webhook => {
        let matchesIntegration = webhook.belongsTo('integration').id() === this.integration.id;
        return matchesIntegration && !webhook.isNew && !webhook.isDeleted;
      });
    }),
    iconImageStyle: Ember.computed('integration.iconImage', function () {
      let url = this.integration.iconImage;

      if (url) {
        let styles = ["background-image: url(".concat(url, ")"), 'background-size: 50%', 'background-position: 50%', 'background-repeat: no-repeat'];
        return Ember.String.htmlSafe(styles.join('; '));
      }
    }),
    actions: {
      triggerIconFileDialog() {
        let input = document.querySelector('input[type="file"][name="iconImage"]');
        input.click();
      },

      setIconImage(_ref) {
        let [image] = _ref;
        this.integration.set('iconImage', image.url);
      },

      save() {
        return this.save.perform();
      },

      toggleUnsavedChangesModal(transition) {
        let leaveTransition = this.leaveScreenTransition;

        if (!transition && this.showUnsavedChangesModal) {
          this.set('leaveScreenTransition', null);
          this.set('showUnsavedChangesModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveScreenTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.save.isRunning) {
            return this.save.last.then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showUnsavedChangesModal', true);
        }
      },

      leaveScreen() {
        let transition = this.leaveScreenTransition;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on model props


        this.integration.rollbackAttributes();
        return transition.retry();
      },

      deleteIntegration() {
        this.integration.destroyRecord();
      },

      confirmIntegrationDeletion() {
        this.set('showDeleteIntegrationModal', true);
      },

      cancelIntegrationDeletion() {
        this.set('showDeleteIntegrationModal', false);
      },

      confirmWebhookDeletion(webhook) {
        this.set('webhookToDelete', webhook);
      },

      cancelWebhookDeletion() {
        this.set('webhookToDelete', null);
      },

      deleteWebhook() {
        return this.webhookToDelete.destroyRecord();
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      return yield this.integration.save();
    }),
    copyContentKey: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.integration.contentKey.secret);
      yield (0, _emberConcurrency.timeout)(3000);
    }),
    copyAdminKey: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.integration.adminKey.secret);
      yield (0, _emberConcurrency.timeout)(3000);
    }),
    copyApiUrl: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.apiUrl);
      yield (0, _emberConcurrency.timeout)(3000);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integration/webhooks/edit", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    webhook: Ember.computed.alias('model'),
    actions: {
      save() {
        return this.webhook.save();
      },

      cancel() {
        // 'new' route's dectivate hook takes care of rollback
        return this.webhook.get('integration').then(integration => {
          this.transitionToRoute('settings.integration', integration);
        });
      }

    },

    reset() {
      this.webhook.rollbackAttributes();
      this.webhook.errors.clear();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integration/webhooks/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    webhook: Ember.computed.alias('model'),
    actions: {
      save() {
        return this.webhook.save();
      },

      cancel() {
        // 'new' route's dectivate hook takes care of rollback
        return this.webhook.get('integration').then(integration => {
          this.transitionToRoute('settings.integration', integration);
        });
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    settings: Ember.inject.service(),
    store: Ember.inject.service(),
    _allIntegrations: null,

    init() {
      this._super(...arguments);

      this._allIntegrations = this.store.peekAll('integration');
    },

    // filter over the live query so that the list is automatically updated
    // as integrations are added/removed
    integrations: Ember.computed('_allIntegrations.@each.{isNew,type}', function () {
      return this._allIntegrations.reject(integration => {
        return integration.isNew || integration.type !== 'custom';
      });
    }),
    // use ember-concurrency so that we can use the derived state to show
    // a spinner only in the integrations list and avoid delaying the whole
    // screen display
    fetchIntegrations: (0, _emberConcurrency.task)(function* () {
      return yield this.store.findAll('integration');
    }),

    // used by individual integration routes' `model` hooks
    integrationModelHook(prop, value, route, transition) {
      let integration = this.store.peekAll('integration').findBy(prop, value);

      if (integration) {
        return integration;
      }

      return this.fetchIntegrations.perform().then(integrations => {
        let integration = integrations.findBy(prop, value);

        if (!integration) {
          let path = transition.intent.url.replace(/^\//, '');
          return route.replaceWith('error404', {
            path,
            status: 404
          });
        }

        return integration;
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations/amp", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    notifications: Ember.inject.service(),
    settings: Ember.inject.service(),
    leaveSettingsTransition: null,
    ampSettings: Ember.computed.alias('settings.amp'),
    actions: {
      update(value) {
        this.set('ampSettings', value);
      },

      save() {
        this.save.perform();
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('save.isRunning')) {
            return this.get('save.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;
        let settings = this.settings;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on settings model


        settings.rollbackAttributes();
        return transition.retry();
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      let amp = this.ampSettings;
      let settings = this.settings;
      settings.set('amp', amp);

      try {
        return yield settings.save();
      } catch (error) {
        this.notifications.showAPIError(error);
        throw error;
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    integration: Ember.computed.alias('model'),
    actions: {
      save() {
        return this.integration.save();
      },

      cancel() {
        // 'new' route's dectivate hook takes care of rollback
        this.transitionToRoute('settings.integrations');
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations/search", ["exports", "ghost-admin/config/environment", "ghost-admin/utils/copy-text-to-clipboard", "ember-concurrency"], function (_exports, _environment, _copyTextToClipboard, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    ghostPaths: Ember.inject.service(),
    isTesting: undefined,

    init() {
      this._super(...arguments);

      if (this.isTesting === undefined) {
        this.isTesting = _environment.default.environment === 'test';
      }
    },

    integration: Ember.computed.alias('model'),
    apiUrl: Ember.computed(function () {
      let origin = window.location.origin;
      let subdir = this.ghostPaths.subdir;
      let url = this.ghostPaths.url.join(origin, subdir);
      return url.replace(/\/$/, '');
    }),
    copyContentKey: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.integration.contentKey.secret);
      yield (0, _emberConcurrency.timeout)(3000);
    }),
    copyApiUrl: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.apiUrl);
      yield (0, _emberConcurrency.timeout)(3000);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations/slack", ["exports", "ghost-admin/utils/bound-one-way", "ember-ajax/errors", "ember-concurrency"], function (_exports, _boundOneWay, _errors, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    ghostPaths: Ember.inject.service(),
    ajax: Ember.inject.service(),
    notifications: Ember.inject.service(),
    settings: Ember.inject.service(),
    leaveSettingsTransition: null,
    slackArray: null,

    init() {
      this._super(...arguments);

      this.slackArray = [];
    },

    slackSettings: (0, _boundOneWay.default)('settings.slack.firstObject'),
    testNotificationDisabled: Ember.computed.empty('slackSettings.url'),
    actions: {
      save() {
        this.save.perform();
      },

      updateURL(value) {
        value = typeof value === 'string' ? value.trim() : value;
        this.set('slackSettings.url', value);
        this.get('slackSettings.errors').clear();
      },

      updateUsername(value) {
        value = typeof value === 'string' ? value.trimLeft() : value;
        this.set('slackSettings.username', value);
        this.get('slackSettings.errors').clear();
      },

      triggerDirtyState() {
        let slack = this.slackSettings;
        let slackArray = this.slackArray;
        let settings = this.settings; // Hack to trigger the `isDirty` state on the settings model by setting a new Array
        // for slack rather that replacing the existing one which would still point to the
        // same reference and therfore not setting the model into a dirty state

        slackArray.clear().pushObject(slack);
        settings.set('slack', slackArray);
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('save.isRunning')) {
            return this.get('save.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;
        let settings = this.settings;
        let slackArray = this.slackArray;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on model props


        settings.rollbackAttributes();
        slackArray.clear();
        return transition.retry();
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      let slack = this.slackSettings;
      let settings = this.settings;
      let slackArray = this.slackArray;

      try {
        yield slack.validate(); // clear existing objects in slackArray to make sure we only push the validated one

        slackArray.clear().pushObject(slack);
        yield settings.set('slack', slackArray);
        return yield settings.save();
      } catch (error) {
        if (error) {
          this.notifications.showAPIError(error);
          throw error;
        }
      }
    }).drop(),
    sendTestNotification: (0, _emberConcurrency.task)(function* () {
      let notifications = this.notifications;
      let slackApi = this.get('ghostPaths.url').api('slack', 'test');

      try {
        yield this.save.perform();
        yield this.ajax.post(slackApi);
        notifications.showNotification('Check your Slack channel for the test message!', {
          type: 'info',
          key: 'slack-test.send.success'
        });
        return true;
      } catch (error) {
        notifications.showAPIError(error, {
          key: 'slack-test:send'
        });

        if (!(0, _errors.isInvalidError)(error)) {
          throw error;
        }
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations/unsplash", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    notifications: Ember.inject.service(),
    settings: Ember.inject.service(),
    dirtyAttributes: null,
    rollbackValue: null,
    leaveSettingsTransition: null,
    unsplashSettings: Ember.computed.alias('settings.unsplash'),
    actions: {
      save() {
        this.save.perform();
      },

      update(value) {
        if (!this.dirtyAttributes) {
          this.set('rollbackValue', this.get('unsplashSettings.isActive'));
        }

        this.set('unsplashSettings.isActive', value);
        this.set('dirtyAttributes', true);
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('save.isRunning')) {
            return this.get('save.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on model props


        this.set('unsplashSettings.isActive', this.rollbackValue);
        this.set('dirtyAttributes', false);
        this.set('rollbackValue', null);
        return transition.retry();
      }

    },
    save: (0, _emberConcurrency.task)(function* () {
      let unsplash = this.unsplashSettings;
      let settings = this.settings;

      try {
        settings.set('unsplash', unsplash);
        this.set('dirtyAttributes', false);
        this.set('rollbackValue', null);
        return yield settings.save();
      } catch (error) {
        if (error) {
          this.notifications.showAPIError(error);
          throw error;
        }
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/integrations/zapier", ["exports", "ghost-admin/config/environment", "ghost-admin/utils/copy-text-to-clipboard", "ember-concurrency"], function (_exports, _environment, _copyTextToClipboard, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    ghostPaths: Ember.inject.service(),
    isTesting: undefined,

    init() {
      this._super(...arguments);

      if (this.isTesting === undefined) {
        this.isTesting = _environment.default.environment === 'test';
      }
    },

    integration: Ember.computed.alias('model'),
    apiUrl: Ember.computed(function () {
      let origin = window.location.origin;
      let subdir = this.ghostPaths.subdir;
      let url = this.ghostPaths.url.join(origin, subdir);
      return url.replace(/\/$/, '');
    }),
    copyAdminKey: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.integration.adminKey.secret);
      yield (0, _emberConcurrency.timeout)(3000);
    }),
    copyApiUrl: (0, _emberConcurrency.task)(function* () {
      (0, _copyTextToClipboard.default)(this.apiUrl);
      yield (0, _emberConcurrency.timeout)(3000);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/labs", ["exports", "jquery", "ghost-admin/config/environment", "ghost-admin/services/ajax", "ember-concurrency"], function (_exports, _jquery, _environment, _ajax, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  const {
    Promise
  } = Ember.RSVP;
  const IMPORT_MIME_TYPES = ['application/json', 'application/zip', 'application/x-zip-compressed'];
  const JSON_EXTENSION = ['json'];
  const JSON_MIME_TYPE = ['application/json'];
  const YAML_EXTENSION = ['yaml'];
  const YAML_MIME_TYPE = ['text/vnd.yaml', 'application/vnd.yaml', 'text/x-yaml', 'application/x-yaml'];

  var _default = Ember.Controller.extend({
    ajax: Ember.inject.service(),
    config: Ember.inject.service(),
    feature: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    session: Ember.inject.service(),
    settings: Ember.inject.service(),
    importErrors: null,
    importSuccessful: false,
    showDeleteAllModal: false,
    showMemberConfig: false,
    submitting: false,
    uploadButtonText: 'Import',
    importMimeType: null,
    jsonExtension: null,
    jsonMimeType: null,
    yamlExtension: null,
    yamlMimeType: null,

    init() {
      this._super(...arguments);

      this.importMimeType = IMPORT_MIME_TYPES;
      this.jsonExtension = JSON_EXTENSION;
      this.jsonMimeType = JSON_MIME_TYPE;
      this.yamlExtension = YAML_EXTENSION;
      this.yamlMimeType = YAML_MIME_TYPE;
    },

    subscriptionSettings: Ember.computed('settings.membersSubscriptionSettings', function () {
      let subscriptionSettings = this.parseSubscriptionSettings(this.get('settings.membersSubscriptionSettings'));
      let stripeProcessor = subscriptionSettings.paymentProcessors.find(proc => {
        return proc.adapter === 'stripe';
      });
      let monthlyPlan = stripeProcessor.config.plans.find(plan => plan.interval === 'month');
      let yearlyPlan = stripeProcessor.config.plans.find(plan => plan.interval === 'year');
      monthlyPlan.dollarAmount = monthlyPlan.amount / 100;
      yearlyPlan.dollarAmount = yearlyPlan.amount / 100;
      stripeProcessor.config.plans = {
        monthly: monthlyPlan,
        yearly: yearlyPlan
      };
      subscriptionSettings.stripeConfig = stripeProcessor.config;
      return subscriptionSettings;
    }),
    actions: {
      onUpload(file) {
        let formData = new FormData();
        let notifications = this.notifications;
        let currentUserId = this.get('session.user.id');
        let dbUrl = this.get('ghostPaths.url').api('db');
        this.set('uploadButtonText', 'Importing');
        this.set('importErrors', null);
        this.set('importSuccessful', false);
        return this._validate(file).then(() => {
          formData.append('importfile', file);
          return this.ajax.post(dbUrl, {
            data: formData,
            dataType: 'json',
            cache: false,
            contentType: false,
            processData: false
          });
        }).then(response => {
          let store = this.store;
          this.set('importSuccessful', true);

          if (response.problems) {
            this.set('importErrors', response.problems);
          } // Clear the store, so that all the new data gets fetched correctly.


          store.unloadAll(); // NOTE: workaround for behaviour change in Ember 2.13
          // store.unloadAll has some async tendencies so we need to schedule
          // the reload of the current user once the unload has finished
          // https://github.com/emberjs/data/issues/4963

          Ember.run.schedule('destroy', this, () => {
            // Reload currentUser and set session
            this.set('session.user', store.findRecord('user', currentUserId)); // TODO: keep as notification, add link to view content

            notifications.showNotification('Import successful.', {
              key: 'import.upload.success'
            }); // reload settings

            return this.settings.reload().then(settings => {
              this.feature.fetch();
              this.config.set('blogTitle', settings.get('title'));
            });
          });
        }).catch(response => {
          if ((0, _ajax.isUnsupportedMediaTypeError)(response) || (0, _ajax.isRequestEntityTooLargeError)(response)) {
            this.set('importErrors', [response]);
          } else if (response && response.payload.errors && Ember.isArray(response.payload.errors)) {
            this.set('importErrors', response.payload.errors);
          } else {
            this.set('importErrors', [{
              message: 'Import failed due to an unknown error. Check the Web Inspector console and network tabs for errors.'
            }]);
          }

          throw response;
        }).finally(() => {
          this.set('uploadButtonText', 'Import');
        });
      },

      downloadFile(endpoint) {
        let downloadURL = this.get('ghostPaths.url').api(endpoint);
        let iframe = (0, _jquery.default)('#iframeDownload');

        if (iframe.length === 0) {
          iframe = (0, _jquery.default)('<iframe>', {
            id: 'iframeDownload'
          }).hide().appendTo('body');
        }

        iframe.attr('src', downloadURL);
      },

      toggleDeleteAllModal() {
        this.toggleProperty('showDeleteAllModal');
      },

      toggleMemberConfig() {
        this.toggleProperty('showMemberConfig');
      },

      /**
       * Opens a file selection dialog - Triggered by "Upload x" buttons,
       * searches for the hidden file input within the .gh-setting element
       * containing the clicked button then simulates a click
       * @param  {MouseEvent} event - MouseEvent fired by the button click
       */
      triggerFileDialog(event) {
        // simulate click to open file dialog
        // using jQuery because IE11 doesn't support MouseEvent
        (0, _jquery.default)(event.target).closest('.gh-setting-action').find('input[type="file"]').click();
      },

      setSubscriptionSettings(key, event) {
        let subscriptionSettings = this.parseSubscriptionSettings(this.get('settings.membersSubscriptionSettings'));
        let stripeProcessor = subscriptionSettings.paymentProcessors.find(proc => {
          return proc.adapter === 'stripe';
        });
        let stripeConfig = stripeProcessor.config;
        stripeConfig.product = {
          name: this.settings.get('title')
        };

        if (key === 'isPaid') {
          subscriptionSettings.isPaid = event;
        }

        if (key === 'secret_token' || key === 'public_token') {
          stripeConfig[key] = event.target.value;
        }

        if (key === 'month' || key === 'year') {
          stripeConfig.plans = stripeConfig.plans.map(plan => {
            if (key === plan.interval) {
              plan.amount = event.target.value * 100;
            }

            return plan;
          });
        }

        this.set('settings.membersSubscriptionSettings', JSON.stringify(subscriptionSettings));
      }

    },

    parseSubscriptionSettings(settingsString) {
      try {
        return JSON.parse(settingsString);
      } catch (e) {
        return {
          isPaid: false,
          paymentProcessors: [{
            adapter: 'stripe',
            config: {
              secret_token: '',
              public_token: '',
              product: {
                name: this.settings.get('title')
              },
              plans: [{
                name: 'Monthly',
                currency: 'usd',
                interval: 'month',
                amount: ''
              }, {
                name: 'Yearly',
                currency: 'usd',
                interval: 'year',
                amount: ''
              }]
            }
          }]
        };
      }
    },

    // TODO: convert to ember-concurrency task
    _validate(file) {
      // Windows doesn't have mime-types for json files by default, so we
      // need to have some additional checking
      if (file.type === '') {
        // First check file extension so we can early return
        let [, extension] = /(?:\.([^.]+))?$/.exec(file.name);

        if (!extension || extension.toLowerCase() !== 'json') {
          return Ember.RSVP.reject(new _ajax.UnsupportedMediaTypeError());
        }

        return new Promise((resolve, reject) => {
          // Extension is correct, so check the contents of the file
          let reader = new FileReader();

          reader.onload = function () {
            let {
              result
            } = reader;

            try {
              JSON.parse(result);
              return resolve();
            } catch (e) {
              return reject(new _ajax.UnsupportedMediaTypeError());
            }
          };

          reader.readAsText(file);
        });
      }

      let accept = this.importMimeType;

      if (!Ember.isBlank(accept) && file && accept.indexOf(file.type) === -1) {
        return Ember.RSVP.reject(new _ajax.UnsupportedMediaTypeError());
      }

      return Ember.RSVP.resolve();
    },

    sendTestEmail: (0, _emberConcurrency.task)(function* () {
      let notifications = this.notifications;
      let emailUrl = this.get('ghostPaths.url').api('mail', 'test');

      try {
        yield this.ajax.post(emailUrl);
        notifications.showAlert('Check your email for the test message.', {
          type: 'info',
          key: 'test-email.send.success'
        });
        return true;
      } catch (error) {
        notifications.showAPIError(error, {
          key: 'test-email:send'
        });
      }
    }).drop(),
    saveSettings: (0, _emberConcurrency.task)(function* () {
      try {
        return yield this.settings.save();
      } catch (error) {
        throw error;
      }
    }).drop(),
    redirectUploadResult: (0, _emberConcurrency.task)(function* (success) {
      this.set('redirectSuccess', success);
      this.set('redirectFailure', !success);
      yield (0, _emberConcurrency.timeout)(_environment.default.environment === 'test' ? 100 : 5000);
      this.set('redirectSuccess', null);
      this.set('redirectFailure', null);
      return true;
    }).drop(),
    routesUploadResult: (0, _emberConcurrency.task)(function* (success) {
      this.set('routesSuccess', success);
      this.set('routesFailure', !success);
      yield (0, _emberConcurrency.timeout)(_environment.default.environment === 'test' ? 100 : 5000);
      this.set('routesSuccess', null);
      this.set('routesFailure', null);
      return true;
    }).drop(),

    reset() {
      this.set('importErrors', null);
      this.set('importSuccessful', false);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/tags", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    tagController: Ember.inject.controller('settings.tags.tag'),
    tags: Ember.computed.alias('model'),
    selectedTag: Ember.computed.alias('tagController.tag'),
    tagListFocused: Ember.computed.equal('keyboardFocus', 'tagList'),
    tagContentFocused: Ember.computed.equal('keyboardFocus', 'tagContent'),
    filteredTags: Ember.computed('tags.@each.isNew', function () {
      return this.tags.filterBy('isNew', false);
    }),
    // tags are sorted by name
    sortedTags: Ember.computed.sort('filteredTags', function (tagA, tagB) {
      // ignorePunctuation means the # in internal tag names is ignored
      return tagA.name.localeCompare(tagB.name, undefined, {
        ignorePunctuation: true
      });
    }),
    actions: {
      leftMobile() {
        let firstTag = this.get('tags.firstObject'); // redirect to first tag if possible so that you're not left with
        // tag settings blank slate when switching from portrait to landscape

        if (firstTag && !this.get('tagController.tag')) {
          this.transitionToRoute('settings.tags.tag', firstTag);
        }
      }

    },

    scrollTagIntoView(tag) {
      Ember.run.scheduleOnce('afterRender', this, function () {
        let id = "#gh-tag-".concat(tag.get('id'));
        let element = document.querySelector(id);

        if (element) {
          let scroll = document.querySelector('.tag-list');
          let {
            scrollTop
          } = scroll;
          let scrollHeight = scroll.offsetHeight;
          let element = document.querySelector(id);
          let elementTop = element.offsetTop;
          let elementHeight = element.offsetHeight;

          if (elementTop < scrollTop) {
            element.scrollIntoView(true);
          }

          if (elementTop + elementHeight > scrollTop + scrollHeight) {
            element.scrollIntoView(false);
          }
        }
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/settings/tags/tag", ["exports", "ghost-admin/utils/window-proxy"], function (_exports, _windowProxy) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    tagsController: Ember.inject.controller('settings.tags'),
    notifications: Ember.inject.service(),
    router: Ember.inject.service(),
    showDeleteTagModal: false,
    tag: Ember.computed.alias('model'),
    isMobile: Ember.computed.alias('tagsController.isMobile'),
    actions: {
      setProperty(propKey, value) {
        this._saveTagProperty(propKey, value);
      },

      toggleDeleteTagModal() {
        this.toggleProperty('showDeleteTagModal');
      },

      deleteTag() {
        return this._deleteTag();
      }

    },

    _saveTagProperty(propKey, newValue) {
      let tag = this.tag;
      let isNewTag = tag.get('isNew');
      let currentValue = tag.get(propKey);

      if (newValue) {
        newValue = newValue.trim();
      } // Quit if there was no change


      if (newValue === currentValue) {
        return;
      }

      tag.set(propKey, newValue); // TODO: This is required until .validate/.save mark fields as validated

      tag.get('hasValidated').addObject(propKey);
      tag.save().then(savedTag => {
        // replace 'new' route with 'tag' route
        this.replaceRoute('settings.tags.tag', savedTag); // update the URL if the slug changed

        if (propKey === 'slug' && !isNewTag) {
          let currentPath = window.location.hash;
          let newPath = currentPath.split('/');
          newPath[newPath.length - 1] = savedTag.get('slug');
          newPath = newPath.join('/');

          _windowProxy.default.replaceState({
            path: newPath
          }, '', newPath);
        }
      }).catch(error => {
        if (error) {
          this.notifications.showAPIError(error, {
            key: 'tag.save'
          });
        }
      });
    },

    _deleteTag() {
      let tag = this.tag;
      return tag.destroyRecord().then(() => {
        this._deleteTagSuccess();
      }, error => {
        this._deleteTagFailure(error);
      });
    },

    _deleteTagSuccess() {
      let currentRoute = this.router.currentRouteName || '';

      if (currentRoute.match(/^settings\.tags/)) {
        this.transitionToRoute('settings.tags.index');
      }
    },

    _deleteTagFailure(error) {
      this.notifications.showAPIError(error, {
        key: 'tag.delete'
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/site", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    guid: Ember.computed.alias('model')
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/staff/index", ["exports", "ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const ROLES = [{
    name: 'All users',
    value: null
  }, {
    name: 'Your Contributors',
    value: 'Contributors'
  }, {
    name: 'Owner',
    value: 'Owner'
  }, {
    name: 'Administrators',
    value: 'Administrator'
  }, {
    name: 'Editors',
    value: 'Editor'
  }, {
    name: 'Authors',
    value: 'Author'
  }, {
    name: 'Contributors',
    value: 'Contributor'
  }];

  var _default = Ember.Controller.extend({
    session: Ember.inject.service(),
    store: Ember.inject.service(),
    showInviteUserModal: false,
    role: null,
    userOrder: null,
    _availableRoles: null,

    init() {
      this._super(...arguments);

      this.inviteOrder = ['email'];
      this.userOrder = ['name', 'email'];
      this._availableRoles = ROLES;
    },

    currentUser: Ember.computed.alias('model'),
    sortedActiveUsers: Ember.computed.sort('activeUsers', 'userOrder'),
    sortedSuspendedUsers: Ember.computed.sort('suspendedUsers', 'userOrder'),
    availableRoles: Ember.computed('_availableRoles.[]', 'currentUser', function () {
      let options = this.get('_availableRoles');

      if (this.get('currentUser').isAuthorOrContributor) {
        return options.filter(role => role.value !== 'Contributors');
      }

      return options;
    }),
    selectedRole: Ember.computed('role', function () {
      let roles = this.get('availableRoles');
      return roles.findBy('value', this.get('role'));
    }),
    allUsers: Ember.computed(function () {
      return this.store.peekAll('user');
    }),
    activeUsers: Ember.computed('allUsers.@each.{status,roles,parentId}', 'role', 'currentUser', function () {
      return this.getUsers('active');
    }),
    suspendedUsers: Ember.computed('allUsers.@each.{status,roles,parentId}', 'role', 'currentUser', function () {
      return this.getUsers('inactive');
    }),
    actions: {
      toggleInviteUserModal() {
        this.toggleProperty('showInviteUserModal');
      },

      changeRole(role) {
        this.set('role', Ember.get(role, 'value'));
      }

    },
    backgroundUpdate: (0, _emberConcurrency.task)(function* () {
      let users = this.fetchUsers.perform();

      try {
        yield Ember.RSVP.all([users]);
      } catch (error) {
        this.send('error', error);
      }
    }),
    fetchUsers: (0, _emberConcurrency.task)(function* () {
      yield this.store.query('user', {
        limit: 'all',
        include: 'parents'
      });
    }),

    getUsers() {
      let status = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'active';
      let role = this.role;
      return this.allUsers.filter(user => {
        let isParent = false;

        if (role === 'Contributors') {
          isParent = this.currentUser.get('id') === user.parentId && user.roles.get('firstObject').name === 'Contributor';
        }

        return user.status === status && (role ? isParent || user.roles.get('firstObject').name === role : true);
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/staff/user", ["exports", "ghost-admin/utils/bound-one-way", "ghost-admin/utils/isNumber", "validator", "ghost-admin/utils/window-proxy", "ember-concurrency"], function (_exports, _boundOneWay, _isNumber, _validator, _windowProxy, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Controller.extend({
    ajax: Ember.inject.service(),
    config: Ember.inject.service(),
    dropdown: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    session: Ember.inject.service(),
    slugGenerator: Ember.inject.service(),
    store: Ember.inject.service(),
    leaveSettingsTransition: null,
    dirtyAttributes: false,
    showDeleteUserModal: false,
    showSuspendUserModal: false,
    showTransferOwnerModal: false,
    showUploadCoverModal: false,
    showUplaodImageModal: false,
    _scratchFacebook: null,
    _scratchTwitter: null,
    saveHandlers: (0, _emberConcurrency.taskGroup)().enqueue(),
    user: Ember.computed.alias('model'),
    currentUser: Ember.computed.alias('session.user'),
    email: Ember.computed.readOnly('user.email'),
    slugValue: (0, _boundOneWay.default)('user.slug'),
    canAssignRoles: Ember.computed.or('currentUser.isAdmin', 'currentUser.isOwner'),
    canChangeEmail: Ember.computed.not('isAdminUserOnOwnerProfile'),
    canMakeOwner: Ember.computed.and('currentUser.isOwner', 'isNotOwnProfile', 'user.isAdmin', 'isNotSuspended'),
    isAdminUserOnOwnerProfile: Ember.computed.and('currentUser.isAdmin', 'user.isOwner'),
    isNotOwnersProfile: Ember.computed.not('user.isOwner'),
    isNotSuspended: Ember.computed.not('user.isSuspended'),
    rolesDropdownIsVisible: Ember.computed.and('isNotOwnProfile', 'canAssignRoles', 'isNotOwnersProfile'),
    userActionsAreVisible: Ember.computed.or('deleteUserActionIsVisible', 'canMakeOwner'),
    isNotOwnProfile: Ember.computed.not('isOwnProfile'),
    isOwnProfile: Ember.computed('user.id', 'currentUser.id', function () {
      return this.get('user.id') === this.get('currentUser.id');
    }),
    deleteUserActionIsVisible: Ember.computed('currentUser', 'canAssignRoles', 'user', function () {
      if (this.canAssignRoles && this.isNotOwnProfile && !this.get('user.isOwner') || this.get('currentUser.isEditor') && (this.isNotOwnProfile || this.get('user.isAuthorOrContributor'))) {
        return true;
      }
    }),
    coverTitle: Ember.computed('user.name', function () {
      return "".concat(this.get('user.name'), "'s Cover Image");
    }),
    roles: Ember.computed(function () {
      return this.store.query('role', {
        permissions: 'assign'
      });
    }),
    parentUser: Ember.computed('user.parentId', function () {
      return this.store.queryRecord('user', {
        id: this.get('user.parentId')
      });
    }),
    actions: {
      changeRole(newRole) {
        this.user.set('role', newRole);
        this.set('dirtyAttributes', true);
      },

      deleteUser() {
        return this._deleteUser().then(() => {
          this._deleteUserSuccess();
        }, () => {
          this._deleteUserFailure();
        });
      },

      toggleDeleteUserModal() {
        if (this.deleteUserActionIsVisible) {
          this.toggleProperty('showDeleteUserModal');
        }
      },

      suspendUser() {
        this.user.set('status', 'inactive');
        return this.save.perform();
      },

      toggleSuspendUserModal() {
        if (this.deleteUserActionIsVisible) {
          this.toggleProperty('showSuspendUserModal');
        }
      },

      unsuspendUser() {
        this.user.set('status', 'active');
        return this.save.perform();
      },

      toggleUnsuspendUserModal() {
        if (this.deleteUserActionIsVisible) {
          this.toggleProperty('showUnsuspendUserModal');
        }
      },

      validateFacebookUrl() {
        let newUrl = this._scratchFacebook;
        let oldUrl = this.get('user.facebook');
        let errMessage = ''; // reset errors and validation

        this.get('user.errors').remove('facebook');
        this.get('user.hasValidated').removeObject('facebook');

        if (newUrl === '') {
          // Clear out the Facebook url
          this.set('user.facebook', '');
          return;
        } // _scratchFacebook will be null unless the user has input something


        if (!newUrl) {
          newUrl = oldUrl;
        }

        try {
          // strip any facebook URLs out
          newUrl = newUrl.replace(/(https?:\/\/)?(www\.)?facebook\.com/i, ''); // don't allow any non-facebook urls

          if (newUrl.match(/^(http|\/\/)/i)) {
            throw 'invalid url';
          } // strip leading / if we have one then concat to full facebook URL


          newUrl = newUrl.replace(/^\//, '');
          newUrl = "https://www.facebook.com/".concat(newUrl); // don't allow URL if it's not valid

          if (!_validator.default.isURL(newUrl)) {
            throw 'invalid url';
          }

          this.set('user.facebook', '');
          Ember.run.schedule('afterRender', this, function () {
            this.set('user.facebook', newUrl);
          });
        } catch (e) {
          if (e === 'invalid url') {
            errMessage = 'The URL must be in a format like ' + 'https://www.facebook.com/yourPage';
            this.get('user.errors').add('facebook', errMessage);
            return;
          }

          throw e;
        } finally {
          this.get('user.hasValidated').pushObject('facebook');
        }
      },

      validateTwitterUrl() {
        let newUrl = this._scratchTwitter;
        let oldUrl = this.get('user.twitter');
        let errMessage = ''; // reset errors and validation

        this.get('user.errors').remove('twitter');
        this.get('user.hasValidated').removeObject('twitter');

        if (newUrl === '') {
          // Clear out the Twitter url
          this.set('user.twitter', '');
          return;
        } // _scratchTwitter will be null unless the user has input something


        if (!newUrl) {
          newUrl = oldUrl;
        }

        if (newUrl.match(/(?:twitter\.com\/)(\S+)/) || newUrl.match(/([a-z\d.]+)/i)) {
          let username = [];

          if (newUrl.match(/(?:twitter\.com\/)(\S+)/)) {
            [, username] = newUrl.match(/(?:twitter\.com\/)(\S+)/);
          } else {
            [username] = newUrl.match(/([^/]+)\/?$/mi);
          } // check if username starts with http or www and show error if so


          if (username.match(/^(http|www)|(\/)/) || !username.match(/^[a-z\d._]{1,15}$/mi)) {
            errMessage = !username.match(/^[a-z\d._]{1,15}$/mi) ? 'Your Username is not a valid Twitter Username' : 'The URL must be in a format like https://twitter.com/yourUsername';
            this.get('user.errors').add('twitter', errMessage);
            this.get('user.hasValidated').pushObject('twitter');
            return;
          }

          newUrl = "https://twitter.com/".concat(username);
          this.get('user.hasValidated').pushObject('twitter');
          this.set('user.twitter', '');
          Ember.run.schedule('afterRender', this, function () {
            this.set('user.twitter', newUrl);
          });
        } else {
          errMessage = 'The URL must be in a format like ' + 'https://twitter.com/yourUsername';
          this.get('user.errors').add('twitter', errMessage);
          this.get('user.hasValidated').pushObject('twitter');
          return;
        }
      },

      transferOwnership() {
        let user = this.user;
        let url = this.get('ghostPaths.url').api('users', 'owner');
        this.dropdown.closeDropdowns();
        return this.ajax.put(url, {
          data: {
            owner: [{
              id: user.get('id')
            }]
          }
        }).then(response => {
          // manually update the roles for the users that just changed roles
          // because store.pushPayload is not working with embedded relations
          if (response && Ember.isArray(response.users)) {
            response.users.forEach(userJSON => {
              let user = this.store.peekRecord('user', userJSON.id);
              let role = this.store.peekRecord('role', userJSON.roles[0].id);
              user.set('role', role);
            });
          }

          this.notifications.showAlert("Ownership successfully transferred to ".concat(user.get('name')), {
            type: 'success',
            key: 'owner.transfer.success'
          });
        }).catch(error => {
          this.notifications.showAPIError(error, {
            key: 'owner.transfer'
          });
        });
      },

      toggleLeaveSettingsModal(transition) {
        let leaveTransition = this.leaveSettingsTransition;

        if (!transition && this.showLeaveSettingsModal) {
          this.set('leaveSettingsTransition', null);
          this.set('showLeaveSettingsModal', false);
          return;
        }

        if (!leaveTransition || transition.targetName === leaveTransition.targetName) {
          this.set('leaveSettingsTransition', transition); // if a save is running, wait for it to finish then transition

          if (this.get('saveHandlers.isRunning')) {
            return this.get('saveHandlers.last').then(() => {
              transition.retry();
            });
          } // we genuinely have unsaved data, show the modal


          this.set('showLeaveSettingsModal', true);
        }
      },

      leaveSettings() {
        let transition = this.leaveSettingsTransition;
        let user = this.user;

        if (!transition) {
          this.notifications.showAlert('Sorry, there was an error in the application. Please let the Ghost team know what happened.', {
            type: 'error'
          });
          return;
        } // roll back changes on user props


        user.rollbackAttributes(); // roll back the slugValue property

        if (this.dirtyAttributes) {
          this.set('slugValue', user.get('slug'));
          this.set('dirtyAttributes', false);
        }

        return transition.retry();
      },

      toggleTransferOwnerModal() {
        if (this.canMakeOwner) {
          this.toggleProperty('showTransferOwnerModal');
        }
      },

      toggleUploadCoverModal() {
        this.toggleProperty('showUploadCoverModal');
      }

    },

    _deleteUser() {
      if (this.deleteUserActionIsVisible) {
        let user = this.user;
        return user.destroyRecord();
      }
    },

    _deleteUserSuccess() {
      this.notifications.closeAlerts('user.delete');
      this.store.unloadAll('post');
      this.transitionToRoute('staff');
    },

    _deleteUserFailure() {
      this.notifications.showAlert('The user could not be deleted. Please try again.', {
        type: 'error',
        key: 'user.delete.failed'
      });
    },

    updateSlug: (0, _emberConcurrency.task)(function* (newSlug) {
      let slug = this.get('user.slug');
      newSlug = newSlug || slug;
      newSlug = newSlug.trim(); // Ignore unchanged slugs or candidate slugs that are empty

      if (!newSlug || slug === newSlug) {
        this.set('slugValue', slug);
        return true;
      }

      let serverSlug = yield this.slugGenerator.generateSlug('user', newSlug); // If after getting the sanitized and unique slug back from the API
      // we end up with a slug that matches the existing slug, abort the change

      if (serverSlug === slug) {
        return true;
      } // Because the server transforms the candidate slug by stripping
      // certain characters and appending a number onto the end of slugs
      // to enforce uniqueness, there are cases where we can get back a
      // candidate slug that is a duplicate of the original except for
      // the trailing incrementor (e.g., this-is-a-slug and this-is-a-slug-2)
      // get the last token out of the slug candidate and see if it's a number


      let slugTokens = serverSlug.split('-');
      let check = Number(slugTokens.pop()); // if the candidate slug is the same as the existing slug except
      // for the incrementor then the existing slug should be used

      if ((0, _isNumber.default)(check) && check > 0) {
        if (slug === slugTokens.join('-') && serverSlug !== newSlug) {
          this.set('slugValue', slug);
          return true;
        }
      }

      this.set('slugValue', serverSlug);
      this.set('dirtyAttributes', true);
      return true;
    }).group('saveHandlers'),
    save: (0, _emberConcurrency.task)(function* () {
      let user = this.user;
      let slugValue = this.slugValue;
      let slugChanged;

      if (user.get('slug') !== slugValue) {
        slugChanged = true;
        user.set('slug', slugValue);
      }

      try {
        user = yield user.save({
          format: false
        }); // If the user's slug has changed, change the URL and replace
        // the history so refresh and back button still work

        if (slugChanged) {
          let currentPath = window.location.hash;
          let newPath = currentPath.split('/');
          newPath[newPath.length - 1] = user.get('slug');
          newPath = newPath.join('/');

          _windowProxy.default.replaceState({
            path: newPath
          }, '', newPath);
        }

        this.set('dirtyAttributes', false);
        this.notifications.closeAlerts('user.update');
        return user;
      } catch (error) {
        // validation engine returns undefined so we have to check
        // before treating the failure as an API error
        if (error) {
          this.notifications.showAPIError(error, {
            key: 'user.update'
          });
        }
      }
    }).group('saveHandlers')
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/subscribers", ["exports", "jquery", "ghost-admin/utils/ghost-paths", "moment", "ember-concurrency"], function (_exports, _jquery, _ghostPaths, _moment, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const orderMap = {
    email: 'email',
    created_at: 'createdAtUTC',
    status: 'status'
  };
  /* eslint-disable ghost/ember/alias-model-in-controller */

  var _default = Ember.Controller.extend({
    session: Ember.inject.service(),
    queryParams: ['order', 'direction'],
    order: 'created_at',
    direction: 'desc',
    subscribers: null,
    subscriberToDelete: null,

    init() {
      this._super(...arguments);

      this.set('subscribers', this.store.peekAll('subscriber'));
    },

    filteredSubscribers: Ember.computed('subscribers.@each.{email,createdAtUTC}', function () {
      return this.subscribers.toArray().filter(subscriber => {
        return !subscriber.isNew && !subscriber.isDeleted;
      });
    }),
    sortedSubscribers: Ember.computed('order', 'direction', 'subscribers.@each.{email,createdAtUTC,status}', function () {
      let {
        filteredSubscribers,
        order,
        direction
      } = this;
      let sorted = filteredSubscribers.sort((a, b) => {
        let values = [a.get(orderMap[order]), b.get(orderMap[order])];

        if (direction === 'desc') {
          values = values.reverse();
        }

        if (typeof values[0] === 'string') {
          return values[0].localeCompare(values[1], undefined, {
            ignorePunctuation: true
          });
        }

        if (typeof values[0] === 'object' && values[0]._isAMomentObject) {
          return values[0].valueOf() - values[1].valueOf();
        }

        return values[0] - values[1];
      });
      return sorted;
    }),
    actions: {
      deleteSubscriber(subscriber) {
        this.set('subscriberToDelete', subscriber);
      },

      confirmDeleteSubscriber() {
        let subscriber = this.subscriberToDelete;
        return subscriber.destroyRecord().then(() => {
          this.set('subscriberToDelete', null);
        });
      },

      cancelDeleteSubscriber() {
        this.set('subscriberToDelete', null);
      },

      exportData() {
        let exportUrl = (0, _ghostPaths.default)().url.api('subscribers/csv');
        let accessToken = this.get('session.data.authenticated.access_token');
        let downloadURL = "".concat(exportUrl, "?access_token=").concat(accessToken);
        let iframe = (0, _jquery.default)('#iframeDownload');

        if (iframe.length === 0) {
          iframe = (0, _jquery.default)('<iframe>', {
            id: 'iframeDownload'
          }).hide().appendTo('body');
        }

        iframe.attr('src', downloadURL);
      }

    },
    fetchSubscribers: (0, _emberConcurrency.task)(function* () {
      let newFetchDate = new Date();
      let results;

      if (this._hasFetchedAll) {
        // fetch any records modified since last fetch
        results = yield this.store.query('subscriber', {
          limit: 'all',
          filter: "updated_at:>='".concat(_moment.default.utc(this._lastFetchDate).format('YYYY-MM-DD HH:mm:ss'), "'")
        });
      } else {
        // fetch all records in batches of 200
        while (!results || results.meta.pagination.page < results.meta.pagination.pages) {
          results = yield this.store.query('subscriber', {
            limit: 200,
            order: "".concat(this.order, " ").concat(this.direction),
            page: results ? results.meta.pagination.page + 1 : 1
          });
        }

        this._hasFetchedAll = true;
      }

      this._lastFetchDate = newFetchDate;
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/controllers/subscribers/import", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable ghost/ember/alias-model-in-controller */
  var _default = Ember.Controller.extend({
    subscribers: Ember.inject.controller(),
    router: Ember.inject.service(),
    actions: {
      fetchNewSubscribers() {
        this.subscribers.fetchSubscribers.perform();
      },

      close() {
        this.router.transitionTo('subscribers');
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/and", ["exports", "ember-truth-helpers/helpers/and"], function (_exports, _and) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _and.default;
    }
  });
  Object.defineProperty(_exports, "and", {
    enumerable: true,
    get: function get() {
      return _and.and;
    }
  });
});
;define("ghost-admin/helpers/app-version", ["exports", "ghost-admin/config/environment", "ember-cli-app-version/utils/regexp"], function (_exports, _environment, _regexp) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.appVersion = appVersion;
  _exports.default = void 0;

  function appVersion(_) {
    let hash = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    const version = _environment.default.APP.version; // e.g. 1.0.0-alpha.1+4jds75hf
    // Allow use of 'hideSha' and 'hideVersion' For backwards compatibility

    let versionOnly = hash.versionOnly || hash.hideSha;
    let shaOnly = hash.shaOnly || hash.hideVersion;
    let match = null;

    if (versionOnly) {
      if (hash.showExtended) {
        match = version.match(_regexp.versionExtendedRegExp); // 1.0.0-alpha.1
      } // Fallback to just version


      if (!match) {
        match = version.match(_regexp.versionRegExp); // 1.0.0
      }
    }

    if (shaOnly) {
      match = version.match(_regexp.shaRegExp); // 4jds75hf
    }

    return match ? match[0] : version;
  }

  var _default = Ember.Helper.helper(appVersion);

  _exports.default = _default;
});
;define("ghost-admin/helpers/assign", ["exports", "ember-assign-helper/helpers/assign"], function (_exports, _assign) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _assign.default;
    }
  });
  Object.defineProperty(_exports, "assign", {
    enumerable: true,
    get: function get() {
      return _assign.assign;
    }
  });
});
;define("ghost-admin/helpers/author-names", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.authorNames = authorNames;
  _exports.default = void 0;

  function authorNames(_ref
  /*, hash*/
  ) {
    let [authors] = _ref;

    if (!authors || Ember.isEmpty(authors)) {
      return;
    }

    return authors.mapBy('name').join(', ');
  }

  var _default = Ember.Helper.helper(authorNames);

  _exports.default = _default;
});
;define("ghost-admin/helpers/background-image-style", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.backgroundImageStyle = backgroundImageStyle;
  _exports.default = void 0;

  function backgroundImageStyle(_ref
  /*, hash*/
  ) {
    let [url] = _ref;

    if (url) {
      let safeUrl = encodeURI(decodeURI(url));
      return Ember.String.htmlSafe("background-image: url(".concat(safeUrl, ");"));
    }

    return '';
  }

  var _default = Ember.Helper.helper(backgroundImageStyle);

  _exports.default = _default;
});
;define("ghost-admin/helpers/camelize", ["exports", "ember-cli-string-helpers/helpers/camelize"], function (_exports, _camelize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _camelize.default;
    }
  });
  Object.defineProperty(_exports, "camelize", {
    enumerable: true,
    get: function get() {
      return _camelize.camelize;
    }
  });
});
;define("ghost-admin/helpers/cancel-all", ["exports", "ember-concurrency/helpers/cancel-all"], function (_exports, _cancelAll) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _cancelAll.default;
    }
  });
});
;define("ghost-admin/helpers/capitalize", ["exports", "ember-cli-string-helpers/helpers/capitalize"], function (_exports, _capitalize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _capitalize.default;
    }
  });
  Object.defineProperty(_exports, "capitalize", {
    enumerable: true,
    get: function get() {
      return _capitalize.capitalize;
    }
  });
});
;define("ghost-admin/helpers/classify", ["exports", "ember-cli-string-helpers/helpers/classify"], function (_exports, _classify) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _classify.default;
    }
  });
  Object.defineProperty(_exports, "classify", {
    enumerable: true,
    get: function get() {
      return _classify.classify;
    }
  });
});
;define("ghost-admin/helpers/clean-basic-html", ["exports", "koenig-editor/helpers/clean-basic-html"], function (_exports, _cleanBasicHtml) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _cleanBasicHtml.default;
    }
  });
});
;define("ghost-admin/helpers/dasherize", ["exports", "ember-cli-string-helpers/helpers/dasherize"], function (_exports, _dasherize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _dasherize.default;
    }
  });
  Object.defineProperty(_exports, "dasherize", {
    enumerable: true,
    get: function get() {
      return _dasherize.dasherize;
    }
  });
});
;define("ghost-admin/helpers/ember-power-calendar-day-classes", ["exports", "ember-power-calendar/helpers/ember-power-calendar-day-classes"], function (_exports, _emberPowerCalendarDayClasses) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberPowerCalendarDayClasses.default;
    }
  });
  Object.defineProperty(_exports, "emberPowerCalendarDayClasses", {
    enumerable: true,
    get: function get() {
      return _emberPowerCalendarDayClasses.emberPowerCalendarDayClasses;
    }
  });
});
;define("ghost-admin/helpers/ember-power-select-is-group", ["exports", "ember-power-select/helpers/ember-power-select-is-group"], function (_exports, _emberPowerSelectIsGroup) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberPowerSelectIsGroup.default;
    }
  });
  Object.defineProperty(_exports, "emberPowerSelectIsGroup", {
    enumerable: true,
    get: function get() {
      return _emberPowerSelectIsGroup.emberPowerSelectIsGroup;
    }
  });
});
;define("ghost-admin/helpers/ember-power-select-is-selected", ["exports", "ember-power-select/helpers/ember-power-select-is-selected"], function (_exports, _emberPowerSelectIsSelected) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberPowerSelectIsSelected.default;
    }
  });
  Object.defineProperty(_exports, "emberPowerSelectIsSelected", {
    enumerable: true,
    get: function get() {
      return _emberPowerSelectIsSelected.emberPowerSelectIsSelected;
    }
  });
});
;define("ghost-admin/helpers/ember-power-select-true-string-if-present", ["exports", "ember-power-select/helpers/ember-power-select-true-string-if-present"], function (_exports, _emberPowerSelectTrueStringIfPresent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberPowerSelectTrueStringIfPresent.default;
    }
  });
  Object.defineProperty(_exports, "emberPowerSelectTrueStringIfPresent", {
    enumerable: true,
    get: function get() {
      return _emberPowerSelectTrueStringIfPresent.emberPowerSelectTrueStringIfPresent;
    }
  });
});
;define("ghost-admin/helpers/eq", ["exports", "ember-truth-helpers/helpers/equal"], function (_exports, _equal) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _equal.default;
    }
  });
  Object.defineProperty(_exports, "equal", {
    enumerable: true,
    get: function get() {
      return _equal.equal;
    }
  });
});
;define("ghost-admin/helpers/event-name", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.eventName = eventName;
  _exports.default = _exports.AVAILABLE_EVENTS = void 0;
  const AVAILABLE_EVENTS = [// GROUPNAME: Global
  {
    event: 'site.changed',
    name: 'Site changed (rebuild)',
    group: 'Global'
  }, // GROUPNAME: Posts
  {
    event: 'post.added',
    name: 'Post created',
    group: 'Posts'
  }, {
    event: 'post.deleted',
    name: 'Post deleted',
    group: 'Posts'
  }, {
    event: 'post.edited',
    name: 'Post updated',
    group: 'Posts'
  }, {
    event: 'post.published',
    name: 'Post published',
    group: 'Posts'
  }, {
    event: 'post.published.edited',
    name: 'Published post updated',
    group: 'Posts'
  }, {
    event: 'post.unpublished',
    name: 'Post unpublished',
    group: 'Posts'
  }, {
    event: 'post.tag.attached',
    name: 'Tag added to post',
    group: 'Posts'
  }, {
    event: 'post.tag.detached',
    name: 'Tag removed from post',
    group: 'Posts'
  }, // GROUPNAME: Pages
  {
    event: 'page.added',
    name: 'Page created',
    group: 'Pages'
  }, {
    event: 'page.deleted',
    name: 'Page deleted',
    group: 'Pages'
  }, {
    event: 'page.edited',
    name: 'Page updated',
    group: 'Pages'
  }, {
    event: 'page.published',
    name: 'Page published',
    group: 'Pages'
  }, {
    event: 'page.published.edited',
    name: 'Published page updated',
    group: 'Pages'
  }, {
    event: 'page.unpublished',
    name: 'Page unpublished',
    group: 'Pages'
  }, {
    event: 'page.tag.attached',
    name: 'Tag added to page',
    group: 'Pages'
  }, {
    event: 'page.tag.detached',
    name: 'Tag removed from page',
    group: 'Pages'
  }, // GROUPNAME: Tags
  {
    event: 'tag.added',
    name: 'Tag created',
    group: 'Tags'
  }, {
    event: 'tag.edited',
    name: 'Tag updated',
    group: 'Tags'
  }, {
    event: 'tag.deleted',
    name: 'Tag deleted',
    group: 'Tags'
  }, // GROUPNAME: Subscribers
  {
    event: 'subscriber.added',
    name: 'Subscriber added',
    group: 'Subscribers'
  }, {
    event: 'subscriber.deleted',
    name: 'Subscriber deleted',
    group: 'Subscribers'
  }];
  _exports.AVAILABLE_EVENTS = AVAILABLE_EVENTS;

  function eventName(_ref
  /*, hash*/
  ) {
    let [event] = _ref;
    let match = AVAILABLE_EVENTS.findBy('event', event);
    return match ? match.name : event;
  }

  var _default = Ember.Helper.helper(eventName);

  _exports.default = _default;
});
;define("ghost-admin/helpers/gh-count-characters", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.countCharacters = countCharacters;
  _exports.default = void 0;

  function countCharacters(params) {
    if (!params || !params.length) {
      return;
    }

    let el = document.createElement('span');
    let content = params[0] || ''; // convert to array so that we get accurate symbol counts for multibyte chars
    // this will still count emoji+modifer as two chars

    let {
      length
    } = Array.from(content);
    el.className = 'word-count';

    if (length > 180) {
      el.style.color = '#f05230';
    } else {
      el.style.color = '#738a94';
    }

    el.innerHTML = 200 - length;
    return Ember.String.htmlSafe(el.outerHTML);
  }

  var _default = Ember.Helper.helper(function (params) {
    return countCharacters(params);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/gh-count-down-characters", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.countDownCharacters = countDownCharacters;
  _exports.default = void 0;

  function countDownCharacters(params) {
    if (!params || params.length < 2) {
      return;
    }

    let el = document.createElement('span');
    let [content, maxCharacters] = params; // convert to array so that we get accurate symbol counts for multibyte chars
    // this will still count emoji+modifer as two chars

    let {
      length
    } = Array.from(content || '');
    el.className = 'word-count';

    if (length > maxCharacters) {
      el.style.color = '#E25440';
    } else {
      el.style.color = '#9FBB58';
    }

    el.innerHTML = length;
    return Ember.String.htmlSafe(el.outerHTML);
  }

  var _default = Ember.Helper.helper(function (params) {
    return countDownCharacters(params);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/gh-format-html", ["exports", "ghost-admin/utils/caja-sanitizers"], function (_exports, _cajaSanitizers) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* global html_sanitize*/
  var _default = Ember.Helper.helper(function (params) {
    if (!params || !params.length) {
      return;
    }

    let escapedhtml = params[0] || ''; // replace script and iFrame

    escapedhtml = escapedhtml.replace(/<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi, '<pre class="js-embed-placeholder">Embedded JavaScript</pre>');
    escapedhtml = escapedhtml.replace(/<iframe\b[^<]*(?:(?!<\/iframe>)<[^<]*)*<\/iframe>/gi, '<pre class="iframe-embed-placeholder">Embedded iFrame</pre>'); // sanitize HTML

    /* eslint-disable camelcase */

    escapedhtml = html_sanitize(escapedhtml, _cajaSanitizers.default.url, _cajaSanitizers.default.id);
    /* eslint-enable camelcase */

    return Ember.String.htmlSafe(escapedhtml);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/gh-format-post-time", ["exports", "moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Helper.extend({
    settings: Ember.inject.service(),

    compute(_ref, _ref2) {
      let [timeago] = _ref;
      let {
        draft,
        scheduled,
        published
      } = _ref2;
      (true && !(timeago) && Ember.assert('You must pass a time to the gh-format-post-time helper', timeago));

      if (draft) {
        // No special handling for drafts, just use moment.from
        return (0, _moment.default)(timeago).from(_moment.default.utc());
      }

      let timezone = this.get('settings.activeTimezone');

      let time = _moment.default.tz(timeago, timezone);

      let now = _moment.default.tz(_moment.default.utc(), timezone); // If not a draft and post was published <= 15 minutes ago
      // or scheduled to be published <= 15 minutes from now, use moment.from


      if (Math.abs(now.diff(time, 'minutes')) <= 15) {
        return time.from(now);
      } // If scheduled for or published on the same day, render the time + Today


      if (time.isSame(now, 'day')) {
        let formatted = time.format('HH:mm [Today]');
        return scheduled ? "at ".concat(formatted) : formatted;
      } // if published yesterday, render time + yesterday
      // This check comes before scheduled, because there are likely to be more published
      // posts than scheduled posts.


      if (published && time.isSame(now.clone().subtract(1, 'days').startOf('day'), 'day')) {
        return time.format('HH:mm [Yesterday]');
      } // if scheduled for tomorrow, render the time + Tomorrow


      if (scheduled && time.isSame(now.clone().add(1, 'days').startOf('day'), 'day')) {
        return time.format('[at] HH:mm [Tomorrow]');
      } // Else, render just the date if published, or the time & date if scheduled


      let format = scheduled ? '[at] HH:mm [on] DD MMM YYYY' : 'DD MMM YYYY';
      return time.format(format);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/gh-path", ["exports", "ghost-admin/utils/ghost-paths"], function (_exports, _ghostPaths) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // Handlebars Helper {{gh-path}}
  // Usage: Assume 'http://www.myghostblog.org/myblog/'
  // {{gh-path}} or {{gh-path 'blog'}} for Ghost's root (/myblog/)
  // {{gh-path 'admin'}} for Ghost's admin root (/myblog/ghost/)
  // {{gh-path 'api'}} for Ghost's api root (/myblog/ghost/api/v0.1/)
  //
  // DO NOT USE - admin asset paths are now relative because we are using hash urls
  //              and the gh-path helper can get in the way of asset rewriting
  // {{gh-path 'asset' '/img/hi.png'}} for resolved url (/myblog/ghost/assets/img/hi.png)
  var _default = Ember.Helper.helper(function (params) {
    let paths = (0, _ghostPaths.default)();
    let [path, url] = params;
    let base;

    if (!path) {
      path = 'blog';
    }

    if (!/^(blog|admin|asset|api)$/.test(path)) {
      url = path;
      path = 'blog';
    }

    switch (path.toString()) {
      case 'blog':
        base = paths.blogRoot;
        break;

      case 'admin':
        base = paths.adminRoot;
        break;

      case 'asset':
        base = paths.assetRoot;
        break;

      case 'api':
        base = paths.apiRoot;
        break;

      default:
        base = paths.blogRoot;
        break;
    } // handle leading and trailing slashes


    base = base[base.length - 1] !== '/' ? "".concat(base, "/") : base;

    if (url && url.length > 0) {
      if (url[0] === '/') {
        url = url.substr(1);
      }

      base = base + url;
    }

    return Ember.String.htmlSafe(base);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/gh-user-can-admin", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ghUserCanAdmin = ghUserCanAdmin;
  _exports.default = void 0;

  // Handlebars Helper {{gh-user-can-admin}} group users by admin and owner using if, or group them author using unless
  // Usage: call helper as with aparameter of session.user
  // e.g - {{#if (gh-user-can-admin session.user)}} 'block content' {{/if}}
  // @param session.user
  function ghUserCanAdmin(params) {
    return !!params[0].get('isOwnerOrAdmin');
  }

  var _default = Ember.Helper.helper(function (params) {
    return ghUserCanAdmin(params);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/gt", ["exports", "ember-truth-helpers/helpers/gt"], function (_exports, _gt) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _gt.default;
    }
  });
  Object.defineProperty(_exports, "gt", {
    enumerable: true,
    get: function get() {
      return _gt.gt;
    }
  });
});
;define("ghost-admin/helpers/gte", ["exports", "ember-truth-helpers/helpers/gte"], function (_exports, _gte) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _gte.default;
    }
  });
  Object.defineProperty(_exports, "gte", {
    enumerable: true,
    get: function get() {
      return _gte.gte;
    }
  });
});
;define("ghost-admin/helpers/highlighted-text", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.highlightedText = highlightedText;
  _exports.default = void 0;

  function highlightedText(_ref) {
    let [text, termToHighlight] = _ref;
    // replace any non-word character with an escaped character
    let sanitisedTerm = termToHighlight.replace(new RegExp(/\W/ig), '\\$&');
    return Ember.String.htmlSafe(text.replace(new RegExp(sanitisedTerm, 'ig'), '<span class="highlight">$&</span>'));
  }

  var _default = Ember.Helper.helper(highlightedText);

  _exports.default = _default;
});
;define("ghost-admin/helpers/html-safe", ["exports", "ember-cli-string-helpers/helpers/html-safe"], function (_exports, _htmlSafe) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _htmlSafe.default;
    }
  });
  Object.defineProperty(_exports, "htmlSafe", {
    enumerable: true,
    get: function get() {
      return _htmlSafe.htmlSafe;
    }
  });
});
;define("ghost-admin/helpers/humanize", ["exports", "ember-cli-string-helpers/helpers/humanize"], function (_exports, _humanize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _humanize.default;
    }
  });
  Object.defineProperty(_exports, "humanize", {
    enumerable: true,
    get: function get() {
      return _humanize.humanize;
    }
  });
});
;define("ghost-admin/helpers/integration-icon-style", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.integrationLogoStyle = integrationLogoStyle;
  _exports.default = void 0;

  function integrationLogoStyle(_ref
  /*, hash*/
  ) {
    let [integration] = _ref;

    if (integration.iconImage) {
      let style = "background-image:url(".concat(integration.iconImage, ");background-size:45px;");
      return Ember.String.htmlSafe(style);
    }
  }

  var _default = Ember.Helper.helper(integrationLogoStyle);

  _exports.default = _default;
});
;define("ghost-admin/helpers/is-after", ["exports", "ember-moment/helpers/is-after"], function (_exports, _isAfter) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isAfter.default;
    }
  });
});
;define("ghost-admin/helpers/is-array", ["exports", "ember-truth-helpers/helpers/is-array"], function (_exports, _isArray) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isArray.default;
    }
  });
  Object.defineProperty(_exports, "isArray", {
    enumerable: true,
    get: function get() {
      return _isArray.isArray;
    }
  });
});
;define("ghost-admin/helpers/is-before", ["exports", "ember-moment/helpers/is-before"], function (_exports, _isBefore) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isBefore.default;
    }
  });
});
;define("ghost-admin/helpers/is-between", ["exports", "ember-moment/helpers/is-between"], function (_exports, _isBetween) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isBetween.default;
    }
  });
});
;define("ghost-admin/helpers/is-empty", ["exports", "ember-truth-helpers/helpers/is-empty"], function (_exports, _isEmpty) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isEmpty.default;
    }
  });
});
;define("ghost-admin/helpers/is-equal", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.isEqual = isEqual;
  _exports.default = void 0;

  function isEqual(params) {
    let [lhs, rhs] = params;
    return lhs === rhs;
  }

  var _default = Ember.Helper.helper(function (params) {
    return isEqual(params);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/is-not", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.isNot = isNot;
  _exports.default = void 0;

  function isNot(params) {
    return !params;
  }

  var _default = Ember.Helper.helper(function (params) {
    return isNot(params);
  });

  _exports.default = _default;
});
;define("ghost-admin/helpers/is-same-or-after", ["exports", "ember-moment/helpers/is-same-or-after"], function (_exports, _isSameOrAfter) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isSameOrAfter.default;
    }
  });
});
;define("ghost-admin/helpers/is-same-or-before", ["exports", "ember-moment/helpers/is-same-or-before"], function (_exports, _isSameOrBefore) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isSameOrBefore.default;
    }
  });
});
;define("ghost-admin/helpers/is-same", ["exports", "ember-moment/helpers/is-same"], function (_exports, _isSame) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _isSame.default;
    }
  });
});
;define("ghost-admin/helpers/kg-style", ["exports", "koenig-editor/helpers/kg-style"], function (_exports, _kgStyle) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _kgStyle.default;
    }
  });
  Object.defineProperty(_exports, "kgStyle", {
    enumerable: true,
    get: function get() {
      return _kgStyle.kgStyle;
    }
  });
});
;define("ghost-admin/helpers/lf-lock-model", ["exports", "liquid-fire/helpers/lf-lock-model"], function (_exports, _lfLockModel) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _lfLockModel.default;
    }
  });
  Object.defineProperty(_exports, "lfLockModel", {
    enumerable: true,
    get: function get() {
      return _lfLockModel.lfLockModel;
    }
  });
});
;define("ghost-admin/helpers/lf-or", ["exports", "liquid-fire/helpers/lf-or"], function (_exports, _lfOr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _lfOr.default;
    }
  });
  Object.defineProperty(_exports, "lfOr", {
    enumerable: true,
    get: function get() {
      return _lfOr.lfOr;
    }
  });
});
;define("ghost-admin/helpers/lowercase", ["exports", "ember-cli-string-helpers/helpers/lowercase"], function (_exports, _lowercase) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _lowercase.default;
    }
  });
  Object.defineProperty(_exports, "lowercase", {
    enumerable: true,
    get: function get() {
      return _lowercase.lowercase;
    }
  });
});
;define("ghost-admin/helpers/lt", ["exports", "ember-truth-helpers/helpers/lt"], function (_exports, _lt) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _lt.default;
    }
  });
  Object.defineProperty(_exports, "lt", {
    enumerable: true,
    get: function get() {
      return _lt.lt;
    }
  });
});
;define("ghost-admin/helpers/lte", ["exports", "ember-truth-helpers/helpers/lte"], function (_exports, _lte) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _lte.default;
    }
  });
  Object.defineProperty(_exports, "lte", {
    enumerable: true,
    get: function get() {
      return _lte.lte;
    }
  });
});
;define("ghost-admin/helpers/moment-add", ["exports", "ember-moment/helpers/moment-add"], function (_exports, _momentAdd) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentAdd.default;
    }
  });
});
;define("ghost-admin/helpers/moment-calendar", ["exports", "ember-moment/helpers/moment-calendar"], function (_exports, _momentCalendar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentCalendar.default;
    }
  });
});
;define("ghost-admin/helpers/moment-diff", ["exports", "ember-moment/helpers/moment-diff"], function (_exports, _momentDiff) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentDiff.default;
    }
  });
});
;define("ghost-admin/helpers/moment-duration", ["exports", "ember-moment/helpers/moment-duration"], function (_exports, _momentDuration) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentDuration.default;
    }
  });
});
;define("ghost-admin/helpers/moment-format", ["exports", "ember-moment/helpers/moment-format"], function (_exports, _momentFormat) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentFormat.default;
    }
  });
});
;define("ghost-admin/helpers/moment-from-now", ["exports", "ember-moment/helpers/moment-from-now"], function (_exports, _momentFromNow) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentFromNow.default;
    }
  });
});
;define("ghost-admin/helpers/moment-from", ["exports", "ember-moment/helpers/moment-from"], function (_exports, _momentFrom) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentFrom.default;
    }
  });
});
;define("ghost-admin/helpers/moment-subtract", ["exports", "ember-moment/helpers/moment-subtract"], function (_exports, _momentSubtract) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentSubtract.default;
    }
  });
});
;define("ghost-admin/helpers/moment-to-date", ["exports", "ember-moment/helpers/moment-to-date"], function (_exports, _momentToDate) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentToDate.default;
    }
  });
});
;define("ghost-admin/helpers/moment-to-now", ["exports", "ember-moment/helpers/moment-to-now"], function (_exports, _momentToNow) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentToNow.default;
    }
  });
});
;define("ghost-admin/helpers/moment-to", ["exports", "ember-moment/helpers/moment-to"], function (_exports, _momentTo) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _momentTo.default;
    }
  });
});
;define("ghost-admin/helpers/moment-unix", ["exports", "ember-moment/helpers/unix"], function (_exports, _unix) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _unix.default;
    }
  });
});
;define("ghost-admin/helpers/moment", ["exports", "ember-moment/helpers/moment"], function (_exports, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _moment.default;
    }
  });
});
;define("ghost-admin/helpers/not-eq", ["exports", "ember-truth-helpers/helpers/not-equal"], function (_exports, _notEqual) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _notEqual.default;
    }
  });
  Object.defineProperty(_exports, "notEq", {
    enumerable: true,
    get: function get() {
      return _notEqual.notEq;
    }
  });
});
;define("ghost-admin/helpers/not", ["exports", "ember-truth-helpers/helpers/not"], function (_exports, _not) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _not.default;
    }
  });
  Object.defineProperty(_exports, "not", {
    enumerable: true,
    get: function get() {
      return _not.not;
    }
  });
});
;define("ghost-admin/helpers/now", ["exports", "ember-moment/helpers/now"], function (_exports, _now) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _now.default;
    }
  });
});
;define("ghost-admin/helpers/one-way-select/contains", ["exports", "ember-one-way-select/helpers/one-way-select/contains"], function (_exports, _contains) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _contains.default;
    }
  });
  Object.defineProperty(_exports, "contains", {
    enumerable: true,
    get: function get() {
      return _contains.contains;
    }
  });
});
;define("ghost-admin/helpers/optional", ["exports", "ember-composable-helpers/helpers/optional"], function (_exports, _optional) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _optional.default;
    }
  });
  Object.defineProperty(_exports, "optional", {
    enumerable: true,
    get: function get() {
      return _optional.optional;
    }
  });
});
;define("ghost-admin/helpers/or", ["exports", "ember-truth-helpers/helpers/or"], function (_exports, _or) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _or.default;
    }
  });
  Object.defineProperty(_exports, "or", {
    enumerable: true,
    get: function get() {
      return _or.or;
    }
  });
});
;define("ghost-admin/helpers/perform", ["exports", "ember-concurrency/helpers/perform"], function (_exports, _perform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _perform.default;
    }
  });
});
;define("ghost-admin/helpers/pluralize", ["exports", "ember-inflector/lib/helpers/pluralize"], function (_exports, _pluralize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _pluralize.default;
  _exports.default = _default;
});
;define("ghost-admin/helpers/power-calendar-format-date", ["exports", "ember-power-calendar/helpers/power-calendar-format-date"], function (_exports, _powerCalendarFormatDate) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerCalendarFormatDate.default;
    }
  });
  Object.defineProperty(_exports, "powerCalendarFormatDate", {
    enumerable: true,
    get: function get() {
      return _powerCalendarFormatDate.powerCalendarFormatDate;
    }
  });
});
;define("ghost-admin/helpers/route-action", ["exports", "ember-route-action-helper/helpers/route-action"], function (_exports, _routeAction) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _routeAction.default;
    }
  });
});
;define("ghost-admin/helpers/sanitize-html", ["exports", "koenig-editor/helpers/sanitize-html"], function (_exports, _sanitizeHtml) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _sanitizeHtml.default;
    }
  });
  Object.defineProperty(_exports, "sanitizeHtml", {
    enumerable: true,
    get: function get() {
      return _sanitizeHtml.sanitizeHtml;
    }
  });
});
;define("ghost-admin/helpers/singularize", ["exports", "ember-inflector/lib/helpers/singularize"], function (_exports, _singularize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _singularize.default;
  _exports.default = _default;
});
;define("ghost-admin/helpers/subscribers-query-params", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.subscribersQueryParams = subscribersQueryParams;
  _exports.default = void 0;

  function subscribersQueryParams(_ref) {
    let [order, currentOrder, direction] = _ref;

    // if order hasn't changed we toggle the direction
    if (order === currentOrder) {
      direction = direction === 'asc' ? 'desc' : 'asc';
    }

    return ['subscribers', {
      isQueryParams: true,
      values: {
        order,
        direction
      }
    }];
  }

  var _default = Ember.Helper.helper(subscribersQueryParams);

  _exports.default = _default;
});
;define("ghost-admin/helpers/subscribers-sort-icon", ["exports", "ghost-admin/helpers/svg-jar"], function (_exports, _svgJar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.subscribersSortIcon = subscribersSortIcon;
  _exports.default = void 0;

  function subscribersSortIcon(_ref) {
    let [order, currentOrder, direction] = _ref;

    if (order === currentOrder) {
      if (direction === 'asc') {
        return (0, _svgJar.svgJar)('arrow-up', {
          class: 'ih2 mr2'
        });
      } else {
        return (0, _svgJar.svgJar)('arrow-down', {
          class: 'ih2 mr2'
        });
      }
    }
  }

  var _default = Ember.Helper.helper(subscribersSortIcon);

  _exports.default = _default;
});
;define("ghost-admin/helpers/svg-jar", ["exports", "ember-svg-jar/utils/make-helper", "ember-svg-jar/utils/make-svg"], function (_exports, _makeHelper, _makeSvg) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.svgJar = svgJar;
  _exports.default = void 0;

  function getInlineAsset(assetId) {
    try {
      /* eslint-disable global-require */
      return require("ember-svg-jar/inlined/".concat(assetId)).default;
    } catch (err) {
      return null;
    }
  }

  function svgJar(assetId, svgAttrs) {
    return (0, _makeSvg.default)(assetId, svgAttrs, getInlineAsset);
  }

  var _default = (0, _makeHelper.default)(svgJar);

  _exports.default = _default;
});
;define("ghost-admin/helpers/task", ["exports", "ember-concurrency/helpers/task"], function (_exports, _task) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _task.default;
    }
  });
});
;define("ghost-admin/helpers/titleize", ["exports", "ember-cli-string-helpers/helpers/titleize"], function (_exports, _titleize) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _titleize.default;
    }
  });
  Object.defineProperty(_exports, "titleize", {
    enumerable: true,
    get: function get() {
      return _titleize.titleize;
    }
  });
});
;define("ghost-admin/helpers/toggle", ["exports", "ember-composable-helpers/helpers/toggle"], function (_exports, _toggle) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _toggle.default;
    }
  });
  Object.defineProperty(_exports, "toggle", {
    enumerable: true,
    get: function get() {
      return _toggle.toggle;
    }
  });
});
;define("ghost-admin/helpers/trim", ["exports", "ember-cli-string-helpers/helpers/trim"], function (_exports, _trim) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _trim.default;
    }
  });
  Object.defineProperty(_exports, "trim", {
    enumerable: true,
    get: function get() {
      return _trim.trim;
    }
  });
});
;define("ghost-admin/helpers/truncate", ["exports", "ember-cli-string-helpers/helpers/truncate"], function (_exports, _truncate) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _truncate.default;
    }
  });
  Object.defineProperty(_exports, "truncate", {
    enumerable: true,
    get: function get() {
      return _truncate.truncate;
    }
  });
});
;define("ghost-admin/helpers/ui-btn-span", ["exports", "ghost-admin/helpers/ui-btn"], function (_exports, _uiBtn) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.uiBtnSpan = uiBtnSpan;
  _exports.default = void 0;

  function uiBtnSpan(_ref, hash) {
    let [style] = _ref;
    return (0, _uiBtn.btnStyles)(Object.assign({}, {
      style
    }, hash)).span;
  }

  var _default = Ember.Helper.helper(uiBtnSpan);

  _exports.default = _default;
});
;define("ghost-admin/helpers/ui-btn", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.btnStyles = btnStyles;
  _exports.uiBtn = uiBtn;
  _exports.default = void 0;

  function btnStyles() {
    let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let button = 'dib midgrey btn-base br3 ba b--lightgrey-l1 pointer glow';
    let span = 'f8 fw5 tracked-2 dib pt0 pb0 tc'; // Set style

    if (options.style) {
      switch (options.style) {
        case 'outline-white':
        case 'outline-white--s':
          button = 'bg-transparent dib white btn-base br3 ba b--white-60 pointer highlight-white';
          span = 'f8 fw5 tracked-2 dib pt0 pb0 tc';
          break;

        case 'outline-blue':
        case 'outline-blue--s':
          button = 'bg-transparent dib blue btn-base br3 ba b--lightgrey pointer glow';
          span = 'f8 fw5 tracked-2 dib pt0 pb0 tc';
          break;

        case 'blue':
        case 'blue--s':
          button = 'dib bw0 white br3 btn-base btn-blue pointer';
          span = 'f8 fw5 tracked-2 dib pt0 pb0 tc';
          break;

        case 'green':
        case 'green--s':
          button = 'dib bw0 white br3 btn-base btn-green pointer';
          span = 'f8 fw5 tracked-2 dib pt0 pb0 tc';
          break;

        case 'red':
        case 'red--s':
          button = 'dib bw0 white br3 btn-base btn-red pointer';
          span = 'f8 fw5 tracked-2 dib pt0 pb0 tc';
          break;
      } // Style ending with '--s' means small button


      if (options.style.substr(options.style.length - 3) === '--s') {
        button = "".concat(button, " btn-small");
      }
    }

    if (options.class) {
      button = "".concat(button, " ").concat(options.class);
    }

    button = "sans-serif ".concat(button, " flex-shrink-0");
    span = "".concat(span, " inline-flex items-center");
    return {
      button: button,
      span: span
    };
  }

  function uiBtn(_ref, hash) {
    let [style] = _ref;
    return btnStyles(Object.assign({}, {
      style
    }, hash)).button;
  }

  var _default = Ember.Helper.helper(uiBtn);

  _exports.default = _default;
});
;define("ghost-admin/helpers/ui-text", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.uiText = uiText;
  _exports.default = void 0;

  function uiText(_ref) {
    let [style] = _ref;
    let cssClass = '';

    switch (style) {
      case 'h1':
        cssClass = 'f-subheadline fw7 tracked-3 lh-heading ma0 pa0';
        break;

      case 'h2':
        cssClass = 'f2 fw6 tracked-2 tracked-3 lh-title ma0 pa0';
        break;

      case 'h3':
        cssClass = 'f5 fw6 tracked-2 lh-title ma0 pa0';
        break;

      case 'h4':
        cssClass = 'f7 fw6 tracked-2 lh-copy ma0 pa0';
        break;

      case 'h5':
        cssClass = 'f8 fw6 tracked-2 lh-copy ma0 pa0';
        break;

      case 'h6':
        cssClass = 'f-small ttu fw4 tracked-3 lh-copy ma0 pa0';
        break;

      case 'tl':
        cssClass = 'f6 fw3 lh-copy tracked-1 ma0 pa0';
        break;

      case 't':
        cssClass = 'f7 fw3 lh-copy tracked-1 ma0 pa0';
        break;

      case 'ts':
        cssClass = 'f8 fw3 lh-copy tracked-2 ma0 pa0';
        break;

      case 'txs':
        cssClass = 'f-small fw3 lh-copy tracked-3 ma0 pa0';
        break;
    }

    return cssClass;
  }

  var _default = Ember.Helper.helper(uiText);

  _exports.default = _default;
});
;define("ghost-admin/helpers/underscore", ["exports", "ember-cli-string-helpers/helpers/underscore"], function (_exports, _underscore) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _underscore.default;
    }
  });
  Object.defineProperty(_exports, "underscore", {
    enumerable: true,
    get: function get() {
      return _underscore.underscore;
    }
  });
});
;define("ghost-admin/helpers/unix", ["exports", "ember-moment/helpers/unix"], function (_exports, _unix) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _unix.default;
    }
  });
});
;define("ghost-admin/helpers/uppercase", ["exports", "ember-cli-string-helpers/helpers/uppercase"], function (_exports, _uppercase) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _uppercase.default;
    }
  });
  Object.defineProperty(_exports, "uppercase", {
    enumerable: true,
    get: function get() {
      return _uppercase.uppercase;
    }
  });
});
;define("ghost-admin/helpers/user-agent", ["exports", "ember-useragent/helpers/user-agent"], function (_exports, _userAgent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _userAgent.default;
    }
  });
});
;define("ghost-admin/helpers/utc", ["exports", "ember-moment/helpers/utc"], function (_exports, _utc) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _utc.default;
    }
  });
  Object.defineProperty(_exports, "utc", {
    enumerable: true,
    get: function get() {
      return _utc.utc;
    }
  });
});
;define("ghost-admin/helpers/w", ["exports", "ember-cli-string-helpers/helpers/w"], function (_exports, _w) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _w.default;
    }
  });
  Object.defineProperty(_exports, "w", {
    enumerable: true,
    get: function get() {
      return _w.w;
    }
  });
});
;define("ghost-admin/helpers/xor", ["exports", "ember-truth-helpers/helpers/xor"], function (_exports, _xor) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _xor.default;
    }
  });
  Object.defineProperty(_exports, "xor", {
    enumerable: true,
    get: function get() {
      return _xor.xor;
    }
  });
});
;define("ghost-admin/initializers/app-version", ["exports", "ember-cli-app-version/initializer-factory", "ghost-admin/config/environment"], function (_exports, _initializerFactory, _environment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  let name, version;

  if (_environment.default.APP) {
    name = _environment.default.APP.name;
    version = _environment.default.APP.version;
  }

  var _default = {
    name: 'App Version',
    initialize: (0, _initializerFactory.default)(name, version)
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/container-debug-adapter", ["exports", "ember-resolver/resolvers/classic/container-debug-adapter"], function (_exports, _containerDebugAdapter) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = {
    name: 'container-debug-adapter',

    initialize() {
      let app = arguments[1] || arguments[0];
      app.register('container-debug-adapter:main', _containerDebugAdapter.default);
      app.inject('container-debug-adapter:main', 'namespace', 'application:main');
    }

  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/coordinator-setup", ["exports", "ghost-admin/models/coordinator"], function (_exports, _coordinator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = {
    name: "setup coordinator",
    initialize: function initialize() {
      let app = arguments[1] || arguments[0];
      app.register("drag:coordinator", _coordinator.default);
      app.inject("component", "coordinator", "drag:coordinator");
    }
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/debug", ["exports", "@html-next/vertical-collection/-debug"], function (_exports, _debug) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = {
    name: 'vertical-collection-debug',

    initialize() {}

  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/ember-cli-mirage", ["exports", "ghost-admin/config/environment", "ghost-admin/mirage/config", "ember-cli-mirage/get-rfc232-test-context", "ember-cli-mirage/start-mirage"], function (_exports, _environment, _config, _getRfc232TestContext, _startMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.startMirage = startMirage;
  _exports.default = void 0;
  //
  // This initializer does two things:
  //
  // 1. Pulls the mirage config objects from the application's config and
  //    registers them in the container so `ember-cli-mirage/start-mirage` can
  //    find them (since it doesn't have access to the app's namespace).
  // 2. Provides legacy support for auto-starting mirage in pre-rfc268 acceptance
  //    tests.
  //
  var _default = {
    name: 'ember-cli-mirage',

    initialize(application) {
      if (_config.default) {
        application.register('mirage:base-config', _config.default, {
          instantiate: false
        });
      }

      if (_config.testConfig) {
        application.register('mirage:test-config', _config.testConfig, {
          instantiate: false
        });
      }

      _environment.default['ember-cli-mirage'] = _environment.default['ember-cli-mirage'] || {};

      if (_shouldUseMirage(_environment.default.environment, _environment.default['ember-cli-mirage'])) {
        startMirage(_environment.default);
      }
    }

  };
  _exports.default = _default;

  function startMirage() {
    let env = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _environment.default;
    return (0, _startMirage.default)(null, {
      env,
      baseConfig: _config.default,
      testConfig: _config.testConfig
    });
  }

  function _shouldUseMirage(env, addonConfig) {
    if (typeof FastBoot !== 'undefined') {
      return false;
    }

    if ((0, _getRfc232TestContext.default)()) {
      return false;
    }

    let userDeclaredEnabled = typeof addonConfig.enabled !== 'undefined';

    let defaultEnabled = _defaultEnabled(env, addonConfig);

    return userDeclaredEnabled ? addonConfig.enabled : defaultEnabled;
  }
  /*
    Returns a boolean specifying the default behavior for whether
    to initialize Mirage.
  */


  function _defaultEnabled(env, addonConfig) {
    let usingInDev = env === 'development' && !addonConfig.usingProxy;
    let usingInTest = env === 'test';
    return usingInDev || usingInTest;
  }
});
;define("ghost-admin/initializers/ember-concurrency", ["exports", "ember-concurrency/initializers/ember-concurrency"], function (_exports, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberConcurrency.default;
    }
  });
});
;define("ghost-admin/initializers/ember-data", ["exports", "ember-data/setup-container", "ember-data"], function (_exports, _setupContainer, _emberData) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /*
  
    This code initializes Ember-Data onto an Ember application.
  
    If an Ember.js developer defines a subclass of DS.Store on their application,
    as `App.StoreService` (or via a module system that resolves to `service:store`)
    this code will automatically instantiate it and make it available on the
    router.
  
    Additionally, after an application's controllers have been injected, they will
    each have the store made available to them.
  
    For example, imagine an Ember.js application with the following classes:
  
    ```app/services/store.js
    import DS from 'ember-data';
  
    export default DS.Store.extend({
      adapter: 'custom'
    });
    ```
  
    ```app/controllers/posts.js
    import { Controller } from '@ember/controller';
  
    export default Controller.extend({
      // ...
    });
  
    When the application is initialized, `ApplicationStore` will automatically be
    instantiated, and the instance of `PostsController` will have its `store`
    property set to that instance.
  
    Note that this code will only be run if the `ember-application` package is
    loaded. If Ember Data is being used in an environment other than a
    typical application (e.g., node.js where only `ember-runtime` is available),
    this code will be ignored.
  */
  var _default = {
    name: 'ember-data',
    initialize: _setupContainer.default
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/ember-simple-auth", ["exports", "ember-simple-auth/configuration", "ghost-admin/config/environment", "ghost-admin/utils/ghost-paths", "ember-simple-auth/initializers/setup-session", "ember-simple-auth/initializers/setup-session-restoration", "ember-simple-auth/initializers/setup-session-service"], function (_exports, _configuration, _environment, _ghostPaths, _setupSession, _setupSessionRestoration, _setupSessionService) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = {
    name: 'ember-simple-auth',

    initialize(registry) {
      let config = _environment.default['ember-simple-auth'] || {};
      config.rootURL = (0, _ghostPaths.default)().adminRoot;

      _configuration.default.load(config);

      (0, _setupSession.default)(registry);
      (0, _setupSessionService.default)(registry);
      (0, _setupSessionRestoration.default)(registry);
    }

  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/export-application-global", ["exports", "ghost-admin/config/environment"], function (_exports, _environment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.initialize = initialize;
  _exports.default = void 0;

  function initialize() {
    var application = arguments[1] || arguments[0];

    if (_environment.default.exportApplicationGlobal !== false) {
      var theGlobal;

      if (typeof window !== 'undefined') {
        theGlobal = window;
      } else if (typeof global !== 'undefined') {
        theGlobal = global;
      } else if (typeof self !== 'undefined') {
        theGlobal = self;
      } else {
        // no reasonable global, just bail
        return;
      }

      var value = _environment.default.exportApplicationGlobal;
      var globalName;

      if (typeof value === 'string') {
        globalName = value;
      } else {
        globalName = Ember.String.classify(_environment.default.modulePrefix);
      }

      if (!theGlobal[globalName]) {
        theGlobal[globalName] = application;
        application.reopen({
          willDestroy: function willDestroy() {
            this._super.apply(this, arguments);

            delete theGlobal[globalName];
          }
        });
      }
    }
  }

  var _default = {
    name: 'export-application-global',
    initialize: initialize
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/hide-loading-screen", ["exports", "ghost-admin/instance-initializers/hide-loading-screen"], function (_exports, _hideLoadingScreen) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.initialize = initialize;
  _exports.default = void 0;
  const EMBER_VERSION_REGEX = /^([0-9]+)\.([0-9]+)\.([0-9]+)(?:(?:-(alpha|beta)\.([0-9]+)(?:\.([0-9]+))?)?)?(?:\+(canary))?(?:\.([0-9abcdef]+))?(?:-([A-Za-z0-9.-]+))?(?:\+([A-Za-z0-9.-]+))?$/;
  /**
   * VERSION_INFO[i] is as follows:
   *
   * 0  complete version string
   * 1  major version
   * 2  minor version
   * 3  trivial version
   * 4  pre-release type (optional: "alpha" or "beta" or undefined for stable releases)
   * 5  pre-release version (optional)
   * 6  pre-release sub-version (optional)
   * 7  canary (optional: "canary", or undefined for stable releases)
   * 8  SHA (optional)
   *
   * @private
   */

  const VERSION_INFO = EMBER_VERSION_REGEX.exec(Ember.VERSION);
  const isPre111 = parseInt(VERSION_INFO[1], 10) < 2 && parseInt(VERSION_INFO[2], 10) < 12;

  function initialize() {
    if (isPre111) {
      let [registry, application] = arguments;

      _hideLoadingScreen.default.initialize(registry, application);
    }
  }

  var _default = {
    name: 'hide-loading-screen',
    initialize
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/liquid-fire", ["exports", "liquid-fire/ember-internals", "liquid-fire/velocity-ext"], function (_exports, _emberInternals, _velocityExt) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  (0, _emberInternals.initialize)();
  var _default = {
    name: 'liquid-fire',
    initialize: function initialize() {}
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/trailing-hash", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  let trailingHash = Ember.HashLocation.extend({
    formatURL() {
      let url = this._super(...arguments);

      if (url.indexOf('?') > 0) {
        return url.replace(/([^/])\?/, '$1/?');
      } else {
        return url.replace(/\/?$/, '/');
      }
    }

  });
  var _default = {
    name: 'registerTrailingHashLocation',

    initialize(application) {
      application.register('location:trailing-hash', trailingHash);
    }

  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/upgrade-status", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.initialize = initialize;
  _exports.default = void 0;

  function initialize(application) {
    application.inject('route', 'upgradeStatus', 'service:upgrade-status');
  }

  var _default = {
    name: 'upgrade-status',
    initialize
  };
  _exports.default = _default;
});
;define("ghost-admin/initializers/user-agent", ["exports", "ember-useragent/initializers/user-agent"], function (_exports, _userAgent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _userAgent.default;
    }
  });
  Object.defineProperty(_exports, "initialize", {
    enumerable: true,
    get: function get() {
      return _userAgent.initialize;
    }
  });
});
;define("ghost-admin/initializers/viewport-config", ["exports", "ember-in-viewport/initializers/viewport-config"], function (_exports, _viewportConfig) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _viewportConfig.default;
    }
  });
  Object.defineProperty(_exports, "initialize", {
    enumerable: true,
    get: function get() {
      return _viewportConfig.initialize;
    }
  });
});
;define("ghost-admin/instance-initializers/ember-cli-mirage-autostart", ["exports", "ember-cli-mirage/instance-initializers/ember-cli-mirage-autostart"], function (_exports, _emberCliMirageAutostart) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _emberCliMirageAutostart.default;
    }
  });
});
;define("ghost-admin/instance-initializers/ember-data", ["exports", "ember-data/initialize-store-service"], function (_exports, _initializeStoreService) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = {
    name: 'ember-data',
    initialize: _initializeStoreService.default
  };
  _exports.default = _default;
});
;define("ghost-admin/instance-initializers/ember-simple-auth", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  // This is only needed for backwards compatibility and will be removed in the
  // next major release of ember-simple-auth. Unfortunately, there is no way to
  // deprecate this without hooking into Ember's internals…
  var _default = {
    name: 'ember-simple-auth',

    initialize() {}

  };
  _exports.default = _default;
});
;define("ghost-admin/instance-initializers/hide-loading-screen", ["exports", "ghost-admin/config/environment"], function (_exports, _environment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.initialize = initialize;
  _exports.default = void 0;
  const userConfig = _environment.default['ember-load'] || {};

  function initialize() {
    let instance = arguments[1] || arguments[0];
    let container = arguments[1] ? arguments[0] : instance.container;

    if (Ember.View) {
      let ApplicationView = container.lookupFactory ? container.lookupFactory('view:application') : instance.resolveRegistration('view:application');
      ApplicationView.reopen({
        didInsertElement() {
          this._super(...arguments);

          let loadingIndicatorClass = userConfig.loadingIndicatorClass || 'ember-load-indicator';
          document.querySelectorAll(".".concat(loadingIndicatorClass)).forEach(e => e.remove());
        }

      });
    }
  }

  var _default = {
    name: 'hide-loading-screen-instance',
    initialize
  };
  _exports.default = _default;
});
;define("ghost-admin/instance-initializers/user-agent", ["exports", "ember-useragent/instance-initializers/user-agent"], function (_exports, _userAgent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _userAgent.default;
    }
  });
  Object.defineProperty(_exports, "initialize", {
    enumerable: true,
    get: function get() {
      return _userAgent.initialize;
    }
  });
});
;define("ghost-admin/koenig-editor/tests/addon.lint-test", [], function () {
  "use strict";

  describe('ESLint | addon', function () {
    it('addon/components/kg-action-bar.js', function () {// test passed
    });
    it('addon/components/koenig-basic-html-input.js', function () {// test passed
    });
    it('addon/components/koenig-caption-input.js', function () {// test passed
    });
    it('addon/components/koenig-card-code.js', function () {// test passed
    });
    it('addon/components/koenig-card-embed.js', function () {// test passed
    });
    it('addon/components/koenig-card-gallery.js', function () {// test passed
    });
    it('addon/components/koenig-card-hr.js', function () {// test passed
    });
    it('addon/components/koenig-card-html.js', function () {// test passed
    });
    it('addon/components/koenig-card-image.js', function () {// test passed
    });
    it('addon/components/koenig-card-markdown.js', function () {// test passed
    });
    it('addon/components/koenig-card.js', function () {// test passed
    });
    it('addon/components/koenig-editor.js', function () {// test passed
    });
    it('addon/components/koenig-link-input.js', function () {// test passed
    });
    it('addon/components/koenig-link-toolbar.js', function () {// test passed
    });
    it('addon/components/koenig-menu-content.js', function () {// test passed
    });
    it('addon/components/koenig-plus-menu.js', function () {// test passed
    });
    it('addon/components/koenig-slash-menu.js', function () {// test passed
    });
    it('addon/components/koenig-toolbar.js', function () {// test passed
    });
    it('addon/helpers/clean-basic-html.js', function () {// test passed
    });
    it('addon/helpers/kg-style.js', function () {// test passed
    });
    it('addon/helpers/sanitize-html.js', function () {// test passed
    });
    it('addon/lib/dnd/constants.js', function () {// test passed
    });
    it('addon/lib/dnd/container.js', function () {// test passed
    });
    it('addon/lib/dnd/scroll-handler.js', function () {// test passed
    });
    it('addon/lib/dnd/utils.js', function () {// test passed
    });
    it('addon/options/atoms.js', function () {// test passed
    });
    it('addon/options/basic-html-parser-plugins.js', function () {// test passed
    });
    it('addon/options/cards.js', function () {// test passed
    });
    it('addon/options/key-commands.js', function () {// test passed
    });
    it('addon/options/text-expansions.js', function () {// test passed
    });
    it('addon/services/koenig-drag-drop-handler.js', function () {// test passed
    });
    it('addon/services/koenig-ui.js', function () {// test passed
    });
    it('addon/utils/create-component-card.js', function () {// test passed
    });
    it('addon/utils/markup-utils.js', function () {// test passed
    });
    it('addon/utils/oembed.js', function () {// test passed
    });
    it('addon/utils/reading-time.js', function () {// test passed
    });
  });
});
;define("ghost-admin/koenig-editor/tests/app.lint-test", [], function () {
  "use strict";

  describe('ESLint | app', function () {
    it('app/components/kg-action-bar.js', function () {// test passed
    });
    it('app/components/koenig-basic-html-input.js', function () {// test passed
    });
    it('app/components/koenig-caption-input.js', function () {// test passed
    });
    it('app/components/koenig-card-code.js', function () {// test passed
    });
    it('app/components/koenig-card-embed.js', function () {// test passed
    });
    it('app/components/koenig-card-gallery.js', function () {// test passed
    });
    it('app/components/koenig-card-hr.js', function () {// test passed
    });
    it('app/components/koenig-card-html.js', function () {// test passed
    });
    it('app/components/koenig-card-image.js', function () {// test passed
    });
    it('app/components/koenig-card-markdown.js', function () {// test passed
    });
    it('app/components/koenig-card.js', function () {// test passed
    });
    it('app/components/koenig-editor.js', function () {// test passed
    });
    it('app/components/koenig-link-input.js', function () {// test passed
    });
    it('app/components/koenig-link-toolbar.js', function () {// test passed
    });
    it('app/components/koenig-menu-content.js', function () {// test passed
    });
    it('app/components/koenig-plus-menu.js', function () {// test passed
    });
    it('app/components/koenig-slash-menu.js', function () {// test passed
    });
    it('app/components/koenig-toolbar.js', function () {// test passed
    });
    it('app/helpers/clean-basic-html.js', function () {// test passed
    });
    it('app/helpers/kg-style.js', function () {// test passed
    });
    it('app/helpers/sanitize-html.js', function () {// test passed
    });
    it('app/services/koenig-drag-drop-handler.js', function () {// test passed
    });
    it('app/services/koenig-ui.js', function () {// test passed
    });
  });
});
;define("ghost-admin/mirage/config", ["exports", "ghost-admin/mirage/config/api-keys", "ghost-admin/mirage/config/authentication", "ghost-admin/mirage/config/config", "ghost-admin/mirage/config/integrations", "ghost-admin/mirage/config/members", "ghost-admin/mirage/config/pages", "ghost-admin/mirage/config/posts", "ghost-admin/mirage/config/roles", "ghost-admin/mirage/config/settings", "ghost-admin/mirage/config/site", "ghost-admin/mirage/config/slugs", "ghost-admin/mirage/config/subscribers", "ghost-admin/mirage/config/tags", "ghost-admin/mirage/config/themes", "ghost-admin/mirage/config/uploads", "ghost-admin/mirage/config/users", "ghost-admin/mirage/config/webhooks"], function (_exports, _apiKeys, _authentication, _config, _integrations, _members, _pages, _posts, _roles, _settings, _site, _slugs, _subscribers, _tags, _themes, _uploads, _users, _webhooks) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;
  _exports.testConfig = testConfig;

  // import {versionMismatchResponse} from 'utils';
  function _default() {
    // allow any local requests outside of the namespace (configured below) to hit the real server
    // _must_ be called before the namespace property is set
    this.passthrough('/ghost/assets/**'); // this.urlPrefix = '';    // make this `http://localhost:8080`, for example, if your API is on a different server

    this.namespace = '/ghost/api/v2/admin'; // make this `api`, for example, if your API is namespaced

    this.timing = 1000; // delay for each request, automatically set to 0 during testing

    this.logging = true; // Mock endpoints here to override real API requests during development, eg...
    // this.put('/posts/:id/', versionMismatchResponse);
    // mockTags(this);
    // this.loadFixtures('settings');
    //mockMembers(this);
    // keep this line, it allows all other API requests to hit the real server

    this.passthrough(); // add any external domains to make sure those get passed through too

    this.passthrough('https://count.ghost.org/');
    this.passthrough('http://www.gravatar.com/**');
    this.passthrough('https://cdn.jsdelivr.net/**');
    this.passthrough('https://api.unsplash.com/**');
  } // Mock all endpoints here as there is no real API during testing


  function testConfig() {
    // this.urlPrefix = '';    // make this `http://localhost:8080`, for example, if your API is on a different server
    this.namespace = '/ghost/api/v2/admin'; // make this `api`, for example, if your API is namespaced
    // this.timing = 400;      // delay for each request, automatically set to 0 during testing

    this.logging = false;
    (0, _apiKeys.default)(this);
    (0, _authentication.default)(this);
    (0, _config.default)(this);
    (0, _integrations.default)(this);
    (0, _members.default)(this);
    (0, _pages.default)(this);
    (0, _posts.default)(this);
    (0, _roles.default)(this);
    (0, _settings.default)(this);
    (0, _site.default)(this);
    (0, _slugs.default)(this);
    (0, _subscribers.default)(this);
    (0, _tags.default)(this);
    (0, _themes.default)(this);
    (0, _uploads.default)(this);
    (0, _users.default)(this);
    (0, _webhooks.default)(this);
    /* Notifications -------------------------------------------------------- */

    this.get('/notifications/');
    /* Integrations - Slack Test Notification ------------------------------- */

    this.post('/slack/test', function () {
      return {};
    });
    /* External sites ------------------------------------------------------- */

    let downloadCount = 0;
    this.get('https://count.ghost.org/', function () {
      downloadCount += 1;
      return {
        count: downloadCount
      };
    });
    this.head('http://www.gravatar.com/avatar/:md5', function () {
      return '';
    }, 200);
    this.get('http://www.gravatar.com/avatar/:md5', function () {
      return '';
    }, 200);
  }
});
;define("ghost-admin/mirage/config/api-keys", ["exports", "ghost-admin/mirage/utils"], function (_exports, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockApiKeys;

  function mockApiKeys(server) {
    server.get('/api-keys/', (0, _utils.paginatedResponse)('api-keys'));
    server.post('/api-keys/');
    server.put('/api-keys/:id/');
    server.del('/api-keys/:id/');
  }
});
;define("ghost-admin/mirage/config/authentication", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockAuthentication;

  /* eslint-disable camelcase */
  function mockAuthentication(server) {
    server.post('/session', function () {
      // Password sign-in
      return new _emberCliMirage.Response(201);
    });
    server.post('/authentication/passwordreset', function (schema, request) {
      let {
        passwordreset
      } = JSON.parse(request.requestBody); // eslint-disable-next-line ember-suave/prefer-destructuring

      let email = passwordreset[0].email;

      if (email === 'unknown@example.com') {
        return new _emberCliMirage.Response(404, {}, {
          errors: [{
            message: 'There is no user with that email address.',
            type: 'NotFoundError'
          }]
        });
      } else {
        return {
          passwordreset: [{
            message: 'Check your email for further instructions.'
          }]
        };
      }
    });
    /* Setup ---------------------------------------------------------------- */

    server.post('/authentication/setup', function (_ref, request) {
      let {
        roles,
        users
      } = _ref;
      let attrs = JSON.parse(request.requestBody).setup;
      let role = roles.findBy({
        name: 'Owner'
      }); // create owner role unless already exists

      if (!role) {
        role = roles.create({
          name: 'Owner'
        });
      }

      attrs.roles = [role];

      if (!Ember.isBlank(attrs.email)) {
        attrs.slug = attrs.email.split('@')[0].dasherize();
      } // NOTE: server does not use the user factory to fill in blank fields


      return users.create(attrs);
    });
    server.get('/authentication/setup/', function () {
      return {
        setup: [{
          status: true
        }]
      };
    });
  }
});
;define("ghost-admin/mirage/config/config", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockConfig;

  function mockConfig(server) {
    server.get('/config/', function (_ref, request) {
      let {
        db
      } = _ref;

      if (!request.requestHeaders.Authorization) {
        return new _emberCliMirage.Response(403, {}, {
          errors: [{
            type: 'NoPermissionError',
            message: 'Authorization failed',
            context: 'Unable to determine the authenticated user or integration. Check that cookies are being passed through if using session authentication.'
          }]
        });
      }

      if (Ember.isEmpty(db.configs)) {
        server.loadFixtures('configs');
      }

      return {
        config: db.configs.find(1)
      };
    });
  }
});
;define("ghost-admin/mirage/config/integrations", ["exports", "moment", "ember-cli-mirage", "ghost-admin/mirage/utils"], function (_exports, _moment, _emberCliMirage, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockIntegrations;

  function mockIntegrations(server) {
    server.get('/integrations/', (0, _utils.paginatedResponse)('integrations'));
    server.post('/integrations/', function (_ref, _ref2) {
      let {
        integrations
      } = _ref;
      let {
        requestBody
      } = _ref2;
      let body = JSON.parse(requestBody);
      let [params] = body.integrations; // all integrations created via the API have a type of 'custom'

      params.type = 'custom';

      if (!params.name) {
        return new _emberCliMirage.Response(422, {}, {
          errors: [{
            type: 'ValidationError',
            message: 'Name is required',
            property: 'name'
          }]
        });
      }

      if (integrations.findBy({
        name: params.name
      }) || params.name.match(/Duplicate/i)) {
        return new _emberCliMirage.Response(422, {}, {
          errors: [{
            type: 'ValidationError',
            message: 'Name has already been used',
            property: 'name'
          }]
        });
      } // allow factory to create defaults


      if (!params.slug) {
        delete params.slug;
      } // use factory creation to auto-create api keys


      return server.create('integration', params);
    });
    server.put('/integrations/:id/', function (schema, _ref3) {
      let {
        params
      } = _ref3;
      let {
        integrations,
        apiKeys,
        webhooks
      } = schema;
      let attrs = this.normalizedRequestAttrs();
      let integration = integrations.find(params.id);
      let _apiKeys = [];
      let _webhooks = []; // this is required to work around an issue with ember-cli-mirage and
      // embedded records. The `attrs` object will contain POJOs of the
      // embedded apiKeys and webhooks but mirage expects schema model
      // objects for relations so we need to fetch model records and replace
      // the relationship keys

      attrs.apiKeys.forEach(apiKey => {
        _apiKeys.push(apiKeys.find(apiKey.id));
      });
      attrs.webhooks.forEach(webhook => {
        _webhooks.push(webhooks.find(webhook.id));
      });
      attrs.apiKeys = _apiKeys;
      attrs.webhooks = _webhooks;
      attrs.updatedAt = _moment.default.utc().format();
      return integration.update(attrs);
    });
    server.del('/integrations/:id/');
  }
});
;define("ghost-admin/mirage/config/members", ["exports", "ghost-admin/mirage/utils"], function (_exports, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockMembers;

  function mockMembers(server) {
    server.get('/members/', (0, _utils.paginatedResponse)('members'));
    server.get('/members/:id/', function (_ref, _ref2) {
      let {
        members
      } = _ref;
      let {
        params
      } = _ref2;
      let {
        id
      } = params;
      let member = members.find(id);
      return member || new Response(404, {}, {
        errors: [{
          type: 'NotFoundError',
          message: 'Member not found.'
        }]
      });
    });
    server.del('/members/:id/');
  }
});
;define("ghost-admin/mirage/config/pages", ["exports", "moment", "ember-cli-mirage", "ghost-admin/mirage/utils"], function (_exports, _moment, _emberCliMirage, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockPages;

  function normalizeBooleanParams(arr) {
    if (!Ember.isArray(arr)) {
      return arr;
    }

    return arr.map(i => {
      if (i === 'true') {
        return true;
      } else if (i === 'false') {
        return false;
      } else {
        return i;
      }
    });
  } // TODO: use GQL to parse filter string?


  function extractFilterParam(param, filter) {
    let filterRegex = new RegExp("".concat(param, ":(.*?)(?:\\+|$)"));
    let match;
    let [, result] = filter.match(filterRegex) || [];

    if (result && result.startsWith('[')) {
      match = result.replace(/^\[|\]$/g, '').split(',');
    } else if (result) {
      match = [result];
    }

    return normalizeBooleanParams(match);
  } // NOTE: mirage requires Model objects when saving relationships, however the
  // `attrs` on POST/PUT requests will contain POJOs for authors and tags so we
  // need to replace them


  function extractAuthors(pageAttrs, users) {
    return pageAttrs.authors.map(author => users.find(author.id));
  }

  function extractTags(pageAttrs, tags) {
    return pageAttrs.tags.map(requestTag => {
      let tag = tags.find(requestTag.id);

      if (!tag) {
        tag = tag.create(requestTag);
      }

      return tag;
    });
  }

  function mockPages(server) {
    server.post('/pages', function (_ref) {
      let {
        pages,
        users,
        tags
      } = _ref;
      let attrs = this.normalizedRequestAttrs();
      attrs.authors = extractAuthors(attrs, users);
      attrs.tags = extractTags(attrs, tags);

      if (Ember.isBlank(attrs.slug) && !Ember.isBlank(attrs.title)) {
        attrs.slug = Ember.String.dasherize(attrs.title);
      }

      return pages.create(attrs);
    }); // TODO: handle authors filter

    server.get('/pages/', function (_ref2, _ref3) {
      let {
        pages
      } = _ref2;
      let {
        queryParams
      } = _ref3;
      let {
        filter,
        page,
        limit
      } = queryParams;
      page = +page || 1;
      limit = +limit || 15;
      let statusFilter = extractFilterParam('status', filter);
      let collection = pages.all().filter(page => {
        let matchesStatus = true;

        if (!Ember.isEmpty(statusFilter)) {
          matchesStatus = statusFilter.includes(page.status);
        }

        return matchesStatus;
      });
      return (0, _utils.paginateModelCollection)('pages', collection, page, limit);
    });
    server.get('/pages/:id/', function (_ref4, _ref5) {
      let {
        pages
      } = _ref4;
      let {
        params
      } = _ref5;
      let {
        id
      } = params;
      let page = pages.find(id);
      return page || new _emberCliMirage.Response(404, {}, {
        errors: [{
          type: 'NotFoundError',
          message: 'Page not found.'
        }]
      });
    });
    server.put('/pages/:id/', function (_ref6, _ref7) {
      let {
        pages,
        users,
        tags
      } = _ref6;
      let {
        params
      } = _ref7;
      let attrs = this.normalizedRequestAttrs();
      let page = pages.find(params.id);
      attrs.authors = extractAuthors(attrs, users);
      attrs.tags = extractTags(attrs, tags);
      attrs.updatedAt = _moment.default.utc().toDate();
      return page.update(attrs);
    });
    server.del('/pages/:id/');
  }
});
;define("ghost-admin/mirage/config/posts", ["exports", "moment", "ember-cli-mirage", "ghost-admin/mirage/utils"], function (_exports, _moment, _emberCliMirage, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockPosts;

  function normalizeBooleanParams(arr) {
    if (!Ember.isArray(arr)) {
      return arr;
    }

    return arr.map(i => {
      if (i === 'true') {
        return true;
      } else if (i === 'false') {
        return false;
      } else {
        return i;
      }
    });
  } // TODO: use GQL to parse filter string?


  function extractFilterParam(param, filter) {
    let filterRegex = new RegExp("".concat(param, ":(.*?)(?:\\+|$)"));
    let match;
    let [, result] = filter.match(filterRegex) || [];

    if (result.startsWith('[')) {
      match = result.replace(/^\[|\]$/g, '').split(',');
    } else if (result) {
      match = [result];
    }

    return normalizeBooleanParams(match);
  } // NOTE: mirage requires Model objects when saving relationships, however the
  // `attrs` on POST/PUT requests will contain POJOs for authors and tags so we
  // need to replace them


  function extractAuthors(postAttrs, users) {
    return postAttrs.authors.map(author => users.find(author.id));
  }

  function extractTags(postAttrs, tags) {
    return postAttrs.tags.map(requestTag => {
      let tag = tags.find(requestTag.id);

      if (!tag) {
        tag = tag.create(requestTag);
      }

      return tag;
    });
  }

  function mockPosts(server) {
    server.post('/posts', function (_ref) {
      let {
        posts,
        users,
        tags
      } = _ref;
      let attrs = this.normalizedRequestAttrs();
      attrs.authors = extractAuthors(attrs, users);
      attrs.tags = extractTags(attrs, tags);

      if (Ember.isBlank(attrs.slug) && !Ember.isBlank(attrs.title)) {
        attrs.slug = Ember.String.dasherize(attrs.title);
      }

      return posts.create(attrs);
    }); // TODO: handle authors filter

    server.get('/posts/', function (_ref2, _ref3) {
      let {
        posts
      } = _ref2;
      let {
        queryParams
      } = _ref3;
      let {
        filter,
        page,
        limit
      } = queryParams;
      page = +page || 1;
      limit = +limit || 15;
      let statusFilter = extractFilterParam('status', filter);
      let collection = posts.all().filter(post => {
        let matchesStatus = true;

        if (!Ember.isEmpty(statusFilter)) {
          matchesStatus = statusFilter.includes(post.status);
        }

        return matchesStatus;
      });
      return (0, _utils.paginateModelCollection)('posts', collection, page, limit);
    });
    server.get('/posts/:id/', function (_ref4, _ref5) {
      let {
        posts
      } = _ref4;
      let {
        params
      } = _ref5;
      let {
        id
      } = params;
      let post = posts.find(id);
      return post || new _emberCliMirage.Response(404, {}, {
        errors: [{
          type: 'NotFoundError',
          message: 'Post not found.'
        }]
      });
    });
    server.put('/posts/:id/', function (_ref6, _ref7) {
      let {
        posts,
        users,
        tags
      } = _ref6;
      let {
        params
      } = _ref7;
      let attrs = this.normalizedRequestAttrs();
      let post = posts.find(params.id);
      attrs.authors = extractAuthors(attrs, users);
      attrs.tags = extractTags(attrs, tags);
      attrs.updatedAt = _moment.default.utc().toDate();
      return post.update(attrs);
    });
    server.del('/posts/:id/');
  }
});
;define("ghost-admin/mirage/config/roles", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockRoles;

  function mockRoles(server) {
    server.get('/roles/', function (_ref, _ref2) {
      let {
        roles
      } = _ref;
      let {
        queryParams
      } = _ref2;

      if (queryParams.permissions === 'assign') {
        return roles.find([1, 2, 3]);
      }

      return roles.all();
    });
  }
});
;define("ghost-admin/mirage/config/settings", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockSettings;

  function mockSettings(server) {
    server.get('/settings/', function (_ref, _ref2) {
      let {
        db
      } = _ref;
      let {
        queryParams
      } = _ref2;
      let {
        type
      } = queryParams;
      let filters = type.split(',');
      let settings = [];

      if (!db.settings.length) {
        server.loadFixtures('settings');
      }

      filters.forEach(type => {
        settings.pushObjects(db.settings.where({
          type
        }));
      });
      return {
        settings,
        meta: {
          filters: {
            type
          }
        }
      };
    });
    server.put('/settings/', function (_ref3, _ref4) {
      let {
        db
      } = _ref3;
      let {
        requestBody
      } = _ref4;
      let newSettings = JSON.parse(requestBody).settings;
      newSettings.forEach(newSetting => {
        let {
          key
        } = newSetting;

        if (db.settings.where({
          key
        }).length > 0) {
          db.settings.update({
            key
          }, newSetting);
        } else {
          newSetting.type = newSetting.type || 'blog';
          db.settings.insert(newSetting);
        }
      });
      return {
        meta: {},
        settings: db.settings
      };
    });
  }
});
;define("ghost-admin/mirage/config/site", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockSite;

  function mockSite(server) {
    server.get('/site/', function (_ref) {
      let {
        db
      } = _ref;

      if (Ember.isEmpty(db.sites)) {
        server.loadFixtures('sites');
      }

      return {
        site: db.sites.find(1)
      };
    });
  }
});
;define("ghost-admin/mirage/config/slugs", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockSlugs;

  function mockSlugs(server) {
    server.get('/slugs/post/:slug/', function (schema, request) {
      return {
        slugs: [{
          slug: Ember.String.dasherize(decodeURIComponent(request.params.slug))
        }]
      };
    });
    server.get('/slugs/user/:slug/', function (schema, request) {
      return {
        slugs: [{
          slug: Ember.String.dasherize(decodeURIComponent(request.params.slug))
        }]
      };
    });
  }
});
;define("ghost-admin/mirage/config/subscribers", ["exports", "ember-cli-mirage", "ghost-admin/mirage/utils"], function (_exports, _emberCliMirage, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockSubscribers;

  /* eslint-disable camelcase */
  function mockSubscribers(server) {
    server.get('/subscribers/', (0, _utils.paginatedResponse)('subscribers'));
    server.post('/subscribers/', function (_ref) {
      let {
        subscribers
      } = _ref;
      let attrs = this.normalizedRequestAttrs();
      let subscriber = subscribers.findBy({
        email: attrs.email
      });

      if (subscriber) {
        return new _emberCliMirage.Response(422, {}, {
          errors: [{
            type: 'ValidationError',
            message: 'Validation error, cannot save subscriber.',
            context: 'Email address is already subscribed.',
            property: null
          }]
        });
      } else {
        attrs.createdAt = new Date();
        attrs.createdBy = 0;
        return subscribers.create(attrs);
      }
    });
    server.put('/subscribers/:id/');
    server.post('/subscribers/csv/', function () {
      // NB: we get a raw FormData object with no way to inspect it in Chrome
      // until version 50 adds the additional read methods
      // https://developer.mozilla.org/en-US/docs/Web/API/FormData#Browser_compatibility
      server.createList('subscriber', 50);
      return {
        meta: {
          stats: {
            imported: 50,
            duplicates: 3,
            invalid: 2
          }
        }
      };
    });
    server.del('/subscribers/:id/');
  }
});
;define("ghost-admin/mirage/config/tags", ["exports", "ghost-admin/mirage/utils"], function (_exports, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockTags;

  function mockTags(server) {
    server.post('/tags/', function (_ref) {
      let {
        tags
      } = _ref;
      let attrs = this.normalizedRequestAttrs();

      if (Ember.isBlank(attrs.slug) && !Ember.isBlank(attrs.name)) {
        attrs.slug = Ember.String.dasherize(attrs.name);
      } // NOTE: this does not use the tag factory to fill in blank fields


      return tags.create(attrs);
    });
    server.get('/tags/', (0, _utils.paginatedResponse)('tags'));
    server.get('/tags/slug/:slug/', function (_ref2, _ref3) {
      let {
        tags
      } = _ref2;
      let {
        params: {
          slug
        }
      } = _ref3;
      // TODO: remove post_count unless requested?
      return tags.findBy({
        slug
      });
    });
    server.put('/tags/:id/');
    server.del('/tags/:id/');
  }
});
;define("ghost-admin/mirage/config/themes", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockThemes;
  let themeCount = 1;

  function mockThemes(server) {
    server.get('/themes');
    server.post('/themes/upload/', function (_ref) {
      let {
        themes
      } = _ref;
      // pretender/mirage doesn't currently process FormData so we can't use
      // any info passed in through the request
      let theme = {
        name: "test-".concat(themeCount),
        package: {
          name: "Test ".concat(themeCount),
          version: '0.1'
        }
      };
      themeCount += 1;
      theme = themes.create(theme);
      return {
        themes: [theme]
      };
    });
    server.del('/themes/:theme/', function (_ref2, _ref3) {
      let {
        themes
      } = _ref2;
      let {
        params
      } = _ref3;
      themes.findBy({
        name: params.theme
      }).destroy();
      return new _emberCliMirage.Response(204);
    });
    server.put('/themes/:theme/activate/', function (_ref4, _ref5) {
      let {
        themes
      } = _ref4;
      let {
        params
      } = _ref5;
      themes.all().update('active', false);
      let theme = themes.findBy({
        name: params.theme
      }).update({
        active: true
      });
      return {
        themes: [theme]
      };
    });
  }
});
;define("ghost-admin/mirage/config/uploads", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockUploads;

  const fileUploadResponse = function fileUploadResponse(db, _ref) {
    let {
      requestBody
    } = _ref;
    // let [ref] = requestBody.getAll('ref');
    let [purpose] = requestBody.getAll('purpose');
    let [file] = requestBody.getAll('file');
    let now = new Date();
    let year = now.getFullYear();
    let month = "".concat(now.getMonth());

    if (month.length === 1) {
      month = "0".concat(month);
    }

    if (['image', 'profile_image', 'icon'].includes(purpose)) {
      return {
        images: [{
          url: "/content/images/".concat(year, "/").concat(month, "/").concat(file.name)
        }]
      };
    }
  };

  function mockUploads(server) {
    server.post('/images/upload/', fileUploadResponse, 200, {
      timing: 100
    });
  }
});
;define("ghost-admin/mirage/config/users", ["exports", "ember-cli-mirage", "ghost-admin/mirage/utils"], function (_exports, _emberCliMirage, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockUsers;

  function mockUsers(server) {
    // /users/me = Always return the user with ID=1
    server.get('/users/me/', function (_ref) {
      let {
        users
      } = _ref;
      let user = users.find(1);

      if (user) {
        return user;
      } else {
        return new _emberCliMirage.Response(404, {}, {
          errors: [{
            message: 'Not found',
            type: 'NotFoundError'
          }]
        });
      }
    });
    server.get('/users/', function (_ref2, _ref3) {
      let {
        users
      } = _ref2;
      let {
        queryParams
      } = _ref3;
      let page = +queryParams.page || 1; // NOTE: this is naive and only set up to work with queries that are
      // actually used - if you use a different filter in the app, add it here!

      let collection = users.where(function (user) {
        let statusMatch = true;

        if (queryParams.filter === 'status:-inactive') {
          statusMatch = user.status !== 'inactive';
        } else if (queryParams.filter === 'status:inactive') {
          statusMatch = user.status === 'inactive';
        } else if (queryParams.status && queryParams.status !== 'all') {
          statusMatch = user.status === queryParams.status;
        }

        return statusMatch;
      });
      return (0, _utils.paginateModelCollection)('users', collection, page, queryParams.limit);
    });
    server.get('/users/slug/:slug/', function (_ref4, _ref5) {
      let {
        users
      } = _ref4;
      let {
        params,
        queryParams
      } = _ref5;
      let user = users.findBy({
        slug: params.slug
      });
      user.postCount = queryParams.include.match(/count\.posts/);
      return user;
    });
    server.get('/users/:id', function (_ref6, _ref7) {
      let {
        users
      } = _ref6;
      let {
        params,
        queryParams
      } = _ref7;
      let user = users.find(params.id);
      user.postCount = queryParams.include.match(/count\.posts/);
      return user;
    });
    server.put('/users/:id/', function (_ref8, _ref9) {
      let {
        users
      } = _ref8;
      let {
        params
      } = _ref9;
      let {
        id
      } = params;

      if (id === 'password') {
        return {
          password: [{
            message: 'Password changed successfully.'
          }]
        };
      } else {
        let attrs = this.normalizedRequestAttrs(); // TODO: why is our custom serializer causing .update to throw
        // children.update is not a function?
        // https://github.com/samselikoff/ember-cli-mirage/issues/964

        delete attrs.roles;
        return users.find(id).update(attrs);
      }
    });
    server.del('/users/:id/');
  }
});
;define("ghost-admin/mirage/config/webhooks", ["exports", "ember-cli-mirage", "ghost-admin/mirage/utils"], function (_exports, _emberCliMirage, _utils) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = mockWebhooks;

  function mockWebhooks(server) {
    server.get('/webhooks/', (0, _utils.paginatedResponse)('webhooks'));
    server.post('/webhooks/', function (_ref) {
      let {
        webhooks
      } = _ref;
      let attrs = this.normalizedRequestAttrs();
      let errors = [];

      if (!attrs.name) {
        errors.push({
          type: 'ValidationError',
          message: 'Name is required',
          property: 'name'
        });
      }

      if (!attrs.event) {
        errors.push({
          type: 'ValidationError',
          message: 'Event is required',
          property: 'event'
        });
      }

      if (!attrs.targetUrl) {
        errors.push({
          type: 'ValidationError',
          message: 'Target URL is required',
          property: 'target_url'
        });
      }

      if (attrs.name && (webhooks.findBy({
        name: attrs.name,
        integrationId: attrs.integrationId
      }) || attrs.name.match(/Duplicate/i))) {
        errors.push({
          type: 'ValidationError',
          message: 'Name has already been used',
          property: 'name'
        });
      } // TODO: check server-side validation


      if (webhooks.findBy({
        targetUrl: attrs.targetUrl,
        event: attrs.event
      })) {
        errors.push({
          type: 'ValidationError',
          message: 'Target URL has already been used for this event',
          property: 'target_url'
        });
      }

      if (!Ember.isEmpty(errors)) {
        return new _emberCliMirage.Response(422, {}, {
          errors
        });
      }

      return webhooks.create(attrs);
    });
    server.put('/webhooks/:id/');
    server.del('/webhooks/:id/');
  }
});
;define("ghost-admin/mirage/factories/api-key", ["exports", "moment", "ember-cli-mirage"], function (_exports, _moment, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    type: 'content',

    secret() {
      if (this.integration) {
        return "".concat(this.integration.slug, "_").concat(this.type, "_key-12345");
      }

      return "".concat(this.type, "_key-12345");
    },

    lastSeenAt() {
      return _moment.default.utc().format();
    },

    createdAt() {
      return _moment.default.utc().format();
    },

    createdBy: 1,

    updatedAt() {
      return _moment.default.utc().format();
    },

    updatedBy: 1
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/integration", ["exports", "moment", "ember-cli-mirage"], function (_exports, _moment, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    name(i) {
      return "Integration ".concat(i + 1);
    },

    slug() {
      return this.name.toLowerCase().replace(' ', '-');
    },

    description: null,
    iconImage: null,
    type: 'custom',

    createdAt() {
      return _moment.default.utc().format();
    },

    createdBy: 1,

    updatedAt() {
      return _moment.default.utc().format();
    },

    updatedBy: 1,

    afterCreate(integration, server) {
      let contentKey = server.create('api-key', {
        type: 'content',
        integration
      });
      let adminKey = server.create('api-key', {
        type: 'admin',
        integration
      });
      integration.apiKeyIds = [contentKey.id, adminKey.id];
      integration.save();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/invite", ["exports", "moment", "ember-cli-mirage"], function (_exports, _moment, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    token(i) {
      return "".concat(i, "-token");
    },

    email(i) {
      return "invited-user-".concat(i, "@example.com");
    },

    expires() {
      return _moment.default.utc().add(1, 'day').valueOf();
    },

    createdAt() {
      return _moment.default.utc().format();
    },

    createdBy() {
      return 1;
    },

    updatedAt() {
      return _moment.default.utc().format();
    },

    updatedBy() {
      return 1;
    },

    status() {
      return 'sent';
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/member", ["exports", "faker", "moment", "ember-cli-mirage"], function (_exports, _faker, _moment, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  let randomDate = function randomDate() {
    let start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _moment.default)().subtract(30, 'days').toDate();
    let end = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new Date();
    return new Date(start.getTime() + Math.random() * (end.getTime() - start.getTime()));
  };

  var _default = _emberCliMirage.Factory.extend({
    name() {
      return "".concat(_faker.default.name.firstName(), " ").concat(_faker.default.name.lastName());
    },

    email: _faker.default.internet.email,

    createdAt() {
      return randomDate();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/notification", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    dismissible: true,
    message: 'This is an alert',
    status: 'alert',
    type: 'error'
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/post", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    codeinjectionFoot: null,
    codeinjectionHead: null,
    createdAt: '2015-09-11T09:44:29.871Z',
    createdBy: 1,
    customExcerpt: null,
    customTemplate: null,

    description(i) {
      return "Title for post ".concat(i, ".");
    },

    featured: false,

    featureImage(i) {
      return "/content/images/2015/10/post-".concat(i, ".jpg");
    },

    html(i) {
      return "<p>HTML for post ".concat(i, ".</p>");
    },

    locale: null,

    metaDescription(i) {
      return "Meta description for post ".concat(i, ".");
    },

    metaTitle(i) {
      return "Meta Title for post ".concat(i);
    },

    ogDescription: null,
    ogImage: null,
    ogTitle: null,

    excerpt(i) {
      return "Excerpt for post ".concat(i, ".");
    },

    plaintext(i) {
      return "Plaintext for post ".concat(i, ".");
    },

    publishedAt: '2015-12-19T16:25:07.000Z',
    publishedBy: 1,

    status(i) {
      let statuses = ['draft', 'published', 'scheduled'];
      return statuses[i % statuses.length];
    },

    title(i) {
      return "Post ".concat(i);
    },

    twitterDescription: null,
    twitterImage: null,
    twitterTitle: null,
    updatedAt: '2015-10-19T16:25:07.756Z',
    updatedBy: 1,

    uuid(i) {
      return "post-".concat(i);
    },

    authors() {
      return [];
    },

    tags() {
      return [];
    },

    afterCreate(post, server) {
      if (Ember.isEmpty(post.authors)) {
        let user = server.schema.users.find(1);

        if (!user) {
          let role = server.schema.roles.find({
            name: 'Administrator'
          }) || server.create('role', {
            name: 'Administrator'
          });
          user = server.create('user', {
            roles: [role]
          });
        }

        post.authors = [user];
        post.save();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/role", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    createdAt: '2013-11-25T14:48:11.000Z',
    createdBy: 1,

    description(i) {
      return "Role ".concat(i);
    },

    name: '',
    updatedAt: '2013-11-25T14:48:11.000Z',
    updatedBy: 1
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/subscriber", ["exports", "faker", "moment", "ember-cli-mirage"], function (_exports, _faker, _moment, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  let randomDate = function randomDate() {
    let start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _moment.default)().subtract(30, 'days').toDate();
    let end = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new Date();
    return new Date(start.getTime() + Math.random() * (end.getTime() - start.getTime()));
  };

  let statuses = ['pending', 'subscribed'];

  var _default = _emberCliMirage.Factory.extend({
    name() {
      return "".concat(_faker.default.name.firstName(), " ").concat(_faker.default.name.lastName());
    },

    email: _faker.default.internet.email,

    status() {
      return statuses[Math.floor(Math.random() * statuses.length)];
    },

    createdAt() {
      return randomDate();
    },

    updatedAt: null,
    createdBy: 0,
    updatedBy: null,
    unsubscribedAt: null
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/tag", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    createdAt: '2015-09-11T09:44:29.871Z',
    createdBy: 1,

    description(i) {
      return "Description for tag ".concat(i, ".");
    },

    visibility: 'public',

    featureImage(i) {
      return "/content/images/2015/10/tag-".concat(i, ".jpg");
    },

    metaDescription(i) {
      return "Meta description for tag ".concat(i, ".");
    },

    metaTitle(i) {
      return "Meta Title for tag ".concat(i);
    },

    name(i) {
      return "Tag ".concat(i);
    },

    parent: null,

    slug(i) {
      return "tag-".concat(i);
    },

    updatedAt: '2015-10-19T16:25:07.756Z',
    updatedBy: 1,

    count() {
      // this gets updated automatically by the tag serializer
      return {
        posts: 0
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/user", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    accessibility: null,
    bio: null,
    coverImage: null,
    createdAt: '2015-09-02T13:41:50.000Z',
    createdBy: null,

    email(i) {
      return "user-".concat(i, "@example.com");
    },

    profileImage: '//www.gravatar.com/avatar/3ae045bc198a157401827c8455cd7c99?s=250&d=mm&r=x',
    lastLogin: '2015-11-02T16:12:05.000Z',
    location: null,
    metaDescription: null,
    metaTitle: null,

    name(i) {
      return "User ".concat(i);
    },

    slug(i) {
      return "user-".concat(i);
    },

    status: 'active',
    tour: null,
    updatedAt: '2015-11-02T16:12:05.000Z',
    updatedBy: '1',
    website: 'http://example.com',

    posts() {
      return [];
    },

    roles() {
      return [];
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/factories/webhook", ["exports", "moment", "ghost-admin/helpers/event-name", "ember-cli-mirage"], function (_exports, _moment, _eventName, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Factory.extend({
    name(i) {
      return "Integration ".concat(i + 1);
    },

    event(i) {
      let event = _eventName.AVAILABLE_EVENTS[i % 3];
      return event.event;
    },

    target(i) {
      return "https://example.com/".concat(i + 1);
    },

    lastTriggeredAt: null,

    createdAt() {
      return _moment.default.utc().format();
    },

    createdBy: 1,

    updatedAt() {
      return _moment.default.utc().format();
    },

    updatedBy: 1
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/fixtures/configs", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = [{
    clientExtensions: {},
    database: 'mysql',
    enableDeveloperExperiments: true,
    environment: 'development',
    labs: {
      publicAPI: true,
      subscribers: false
    },
    mail: 'SMTP',
    version: '2.15.0',
    useGravatar: 'true'
  }];
  _exports.default = _default;
});
;define("ghost-admin/mirage/fixtures/roles", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = [{
    id: 1,
    name: 'Administrator',
    description: 'Administrators',
    created_at: '2015-11-13T16:01:29.131Z',
    created_by: 1,
    updated_at: '2015-11-13T16:01:29.131Z',
    updated_by: 1
  }, {
    id: 2,
    name: 'Editor',
    description: 'Editors',
    created_at: '2015-11-13T16:01:29.131Z',
    created_by: 1,
    updated_at: '2015-11-13T16:01:29.131Z',
    updated_by: 1
  }, {
    id: 3,
    name: 'Author',
    description: 'Authors',
    created_at: '2015-11-13T16:01:29.131Z',
    created_by: 1,
    updated_at: '2015-11-13T16:01:29.131Z',
    updated_by: 1
  }, {
    id: 4,
    name: 'Owner',
    description: 'Blog Owner',
    created_at: '2015-11-13T16:01:29.132Z',
    created_by: 1,
    updated_at: '2015-11-13T16:01:29.132Z',
    updated_by: 1
  }, {
    id: 5,
    name: 'Contributor',
    description: 'Contributors',
    created_at: '2015-11-13T16:01:29.132Z',
    created_by: 1,
    updated_at: '2015-11-13T16:01:29.132Z',
    updated_by: 1
  }];
  _exports.default = _default;
});
;define("ghost-admin/mirage/fixtures/settings", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = [{
    id: 1,
    created_at: '2015-09-11T09:44:30.805Z',
    created_by: 1,
    key: 'title',
    type: 'blog',
    updated_at: '2015-10-04T16:26:05.195Z',
    updated_by: 1,
    value: 'Test Blog'
  }, {
    id: 2,
    created_at: '2015-09-11T09:44:30.806Z',
    created_by: 1,
    key: 'description',
    type: 'blog',
    updated_at: '2015-10-04T16:26:05.198Z',
    updated_by: 1,
    value: 'Thoughts, stories and ideas.'
  }, {
    id: 3,
    key: 'logo',
    value: '/content/images/2013/Nov/logo.png',
    type: 'blog',
    created_at: '2013-11-25T14:48:11.000Z',
    created_by: 1,
    updated_at: '2015-10-27T17:39:58.273Z',
    updated_by: 1
  }, {
    id: 4,
    key: 'cover_image',
    value: '/content/images/2014/Feb/cover.jpg',
    type: 'blog',
    created_at: '2013-11-25T14:48:11.000Z',
    created_by: 1,
    updated_at: '2015-10-27T17:39:58.276Z',
    updated_by: 1
  }, {
    id: 5,
    key: 'default_lang',
    value: 'en_US',
    type: 'blog',
    created_at: '2013-11-25T14:48:11.000Z',
    created_by: 1,
    updated_at: '2015-10-27T17:39:58.278Z',
    updated_by: 1
  }, {
    id: 10,
    created_at: '2015-09-11T09:44:30.809Z',
    created_by: 1,
    key: 'codeinjection_head',
    type: 'blog',
    updated_at: '2015-09-23T13:32:49.858Z',
    updated_by: 1,
    value: ''
  }, {
    id: 11,
    created_at: '2015-09-11T09:44:30.809Z',
    created_by: 1,
    key: 'codeinjection_foot',
    type: 'blog',
    updated_at: '2015-09-23T13:32:49.858Z',
    updated_by: 1,
    value: ''
  }, {
    id: 12,
    key: 'labs',
    value: '{"subscribers":true}',
    type: 'blog',
    created_at: '2015-01-12T18:29:01.000Z',
    created_by: 1,
    updated_at: '2015-10-27T17:39:58.288Z',
    updated_by: 1
  }, {
    id: 13,
    created_at: '2015-09-11T09:44:30.810Z',
    created_by: 1,
    key: 'navigation',
    type: 'blog',
    updated_at: '2015-09-23T13:32:49.868Z',
    updated_by: 1,
    value: JSON.stringify([{
      label: 'Home',
      url: '/'
    }, {
      label: 'About',
      url: '/about'
    }])
  }, {
    id: 14,
    created_at: '2015-09-11T09:44:30.810Z',
    created_by: 1,
    key: 'is_private',
    type: 'blog',
    updated_at: '2015-09-23T13:32:49.868Z',
    updated_by: 1,
    value: false
  }, {
    id: 15,
    created_at: '2015-09-11T09:44:30.810Z',
    created_by: 1,
    key: 'password',
    type: 'blog',
    updated_at: '2015-09-23T13:32:49.868Z',
    updated_by: 1,
    value: ''
  }, {
    id: 16,
    created_at: '2016-05-05T15:04:03.115Z',
    created_by: 1,
    key: 'slack',
    type: 'blog',
    updated_at: '2016-05-05T18:33:09.168Z',
    updated_by: 1,
    value: '[{"url":""}]'
  }, {
    id: 17,
    created_at: '2016-05-05T15:40:12.133Z',
    created_by: 1,
    key: 'facebook',
    type: 'blog',
    updated_at: '2016-05-08T15:20:25.953Z',
    updated_by: 1,
    value: 'test'
  }, {
    id: 18,
    created_at: '2016-05-05T15:40:12.134Z',
    created_by: 1,
    key: 'twitter',
    type: 'blog',
    updated_at: '2016-05-08T15:20:25.954Z',
    updated_by: 1,
    value: '@test'
  }, {
    id: 19,
    created_at: '2015-09-11T09:44:30.810Z',
    created_by: 1,
    key: 'active_timezone',
    type: 'blog',
    updated_at: '2015-09-23T13:32:49.868Z',
    updated_by: 1,
    value: 'Etc/UTC'
  }, {
    id: 21,
    created_at: '2017-01-09T08:40:59.000Z',
    created_by: 1,
    key: 'amp',
    type: 'blog',
    updated_at: '2017-01-09T08:49:42.991Z',
    updated_by: 1,
    value: 'true'
  }, {
    id: 22,
    key: 'icon',
    value: '/content/images/2014/Feb/favicon.ico',
    type: 'blog',
    created_at: '2013-11-25T14:48:11.000Z',
    created_by: 1,
    updated_at: '2015-10-27T17:39:58.276Z',
    updated_by: 1
  }];
  _exports.default = _default;
});
;define("ghost-admin/mirage/fixtures/sites", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = [{
    title: 'Test Blog',
    url: "".concat(window.location.origin, "/")
  }];
  _exports.default = _default;
});
;define("ghost-admin/mirage/fixtures/themes", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = [{
    name: 'casper',
    package: {
      name: 'Blog',
      version: '1.0'
    },
    active: true
  }, {
    name: 'foo',
    package: {
      name: 'Foo',
      version: '0.1'
    }
  }, {
    name: 'bar'
  }];
  _exports.default = _default;
});
;define("ghost-admin/mirage/fixtures/timezones", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = [{
    name: 'Pacific/Pago_Pago',
    label: '(GMT -11:00) Midway Island, Samoa'
  }, {
    name: 'Pacific/Honolulu',
    label: '(GMT -10:00) Hawaii'
  }, {
    name: 'America/Anchorage',
    label: '(GMT -9:00) Alaska'
  }, {
    name: 'America/Tijuana',
    label: '(GMT -8:00) Chihuahua, La Paz, Mazatlan'
  }, {
    name: 'America/Los_Angeles',
    label: '(GMT -8:00) Pacific Time (US & Canada); Tijuana'
  }, {
    name: 'America/Phoenix',
    label: '(GMT -7:00) Arizona'
  }, {
    name: 'America/Denver',
    label: '(GMT -7:00) Mountain Time (US & Canada)'
  }, {
    name: 'America/Costa_Rica',
    label: '(GMT -6:00) Central America'
  }, {
    name: 'America/Chicago',
    label: '(GMT -6:00) Central Time (US & Canada)'
  }, {
    name: 'America/Mexico_City',
    label: '(GMT -6:00) Guadalajara, Mexico City, Monterrey'
  }, {
    name: 'America/Regina',
    label: '(GMT -6:00) Saskatchewan'
  }, {
    name: 'America/Bogota',
    label: '(GMT -5:00) Bogota, Lima, Quito'
  }, {
    name: 'America/New_York',
    label: '(GMT -5:00) Eastern Time (US & Canada)'
  }, {
    name: 'America/Fort_Wayne',
    label: '(GMT -5:00) Indiana (East)'
  }, {
    name: 'America/Caracas',
    label: '(GMT -4:00) Caracas, La Paz'
  }, {
    name: 'America/Halifax',
    label: '(GMT -4:00) Atlantic Time (Canada); Brasilia, Greenland'
  }, {
    name: 'America/Santiago',
    label: '(GMT -4:00) Santiago'
  }, {
    name: 'America/St_Johns',
    label: '(GMT -3:30) Newfoundland'
  }, {
    name: 'America/Argentina/Buenos_Aires',
    label: '(GMT -3:00) Buenos Aires, Georgetown'
  }, {
    name: 'America/Noronha',
    label: '(GMT -2:00) Fernando de Noronha'
  }, {
    name: 'Atlantic/Azores',
    label: '(GMT -1:00) Azores'
  }, {
    name: 'Atlantic/Cape_Verde',
    label: '(GMT -1:00) Cape Verde Is.'
  }, {
    name: 'Etc/UTC',
    label: '(GMT) UTC'
  }, {
    name: 'Africa/Casablanca',
    label: '(GMT +0:00) Casablanca, Monrovia'
  }, {
    name: 'Europe/Dublin',
    label: '(GMT +0:00) Dublin, Edinburgh, London'
  }, {
    name: 'Europe/Amsterdam',
    label: '(GMT +1:00) Amsterdam, Berlin, Rome, Stockholm, Vienna'
  }, {
    name: 'Europe/Prague',
    label: '(GMT +1:00) Belgrade, Bratislava, Budapest, Prague'
  }, {
    name: 'Europe/Paris',
    label: '(GMT +1:00) Brussels, Copenhagen, Madrid, Paris'
  }, {
    name: 'Europe/Warsaw',
    label: '(GMT +1:00) Sarajevo, Skopje, Warsaw, Zagreb'
  }, {
    name: 'Africa/Lagos',
    label: '(GMT +1:00) West Central Africa'
  }, {
    name: 'Europe/Istanbul',
    label: '(GMT +2:00) Athens, Beirut, Bucharest, Istanbul'
  }, {
    name: 'Africa/Cairo',
    label: '(GMT +2:00) Cairo, Egypt'
  }, {
    name: 'Africa/Maputo',
    label: '(GMT +2:00) Harare'
  }, {
    name: 'Europe/Kiev',
    label: '(GMT +2:00) Helsinki, Kiev, Riga, Sofia, Tallinn, Vilnius'
  }, {
    name: 'Asia/Jerusalem',
    label: '(GMT +2:00) Jerusalem'
  }, {
    name: 'Africa/Johannesburg',
    label: '(GMT +2:00) Pretoria'
  }, {
    name: 'Asia/Baghdad',
    label: '(GMT +3:00) Baghdad'
  }, {
    name: 'Asia/Riyadh',
    label: '(GMT +3:00) Kuwait, Nairobi, Riyadh'
  }, {
    name: 'Europe/Moscow',
    label: '(GMT +3:00) Moscow, St. Petersburg, Volgograd'
  }, {
    name: 'Asia/Tehran',
    label: '(GMT +3:30) Tehran'
  }, {
    name: 'Asia/Dubai',
    label: '(GMT +4:00) Abu Dhabi, Muscat'
  }, {
    name: 'Asia/Baku',
    label: '(GMT +4:00) Baku, Tbilisi, Yerevan'
  }, {
    name: 'Asia/Kabul',
    label: '(GMT +4:30) Kabul'
  }, {
    name: 'Asia/Karachi',
    label: '(GMT +5:00) Islamabad, Karachi, Tashkent'
  }, {
    name: 'Asia/Yekaterinburg',
    label: '(GMT +5:00) Yekaterinburg'
  }, {
    name: 'Asia/Kolkata',
    label: '(GMT +5:30) Chennai, Calcutta, Mumbai, New Delhi'
  }, {
    name: 'Asia/Kathmandu',
    label: '(GMT +5:45) Katmandu'
  }, {
    name: 'Asia/Almaty',
    label: '(GMT +6:00) Almaty, Novosibirsk'
  }, {
    name: 'Asia/Dhaka',
    label: '(GMT +6:00) Astana, Dhaka, Sri Jayawardenepura'
  }, {
    name: 'Asia/Rangoon',
    label: '(GMT +6:30) Rangoon'
  }, {
    name: 'Asia/Bangkok',
    label: '(GMT +7:00) Bangkok, Hanoi, Jakarta'
  }, {
    name: 'Asia/Krasnoyarsk',
    label: '(GMT +7:00) Krasnoyarsk'
  }, {
    name: 'Asia/Hong_Kong',
    label: '(GMT +8:00) Beijing, Chongqing, Hong Kong, Urumqi'
  }, {
    name: 'Asia/Irkutsk',
    label: '(GMT +8:00) Irkutsk, Ulaan Bataar'
  }, {
    name: 'Asia/Singapore',
    label: '(GMT +8:00) Kuala Lumpur, Perth, Singapore, Taipei'
  }, {
    name: 'Asia/Tokyo',
    label: '(GMT +9:00) Osaka, Sapporo, Tokyo'
  }, {
    name: 'Asia/Seoul',
    label: '(GMT +9:00) Seoul'
  }, {
    name: 'Asia/Yakutsk',
    label: '(GMT +9:00) Yakutsk'
  }, {
    name: 'Australia/Adelaide',
    label: '(GMT +9:30) Adelaide'
  }, {
    name: 'Australia/Darwin',
    label: '(GMT +9:30) Darwin'
  }, {
    name: 'Australia/Brisbane',
    label: '(GMT +10:00) Brisbane, Guam, Port Moresby'
  }, {
    name: 'Australia/Sydney',
    label: '(GMT +10:00) Canberra, Hobart, Melbourne, Sydney, Vladivostok'
  }, {
    name: 'Asia/Magadan',
    label: '(GMT +11:00) Magadan, Soloman Is., New Caledonia'
  }, {
    name: 'Pacific/Auckland',
    label: '(GMT +12:00) Auckland, Wellington'
  }, {
    name: 'Pacific/Fiji',
    label: '(GMT +12:00) Fiji, Kamchatka, Marshall Is.'
  }, {
    name: 'Pacific/Kwajalein',
    label: '(GMT +12:00) International Date Line West'
  }];
  _exports.default = _default;
});
;define("ghost-admin/mirage/models/api-key", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    integration: (0, _emberCliMirage.belongsTo)()
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/config", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/integration", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    apiKeys: (0, _emberCliMirage.hasMany)(),
    webhooks: (0, _emberCliMirage.hasMany)()
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/invite", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    role: (0, _emberCliMirage.belongsTo)()
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/member", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/notification", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/page", ["exports", "ghost-admin/mirage/models/post"], function (_exports, _post) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _post.default.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/post", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    tags: (0, _emberCliMirage.hasMany)(),
    authors: (0, _emberCliMirage.hasMany)('user')
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/role", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/site", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/subscriber", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/tag", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    posts: (0, _emberCliMirage.hasMany)()
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/theme", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/user", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    // used by the serializer to determine whether
    // or not to include the post count
    postCount: false,
    roles: (0, _emberCliMirage.hasMany)(),
    posts: (0, _emberCliMirage.hasMany)()
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/models/webhook", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.Model.extend({
    integration: (0, _emberCliMirage.belongsTo)()
  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/scenarios/default", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  function _default(server) {
    // Seed your development database using your factories. This
    // data will not be loaded in your tests.
    // server.createList('contact', 10);
    server.createList('subscriber', 125);
    server.createList('tag', 100);
    server.create('integration', {
      name: 'Demo'
    });
    server.createList('member', 125);
  }
});
;define("ghost-admin/mirage/serializers/application", ["exports", "ember-cli-mirage", "ember-cli-mirage/utils/inflector"], function (_exports, _emberCliMirage, _inflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberCliMirage.RestSerializer.extend({
    keyForCollection(collection) {
      return Ember.String.underscore((0, _inflector.pluralize)(collection));
    },

    keyForAttribute(attr) {
      return Ember.String.underscore(attr);
    },

    keyForRelationship(relationship) {
      return Ember.String.underscore(relationship);
    },

    keyForEmbeddedRelationship(relationship) {
      return Ember.String.underscore(relationship);
    },

    serialize(object, request) {
      // Ember expects pluralized responses for the post, user, and invite models,
      // and this shortcut will ensure that those models are pluralized
      if (this.isModel(object) && ['post', 'user', 'invite'].includes(object.modelName)) {
        object = new _emberCliMirage.Collection(object.modelName, [object]);
      }

      let json = _emberCliMirage.RestSerializer.prototype.serialize.call(this, object, request);

      if (this.isCollection(object) && object.meta) {
        json.meta = object.meta;
      }

      return json;
    },

    // POST and PUT request send data in pluralized attributes for all models,
    // so we extract it here - this allows #normalizedRequestAttrs to work
    // in route functions
    normalize(body, modelName) {
      // sometimes mirage doesn't include a modelName, so we extrapolate it from
      // the first element of Object.keys
      modelName = (0, _inflector.pluralize)(modelName) || Object.keys(body)[0];
      let [attributes] = body[modelName] || [{}];
      return {
        data: {
          attributes
        }
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/serializers/integration", ["exports", "ghost-admin/mirage/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    embed: true,

    include(request) {
      if (!request.queryParams.include) {
        return;
      }

      return request.queryParams.include.split(',').map(Ember.String.camelize);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/serializers/page", ["exports", "ghost-admin/mirage/serializers/post"], function (_exports, _post) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _post.default.extend({});

  _exports.default = _default;
});
;define("ghost-admin/mirage/serializers/post", ["exports", "ghost-admin/mirage/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    embed: true,

    include()
    /*request*/
    {
      let includes = [];
      includes.push('tags');
      includes.push('authors');
      return includes;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/serializers/tag", ["exports", "ghost-admin/mirage/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    // make the tag.count.posts value dynamic
    serialize(tagModelOrCollection, request) {
      let updatePostCount = tag => {
        tag.update('count', {
          posts: tag.postIds.length
        });
      };

      if (this.isModel(tagModelOrCollection)) {
        updatePostCount(tagModelOrCollection);
      } else {
        tagModelOrCollection.models.forEach(updatePostCount);
      }

      return _application.default.prototype.serialize.call(this, tagModelOrCollection, request);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/serializers/user", ["exports", "ghost-admin/mirage/serializers/application", "ember-cli-mirage"], function (_exports, _application, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    embed: true,

    include(request) {
      if (request.queryParams.include && request.queryParams.include.indexOf('roles') >= 0) {
        return ['roles'];
      }

      return [];
    },

    serialize(object, request) {
      if (this.isCollection(object)) {
        return _application.default.prototype.serialize.call(this, object, request);
      }

      let {
        user
      } = _emberCliMirage.RestSerializer.prototype.serialize.call(this, object, request);

      if (object.postCount) {
        let posts = object.posts.models.length;
        user.count = {
          posts
        };
      }

      return {
        users: [user]
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mirage/utils", ["exports", "ember-cli-mirage"], function (_exports, _emberCliMirage) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.paginatedResponse = paginatedResponse;
  _exports.paginateModelCollection = paginateModelCollection;
  _exports.maintenanceResponse = maintenanceResponse;
  _exports.versionMismatchResponse = versionMismatchResponse;

  /* eslint-disable max-statements-per-line */
  function paginatedResponse(modelName) {
    return function (schema, request) {
      let page = +request.queryParams.page || 1;
      let limit = request.queryParams.limit;
      let collection = schema[modelName].all();

      if (limit !== 'all') {
        limit = +request.queryParams.limit || 15;
      }

      return paginateModelCollection(modelName, collection, page, limit);
    };
  }

  function paginateModelCollection(modelName, collection, page, limit) {
    let pages, next, prev, models;

    if (limit === 'all') {
      pages = 1;
    } else {
      limit = +limit;
      let start = (page - 1) * limit;
      let end = start + limit;
      pages = Math.ceil(collection.models.length / limit);
      models = collection.models.slice(start, end);

      if (start > 0) {
        prev = page - 1;
      }

      if (end < collection.models.length) {
        next = page + 1;
      }
    }

    collection.meta = {
      pagination: {
        page,
        limit,
        pages,
        total: collection.models.length,
        next: next || null,
        prev: prev || null
      }
    };

    if (models) {
      collection.models = models;
    }

    return collection;
  }

  function maintenanceResponse() {
    return new _emberCliMirage.Response(503, {}, {
      errors: [{
        type: 'Maintenance'
      }]
    });
  }

  function versionMismatchResponse() {
    return new _emberCliMirage.Response(400, {}, {
      errors: [{
        type: 'VersionMismatchError'
      }]
    });
  }
});
;define("ghost-admin/mixins/body-event-listener", ["exports", "jquery"], function (_exports, _jquery) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  function K() {
    return this;
  } // Code modified from Addepar/ember-widgets
  // https://github.com/Addepar/ember-widgets/blob/master/src/mixins.coffee#L39


  var _default = Ember.Mixin.create({
    bodyElementSelector: 'html',
    bodyClick: K,

    init() {
      this._super(...arguments);

      return Ember.run.next(this, this._setupDocumentHandlers);
    },

    willDestroy() {
      this._super(...arguments);

      return this._removeDocumentHandlers();
    },

    _setupDocumentHandlers() {
      if (this._clickHandler) {
        return;
      }

      this._clickHandler = event => this.bodyClick(event);

      return (0, _jquery.default)(this.bodyElementSelector).on('click', this._clickHandler);
    },

    _removeDocumentHandlers() {
      (0, _jquery.default)(this.bodyElementSelector).off('click', this._clickHandler);
      this._clickHandler = null;
    },

    // http://stackoverflow.com/questions/152975/how-to-detect-a-click-outside-an-element
    click(event) {
      return event.stopPropagation();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/current-user-settings", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Mixin.create({
    transitionAuthor() {
      return user => {
        if (user.get('isAuthorOrContributor')) {
          return this.transitionTo('staff.user', user);
        }

        return user;
      };
    },

    transitionEditor() {
      return user => {
        if (user.get('isEditor')) {
          return this.transitionTo('staff');
        }

        return user;
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/dropdown-mixin", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /*
    Dropdowns and their buttons are evented and do not propagate clicks.
  */
  var _default = Ember.Mixin.create(Ember.Evented, {
    classNameBindings: ['isOpen:open:closed'],
    isOpen: false,

    click(event) {
      this._super(event);

      return event.stopPropagation();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/id-url", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Mixin.create({
    buildURL(_modelName, _id, _snapshot, _requestType, query) {
      let url = this._super(...arguments);

      if (query && !Ember.isBlank(query.id)) {
        url += "".concat(query.id, "/");
        delete query.id;
      }

      return url;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/settings-menu-component", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Mixin.create({
    showSettingsMenu: false,
    isViewingSubview: Ember.computed('showSettingsMenu', {
      get() {
        return false;
      },

      set(key, value) {
        // Not viewing a subview if we can't even see the PSM
        if (!this.showSettingsMenu) {
          return false;
        }

        return value;
      }

    }),
    actions: {
      showSubview() {
        this.set('isViewingSubview', true);
      },

      closeSubview() {
        this.set('isViewingSubview', false);
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/shortcuts-route", ["exports", "ghost-admin/mixins/shortcuts"], function (_exports, _shortcuts) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * Only routes can implement shortcuts.
   * If you need to trigger actions on the controller,
   * simply call them with `this.get('controller').send('action')`.
   *
   * To implement shortcuts, add this mixin to your `extend()`,
   * and implement a `shortcuts` hash.
   * In this hash, keys are shortcut combinations and values are route action names.
   *  (see [keymaster docs](https://github.com/madrobby/keymaster/blob/master/README.markdown)),
   *
   * ```javascript
   * shortcuts: {
   *     'ctrl+s, command+s': 'save'
   * }
   * ```
   * For more complex actions, shortcuts can instead have their value
   * be an object like {action, options}
   * ```javascript
   * shortcuts: {
   *      'ctrl+k': {action: 'markdownShortcut', options: 'createLink'}
   * }
   * ```
   * You can set the scope of your shortcut by passing a scope property.
   * ```javascript
   * shortcuts : {
   *   'enter': {action : 'confirmModal', scope: 'modal'}
   * }
   * ```
   * If you don't specify a scope, we use a default scope called "default".
   * To have all your shortcut work in all scopes, give it the scope "all".
   * Find out more at the keymaster docs
   */
  var _default = Ember.Mixin.create(_shortcuts.default, {
    activate() {
      this._super(...arguments);

      this.registerShortcuts();
    },

    deactivate() {
      this._super(...arguments);

      this.removeShortcuts();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/shortcuts", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // Configure KeyMaster to respond to all shortcuts,
  // even inside of
  // input, textarea, and select.
  key.filter = function () {
    return true;
  };

  key.setScope('default');
  /**
   * Only routes can implement shortcuts.
   * If you need to trigger actions on the controller,
   * simply call them with `this.get('controller').send('action')`.
   *
   * To implement shortcuts, add this mixin to your `extend()`,
   * and implement a `shortcuts` hash.
   * In this hash, keys are shortcut combinations and values are route action names.
   *  (see [keymaster docs](https://github.com/madrobby/keymaster/blob/master/README.markdown)),
   *
   * ```javascript
   * shortcuts: {
   *     'ctrl+s, command+s': 'save'
   * }
   * ```
   * For more complex actions, shortcuts can instead have their value
   * be an object like {action, options}
   * ```javascript
   * shortcuts: {
   *      'ctrl+k': {action: 'markdownShortcut', options: 'createLink'}
   * }
   * ```
   * You can set the scope of your shortcut by passing a scope property.
   * ```javascript
   * shortcuts : {
   *   'enter': {action : 'confirmModal', scope: 'modal'}
   * }
   * ```
   * If you don't specify a scope, we use a default scope called "default".
   * To have all your shortcut work in all scopes, give it the scope "all".
   * Find out more at the keymaster docs
   */

  var _default = Ember.Mixin.create({
    registerShortcuts() {
      let shortcuts = this.shortcuts;
      Object.keys(shortcuts).forEach(shortcut => {
        let scope = shortcuts[shortcut].scope || 'default';
        let action = shortcuts[shortcut];
        let options;

        if (Ember.typeOf(action) !== 'string') {
          options = action.options;
          action = action.action;
        }

        key(shortcut, scope, event => {
          // stop things like ctrl+s from actually opening a save dialogue
          event.preventDefault();
          Ember.run(this, function () {
            this.send(action, options);
          });
        });
      });
    },

    removeShortcuts() {
      let shortcuts = this.shortcuts;
      Object.keys(shortcuts).forEach(shortcut => {
        let scope = shortcuts[shortcut].scope || 'default';
        key.unbind(shortcut, scope);
      });
    },

    willDestroy() {
      this._super(...arguments);

      this.removeShortcuts();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/slug-url", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Mixin.create({
    buildURL(_modelName, _id, _snapshot, _requestType, query) {
      let url = this._super(...arguments);

      if (query && !Ember.isBlank(query.slug)) {
        url += "slug/".concat(query.slug, "/");
        delete query.slug;
      }

      return url;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/text-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const keyCodes = {
    13: 'Enter',
    9: 'Tab'
  };

  var _default = Ember.Mixin.create({
    userAgent: Ember.inject.service(),
    attributeBindings: ['autofocus'],
    selectOnClick: false,
    shouldFocus: false,
    stopEnterKeyDownPropagation: false,
    autofocus: Ember.computed(function () {
      if (this.shouldFocus) {
        return this.userAgent.os.isIOS ? false : 'autofocus';
      }

      return false;
    }),

    didInsertElement() {
      this._super(...arguments);

      this._focus();
    },

    click(event) {
      if (this.selectOnClick) {
        event.currentTarget.select();
      }
    },

    keyDown(event) {
      // stop event propagation when pressing "enter"
      // most useful in the case when undesired (global) keyboard shortcuts
      // are getting triggered while interacting with this particular input element.
      if (event.keyCode === 13 && this.stopEnterKeyDownPropagation) {
        event.stopPropagation();
        return true;
      } // prevent default TAB behaviour if we have a keyEvent for it


      if (event.keyCode === 9 && typeof this.get('keyEvents.Tab') === 'function') {
        event.preventDefault();
      }

      this._super(...arguments);
    },

    keyPress(event) {
      // prevent default ENTER behaviour if we have a keyEvent for it
      if (event.keyCode === 13 && typeof this.get('keyEvents.Enter') === 'function') {
        event.preventDefault();
      }

      this._super(...arguments);
    },

    keyUp(event) {
      if (event.keyCode) {
        let methodName = this._getMethodFromKeyCode(event.keyCode);

        let method = this.get("keyEvents.".concat(methodName));

        if (method) {
          method(event.target.value);
        }
      }
    },

    _focus() {
      // Until mobile safari has better support
      // for focusing, we just ignore it
      if (this.shouldFocus && !this.userAgent.os.isIOS) {
        this.element.focus();
      }
    },

    _getMethodFromKeyCode(keyCode) {
      let methodName = keyCodes[keyCode.toString()];
      return methodName;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/unauthenticated-route-mixin", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Mixin.create({
    ajax: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    session: Ember.inject.service(),
    routeIfAlreadyAuthenticated: 'home',

    beforeModel() {
      let authUrl = this.get('ghostPaths.url').api('authentication', 'setup'); // check the state of the setup process via the API

      return this.ajax.request(authUrl).then(result => {
        let [setup] = result.setup;

        if (setup.status !== true) {
          this.transitionTo('setup');
        } else {
          // NOTE: this is the same as ESA's UnauthenticatedRouteMixin,
          // adding that mixin to this and calling _super wasn't calling
          // the ESA mixin's beforeModel method
          if (this.session.get('isAuthenticated')) {
            let routeIfAlreadyAuthenticated = this.routeIfAlreadyAuthenticated;
            return this.transitionTo(routeIfAlreadyAuthenticated);
          } else {
            return this._super(...arguments);
          }
        }
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/validation-engine", ["exports", "ember-data", "ghost-admin/validators/integration", "ghost-admin/validators/invite-user", "ember-data/model", "ghost-admin/validators/nav-item", "ghost-admin/validators/post", "ghost-admin/validators/setting", "ghost-admin/validators/slack-integration", "ghost-admin/validators/subscriber", "ghost-admin/validators/tag-settings", "ghost-admin/validators/user", "ghost-admin/validators/webhook"], function (_exports, _emberData, _integration, _inviteUser, _model, _navItem, _post, _setting, _slackIntegration, _subscriber, _tagSettings, _user, _webhook) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    Errors
  } = _emberData.default;
  /**
  * The class that gets this mixin will receive these properties and functions.
  * It will be able to validate any properties on itself (or the model it passes to validate())
  * with the use of a declared validator.
  */

  var _default = Ember.Mixin.create({
    // these validators can be passed a model to validate when the class that
    // mixes in the ValidationEngine declares a validationType equal to a key on this object.
    // the model is either passed in via `this.validate({ model: object })`
    // or by calling `this.validate()` without the model property.
    // in that case the model will be the class that the ValidationEngine
    // was mixed into, i.e. the controller or Ember Data model.
    validators: {
      inviteUser: _inviteUser.default,
      navItem: _navItem.default,
      post: _post.default,
      setting: _setting.default,
      slackIntegration: _slackIntegration.default,
      subscriber: _subscriber.default,
      tag: _tagSettings.default,
      user: _user.default,
      integration: _integration.default,
      webhook: _webhook.default
    },
    // This adds the Errors object to the validation engine, and shouldn't affect
    // ember-data models because they essentially use the same thing
    errors: null,
    // Store whether a property has been validated yet, so that we know whether or not
    // to show error / success validation for a field
    hasValidated: null,

    init() {
      this._super(...arguments);

      this.set('errors', Errors.create());
      this.set('hasValidated', Ember.A());
    },

    /**
    * Passes the model to the validator specified by validationType.
    * Returns a promise that will resolve if validation succeeds, and reject if not.
    * Some options can be specified:
    *
    * `model: Object` - you can specify the model to be validated, rather than pass the default value of `this`,
    *                   the class that mixes in this mixin.
    *
    * `property: String` - you can specify a specific property to validate. If
    * 					   no property is specified, the entire model will be
    * 					   validated
    */
    validate(opts) {
      let model = this;
      let hasValidated, type, validator;
      opts = opts || {};

      if (opts.model) {
        model = opts.model;
      } else if (this instanceof _model.default) {
        model = this;
      } else if (this.model) {
        model = this.model;
      }

      type = this.validationType || model.get('validationType');
      validator = this.get("validators.".concat(type)) || model.get("validators.".concat(type));
      hasValidated = this.hasValidated;
      opts.validationType = type;
      return new Ember.RSVP.Promise((resolve, reject) => {
        let passed;

        if (!type || !validator) {
          return reject(["The validator specified, \"".concat(type, "\", did not exist!")]);
        }

        if (opts.property) {
          // If property isn't in `hasValidated`, add it to mark that this field can show a validation result
          hasValidated.addObject(opts.property);
          model.get('errors').remove(opts.property);
        } else {
          model.get('errors').clear();
        }

        passed = validator.check(model, opts.property);
        return passed ? resolve() : reject();
      });
    },

    /**
    * The primary goal of this method is to override the `save` method on Ember Data models.
    * This allows us to run validation before actually trying to save the model to the server.
    * You can supply options to be passed into the `validate` method, since the ED `save` method takes no options.
    */
    save(options) {
      let {
        _super
      } = this;
      options = options || {};
      options.wasSave = true; // model.destroyRecord() calls model.save() behind the scenes.
      // in that case, we don't need validation checks or error propagation,
      // because the model itself is being destroyed.

      if (this.isDeleted) {
        return this._super(...arguments);
      } // If validation fails, reject with validation errors.
      // If save to the server fails, reject with server response.


      return this.validate(options).then(() => {
        if (typeof this.beforeSave === 'function') {
          return this.beforeSave().then(() => {
            return _super.call(this, options);
          });
        }

        return _super.call(this, options);
      }).catch(result => {
        // server save failed or validator type doesn't exist
        if (result && !Ember.isArray(result)) {
          throw result;
        }

        return Ember.RSVP.reject(result);
      });
    },

    actions: {
      validate(property) {
        this.validate({
          property
        });
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/mixins/validation-state", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Mixin.create({
    errors: null,
    property: '',
    hasValidated: Ember.A(),
    hasError: false,

    setHasError() {
      let property = this.property;
      let errors = this.errors;
      let hasValidated = this.hasValidated; // if we aren't looking at a specific property we always want an error class

      if (!property && errors && !errors.get('isEmpty')) {
        this.set('hasError', true);
        return;
      } // If we haven't yet validated this field, there is no validation class needed


      if (!hasValidated || !hasValidated.includes(property)) {
        this.set('hasError', false);
        return;
      }

      if (errors && !Ember.isEmpty(errors.errorsFor(property))) {
        this.set('hasError', true);
        return;
      }

      this.set('hasError', false);
    },

    // eslint-disable-next-line ghost/ember/no-observers
    hasErrorObserver: Ember.observer('errors.[]', 'property', 'hasValidated.[]', function () {
      Ember.run.once(this, 'setHasError'); // this.setHasError();
    }).on('init')
  });

  _exports.default = _default;
});
;define("ghost-admin/models/api-key", ["exports", "ember-data/model", "ember-data/attr", "ember-data/relationships"], function (_exports, _model, _attr, _relationships) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend({
    type: (0, _attr.default)('string'),
    secret: (0, _attr.default)('string'),
    lastSeenAtUTC: (0, _attr.default)('moment-utc'),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    createdBy: (0, _attr.default)('number'),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    updatedBy: (0, _attr.default)('number'),
    integration: (0, _relationships.belongsTo)('integration')
  });

  _exports.default = _default;
});
;define("ghost-admin/models/coordinator", ["exports", "ghost-admin/models/obj-hash"], function (_exports, _objHash) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Object.extend(Ember.Evented, {
    objectMap: Ember.computed(function () {
      return _objHash.default.create();
    }),
    getObject: function getObject(id, ops) {
      ops = ops || {};
      var payload = this.get('objectMap').getObj(id);

      if (payload.ops.source) {
        payload.ops.source.sendAction('action', payload.obj);
      }

      if (payload.ops.target) {
        payload.ops.target.sendAction('action', payload.obj);
      }

      this.trigger("objectMoved", {
        obj: payload.obj,
        source: payload.ops.source,
        target: ops.target
      });
      return payload.obj;
    },
    setObject: function setObject(obj, ops) {
      ops = ops || {};
      return this.get('objectMap').add({
        obj: obj,
        ops: ops
      });
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/models/integration", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr", "ember-data/relationships"], function (_exports, _model, _validationEngine, _attr, _relationships) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend(_validationEngine.default, {
    validationType: 'integration',
    name: (0, _attr.default)('string'),
    slug: (0, _attr.default)('string'),
    type: (0, _attr.default)('string'),
    iconImage: (0, _attr.default)('string'),
    description: (0, _attr.default)('string'),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    createdBy: (0, _attr.default)('number'),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    updatedBy: (0, _attr.default)('number'),
    apiKeys: (0, _relationships.hasMany)('api-key', {
      embedded: 'always',
      async: false
    }),
    webhooks: (0, _relationships.hasMany)('webhook', {
      embedded: 'always',
      async: false
    }),
    adminKey: Ember.computed('apiKeys.[]', function () {
      return this.apiKeys.findBy('type', 'admin');
    }),
    contentKey: Ember.computed('apiKeys.[]', function () {
      return this.apiKeys.findBy('type', 'content');
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/models/member-subscription", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Object.extend({
    adapter: '',
    amount: 0,
    plan: '',
    status: '',
    validUntil: 0
  });

  _exports.default = _default;
});
;define("ghost-admin/models/member", ["exports", "ember-data", "ember-data/attr"], function (_exports, _emberData, _attr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _emberData.default.Model.extend({
    name: (0, _attr.default)('string'),
    email: (0, _attr.default)('string'),
    createdAt: (0, _attr.default)('moment-utc'),
    subscriptions: (0, _attr.default)('member-subscription')
  });

  _exports.default = _default;
});
;define("ghost-admin/models/navigation-item", ["exports", "ghost-admin/mixins/validation-engine"], function (_exports, _validationEngine) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Object.extend(_validationEngine.default, {
    label: '',
    url: '',
    isNew: false,
    validationType: 'navItem',
    isComplete: Ember.computed('label', 'url', function () {
      let {
        label,
        url
      } = this;
      return !Ember.isBlank(label) && !Ember.isBlank(url);
    }),
    isBlank: Ember.computed('label', 'url', function () {
      let {
        label,
        url
      } = this;
      return Ember.isBlank(label) && Ember.isBlank(url);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/models/notification", ["exports", "ember-data/model", "ember-data/attr"], function (_exports, _model, _attr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend({
    custom: (0, _attr.default)('boolean'),
    dismissible: (0, _attr.default)('boolean'),
    key: (0, _attr.default)('string'),
    message: (0, _attr.default)('string'),
    status: (0, _attr.default)('string'),
    type: (0, _attr.default)('string')
  });

  _exports.default = _default;
});
;define("ghost-admin/models/obj-hash", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Object.extend({
    contentLength: 0,
    length: Ember.computed.alias('contentLength'),
    init: function init() {
      this._super();

      this.content = {};
    },
    add: function add(obj) {
      var id = this.generateId();
      this.get('content')[id] = obj;
      this.incrementProperty("contentLength");
      return id;
    },
    getObj: function getObj(key) {
      var res = this.get('content')[key];

      if (!res) {
        throw "no obj for key " + key;
      }

      return res;
    },
    generateId: function generateId() {
      var num = Math.random() * 1000000000000.0;
      num = parseInt(num);
      num = "" + num;
      return num;
    },
    keys: function keys() {
      var res = [];

      for (var key in this.get('content')) {
        res.push(key);
      }

      return Ember.A(res);
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/models/page", ["exports", "ghost-admin/models/post"], function (_exports, _post) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _post.default.extend({
    displayName: 'page'
  });

  _exports.default = _default;
});
;define("ghost-admin/models/post", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr", "ghost-admin/utils/bound-one-way", "moment", "ember-data/relationships"], function (_exports, _model, _validationEngine, _attr, _boundOneWay, _moment, _relationships) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  // ember-cli-shims doesn't export these so we must get them manually
  const {
    Comparable
  } = Ember;

  function statusCompare(postA, postB) {
    let status1 = postA.get('status');
    let status2 = postB.get('status'); // if any of those is empty

    if (!status1 && !status2) {
      return 0;
    }

    if (!status1 && status2) {
      return -1;
    }

    if (!status2 && status1) {
      return 1;
    } // We have to make sure, that scheduled posts will be listed first
    // after that, draft and published will be sorted alphabetically and don't need
    // any manual comparison.


    if (status1 === 'scheduled' && (status2 === 'draft' || status2 === 'published')) {
      return -1;
    }

    if (status2 === 'scheduled' && (status1 === 'draft' || status1 === 'published')) {
      return 1;
    }

    return Ember.compare(status1.valueOf(), status2.valueOf());
  }

  function publishedAtCompare(postA, postB) {
    let published1 = postA.get('publishedAtUTC');
    let published2 = postB.get('publishedAtUTC');

    if (!published1 && !published2) {
      return 0;
    }

    if (!published1 && published2) {
      return -1;
    }

    if (!published2 && published1) {
      return 1;
    }

    return Ember.compare(published1.valueOf(), published2.valueOf());
  }

  var _default = _model.default.extend(Comparable, _validationEngine.default, {
    config: Ember.inject.service(),
    feature: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    clock: Ember.inject.service(),
    settings: Ember.inject.service(),
    notifications: Ember.inject.service(),
    rcServices: Ember.inject.service('rc-services'),
    displayName: 'post',
    validationType: 'post',
    createdAtUTC: (0, _attr.default)('moment-utc'),
    excerpt: (0, _attr.default)('string'),
    customExcerpt: (0, _attr.default)('string'),
    featured: (0, _attr.default)('boolean', {
      defaultValue: false
    }),
    announce: (0, _attr.default)('boolean', {
      defaultValue: false
    }),
    collaborate: (0, _attr.default)('boolean', {
      defaultValue: false
    }),
    isPrivate: (0, _attr.default)('boolean', {
      defaultValue: false
    }),
    featureImage: (0, _attr.default)('string'),
    canonicalUrl: (0, _attr.default)('string'),
    codeinjectionFoot: (0, _attr.default)('string', {
      defaultValue: ''
    }),
    codeinjectionHead: (0, _attr.default)('string', {
      defaultValue: ''
    }),
    customTemplate: (0, _attr.default)('string'),
    ogImage: (0, _attr.default)('string'),
    ogTitle: (0, _attr.default)('string'),
    ogDescription: (0, _attr.default)('string'),
    twitterImage: (0, _attr.default)('string'),
    twitterTitle: (0, _attr.default)('string'),
    twitterDescription: (0, _attr.default)('string'),
    rcImage: (0, _attr.default)('string'),
    rcTitle: (0, _attr.default)('string'),
    rcDescription: (0, _attr.default)('string'),
    html: (0, _attr.default)('string'),
    locale: (0, _attr.default)('string'),
    metaDescription: (0, _attr.default)('string'),
    metaTitle: (0, _attr.default)('string'),
    mobiledoc: (0, _attr.default)('json-string'),
    plaintext: (0, _attr.default)('string'),
    publishedAtUTC: (0, _attr.default)('moment-utc'),
    slug: (0, _attr.default)('string'),
    status: (0, _attr.default)('string', {
      defaultValue: 'draft'
    }),
    title: (0, _attr.default)('string', {
      defaultValue: ''
    }),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    updatedBy: (0, _attr.default)('number'),
    url: (0, _attr.default)('string'),
    uuid: (0, _attr.default)('string'),
    roomName: (0, _attr.default)('string'),
    roomId: (0, _attr.default)('string'),
    discussionRoomId: (0, _attr.default)('string'),
    discussionRoomName: (0, _attr.default)('string'),
    discussionRoomType: (0, _attr.default)('string', {
      defaultValue: 'c'
    }),
    authors: (0, _relationships.hasMany)('user', {
      embedded: 'always',
      async: false
    }),
    createdBy: (0, _relationships.belongsTo)('user', {
      async: true
    }),
    publishedBy: (0, _relationships.belongsTo)('user', {
      async: true
    }),
    tags: (0, _relationships.hasMany)('tag', {
      embedded: 'always',
      async: false
    }),
    primaryAuthor: Ember.computed('authors.[]', function () {
      return this.get('authors.firstObject');
    }),
    announceChanged: false,
    scratch: null,
    titleScratch: null,
    // HACK: used for validation so that date/time can be validated based on
    // eventual status rather than current status
    statusScratch: null,
    // For use by date/time pickers - will be validated then converted to UTC
    // on save. Updated by an observer whenever publishedAtUTC changes.
    // Everything that revolves around publishedAtUTC only cares about the saved
    // value so this should be almost entirely internal
    publishedAtBlogDate: '',
    publishedAtBlogTime: '',
    roomNameScratch: (0, _boundOneWay.default)('roomName'),
    canonicalUrlScratch: (0, _boundOneWay.default)('canonicalUrl'),
    customExcerptScratch: (0, _boundOneWay.default)('customExcerpt'),
    codeinjectionFootScratch: (0, _boundOneWay.default)('codeinjectionFoot'),
    codeinjectionHeadScratch: (0, _boundOneWay.default)('codeinjectionHead'),
    metaDescriptionScratch: (0, _boundOneWay.default)('metaDescription'),
    metaTitleScratch: (0, _boundOneWay.default)('metaTitle'),
    ogDescriptionScratch: (0, _boundOneWay.default)('ogDescription'),
    ogTitleScratch: (0, _boundOneWay.default)('ogTitle'),
    twitterDescriptionScratch: (0, _boundOneWay.default)('twitterDescription'),
    twitterTitleScratch: (0, _boundOneWay.default)('twitterTitle'),
    rcDescriptionScratch: (0, _boundOneWay.default)('rcDescription'),
    rcTitleScratch: (0, _boundOneWay.default)('rcTitle'),
    isPublished: Ember.computed.equal('status', 'published'),
    isDraft: Ember.computed.equal('status', 'draft'),
    internalTags: Ember.computed.filterBy('tags', 'isInternal', true),
    isScheduled: Ember.computed.equal('status', 'scheduled'),
    previewUrl: Ember.computed('uuid', 'ghostPaths.url', 'config.blogUrl', function () {
      let blogUrl = this.get('config.blogUrl');
      let uuid = this.uuid; // routeKeywords.preview: 'p'

      let previewKeyword = 'p'; // New posts don't have a preview

      if (!uuid) {
        return '';
      }

      return this.get('ghostPaths.url').join(blogUrl, previewKeyword, uuid);
    }),
    // check every second to see if we're past the scheduled time
    // will only re-compute if this property is being observed elsewhere
    pastScheduledTime: Ember.computed('isScheduled', 'publishedAtUTC', 'clock.second', function () {
      if (this.isScheduled) {
        let now = _moment.default.utc();

        let publishedAtUTC = this.publishedAtUTC || now;
        let pastScheduledTime = publishedAtUTC.diff(now, 'hours', true) < 0; // force a recompute

        this.get('clock.second');
        return pastScheduledTime;
      } else {
        return false;
      }
    }),
    publishedAtBlogTZ: Ember.computed('publishedAtBlogDate', 'publishedAtBlogTime', 'settings.activeTimezone', {
      get() {
        return this._getPublishedAtBlogTZ();
      },

      set(key, value) {
        let momentValue = value ? (0, _moment.default)(value) : null;

        this._setPublishedAtBlogStrings(momentValue);

        return this._getPublishedAtBlogTZ();
      }

    }),

    _getPublishedAtBlogTZ() {
      let publishedAtUTC = this.publishedAtUTC;
      let publishedAtBlogDate = this.publishedAtBlogDate;
      let publishedAtBlogTime = this.publishedAtBlogTime;
      let blogTimezone = this.get('settings.activeTimezone');

      if (!publishedAtUTC && Ember.isBlank(publishedAtBlogDate) && Ember.isBlank(publishedAtBlogTime)) {
        return null;
      }

      if (publishedAtBlogDate && publishedAtBlogTime) {
        let publishedAtBlog = _moment.default.tz("".concat(publishedAtBlogDate, " ").concat(publishedAtBlogTime), blogTimezone);
        /**
         * Note:
         * If you create a post and publish it, we send seconds to the database.
         * If you edit the post afterwards, ember would send the date without seconds, because
         * the `publishedAtUTC` is based on `publishedAtBlogTime`, which is only in seconds.
         * The date time picker doesn't use seconds.
         *
         * This condition prevents the case:
         *   - you edit a post, but you don't change the published_at time
         *   - we keep the original date with seconds
         *
         * See https://github.com/TryGhost/Ghost/issues/8603#issuecomment-309538395.
         */


        if (publishedAtUTC && publishedAtBlog.diff(publishedAtUTC.clone().startOf('minutes')) === 0) {
          return publishedAtUTC;
        }

        return publishedAtBlog;
      } else {
        return _moment.default.tz(this.publishedAtUTC, blogTimezone);
      }
    },

    // TODO: is there a better way to handle this?
    // eslint-disable-next-line ghost/ember/no-observers
    _setPublishedAtBlogTZ: Ember.observer('publishedAtUTC', 'settings.activeTimezone', function () {
      let publishedAtUTC = this.publishedAtUTC;

      this._setPublishedAtBlogStrings(publishedAtUTC);
    }).on('init'),

    _setPublishedAtBlogStrings(momentDate) {
      if (momentDate) {
        let blogTimezone = this.get('settings.activeTimezone');

        let publishedAtBlog = _moment.default.tz(momentDate, blogTimezone);

        this.set('publishedAtBlogDate', publishedAtBlog.format('YYYY-MM-DD'));
        this.set('publishedAtBlogTime', publishedAtBlog.format('HH:mm'));
      } else {
        this.set('publishedAtBlogDate', '');
        this.set('publishedAtBlogTime', '');
      }
    },

    // remove client-generated tags, which have `id: null`.
    // Ember Data won't recognize/update them automatically
    // when returned from the server with ids.
    // https://github.com/emberjs/data/issues/1829
    updateTags() {
      let tags = this.tags;
      let oldTags = tags.filterBy('id', null);
      tags.removeObjects(oldTags);
      oldTags.invoke('deleteRecord');
    },

    isAuthoredByUser(user) {
      return this.authors.includes(user);
    },

    tryCollaboration(user) {
      if (!this.get('settings.canCollaborate')) {
        return false;
      }

      this.authors.pushObject(user);
      return this.rcServices.collaborate(user.get('rc_id'), this.get('id'), this).then(result => {
        const collaborated = result.data[0].collaborate;

        if (!collaborated) {
          this.authors.removeObjects(user);
        }

        return collaborated;
      });
    },

    // a custom sort function is needed in order to sort the posts list the same way the server would:
    //     status: scheduled, draft, published
    //     publishedAt: DESC
    //     updatedAt: DESC
    //     id: DESC
    compare(postA, postB) {
      let updated1 = postA.get('updatedAtUTC');
      let updated2 = postB.get('updatedAtUTC');
      let idResult, publishedAtResult, statusResult, updatedAtResult; // when `updatedAt` is undefined, the model is still
      // being written to with the results from the server

      if (postA.get('isNew') || !updated1) {
        return -1;
      }

      if (postB.get('isNew') || !updated2) {
        return 1;
      } // TODO: revisit the ID sorting because we no longer have auto-incrementing IDs


      idResult = Ember.compare(postA.get('id'), postB.get('id'));
      statusResult = statusCompare(postA, postB);
      updatedAtResult = Ember.compare(updated1.valueOf(), updated2.valueOf());
      publishedAtResult = publishedAtCompare(postA, postB);

      if (statusResult === 0) {
        if (publishedAtResult === 0) {
          if (updatedAtResult === 0) {
            // This should be DESC
            return idResult * -1;
          } // This should be DESC


          return updatedAtResult * -1;
        } // This should be DESC


        return publishedAtResult * -1;
      }

      return statusResult;
    },

    // this is a hook added by the ValidationEngine mixin and is called after
    // successful validation and before this.save()
    //
    // the publishedAtBlog{Date/Time} strings are set separately so they can be
    // validated, grab that time if it exists and set the publishedAtUTC
    // 
    // Add announcing room, announce, and commentId.
    beforeSave() {
      let publishedAtBlogTZ = this.publishedAtBlogTZ;
      let publishedAtUTC = publishedAtBlogTZ ? publishedAtBlogTZ.utc() : null;
      this.set('publishedAtUTC', publishedAtUTC);
      let roomName = this.roomName;
      const roomId = roomName ? this.roomId : this.get('settings.roomId');
      roomName = roomName ? roomName : this.get('settings.roomName');
      this.set('roomName', roomName);
      this.set('roomId', roomId);

      if (!this.announceChanged) {
        const announce = this.get('settings.isAnnounced');
        this.set('announce', announce);
      }

      return new Ember.RSVP.Promise(resolve => {
        if (this.get('settings.isComments') && this.isPublished && !this.discussionRoomId && this.displayName === 'post') {
          return this.rcServices.createDiscussion(this.title, this.discussionRoomType).then(room => {
            const [{
              created,
              rid,
              type,
              roomname
            }] = room.data;

            if (created) {
              this.set('discussionRoomId', rid);
              this.set('discussionRoomName', roomname);
              this.set('discussionRoomType', type);
            }

            return resolve();
          });
        }

        return resolve();
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/models/role", ["exports", "ember-data/model", "ember-data/attr"], function (_exports, _model, _attr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _model.default.extend({
    name: (0, _attr.default)('string'),
    description: (0, _attr.default)('string'),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    createdBy: (0, _attr.default)('number'),
    updatedBy: (0, _attr.default)('number'),
    lowerCaseName: Ember.computed('name', function () {
      return (this.name || '').toLocaleLowerCase();
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/models/setting", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr"], function (_exports, _model, _validationEngine, _attr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _model.default.extend(_validationEngine.default, {
    validationType: 'setting',
    rcServices: Ember.inject.service('rc-services'),
    title: (0, _attr.default)('string'),
    description: (0, _attr.default)('string'),
    logo: (0, _attr.default)('string'),
    coverImage: (0, _attr.default)('string'),
    icon: (0, _attr.default)('string'),
    defaultLocale: (0, _attr.default)('string'),
    activeTimezone: (0, _attr.default)('string', {
      defaultValue: 'Etc/UTC'
    }),
    codeinjectionHead: (0, _attr.default)('string'),
    codeinjectionFoot: (0, _attr.default)('string'),
    facebook: (0, _attr.default)('facebook-url-user'),
    twitter: (0, _attr.default)('twitter-url-user'),
    roomId: (0, _attr.default)('string'),
    labs: (0, _attr.default)('string'),
    navigation: (0, _attr.default)('navigation-settings'),
    serverUrl: (0, _attr.default)('string'),
    isAnnounced: (0, _attr.default)('boolean'),
    isPrivate: (0, _attr.default)('boolean'),
    isAuthorsRooms: (0, _attr.default)('boolean'),
    announceToken: (0, _attr.default)('string'),
    settingsToken: (0, _attr.default)('string'),
    isComments: (0, _attr.default)('boolean'),
    inviteOnly: (0, _attr.default)('boolean'),
    canCollaborate: (0, _attr.default)('boolean'),
    publicHash: (0, _attr.default)('string'),
    password: (0, _attr.default)('string'),
    slack: (0, _attr.default)('slack-settings'),
    amp: (0, _attr.default)('boolean'),
    unsplash: (0, _attr.default)('unsplash-settings', {
      defaultValue() {
        return {
          isActive: true
        };
      }

    }),
    membersSubscriptionSettings: (0, _attr.default)('string'),
    roomName: Ember.computed('roomId', function () {
      this.rcServices.getRoom({
        rid: this.roomId
      }).then(room => {
        const {
          data: [{
            exist,
            roomname
          }]
        } = room;

        if (!exist) {
          this.set('roomName', 'NOT EXIST');
        }

        this.set('roomName', roomname);
      });
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/models/slack-integration", ["exports", "ghost-admin/mixins/validation-engine"], function (_exports, _validationEngine) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Object.extend(_validationEngine.default, {
    // values entered here will act as defaults
    url: '',
    username: '',
    validationType: 'slackIntegration',
    isActive: Ember.computed('url', function () {
      let url = this.url;
      return !Ember.isBlank(url);
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/models/subscriber", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr", "ember-data/relationships"], function (_exports, _model, _validationEngine, _attr, _relationships) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend(_validationEngine.default, {
    validationType: 'subscriber',
    name: (0, _attr.default)('string'),
    email: (0, _attr.default)('string'),
    status: (0, _attr.default)('string'),
    subscribedUrl: (0, _attr.default)('string'),
    subscribedReferrer: (0, _attr.default)('string'),
    unsubscribedUrl: (0, _attr.default)('string'),
    unsubscribedAtUTC: (0, _attr.default)('moment-utc'),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    createdBy: (0, _attr.default)('number'),
    updatedBy: (0, _attr.default)('number'),
    post: (0, _relationships.belongsTo)('post')
  });

  _exports.default = _default;
});
;define("ghost-admin/models/tag", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr"], function (_exports, _model, _validationEngine, _attr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend(_validationEngine.default, {
    validationType: 'tag',
    name: (0, _attr.default)('string'),
    slug: (0, _attr.default)('string'),
    description: (0, _attr.default)('string'),
    parent: (0, _attr.default)('string'),
    // unused
    metaTitle: (0, _attr.default)('string'),
    metaDescription: (0, _attr.default)('string'),
    featureImage: (0, _attr.default)('string'),
    visibility: (0, _attr.default)('string', {
      defaultValue: 'public'
    }),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    createdBy: (0, _attr.default)('number'),
    updatedBy: (0, _attr.default)('number'),
    count: (0, _attr.default)('raw'),
    isInternal: Ember.computed.equal('visibility', 'internal'),
    isPublic: Ember.computed.equal('visibility', 'public'),
    feature: Ember.inject.service(),

    updateVisibility() {
      let internalRegex = /^#.?/;
      this.set('visibility', internalRegex.test(this.name) ? 'internal' : 'public');
    },

    save() {
      if (this.get('changedAttributes.name') && !this.isDeleted) {
        this.updateVisibility();
      }

      return this._super(...arguments);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/models/theme", ["exports", "ember-data/model", "ember-data/attr"], function (_exports, _model, _attr) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend({
    active: (0, _attr.default)('boolean'),
    errors: (0, _attr.default)('raw'),
    name: (0, _attr.default)('string'),
    package: (0, _attr.default)('raw'),
    templates: (0, _attr.default)('raw', {
      defaultValue: () => []
    }),
    warnings: (0, _attr.default)('raw'),
    customTemplates: Ember.computed('templates.[]', function () {
      let templates = this.templates || [];
      return templates.filter(function (template) {
        return Ember.isBlank(template.slug);
      });
    }),
    slugTemplates: Ember.computed('templates.[]', function () {
      let templates = this.templates || [];
      return templates.filter(function (template) {
        return !Ember.isBlank(template.slug);
      });
    }),

    activate() {
      let adapter = this.store.adapterFor(this.constructor.modelName);
      return adapter.activate(this).then(() => {
        // the server only gives us the newly active theme back so we need
        // to manually mark other themes as inactive in the store
        let activeThemes = this.store.peekAll('theme').filterBy('active', true);
        activeThemes.forEach(theme => {
          if (theme !== this) {
            // store.push is necessary to avoid dirty records that cause
            // problems when we get new data back in subsequent requests
            this.store.push({
              data: {
                id: theme.id,
                type: 'theme',
                attributes: {
                  active: false
                }
              }
            });
          }
        });
        return this;
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/models/unsplash-integration", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Object.extend({
    isActive: false
  });

  _exports.default = _default;
});
;define("ghost-admin/models/user", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr", "ember-data/relationships", "ember-concurrency"], function (_exports, _model, _validationEngine, _attr, _relationships, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _model.default.extend(_validationEngine.default, {
    validationType: 'user',
    name: (0, _attr.default)('string'),
    rc_id: (0, _attr.default)('string'),
    rc_username: (0, _attr.default)('string'),
    slug: (0, _attr.default)('string'),
    email: (0, _attr.default)('string'),
    profileImage: (0, _attr.default)('string'),
    coverImage: (0, _attr.default)('string'),
    bio: (0, _attr.default)('string'),
    website: (0, _attr.default)('string'),
    location: (0, _attr.default)('string'),
    accessibility: (0, _attr.default)('string'),
    status: (0, _attr.default)('string'),
    locale: (0, _attr.default)('string'),
    metaTitle: (0, _attr.default)('string'),
    metaDescription: (0, _attr.default)('string'),
    lastLoginUTC: (0, _attr.default)('moment-utc'),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    // CreatedBy will be updated from server
    createdBy: (0, _attr.default)('number'),
    parentId: (0, _attr.default)('string', {
      defaultValue: '1'
    }),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    updatedBy: (0, _attr.default)('number'),
    roles: (0, _relationships.hasMany)('role', {
      embedded: 'always',
      async: false
    }),
    count: (0, _attr.default)('raw'),
    facebook: (0, _attr.default)('facebook-url-user'),
    twitter: (0, _attr.default)('twitter-url-user'),
    tour: (0, _attr.default)('json-string'),
    ghostPaths: Ember.inject.service(),
    ajax: Ember.inject.service(),
    session: Ember.inject.service(),
    notifications: Ember.inject.service(),
    config: Ember.inject.service(),
    // TODO: Once client-side permissions are in place,
    // remove the hard role check.
    isContributor: Ember.computed.equal('role.name', 'Contributor'),
    isAuthor: Ember.computed.equal('role.name', 'Author'),
    isEditor: Ember.computed.equal('role.name', 'Editor'),
    isAdmin: Ember.computed.equal('role.name', 'Administrator'),
    isOwner: Ember.computed.equal('role.name', 'Owner'),
    // These are used in enough places that it's useful to throw them here
    isOwnerOrAdmin: Ember.computed.or('isOwner', 'isAdmin'),
    isAuthorOrContributor: Ember.computed.or('isAuthor', 'isContributor'),
    isLoggedIn: Ember.computed('id', 'session.user.id', function () {
      return this.id === this.get('session.user.id');
    }),
    isActive: Ember.computed('status', function () {
      // TODO: review "locked" as an "active" status
      return ['active', 'warn-1', 'warn-2', 'warn-3', 'warn-4', 'locked'].indexOf(this.status) > -1;
    }),
    isSuspended: Ember.computed.equal('status', 'inactive'),
    isLocked: Ember.computed.equal('status', 'locked'),
    role: Ember.computed('roles', {
      get() {
        return this.get('roles.firstObject');
      },

      set(key, value) {
        // Only one role per user, so remove any old data.
        this.roles.clear();
        this.roles.pushObject(value);
        return value;
      }

    }),
    profileImageUrl: Ember.computed('ghostPaths.assetRoot', 'profileImage', function () {
      // keep path separate so asset rewriting correctly picks it up
      let defaultImage = '/img/user-image.png';
      let defaultPath = this.ghostPaths.assetRoot.replace(/\/$/, '') + defaultImage;
      return this.profileImage || defaultPath;
    }),
    coverImageUrl: Ember.computed('ghostPaths.assetRoot', 'coverImage', function () {
      // keep path separate so asset rewriting correctly picks it up
      let defaultImage = '/img/user-cover.png';
      let defaultPath = this.ghostPaths.assetRoot.replace(/\/$/, '') + defaultImage;
      return this.coverImage || defaultPath;
    }),
    saveNewPassword: (0, _emberConcurrency.task)(function* () {
      let validation = this.isLoggedIn ? 'ownPasswordChange' : 'passwordChange';

      try {
        yield this.validate({
          property: validation
        });
      } catch (e) {
        // validation error, don't do anything
        return;
      }

      try {
        let url = this.get('ghostPaths.url').api('users', 'password');
        yield this.ajax.put(url, {
          data: {
            password: [{
              user_id: this.id,
              oldPassword: this.password,
              newPassword: this.newPassword,
              ne2Password: this.ne2Password
            }]
          }
        });
        this.setProperties({
          password: '',
          newPassword: '',
          ne2Password: ''
        });
        this.notifications.showNotification('Password updated.', {
          type: 'success',
          key: 'user.change-password.success'
        }); // clear errors manually for ne2password because validation
        // engine only clears the "validated proeprty"
        // TODO: clean up once we have a better validations library

        this.errors.remove('ne2Password');
        return true;
      } catch (error) {
        this.notifications.showAPIError(error, {
          key: 'user.change-password'
        });
      }
    }).drop()
  });

  _exports.default = _default;
});
;define("ghost-admin/models/webhook", ["exports", "ember-data/model", "ghost-admin/mixins/validation-engine", "ember-data/attr", "ember-data/relationships"], function (_exports, _model, _validationEngine, _attr, _relationships) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _model.default.extend(_validationEngine.default, {
    validationType: 'webhook',
    name: (0, _attr.default)('string'),
    event: (0, _attr.default)('string'),
    targetUrl: (0, _attr.default)('string'),
    secret: (0, _attr.default)('string'),
    lastTriggeredAtUTC: (0, _attr.default)('moment-utc'),
    createdAtUTC: (0, _attr.default)('moment-utc'),
    createdBy: (0, _attr.default)('number'),
    updatedAtUTC: (0, _attr.default)('moment-utc'),
    updatedBy: (0, _attr.default)('number'),
    integration: (0, _relationships.belongsTo)('integration')
  });

  _exports.default = _default;
});
;define("ghost-admin/resolver", ["exports", "ember-resolver"], function (_exports, _emberResolver) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _emberResolver.default;
  _exports.default = _default;
});
;define("ghost-admin/router", ["exports", "ghost-admin/config/environment", "ghost-admin/utils/ghost-paths"], function (_exports, _environment, _ghostPaths) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const Router = Ember.Router.extend({
    location: _environment.default.locationType,
    // use HTML5 History API instead of hash-tag based URLs
    rootURL: (0, _ghostPaths.default)().adminRoot // admin interface lives under sub-directory /ghost

  });
  Router.map(function () {
    this.route('home', {
      path: '/'
    });
    this.route('signin');
    this.route('setup');
    this.route('about');
    this.route('site');
    this.route('posts', function () {});
    this.route('pages', function () {});
    this.route('editor', function () {
      this.route('new', {
        path: ':type'
      });
      this.route('edit', {
        path: ':type/:post_id'
      });
    });
    this.route('staff', function () {
      this.route('user', {
        path: ':user_slug'
      });
    });
    this.route('settings.general', {
      path: '/settings/general'
    });
    this.route('settings.tags', {
      path: '/settings/tags'
    }, function () {
      this.route('tag', {
        path: ':tag_slug'
      });
      this.route('new');
    });
    this.route('settings.labs', {
      path: '/settings/labs'
    });
    this.route('settings.code-injection', {
      path: '/settings/code-injection'
    });
    this.route('settings.design', {
      path: '/settings/design'
    }, function () {
      this.route('uploadtheme');
    });
    this.route('settings.integrations', {
      path: '/settings/integrations'
    }, function () {
      this.route('new');
    });
    this.route('settings.integration', {
      path: '/settings/integrations/:integration_id'
    }, function () {
      this.route('webhooks.new', {
        path: 'webhooks/new'
      });
      this.route('webhooks.edit', {
        path: 'webhooks/:webhook_id'
      });
    });
    this.route('settings.integrations.slack', {
      path: '/settings/integrations/slack'
    });
    this.route('settings.integrations.amp', {
      path: '/settings/integrations/amp'
    });
    this.route('settings.integrations.unsplash', {
      path: '/settings/integrations/unsplash'
    });
    this.route('settings.integrations.zapier', {
      path: '/settings/integrations/zapier'
    });
    this.route('settings.integrations.search', {
      path: '/settings/integrations/search'
    });
    this.route('members');
    this.route('member', {
      path: '/members/:member_id'
    });
    this.route('subscribers', function () {
      this.route('new');
      this.route('import');
    });
    this.route('error404', {
      path: '/*path'
    });
  });
  var _default = Router;
  _exports.default = _default;
});
;define("ghost-admin/routes/about", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    buildRouteInfoMetadata() {
      return {
        titleToken: 'About'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/application", ["exports", "ember-simple-auth/mixins/application-route-mixin", "ember-simple-auth/configuration", "ghost-admin/mixins/shortcuts-route", "ghost-admin/utils/ctrl-or-cmd", "ghost-admin/utils/window-proxy", "ember-ajax/errors", "ghost-admin/services/ajax"], function (_exports, _applicationRouteMixin, _configuration, _shortcutsRoute, _ctrlOrCmd, _windowProxy, _errors, _ajax) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  function K() {
    return this;
  }

  let shortcuts = {};
  shortcuts.esc = {
    action: 'closeMenus',
    scope: 'default'
  };
  shortcuts["".concat(_ctrlOrCmd.default, "+s")] = {
    action: 'save',
    scope: 'all'
  };

  var _default = Ember.Route.extend(_applicationRouteMixin.default, _shortcutsRoute.default, {
    ajax: Ember.inject.service(),
    config: Ember.inject.service(),
    feature: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    notifications: Ember.inject.service(),
    router: Ember.inject.service(),
    settings: Ember.inject.service(),
    tour: Ember.inject.service(),
    ui: Ember.inject.service(),
    shortcuts,
    routeAfterAuthentication: 'home',

    init() {
      this._super(...arguments);

      this.router.on('routeDidChange', () => {
        this.notifications.displayDelayed();
      });
    },

    beforeModel() {
      return this.config.fetchUnauthenticated();
    },

    afterModel(model, transition) {
      this._super(...arguments);

      if (this.get('session.isAuthenticated')) {
        this.set('appLoadTransition', transition);
        transition.send('loadServerNotifications');
        let configPromise = this.config.fetchAuthenticated();
        let featurePromise = this.feature.fetch();
        let settingsPromise = this.settings.fetch();
        let tourPromise = this.tour.fetchViewed(); // return the feature/settings load promises so that we block until
        // they are loaded to enable synchronous access everywhere

        return Ember.RSVP.all([configPromise, featurePromise, settingsPromise, tourPromise]).then(results => {
          this._appLoaded = true;
          return results;
        });
      }

      this._appLoaded = true;
    },

    actions: {
      closeMenus() {
        this.ui.closeMenus();
      },

      didTransition() {
        this.set('appLoadTransition', null);
        this.send('closeMenus');
      },

      signedIn() {
        this.notifications.clearAll();
        this.send('loadServerNotifications', true);
      },

      authorizationFailed() {
        _windowProxy.default.replaceLocation(_configuration.default.rootURL);
      },

      loadServerNotifications(isDelayed) {
        if (this.get('session.isAuthenticated')) {
          this.get('session.user').then(user => {
            if (!user.get('isAuthorOrContributor')) {
              this.store.findAll('notification', {
                reload: true
              }).then(serverNotifications => {
                serverNotifications.forEach(notification => {
                  if (notification.get('top') || notification.get('custom')) {
                    this.notifications.handleNotification(notification, isDelayed);
                  } else {
                    this.upgradeStatus.handleUpgradeNotification(notification);
                  }
                });
              });
            }
          });
        }
      },

      // noop default for unhandled save (used from shortcuts)
      save: K,

      error(error, transition) {
        // unauthoirized errors are already handled in the ajax service
        if ((0, _errors.isUnauthorizedError)(error)) {
          return false;
        }

        if ((0, _errors.isNotFoundError)(error)) {
          if (transition) {
            transition.abort();
          }

          let routeInfo = transition.to;
          let router = this.router;
          let params = [];

          for (let key of Object.keys(routeInfo.params)) {
            params.push(routeInfo.params[key]);
          }

          let url = router.urlFor(routeInfo.name, ...params).replace(/^#\//, '').replace(/^\//, '').replace(/^ghost\//, '');
          return this.replaceWith('error404', url);
        }

        if ((0, _ajax.isVersionMismatchError)(error)) {
          if (transition) {
            transition.abort();
          }

          this.upgradeStatus.requireUpgrade();

          if (this._appLoaded) {
            return false;
          }
        }

        if ((0, _ajax.isMaintenanceError)(error)) {
          if (transition) {
            transition.abort();
          }

          this.upgradeStatus.maintenanceAlert();

          if (this._appLoaded) {
            return false;
          }
        }

        if ((0, _errors.isAjaxError)(error) || error && error.payload && Ember.isArray(error.payload.errors)) {
          this.notifications.showAPIError(error); // don't show the 500 page if we weren't navigating

          if (!transition) {
            return false;
          }
        } // fallback to 500 error page


        return true;
      }

    },

    sessionAuthenticated() {
      if (this.get('session.skipAuthSuccessHandler')) {
        return;
      } // standard ESA post-sign-in redirect


      this._super(...arguments); // trigger post-sign-in background behaviour


      this.get('session.user').then(user => {
        this.send('signedIn', user);
      });
    },

    sessionInvalidated() {
      let transition = this.appLoadTransition;

      if (transition) {
        transition.send('authorizationFailed');
      } else {
        Ember.run.scheduleOnce('routerTransitions', this, function () {
          this.send('authorizationFailed');
        });
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/authenticated", ["exports", "ember-simple-auth/mixins/authenticated-route-mixin"], function (_exports, _authenticatedRouteMixin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend(_authenticatedRouteMixin.default, {
    authenticationRoute: 'signin'
  });

  _exports.default = _default;
});
;define("ghost-admin/routes/editor", ["exports", "jquery", "ghost-admin/routes/authenticated", "ghost-admin/mixins/shortcuts-route", "ghost-admin/utils/ctrl-or-cmd"], function (_exports, _jquery, _authenticated, _shortcutsRoute, _ctrlOrCmd) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  let generalShortcuts = {};
  generalShortcuts["".concat(_ctrlOrCmd.default, "+shift+p")] = 'publish';

  var _default = _authenticated.default.extend(_shortcutsRoute.default, {
    feature: Ember.inject.service(),
    notifications: Ember.inject.service(),
    userAgent: Ember.inject.service(),
    ui: Ember.inject.service(),
    classNames: ['editor'],
    shortcuts: generalShortcuts,

    activate() {
      this._super(...arguments);

      this.ui.set('isFullScreen', true);
    },

    setupController() {
      this._super(...arguments); // edge has known issues


      if (this.userAgent.browser.isEdge) {
        this.notifications.showAlert(Ember.String.htmlSafe('Microsoft Edge is not currently supported. Please switch to <a href="https://ghost.org/downloads/" target="_blank" rel="noopener">Ghost Desktop</a> or a recent version of Chrome/Firefox/Safari.'), {
          type: 'info',
          key: 'koenig.browserSupport'
        });
      } // mobile browsers are not currently supported


      if (this.userAgent.device.isMobile || this.userAgent.device.isTablet) {
        this.notifications.showAlert(Ember.String.htmlSafe('Mobile editing is not currently supported. Please use a desktop browser or <a href="https://ghost.org/downloads/" target="_blank" rel="noopener">Ghost Desktop</a>.'), {
          type: 'info',
          key: 'koenig.browserSupport'
        });
      }
    },

    deactivate() {
      this._super(...arguments);

      this.ui.set('isFullScreen', false);
    },

    actions: {
      save() {
        this._blurAndScheduleAction(function () {
          this.controller.send('save');
        });
      },

      publish() {
        this._blurAndScheduleAction(function () {
          this.controller.send('setSaveType', 'publish');
          this.controller.send('save');
        });
      },

      authorizationFailed() {
        this.controller.send('toggleReAuthenticateModal');
      },

      redirectToContentScreen() {
        this.transitionTo('posts');
      },

      willTransition(transition) {
        // exit early if an upgrade is required because our extended route
        // class will abort the transition and show an error
        if (this.get('upgradeStatus.isRequired')) {
          return this._super(...arguments);
        }

        this.controller.willTransition(transition);
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: () => {
          return this.get('controller.post.title') || 'Editor';
        }
      };
    },

    _blurAndScheduleAction(func) {
      let selectedElement = (0, _jquery.default)(document.activeElement); // TODO: we should trigger a blur for textareas as well as text inputs

      if (selectedElement.is('input[type="text"]')) {
        selectedElement.trigger('focusout');
      } // wait for actions triggered by the focusout to finish before saving


      Ember.run.scheduleOnce('actions', this, func);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/editor/edit", ["exports", "ghost-admin/routes/authenticated", "ember-inflector"], function (_exports, _authenticated, _emberInflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    beforeModel(transition) {
      this._super(...arguments); // if the transition is not new->edit, reset the post on the controller
      // so that the editor view is cleared before showing the loading state


      if (transition.urlMethod !== 'replace') {
        let editor = this.controllerFor('editor');
        editor.set('post', null);
        editor.reset();
      }
    },

    model(params, transition) {
      // eslint-disable-next-line camelcase
      let {
        type: modelName,
        post_id
      } = params;

      if (!['post', 'page'].includes(modelName)) {
        let path = transition.intent.url.replace(/^\//, '');
        return this.replaceWith('error404', {
          path,
          status: 404
        });
      }

      let query = {
        id: post_id,
        status: 'all'
      };
      return this.store.query(modelName, query).then(records => records.get('firstObject'));
    },

    // the API will return a post even if the logged in user doesn't have
    // permission to edit it (all posts are public) so we need to do our
    // own permissions check and redirect if necessary
    afterModel(post) {
      this._super(...arguments);

      return this.get('session.user').then(user => {
        let returnRoute = "".concat((0, _emberInflector.pluralize)(post.constructor.modelName), ".index");

        if (!post.isAuthoredByUser(user) && post.collaborate) {
          return post.tryCollaboration(user).then(res => {
            if (!res) {
              return this.replaceWith(returnRoute);
            }

            return this.refresh();
          });
        }

        if (!post.isAuthoredByUser(user)) {
          return this.replaceWith(returnRoute);
        } // If the post is not a draft and user is contributor, redirect to index


        if (user.get('isContributor') && !post.get('isDraft')) {
          return this.replaceWith(returnRoute);
        }
      });
    },

    serialize(model) {
      return {
        type: model.constructor.modelName,
        post_id: model.id
      };
    },

    // there's no specific controller for this route, instead all editor
    // handling is done on the editor route/controler
    setupController(controller, post) {
      let editor = this.controllerFor('editor');
      editor.setPost(post);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/editor/index", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    beforeModel() {
      this._super(...arguments);

      this.replaceWith('editor.new', 'post');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/editor/new", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    model(params, transition) {
      let {
        type: modelName
      } = params;

      if (!['post', 'page'].includes(modelName)) {
        let path = transition.intent.url.replace(/^\//, '');
        return this.replaceWith('error404', {
          path,
          status: 404
        });
      }

      return this.get('session.user').then(user => {
        let users = [user];

        if (user.get('isContributor')) {
          const pid = user.get('parentId');
          return this.store.queryRecord('user', {
            id: pid
          }).then(parent => {
            if (parent) {
              users.push(parent);
            }

            return this.store.createRecord(modelName, {
              authors: users
            });
          });
        }

        return this.store.createRecord(modelName, {
          authors: users
        });
      });
    },

    // there's no specific controller for this route, instead all editor
    // handling is done on the editor route/controler
    setupController(controller, newPost) {
      let editor = this.controllerFor('editor');
      editor.setPost(newPost);
    },

    buildRouteInfoMetadata() {
      return {
        mainClasses: ['editor-new']
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/error404", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    controllerName: 'error',
    templateName: 'error',

    model() {
      return {
        status: 404
      };
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Error'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/home", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    beforeModel() {
      this.transitionTo('site');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/member", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    model(params) {
      this._isMemberUpdated = true;
      return this.store.findRecord('member', params.member_id, {
        reload: true
      });
    },

    setupController(controller, model) {
      this._super(...arguments);

      if (!this._isMemberUpdated) {
        controller.fetchMember.perform(model.get('id'));
      }
    },

    deactivate() {
      this._super(...arguments); // clear the properties


      this._isMemberUpdated = false;
    },

    titleToken() {
      return this.controller.get('member.name');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/members", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    config: Ember.inject.service(),

    // redirect to posts screen if:
    // - developer experiments aren't enabled
    // - TODO: members is disabled?
    // - logged in user isn't owner/admin
    beforeModel() {
      this._super(...arguments);

      if (!this.config.get('enableDeveloperExperiments')) {
        return this.transitionTo('home');
      }

      return this.session.user.then(user => {
        if (!user.isOwnerOrAdmin) {
          return this.transitionTo('home');
        }
      });
    },

    setupController(controller) {
      this._super(...arguments);

      controller.fetchMembers.perform();
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Members'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/pages", ["exports", "ghost-admin/routes/posts"], function (_exports, _posts) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _posts.default.extend({
    modelName: 'page',

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Pages'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/posts", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    infinity: Ember.inject.service(),
    queryParams: {
      type: {
        refreshModel: true,
        replace: true
      },
      author: {
        refreshModel: true,
        replace: true
      },
      tag: {
        refreshModel: true,
        replace: true
      },
      order: {
        refreshModel: true,
        replace: true
      }
    },
    modelName: 'post',
    perPage: 30,

    model(params) {
      return this.session.user.then(user => {
        let queryParams = {};
        let filterParams = {
          tag: params.tag,
          authors: user.slug
        };
        let paginationParams = {
          perPageParam: 'limit',
          totalPagesParam: 'meta.pagination.pages'
        };
        Ember.assign(filterParams, this._getTypeFilters(params.type));

        if (params.type === 'featured') {
          filterParams.featured = true;
        }

        if (user.isContributor) {
          // Contributors can only view their own draft posts
          filterParams.status = 'draft';
        }

        let filter = this._filterString(filterParams);

        if (!Ember.isBlank(filter)) {
          queryParams.filter = filter;
        }

        if (!Ember.isBlank(params.order)) {
          queryParams.order = params.order;
        }

        let perPage = this.perPage;
        let paginationSettings = Ember.assign({
          perPage,
          startingPage: 1
        }, paginationParams, queryParams);
        return this.infinity.model(this.modelName, paginationSettings);
      });
    },

    // trigger a background load of all tags and authors for use in the filter dropdowns
    setupController(controller) {
      this._super(...arguments);

      if (!controller._hasLoadedTags) {
        this.store.query('tag', {
          limit: 'all'
        }).then(() => {
          controller._hasLoadedTags = true;
        });
      }

      this.session.user.then(user => {
        if (!user.isAuthorOrContributor && !controller._hasLoadedAuthors) {
          this.store.query('user', {
            limit: 'all'
          }).then(() => {
            controller._hasLoadedAuthors = true;
          });
        }
      });
    },

    actions: {
      queryParamsDidChange() {
        // scroll back to the top
        let contentList = document.querySelector('.content-list');

        if (contentList) {
          contentList.scrollTop = 0;
        }

        this._super(...arguments);
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Posts'
      };
    },

    _getTypeFilters(type) {
      let status = '[draft,scheduled,published]';

      switch (type) {
        case 'draft':
          status = 'draft';
          break;

        case 'published':
          status = 'published';
          break;

        case 'scheduled':
          status = 'scheduled';
          break;
      }

      return {
        status
      };
    },

    _filterString(filter) {
      return Object.keys(filter).map(key => {
        let value = filter[key];

        if (!Ember.isBlank(value)) {
          return "".concat(key, ":").concat(filter[key]);
        }
      }).compact().join('+');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/code-injection", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    model() {
      return this.settings.reload();
    },

    actions: {
      save() {
        this.controller.send('save');
      },

      willTransition(transition) {
        let controller = this.controller;
        let settings = this.settings;
        let modelIsDirty = settings.get('hasDirtyAttributes');

        if (modelIsDirty) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - Code injection'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/design", ["exports", "jquery", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _jquery, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor());
    },

    model() {
      return Ember.RSVP.hash({
        settings: this.settings.reload(),
        themes: this.store.findAll('theme')
      });
    },

    setupController(controller) {
      // reset the leave setting transition
      controller.set('leaveSettingsTransition', null);
      controller.set('themes', this.store.peekAll('theme'));
      this.controller.send('reset');
    },

    actions: {
      save() {
        // since shortcuts are run on the route, we have to signal to the components
        // on the page that we're about to save.
        (0, _jquery.default)('.page-actions .gh-btn-blue').focus();
        this.controller.send('save');
      },

      willTransition(transition) {
        let controller = this.controller;
        let modelIsDirty = controller.dirtyAttributes;

        if (modelIsDirty) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      },

      activateTheme(theme) {
        return this.controller.send('activateTheme', theme);
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - Design'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/design/uploadtheme", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    model() {
      return this.store.findAll('theme');
    },

    setupController(controller, model) {
      controller.set('themes', model);
    },

    actions: {
      cancel() {
        this.transitionTo('settings.design');
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/general", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    config: Ember.inject.service(),
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    model() {
      return Ember.RSVP.hash({
        settings: this.settings.reload(),
        availableTimezones: this.get('config.availableTimezones')
      });
    },

    setupController(controller, models) {
      // reset the leave setting transition
      controller.set('leaveSettingsTransition', null);
      controller.set('availableTimezones', models.availableTimezones);
    },

    actions: {
      save() {
        return this.controller.send('save');
      },

      reloadSettings() {
        return this.settings.reload();
      },

      willTransition(transition) {
        let controller = this.controller;
        let settings = this.settings;
        let settingsIsDirty = settings.get('hasDirtyAttributes');

        if (settingsIsDirty) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - General'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integration", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    router: Ember.inject.service(),

    init() {
      this._super(...arguments);

      this.router.on('routeWillChange', transition => {
        this.showUnsavedChangesModal(transition);
      });
    },

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    model(params, transition) {
      // use the integrations controller to fetch all integrations and pick
      // out the one we want. Allows navigation back to integrations screen
      // without a loading state
      return this.controllerFor('settings.integrations').integrationModelHook('id', params.integration_id, this, transition);
    },

    actions: {
      save() {
        this.controller.send('save');
      }

    },

    showUnsavedChangesModal(transition) {
      if (transition.from && transition.from.name.match(/^settings\.integration\./) && transition.targetName) {
        let {
          controller
        } = this; // check to see if we're navigating away from the custom integration
        // route - we want to allow editing webhooks without showing the
        // "unsaved changes" confirmation modal

        let isExternalRoute = // allow sub-routes of settings.integration
        !(transition.targetName || '').match(/^settings\.integration\./) // do not allow changes in integration
        // .to will be the index, so use .to.parent to get the route with the params
        || transition.to.parent.params.integration_id !== controller.integration.id;

        if (isExternalRoute && !controller.integration.isDeleted && controller.integration.hasDirtyAttributes) {
          transition.abort();
          controller.send('toggleUnsavedChangesModal', transition);
          return;
        }
      }
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - Integrations'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integration/webhooks/edit", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    model(params) {
      let integration = this.modelFor('settings.integration');
      let webhook = integration.webhooks.findBy('id', params.webhook_id);
      return webhook;
    },

    deactivate() {
      this._super(...arguments);

      this.controller.reset();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integration/webhooks/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    model() {
      let integration = this.modelFor('settings.integration');
      return this.store.createRecord('webhook', {
        integration
      });
    },

    deactivate() {
      this._super(...arguments);

      this.controller.webhook.rollbackAttributes();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    setupController(controller) {
      // kick off the background fetch of integrations so that we can
      // show the screen immediately
      controller.fetchIntegrations.perform();
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - Integrations'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations/amp", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor()).then(this.settings.reload());
    },

    actions: {
      save() {
        this.controller.send('save');
      },

      willTransition(transition) {
        let controller = this.controller;
        let modelIsDirty = this.settings.get('hasDirtyAttributes');

        if (modelIsDirty) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'AMP'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    model() {
      return this.store.createRecord('integration');
    },

    deactivate() {
      this._super(...arguments);

      this.controller.integration.rollbackAttributes();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations/search", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    model(params, transition) {
      // use the integrations controller to fetch all integrations and pick
      // out the one we want. Allows navigation back to integrations screen
      // without a loading state
      return this.controllerFor('settings.integrations').integrationModelHook('slug', 'search', this, transition);
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Search'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations/slack", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor()).then(this.settings.reload());
    },

    actions: {
      save() {
        this.controller.send('save');
      },

      willTransition(transition) {
        let controller = this.controller;
        let settings = this.settings;
        let modelIsDirty = settings.get('hasDirtyAttributes');

        if (modelIsDirty) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Slack'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations/unsplash", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings", "ghost-admin/models/unsplash-integration"], function (_exports, _authenticated, _currentUserSettings, _unsplashIntegration) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    config: Ember.inject.service(),
    settings: Ember.inject.service(),

    // reload settings to ensure we have latest values and pre-configure
    // Unsplash to be active if the server doesn't have any unsplash setting
    beforeModel() {
      this._super(...arguments);

      let {
        settings
      } = this;
      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor()).then(this.settings.reload()).then(() => {
        if (settings.get('unsplash')) {
          return;
        } // server doesn't have any unsplash settings by default but it can provide
        // overrides via config:
        // - isActive: use as default but allow settings override
        // - applicationId: total override, no field is shown if present


        let unsplash = _unsplashIntegration.default.create({
          isActive: true
        });

        settings.set('unsplash', unsplash);
      });
    },

    actions: {
      save() {
        this.controller.send('save');
      },

      willTransition(transition) {
        let controller = this.controller;
        let modelIsDirty = controller.dirtyAttributes;

        if (modelIsDirty) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Unsplash'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/integrations/zapier", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    model(params, transition) {
      // use the integrations controller to fetch all integrations and pick
      // out the one we want. Allows navigation back to integrations screen
      // without a loading state
      return this.controllerFor('settings.integrations').integrationModelHook('slug', 'zapier', this, transition);
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Zapier'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/labs", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    settings: Ember.inject.service(),

    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor()).then(this.transitionEditor());
    },

    model() {
      return this.settings.reload();
    },

    resetController(controller, isExiting) {
      if (isExiting) {
        controller.reset();
      }
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - Labs'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/tags", ["exports", "jquery", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings", "ghost-admin/mixins/shortcuts-route"], function (_exports, _jquery, _authenticated, _currentUserSettings, _shortcutsRoute) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* global key */
  var _default = _authenticated.default.extend(_currentUserSettings.default, _shortcutsRoute.default, {
    shortcuts: null,

    init() {
      this._super(...arguments);

      this.shortcuts = {
        'up, k': 'moveUp',
        'down, j': 'moveDown',
        left: 'focusList',
        right: 'focusContent',
        c: 'newTag'
      };
    },

    // authors aren't allowed to manage tags
    beforeModel() {
      this._super(...arguments);

      return this.get('session.user').then(this.transitionAuthor());
    },

    // set model to a live array so all tags are shown and created/deleted tags
    // are automatically added/removed. Also load all tags in the background,
    // pausing to show the loading spinner if no tags have been loaded yet
    model() {
      let promise = this.store.query('tag', {
        limit: 'all',
        include: 'count.posts'
      });
      let tags = this.store.peekAll('tag');

      if (this.store.peekAll('tag').get('length') === 0) {
        return promise.then(() => tags);
      } else {
        return tags;
      }
    },

    deactivate() {
      this._super(...arguments);

      if (!this.isDestroyed && !this.isDestroying) {
        this.send('resetShortcutsScope');
      }
    },

    actions: {
      moveUp() {
        if (this.controller.get('tagContentFocused')) {
          this.scrollContent(-1);
        } else {
          this.stepThroughTags(-1);
        }
      },

      moveDown() {
        if (this.controller.get('tagContentFocused')) {
          this.scrollContent(1);
        } else {
          this.stepThroughTags(1);
        }
      },

      focusList() {
        this.set('controller.keyboardFocus', 'tagList');
      },

      focusContent() {
        this.set('controller.keyboardFocus', 'tagContent');
      },

      newTag() {
        this.transitionTo('settings.tags.new');
      },

      resetShortcutsScope() {
        key.setScope('default');
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Settings - Tags'
      };
    },

    stepThroughTags(step) {
      let currentTag = this.modelFor('settings.tags.tag');
      let tags = this.get('controller.sortedTags');
      let length = tags.get('length');

      if (currentTag && length) {
        let newPosition = tags.indexOf(currentTag) + step;

        if (newPosition >= length) {
          return;
        } else if (newPosition < 0) {
          return;
        }

        this.transitionTo('settings.tags.tag', tags.objectAt(newPosition));
      }
    },

    scrollContent(amount) {
      let content = (0, _jquery.default)('.tag-settings-pane');
      let scrolled = content.scrollTop();
      content.scrollTop(scrolled + 50 * amount);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/tags/index", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    mediaQueries: Ember.inject.service(),

    beforeModel() {
      let firstTag = this.modelFor('settings.tags').get('firstObject');

      this._super(...arguments);

      if (firstTag && !this.get('mediaQueries.maxWidth600')) {
        this.transitionTo('settings.tags.tag', firstTag);
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/tags/new", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    controllerName: 'settings.tags.tag',

    model() {
      return this.store.createRecord('tag');
    },

    renderTemplate() {
      this.render('settings.tags.tag');
    },

    // reset the model so that mobile screens react to an empty selectedTag
    deactivate() {
      this._super(...arguments);

      this.set('controller.model', null);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/settings/tags/tag", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _authenticated.default.extend({
    model(params) {
      return this.store.queryRecord('tag', {
        slug: params.tag_slug
      });
    },

    serialize(model) {
      return {
        tag_slug: model.get('slug')
      };
    },

    setupController(controller, model) {
      this._super(...arguments);

      this.controllerFor('settings.tags').scrollTagIntoView(model);
    },

    // reset the model so that mobile screens react to an empty selectedTag
    deactivate() {
      this._super(...arguments);

      this.set('controller.model', null);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/setup", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    ghostPaths: Ember.inject.service(),
    session: Ember.inject.service(),
    ajax: Ember.inject.service(),
    config: Ember.inject.service(),

    // use the beforeModel hook to check to see whether or not setup has been
    // previously completed.  If it has, stop the transition into the setup page.
    beforeModel() {
      this._super(...arguments);

      if (this.get('session.isAuthenticated')) {
        return this.transitionTo('home');
      }

      let authUrl = this.get('ghostPaths.url').api('authentication', 'setup'); // check the state of the setup process via the API

      return this.ajax.request(authUrl).then(result => {
        let [setup] = result.setup;

        if (setup.status) {
          return this.transitionTo('signin');
        }
      });
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Setup',
        bodyClasses: ['unauthenticated-route']
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/signin", ["exports", "ember-data", "ghost-admin/mixins/unauthenticated-route-mixin"], function (_exports, _emberData, _unauthenticatedRouteMixin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    Errors
  } = _emberData.default;

  const defaultModel = function defaultModel() {
    return Ember.Object.create({
      identification: '',
      password: '',
      errors: Errors.create()
    });
  };

  var _default = Ember.Route.extend(_unauthenticatedRouteMixin.default, {
    model() {
      return defaultModel();
    },

    // the deactivate hook is called after a route has been exited.
    deactivate() {
      let controller = this.controllerFor('signin');

      this._super(...arguments); // clear the properties that hold the credentials when we're no longer on the signin screen


      controller.set('signin', defaultModel());
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Sign In',
        bodyClasses: ['unauthenticated-route']
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/site", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    model() {
      return new Date().valueOf();
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Site'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/staff/index", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    infinity: Ember.inject.service(),
    session: Ember.inject.service(),

    model() {
      return this.session.user;
    },

    setupController(controller) {
      this._super(...arguments);

      controller.backgroundUpdate.perform();
    },

    actions: {
      reload() {
        this.controller.backgroundUpdate.perform();
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Staff'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/staff/user", ["exports", "ghost-admin/routes/authenticated", "ghost-admin/mixins/current-user-settings"], function (_exports, _authenticated, _currentUserSettings) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _authenticated.default.extend(_currentUserSettings.default, {
    model(params) {
      return this.store.queryRecord('user', {
        slug: params.user_slug,
        include: 'count.posts,parents'
      });
    },

    afterModel(user) {
      this._super(...arguments);

      return this.get('session.user').then(currentUser => {
        let isOwnProfile = user.get('id') === currentUser.get('id');
        let isAuthorOrContributor = currentUser.get('isAuthorOrContributor');
        let isEditor = currentUser.get('isEditor');
        let isParent = user.get('parentId') === currentUser.get('id');

        if (isAuthorOrContributor && !isOwnProfile) {
          this.transitionTo('staff.user', currentUser);
        } else if (isEditor && !isOwnProfile && !isParent) {
          this.transitionTo('staff');
        }
      });
    },

    serialize(model) {
      return {
        user_slug: model.get('slug')
      };
    },

    actions: {
      didTransition() {
        this.modelFor('staff.user').get('errors').clear();
      },

      save() {
        this.get('controller.save').perform();
      },

      willTransition(transition) {
        let controller = this.controller;
        let user = controller.user;
        let dirtyAttributes = controller.dirtyAttributes;
        let modelIsDirty = user.get('hasDirtyAttributes'); // always reset the password properties on the user model when leaving

        if (user) {
          user.set('password', '');
          user.set('newPassword', '');
          user.set('ne2Password', '');
        }

        if (modelIsDirty || dirtyAttributes) {
          transition.abort();
          controller.send('toggleLeaveSettingsModal', transition);
          return;
        }
      }

    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Staff - User'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/subscribers", ["exports", "ghost-admin/routes/authenticated"], function (_exports, _authenticated) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _authenticated.default.extend({
    feature: Ember.inject.service(),

    // redirect if subscribers is disabled or user isn't owner/admin
    beforeModel() {
      this._super(...arguments);

      let promises = {
        user: this.get('session.user'),
        subscribers: this.get('feature.subscribers')
      };
      return Ember.RSVP.hash(promises).then(hash => {
        let {
          user,
          subscribers
        } = hash;

        if (!subscribers || !user.isOwnerOrAdmin) {
          return this.transitionTo('home');
        }
      });
    },

    setupController(controller) {
      this._super(...arguments);

      controller.fetchSubscribers.perform();
    },

    buildRouteInfoMetadata() {
      return {
        titleToken: 'Subscribers'
      };
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/routes/subscribers/import", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({});

  _exports.default = _default;
});
;define("ghost-admin/routes/subscribers/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Route.extend({
    model() {
      return this.store.createRecord('subscriber');
    },

    setupController(controller, model) {
      controller.set('subscriber', model);
    },

    deactivate() {
      let subscriber = this.controller.get('subscriber');

      this._super(...arguments);

      if (subscriber.get('isNew')) {
        this.rollbackModel();
      }
    },

    actions: {
      save() {
        let subscriber = this.controller.get('subscriber');
        return subscriber.save();
      },

      cancel() {
        this.rollbackModel();
        this.transitionTo('subscribers');
      }

    },

    rollbackModel() {
      let subscriber = this.controller.get('subscriber');
      subscriber.rollbackAttributes();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/api-key", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    attrs: {
      lastSeenAtUTC: {
        key: 'last_seen_at'
      },
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/application", ["exports", "ember-data/serializers/rest", "ember-inflector"], function (_exports, _rest, _emberInflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _rest.default.extend({
    serialize()
    /*snapshot, options*/
    {
      let json = this._super(...arguments); // don't send attributes that are updated automatically on the server


      delete json.created_by;
      delete json.updated_by;
      return json;
    },

    serializeIntoHash(hash, type, record, options) {
      // Our API expects an id on the posted object
      options = options || {};
      options.includeId = true; // We have a plural root in the API

      let root = (0, _emberInflector.pluralize)(type.modelName);
      let data = this.serialize(record, options);
      hash[root] = [data];
    },

    keyForAttribute(attr) {
      return Ember.String.decamelize(attr);
    },

    keyForRelationship(key, typeClass, method) {
      let transform = method === 'serialize' ? Ember.String.underscore : Ember.String.camelize;

      if (typeClass === 'belongsTo' && !key.match(/(Id|By)$/)) {
        let transformed = "".concat(transform(key), "_id");
        return transformed;
      }

      return transform(key);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/integration", ["exports", "ghost-admin/serializers/application", "ember-data/serializers/embedded-records-mixin"], function (_exports, _application, _embeddedRecordsMixin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend(_embeddedRecordsMixin.default, {
    attrs: {
      apiKeys: {
        embedded: 'always'
      },
      webhooks: {
        embedded: 'always'
      },
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/invite", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    attrs: {
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/notification", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    attrs: {
      key: {
        key: 'location'
      }
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/page", ["exports", "ghost-admin/serializers/post"], function (_exports, _post) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _post.default.extend({});

  _exports.default = _default;
});
;define("ghost-admin/serializers/post", ["exports", "ghost-admin/serializers/application", "ember-data/serializers/embedded-records-mixin", "ember-inflector"], function (_exports, _application, _embeddedRecordsMixin, _emberInflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _application.default.extend(_embeddedRecordsMixin.default, {
    // settings for the EmbeddedRecordsMixin.
    attrs: {
      authors: {
        embedded: 'always'
      },
      tags: {
        embedded: 'always'
      },
      publishedAtUTC: {
        key: 'published_at'
      },
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    },

    normalizeSingleResponse(store, primaryModelClass, payload) {
      let root = this.keyForAttribute(primaryModelClass.modelName);
      let pluralizedRoot = (0, _emberInflector.pluralize)(primaryModelClass.modelName);

      if (payload[pluralizedRoot]) {
        payload[root] = payload[pluralizedRoot][0];
        delete payload[pluralizedRoot];
      }

      return this._super(...arguments);
    },

    normalizeArrayResponse() {
      return this._super(...arguments);
    },

    serialize()
    /*snapshot, options*/
    {
      let json = this._super(...arguments); // Properties that exist on the model but we don't want sent in the payload


      delete json.uuid; // Inserted locally as a convenience.

      delete json.author_id; // Read-only virtual property.

      delete json.url; // Deprecated property (replaced with data.authors)

      delete json.author;
      return json;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/role", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    attrs: {
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/setting", ["exports", "ghost-admin/serializers/application", "ember-inflector"], function (_exports, _application, _emberInflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    serializeIntoHash(hash, type, record, options) {
      // Settings API does not want ids
      options = options || {};
      options.includeId = false;
      let root = (0, _emberInflector.pluralize)(type.modelName);
      let data = this.serialize(record, options);
      let payload = [];
      delete data.id;
      Object.keys(data).forEach(k => {
        payload.push({
          key: k,
          value: data[k]
        });
      });
      hash[root] = payload;
    },

    normalizeArrayResponse(store, primaryModelClass, _payload, id, requestType) {
      let payload = {
        settings: [this._extractObjectFromArrayPayload(_payload)]
      };
      return this._super(store, primaryModelClass, payload, id, requestType);
    },

    normalizeSingleResponse(store, primaryModelClass, _payload, id, requestType) {
      let payload = {
        setting: this._extractObjectFromArrayPayload(_payload)
      };
      return this._super(store, primaryModelClass, payload, id, requestType);
    },

    _extractObjectFromArrayPayload(_payload) {
      let payload = {
        id: '0'
      };

      _payload.settings.forEach(setting => {
        payload[setting.key] = setting.value;
      });

      return payload;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/subscriber", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    attrs: {
      unsubscribedAtUTC: {
        key: 'unsubscribed_at'
      },
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    },

    serialize() {
      let json = this._super(...arguments); // the API can't handle `status` being `null`


      if (!json.status) {
        delete json.status;
      }

      return json;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/tag", ["exports", "ghost-admin/serializers/application", "ember-inflector"], function (_exports, _application, _emberInflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _application.default.extend({
    attrs: {
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    },

    serialize()
    /*snapshot, options*/
    {
      let json = this._super(...arguments); // Properties that exist on the model but we don't want sent in the payload


      delete json.count;
      return json;
    },

    // if we use `queryRecord` ensure we grab the first record to avoid
    // DS.SERIALIZER.REST.QUERYRECORD-ARRAY-RESPONSE deprecations
    normalizeResponse(store, primaryModelClass, payload, id, requestType) {
      if (requestType === 'queryRecord') {
        let singular = primaryModelClass.modelName;
        let plural = (0, _emberInflector.pluralize)(singular);

        if (payload[plural]) {
          payload[singular] = payload[plural][0];
          delete payload[plural];
        }
      }

      return this._super(...arguments);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/theme", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    primaryKey: 'name'
  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/user", ["exports", "ghost-admin/serializers/application", "ember-data/serializers/embedded-records-mixin", "ember-inflector"], function (_exports, _application, _embeddedRecordsMixin, _emberInflector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend(_embeddedRecordsMixin.default, {
    attrs: {
      roles: {
        embedded: 'always'
      },
      lastLoginUTC: {
        key: 'last_seen'
      },
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    },

    extractSingle(store, primaryType, payload) {
      let root = this.keyForAttribute(primaryType.modelName);
      let pluralizedRoot = (0, _emberInflector.pluralize)(primaryType.modelName);
      payload[root] = payload[pluralizedRoot][0];
      delete payload[pluralizedRoot];
      return this._super(...arguments);
    },

    normalizeSingleResponse(store, primaryModelClass, payload) {
      let root = this.keyForAttribute(primaryModelClass.modelName);
      let pluralizedRoot = (0, _emberInflector.pluralize)(primaryModelClass.modelName);

      if (payload[pluralizedRoot]) {
        payload[root] = payload[pluralizedRoot][0];
        delete payload[pluralizedRoot];
      }

      return this._super(...arguments);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/serializers/webhook", ["exports", "ghost-admin/serializers/application"], function (_exports, _application) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _application.default.extend({
    attrs: {
      lastTriggeredAtUTC: {
        key: 'last_triggered_at'
      },
      createdAtUTC: {
        key: 'created_at'
      },
      updatedAtUTC: {
        key: 'updated_at'
      }
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/services/-observer-admin", ["exports", "ember-in-viewport/services/-observer-admin"], function (_exports, _observerAdmin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _observerAdmin.default;
    }
  });
});
;define("ghost-admin/services/-raf-admin", ["exports", "ember-in-viewport/services/-raf-admin"], function (_exports, _rafAdmin) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _rafAdmin.default;
    }
  });
});
;define("ghost-admin/services/ajax", ["exports", "ember-ajax/services/ajax", "ghost-admin/config/environment", "ember-ajax/errors"], function (_exports, _ajax, _environment, _errors) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.isVersionMismatchError = isVersionMismatchError;
  _exports.isServerUnreachableError = isServerUnreachableError;
  _exports.isRequestEntityTooLargeError = isRequestEntityTooLargeError;
  _exports.isUnsupportedMediaTypeError = isUnsupportedMediaTypeError;
  _exports.isMaintenanceError = isMaintenanceError;
  _exports.isThemeValidationError = isThemeValidationError;
  _exports.default = _exports.ThemeValidationError = _exports.MaintenanceError = _exports.UnsupportedMediaTypeError = _exports.RequestEntityTooLargeError = _exports.ServerUnreachableError = _exports.VersionMismatchError = void 0;
  const JSON_CONTENT_TYPE = 'application/json';
  const GHOST_REQUEST = /\/ghost\/api\//;

  function isJSONContentType(header) {
    if (!header || Ember.isNone(header)) {
      return false;
    }

    return header.indexOf(JSON_CONTENT_TYPE) === 0;
  }
  /* Version mismatch error */


  class VersionMismatchError extends _errors.AjaxError {
    constructor(payload) {
      super(payload, 'API server is running a newer version of Ghost, please upgrade.');
    }

  }

  _exports.VersionMismatchError = VersionMismatchError;

  function isVersionMismatchError(errorOrStatus, payload) {
    if ((0, _errors.isAjaxError)(errorOrStatus)) {
      return errorOrStatus instanceof VersionMismatchError;
    } else {
      return Ember.get(payload || {}, 'errors.firstObject.type') === 'VersionMismatchError';
    }
  }
  /* Server unreachable error */


  class ServerUnreachableError extends _errors.AjaxError {
    constructor(payload) {
      super(payload, 'Server was unreachable');
    }

  }

  _exports.ServerUnreachableError = ServerUnreachableError;

  function isServerUnreachableError(error) {
    if ((0, _errors.isAjaxError)(error)) {
      return error instanceof ServerUnreachableError;
    } else {
      return error === 0 || error === '0';
    }
  }
  /* Request entity too large error */


  class RequestEntityTooLargeError extends _errors.AjaxError {
    constructor(payload) {
      super(payload, 'Request is larger than the maximum file size the server allows');
    }

  }

  _exports.RequestEntityTooLargeError = RequestEntityTooLargeError;

  function isRequestEntityTooLargeError(errorOrStatus) {
    if ((0, _errors.isAjaxError)(errorOrStatus)) {
      return errorOrStatus instanceof RequestEntityTooLargeError;
    } else {
      return errorOrStatus === 413;
    }
  }
  /* Unsupported media type error */


  class UnsupportedMediaTypeError extends _errors.AjaxError {
    constructor(payload) {
      super(payload, 'Request contains an unknown or unsupported file type.');
    }

  }

  _exports.UnsupportedMediaTypeError = UnsupportedMediaTypeError;

  function isUnsupportedMediaTypeError(errorOrStatus) {
    if ((0, _errors.isAjaxError)(errorOrStatus)) {
      return errorOrStatus instanceof UnsupportedMediaTypeError;
    } else {
      return errorOrStatus === 415;
    }
  }
  /* Maintenance error */


  class MaintenanceError extends _errors.AjaxError {
    constructor(payload) {
      super(payload, 'Ghost is currently undergoing maintenance, please wait a moment then retry.');
    }

  }

  _exports.MaintenanceError = MaintenanceError;

  function isMaintenanceError(errorOrStatus) {
    if ((0, _errors.isAjaxError)(errorOrStatus)) {
      return errorOrStatus instanceof MaintenanceError;
    } else {
      return errorOrStatus === 503;
    }
  }
  /* Theme validation error */


  class ThemeValidationError extends _errors.AjaxError {
    constructor(payload) {
      super(payload, 'Theme is not compatible or contains errors.');
    }

  }

  _exports.ThemeValidationError = ThemeValidationError;

  function isThemeValidationError(errorOrStatus, payload) {
    if ((0, _errors.isAjaxError)(errorOrStatus)) {
      return errorOrStatus instanceof ThemeValidationError;
    } else {
      return Ember.get(payload || {}, 'errors.firstObject.type') === 'ThemeValidationError';
    }
  }
  /* end: custom error types */


  let ajaxService = _ajax.default.extend({
    session: Ember.inject.service(),
    isTesting: undefined,
    // flag to tell our ESA authenticator not to try an invalidate DELETE request
    // because it's been triggered by this service's 401 handling which means the
    // DELETE would fail and get stuck in an infinite loop
    // TODO: find a more elegant way to handle this
    skipSessionDeletion: false,

    get headers() {
      let headers = {};
      headers['X-Ghost-Version'] = _environment.default.APP.version;
      headers['App-Pragma'] = 'no-cache';

      if (this.session.isAuthenticated && this.isTesting) {
        headers.Authorization = 'Test';
      }

      return headers;
    },

    init() {
      this._super(...arguments);

      if (this.isTesting === undefined) {
        this.isTesting = _environment.default.environment === 'test';
      }
    },

    // ember-ajax recognises `application/vnd.api+json` as a JSON-API request
    // and formats appropriately, we want to handle `application/json` the same
    _makeRequest(hash) {
      if (isJSONContentType(hash.contentType) && hash.type !== 'GET') {
        if (typeof hash.data === 'object') {
          hash.data = JSON.stringify(hash.data);
        }
      }

      hash.withCredentials = true;
      return this._super(hash);
    },

    handleResponse(status, headers, payload, request) {
      if (this.isVersionMismatchError(status, headers, payload)) {
        return new VersionMismatchError(payload);
      } else if (this.isServerUnreachableError(status, headers, payload)) {
        return new ServerUnreachableError(payload);
      } else if (this.isRequestEntityTooLargeError(status, headers, payload)) {
        return new RequestEntityTooLargeError(payload);
      } else if (this.isUnsupportedMediaTypeError(status, headers, payload)) {
        return new UnsupportedMediaTypeError(payload);
      } else if (this.isMaintenanceError(status, headers, payload)) {
        return new MaintenanceError(payload);
      } else if (this.isThemeValidationError(status, headers, payload)) {
        return new ThemeValidationError(payload);
      }

      let isGhostRequest = GHOST_REQUEST.test(request.url);
      let isAuthenticated = this.get('session.isAuthenticated');
      let isUnauthorized = this.isUnauthorizedError(status, headers, payload);

      if (isAuthenticated && isGhostRequest && isUnauthorized) {
        this.skipSessionDeletion = true;
        this.session.invalidate();
      }

      return this._super(...arguments);
    },

    normalizeErrorResponse(status, headers, payload) {
      if (payload && typeof payload === 'object') {
        let errors = payload.error || payload.errors || payload.message || undefined;

        if (errors) {
          if (!Ember.isArray(errors)) {
            errors = [errors];
          }

          payload.errors = errors.map(function (error) {
            if (typeof error === 'string') {
              return {
                message: error
              };
            } else {
              return error;
            }
          });
        }
      }

      return this._super(status, headers, payload);
    },

    isVersionMismatchError(status, headers, payload) {
      return isVersionMismatchError(status, payload);
    },

    isServerUnreachableError(status) {
      return isServerUnreachableError(status);
    },

    isRequestEntityTooLargeError(status) {
      return isRequestEntityTooLargeError(status);
    },

    isUnsupportedMediaTypeError(status) {
      return isUnsupportedMediaTypeError(status);
    },

    isMaintenanceError(status, headers, payload) {
      return isMaintenanceError(status, payload);
    },

    isThemeValidationError(status, headers, payload) {
      return isThemeValidationError(status, payload);
    }

  }); // we need to reopen so that internal methods use the correct contentType


  ajaxService.reopen({
    contentType: 'application/json; charset=UTF-8'
  });
  var _default = ajaxService;
  _exports.default = _default;
});
;define("ghost-admin/services/clock", ["exports", "ghost-admin/config/environment", "moment"], function (_exports, _environment, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const ONE_SECOND = 1000; // Creates a clock service to run intervals.

  var _default = Ember.Service.extend({
    second: null,
    minute: null,
    hour: null,

    init() {
      this._super(...arguments);

      this.tick();
    },

    tick() {
      let now = (0, _moment.default)().utc();
      this.setProperties({
        second: now.seconds(),
        minute: now.minutes(),
        hour: now.hours()
      });

      if (_environment.default.environment !== 'test') {
        Ember.run.later(() => {
          this.tick();
        }, ONE_SECOND);
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/config", ["exports", "@tryghost/timezone-data"], function (_exports, _timezoneData) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  // ember-cli-shims doesn't export _ProxyMixin
  const {
    _ProxyMixin
  } = Ember;

  var _default = Ember.Service.extend(_ProxyMixin, {
    ajax: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    session: Ember.inject.service(),
    content: null,

    init() {
      this._super(...arguments);

      this.content = {};
    },

    fetch() {
      let promises = [];
      promises.push(this.fetchUnauthenticated());

      if (this.session.isAuthenticated) {
        promises.push(this.fetchAuthenticated());
      }

      return Ember.RSVP.all(promises);
    },

    fetchUnauthenticated() {
      let siteUrl = this.ghostPaths.url.api('site');
      return this.ajax.request(siteUrl).then((_ref) => {
        let {
          site
        } = _ref;
        // normalize url to non-trailing-slash
        site.blogUrl = site.url.replace(/\/$/, '');
        site.blogTitle = site.title;
        delete site.url;
        delete site.title;
        Object.assign(this.content, site);
      }).then(() => {
        this.notifyPropertyChange('content');
      });
    },

    fetchAuthenticated() {
      let configUrl = this.ghostPaths.url.api('config');
      return this.ajax.request(configUrl).then((_ref2) => {
        let {
          config
        } = _ref2;
        Object.assign(this.content, config);
      }).then(() => {
        this.notifyPropertyChange('content');
      });
    },

    availableTimezones: Ember.computed(function () {
      return Ember.RSVP.resolve(_timezoneData.default);
    }),
    blogDomain: Ember.computed('blogUrl', function () {
      let blogUrl = this.get('blogUrl');
      let blogDomain = blogUrl.replace(/^https?:\/\//, '').replace(/\/?$/, '');
      return blogDomain;
    })
  });

  _exports.default = _default;
});
;define("ghost-admin/services/cookies", ["exports", "ember-cookies/services/cookies"], function (_exports, _cookies) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _cookies.default;
  _exports.default = _default;
});
;define("ghost-admin/services/drag-coordinator", ["exports", "ember-drag-drop/services/drag-coordinator"], function (_exports, _dragCoordinator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = _dragCoordinator.default;
  _exports.default = _default;
});
;define("ghost-admin/services/dropdown", ["exports", "jquery", "ghost-admin/mixins/body-event-listener"], function (_exports, _jquery, _bodyEventListener) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // This is used by the dropdown initializer to manage closing & toggling
  var _default = Ember.Service.extend(Ember.Evented, _bodyEventListener.default, {
    bodyClick(event) {
      let dropdownSelector = '.ember-basic-dropdown-trigger, .ember-basic-dropdown-content';

      if ((0, _jquery.default)(event.target).closest(dropdownSelector).length <= 0) {
        this.closeDropdowns();
      }
    },

    closeDropdowns() {
      this.trigger('close');
    },

    toggleDropdown(dropdownName, dropdownButton) {
      this.trigger('toggle', {
        target: dropdownName,
        button: dropdownButton
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/ember-load-config", ["exports", "ember-load/services/ember-load-config", "ghost-admin/config/environment"], function (_exports, _emberLoadConfig, _environment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const userConfig = _environment.default['ember-load'] || {};

  var _default = _emberLoadConfig.default.extend({
    loadingIndicatorClass: userConfig.loadingIndicatorClass
  });

  _exports.default = _default;
});
;define("ghost-admin/services/event-bus", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend(Ember.Evented, {
    publish() {
      return this.trigger(...arguments);
    },

    subscribe() {
      return this.on(...arguments);
    },

    unsubscribe() {
      return this.off(...arguments);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/feature", ["exports", "jquery"], function (_exports, _jquery) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.feature = feature;
  _exports.default = void 0;

  function feature(name) {
    let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    let {
      user,
      onChange
    } = options;
    let watchedProps = user ? ["accessibility.".concat(name)] : ["config.".concat(name), "labs.".concat(name)];
    return Ember.computed.apply(Ember, watchedProps.concat({
      get() {
        let enabled = false;

        if (user) {
          enabled = this.get("accessibility.".concat(name));
        } else if (this.get("config.".concat(name))) {
          enabled = this.get("config.".concat(name));
        } else {
          enabled = this.get("labs.".concat(name)) || false;
        }

        if (options.developer) {
          enabled = enabled && this.get('config.enableDeveloperExperiments');
        }

        return enabled;
      },

      set(key, value) {
        this.update(key, value, options);

        if (onChange) {
          // value must be passed here because the value isn't set until
          // the setter function returns
          this.get(onChange).bind(this)(value);
        }

        return value;
      }

    }));
  }

  var _default = Ember.Service.extend({
    store: Ember.inject.service(),
    config: Ember.inject.service(),
    session: Ember.inject.service(),
    settings: Ember.inject.service(),
    notifications: Ember.inject.service(),
    lazyLoader: Ember.inject.service(),
    publicAPI: feature('publicAPI'),
    subscribers: feature('subscribers'),
    members: feature('members'),
    nightShift: feature('nightShift', {
      user: true,
      onChange: '_setAdminTheme'
    }),
    _user: null,
    labs: Ember.computed('settings.labs', function () {
      let labs = this.get('settings.labs');

      try {
        return JSON.parse(labs) || {};
      } catch (e) {
        return {};
      }
    }),
    accessibility: Ember.computed('_user.accessibility', function () {
      let accessibility = this.get('_user.accessibility');

      try {
        return JSON.parse(accessibility) || {};
      } catch (e) {
        return {};
      }
    }),

    fetch() {
      return Ember.RSVP.hash({
        settings: this.settings.fetch(),
        user: this.get('session.user')
      }).then((_ref) => {
        let {
          user
        } = _ref;
        this.set('_user', user);
        return this._setAdminTheme().then(() => true);
      });
    },

    update(key, value) {
      let options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      let serviceProperty = options.user ? 'accessibility' : 'labs';
      let model = this.get(options.user ? '_user' : 'settings');
      let featureObject = this.get(serviceProperty); // set the new key value for either the labs property or the accessibility property

      Ember.set(featureObject, key, value); // update the 'labs' or 'accessibility' key of the model

      model.set(serviceProperty, JSON.stringify(featureObject));
      return model.save().then(() => {
        // return the labs key value that we get from the server
        this.notifyPropertyChange(serviceProperty);
        return this.get("".concat(serviceProperty, ".").concat(key));
      }).catch(error => {
        model.rollbackAttributes();
        this.notifyPropertyChange(serviceProperty); // we'll always have an errors object unless we hit a
        // validation error

        if (!error) {
          throw new Ember.Error("Validation of the feature service ".concat(options.user ? 'user' : 'settings', " model failed when updating ").concat(serviceProperty, "."));
        }

        this.notifications.showAPIError(error);
        return this.get("".concat(serviceProperty, ".").concat(key));
      });
    },

    _setAdminTheme(enabled) {
      let nightShift = enabled;

      if (typeof nightShift === 'undefined') {
        nightShift = enabled || this.nightShift;
      }

      return this.lazyLoader.loadStyle('dark', 'assets/ghost-dark.css', true).then(() => {
        (0, _jquery.default)('link[title=dark]').prop('disabled', !nightShift);
        (0, _jquery.default)('link[title=light]').prop('disabled', nightShift);
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/ghost-paths", ["exports", "ghost-admin/utils/ghost-paths"], function (_exports, _ghostPaths) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend((0, _ghostPaths.default)());

  _exports.default = _default;
});
;define("ghost-admin/services/infinity", ["exports", "ember-infinity/services/infinity"], function (_exports, _infinity) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _infinity.default;
    }
  });
});
;define("ghost-admin/services/koenig-drag-drop-handler", ["exports", "koenig-editor/services/koenig-drag-drop-handler"], function (_exports, _koenigDragDropHandler) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigDragDropHandler.default;
    }
  });
});
;define("ghost-admin/services/koenig-ui", ["exports", "koenig-editor/services/koenig-ui"], function (_exports, _koenigUi) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _koenigUi.default;
    }
  });
});
;define("ghost-admin/services/lazy-loader", ["exports", "ghost-admin/config/environment"], function (_exports, _environment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend({
    ajax: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    // This is needed so we can disable it in unit tests
    testing: undefined,
    scriptPromises: null,

    init() {
      this._super(...arguments);

      this.scriptPromises = {};

      if (this.testing === undefined) {
        this.testing = _environment.default.environment === 'test';
      }
    },

    loadScript(key, url) {
      if (this.testing) {
        return Ember.RSVP.resolve();
      }

      if (this.scriptPromises[key]) {
        return this.scriptPromises[key];
      }

      let scriptPromise = new Ember.RSVP.Promise((resolve, reject) => {
        let {
          adminRoot
        } = this.ghostPaths;
        let script = document.createElement('script');
        script.type = 'text/javascript';
        script.async = true;
        script.src = "".concat(adminRoot).concat(url);
        let el = document.getElementsByTagName('script')[0];
        el.parentNode.insertBefore(script, el);
        script.addEventListener('load', () => {
          resolve();
        });
        script.addEventListener('error', () => {
          reject(new Error("".concat(url, " failed to load")));
        });
      });
      this.scriptPromises[key] = scriptPromise;
      return scriptPromise;
    },

    loadStyle(key, url) {
      let alternate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

      if (this.testing || document.querySelector("#".concat(key, "-styles"))) {
        return Ember.RSVP.resolve();
      }

      return new Ember.RSVP.Promise((resolve, reject) => {
        let link = document.createElement('link');
        link.id = "".concat(key, "-styles");
        link.rel = alternate ? 'alternate stylesheet' : 'stylesheet';
        link.href = "".concat(this.ghostPaths.adminRoot).concat(url);

        link.onload = () => {
          link.onload = null;

          if (alternate) {
            // If stylesheet is alternate and we disable the stylesheet before injecting into the DOM,
            // the onload handler never gets called. Thus, we should disable the link after it has finished loading
            link.disabled = true;
          }

          resolve();
        };

        link.onerror = reject;

        if (alternate) {
          link.title = key;
        }

        document.querySelector('head').appendChild(link);
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/liquid-fire-transitions", ["exports", "liquid-wormhole/services/liquid-fire-transitions"], function (_exports, _liquidFireTransitions) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidFireTransitions.default;
    }
  });
});
;define("ghost-admin/services/liquid-wormhole", ["exports", "liquid-wormhole/services/liquid-wormhole"], function (_exports, _liquidWormhole) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _liquidWormhole.default;
    }
  });
});
;define("ghost-admin/services/media-queries", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const MEDIA_QUERIES = {
    maxWidth600: '(max-width: 600px)',
    isMobile: '(max-width: 800px)',
    maxWidth900: '(max-width: 900px)',
    maxWidth1000: '(max-width: 1000px)'
  };

  var _default = Ember.Service.extend(Ember.Evented, {
    init() {
      this._super(...arguments);

      this._handlers = [];
      this.loadQueries(MEDIA_QUERIES);
    },

    loadQueries(queries) {
      Object.keys(queries).forEach(key => {
        this.loadQuery(key, queries[key]);
      });
    },

    loadQuery(key, queryString) {
      let query = window.matchMedia(queryString);
      this.set(key, query.matches);
      let handler = Ember.run.bind(this, () => {
        let lastValue = this.get(key);
        let newValue = query.matches;

        if (lastValue !== newValue) {
          this.set(key, newValue);
          this.trigger('change', key, newValue);
        }
      });
      query.addListener(handler);

      this._handlers.push([query, handler]);
    },

    willDestroy() {
      this._handlers.forEach((_ref) => {
        let [query, handler] = _ref;
        query.removeListener(handler);
      });

      this._super(...arguments);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/media", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // dummy service to account for not having the ember-responsive dependency
  // available for ember-light-table (we don't use it so no need for the dep)
  // see https://github.com/offirgolan/ember-light-table/issues/576
  var _default = Ember.Service.extend({});

  _exports.default = _default;
});
;define("ghost-admin/services/moment", ["exports", "ember-moment/services/moment", "ghost-admin/config/environment"], function (_exports, _moment, _environment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    get
  } = Ember;

  var _default = _moment.default.extend({
    defaultFormat: get(_environment.default, 'moment.outputFormat')
  });

  _exports.default = _default;
});
;define("ghost-admin/services/notifications", ["exports", "ghost-admin/services/ajax"], function (_exports, _ajax) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // Notification keys take the form of "noun.verb.message", eg:
  //
  // "invite.resend.api-error"
  // "user.invite.already-invited"
  //
  // The "noun.verb" part will be used as the "key base" in duplicate checks
  // to avoid stacking of multiple error messages whilst leaving enough
  // specificity to re-use keys for i18n lookups
  var _default = Ember.Service.extend({
    delayedNotifications: null,
    content: null,

    init() {
      this._super(...arguments);

      this.delayedNotifications = Ember.A();
      this.content = Ember.A();
    },

    upgradeStatus: Ember.inject.service(),
    alerts: Ember.computed.filter('content', function (notification) {
      let status = Ember.get(notification, 'status');
      return status === 'alert';
    }),
    notifications: Ember.computed.filter('content', function (notification) {
      let status = Ember.get(notification, 'status');
      return status === 'notification';
    }),

    handleNotification(message, delayed) {
      // If this is an alert message from the server, treat it as html safe
      if (typeof message.toJSON === 'function' && message.get('status') === 'alert') {
        message.set('message', Ember.String.htmlSafe(message.get('message')));
      }

      if (!Ember.get(message, 'status')) {
        Ember.set(message, 'status', 'notification');
      } // close existing duplicate alerts/notifications to avoid stacking


      if (Ember.get(message, 'key')) {
        this._removeItems(Ember.get(message, 'status'), Ember.get(message, 'key'));
      } // close existing alerts/notifications which have the same text to avoid stacking


      let newText = Ember.get(message, 'message').string || Ember.get(message, 'message');
      this.set('content', this.content.reject(notification => {
        let existingText = Ember.get(notification, 'message').string || Ember.get(notification, 'message');
        return existingText === newText;
      }));

      if (!delayed) {
        this.content.pushObject(message);
      } else {
        this.delayedNotifications.pushObject(message);
      }
    },

    showAlert(message, options) {
      options = options || {};
      this.handleNotification({
        message,
        status: 'alert',
        type: options.type,
        key: options.key
      }, options.delayed);
    },

    showNotification(message, options) {
      options = options || {};

      if (!options.doNotCloseNotifications) {
        this.closeNotifications();
      } else {
        // TODO: this should be removed along with showErrors
        options.key = undefined;
      }

      this.handleNotification({
        message,
        status: 'notification',
        type: options.type,
        key: options.key
      }, options.delayed);
    },

    showAPIError(resp, options) {
      // handle "global" errors
      if ((0, _ajax.isVersionMismatchError)(resp)) {
        return this.upgradeStatus.requireUpgrade();
      } else if ((0, _ajax.isMaintenanceError)(resp)) {
        return this.upgradeStatus.maintenanceAlert();
      } // loop over ember-ajax errors object


      if (resp && resp.payload && Ember.isArray(resp.payload.errors)) {
        return resp.payload.errors.forEach(error => {
          this._showAPIError(error, options);
        });
      }

      this._showAPIError(resp, options);
    },

    _showAPIError(resp, options) {
      options = options || {};
      options.type = options.type || 'error'; // if possible use the title to get a unique key
      // - we only show one alert for each key so if we get multiple errors
      //   only the last one will be shown

      if (!options.key && !Ember.isBlank(Ember.get(resp, 'title'))) {
        options.key = Ember.String.dasherize(Ember.get(resp, 'title'));
      }

      options.key = ['api-error', options.key].compact().join('.');
      let msg = options.defaultErrorText || 'There was a problem on the server, please try again.';

      if (resp instanceof String) {
        msg = resp;
      } else if (!Ember.isBlank(Ember.get(resp, 'detail'))) {
        msg = resp.detail;
      } else if (!Ember.isBlank(Ember.get(resp, 'message'))) {
        msg = resp.message;
      }

      if (!Ember.isBlank(Ember.get(resp, 'context'))) {
        msg = "".concat(msg, " ").concat(Ember.get(resp, 'context'));
      }

      this.showAlert(msg, options);
    },

    displayDelayed() {
      this.delayedNotifications.forEach(message => {
        this.content.pushObject(message);
      });
      this.set('delayedNotifications', []);
    },

    closeNotification(notification) {
      let content = this.content;

      if (typeof notification.toJSON === 'function') {
        notification.deleteRecord();
        notification.save().finally(() => {
          content.removeObject(notification);
        });
      } else {
        content.removeObject(notification);
      }
    },

    closeNotifications(key) {
      this._removeItems('notification', key);
    },

    closeAlerts(key) {
      this._removeItems('alert', key);
    },

    clearAll() {
      this.content.clear();
    },

    _removeItems(status, key) {
      if (key) {
        let keyBase = this._getKeyBase(key); // TODO: keys should only have . special char but we should
        // probably use a better regexp escaping function/polyfill


        let escapedKeyBase = keyBase.replace('.', '\\.');
        let keyRegex = new RegExp("^".concat(escapedKeyBase));
        this.set('content', this.content.reject(item => {
          let itemKey = Ember.get(item, 'key');
          let itemStatus = Ember.get(item, 'status');
          return itemStatus === status && itemKey && itemKey.match(keyRegex);
        }));
      } else {
        this.set('content', this.content.rejectBy('status', status));
      }
    },

    // take a key and return the first two elements, eg:
    // "invite.revoke.failed" => "invite.revoke"
    _getKeyBase(key) {
      return key.split('.').slice(0, 2).join('.');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/power-calendar", ["exports", "ember-power-calendar/services/power-calendar"], function (_exports, _powerCalendar) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _powerCalendar.default;
    }
  });
});
;define("ghost-admin/services/rc-services", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend({
    ajax: Ember.inject.service(),
    ghostPaths: Ember.inject.service(),
    session: Ember.inject.service(),

    init() {
      this._super(...arguments);
    },

    getRoom(query) {
      let url = this.get('ghostPaths.url').api('rcapi');
      return this.ajax.request(url, {
        data: query
      });
    },

    getUser(username) {
      const query = {
        uname: username
      };
      let url = this.get('ghostPaths.url').api('rcapi');
      return this.ajax.request(url, {
        data: query
      });
    },

    addUser(username, role) {
      let url = this.get('ghostPaths.url').api('authentication', 'adduser');
      const pid = this.get('session.user.id');
      return this.ajax.post(url, {
        dataType: 'json',
        user: {
          id: '1'
        },
        data: {
          user: [{
            rc_username: username,
            role: role,
            created_by: pid
          }]
        }
      });
    },

    createDiscussion(title) {
      let type = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'c';
      const url = this.get('ghostPaths.url').api('rcapi', 'discussion');
      return this.ajax.post(url, {
        dataType: 'json',
        data: {
          room: [{
            title,
            type
          }]
        }
      });
    },

    collaborate(userId, postId, post) {
      let url = this.get('ghostPaths.url').api('rcapi', 'collaborate');
      return this.ajax.post(url, {
        dataType: 'json',
        data: {
          collaboration: [{
            rc_id: userId,
            post_id: postId,
            post: post
          }]
        }
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/resize-detector", ["exports", "element-resize-detector"], function (_exports, _elementResizeDetector) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend({
    init() {
      this._super(...arguments);

      this.detector = (0, _elementResizeDetector.default)({
        strategy: 'scroll'
      });
    },

    setup(selector, callback) {
      let element = document.querySelector(selector);

      if (!element) {
        // eslint-disable-next-line
        console.error("service:resize-detector - could not find element matching ".concat(selector));
      }

      this.detector.listenTo(element, callback);
    },

    teardown(selector, callback) {
      let element = document.querySelector(selector);

      if (element) {
        this.detector.removeListener(element, callback);
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/session", ["exports", "ember-simple-auth/services/session"], function (_exports, _session) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _session.default.extend({
    feature: Ember.inject.service(),
    dataStore: Ember.inject.service('store'),
    // SessionService.store already exists
    tour: Ember.inject.service(),
    user: Ember.computed(function () {
      return this.dataStore.queryRecord('user', {
        id: 'me'
      });
    }),

    authenticate() {
      // ensure any cached this.user value is removed and re-fetched
      this.notifyPropertyChange('user');
      return this._super(...arguments).then(authResult => {
        // TODO: remove duplication with application.afterModel
        let preloadPromises = [this.feature.fetch(), this.tour.fetchViewed()];
        return Ember.RSVP.all(preloadPromises).then(() => authResult);
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/settings", ["exports", "ghost-admin/mixins/validation-engine"], function (_exports, _validationEngine) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  // ember-cli-shims doesn't export _ProxyMixin
  const {
    _ProxyMixin
  } = Ember;

  var _default = Ember.Service.extend(_ProxyMixin, _validationEngine.default, {
    store: Ember.inject.service(),
    // will be set to the single Settings model, it's a reference so any later
    // changes to the settings object in the store will be reflected
    content: null,
    validationType: 'setting',
    _loadingPromise: null,
    // this is an odd case where we only want to react to changes that we get
    // back from the API rather than local updates
    settledIcon: '',

    // the settings API endpoint is a little weird as it's singular and we have
    // to pass in all types - if we ever fetch settings without all types then
    // save we have problems with the missing settings being removed or reset
    _loadSettings() {
      if (!this._loadingPromise) {
        this._loadingPromise = this.store.queryRecord('setting', {
          type: 'blog,rc_settings,theme,private,members'
        }).then(settings => {
          this._loadingPromise = null;
          return settings;
        });
      }

      return this._loadingPromise;
    },

    fetch() {
      if (!this.content) {
        return this.reload();
      } else {
        return Ember.RSVP.resolve(this);
      }
    },

    reload() {
      return this._loadSettings().then(settings => {
        this.set('content', settings);
        this.set('settledIcon', Ember.get(settings, 'icon'));
        return this;
      });
    },

    save() {
      let settings = this.content;

      if (!settings) {
        return false;
      }

      return settings.save().then(settings => {
        this.set('settledIcon', Ember.get(settings, 'icon'));
        return settings;
      });
    },

    rollbackAttributes() {
      return this.content.rollbackAttributes();
    },

    changedAttributes() {
      return this.content.changedAttributes();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/slug-generator", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const {
    resolve
  } = Ember.RSVP;

  var _default = Ember.Service.extend({
    ghostPaths: Ember.inject.service(),
    ajax: Ember.inject.service(),

    generateSlug(slugType, textToSlugify) {
      let url;

      if (!textToSlugify) {
        return resolve('');
      }

      url = this.get('ghostPaths.url').api('slugs', slugType, encodeURIComponent(textToSlugify));
      return this.ajax.request(url).then(response => {
        let [firstSlug] = response.slugs;
        let {
          slug
        } = firstSlug;
        return slug;
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/text-measurer", ["exports", "ember-text-measurer/services/text-measurer"], function (_exports, _textMeasurer) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _textMeasurer.default;
    }
  });
});
;define("ghost-admin/services/tour", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend(Ember.Evented, {
    ghostPaths: Ember.inject.service(),
    session: Ember.inject.service(),
    // this service is responsible for managing tour item visibility and syncing
    // the viewed state with the server
    //
    // tour items need to be centrally defined here so that we have a single
    // source of truth for marking all tour items as viewed
    //
    // a {{gh-tour-item "unique-id"}} component can be inserted in any template,
    // this will use the tour service to grab content and determine visibility
    // with the component in control of rendering the throbber/controlling the
    // modal - this allows the component lifecycle hooks to perform automatic
    // display/cleanup when the relevant UI is visible.
    viewed: null,
    // IDs should **NOT** be changed if they have been part of a release unless
    // the re-display of the throbber should be forced. In that case it may be
    // useful to add a version number eg. `my-feature` -> `my-feature-v2`.
    // Format is as follows:
    //
    // {
    //     id: 'test',
    //     title: 'This is a test',
    //     message: 'This is a test of our <strong>feature tour</strong> feature'
    // }
    //
    // TODO: it may be better to keep this configuration elsewhere to keep the
    // service clean. Eventually we'll want apps to be able to register their
    // own throbbers and tour content
    throbbers: null,

    init() {
      this._super(...arguments);

      let adminUrl = "".concat(window.location.origin).concat(this.get('ghostPaths.url').admin());
      let adminDisplayUrl = adminUrl.replace("".concat(window.location.protocol, "//"), '');
      this.viewed = [];
      this.throbbers = [{
        id: 'getting-started',
        title: 'Getting started with Ghost',
        message: "Welcome to Ghost Admin! From here you can browse your site, manage your content, and edit your settings.<br><br>You can always login to Ghost Admin by visiting <a href=\"".concat(adminUrl, "\" target=\"_blank\">").concat(adminDisplayUrl, "</a>")
      }, {
        id: 'using-the-editor',
        title: 'Using the Ghost editor',
        message: 'Ghost uses Markdown to allow you to write and format content quickly and easily. This toolbar also helps! Hit the <strong>?</strong> icon for more editor shortcuts.'
      }, {
        id: 'featured-post',
        title: 'Setting a featured post',
        message: 'Depending on your theme, featured posts receive special styling to make them stand out as a particularly important or emphasised story.'
      }, {
        id: 'upload-a-theme',
        title: 'Customising your publication',
        message: 'Using custom themes you can completely control the look and feel of your site to suit your brand. Here\'s a full guide to help: <strong><a href="https://docs.ghost.org/api/handlebars-themes/" target="_blank">https://docs.ghost.org/api/handlebars-themes/</a></strong>'
      }];
    },

    _activeThrobbers: Ember.computed('viewed.[]', 'throbbers.[]', function () {
      // return throbbers that haven't been viewed
      let viewed = this.viewed;
      let throbbers = this.throbbers;
      return throbbers.reject(throbber => viewed.includes(throbber.id));
    }),

    // retrieve the IDs of the viewed throbbers from the server, always returns
    // a promise
    fetchViewed() {
      return this.get('session.user').then(user => {
        let viewed = user.get('tour') || [];
        this.set('viewed', viewed);
        return viewed;
      });
    },

    // save the list of viewed throbbers to the server overwriting the
    // entire list
    syncViewed() {
      let viewed = this.viewed;
      return this.get('session.user').then(user => {
        user.set('tour', viewed);
        return user.save();
      });
    },

    // returns throbber content for a given ID only if that throbber hasn't been
    // viewed. Used by the {{gh-tour-item}} component to determine visibility
    activeThrobber(id) {
      let activeThrobbers = this._activeThrobbers;
      return activeThrobbers.findBy('id', id);
    },

    // when a throbber is opened the component will call this method to mark
    // it as viewed and sync with the server. Always returns a promise
    markThrobberAsViewed(id) {
      let viewed = this.viewed;

      if (!viewed.includes(id)) {
        viewed.pushObject(id);
        this.trigger('viewed', id);
        return this.syncViewed();
      } else {
        return Ember.RSVP.resolve(true);
      }
    },

    // opting-out will use the list of IDs defined in this file making it the
    // single-source-of-truth and allowing future client updates to control when
    // new UI should be surfaced through tour items
    optOut() {
      let allThrobberIds = this.throbbers.mapBy('id');
      this.set('viewed', allThrobberIds);
      this.trigger('optOut');
      return this.syncViewed();
    },

    // this is not used anywhere at the moment but it's useful to use via ember
    // inspector as a reset mechanism
    reEnable() {
      this.set('viewed', []);
      return this.syncViewed();
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/ui", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  function collectMetadataClasses(transition, prop) {
    let oldClasses = [];
    let newClasses = [];
    let {
      from,
      to
    } = transition;

    while (from) {
      oldClasses = oldClasses.concat(Ember.get(from, "metadata.".concat(prop)) || []);
      from = from.parent;
    }

    while (to) {
      newClasses = newClasses.concat(Ember.get(to, "metadata.".concat(prop)) || []);
      to = to.parent;
    }

    return {
      oldClasses,
      newClasses
    };
  }

  function updateBodyClasses(transition) {
    let {
      body
    } = document;
    let {
      oldClasses,
      newClasses
    } = collectMetadataClasses(transition, 'bodyClasses');
    oldClasses.forEach(oldClass => {
      body.classList.remove(oldClass);
    });
    newClasses.forEach(newClass => {
      body.classList.add(newClass);
    });
  }

  var _default = Ember.Service.extend({
    config: Ember.inject.service(),
    dropdown: Ember.inject.service(),
    mediaQueries: Ember.inject.service(),
    router: Ember.inject.service(),
    isFullScreen: false,
    showMobileMenu: false,
    showSettingsMenu: false,
    mainClass: '',
    hasSideNav: Ember.computed.not('isSideNavHidden'),
    isMobile: Ember.computed.reads('mediaQueries.isMobile'),
    isSideNavHidden: Ember.computed.or('isFullScreen', 'isMobile'),

    init() {
      this._super(...arguments);

      this.router.on('routeDidChange', transition => {
        updateBodyClasses(transition);
        this.updateDocumentTitle();
        let {
          newClasses: mainClasses
        } = collectMetadataClasses(transition, 'mainClasses');
        this.set('mainClass', mainClasses.join(' '));
      });
    },

    closeMenus() {
      this.dropdown.closeDropdowns();
      this.setProperties({
        showSettingsMenu: false,
        showMobileMenu: false
      });
    },

    closeMobileMenu() {
      this.set('showMobileMenu', false);
    },

    openMobileMenu() {
      this.set('showMobileMenu', true);
    },

    openSettingsMenu() {
      this.set('showSettingsMenu', true);
    },

    updateDocumentTitle() {
      let {
        currentRoute
      } = this.router;
      let tokens = [];

      while (currentRoute) {
        let titleToken = Ember.get(currentRoute, 'metadata.titleToken');

        if (typeof titleToken === 'function') {
          titleToken = titleToken();
        }

        if (titleToken) {
          tokens.unshift(titleToken);
        }

        currentRoute = currentRoute.parent;
      }

      let blogTitle = this.config.get('blogTitle');

      if (!Ember.isEmpty(tokens)) {
        window.document.title = "".concat(tokens.join(' - '), " - ").concat(blogTitle);
      } else {
        window.document.title = blogTitle;
      }
    },

    actions: {
      closeMenus() {
        this.closeMenus();
      },

      closeMobileMenu() {
        this.closeMobileMenu();
      },

      openMobileMenu() {
        this.openMobileMenu();
      },

      openSettingsMenu() {
        this.openSettingsMenu();
      }

    }
  });

  _exports.default = _default;
});
;define("ghost-admin/services/unsplash", ["exports", "fetch", "ember-concurrency"], function (_exports, _fetch, _emberConcurrency) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const API_URL = 'https://api.unsplash.com';
  const API_VERSION = 'v1';
  const DEBOUNCE_MS = 600;

  var _default = Ember.Service.extend({
    config: Ember.inject.service(),
    settings: Ember.inject.service(),
    columnCount: 3,
    columns: null,
    error: '',
    photos: null,
    searchTerm: '',
    _columnHeights: null,
    _pagination: null,
    applicationId: '8672af113b0a8573edae3aa3713886265d9bb741d707f6c01a486cde8c278980',
    isLoading: Ember.computed.or('_search.isRunning', '_loadingTasks.isRunning'),

    init() {
      this._super(...arguments);

      this._reset();

      this.loadNew();
    },

    loadNew() {
      this._reset();

      return this._loadNew.perform();
    },

    loadNextPage() {
      // protect against scroll trigger firing when the photos are reset
      if (this.get('_search.isRunning')) {
        return;
      }

      if (Ember.isEmpty(this.photos)) {
        return this._loadNew.perform();
      }

      if (this._pagination.next) {
        return this._loadNextPage.perform();
      } // TODO: return error?


      return Ember.RSVP.reject();
    },

    updateSearch(term) {
      if (term === this.searchTerm) {
        return;
      }

      this.set('searchTerm', term);

      this._reset();

      if (term) {
        return this._search.perform(term);
      } else {
        return this._loadNew.perform();
      }
    },

    retryLastRequest() {
      return this._retryLastRequest.perform();
    },

    changeColumnCount(newColumnCount) {
      if (newColumnCount !== this.columnCount) {
        this.set('columnCount', newColumnCount);

        this._resetColumns();
      }
    },

    // let Unsplash know that the photo was inserted
    // https://medium.com/unsplash/unsplash-api-guidelines-triggering-a-download-c39b24e99e02
    triggerDownload(photo) {
      if (photo.links.download_location) {
        this._makeRequest(photo.links.download_location, {
          ignoreErrors: true
        });
      }
    },

    actions: {
      updateSearch(term) {
        return this.updateSearch(term);
      }

    },
    _loadingTasks: (0, _emberConcurrency.taskGroup)().drop(),
    _loadNew: (0, _emberConcurrency.task)(function* () {
      let url = "".concat(API_URL, "/photos?per_page=30");
      yield this._makeRequest(url);
    }).group('_loadingTasks'),
    _loadNextPage: (0, _emberConcurrency.task)(function* () {
      yield this._makeRequest(this._pagination.next);
    }).group('_loadingTasks'),
    _retryLastRequest: (0, _emberConcurrency.task)(function* () {
      yield this._makeRequest(this._lastRequestUrl);
    }).group('_loadingTasks'),
    _search: (0, _emberConcurrency.task)(function* (term) {
      yield (0, _emberConcurrency.timeout)(DEBOUNCE_MS);
      let url = "".concat(API_URL, "/search/photos?query=").concat(term, "&per_page=30");
      yield this._makeRequest(url);
    }).restartable(),

    _addPhotosFromResponse(response) {
      let photos = response.results || response;
      photos.forEach(photo => this._addPhoto(photo));
    },

    _addPhoto(photo) {
      // pre-calculate ratio for later use
      photo.ratio = photo.height / photo.width; // add to general photo list

      this.photos.pushObject(photo); // add to least populated column

      this._addPhotoToColumns(photo);
    },

    _addPhotoToColumns(photo) {
      let min = Math.min(...this._columnHeights);

      let columnIndex = this._columnHeights.indexOf(min); // use a fixed width when calculating height to compensate for different
      // overall image sizes


      this._columnHeights[columnIndex] += 300 * photo.ratio;
      this.columns[columnIndex].pushObject(photo);
    },

    _reset() {
      this.set('photos', []);
      this._pagination = {};

      this._resetColumns();
    },

    _resetColumns() {
      let columns = [];
      let columnHeights = []; // pre-fill column arrays based on columnCount

      for (let i = 0; i < this.columnCount; i += 1) {
        columns[i] = [];
        columnHeights[i] = 0;
      }

      this.set('columns', columns);
      this._columnHeights = columnHeights;

      if (!Ember.isEmpty(this.photos)) {
        this.photos.forEach(photo => {
          this._addPhotoToColumns(photo);
        });
      }
    },

    _makeRequest(url) {
      let _options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      let defaultOptions = {
        ignoreErrors: false
      };
      let headers = {};
      let options = {};
      Ember.assign(options, defaultOptions, _options); // clear any previous error

      this.set('error', ''); // store the url so it can be retried if needed

      this._lastRequestUrl = url;
      headers.Authorization = "Client-ID ".concat(this.applicationId);
      headers['Accept-Version'] = API_VERSION;
      headers['App-Pragma'] = 'no-cache';
      headers['X-Unsplash-Cache'] = true;
      return (0, _fetch.default)(url, {
        headers
      }).then(response => this._checkStatus(response)).then(response => this._extractPagination(response)).then(response => response.json()).then(response => this._addPhotosFromResponse(response)).catch(() => {
        // if the error text isn't already set then we've get a connection error from `fetch`
        if (!options.ignoreErrors && !this.error) {
          this.set('error', 'Uh-oh! Trouble reaching the Unsplash API, please check your connection');
        }
      });
    },

    _checkStatus(response) {
      // successful request
      if (response.status >= 200 && response.status < 300) {
        return Ember.RSVP.resolve(response);
      }

      let errorText = '';
      let responseTextPromise = Ember.RSVP.resolve();

      if (response.headers.map['content-type'] === 'application/json') {
        responseTextPromise = response.json().then(json => json.errors[0]);
      } else if (response.headers.map['content-type'] === 'text/xml') {
        responseTextPromise = response.text();
      }

      return responseTextPromise.then(responseText => {
        if (response.status === 403 && response.headers.map['x-ratelimit-remaining'] === '0') {
          // we've hit the ratelimit on the API
          errorText = 'Unsplash API rate limit reached, please try again later.';
        }

        errorText = errorText || responseText || "Error ".concat(response.status, ": Uh-oh! Trouble reaching the Unsplash API"); // set error text for display in UI

        this.set('error', errorText); // throw error to prevent further processing

        let error = new Error(errorText);
        error.response = response;
        throw error;
      });
    },

    _extractPagination(response) {
      let pagination = {};
      let linkRegex = new RegExp('<(.*)>; rel="(.*)"');
      let {
        link
      } = response.headers.map;

      if (link) {
        link.split(',').forEach(link => {
          let [, url, rel] = linkRegex.exec(link);
          pagination[rel] = url;
        });
      }

      this._pagination = pagination;
      return response;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/upgrade-status", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Service.extend({
    notifications: Ember.inject.service(),
    isRequired: false,
    message: '',

    // called when notifications are fetched during app boot for notifications
    // where the `location` is not 'top' and `custom` is false
    handleUpgradeNotification(notification) {
      let message = Ember.get(notification, 'message');
      Ember.set(this, 'message', Ember.String.htmlSafe(message));
    },

    // called when a MaintenanceError is encountered
    maintenanceAlert() {
      this.notifications.showAlert('Sorry, Ghost is currently undergoing maintenance, please wait a moment then try again.', {
        type: 'error',
        key: 'api-error.under-maintenance'
      });
    },

    // called when a VersionMismatchError is encountered
    requireUpgrade() {
      Ember.set(this, 'isRequired', true);
      this.notifications.showAlert('Ghost has been upgraded, please copy any unsaved data and refresh the page to continue.', {
        type: 'error',
        key: 'api-error.upgrade-required'
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/services/user-agent", ["exports", "ember-useragent/services/user-agent"], function (_exports, _userAgent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _userAgent.default;
    }
  });
});
;define("ghost-admin/session-stores/application", ["exports", "ember-simple-auth/session-stores/ephemeral"], function (_exports, _ephemeral) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  // Ghost already uses a cookie to store it's session so we don't need to keep
  // track of any other peristent login state separately in Ember Simple Auth
  var _default = _ephemeral.default.extend({
    session: Ember.inject.service(),

    // when loading the app we want ESA to try fetching the currently logged
    // in user. This will succeed/fail depending on whether we have a valid
    // session cookie or not so we can use that as an indication of the session
    // being authenticated
    restore() {
      return this.session.user.then(() => {
        // provide the necessary data for internal-session to mark the
        // session as authenticated
        let data = {
          authenticated: {
            authenticator: 'authenticator:cookie'
          }
        };
        this.persist(data);
        return data;
      }).catch(() => {
        // ensure the session.user doesn't return the same rejected promise
        // after a succussful login
        this.session.notifyPropertyChange('user');
        return Ember.RSVP.reject();
      });
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/templates/-contributors", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "mmbf1o10",
    "block": "{\"symbols\":[],\"statements\":[[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/JohnONolan\"],[11,\"title\",\"JohnONolan\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/JohnONolan.jpeg\"],[11,\"alt\",\"JohnONolan\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/ErisDS\"],[11,\"title\",\"ErisDS\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/ErisDS.jpeg\"],[11,\"alt\",\"ErisDS\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/sebgie\"],[11,\"title\",\"sebgie\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/sebgie.png\"],[11,\"alt\",\"sebgie\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/frantzypants\"],[11,\"title\",\"frantzypants\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/frantzypants.jpeg\"],[11,\"alt\",\"frantzypants\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/cobbspur\"],[11,\"title\",\"cobbspur\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/cobbspur.jpeg\"],[11,\"alt\",\"cobbspur\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/kevinansfield\"],[11,\"title\",\"kevinansfield\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/kevinansfield.jpeg\"],[11,\"alt\",\"kevinansfield\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/GeorginaLusby\"],[11,\"title\",\"GeorginaLusby\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/GeorginaLusby.jpeg\"],[11,\"alt\",\"GeorginaLusby\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/AileenCGN\"],[11,\"title\",\"AileenCGN\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/AileenCGN.jpeg\"],[11,\"alt\",\"AileenCGN\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/kirrg001\"],[11,\"title\",\"kirrg001\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/kirrg001.jpeg\"],[11,\"alt\",\"kirrg001\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/disordinary\"],[11,\"title\",\"disordinary\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/disordinary.jpeg\"],[11,\"alt\",\"disordinary\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/acburdine\"],[11,\"title\",\"acburdine\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/acburdine.jpeg\"],[11,\"alt\",\"acburdine\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/dbalders\"],[11,\"title\",\"dbalders\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/dbalders.jpeg\"],[11,\"alt\",\"dbalders\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/felixrieseberg\"],[11,\"title\",\"felixrieseberg\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/felixrieseberg.jpeg\"],[11,\"alt\",\"felixrieseberg\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/jaswilli\"],[11,\"title\",\"jaswilli\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/jaswilli.jpeg\"],[11,\"alt\",\"jaswilli\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/halfdan\"],[11,\"title\",\"halfdan\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/halfdan.jpeg\"],[11,\"alt\",\"halfdan\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/kevinkucharczyk\"],[11,\"title\",\"kevinkucharczyk\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/kevinkucharczyk.jpeg\"],[11,\"alt\",\"kevinkucharczyk\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/mixonic\"],[11,\"title\",\"mixonic\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/mixonic.png\"],[11,\"alt\",\"mixonic\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/rwjblue\"],[11,\"title\",\"rwjblue\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/rwjblue.jpeg\"],[11,\"alt\",\"rwjblue\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/tgriesser\"],[11,\"title\",\"tgriesser\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/tgriesser.png\"],[11,\"alt\",\"tgriesser\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"article\"],[9],[0,\"\\n    \"],[7,\"a\"],[11,\"href\",\"https://github.com/bnookala\"],[11,\"title\",\"bnookala\"],[9],[0,\"\\n        \"],[7,\"img\"],[11,\"src\",\"assets/img/contributors/bnookala.jpeg\"],[11,\"alt\",\"bnookala\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/-contributors.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/about", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "3zsP7CoQ",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas js-settings-content\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[1,[29,\"svg-jar\",[\"ghost-logo\"],[[\"class\",\"alt\"],[\"gh-logo\",\"Ghost\"]]],false],[0,\"\\n    \"],[10],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"gh-env-details\"],[9],[0,\"\\n            \"],[7,\"ul\"],[11,\"class\",\"gh-env-list\"],[9],[0,\"\\n                \"],[7,\"li\"],[11,\"class\",\"gh-env-list-version\"],[9],[7,\"strong\"],[9],[0,\"Version\"],[10],[0,\" \"],[1,[25,[\"config\",\"version\"]],false],[10],[0,\"\\n                \"],[7,\"li\"],[9],[7,\"strong\"],[9],[0,\"Environment\"],[10],[0,\" \"],[1,[25,[\"config\",\"environment\"]],false],[10],[0,\"\\n                \"],[7,\"li\"],[11,\"class\",\"gh-env-list-database-type\"],[9],[7,\"strong\"],[9],[0,\"Database\"],[10],[0,\" \"],[1,[25,[\"config\",\"database\"]],false],[10],[0,\"\\n                \"],[7,\"li\"],[9],[7,\"strong\"],[9],[0,\"Mail\"],[10],[0,\" \"],[4,\"if\",[[25,[\"config\",\"mail\"]]],null,{\"statements\":[[1,[25,[\"config\",\"mail\"]],false]],\"parameters\":[]},{\"statements\":[[0,\"Native\"]],\"parameters\":[]}],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-env-help\"],[9],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"gh-btn\"],[11,\"href\",\"https://docs.ghost.org\"],[11,\"target\",\"_blank\"],[9],[7,\"span\"],[9],[0,\"User Documentation\"],[10],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"gh-btn\"],[11,\"href\",\"https://forum.ghost.org/\"],[11,\"target\",\"_blank\"],[9],[7,\"span\"],[9],[0,\"Get Help With Ghost\"],[10],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"upgradeStatus\",\"message\"]]],null,{\"statements\":[[0,\"            \"],[7,\"section\"],[11,\"class\",\"gh-upgrade-notification\"],[9],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"\\n                    \"],[7,\"strong\"],[9],[0,\"Update available!\"],[10],[0,\" \"],[1,[25,[\"upgradeStatus\",\"message\"]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"gh-credits\"],[9],[0,\"\\n            \"],[7,\"h2\"],[9],[0,\"The People Who Made it Possible\"],[10],[0,\"\\n\\n            \"],[7,\"section\"],[11,\"class\",\"gh-contributors\"],[9],[0,\"\\n                \"],[16,\"contributors\",[]],[0,\"\\n            \"],[10],[0,\"\\n\\n            \"],[7,\"p\"],[9],[0,\"Ghost is built by an incredible group of contributors from all over the world. Here are just a few of the people who helped create the version you’re using right now.\"],[10],[0,\"\\n\\n            \"],[7,\"a\"],[11,\"class\",\"gh-btn gh-btn-blue\"],[11,\"href\",\"https://docs.ghost.org/concepts/contributing/\"],[11,\"target\",\"_blank\"],[9],[7,\"span\"],[9],[0,\"Find out how you can get involved\"],[10],[10],[0,\"\\n\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"footer\"],[11,\"class\",\"gh-copyright-info\"],[9],[0,\"\\n            Copyright © 2013 – \"],[1,[23,\"copyrightYear\"],false],[0,\" Ghost Foundation, released under the \"],[7,\"a\"],[11,\"href\",\"https://github.com/TryGhost/Ghost/blob/master/LICENSE\"],[11,\"target\",\"_blank\"],[9],[0,\"MIT license\"],[10],[0,\".\\n            \"],[7,\"br\"],[9],[10],[0,\"\\n            \"],[7,\"a\"],[11,\"href\",\"https://ghost.org/\"],[11,\"target\",\"_blank\"],[9],[0,\"Ghost\"],[10],[0,\" is a registered trademark of \"],[7,\"a\"],[11,\"href\",\"https://ghost.org/trademark/\"],[11,\"target\",\"_blank\"],[9],[0,\"Ghost Foundation Ltd\"],[10],[0,\".\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":true}",
    "meta": {
      "moduleName": "ghost-admin/templates/about.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/application-error", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "vCpaqF1l",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-view\"],[9],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"error-content error-404 js-error-container\"],[9],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"error-details\"],[9],[0,\"\\n            \"],[7,\"img\"],[11,\"class\",\"error-ghost\"],[11,\"src\",\"assets/img/404-ghost@2x.png\"],[11,\"srcset\",\"assets/img/404-ghost.png 1x, assets/img/404-ghost@2x.png 2x\"],[9],[10],[0,\"\\n            \"],[7,\"section\"],[11,\"class\",\"error-message\"],[9],[0,\"\\n                \"],[7,\"h1\"],[11,\"class\",\"error-code\"],[9],[1,[25,[\"model\",\"code\"]],false],[10],[0,\"\\n                \"],[7,\"h2\"],[11,\"class\",\"error-description\"],[9],[0,\"\\n                    \"],[1,[29,\"or\",[[25,[\"model\",\"payload\",\"errors\",\"firstObject\",\"message\"]],[25,[\"model\",\"message\"]]],null],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[1,[23,\"ember-load-remover\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/application-error.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/application", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "DHCXYrhi",
    "block": "{\"symbols\":[],\"statements\":[[4,\"gh-app\",null,[[\"showSettingsMenu\"],[[25,[\"ui\",\"showSettingsMenu\"]]]],{\"statements\":[[0,\"    \"],[4,\"gh-skip-link\",null,[[\"anchor\"],[\".gh-main\"]],{\"statements\":[[0,\"Skip to main content\"]],\"parameters\":[]},null],[0,\"\\n\\n    \"],[1,[23,\"gh-alerts\"],false],[0,\"\\n\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-viewport \",[29,\"if\",[[25,[\"ui\",\"showSettingsMenu\"]],\"settings-menu-expanded\"],null],\" \",[29,\"if\",[[25,[\"ui\",\"showMobileMenu\"]],\"mobile-menu-expanded\"],null]]]],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"showNavMenu\"]]],null,{\"statements\":[[0,\"            \"],[1,[29,\"gh-nav-menu\",null,[[\"icon\"],[[25,[\"settings\",\"settledIcon\"]]]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"main\"],[12,\"class\",[30,[\"gh-main \",[24,0,[\"ui\",\"mainClass\"]]]]],[11,\"role\",\"main\"],[9],[0,\"\\n            \"],[1,[23,\"outlet\"],false],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[1,[23,\"gh-notifications\"],false],[0,\"\\n\\n        \"],[1,[23,\"gh-content-cover\"],false],[0,\"\\n\\n        \"],[1,[23,\"gh-mobile-nav-bar\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[1,[23,\"ember-load-remover\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/application.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/aspect-ratio-box", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "wZ9phQgq",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"unless\",[[25,[\"isResizing\"]]],null,{\"statements\":[[15,1],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/aspect-ratio-box.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/draggable-object-target", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "zUDfWt5g",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[25,[\"enableClicking\"]]],null,{\"statements\":[[0,\"  \"],[7,\"a\"],[11,\"href\",\"#\"],[9],[0,\"\\n    \"],[15,1],[0,\"\\n  \"],[3,\"action\",[[24,0,[]],\"acceptForDrop\"]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"  \"],[15,1],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/draggable-object-target.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/draggable-object", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "yoaUBpl0",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[25,[\"enableClicking\"]]],null,{\"statements\":[[0,\"  \"],[7,\"a\"],[11,\"href\",\"#\"],[9],[0,\"\\n    \"],[15,1],[0,\"\\n  \"],[3,\"action\",[[24,0,[]],\"selectForDrag\"]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"  \"],[15,1],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/draggable-object.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-activating-list-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "TuUjZS0g",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"link-to\",null,[[\"alternateActive\",\"class\",\"route\"],[[29,\"action\",[[24,0,[]],\"setActive\"],null],[29,\"concat\",[[29,\"if\",[[25,[\"linkClasses\"]],[29,\"-normalize-class\",[\"linkClasses\",[25,[\"linkClasses\"]]],null]],null],\" \"],null],[25,[\"route\"]]]],{\"statements\":[[1,[23,\"title\"],false],[15,1]],\"parameters\":[]},null],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-activating-list-item.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-alert", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "3tiZ9rJw",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-alert-content\"],[9],[0,\"\\n    \"],[1,[25,[\"message\",\"message\"]],false],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"button\"],[11,\"class\",\"gh-alert-close\"],[11,\"data-test-button\",\"close-notification\"],[9],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeNotification\"]],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-alert.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-alerts", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "XxzWxmqB",
    "block": "{\"symbols\":[\"message\"],\"statements\":[[4,\"each\",[[25,[\"messages\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-alert\",null,[[\"message\"],[[24,1,[]]]]],false],[0,\"\\n\"]],\"parameters\":[1]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-alerts.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-app", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "pgxuTCw1",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-app.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-blog-url", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "/wzDp4yD",
    "block": "{\"symbols\":[],\"statements\":[[1,[25,[\"config\",\"blogUrl\"]],true]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-blog-url.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-cm-editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "Ibyh3ebY",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-textarea\",null,[[\"class\",\"value\",\"input\"],[[29,\"concat\",[\"gh-cm-editor-textarea \",[25,[\"textareaClass\"]]],null],[29,\"readonly\",[[25,[\"_value\"]]],null],[29,\"action\",[[24,0,[]],\"updateFromTextarea\"],[[\"value\"],[\"target.value\"]]]]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-cm-editor.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-date-time-picker", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "XY3Xp5Wq",
    "block": "{\"symbols\":[\"dp\"],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-date-time-picker\"],[9],[0,\"\\n\"],[4,\"power-datepicker\",null,[[\"selected\",\"center\",\"onSelect\",\"renderInPlace\",\"disabled\"],[[25,[\"_date\"]],[25,[\"_date\"]],[29,\"action\",[[24,0,[]],\"setDate\"],[[\"value\"],[\"date\"]]],true,[25,[\"disabled\"]]]],{\"statements\":[[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"trigger\"]],\"expected `dp.trigger` to be a contextual component but found a string. Did you mean `(component dp.trigger)`? ('ghost-admin/templates/components/gh-date-time-picker.hbs' @ L9:C11) \"],null]],[[\"tabindex\",\"data-test-date-time-picker-datepicker\"],[\"-1\",true]],{\"statements\":[[0,\"            \"],[7,\"div\"],[12,\"class\",[30,[\"gh-date-time-picker-date \",[29,\"if\",[[25,[\"dateError\"]],\"error\"],null]]]],[9],[0,\"\\n                \"],[7,\"input\"],[11,\"readonly\",\"\"],[12,\"value\",[29,\"moment-format\",[[25,[\"_date\"]],\"MM/DD/YYYY\"],null]],[12,\"disabled\",[23,\"disabled\"]],[11,\"data-test-date-time-picker-date-input\",\"\"],[11,\"type\",\"text\"],[9],[10],[0,\"\\n                \"],[1,[29,\"svg-jar\",[\"calendar\"],null],false],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"content\"]],\"expected `dp.content` to be a contextual component but found a string. Did you mean `(component dp.content)`? ('ghost-admin/templates/components/gh-date-time-picker.hbs' @ L15:C11) \"],null]],[[\"class\"],[\"dropdown-menu\"]],{\"statements\":[[0,\"            \"],[1,[24,1,[\"nav\"]],false],[0,\"\\n            \"],[1,[29,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"days\"]],\"expected `dp.days` to be a contextual component but found a string. Did you mean `(component dp.days)`? ('ghost-admin/templates/components/gh-date-time-picker.hbs' @ L17:C14) \"],null]],[[\"minDate\",\"maxDate\",\"weekdayFormat\"],[[25,[\"_minDate\"]],[25,[\"_maxDate\"]],\"min\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[1]},null],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-date-time-picker-time \",[29,\"if\",[[25,[\"hasError\"]],\"error\"],null]]]],[9],[0,\"\\n        \"],[7,\"input\"],[12,\"value\",[23,\"_time\"]],[12,\"disabled\",[23,\"disabled\"]],[12,\"oninput\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"_time\"]]],null]],[[\"value\"],[\"target.value\"]]]],[12,\"onblur\",[29,\"action\",[[24,0,[]],\"setTime\",[25,[\"_time\"]]],null]],[11,\"data-test-date-time-picker-time-input\",\"\"],[11,\"type\",\"text\"],[9],[10],[0,\"\\n        \"],[7,\"small\"],[11,\"class\",\"gh-date-time-picker-timezone\"],[9],[0,\"(\"],[1,[23,\"timezone\"],false],[0,\")\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"hasError\"]]],null,{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"gh-date-time-picker-error\"],[11,\"data-test-date-time-picker-error\",\"\"],[9],[1,[23,\"dateError\"],false],[1,[23,\"timeError\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-date-time-picker.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-download-count", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "ZxbJ7YzU",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[26,1]],null,{\"statements\":[[0,\"    \"],[15,1,[[25,[\"count\"]]]],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[1,[23,\"count\"],false],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-download-count.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-editor-post-status", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "cPYNPYr9",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"_isSaving\"]]],null,{\"statements\":[[0,\"    Saving...\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"isPublished\"]]],null,{\"statements\":[[0,\"    Published\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"isScheduled\"]]],null,{\"statements\":[[0,\"    Scheduled\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"isNew\"]]],null,{\"statements\":[[0,\"    New\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    Draft\\n\"]],\"parameters\":[]}]],\"parameters\":[]}]],\"parameters\":[]}]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-editor-post-status.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "5uBcI9ad",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1,[[29,\"hash\",null,[[\"headerClass\",\"headerHeight\",\"isDraggedOver\",\"isFullScreen\",\"droppedFiles\",\"uploadedImageUrls\",\"imageMimeTypes\",\"imageExtensions\",\"toggleFullScreen\",\"togglePreview\",\"toggleSplitScreen\",\"uploadComplete\",\"uploadCancelled\",\"uploadImages\"],[[25,[\"headerClass\"]],[25,[\"headerHeight\"]],[25,[\"isDraggedOver\"]],[25,[\"isFullScreen\"]],[25,[\"droppedFiles\"]],[25,[\"uploadedImageUrls\"]],[25,[\"imageMimeTypes\"]],[25,[\"imageExtensions\"]],[29,\"action\",[[24,0,[]],\"toggleFullScreen\"],null],[29,\"action\",[[24,0,[]],\"togglePreview\"],null],[29,\"action\",[[24,0,[]],\"toggleSplitScreen\"],null],[29,\"action\",[[24,0,[]],\"uploadComplete\"],null],[29,\"action\",[[24,0,[]],\"uploadCancelled\"],null],[29,\"action\",[[24,0,[]],\"uploadImages\"],null]]]]]],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-editor.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-error-message", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "bM7Oz41E",
    "block": "{\"symbols\":[],\"statements\":[[1,[23,\"message\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-error-message.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-feature-flag", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "pFMcY3sw",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[7,\"input\"],[12,\"checked\",[23,\"value\"]],[12,\"disabled\",[23,\"disabled\"]],[12,\"id\",[23,\"for\"]],[12,\"name\",[23,\"name\"]],[12,\"onclick\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"value\"]]],null]],[[\"value\"],[\"target.checked\"]]]],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n\"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n\"],[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-feature-flag.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-file-upload", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "YRrRcUx9",
    "block": "{\"symbols\":[],\"statements\":[[7,\"input\"],[11,\"class\",\"gh-input gh-input-file q\"],[11,\"data-url\",\"upload\"],[11,\"name\",\"importfile\"],[12,\"accept\",[30,[[23,\"acceptEncoding\"]]]],[11,\"type\",\"file\"],[9],[10],[0,\"\\n\"],[7,\"button\"],[11,\"id\",\"startupload\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[12,\"disabled\",[23,\"uploadButtonDisabled\"]],[11,\"type\",\"submit\"],[9],[0,\"\\n    \"],[7,\"span\"],[9],[1,[23,\"uploadButtonText\"],false],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"upload\"]],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-file-upload.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-file-uploader", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "QMgU6gE0",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"file\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"failureMessage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"failed\"],[9],[1,[23,\"failureMessage\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[7,\"div\"],[11,\"class\",\"progress-container\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"progress\"],[9],[0,\"\\n            \"],[7,\"div\"],[12,\"class\",[30,[\"bar \",[29,\"if\",[[25,[\"failureMessage\"]],\"fail\"],null]]]],[12,\"style\",[23,\"progressStyle\"]],[9],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"failureMessage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-green\"],[11,\"data-test-upload-try-again-button\",\"\"],[9],[7,\"span\"],[9],[0,\"Try Again\"],[10],[3,\"action\",[[24,0,[]],\"reset\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"upload-form\"],[9],[0,\"\\n\"],[4,\"gh-file-input\",null,[[\"multiple\",\"alt\",\"action\",\"accept\"],[false,[25,[\"labelText\"]],[29,\"action\",[[24,0,[]],\"fileSelected\"],null],[25,[\"accept\"]]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"description\"],[9],[1,[23,\"labelText\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-file-uploader.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-fullscreen-modal", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "TanRBVgo",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"liquid-wormhole\",null,[[\"class\"],[\"fullscreen-modal-container\"]],{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"fullscreen-modal-background\"],[9],[3,\"action\",[[24,0,[]],\"clickOverlay\"]],[10],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[23,\"modalClasses\"]],[9],[0,\"\\n\"],[4,\"if\",[[26,1]],null,{\"statements\":[[0,\"          \"],[15,1],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"          \"],[1,[29,\"component\",[[25,[\"modalPath\"]]],[[\"model\",\"confirm\",\"closeModal\"],[[25,[\"model\"]],[29,\"action\",[[24,0,[]],\"confirm\"],null],[29,\"action\",[[24,0,[]],\"close\"],null]]]],false],[0,\"\\n\"]],\"parameters\":[]}],[0,\"    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-fullscreen-modal.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-image-uploader-with-preview", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "DgOSLPBo",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"image\"]]],null,{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"gh-image-uploader -with-image\"],[9],[0,\"\\n        \"],[7,\"div\"],[9],[7,\"img\"],[12,\"src\",[23,\"image\"]],[9],[10],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"image-cancel\"],[11,\"title\",\"Delete\"],[9],[0,\"\\n            \"],[1,[29,\"svg-jar\",[\"trash\"],null],false],[0,\"\\n            \"],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Delete\"],[10],[0,\"\\n        \"],[3,\"action\",[[24,0,[]],\"remove\"]],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[1,[29,\"gh-image-uploader\",null,[[\"text\",\"altText\",\"allowUnsplash\",\"update\",\"uploadStarted\",\"uploadFinished\"],[[25,[\"text\"]],[25,[\"altText\"]],[25,[\"allowUnsplash\"]],[29,\"action\",[[24,0,[]],\"update\"],null],[29,\"action\",[[24,0,[]],\"uploadStarted\"],null],[29,\"action\",[[24,0,[]],\"uploadFinished\"],null]]]],false],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-image-uploader-with-preview.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-image-uploader", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "+x6mQdBd",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"file\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"failureMessage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"failed\"],[9],[1,[23,\"failureMessage\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[7,\"div\"],[11,\"class\",\"progress-container\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"progress\"],[9],[0,\"\\n            \"],[7,\"div\"],[12,\"class\",[30,[\"bar \",[29,\"if\",[[25,[\"failureMessage\"]],\"fail\"],null]]]],[12,\"style\",[23,\"progressStyle\"]],[9],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"failureMessage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-green\"],[9],[7,\"span\"],[9],[0,\"Try Again\"],[10],[3,\"action\",[[24,0,[]],\"reset\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"upload-form\"],[9],[0,\"\\n\"],[4,\"gh-file-input\",null,[[\"multiple\",\"alt\",\"action\",\"accept\"],[false,[25,[\"description\"]],[29,\"action\",[[24,0,[]],\"fileSelected\"],null],[25,[\"accept\"]]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-btn gh-btn-outline\"],[11,\"data-test-file-input-description\",\"\"],[9],[7,\"span\"],[9],[1,[23,\"description\"],false],[10],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[25,[\"allowUnsplash\"]],[25,[\"settings\",\"unsplash\",\"isActive\"]]],null]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-image-uploader-unsplash\"],[9],[0,\"\\n                \"],[1,[29,\"svg-jar\",[\"unsplash\"],null],false],[0,\"\\n            \"],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"_showUnsplash\",[24,0,[]]],null]]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"\\n\"],[4,\"if\",[[25,[\"_showUnsplash\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-unsplash\",null,[[\"select\",\"close\"],[[29,\"action\",[[24,0,[]],\"addUnsplashPhoto\"],null],[29,\"action\",[[24,0,[]],[29,\"toggle\",[\"_showUnsplash\",[24,0,[]]],null]],null]]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-image-uploader.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-infinity-loader", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "LCClMows",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[26,1]],null,{\"statements\":[[0,\"    \"],[15,1,[[25,[\"infinityModelContent\"]]]],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"isDoneLoading\"]]],null,{\"statements\":[],\"parameters\":[]},{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"gh-loading-spinner\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-infinity-loader.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-koenig-editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "vcCXMjub",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-koenig-editor-pane flex flex-column mih-100\"],[12,\"onmousedown\",[29,\"action\",[[24,0,[]],\"trackMousedown\"],null]],[12,\"onmouseup\",[29,\"action\",[[24,0,[]],\"focusEditor\"],null]],[9],[0,\"\\n    \"],[1,[29,\"gh-textarea\",null,[[\"class\",\"placeholder\",\"tabindex\",\"autoExpand\",\"value\",\"input\",\"focus-out\",\"keyDown\",\"didCreateTextarea\",\"data-test-editor-title-input\"],[\"gh-editor-title\",[25,[\"titlePlaceholder\"]],\"1\",\".gh-koenig-editor\",[29,\"readonly\",[[25,[\"title\"]]],null],[29,\"action\",[[24,0,[]],\"onTitleChange\"],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"onTitleFocusOut\"],null],[29,\"action\",[[24,0,[]],\"onTitleKeydown\"],null],[29,\"action\",[[24,0,[]],\"onTitleCreated\"],null],true]]],false],[0,\"\\n\\n    \"],[1,[29,\"koenig-editor\",null,[[\"mobiledoc\",\"placeholder\",\"autofocus\",\"spellcheck\",\"onChange\",\"didCreateEditor\",\"cursorDidExitAtTop\",\"headerOffset\",\"dropTargetSelector\",\"scrollContainerSelector\",\"scrollOffsetTopSelector\",\"scrollOffsetBottomSelector\",\"wordCountDidChange\"],[[25,[\"body\"]],[25,[\"bodyPlaceholder\"]],[25,[\"bodyAutofocus\"]],true,[29,\"action\",[[24,0,[]],\"onBodyChange\"],null],[29,\"action\",[[24,0,[]],\"onEditorCreated\"],null],[29,\"action\",[[24,0,[]],\"focusTitle\"],null],[25,[\"headerOffset\"]],\".gh-koenig-editor-pane\",[25,[\"scrollContainerSelector\"]],[25,[\"scrollOffsetTopSelector\"]],[25,[\"scrollOffsetBottomSelector\"]],[29,\"action\",[[24,0,[]],[25,[\"onWordCountChange\"]]],null]]]],false],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-koenig-editor.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-loading-spinner", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "vWWGOgQi",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"showSpinner\"]]],null,{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"gh-loading-content\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-loading-spinner\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-loading-spinner.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-markdown-editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "8b7Z4Ly1",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1,[[29,\"hash\",null,[[\"editor\",\"isFullScreen\",\"isSplitScreen\",\"focus\"],[[29,\"component\",[\"gh-simplemde\"],[[\"value\",\"placeholder\",\"autofocus\",\"onChange\",\"onFocus\",\"onBlur\",\"onEditorInit\",\"options\"],[[25,[\"markdown\"]],[25,[\"placeholder\"]],[25,[\"autofocus\"]],[29,\"action\",[[24,0,[]],\"updateMarkdown\"],null],[29,\"action\",[[24,0,[]],\"updateFocusState\",true],null],[29,\"action\",[[24,0,[]],\"updateFocusState\",false],null],[29,\"action\",[[24,0,[]],\"setEditor\"],null],[25,[\"simpleMDEOptions\"]]]]],[25,[\"_isFullScreen\"]],[25,[\"_isSplitScreen\"]],[29,\"action\",[[24,0,[]],\"focusEditor\"],null]]]]]],[0,\"\\n\\n\"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n    \"],[1,[29,\"gh-file-input\",null,[[\"multiple\",\"action\",\"accept\"],[true,[29,\"action\",[[24,0,[]],[25,[\"onImageFilesSelected\"]]],null],[25,[\"imageMimeTypes\"]]]]],false],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"_showUnsplash\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-unsplash\",null,[[\"select\",\"close\"],[[29,\"action\",[[24,0,[]],\"insertUnsplashPhoto\"],null],[29,\"action\",[[24,0,[]],\"toggleUnsplash\"],null]]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showMarkdownHelp\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"markdown-help\"],[[\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"toggleMarkdownHelp\"],null],\"wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-markdown-editor.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-member-avatar", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "wR/Mwnr0",
    "block": "{\"symbols\":[\"&attrs\"],\"statements\":[[8,\"div\"],[13,\"class\",\"flex items-center justify-center br-100\"],[13,\"style\",[24,0,[\"backgroundStyle\"]]],[14,1],[9],[0,\"\\n    \"],[7,\"span\"],[12,\"class\",[30,[\"db white \",[24,0,[\"initialsClass\"]]]]],[9],[1,[24,0,[\"initials\"]],false],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-member-avatar.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-mobile-nav-bar", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "cNWEFhpn",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"link-to\",null,[[\"data-test-mobile-nav\",\"route\",\"model\"],[\"new-post\",\"editor.new\",\"post\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"pen\"],null],false],[0,\"New post\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"router\",\"currentRouteName\"]],\"posts.index\"],null]],null,{\"statements\":[[0,\"    \"],[4,\"link-to\",null,[[\"classNames\",\"data-test-mobile-nav\",\"query\",\"route\"],[\"active\",\"posts\",[29,\"hash\",null,[[\"type\"],[null]]],\"posts\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"content\"],null],false],[0,\"Posts\"]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[4,\"link-to\",null,[[\"route\"],[\"posts\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"content\"],[[\"data-test-mobile-nav\"],[\"posts\"]]],false],[0,\"Posts\"]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]}],[4,\"link-to\",null,[[\"classNames\",\"data-test-mobile-nav\",\"route\"],[\"gh-nav-main-users\",\"staff\",\"staff\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"account-group\"],null],false],[0,\"Staff\"]],\"parameters\":[]},null],[0,\"\\n\"],[7,\"div\"],[11,\"class\",\"gh-mobile-nav-bar-more\"],[9],[1,[29,\"svg-jar\",[\"icon\"],[[\"class\"],[\"icon-gh\"]]],false],[0,\"More\"],[3,\"action\",[[24,0,[]],\"openMobileMenu\"],[[\"target\",\"data-test-mobile-nav\"],[[25,[\"ui\"]],\"more\"]]],[10],[0,\"\\n\"],[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-mobile-nav-bar.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-mobiledoc", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "mTpgSmrN",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-mobiledoc.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-modal-dialog", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "VJSXUj7h",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[7,\"div\"],[11,\"class\",\"modal-container js-modal-container\"],[9],[0,\"\\n    \"],[7,\"article\"],[12,\"class\",[30,[[23,\"klass\"],\" js-modal\"]]],[9],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"modal-content\"],[9],[0,\"\\n            \"],[4,\"if\",[[25,[\"title\"]]],null,{\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[7,\"h1\"],[9],[1,[23,\"title\"],false],[10],[10]],\"parameters\":[]},null],[0,\"\\n            \"],[4,\"if\",[[25,[\"showClose\"]]],null,{\"statements\":[[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"section\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n                \"],[15,1],[0,\"\\n            \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"confirm\"]]],null,{\"statements\":[[0,\"            \"],[7,\"footer\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n\"],[0,\"                \"],[7,\"button\"],[12,\"class\",[30,[[23,\"rejectButtonClass\"],\" btn-minor\"]]],[11,\"data-test-modal-reject-button\",\"\"],[11,\"type\",\"button\"],[9],[1,[25,[\"confirm\",\"reject\",\"text\"]],false],[3,\"action\",[[24,0,[]],\"confirm\",\"reject\"]],[10],[7,\"button\"],[12,\"class\",[30,[[23,\"acceptButtonClass\"]]]],[11,\"data-test-modal-accept-button\",\"\"],[11,\"type\",\"button\"],[9],[1,[25,[\"confirm\",\"accept\",\"text\"]],false],[3,\"action\",[[24,0,[]],\"confirm\",\"accept\"]],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[3,\"action\",[[24,0,[]],\"noBubble\"],[[\"bubbles\",\"preventDefault\"],[false,false]]],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\"],[7,\"div\"],[11,\"class\",\"modal-background js-modal-background\"],[9],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-modal-dialog.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-nav-menu", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "SEtjvwgx",
    "block": "{\"symbols\":[\"menuItem\",\"dropdown\",\"menuItem\"],\"statements\":[[4,\"gh-basic-dropdown\",null,[[\"horizontalPosition\",\"calculatePosition\"],[\"right\",[25,[\"userDropdownPosition\"]]]],{\"statements\":[[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,2,[\"trigger\"]],\"expected `dropdown.trigger` to be a contextual component but found a string. Did you mean `(component dropdown.trigger)`? ('ghost-admin/templates/components/gh-nav-menu.hbs' @ L2:C7) \"],null]],[[\"tagName\",\"class\"],[\"header\",\"gh-nav-menu\"]],{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"gh-nav-menu-icon\"],[12,\"style\",[23,\"iconStyle\"]],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-nav-menu-details\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-nav-menu-details-blog\"],[9],[1,[25,[\"config\",\"blogTitle\"]],false],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-nav-menu-details-user\"],[9],[1,[25,[\"session\",\"user\",\"name\"]],false],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,2,[\"content\"]],\"expected `dropdown.content` to be a contextual component but found a string. Did you mean `(component dropdown.content)`? ('ghost-admin/templates/components/gh-nav-menu.hbs' @ L10:C7) \"],null]],[[\"class\"],[\"gh-nav-menu-dropdown\"]],{\"statements\":[[0,\"        \"],[7,\"ul\"],[11,\"class\",\"dropdown-menu dropdown-triangle-top\"],[11,\"role\",\"menu\"],[9],[0,\"\\n            \"],[7,\"li\"],[11,\"role\",\"presentation\"],[9],[4,\"link-to\",null,[[\"classNames\",\"role\",\"tabindex\",\"data-test-nav\",\"route\"],[\"dropdown-item\",\"menuitem\",\"-1\",\"about\",\"about\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"store\"],null],false],[0,\" About Ghost\"]],\"parameters\":[]},null],[10],[0,\"\\n            \"],[7,\"li\"],[11,\"class\",\"divider\"],[9],[10],[0,\"\\n            \"],[7,\"li\"],[11,\"role\",\"presentation\"],[9],[4,\"link-to\",null,[[\"classNames\",\"role\",\"tabindex\",\"data-test-nav\",\"route\",\"model\"],[\"dropdown-item\",\"menuitem\",\"-1\",\"user-profile\",\"staff.user\",[25,[\"session\",\"user\",\"slug\"]]]],{\"statements\":[[1,[29,\"svg-jar\",[\"user-circle\"],null],false],[0,\" Your Profile\"]],\"parameters\":[]},null],[10],[0,\"\\n            \"],[7,\"li\"],[11,\"role\",\"presentation\"],[9],[7,\"a\"],[11,\"class\",\"dropdown-item\"],[11,\"role\",\"menuitem\"],[11,\"tabindex\",\"-1\"],[11,\"href\",\"https://docs.ghost.org/\"],[11,\"target\",\"_blank\"],[9],[1,[29,\"svg-jar\",[\"ambulance\"],null],false],[0,\" Support Center\"],[10],[10],[0,\"\\n            \"],[7,\"li\"],[11,\"role\",\"presentation\"],[9],[7,\"a\"],[11,\"class\",\"dropdown-item\"],[11,\"role\",\"menuitem\"],[11,\"tabindex\",\"-1\"],[11,\"href\",\"https://twitter.com/intent/tweet?text=%40TryGhost+Hi%21+Can+you+help+me+with+&related=TryGhost\"],[11,\"target\",\"_blank\"],[11,\"onclick\",\"window.open(this.href, 'twitter-share', 'width=550,height=235');return false;\"],[9],[1,[29,\"svg-jar\",[\"twitter\"],null],false],[0,\" Tweet @TryGhost!\"],[10],[10],[0,\"\\n            \"],[7,\"li\"],[11,\"role\",\"presentation\"],[9],[7,\"a\"],[11,\"class\",\"dropdown-item\"],[11,\"role\",\"menuitem\"],[11,\"tabindex\",\"-1\"],[11,\"href\",\"https://docs.ghost.org/faq/using-ghost/\"],[11,\"target\",\"_blank\"],[9],[1,[29,\"svg-jar\",[\"book-open\"],null],false],[0,\" How to Use Ghost\"],[10],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDropdownExtension\"]]],null,{\"statements\":[[4,\"each\",[[25,[\"config\",\"clientExtensions\",\"dropdown\",\"items\"]]],null,{\"statements\":[[4,\"if\",[[24,3,[\"divider\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"li\"],[11,\"class\",\"divider\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"li\"],[11,\"role\",\"presentation\"],[9],[0,\"\\n                          \"],[7,\"a\"],[12,\"href\",[30,[[24,3,[\"href\"]]]]],[11,\"target\",\"_blank\"],[12,\"class\",[30,[\"dropdown-item \",[24,3,[\"classes\"]]]]],[11,\"role\",\"menuitem\"],[11,\"tabindex\",\"-1\"],[9],[1,[29,\"svg-jar\",[[24,3,[\"icon\"]]],null],false],[1,[24,3,[\"text\"]],false],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[3]},null],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[3,\"action\",[[24,0,[]],[24,2,[\"actions\",\"close\"]]],[[\"on\",\"preventDefault\"],[\"click\",false]]],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[2]},null],[0,\"\\n\"],[7,\"section\"],[11,\"class\",\"gh-nav-body\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-nav-top\"],[9],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"gh-nav-search\"],[9],[0,\"\\n            \"],[1,[29,\"gh-search-input\",null,[[\"class\"],[\"gh-nav-search-input\"]]],false],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"ul\"],[11,\"class\",\"gh-nav-list gh-nav-main\"],[9],[0,\"\\n            \"],[7,\"li\"],[11,\"class\",\"relative\"],[9],[0,\"\\n                \"],[7,\"a\"],[12,\"href\",[30,[[25,[\"config\",\"blogUrl\"]],\"/\"]]],[11,\"class\",\"gh-secondary-action\"],[11,\"title\",\"Open site in new tab\"],[11,\"target\",\"_blank\"],[9],[0,\"\\n                    \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"expand\"],null],false],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"span\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"current-when\",\"preventDefault\",\"route\"],[\"site\",[25,[\"isOnSite\"]],false,\"site\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"view-site-embed\"],null],false],[0,\"View site\"]],\"parameters\":[]},null],[3,\"action\",[[24,0,[]],\"transitionToOrRefreshSite\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"ul\"],[11,\"class\",\"gh-nav-list gh-nav-manage\"],[9],[0,\"\\n            \"],[7,\"li\"],[11,\"class\",\"gh-nav-list-h\"],[9],[0,\"Manage\"],[10],[0,\"\\n            \"],[7,\"li\"],[11,\"class\",\"relative\"],[9],[0,\"\\n                \"],[4,\"link-to\",null,[[\"classNames\",\"alt\",\"data-test-nav\",\"route\",\"model\"],[\"gh-nav-new-post\",\"New story\",\"new-story\",\"editor.new\",\"post\"]],{\"statements\":[[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"add-stroke\"],null],false],[10]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"router\",\"currentRouteName\"]],\"posts.index\"],null]],null,{\"statements\":[[0,\"                    \"],[4,\"link-to\",null,[[\"classNames\",\"data-test-nav\",\"query\",\"route\"],[\"active\",\"posts\",[29,\"hash\",null,[[\"type\",\"author\",\"tag\",\"order\"],[null,null,null,null]]],\"posts\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"content\"],null],false],[0,\"Posts\"]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"posts\",\"posts\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"content\"],null],false],[0,\"Posts\"]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]}],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"li\"],[9],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"router\",\"currentRouteName\"]],\"pages.index\"],null]],null,{\"statements\":[[0,\"                    \"],[4,\"link-to\",null,[[\"classNames\",\"data-test-nav\",\"query\",\"route\"],[\"active\",\"pages\",[29,\"hash\",null,[[\"type\",\"author\",\"tag\",\"order\"],[null,null,null,null]]],\"pages\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"pages\"],null],false],[0,\"Pages\"]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"pages\",\"pages\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"pages\"],null],false],[0,\"Pages\"]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]}],[0,\"            \"],[10],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[25,[\"feature\",\"members\"]],[29,\"gh-user-can-admin\",[[25,[\"session\",\"user\"]]],null]],null]],null,{\"statements\":[[0,\"                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"current-when\",\"data-test-nav\",\"route\"],[\"members member\",\"members\",\"members\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"members\"],null],false],[0,\"Members\"]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"staff\",\"staff\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"staff\"],null],false],[0,\"Staff\"]],\"parameters\":[]},null],[10],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[25,[\"feature\",\"subscribers\"]],[29,\"gh-user-can-admin\",[[25,[\"session\",\"user\"]]],null]],null]],null,{\"statements\":[[0,\"                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"subscribers\",\"subscribers\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"email\"],null],false],[0,\"Subscribers\"]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\"],[4,\"if\",[[29,\"gh-user-can-admin\",[[25,[\"session\",\"user\"]]],null]],null,{\"statements\":[[0,\"            \"],[7,\"ul\"],[11,\"class\",\"gh-nav-list gh-nav-settings\"],[9],[0,\"\\n                \"],[7,\"li\"],[11,\"class\",\"gh-nav-list-h\"],[9],[0,\"Settings\"],[10],[0,\"\\n                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"settings\",\"settings.general\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"settings\"],null],false],[0,\"General\"]],\"parameters\":[]},null],[10],[0,\"\\n                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"design\",\"settings.design\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"paintbrush\"],null],false],[0,\"Design\"]],\"parameters\":[]},null],[10],[0,\"\\n                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"tags\",\"settings.tags\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"tag\"],null],false],[0,\"Tags\"]],\"parameters\":[]},null],[10],[0,\"\\n                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"code-injection\",\"settings.code-injection\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"brackets\"],null],false],[0,\"Code injection\"]],\"parameters\":[]},null],[10],[0,\"\\n                \"],[7,\"li\"],[9],[4,\"link-to\",null,[[\"current-when\",\"data-test-nav\",\"route\"],[[25,[\"isIntegrationRoute\"]],\"integrations\",\"settings.integrations\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"modules\"],null],false],[0,\"Integrations\"]],\"parameters\":[]},null],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showMenuExtension\"]]],null,{\"statements\":[[0,\"            \"],[7,\"ul\"],[11,\"class\",\"gh-nav-list gh-nav-settings\"],[9],[0,\"\\n                \"],[4,\"if\",[[25,[\"config\",\"clientExtensions\",\"menu\",\"title\"]]],null,{\"statements\":[[7,\"li\"],[11,\"class\",\"gh-nav-list-h\"],[9],[1,[25,[\"config\",\"clientExtensions\",\"menu\",\"title\"]],false],[10]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"each\",[[25,[\"config\",\"clientExtensions\",\"menu\",\"items\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"li\"],[9],[0,\"\\n                        \"],[7,\"a\"],[12,\"href\",[30,[[24,1,[\"href\"]]]]],[11,\"target\",\"_blank\"],[9],[1,[29,\"svg-jar\",[[24,1,[\"icon\"]]],null],false],[1,[24,1,[\"text\"]],false],[10],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showScriptExtension\"]]],null,{\"statements\":[[0,\"            \"],[1,[25,[\"config\",\"clientExtensions\",\"script\",\"container\"]],true],[0,\"\\n            \"],[7,\"script\"],[12,\"src\",[30,[[25,[\"config\",\"clientExtensions\",\"script\",\"src\"]]]]],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"],[4,\"if\",[[29,\"gh-user-can-admin\",[[25,[\"session\",\"user\"]]],null]],null,{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"gh-nav-bottom\"],[9],[0,\"\\n        \"],[7,\"ul\"],[11,\"class\",\"gh-nav-list gh-nav-labs\"],[9],[0,\"\\n            \"],[7,\"li\"],[11,\"class\",\"relative\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-secondary-action\"],[11,\"title\",\"Toggle Night shift\"],[9],[0,\"\\n                    \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"nightshift\"],null],false],[10],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"nightShift\",[24,0,[\"feature\"]]],null]]],[10],[0,\"\\n                \"],[4,\"link-to\",null,[[\"data-test-nav\",\"route\"],[\"labs\",\"settings.labs\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"labs\"],null],false],[0,\"Labs\"]],\"parameters\":[]},null],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\\n\"],[1,[29,\"gh-tour-item\",[\"getting-started\"],[[\"target\",\"throbberAttachment\",\"popoverTriangleClass\",\"throbberOffset\"],[\".gh-nav header\",\"middle right\",\"left-top\",\"106px 2px\"]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-nav-menu.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-navitem", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "ROQ7NRvX",
    "block": "{\"symbols\":[],\"statements\":[[4,\"unless\",[[25,[\"navItem\",\"isNew\"]]],null,{\"statements\":[[0,\"    \"],[7,\"span\"],[11,\"class\",\"gh-blognav-grab\"],[9],[0,\"\\n        \"],[1,[29,\"svg-jar\",[\"grab\"],null],false],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"Reorder\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[7,\"div\"],[11,\"class\",\"gh-blognav-line\"],[9],[0,\"\\n\"],[4,\"gh-validation-status-container\",null,[[\"tagName\",\"class\",\"errors\",\"property\",\"hasValidated\"],[\"span\",\"gh-blognav-label\",[25,[\"navItem\",\"errors\"]],\"label\",[25,[\"navItem\",\"hasValidated\"]]]],{\"statements\":[[0,\"        \"],[1,[29,\"gh-trim-focus-input\",null,[[\"shouldFocus\",\"placeholder\",\"value\",\"input\",\"keyPress\",\"focus-out\",\"data-test-input\"],[[25,[\"navItem\",\"last\"]],\"Label\",[29,\"readonly\",[[25,[\"navItem\",\"label\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"navItem\",\"label\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"clearLabelErrors\"],null],[29,\"action\",[[24,0,[]],\"updateLabel\",[25,[\"navItem\",\"label\"]]],null],\"label\"]]],false],[0,\"\\n        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"navItem\",\"errors\"]],\"label\",\"label\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"gh-validation-status-container\",null,[[\"tagName\",\"class\",\"errors\",\"property\",\"hasValidated\"],[\"span\",\"gh-blognav-url\",[25,[\"navItem\",\"errors\"]],\"url\",[25,[\"navItem\",\"hasValidated\"]]]],{\"statements\":[[0,\"        \"],[1,[29,\"gh-navitem-url-input\",null,[[\"baseUrl\",\"isNew\",\"url\",\"update\",\"clearErrors\",\"data-test-input\"],[[25,[\"baseUrl\"]],[25,[\"navItem\",\"isNew\"]],[29,\"readonly\",[[25,[\"navItem\",\"url\"]]],null],[29,\"action\",[[24,0,[]],\"updateUrl\"],null],[29,\"action\",[[24,0,[]],\"clearUrlErrors\"],null],\"url\"]]],false],[0,\"\\n        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"navItem\",\"errors\"]],\"url\",\"url\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"navItem\",\"isNew\"]]],null,{\"statements\":[[0,\"    \"],[7,\"button\"],[11,\"class\",\"gh-blognav-add\"],[11,\"type\",\"button\"],[9],[0,\"\\n        \"],[1,[29,\"svg-jar\",[\"add\"],null],false],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"Add\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"addItem\"]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[7,\"button\"],[11,\"class\",\"gh-blognav-delete\"],[11,\"type\",\"button\"],[9],[0,\"\\n        \"],[1,[29,\"svg-jar\",[\"trash\"],null],false],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"Delete\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"deleteItem\",[25,[\"navItem\"]]]],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-navitem.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-notification", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "PmdO9Y88",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-notification-content\"],[11,\"data-test-text\",\"notification-content\"],[9],[0,\"\\n    \"],[1,[25,[\"message\",\"message\"]],false],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"button\"],[11,\"class\",\"gh-notification-close\"],[11,\"data-test-button\",\"close-notification\"],[9],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeNotification\"]],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-notification.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-notifications", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "/9OpBVs1",
    "block": "{\"symbols\":[\"message\"],\"statements\":[[4,\"each\",[[25,[\"messages\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-notification\",null,[[\"message\"],[[24,1,[]]]]],false],[0,\"\\n\"]],\"parameters\":[1]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-notifications.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-post-settings-menu", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "lvV0vJmj",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"settings-menu-container\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"id\",\"entry-controls\"],[9],[0,\"\\n        \"],[7,\"div\"],[12,\"class\",[30,[[29,\"if\",[[25,[\"isViewingSubview\"]],\"settings-menu-pane-out-left\",\"settings-menu-pane-in\"],null],\" settings-menu settings-menu-pane\"]]],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"settings-menu-header\"],[9],[0,\"\\n                \"],[7,\"h4\"],[9],[1,[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],false],[0,\" Settings\"],[10],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"close settings-menu-header-action\"],[11,\"data-test-close-settings-menu\",\"\"],[9],[0,\"\\n                    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],\"closeMenus\"],[[\"target\"],[[25,[\"ui\"]]]]],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n                \"],[1,[29,\"gh-image-uploader-with-preview\",null,[[\"image\",\"text\",\"allowUnsplash\",\"update\",\"remove\"],[[25,[\"post\",\"featureImage\"]],[29,\"concat\",[\"Upload \",[25,[\"post\",\"displayName\"]],\" image\"],null],true,[29,\"action\",[[24,0,[]],\"setCoverImage\"],null],[29,\"action\",[[24,0,[]],\"clearCoverImage\"],null]]]],false],[0,\"\\n                \"],[7,\"form\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"for\",\"url\"],[9],[1,[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],false],[0,\" URL\"],[10],[0,\"\\n\"],[4,\"unless\",[[25,[\"post\",\"isNew\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"post\",\"isPublished\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"a\"],[11,\"class\",\"post-view-link\"],[11,\"target\",\"_blank\"],[12,\"href\",[30,[[25,[\"post\",\"url\"]]]]],[9],[0,\"\\n                            View \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\" \"],[1,[29,\"svg-jar\",[\"external\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"a\"],[11,\"class\",\"post-view-link\"],[11,\"target\",\"_blank\"],[12,\"href\",[30,[[25,[\"post\",\"previewUrl\"]]]]],[9],[0,\"\\n                            Preview \"],[1,[29,\"svg-jar\",[\"external\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]},null],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-input-icon gh-icon-link\"],[9],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"link\"],null],false],[0,\"\\n                        \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\"],[\"post-setting-slug\",\"url\",\"post-setting-slug\",[29,\"readonly\",[[25,[\"slugValue\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"slugValue\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"updateSlug\",[25,[\"slugValue\"]]],null],true]]],false],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[1,[29,\"gh-url-preview\",null,[[\"slug\",\"tagName\",\"classNames\"],[[25,[\"slugValue\"]],\"p\",\"description\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[25,[\"allowAnnouncements\"]],[29,\"eq\",[[25,[\"post\",\"displayName\"]],\"post\"],null]],null]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"for\",\"announcement\"],[9],[1,[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],false],[0,\" Announcement\"],[10],[0,\"\\n                        \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"readonly\",\"placeholder\",\"value\",\"focus-out\",\"input\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-room-name\",\"room-name\",\"post-setting-room-name\",[29,\"not\",[[25,[\"allowAuthorRooms\"]]],null],[29,\"truncate\",[[25,[\"roomName\"]],40],null],[29,\"readonly\",[[25,[\"roomName\"]]],null],[29,\"action\",[[24,0,[]],\"validateRoom\",[25,[\"roomNameScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"roomNameScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],true,\"room-name\"]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"roomName\",\"room-name\"]]],false],[0,\"\\n\"],[4,\"if\",[[25,[\"allowAuthorRooms\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"p\"],[9],[0,\"Set Room Name to Announce\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                            \"],[7,\"p\"],[9],[0,\"Cannot Change the Room Name\"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                    \"],[7,\"div\"],[11,\"class\",\"form-group for-checkbox\"],[9],[0,\"\\n                        \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"announce\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[23,\"announce\"]],[11,\"class\",\"gh-input post-settings-announce\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"announce\"]]],null]],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-checkbox\",\"announce\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Announce this \"],[1,[25,[\"post\",\"displayName\"]],false],[10],[0,\"\\n                        \"],[3,\"action\",[[24,0,[]],\"toggleAnnounce\"],[[\"bubbles\"],[\"false\"]]],[10],[0,\"\\n                    \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"allowCollaboration\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"form-group for-checkbox\"],[9],[0,\"\\n                            \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"collaborate\"],[9],[0,\"\\n                            \"],[7,\"input\"],[12,\"checked\",[23,\"collaborate\"]],[11,\"class\",\"gh-input post-settings-collaborate\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"collaborate\"]]],null]],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-checkbox\",\"collaborate\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                            \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                            \"],[7,\"p\"],[9],[0,\"Collaborate with \"],[1,[23,\"roomName\"],false],[10],[0,\"\\n                            \"],[3,\"action\",[[24,0,[]],\"toggleCollaborate\"],[[\"bubbles\"],[\"false\"]]],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[7,\"div\"],[11,\"class\",\"form-group for-checkbox\"],[9],[0,\"\\n                        \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"private\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[25,[\"post\",\"isPrivate\"]]],[11,\"class\",\"gh-input post-settings-announce\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"post\",\"isPrivate\"]]],null]],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-checkbox\",\"privatePost\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Only for \"],[1,[23,\"roomName\"],false],[10],[0,\"\\n                        \"],[3,\"action\",[[24,0,[]],\"togglePrivatePost\"],[[\"bubbles\"],[\"false\"]]],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[25,[\"allowComments\"]],[29,\"eq\",[[25,[\"post\",\"displayName\"]],\"post\"],null]],null]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"form-group for-checkbox\"],[9],[0,\"\\n                        \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"comment\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[23,\"privateComments\"]],[11,\"class\",\"gh-input post-settings-comment\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"privateComments\"]]],null]],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-checkbox\",\"comment\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Private Discussion Room\"],[10],[0,\"\\n                        \"],[3,\"action\",[[24,0,[]],\"toggleComment\"],[[\"bubbles\"],[\"false\"]]],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n\"],[4,\"if\",[[29,\"or\",[[25,[\"post\",\"isDraft\"]],[25,[\"post\",\"isPublished\"]],[25,[\"post\",\"pastScheduledTime\"]]],null]],null,{\"statements\":[[0,\"                        \"],[7,\"label\"],[9],[0,\"Publish Date\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"label\"],[9],[0,\"Scheduled Date\"],[10],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Use the publish menu to re-schedule\"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                    \"],[1,[29,\"gh-date-time-picker\",null,[[\"date\",\"time\",\"setDate\",\"setTime\",\"errors\",\"dateErrorProperty\",\"timeErrorProperty\",\"maxDate\",\"disabled\",\"static\"],[[25,[\"post\",\"publishedAtBlogDate\"]],[25,[\"post\",\"publishedAtBlogTime\"]],[29,\"action\",[[24,0,[]],\"setPublishedAtBlogDate\"],null],[29,\"action\",[[24,0,[]],\"setPublishedAtBlogTime\"],null],[25,[\"post\",\"errors\"]],\"publishedAtBlogDate\",\"publishedAtBlogTime\",\"now\",[25,[\"post\",\"isScheduled\"]],true]]],false],[0,\"\\n                \"],[10],[0,\"\\n\\n\"],[4,\"unless\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"for\",\"tag-input\"],[9],[0,\"Tags\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-psm-tags-input\",null,[[\"post\",\"triggerId\"],[[25,[\"post\"]],\"tag-input\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"customExcerpt\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"custom-excerpt\"],[9],[0,\"Excerpt\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-textarea\",null,[[\"class\",\"id\",\"name\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-custom-excerpt\",\"custom-excerpt\",\"post-setting-custom-excerpt\",[29,\"readonly\",[[25,[\"customExcerptScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"customExcerptScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setCustomExcerpt\",[25,[\"customExcerptScratch\"]]],null],\"true\",\"custom-excerpt\"]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"customExcerpt\",\"custom-excerpt\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"unless\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[4,\"gh-form-group\",null,[[\"class\",\"errors\",\"hasValidated\",\"property\",\"data-test-input\"],[\"for-select\",[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"authors\",\"authors\"]],{\"statements\":[[0,\"                        \"],[7,\"label\"],[11,\"for\",\"author-list\"],[9],[0,\"Authors\"],[10],[0,\"\\n                        \"],[1,[29,\"gh-psm-authors-input\",null,[[\"selectedAuthors\",\"updateAuthors\",\"triggerId\"],[[25,[\"post\",\"authors\"]],[29,\"action\",[[24,0,[]],\"changeAuthors\"],null],\"author-list\"]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"authors\",\"authors\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"\\n                \"],[7,\"ul\"],[11,\"class\",\"nav-list nav-list-block\"],[9],[0,\"\\n                    \"],[7,\"li\"],[11,\"class\",\"nav-list-item\"],[11,\"data-test-button\",\"meta-data\"],[9],[0,\"\\n                        \"],[7,\"button\"],[11,\"type\",\"button\"],[9],[0,\"\\n                            \"],[7,\"b\"],[9],[0,\"Meta Data\"],[10],[0,\"\\n                            \"],[7,\"span\"],[9],[0,\"Extra content for search engines\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"showSubview\",\"meta-data\"]],[10],[0,\"\\n                     \"],[7,\"li\"],[11,\"class\",\"nav-list-item\"],[11,\"data-test-button\",\"announcement-data\"],[9],[0,\"\\n                        \"],[7,\"button\"],[11,\"type\",\"button\"],[9],[0,\"\\n                            \"],[7,\"b\"],[9],[0,\"Announcement Card\"],[10],[0,\"\\n                            \"],[7,\"span\"],[9],[0,\"Customise structured data for Announcement message\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"showSubview\",\"announcement-data\"]],[10],[0,\"\\n                    \"],[7,\"li\"],[11,\"class\",\"nav-list-item\"],[11,\"data-test-button\",\"twitter-data\"],[9],[0,\"\\n                        \"],[7,\"button\"],[11,\"type\",\"button\"],[9],[0,\"\\n                            \"],[7,\"b\"],[9],[0,\"Twitter Card\"],[10],[0,\"\\n                            \"],[7,\"span\"],[9],[0,\"Customise structured data for Twitter\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"showSubview\",\"twitter-data\"]],[10],[0,\"\\n                    \"],[7,\"li\"],[11,\"class\",\"nav-list-item\"],[11,\"data-test-button\",\"facebook-data\"],[9],[0,\"\\n                        \"],[7,\"button\"],[11,\"type\",\"button\"],[9],[0,\"\\n                            \"],[7,\"b\"],[9],[0,\"Facebook Card\"],[10],[0,\"\\n                            \"],[7,\"span\"],[9],[0,\"Customise Open Graph data\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"showSubview\",\"facebook-data\"]],[10],[0,\"\\n                    \"],[7,\"li\"],[11,\"class\",\"nav-list-item\"],[11,\"data-test-button\",\"codeinjection\"],[9],[0,\"\\n                        \"],[7,\"button\"],[11,\"type\",\"button\"],[9],[0,\"\\n                            \"],[7,\"b\"],[9],[0,\"Code Injection\"],[10],[0,\"\\n                            \"],[7,\"span\"],[9],[0,\"Add styles/scripts to the header & footer\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"showSubview\",\"codeinjection\"]],[10],[0,\"\\n                \"],[10],[0,\"\\n\\n\"],[4,\"unless\",[[24,0,[\"session\",\"user\",\"isAuthorOrContributor\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"form-group for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"featured\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[25,[\"post\",\"featured\"]]],[11,\"class\",\"gh-input post-settings-featured\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"post\",\"featured\"]]],null]],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-checkbox\",\"featured\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Feature this \"],[1,[25,[\"post\",\"displayName\"]],false],[10],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"toggleFeatured\"],[[\"bubbles\"],[\"false\"]]],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                \"],[1,[29,\"gh-psm-template-select\",null,[[\"post\",\"onTemplateSelect\"],[[25,[\"post\"]],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"post\",\"customTemplate\"]]],null]],null]]]],false],[0,\"\\n\\n\"],[4,\"unless\",[[25,[\"post\",\"isNew\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-link gh-btn-icon settings-menu-delete-button\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"trash\"],null],false],[0,\" Delete \"],[1,[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],false],[10],[3,\"action\",[[24,0,[]],\"deletePost\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[12,\"class\",[30,[[29,\"if\",[[25,[\"isViewingSubview\"]],\"settings-menu-pane-in\",\"settings-menu-pane-out-right\"],null],\" settings-menu settings-menu-pane\"]]],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"active\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"isViewingSubview\"]]],null,{\"statements\":[[4,\"if\",[[29,\"eq\",[[25,[\"subview\"]],\"meta-data\"],null]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-header subview\"],[9],[0,\"\\n                            \"],[7,\"button\"],[11,\"class\",\"back settings-menu-header-action\"],[11,\"data-test-button\",\"close-psm-subview\"],[9],[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10],[3,\"action\",[[24,0,[]],\"closeSubview\"]],[10],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Meta Data\"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n\\n                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n                            \"],[7,\"form\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"metaTitle\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"meta-title\"],[9],[0,\"Meta Title\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-meta-title\",\"meta-title\",\"post-setting-meta-title\",[29,\"readonly\",[[25,[\"metaTitleScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"metaTitleScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setMetaTitle\",[25,[\"metaTitleScratch\"]]],null],true,\"meta-title\"]]],false],[0,\"\\n                                    \"],[7,\"p\"],[9],[0,\"Recommended: \"],[7,\"b\"],[9],[0,\"70\"],[10],[0,\" characters. You’ve used \"],[1,[29,\"gh-count-down-characters\",[[25,[\"metaTitleScratch\"]],70],null],false],[10],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"post\",\"errors\"]],\"meta-title\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"metaDescription\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"meta-description\"],[9],[0,\"Meta Description\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-textarea\",null,[[\"class\",\"id\",\"name\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-meta-description\",\"meta-description\",\"post-setting-meta-description\",[29,\"readonly\",[[25,[\"metaDescriptionScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"metaDescriptionScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setMetaDescription\",[25,[\"metaDescriptionScratch\"]]],null],\"true\",\"meta-description\"]]],false],[0,\"\\n                                    \"],[7,\"p\"],[9],[0,\"Recommended: \"],[7,\"b\"],[9],[0,\"156\"],[10],[0,\" characters. You’ve used \"],[1,[29,\"gh-count-down-characters\",[[25,[\"metaDescriptionScratch\"]],156],null],false],[10],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"post\",\"errors\"]],\"meta-description\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"canonicalUrl\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"canonicalUrl\"],[9],[0,\"Canonical URL\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"name\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-canonicalUrl\",\"post-setting-canonicalUrl\",[29,\"readonly\",[[25,[\"canonicalUrlScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"canonicalUrlScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setCanonicalUrl\",[25,[\"canonicalUrlScratch\"]]],null],\"true\",\"canonicalUrl\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"post\",\"errors\"]],\"canonicalUrl\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                                    \"],[7,\"label\"],[9],[0,\"Search Engine Result Preview\"],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"seo-preview\"],[9],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"seo-preview-title\"],[9],[1,[29,\"truncate\",[[25,[\"seoTitle\"]],70],null],false],[10],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"seo-preview-link\"],[9],[1,[29,\"truncate\",[[25,[\"seoURL\"]],70],null],false],[10],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"seo-preview-description\"],[9],[1,[29,\"truncate\",[[25,[\"seoDescription\"]],300],null],false],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[3,\"action\",[[24,0,[]],\"discardEnter\"],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"subview\"]],\"announcement-data\"],null]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-header subview\"],[9],[0,\"\\n                            \"],[7,\"button\"],[11,\"class\",\"back settings-menu-header-action\"],[11,\"data-test-button\",\"close-psm-subview\"],[9],[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10],[3,\"action\",[[24,0,[]],\"closeSubview\"]],[10],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Announcement Card\"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n\\n                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n\\n                            \"],[7,\"form\"],[9],[0,\"\\n                                \"],[1,[29,\"gh-image-uploader-with-preview\",null,[[\"image\",\"text\",\"allowUnsplash\",\"update\",\"remove\"],[[25,[\"post\",\"rcImage\"]],\"Add Message image\",true,[29,\"action\",[[24,0,[]],\"setRcImage\"],null],[29,\"action\",[[24,0,[]],\"clearRcImage\"],null]]]],false],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"rcTitle\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"announcement-title\"],[9],[0,\"Message Title\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-rc-title\",\"rc-title\",\"post-setting-rc-title\",[29,\"truncate\",[[25,[\"rcTitle\"]],40],null],[29,\"readonly\",[[25,[\"rcTitleScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"rcTitleScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setRcTitle\",[25,[\"rcTitleScratch\"]]],null],true,\"announcement-title\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"rcTitle\",\"announcement-title\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"rcDescription\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"announcement-description\"],[9],[0,\"Message Description\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-textarea\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"stopEnterKeyDownPropagation\",\"value\",\"input\",\"focus-out\",\"data-test-field\"],[\"post-setting-rc-description\",\"rc-description\",\"post-setting-rc-description\",[29,\"truncate\",[[25,[\"rcDescription\"]],155],null],\"true\",[29,\"readonly\",[[25,[\"rcDescriptionScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"rcDescriptionScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setRcDescription\",[25,[\"rcDescriptionScratch\"]]],null],\"announcement-description\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"rcDescription\",\"rc-description\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                                    \"],[7,\"label\"],[9],[0,\"Preview\"],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"rcImage\"]]],null,{\"statements\":[[0,\"                                        \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-image\"],[12,\"style\",[29,\"background-image-style\",[[25,[\"rcImage\"]]],null]],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                        \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-content\"],[9],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-title\"],[9],[1,[23,\"rcTitle\"],false],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-description\"],[9],[1,[29,\"truncate\",[[25,[\"rcDescription\"]],155],null],false],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-footer\"],[9],[0,\"\\n                                                \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-footer-left\"],[9],[0,\"\\n                                                    \"],[1,[25,[\"config\",\"blogDomain\"]],false],[0,\"\\n                                                \"],[10],[0,\"\\n                                                \"],[7,\"div\"],[11,\"class\",\"gh-rc-preview-footer-right\"],[9],[0,\"\\n                                                \"],[10],[0,\"\\n                                            \"],[10],[0,\"\\n                                        \"],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n\\n                            \"],[3,\"action\",[[24,0,[]],\"discardEnter\"],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"subview\"]],\"twitter-data\"],null]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-header subview\"],[9],[0,\"\\n                            \"],[7,\"button\"],[11,\"class\",\"back settings-menu-header-action\"],[11,\"data-test-button\",\"close-psm-subview\"],[9],[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10],[3,\"action\",[[24,0,[]],\"closeSubview\"]],[10],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Twitter Card\"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n\\n                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n\\n                            \"],[7,\"form\"],[9],[0,\"\\n                                \"],[1,[29,\"gh-image-uploader-with-preview\",null,[[\"image\",\"text\",\"allowUnsplash\",\"update\",\"remove\"],[[25,[\"post\",\"twitterImage\"]],\"Add Twitter image\",true,[29,\"action\",[[24,0,[]],\"setTwitterImage\"],null],[29,\"action\",[[24,0,[]],\"clearTwitterImage\"],null]]]],false],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"twitterTitle\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"twitter-title\"],[9],[0,\"Twitter Title\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-twitter-title\",\"twitter-title\",\"post-setting-twitter-title\",[29,\"truncate\",[[25,[\"twitterTitle\"]],40],null],[29,\"readonly\",[[25,[\"twitterTitleScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"twitterTitleScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setTwitterTitle\",[25,[\"twitterTitleScratch\"]]],null],true,\"twitter-title\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"twitterTitle\",\"twitter-title\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"twitterDescription\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"twitter-description\"],[9],[0,\"Twitter Description\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-textarea\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"stopEnterKeyDownPropagation\",\"value\",\"input\",\"focus-out\",\"data-test-field\"],[\"post-setting-twitter-description\",\"twitter-description\",\"post-setting-twitter-description\",[29,\"truncate\",[[25,[\"twitterDescription\"]],155],null],\"true\",[29,\"readonly\",[[25,[\"twitterDescriptionScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"twitterDescriptionScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setTwitterDescription\",[25,[\"twitterDescriptionScratch\"]]],null],\"twitter-description\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"twitterDescription\",\"twitter-description\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                                    \"],[7,\"label\"],[9],[0,\"Preview\"],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"twitterImage\"]]],null,{\"statements\":[[0,\"                                        \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-image\"],[12,\"style\",[29,\"background-image-style\",[[25,[\"twitterImage\"]]],null]],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                        \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-content\"],[9],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-title\"],[9],[1,[23,\"twitterTitle\"],false],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-description\"],[9],[1,[29,\"truncate\",[[25,[\"twitterDescription\"]],155],null],false],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-footer\"],[9],[0,\"\\n                                                \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-footer-left\"],[9],[0,\"\\n                                                    \"],[1,[25,[\"config\",\"blogDomain\"]],false],[0,\"\\n                                                \"],[10],[0,\"\\n                                                \"],[7,\"div\"],[11,\"class\",\"gh-twitter-preview-footer-right\"],[9],[0,\"\\n                                                \"],[10],[0,\"\\n                                            \"],[10],[0,\"\\n                                        \"],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n\\n                            \"],[3,\"action\",[[24,0,[]],\"discardEnter\"],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"subview\"]],\"facebook-data\"],null]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-header subview\"],[9],[0,\"\\n                            \"],[7,\"button\"],[11,\"class\",\"back settings-menu-header-action\"],[11,\"data-test-button\",\"close-psm-subview\"],[9],[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10],[3,\"action\",[[24,0,[]],\"closeSubview\"]],[10],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Facebook Card\"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n\\n                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n                            \"],[7,\"form\"],[9],[0,\"\\n                                \"],[1,[29,\"gh-image-uploader-with-preview\",null,[[\"image\",\"text\",\"allowUnsplash\",\"update\",\"remove\"],[[25,[\"post\",\"ogImage\"]],\"Add Facebook image\",true,[29,\"action\",[[24,0,[]],\"setOgImage\"],null],[29,\"action\",[[24,0,[]],\"clearOgImage\"],null]]]],false],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"ogTitle\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"og-title\"],[9],[0,\"Facebook Title\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-og-title\",\"og-title\",\"post-setting-og-title\",[29,\"truncate\",[[25,[\"facebookTitle\"]],40],null],[29,\"readonly\",[[25,[\"ogTitleScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"ogTitleScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setOgTitle\",[25,[\"ogTitleScratch\"]]],null],true,\"og-title\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"ogTitle\",\"og-title\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"ogDescription\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"og-description\"],[9],[0,\"Facebook Description\"],[10],[0,\"\\n                                    \"],[1,[29,\"gh-textarea\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"value\",\"input\",\"focus-out\",\"stopEnterKeyDownPropagation\",\"data-test-field\"],[\"post-setting-og-description\",\"og-description\",\"post-setting-og-description\",[29,\"truncate\",[[25,[\"facebookDescription\"]],160],null],[29,\"readonly\",[[25,[\"ogDescriptionScratch\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"ogDescriptionScratch\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setOgDescription\",[25,[\"ogDescriptionScratch\"]]],null],\"true\",\"og-description\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"ogDescription\",\"og-description\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                                    \"],[7,\"label\"],[9],[0,\"Preview\"],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-og-preview\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"facebookImage\"]]],null,{\"statements\":[[0,\"                                        \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-image\"],[12,\"style\",[29,\"background-image-style\",[[25,[\"facebookImage\"]]],null]],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                        \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-content\"],[9],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-title\"],[9],[1,[29,\"truncate\",[[25,[\"facebookTitle\"]],88],null],false],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-description\"],[9],[1,[29,\"truncate\",[[25,[\"facebookDescription\"]],300],null],false],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-footer\"],[9],[0,\"\\n                                                \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-footer-left\"],[9],[0,\"\\n                                                    \"],[1,[25,[\"config\",\"blogDomain\"]],false],[0,\" \"],[7,\"span\"],[11,\"class\",\"gh-og-preview-footer-left-divider\"],[9],[0,\"|\"],[10],[0,\" by \"],[7,\"span\"],[11,\"class\",\"gh-og-preview-footer-author\"],[9],[1,[29,\"author-names\",[[25,[\"post\",\"authors\"]]],null],false],[10],[0,\"\\n                                                \"],[10],[0,\"\\n                                                \"],[7,\"div\"],[11,\"class\",\"gh-og-preview-footer-right\"],[9],[0,\"\\n                                                \"],[10],[0,\"\\n                                            \"],[10],[0,\"\\n                                        \"],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n\\n                            \"],[3,\"action\",[[24,0,[]],\"discardEnter\"],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"eq\",[[25,[\"subview\"]],\"codeinjection\"],null]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-header subview\"],[9],[0,\"\\n                            \"],[7,\"button\"],[11,\"class\",\"back settings-menu-header-action\"],[11,\"data-test-button\",\"close-psm-subview\"],[9],[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10],[3,\"action\",[[24,0,[]],\"closeSubview\"]],[10],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Code Injection\"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n\\n                        \"],[7,\"div\"],[11,\"class\",\"settings-menu-content settings-menu-content-codeinjection\"],[9],[0,\"\\n                            \"],[7,\"form\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"codeinjectionHead\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"codeinjection-head\"],[9],[1,[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],false],[0,\" Header \"],[7,\"code\"],[9],[0,\"{{ghost_head}}\"],[10],[10],[0,\"\\n                                    \"],[1,[29,\"gh-cm-editor\",[[25,[\"codeinjectionHeadScratch\"]]],[[\"id\",\"class\",\"name\",\"focusOut\",\"stopEnterKeyDownPropagation\",\"update\",\"data-test-field\"],[\"post-setting-codeinjection-head\",\"post-setting-codeinjection\",\"post-setting-codeinjection-head\",[29,\"action\",[[24,0,[]],\"setHeaderInjection\",[25,[\"codeinjectionHeadScratch\"]]],null],\"true\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"codeinjectionHeadScratch\"]]],null]],null],\"codeinjection-head\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"codeinjectionHead\",\"codeinjection-head\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"post\",\"errors\"]],[25,[\"post\",\"hasValidated\"]],\"codeinjectionFoot\"]],{\"statements\":[[0,\"                                    \"],[7,\"label\"],[11,\"for\",\"codeinjection-foot\"],[9],[1,[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],false],[0,\" Footer \"],[7,\"code\"],[9],[0,\"{{ghost_foot}}\"],[10],[10],[0,\"\\n                                    \"],[1,[29,\"gh-cm-editor\",[[25,[\"codeinjectionFootScratch\"]]],[[\"id\",\"class\",\"name\",\"focusOut\",\"stopEnterKeyDownPropagation\",\"update\",\"data-test-field\"],[\"post-setting-codeinjection-foot\",\"post-setting-codeinjection\",\"post-setting-codeinjection-foot\",[29,\"action\",[[24,0,[]],\"setFooterInjection\",[25,[\"codeinjectionFootScratch\"]]],null],\"true\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"codeinjectionFootScratch\"]]],null]],null],\"codeinjection-foot\"]]],false],[0,\"\\n                                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"post\",\"errors\"]],\"codeinjectionFoot\",\"codeinjection-foot\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                            \"],[3,\"action\",[[24,0,[]],\"discardEnter\"],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"_showThrobbers\"]]],null,{\"statements\":[[4,\"unless\",[[24,0,[\"session\",\"user\",\"isAuthorOrContributor\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-tour-item\",[\"featured-post\"],[[\"target\",\"throbberAttachment\",\"throbberOffset\",\"popoverTriangleClass\"],[\"label[for='featured'] p\",\"middle middle\",\"0px -20px\",\"bottom-right\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-post-settings-menu.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-posts-list-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "DTkZI/7i",
    "block": "{\"symbols\":[],\"statements\":[[7,\"h3\"],[11,\"class\",\"gh-content-entry-title\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"title\",\"route\",\"models\"],[\"permalink\",\"Edit this post\",\"editor.edit\",[29,\"array\",[[24,0,[\"post\",\"displayName\"]],[24,0,[\"post\",\"id\"]]],null]]],{\"statements\":[[0,\"        \"],[1,[24,0,[\"post\",\"title\"]],false],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\"],[7,\"p\"],[9],[1,[24,0,[\"subText\"]],false],[10],[0,\"\\n\\n\"],[7,\"section\"],[11,\"class\",\"gh-content-entry-meta\"],[9],[0,\"\\n\"],[4,\"if\",[[24,0,[\"isPage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"gh-content-status-draft gh-badge gh-badge-black\"],[9],[0,\"Page\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[24,0,[\"isScheduled\"]]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"gh-content-status-draft gh-badge\"],[9],[0,\"Scheduled\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[24,0,[\"isDraft\"]]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"gh-content-status-draft gh-badge gh-badge-red\"],[9],[0,\"Draft\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[24,0,[\"isFeatured\"]]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"gh-content-status-featured gh-badge gh-badge-blue\"],[9],[0,\"Featured\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[24,0,[\"isPublished\"]],[29,\"not\",[[24,0,[\"post\",\"page\"]]],null]],null]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"gh-content-status-published\"],[9],[0,\"Published\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    by \"],[7,\"span\"],[11,\"class\",\"gh-content-entry-author\"],[9],[1,[24,0,[\"authorNames\"]],false],[10],[0,\" —\\n\\n\"],[4,\"if\",[[24,0,[\"isPublished\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-format-post-time\",[[24,0,[\"post\",\"publishedAtUTC\"]]],[[\"published\"],[true]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"isScheduled\"]]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"green\"],[9],[0,\"Will go live \"],[1,[29,\"gh-format-post-time\",[[24,0,[\"post\",\"publishedAtUTC\"]]],[[\"scheduled\"],[true]]],false],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        Last edited \"],[1,[29,\"gh-format-post-time\",[[24,0,[\"post\",\"updatedAtUTC\"]]],[[\"draft\"],[true]]],false],[0,\"\\n    \"]],\"parameters\":[]}]],\"parameters\":[]}],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-posts-list-item.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-profile-image", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "0igIgOmM",
    "block": "{\"symbols\":[],\"statements\":[[7,\"figure\"],[11,\"class\",\"account-image\"],[9],[0,\"\\n\\n\"],[4,\"unless\",[[25,[\"previewDataURL\"]]],null,{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"placeholder-img\"],[12,\"style\",[23,\"placeholderStyle\"]],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"id\",\"account-image\"],[11,\"class\",\"gravatar-img\"],[12,\"style\",[23,\"avatarStyle\"]],[9],[0,\"\\n            \"],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"User image\"],[10],[0,\"\\n        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"previewDataURL\"]]],null,{\"statements\":[[0,\"        \"],[7,\"img\"],[12,\"src\",[23,\"previewDataURL\"]],[11,\"class\",\"gravatar-img\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"span\"],[11,\"class\",\"edit-account-image\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"openFileDialog\"],null]],[11,\"role\",\"button\"],[9],[0,\"\\n        \"],[1,[29,\"svg-jar\",[\"photos\"],null],false],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"Upload an image\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[1,[29,\"gh-file-input\",null,[[\"alt\",\"name\",\"multiple\",\"action\",\"accept\"],[null,\"uploadimage\",false,[29,\"action\",[[24,0,[]],\"imageSelected\"],null],[25,[\"imageMimeTypes\"]]]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-profile-image.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-progress-bar", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "E3tx3DPi",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-progress-container\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-progress-container-progress\"],[9],[0,\"\\n        \"],[7,\"div\"],[12,\"class\",[30,[\"gh-progress-bar \",[29,\"if\",[[25,[\"isError\"]],\"-error\"],null]]]],[12,\"style\",[23,\"progressStyle\"]],[11,\"data-test-progress-bar\",\"\"],[12,\"data-test-progress-width\",[23,\"percentage\"]],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-progress-bar.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-psm-authors-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "GlmqK1wj",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-token-input\",null,[[\"options\",\"selected\",\"onchange\",\"allowCreation\",\"renderInPlace\",\"triggerId\"],[[25,[\"availableAuthors\"]],[25,[\"selectedAuthors\"]],[29,\"action\",[[24,0,[]],\"updateAuthors\"],null],false,true,[25,[\"triggerId\"]]]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-psm-authors-input.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-psm-tags-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "jIiLwHiz",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-token-input\",null,[[\"extra\",\"onchange\",\"oncreate\",\"options\",\"renderInPlace\",\"selected\",\"showCreateWhen\",\"triggerId\"],[[29,\"hash\",null,[[\"tokenComponent\"],[\"gh-token-input/tag-token\"]]],[29,\"action\",[[24,0,[]],\"updateTags\"],null],[29,\"action\",[[24,0,[]],\"createTag\"],null],[25,[\"availableTags\"]],true,[25,[\"post\",\"tags\"]],[29,\"action\",[[24,0,[]],\"hideCreateOptionOnMatchingTag\"],null],[25,[\"triggerId\"]]]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-psm-tags-input.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-psm-template-select", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "lyKDPRjH",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"customTemplates\"]]],null,{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"form-group for-select\"],[11,\"data-test-custom-template-form\",\"\"],[9],[0,\"\\n        \"],[7,\"label\"],[11,\"for\",\"author-list\"],[9],[0,\"Template\"],[10],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"gh-input-icon gh-icon-user\"],[9],[0,\"\\n            \"],[1,[29,\"svg-jar\",[\"file-text-document\"],null],false],[0,\"\\n            \"],[7,\"span\"],[12,\"class\",[30,[\"gh-select \",[29,\"if\",[[25,[\"matchedSlugTemplate\"]],\"disabled\"],null]]]],[9],[0,\"\\n                \"],[1,[29,\"one-way-select\",[[25,[\"selectedTemplate\"]]],[[\"options\",\"optionValuePath\",\"optionLabelPath\",\"update\",\"disabled\",\"data-test-select\"],[[25,[\"customTemplates\"]],\"filename\",\"name\",[29,\"action\",[[24,0,[]],\"selectTemplate\"],null],[25,[\"matchedSlugTemplate\"]],\"custom-template\"]]],false],[0,\"\\n                \"],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"matchedSlugTemplate\"]]],null,{\"statements\":[[0,\"            \"],[7,\"p\"],[9],[0,\"Post URL matches \"],[1,[25,[\"matchedSlugTemplate\",\"filename\"]],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-psm-template-select.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-publishmenu-draft", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "XSh10owP",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"gh-publishmenu-heading\"],[9],[0,\"Ready to publish your \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\"?\"],[10],[0,\"\\n\"],[7,\"section\"],[11,\"class\",\"gh-publishmenu-content\"],[9],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"saveType\"]],\"publish\"],null],\"active\"],null]]]],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[11,\"data-test-publishmenu-published-option\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[0,\"Set it live now\"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Publish this \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\" immediately\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"setSaveType\",\"publish\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"saveType\"]],\"schedule\"],null],\"active\"],null]]]],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[11,\"data-test-publishmenu-scheduled-option\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[0,\"Schedule it for later\"],[10],[0,\"\\n            \"],[1,[29,\"gh-date-time-picker\",null,[[\"date\",\"time\",\"setDate\",\"setTime\",\"errors\",\"dateErrorProperty\",\"timeErrorProperty\",\"minDate\"],[[25,[\"post\",\"publishedAtBlogDate\"]],[25,[\"post\",\"publishedAtBlogTime\"]],[29,\"action\",[[24,0,[]],\"setDate\"],null],[29,\"action\",[[24,0,[]],\"setTime\"],null],[25,[\"post\",\"errors\"]],\"publishedAtBlogDate\",\"publishedAtBlogTime\",[25,[\"_minDate\"]]]]],false],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Set automatic future publish date\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"setSaveType\",\"schedule\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-publishmenu-draft.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-publishmenu-published", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "FhFPRRlS",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"gh-publishmenu-heading\"],[9],[0,\"Update \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\" status\"],[10],[0,\"\\n\"],[7,\"section\"],[11,\"class\",\"gh-publishmenu-content\"],[9],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"saveType\"]],\"draft\"],null],\"active\"],null]]]],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[11,\"data-test-publishmenu-unpublished-option\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[0,\"Unpublished\"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Revert this \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\" to a private draft\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],[25,[\"setSaveType\"]],\"draft\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"saveType\"]],\"publish\"],null],\"active\"],null]]]],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[11,\"data-test-publishmenu-published-option\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[0,\"Published\"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Display this \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\" publicly\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],[25,[\"setSaveType\"]],\"publish\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-publishmenu-published.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-publishmenu-scheduled", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "aZe2Vb7W",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"gh-publishmenu-heading\"],[9],[0,\"Will be published in \"],[1,[23,\"timeToPublished\"],false],[10],[0,\"\\n\"],[7,\"section\"],[11,\"class\",\"gh-publishmenu-content\"],[9],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"saveType\"]],\"draft\"],null],\"active\"],null]]]],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[11,\"data-test-publishmenu-draft-option\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[0,\"Revert to draft\"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Do not publish\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"setSaveType\",\"draft\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"saveType\"]],\"schedule\"],null],\"active\"],null]]]],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[11,\"data-test-publishmenu-scheduled-option\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[0,\"Schedule for later\"],[10],[0,\"\\n            \"],[1,[29,\"gh-date-time-picker\",null,[[\"date\",\"time\",\"setDate\",\"setTime\",\"errors\",\"dateErrorProperty\",\"timeErrorProperty\",\"minDate\"],[[25,[\"post\",\"publishedAtBlogDate\"]],[25,[\"post\",\"publishedAtBlogTime\"]],[29,\"action\",[[24,0,[]],\"setDate\"],null],[29,\"action\",[[24,0,[]],\"setTime\"],null],[25,[\"post\",\"errors\"]],\"publishedAtBlogDate\",\"publishedAtBlogTime\",[25,[\"_minDate\"]]]]],false],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Set automatic future publish date\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"setSaveType\",\"schedule\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-publishmenu-scheduled.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-publishmenu", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "X1d0p9rR",
    "block": "{\"symbols\":[\"dd\"],\"statements\":[[4,\"basic-dropdown\",null,[[\"verticalPosition\",\"onOpen\",\"onClose\"],[\"below\",[29,\"action\",[[24,0,[]],\"open\"],null],[29,\"action\",[[24,0,[]],\"close\"],null]]],{\"statements\":[[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"trigger\"]],\"expected `dd.trigger` to be a contextual component but found a string. Did you mean `(component dd.trigger)`? ('ghost-admin/templates/components/gh-publishmenu.hbs' @ L2:C7) \"],null]],[[\"class\"],[\"gh-btn gh-btn-outline gh-publishmenu-trigger\"]],{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"data-test-publishmenu-trigger\",\"\"],[9],[1,[23,\"triggerText\"],false],[0,\" \"],[1,[29,\"svg-jar\",[\"arrow-down\"],null],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"content\"]],\"expected `dd.content` to be a contextual component but found a string. Did you mean `(component dd.content)`? ('ghost-admin/templates/components/gh-publishmenu.hbs' @ L6:C7) \"],null]],[[\"class\"],[\"gh-publishmenu-dropdown\"]],{\"statements\":[[4,\"if\",[[29,\"eq\",[[25,[\"displayState\"]],\"published\"],null]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-publishmenu-published\",null,[[\"post\",\"saveType\",\"setSaveType\"],[[25,[\"post\"]],[25,[\"saveType\"]],[29,\"action\",[[24,0,[]],\"setSaveType\"],null]]]],false],[0,\"\\n\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[29,\"eq\",[[25,[\"displayState\"]],\"scheduled\"],null]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-publishmenu-scheduled\",null,[[\"post\",\"saveType\",\"isClosing\",\"setSaveType\"],[[25,[\"post\"]],[25,[\"saveType\"]],[25,[\"isClosing\"]],[29,\"action\",[[24,0,[]],\"setSaveType\"],null]]]],false],[0,\"\\n\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        \"],[1,[29,\"gh-publishmenu-draft\",null,[[\"post\",\"saveType\",\"setSaveType\"],[[25,[\"post\"]],[25,[\"saveType\"]],[29,\"action\",[[24,0,[]],\"setSaveType\"],null]]]],false],[0,\"\\n        \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"\\n\"],[0,\"        \"],[7,\"footer\"],[11,\"class\",\"gh-publishmenu-footer\"],[9],[0,\"\\n            \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-outline gh-btn-link\"],[11,\"data-test-publishmenu-cancel\",\"\"],[9],[0,\"\\n                \"],[7,\"span\"],[9],[0,\"Cancel\"],[10],[0,\"\\n            \"],[3,\"action\",[[24,0,[]],[24,1,[\"actions\",\"close\"]]]],[10],[0,\"\\n            \"],[1,[29,\"gh-task-button\",[[25,[\"buttonText\"]]],[[\"task\",\"successText\",\"runningText\",\"class\",\"data-test-publishmenu-save\"],[[25,[\"save\"]],[25,[\"successText\"]],[25,[\"runningText\"]],\"gh-btn gh-btn-blue gh-publishmenu-button gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[1]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-publishmenu.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-scheduled-post-countdown", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "Qu6wsVI8",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[25,[\"countdown\"]]],null,{\"statements\":[[0,\"    \"],[15,1,[[25,[\"post\"]],[25,[\"countdown\"]]]],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-scheduled-post-countdown.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-scroll-trigger", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "uT2FTYIB",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-scroll-trigger.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-search-input-trigger", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "JPHSfUg0",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"ember-power-select-search\"],[12,\"onmousedown\",[29,\"action\",[[24,0,[]],\"captureMouseDown\"],null]],[9],[0,\"\\n    \"],[7,\"input\"],[11,\"autocomplete\",\"off\"],[11,\"autocorrect\",\"off\"],[11,\"autocapitalize\",\"off\"],[12,\"value\",[29,\"if\",[[25,[\"extra\",\"labelPath\"]],[29,\"get\",[[25,[\"selected\"]],[25,[\"extra\",\"labelPath\"]]],null],[25,[\"selected\"]]],null]],[11,\"spellcheck\",\"false\"],[11,\"role\",\"combobox\"],[12,\"placeholder\",[23,\"placeholder\"]],[12,\"oninput\",[29,\"action\",[[24,0,[]],\"search\"],[[\"value\"],[\"target.value\"]]]],[12,\"onmousedown\",[29,\"action\",[[24,0,[]],\"captureMouseDown\"],null]],[12,\"onkeydown\",[29,\"action\",[[24,0,[]],\"handleKeydown\"],null]],[12,\"onblur\",[29,\"action\",[[24,0,[]],\"resetInput\"],null]],[11,\"type\",\"search\"],[9],[10],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-nav-search-button\"],[9],[1,[29,\"svg-jar\",[\"search\"],null],false],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"Search\"],[10],[3,\"action\",[[24,0,[]],\"focusInput\"]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-search-input-trigger.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-search-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "8lBp3iYx",
    "block": "{\"symbols\":[\"name\",\"select\"],\"statements\":[[4,\"power-select\",null,[[\"search\",\"onchange\",\"placeholder\",\"onopen\",\"onclose\",\"searchEnabled\",\"triggerComponent\",\"renderInPlace\",\"loadingMessage\"],[[29,\"action\",[[24,0,[]],\"search\"],null],[29,\"action\",[[24,0,[]],\"openSelected\"],null],\"Search...\",[29,\"action\",[[24,0,[]],\"onFocus\"],null],[29,\"action\",[[24,0,[]],\"onBlur\"],null],false,\"gh-search-input-trigger\",true,\"Loading\"]],{\"statements\":[[0,\"    \"],[1,[29,\"highlighted-text\",[[24,1,[\"title\"]],[24,2,[\"searchText\"]]],null],false],[0,\"\\n\"]],\"parameters\":[1,2]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-search-input.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-simplemde", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "vclBciy7",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-simplemde.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-site-iframe", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "0rxIneNc",
    "block": "{\"symbols\":[],\"statements\":[[7,\"iframe\"],[11,\"id\",\"site-frame\"],[11,\"class\",\"site-frame\"],[12,\"src\",[30,[[24,0,[\"config\",\"blogUrl\"]],\"/\"]]],[11,\"frameborder\",\"0\"],[11,\"allowtransparency\",\"true\"],[9],[10],[0,\"\\n\\n\"],[7,\"style\"],[9],[0,\"\\n.site-frame {\\n    position: absolute;\\n    top: 0;\\n    right: 0;\\n    bottom: 0;\\n    left: 0;\\n    width: 100%;\\n    height: 100%;\\n    border: none;\\n    transform: translate3d(0, 0, 0);\\n}\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-site-iframe.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-tag-settings-form", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "0uBs+X2O",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[12,\"class\",[30,[[29,\"if\",[[25,[\"isViewingSubview\"]],\"settings-menu-pane-out-left\",\"settings-menu-pane-in\"],null],\" settings-menu settings-menu-pane tag-settings-pane\"]]],[9],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"settings-menu-header \",[29,\"if\",[[25,[\"isMobile\"]],\"subview\"],null]]]],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"isMobile\"]]],null,{\"statements\":[[0,\"            \"],[4,\"link-to\",null,[[\"class\",\"route\"],[\"back settings-menu-header-action\",\"settings.tags\"]],{\"statements\":[[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"h4\"],[9],[1,[23,\"title\"],false],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"            \"],[7,\"h4\"],[9],[1,[23,\"title\"],false],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"    \"],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n        \"],[1,[29,\"gh-image-uploader-with-preview\",null,[[\"image\",\"text\",\"allowUnsplash\",\"update\",\"remove\"],[[25,[\"tag\",\"featureImage\"]],\"Upload tag image\",true,[29,\"action\",[[24,0,[]],\"setCoverImage\"],null],[29,\"action\",[[24,0,[]],\"clearCoverImage\"],null]]]],false],[0,\"\\n        \"],[7,\"form\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"tag\",\"errors\"]],[25,[\"tag\",\"hasValidated\"]],\"name\"]],{\"statements\":[[0,\"                \"],[7,\"label\"],[11,\"for\",\"tag-name\"],[9],[0,\"Name\"],[10],[0,\"\\n                \"],[1,[29,\"gh-text-input\",null,[[\"id\",\"name\",\"value\",\"input\",\"focus-out\"],[\"tag-name\",\"name\",[29,\"readonly\",[[25,[\"scratchName\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"scratchName\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setProperty\",\"name\",[25,[\"scratchName\"]]],null]]]],false],[0,\"\\n                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"tag\",\"errors\"]],\"name\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"tag\",\"errors\"]],[25,[\"tag\",\"hasValidated\"]],\"slug\"]],{\"statements\":[[0,\"                \"],[7,\"label\"],[11,\"for\",\"tag-slug\"],[9],[0,\"URL\"],[10],[0,\"\\n                \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"id\",\"name\",\"focus-out\",\"input\"],[[29,\"readonly\",[[25,[\"scratchSlug\"]]],null],\"tag-slug\",\"slug\",[29,\"action\",[[24,0,[]],\"setProperty\",\"slug\",[25,[\"scratchSlug\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"scratchSlug\"]]],null]],[[\"value\"],[\"target.value\"]]]]]],false],[0,\"\\n                \"],[1,[29,\"gh-url-preview\",null,[[\"prefix\",\"slug\",\"tagName\",\"classNames\"],[\"tag\",[25,[\"scratchSlug\"]],\"p\",\"description\"]]],false],[0,\"\\n                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"activeTag\",\"errors\"]],\"slug\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"tag\",\"errors\"]],[25,[\"tag\",\"hasValidated\"]],\"description\"]],{\"statements\":[[0,\"                \"],[7,\"label\"],[11,\"for\",\"tag-description\"],[9],[0,\"Description\"],[10],[0,\"\\n                \"],[1,[29,\"gh-textarea\",null,[[\"id\",\"name\",\"value\",\"input\",\"focus-out\"],[\"tag-description\",\"description\",[29,\"readonly\",[[25,[\"scratchDescription\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"scratchDescription\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setProperty\",\"description\",[25,[\"scratchDescription\"]]],null]]]],false],[0,\"\\n                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"tag\",\"errors\"]],\"description\"]]],false],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Maximum: \"],[7,\"b\"],[9],[0,\"500\"],[10],[0,\" characters. You’ve used \"],[1,[29,\"gh-count-down-characters\",[[25,[\"scratchDescription\"]],500],null],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"ul\"],[11,\"class\",\"nav-list nav-list-block\"],[9],[0,\"\\n                \"],[7,\"li\"],[11,\"class\",\"nav-list-item\"],[9],[0,\"\\n                    \"],[7,\"button\"],[11,\"class\",\"meta-data-button\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"b\"],[9],[0,\"Meta Data\"],[10],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"Extra content for SEO and social media.\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],\"openMeta\"]],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n\"],[4,\"unless\",[[25,[\"tag\",\"isNew\"]]],null,{\"statements\":[[0,\"                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-link gh-btn-sm gh-btn-icon settings-menu-delete-button\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"trash\"],null],false],[0,\" Delete Tag\"],[10],[3,\"action\",[[24,0,[]],\"deleteTag\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[12,\"class\",[30,[[29,\"if\",[[25,[\"isViewingSubview\"]],\"settings-menu-pane-in\",\"settings-menu-pane-out-right\"],null],\" settings-menu settings-menu-pane tag-meta-settings-pane\"]]],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"settings-menu-header subview\"],[9],[0,\"\\n        \"],[7,\"button\"],[11,\"class\",\"back settings-menu-header-action\"],[9],[1,[29,\"svg-jar\",[\"arrow-left\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Back\"],[10],[3,\"action\",[[24,0,[]],\"closeMeta\"]],[10],[0,\"\\n        \"],[7,\"h4\"],[9],[0,\"Meta Data\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"style\",\"width:23px;\"],[9],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"settings-menu-content\"],[9],[0,\"\\n        \"],[7,\"form\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"tag\",\"errors\"]],[25,[\"tag\",\"hasValidated\"]],\"metaTitle\"]],{\"statements\":[[0,\"                \"],[7,\"label\"],[11,\"for\",\"meta-title\"],[9],[0,\"Meta Title\"],[10],[0,\"\\n                \"],[1,[29,\"gh-text-input\",null,[[\"id\",\"name\",\"value\",\"input\",\"focus-out\"],[\"meta-title\",\"metaTitle\",[29,\"readonly\",[[25,[\"scratchMetaTitle\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"scratchMetaTitle\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setProperty\",\"metaTitle\",[25,[\"scratchMetaTitle\"]]],null]]]],false],[0,\"\\n                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"tag\",\"errors\"]],\"metaTitle\"]]],false],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Recommended: \"],[7,\"b\"],[9],[0,\"70\"],[10],[0,\" characters. You’ve used \"],[1,[29,\"gh-count-down-characters\",[[25,[\"scratchMetaTitle\"]],70],null],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"tag\",\"errors\"]],[25,[\"tag\",\"hasValidated\"]],\"metaDescription\"]],{\"statements\":[[0,\"                \"],[7,\"label\"],[11,\"for\",\"meta-description\"],[9],[0,\"Meta Description\"],[10],[0,\"\\n                \"],[1,[29,\"gh-textarea\",null,[[\"id\",\"name\",\"value\",\"input\",\"focus-out\"],[\"meta-description\",\"metaDescription\",[29,\"readonly\",[[25,[\"scratchMetaDescription\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"scratchMetaDescription\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setProperty\",\"metaDescription\",[25,[\"scratchMetaDescription\"]]],null]]]],false],[0,\"\\n                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"tag\",\"errors\"]],\"metaDescription\"]]],false],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Recommended: \"],[7,\"b\"],[9],[0,\"156\"],[10],[0,\" characters. You’ve used \"],[1,[29,\"gh-count-down-characters\",[[25,[\"scratchMetaDescription\"]],156],null],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                \"],[7,\"label\"],[9],[0,\"Search Engine Result Preview\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"seo-preview\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"seo-preview-title\"],[9],[1,[23,\"seoTitle\"],false],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"seo-preview-link\"],[9],[1,[23,\"seoURL\"],false],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"seo-preview-description\"],[9],[1,[23,\"seoDescription\"],false],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-tag-settings-form.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-tag", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "JkWCTXQa",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"settings-tag\"],[12,\"id\",[30,[\"gh-tag-\",[25,[\"tag\",\"id\"]]]]],[12,\"data-test-tag\",[30,[[25,[\"tag\",\"id\"]]]]],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"route\",\"model\"],[\"tag-edit-button\",\"settings.tags.tag\",[25,[\"tag\"]]]],{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"tag-title\"],[11,\"data-test-name\",\"\"],[9],[1,[25,[\"tag\",\"name\"]],false],[10],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"label label-default\"],[11,\"data-test-slug\",\"\"],[9],[0,\"/\"],[1,[25,[\"tag\",\"slug\"]],false],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"tag\",\"isInternal\"]]],null,{\"statements\":[[0,\"            \"],[7,\"span\"],[11,\"class\",\"label label-blue\"],[11,\"data-test-internal\",\"\"],[9],[0,\"internal\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"p\"],[11,\"class\",\"tag-description\"],[11,\"data-test-description\",\"\"],[9],[1,[25,[\"tag\",\"description\"]],false],[10],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"tags-count\"],[11,\"data-test-post-count\",\"\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"query\",\"route\"],[[29,\"hash\",null,[[\"type\",\"author\",\"tag\",\"order\"],[null,null,[25,[\"tag\",\"slug\"]],null]]],\"posts\"]],{\"statements\":[[0,\"                \"],[1,[25,[\"tag\",\"count\",\"posts\"]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-tag.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-tags-management-container", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "FrUQkjUs",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1,[[24,0,[]]]],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-tags-management-container.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-task-button", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "P+JZqKuk",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[26,1]],null,{\"statements\":[[0,\"    \"],[15,1,[[29,\"hash\",null,[[\"isIdle\",\"isRunning\",\"isSuccess\",\"isFailure\"],[[25,[\"isIdle\"]],[25,[\"isRunning\"]],[25,[\"isSuccess\"]],[25,[\"isFailure\"]]]]]]],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[7,\"span\"],[9],[0,\"\\n    \"],[4,\"if\",[[25,[\"isRunning\"]]],null,{\"statements\":[[1,[29,\"svg-jar\",[\"spinner\"],[[\"class\"],[\"gh-icon-spinner\"]]],false],[1,[23,\"runningText\"],false]],\"parameters\":[]},null],[0,\"\\n    \"],[4,\"if\",[[25,[\"isIdle\"]]],null,{\"statements\":[[1,[23,\"buttonText\"],false]],\"parameters\":[]},null],[0,\"\\n    \"],[4,\"if\",[[25,[\"isSuccess\"]]],null,{\"statements\":[[1,[29,\"svg-jar\",[\"check-circle\"],null],false],[0,\" \"],[1,[23,\"successText\"],false]],\"parameters\":[]},null],[0,\"\\n    \"],[4,\"if\",[[25,[\"isFailure\"]]],null,{\"statements\":[[1,[29,\"svg-jar\",[\"retry\"],null],false],[0,\" \"],[1,[23,\"failureText\"],false]],\"parameters\":[]},null],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-task-button.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-text-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "lv7JhQer",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-text-input.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-theme-error-li", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "GQJf7mpW",
    "block": "{\"symbols\":[\"failure\"],\"statements\":[[7,\"a\"],[11,\"href\",\"\"],[11,\"class\",\"theme-validation-toggle-details\"],[11,\"data-test-toggle-details\",\"\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"theme-validation-type-label\"],[9],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"flex items-center flex-auto\"],[9],[0,\"\\n        \"],[7,\"h4\"],[11,\"class\",\"theme-validation-rule-text\"],[9],[0,\"\\n            \"],[1,[25,[\"error\",\"rule\"]],true],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"theme-validation-rule-icon\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"showDetails\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"svg-jar\",[\"arrow-down\"],null],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n\"]],\"parameters\":[]}],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"toggleDetails\"]],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDetails\"]]],null,{\"statements\":[[0,\"    \"],[7,\"p\"],[11,\"class\",\"theme-validation-details\"],[9],[0,\"\\n        \"],[1,[25,[\"error\",\"details\"]],true],[0,\"\\n    \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"error\",\"failures\"]]],null,{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"theme-validation-list\"],[9],[0,\"\\n            \"],[7,\"h6\"],[9],[0,\"Affected files:\"],[10],[0,\"\\n            \"],[7,\"ul\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"error\",\"failures\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"li\"],[9],[7,\"code\"],[9],[1,[24,1,[\"ref\"]],false],[10],[4,\"if\",[[24,1,[\"message\"]]],null,{\"statements\":[[0,\": \"],[1,[24,1,[\"message\"]],false]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-theme-error-li.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-theme-table", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "RRM6s1MU",
    "block": "{\"symbols\":[\"theme\"],\"statements\":[[7,\"div\"],[11,\"class\",\"apps-grid\"],[11,\"data-test-themes-list\",\"\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"sortedThemes\"]]],null,{\"statements\":[[0,\"\\n\"],[4,\"each\",[[25,[\"sortedThemes\"]]],null,{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[12,\"data-test-theme-id\",[30,[[24,1,[\"name\"]]]]],[12,\"data-test-theme-active\",[30,[[24,1,[\"active\"]]]]],[9],[0,\"\\n        \"],[7,\"div\"],[12,\"class\",[30,[\"apps-card-app \",[29,\"if\",[[24,1,[\"active\"]],\"theme-list-item--active\"],null]]]],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                    \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[11,\"data-test-theme-title\",\"\"],[9],[1,[24,1,[\"label\"]],false],[10],[0,\"\\n                    \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[11,\"data-test-theme-description\",\"\"],[9],[7,\"span\"],[11,\"class\",\"description\"],[9],[0,\"Version \"],[1,[24,1,[\"version\"]],false],[10],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"apps-card-right\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n\"],[4,\"if\",[[24,1,[\"isDeletable\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"a\"],[11,\"href\",\"#\"],[12,\"disabled\",[24,1,[\"active\"]]],[11,\"class\",\"apps-configured-action red-hover\"],[11,\"data-test-theme-delete-button\",\"\"],[9],[0,\"Delete\"],[3,\"action\",[[24,0,[]],[25,[\"deleteTheme\"]],[24,1,[]]]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[7,\"a\"],[11,\"href\",\"#\"],[11,\"class\",\"apps-configured-action darkgrey-hover\"],[11,\"data-test-theme-download-button\",\"\"],[9],[0,\"Download\"],[3,\"action\",[[24,0,[]],[25,[\"downloadTheme\"]],[24,1,[]]]],[10],[0,\"\\n\"],[4,\"if\",[[24,1,[\"active\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"span\"],[11,\"class\",\"gh-badge gh-badge-black apps-configured-action\"],[11,\"data-test-theme-badge\",\"\"],[9],[0,\"Active\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"a\"],[11,\"href\",\"#\"],[11,\"class\",\"apps-configured-action apps-configured-action-activate green-hover\"],[11,\"data-test-theme-activate-button\",\"\"],[9],[0,\"\\n                            Activate\\n                        \"],[3,\"action\",[[24,0,[]],[25,[\"activateTheme\"]],[24,1,[\"model\"]]]],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[0,\"No themes found\"],[10],[0,\"\\n                \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[7,\"span\"],[11,\"class\",\"description\"],[9],[0,\"Please upload a theme to continue\"],[10],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-theme-table.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-timezone-select", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "VYRbcGAS",
    "block": "{\"symbols\":[],\"statements\":[[7,\"span\"],[11,\"class\",\"gh-select\"],[12,\"data-select-text\",[30,[[25,[\"selectedTimezone\",\"label\"]]]]],[11,\"tabindex\",\"0\"],[9],[0,\"\\n    \"],[1,[29,\"one-way-select\",null,[[\"id\",\"name\",\"options\",\"optionValuePath\",\"optionLabelPath\",\"value\",\"update\"],[\"activeTimezone\",\"general[activeTimezone]\",[25,[\"selectableTimezones\"]],\"name\",\"label\",[25,[\"selectedTimezone\"]],[29,\"action\",[[24,0,[]],\"setTimezone\"],null]]]],false],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n\"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"hasTimezoneOverride\"]]],null,{\"statements\":[[0,\"    \"],[7,\"p\"],[9],[0,\"Your timezone has been automatically set to \"],[1,[23,\"activeTimezone\"],false],[0,\".\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[7,\"p\"],[9],[0,\"The local time here is currently \"],[1,[23,\"localTime\"],false],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-timezone-select.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-token-input", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "UJR2BqET",
    "block": "{\"symbols\":[\"option\",\"term\"],\"statements\":[[4,\"gh-token-input/select-multiple\",null,[[\"afterOptionsComponent\",\"allowClear\",\"ariaDescribedBy\",\"ariaInvalid\",\"ariaLabel\",\"ariaLabelledBy\",\"beforeOptionsComponent\",\"class\",\"closeOnSelect\",\"defaultHighlighted\",\"destination\",\"dir\",\"disabled\",\"dropdownClass\",\"extra\",\"horizontalPosition\",\"initiallyOpened\",\"loadingMessage\",\"matcher\",\"matchTriggerWidth\",\"noMatchesMessage\",\"onblur\",\"onchange\",\"onclose\",\"onfocus\",\"oninput\",\"onkeydown\",\"onopen\",\"options\",\"optionsComponent\",\"placeholder\",\"registerAPI\",\"renderInPlace\",\"search\",\"searchEnabled\",\"searchField\",\"searchMessage\",\"searchPlaceholder\",\"selected\",\"selectedItemComponent\",\"tabindex\",\"triggerClass\",\"triggerComponent\",\"triggerId\",\"verticalPosition\",\"data-test-token-input\"],[[25,[\"afterOptionsComponent\"]],[25,[\"allowClear\"]],[25,[\"ariaDescribedBy\"]],[25,[\"ariaInvalid\"]],[25,[\"ariaLabel\"]],[25,[\"ariaLabelledBy\"]],[25,[\"beforeOptionsComponent\"]],[29,\"concat\",[\"gh-token-input \",[25,[\"class\"]]],null],[25,[\"closeOnSelect\"]],[25,[\"defaultHighlighted\"]],[25,[\"destination\"]],[25,[\"dir\"]],[25,[\"disabled\"]],[25,[\"dropdownClass\"]],[25,[\"extra\"]],[25,[\"horizontalPosition\"]],[25,[\"initiallyOpened\"]],[25,[\"loadingMessage\"]],[25,[\"matcher\"]],[25,[\"matchTriggerWidth\"]],[25,[\"noMatchesMessage\"]],[29,\"action\",[[24,0,[]],\"onblur\"],null],[29,\"action\",[[24,0,[]],[25,[\"selectOrCreate\"]]],null],[25,[\"onclose\"]],[29,\"action\",[[24,0,[]],\"onfocus\"],null],[25,[\"oninput\"]],[29,\"action\",[[24,0,[]],\"handleKeydown\"],null],[25,[\"onopen\"]],[25,[\"optionsWithoutSelected\"]],[29,\"or\",[[25,[\"optionsComponent\"]],\"power-select-vertical-collection-options\"],null],[25,[\"placeholder\"]],[25,[\"registerAPI\"]],[25,[\"renderInPlace\"]],[29,\"action\",[[24,0,[]],[25,[\"searchAndSuggest\"]]],null],[25,[\"searchEnabled\"]],[25,[\"searchField\"]],[25,[\"searchMessage\"]],[25,[\"searchPlaceholder\"]],[25,[\"selected\"]],[25,[\"selectedItemComponent\"]],[25,[\"tabindex\"]],[25,[\"triggerClass\"]],[25,[\"triggerComponent\"]],[25,[\"triggerId\"]],[25,[\"verticalPosition\"]],true]],{\"statements\":[[4,\"if\",[[24,1,[\"__isSuggestion__\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-token-input/suggested-option\",null,[[\"option\",\"term\"],[[24,1,[]],[24,2,[]]]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        \"],[1,[29,\"get\",[[24,1,[]],[25,[\"labelField\"]]],null],false],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[1,2]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-token-input.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-token-input/select-multiple", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "AIpHq45N",
    "block": "{\"symbols\":[\"option\",\"select\",\"option\",\"select\",\"&default\",\"&inverse\"],\"statements\":[[4,\"if\",[[26,6]],null,{\"statements\":[[4,\"gh-token-input/select\",null,[[\"afterOptionsComponent\",\"allowClear\",\"ariaDescribedBy\",\"ariaInvalid\",\"ariaLabel\",\"ariaLabelledBy\",\"beforeOptionsComponent\",\"buildSelection\",\"calculatePosition\",\"class\",\"closeOnSelect\",\"defaultHighlighted\",\"destination\",\"dir\",\"disabled\",\"dropdownClass\",\"extra\",\"horizontalPosition\",\"initiallyOpened\",\"loadingMessage\",\"matcher\",\"matchTriggerWidth\",\"noMatchesMessage\",\"onblur\",\"onchange\",\"onclose\",\"onfocus\",\"oninput\",\"onkeydown\",\"onopen\",\"options\",\"optionsComponent\",\"groupComponent\",\"placeholder\",\"registerAPI\",\"renderInPlace\",\"required\",\"scrollTo\",\"search\",\"searchEnabled\",\"searchField\",\"searchMessage\",\"searchPlaceholder\",\"selected\",\"selectedItemComponent\",\"tabindex\",\"tagName\",\"triggerClass\",\"triggerComponent\",\"triggerId\",\"verticalPosition\"],[[25,[\"afterOptionsComponent\"]],[25,[\"allowClear\"]],[25,[\"ariaDescribedBy\"]],[25,[\"ariaInvalid\"]],[25,[\"ariaLabel\"]],[25,[\"ariaLabelledBy\"]],[25,[\"beforeOptionsComponent\"]],[29,\"action\",[[24,0,[]],\"buildSelection\"],null],[25,[\"calculatePosition\"]],[25,[\"class\"]],[25,[\"closeOnSelect\"]],[25,[\"defaultHighlighted\"]],[25,[\"destination\"]],[25,[\"dir\"]],[25,[\"disabled\"]],[25,[\"dropdownClass\"]],[29,\"assign\",[[25,[\"extra\"]],[29,\"hash\",null,[[\"optionMouseDown\",\"optionTouchStart\"],[[29,\"action\",[[24,0,[]],\"optionMouseDown\"],null],[29,\"action\",[[24,0,[]],\"optionTouchStart\"],null]]]]],null],[25,[\"horizontalPosition\"]],[25,[\"initiallyOpened\"]],[25,[\"loadingMessage\"]],[25,[\"matcher\"]],[25,[\"matchTriggerWidth\"]],[25,[\"noMatchesMessage\"]],[25,[\"onblur\"]],[25,[\"onchange\"]],[25,[\"onclose\"]],[29,\"action\",[[24,0,[]],\"handleFocus\"],null],[25,[\"oninput\"]],[29,\"action\",[[24,0,[]],\"handleKeydown\"],null],[29,\"action\",[[24,0,[]],\"handleOpen\"],null],[25,[\"options\"]],[25,[\"optionsComponent\"]],[25,[\"groupComponent\"]],[25,[\"placeholder\"]],[29,\"readonly\",[[25,[\"registerAPI\"]]],null],[25,[\"renderInPlace\"]],[25,[\"required\"]],[25,[\"scrollTo\"]],[25,[\"search\"]],[25,[\"searchEnabled\"]],[25,[\"searchField\"]],[25,[\"searchMessage\"]],[25,[\"searchPlaceholder\"]],[25,[\"selected\"]],[25,[\"selectedItemComponent\"]],[25,[\"computedTabIndex\"]],[25,[\"tagName\"]],[25,[\"concatenatedTriggerClass\"]],[29,\"component\",[[25,[\"triggerComponent\"]]],[[\"tabindex\"],[[25,[\"tabindex\"]]]]],[25,[\"triggerId\"]],[25,[\"verticalPosition\"]]]],{\"statements\":[[0,\"    \"],[15,5,[[24,3,[]],[24,4,[]]]],[0,\"\\n\"]],\"parameters\":[3,4]},{\"statements\":[[0,\"    \"],[15,6],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]},{\"statements\":[[4,\"gh-token-input/select\",null,[[\"afterOptionsComponent\",\"allowClear\",\"ariaDescribedBy\",\"ariaInvalid\",\"ariaLabel\",\"ariaLabelledBy\",\"beforeOptionsComponent\",\"buildSelection\",\"calculatePosition\",\"class\",\"closeOnSelect\",\"defaultHighlighted\",\"destination\",\"dir\",\"disabled\",\"dropdownClass\",\"extra\",\"horizontalPosition\",\"initiallyOpened\",\"loadingMessage\",\"matcher\",\"matchTriggerWidth\",\"noMatchesMessage\",\"onblur\",\"onchange\",\"onclose\",\"onfocus\",\"oninput\",\"onkeydown\",\"onopen\",\"options\",\"optionsComponent\",\"groupComponent\",\"placeholder\",\"registerAPI\",\"renderInPlace\",\"required\",\"scrollTo\",\"search\",\"searchEnabled\",\"searchField\",\"searchMessage\",\"searchPlaceholder\",\"selected\",\"selectedItemComponent\",\"tabindex\",\"tagName\",\"triggerClass\",\"triggerComponent\",\"triggerId\",\"verticalPosition\"],[[25,[\"afterOptionsComponent\"]],[25,[\"allowClear\"]],[25,[\"ariaDescribedBy\"]],[25,[\"ariaInvalid\"]],[25,[\"ariaLabel\"]],[25,[\"ariaLabelledBy\"]],[25,[\"beforeOptionsComponent\"]],[29,\"action\",[[24,0,[]],\"buildSelection\"],null],[25,[\"calculatePosition\"]],[25,[\"class\"]],[25,[\"closeOnSelect\"]],[25,[\"defaultHighlighted\"]],[25,[\"destination\"]],[25,[\"dir\"]],[25,[\"disabled\"]],[25,[\"dropdownClass\"]],[29,\"assign\",[[25,[\"extra\"]],[29,\"hash\",null,[[\"optionMouseDown\",\"optionTouchStart\"],[[29,\"action\",[[24,0,[]],\"optionMouseDown\"],null],[29,\"action\",[[24,0,[]],\"optionTouchStart\"],null]]]]],null],[25,[\"horizontalPosition\"]],[25,[\"initiallyOpened\"]],[25,[\"loadingMessage\"]],[25,[\"matcher\"]],[25,[\"matchTriggerWidth\"]],[25,[\"noMatchesMessage\"]],[25,[\"onblur\"]],[25,[\"onchange\"]],[25,[\"onclose\"]],[29,\"action\",[[24,0,[]],\"handleFocus\"],null],[25,[\"oninput\"]],[29,\"action\",[[24,0,[]],\"handleKeydown\"],null],[29,\"action\",[[24,0,[]],\"handleOpen\"],null],[25,[\"options\"]],[25,[\"optionsComponent\"]],[25,[\"groupComponent\"]],[25,[\"placeholder\"]],[29,\"readonly\",[[25,[\"registerAPI\"]]],null],[25,[\"renderInPlace\"]],[25,[\"required\"]],[25,[\"scrollTo\"]],[25,[\"search\"]],[25,[\"searchEnabled\"]],[25,[\"searchField\"]],[25,[\"searchMessage\"]],[25,[\"searchPlaceholder\"]],[25,[\"selected\"]],[25,[\"selectedItemComponent\"]],[25,[\"computedTabIndex\"]],[25,[\"tagName\"]],[25,[\"concatenatedTriggerClass\"]],[29,\"component\",[[25,[\"triggerComponent\"]]],[[\"tabindex\"],[[25,[\"tabindex\"]]]]],[25,[\"triggerId\"]],[25,[\"verticalPosition\"]]]],{\"statements\":[[0,\"    \"],[15,5,[[24,1,[]],[24,2,[]]]],[0,\"\\n\"]],\"parameters\":[1,2]},null]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-token-input/select-multiple.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-token-input/select", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "ks9aJYeh",
    "block": "{\"symbols\":[\"dropdown\",\"option\",\"term\",\"opt\",\"term\",\"&default\",\"&inverse\"],\"statements\":[[4,\"basic-dropdown\",null,[[\"classNames\",\"horizontalPosition\",\"calculatePosition\",\"destination\",\"initiallyOpened\",\"matchTriggerWidth\",\"onClose\",\"onOpen\",\"registerAPI\",\"renderInPlace\",\"verticalPosition\",\"disabled\"],[[29,\"readonly\",[[25,[\"classNames\"]]],null],[29,\"readonly\",[[25,[\"horizontalPosition\"]]],null],[25,[\"calculatePosition\"]],[29,\"readonly\",[[25,[\"destination\"]]],null],[29,\"readonly\",[[25,[\"initiallyOpened\"]]],null],[29,\"readonly\",[[25,[\"matchTriggerWidth\"]]],null],[29,\"action\",[[24,0,[]],\"onClose\"],null],[29,\"action\",[[24,0,[]],\"onOpen\"],null],[29,\"action\",[[24,0,[]],\"registerAPI\"],null],[29,\"readonly\",[[25,[\"renderInPlace\"]]],null],[29,\"readonly\",[[25,[\"verticalPosition\"]]],null],[29,\"readonly\",[[25,[\"disabled\"]]],null]]],{\"statements\":[[0,\"\\n\"],[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"trigger\"]],\"expected `dropdown.trigger` to be a contextual component but found a string. Did you mean `(component dropdown.trigger)`? ('ghost-admin/templates/components/gh-token-input/select.hbs' @ L23:C5) \"],null]],[[\"tagName\",\"ariaDescribedBy\",\"ariaInvalid\",\"ariaLabel\",\"ariaLabelledBy\",\"ariaRequired\",\"class\",\"id\",\"eventType\",\"onKeyDown\",\"onFocus\",\"onBlur\",\"tabindex\"],[[29,\"readonly\",[[25,[\"_triggerTagName\"]]],null],[29,\"readonly\",[[25,[\"ariaDescribedBy\"]]],null],[29,\"readonly\",[[25,[\"ariaInvalid\"]]],null],[29,\"readonly\",[[25,[\"ariaLabel\"]]],null],[29,\"readonly\",[[25,[\"ariaLabelledBy\"]]],null],[29,\"readonly\",[[25,[\"required\"]]],null],[29,\"readonly\",[[25,[\"concatenatedTriggerClasses\"]]],null],[29,\"readonly\",[[25,[\"triggerId\"]]],null],\"click\",[29,\"action\",[[24,0,[]],\"onTriggerKeydown\"],null],[29,\"action\",[[24,0,[]],\"onTriggerFocus\"],null],[29,\"action\",[[24,0,[]],\"onTriggerBlur\"],null],[29,\"readonly\",[[25,[\"tabindex\"]]],null]]],{\"statements\":[[4,\"component\",[[25,[\"triggerComponent\"]]],[[\"allowClear\",\"buildSelection\",\"extra\",\"listboxId\",\"loadingMessage\",\"onFocus\",\"onBlur\",\"onInput\",\"placeholder\",\"placeholderComponent\",\"onKeydown\",\"searchEnabled\",\"searchField\",\"select\",\"selectedItemComponent\"],[[29,\"readonly\",[[25,[\"allowClear\"]]],null],[29,\"readonly\",[[25,[\"buildSelection\"]]],null],[29,\"readonly\",[[25,[\"extra\"]]],null],[29,\"readonly\",[[25,[\"optionsId\"]]],null],[29,\"readonly\",[[25,[\"loadingMessage\"]]],null],[29,\"action\",[[24,0,[]],\"onFocus\"],null],[29,\"action\",[[24,0,[]],\"onBlur\"],null],[29,\"action\",[[24,0,[]],\"onInput\"],null],[29,\"readonly\",[[25,[\"placeholder\"]]],null],[29,\"readonly\",[[25,[\"placeholderComponent\"]]],null],[29,\"action\",[[24,0,[]],\"onKeydown\"],null],[29,\"readonly\",[[25,[\"searchEnabled\"]]],null],[29,\"readonly\",[[25,[\"searchField\"]]],null],[29,\"readonly\",[[25,[\"publicAPI\"]]],null],[29,\"readonly\",[[25,[\"selectedItemComponent\"]]],null]]],{\"statements\":[[0,\"      \"],[15,6,[[24,4,[]],[24,5,[]]]],[0,\"\\n\"]],\"parameters\":[4,5]},null]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"component\",[[29,\"-assert-implicit-component-helper-argument\",[[24,1,[\"content\"]],\"expected `dropdown.content` to be a contextual component but found a string. Did you mean `(component dropdown.content)`? ('ghost-admin/templates/components/gh-token-input/select.hbs' @ L58:C5) \"],null]],[[\"_contentTagName\",\"class\"],[[25,[\"_contentTagName\"]],[29,\"readonly\",[[25,[\"concatenatedDropdownClasses\"]]],null]]],{\"statements\":[[0,\"    \"],[1,[29,\"component\",[[25,[\"beforeOptionsComponent\"]]],[[\"extra\",\"listboxId\",\"onInput\",\"onKeydown\",\"searchEnabled\",\"onFocus\",\"onBlur\",\"placeholder\",\"placeholderComponent\",\"searchPlaceholder\",\"select\"],[[29,\"readonly\",[[25,[\"extra\"]]],null],[29,\"readonly\",[[25,[\"optionsId\"]]],null],[29,\"action\",[[24,0,[]],\"onInput\"],null],[29,\"action\",[[24,0,[]],\"onKeydown\"],null],[29,\"readonly\",[[25,[\"searchEnabled\"]]],null],[29,\"action\",[[24,0,[]],\"onFocus\"],null],[29,\"action\",[[24,0,[]],\"onBlur\"],null],[29,\"readonly\",[[25,[\"placeholder\"]]],null],[29,\"readonly\",[[25,[\"placeholderComponent\"]]],null],[29,\"readonly\",[[25,[\"searchPlaceholder\"]]],null],[29,\"readonly\",[[25,[\"publicAPI\"]]],null]]]],false],[0,\"\\n\"],[4,\"if\",[[25,[\"mustShowSearchMessage\"]]],null,{\"statements\":[[0,\"      \"],[1,[29,\"component\",[[25,[\"searchMessageComponent\"]]],[[\"searchMessage\",\"select\"],[[29,\"readonly\",[[25,[\"searchMessage\"]]],null],[29,\"readonly\",[[25,[\"publicAPI\"]]],null]]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"mustShowNoMessages\"]]],null,{\"statements\":[[4,\"if\",[[26,7]],null,{\"statements\":[[0,\"        \"],[15,7],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"noMatchesMessage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"ul\"],[11,\"class\",\"ember-power-select-options\"],[11,\"role\",\"listbox\"],[9],[0,\"\\n          \"],[7,\"li\"],[11,\"class\",\"ember-power-select-option ember-power-select-option--no-matches-message\"],[11,\"role\",\"option\"],[9],[0,\"\\n            \"],[1,[23,\"noMatchesMessage\"],false],[0,\"\\n          \"],[10],[0,\"\\n        \"],[10],[0,\"\\n      \"]],\"parameters\":[]},null]],\"parameters\":[]}]],\"parameters\":[]},{\"statements\":[[4,\"component\",[[25,[\"optionsComponent\"]]],[[\"class\",\"extra\",\"groupIndex\",\"loadingMessage\",\"id\",\"options\",\"optionsComponent\",\"groupComponent\",\"select\"],[\"ember-power-select-options\",[29,\"readonly\",[[25,[\"extra\"]]],null],\"\",[29,\"readonly\",[[25,[\"loadingMessage\"]]],null],[29,\"readonly\",[[25,[\"optionsId\"]]],null],[29,\"readonly\",[[25,[\"publicAPI\",\"results\"]]],null],[29,\"readonly\",[[25,[\"optionsComponent\"]]],null],[29,\"readonly\",[[25,[\"groupComponent\"]]],null],[29,\"readonly\",[[25,[\"publicAPI\"]]],null]]],{\"statements\":[[0,\"        \"],[15,6,[[24,2,[]],[24,3,[]]]],[0,\"\\n\"]],\"parameters\":[2,3]},null],[0,\"    \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"    \"],[1,[29,\"component\",[[25,[\"afterOptionsComponent\"]]],[[\"select\",\"extra\"],[[29,\"readonly\",[[25,[\"publicAPI\"]]],null],[29,\"readonly\",[[25,[\"extra\"]]],null]]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[1]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-token-input/select.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-token-input/suggested-option", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "emxCb/lV",
    "block": "{\"symbols\":[],\"statements\":[[1,[25,[\"option\",\"text\"]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-token-input/suggested-option.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-token-input/tag-token", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "G08RRNIq",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-token-input/tag-token.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-token-input/trigger", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "+J9hVFL/",
    "block": "{\"symbols\":[\"opt\",\"idx\",\"&default\"],\"statements\":[[4,\"sortable-objects\",null,[[\"tagName\",\"id\",\"class\",\"sortableObjectList\",\"enableSort\",\"useSwap\",\"sortEndAction\"],[\"ul\",[29,\"concat\",[\"ember-power-select-multiple-options-\",[25,[\"select\",\"uniqueId\"]]],null],\"ember-power-select-multiple-options\",[25,[\"select\",\"selected\"]],true,false,[29,\"action\",[[24,0,[]],\"reorderItems\"],null]]],{\"statements\":[[4,\"each\",[[25,[\"select\",\"selected\"]]],null,{\"statements\":[[4,\"component\",[[29,\"or\",[[25,[\"extra\",\"tokenComponent\"]],\"draggable-object\"],null]],[[\"tagName\",\"class\",\"select\",\"content\",\"idx\",\"isSortable\",\"mouseDown\",\"touchStart\"],[\"li\",\"ember-power-select-multiple-option\",[25,[\"select\"]],[29,\"readonly\",[[24,1,[]]],null],[24,2,[]],true,[29,\"action\",[[24,0,[]],\"handleOptionMouseDown\"],null],[29,\"action\",[[24,0,[]],\"handleOptionTouchStart\"],null]]],{\"statements\":[[4,\"if\",[[25,[\"selectedItemComponent\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"component\",[[25,[\"selectedItemComponent\"]]],[[\"option\",\"select\"],[[29,\"readonly\",[[24,1,[]]],null],[29,\"readonly\",[[25,[\"select\"]]],null]]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                \"],[15,3,[[24,1,[]],[25,[\"select\"]]]],[0,\"\\n\"]],\"parameters\":[]}],[4,\"unless\",[[25,[\"select\",\"disabled\"]]],null,{\"statements\":[[0,\"                \"],[7,\"span\"],[11,\"role\",\"button\"],[11,\"aria-label\",\"remove element\"],[11,\"class\",\"ember-power-select-multiple-remove-btn\"],[12,\"data-selected-index\",[24,2,[]]],[9],[0,\"\\n                    \"],[1,[29,\"svg-jar\",[\"close\"],[[\"data-selected-index\"],[[24,2,[]]]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null]],\"parameters\":[1,2]},{\"statements\":[[4,\"if\",[[29,\"and\",[[25,[\"placeholder\"]],[29,\"not\",[[25,[\"searchEnabled\"]]],null]],null]],null,{\"statements\":[[0,\"            \"],[7,\"span\"],[11,\"class\",\"ember-power-select-placeholder\"],[9],[1,[23,\"placeholder\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]}],[0,\"\\n\"],[4,\"if\",[[25,[\"searchEnabled\"]]],null,{\"statements\":[[0,\"        \"],[7,\"input\"],[11,\"class\",\"ember-power-select-trigger-multiple-input\"],[11,\"tabindex\",\"0\"],[11,\"autocomplete\",\"off\"],[11,\"autocorrect\",\"off\"],[11,\"autocapitalize\",\"off\"],[11,\"spellcheck\",\"false\"],[12,\"id\",[30,[\"ember-power-select-trigger-multiple-input-\",[25,[\"select\",\"uniqueId\"]]]]],[12,\"value\",[25,[\"select\",\"searchText\"]]],[12,\"aria-controls\",[23,\"listboxId\"]],[12,\"style\",[23,\"triggerMultipleInputStyle\"]],[12,\"placeholder\",[23,\"maybePlaceholder\"]],[12,\"disabled\",[25,[\"select\",\"disabled\"]]],[12,\"oninput\",[29,\"action\",[[24,0,[]],\"onInput\"],null]],[12,\"onFocus\",[23,\"onFocus\"]],[12,\"onBlur\",[23,\"onBlur\"]],[12,\"tabindex\",[23,\"tabindex\"]],[12,\"onkeydown\",[29,\"action\",[[24,0,[]],\"onKeydown\"],null]],[11,\"type\",\"search\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[7,\"span\"],[11,\"class\",\"ember-power-select-status-icon\"],[9],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-token-input/trigger.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-tour-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "Q0GMxPn+",
    "block": "{\"symbols\":[],\"statements\":[[4,\"if\",[[25,[\"isVisible\"]]],null,{\"statements\":[[4,\"liquid-tether\",null,[[\"class\",\"target\",\"attachment\",\"targetAttachment\",\"targetOffset\"],[\"throbber-container\",[25,[\"target\"]],\"middle center\",[25,[\"throbberAttachment\"]],[25,[\"throbberOffset\"]]]],{\"statements\":[[0,\"        \"],[7,\"a\"],[11,\"class\",\"throbber-trigger\"],[11,\"href\",\"#\"],[12,\"id\",[23,\"_throbberElementId\"]],[9],[0,\"\\n            \"],[7,\"span\"],[11,\"class\",\"throbber\"],[9],[10],[0,\"\\n        \"],[3,\"action\",[[24,0,[]],\"open\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"isOpen\"]]],null,{\"statements\":[[4,\"liquid-wormhole\",null,[[\"class\"],[\"tour-container\"]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"tour-background\"],[9],[3,\"action\",[[24,0,[]],\"close\"],[[\"on\"],[\"click\"]]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"liquid-tether\",null,[[\"class\",\"target\",\"attachment\",\"targetAttachment\",\"offset\"],[\"tour\",[25,[\"_throbberElementSelector\"]],[25,[\"_popoverAttachment\"]],[25,[\"_popoverTargetAttachment\"]],[25,[\"_popoverOffset\"]]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[12,\"class\",[30,[\"popover-item popover-triangle-\",[23,\"popoverTriangleClass\"]]]],[9],[0,\"\\n                \"],[7,\"h3\"],[11,\"class\",\"popover-title\"],[9],[1,[25,[\"_throbber\",\"title\"]],false],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"popover-body\"],[9],[0,\"\\n                    \"],[1,[25,[\"_throbber\",\"message\"]],true],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"footer\"],[11,\"class\",\"popover-foot\"],[9],[0,\"\\n                    \"],[7,\"a\"],[11,\"class\",\"tour-optout\"],[11,\"href\",\"#\"],[9],[0,\"Not your first time? \"],[7,\"em\"],[9],[0,\"Skip these tips\"],[10],[3,\"action\",[[24,0,[]],\"optOut\"]],[10],[0,\"\\n                    \"],[7,\"a\"],[11,\"class\",\"tour-dismiss gh-btn gh-btn-black\"],[11,\"href\",\"#\"],[9],[7,\"span\"],[9],[0,\"Ok, got it\"],[10],[3,\"action\",[[24,0,[]],\"markAsViewed\"]],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-tour-item.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-unsplash-photo", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "tlz/EZCJ",
    "block": "{\"symbols\":[],\"statements\":[[7,\"a\"],[11,\"class\",\"gh-unsplash-photo\"],[11,\"href\",\"#\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"zoom\"],null]],[12,\"data-unsplash-zoomed-photo\",[29,\"if\",[[25,[\"zoomed\"]],[25,[\"photo\",\"id\"]]],null]],[12,\"data-test-unsplash-photo\",[25,[\"photo\",\"id\"]]],[12,\"style\",[23,\"style\"]],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-photo-container\"],[12,\"style\",[23,\"containerStyle\"]],[11,\"data-test-unsplash-photo-container\",\"\"],[9],[0,\"\\n        \"],[7,\"img\"],[12,\"src\",[23,\"imageUrl\"]],[12,\"alt\",[25,[\"photo\",\"description\"]]],[12,\"width\",[23,\"width\"]],[12,\"height\",[23,\"height\"]],[11,\"data-test-unsplash-photo-image\",\"\"],[9],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-photo-overlay\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-photo-header\"],[9],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"gh-unsplash-button-likes gh-unsplash-button\"],[12,\"href\",[30,[[25,[\"photo\",\"links\",\"html\"]],\"?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit\"]]],[11,\"target\",\"_blank\"],[9],[1,[29,\"svg-jar\",[\"unsplash-heart\"],null],false],[1,[25,[\"photo\",\"likes\"]],false],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"gh-unsplash-button-download gh-unsplash-button\"],[12,\"href\",[30,[[25,[\"photo\",\"links\",\"download\"]],\"/?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit&force=true\"]]],[9],[1,[29,\"svg-jar\",[\"download\"],null],false],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-photo-footer\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-photo-author\"],[9],[0,\"\\n                    \"],[7,\"a\"],[11,\"class\",\"gh-unsplash-photo-author-img\"],[12,\"href\",[30,[[25,[\"photo\",\"user\",\"links\",\"html\"]],\"?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit\"]]],[11,\"target\",\"_blank\"],[9],[0,\"\\n                        \"],[7,\"img\"],[12,\"src\",[30,[[25,[\"photo\",\"user\",\"profile_image\",\"medium\"]]]]],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"a\"],[11,\"class\",\"gh-unsplash-photo-author-name\"],[12,\"href\",[30,[[25,[\"photo\",\"user\",\"links\",\"html\"]],\"?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit\"]]],[11,\"target\",\"_blank\"],[9],[0,\"\\n                        \"],[1,[25,[\"photo\",\"user\",\"name\"]],false],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"gh-unsplash-button\"],[11,\"href\",\"#\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"select\"],null]],[9],[0,\"Insert image\"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-unsplash-photo.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-unsplash", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "q5AnwCp4",
    "block": "{\"symbols\":[\"photos\",\"photo\"],\"statements\":[[4,\"liquid-wormhole\",null,[[\"class\"],[\"unsplash\"]],{\"statements\":[[0,\"    \"],[7,\"div\"],[11,\"class\",\"fullscreen-modal-background\"],[11,\"style\",\"z-index: 999\"],[9],[3,\"action\",[[24,0,[]],\"close\"]],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"absolute top-8 right-8 bottom-8 left-8 br4 overflow-hidden bg-white z-9999\"],[11,\"data-unsplash\",\"\"],[9],[0,\"\\n\"],[0,\"        \"],[7,\"button\"],[11,\"class\",\"absolute top-6 right-6\"],[11,\"type\",\"button\"],[9],[0,\"\\n            \"],[1,[29,\"svg-jar\",[\"close\"],[[\"class\"],[\"w4 stroke-midlightgrey-l2\"]]],false],[0,\"\\n        \"],[3,\"action\",[[24,0,[]],\"close\"]],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"flex flex-column h-100\"],[9],[0,\"\\n\"],[0,\"            \"],[7,\"header\"],[11,\"class\",\"flex-shrink-0 flex flex-row-l flex-column justify-between pt6 pr8 pb6 pl8 pt10-l pr20-l pb10-l pl20-l items-center\"],[9],[0,\"\\n                \"],[7,\"h1\"],[11,\"class\",\"flex items-center darkgrey-d2 w-100 nudge-top--4\"],[9],[0,\"\\n                    \"],[7,\"a\"],[11,\"class\",\"dib w8 mr2\"],[11,\"href\",\"https://unsplash.com/?utm_source=ghost&utm_medium=referral&utm_campaign=api-credit\"],[11,\"target\",\"_blank\"],[9],[1,[29,\"svg-jar\",[\"unsplash\"],[[\"class\"],[\"fill-darkgrey-d2\"]]],false],[10],[0,\"\\n                    Unsplash\\n                \"],[10],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"gh-input-icon mw88-l flex-auto w-100 mt3 mt0-l\"],[9],[0,\"\\n                    \"],[1,[29,\"svg-jar\",[\"search\"],null],false],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"name\",\"placeholder\",\"tabindex\",\"autofocus\",\"autocorrect\",\"value\",\"input\",\"focusIn\",\"focus-out\"],[\"gh-unsplash-search\",\"searchKeyword\",\"Search free high-resolution photos\",\"1\",\"autofocus\",\"off\",[29,\"readonly\",[[25,[\"unsplash\",\"searchTerm\"]]],null],[29,\"action\",[[24,0,[]],\"search\"],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"setKeyScope\"],null],[29,\"action\",[[24,0,[]],\"resetKeyScope\"],null]]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n\"],[0,\"            \"],[7,\"div\"],[11,\"class\",\"relative h-100 overflow-hidden\"],[9],[0,\"\\n\"],[0,\"                \"],[7,\"div\"],[11,\"class\",\"overflow-auto h-100 w-100 pr8 pl8 pr20-l pl20-l\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"unsplash\",\"photos\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"section\"],[11,\"class\",\"gh-unsplash-grid\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"unsplash\",\"columns\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-grid-column\"],[9],[0,\"\\n\"],[4,\"each\",[[24,1,[]]],null,{\"statements\":[[0,\"                                        \"],[1,[29,\"gh-unsplash-photo\",null,[[\"photo\",\"zoom\",\"select\"],[[24,2,[]],[29,\"action\",[[24,0,[]],\"zoomPhoto\"],null],[29,\"action\",[[24,0,[]],\"select\"],null]]]],false],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"                                \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[29,\"and\",[[25,[\"unsplash\",\"searchTerm\"]],[29,\"not\",[[25,[\"unsplash\",\"error\"]],[25,[\"unsplash\",\"isLoading\"]]],null]],null]],null,{\"statements\":[[0,\"                        \"],[7,\"section\"],[11,\"class\",\"gh-unsplash-error h-100 flex items-center justify-center pb30\"],[9],[0,\"\\n                            \"],[7,\"div\"],[9],[0,\"\\n                                \"],[7,\"img\"],[11,\"class\",\"gh-unsplash-error-404\"],[11,\"src\",\"assets/img/unsplash-404.png\"],[11,\"alt\",\"No photos found\"],[9],[10],[0,\"\\n                                \"],[7,\"h4\"],[9],[0,\"No photos found for '\"],[1,[25,[\"unsplash\",\"searchTerm\"]],false],[0,\"'\"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"]],\"parameters\":[]},null]],\"parameters\":[]}],[0,\"\\n\"],[4,\"if\",[[25,[\"unsplash\",\"error\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"section\"],[11,\"class\",\"gh-unsplash-error h-100 flex items-center justify-center pb30\"],[9],[0,\"\\n                            \"],[7,\"div\"],[9],[0,\"\\n                                \"],[7,\"img\"],[11,\"class\",\"gh-unsplash-error-404\"],[11,\"src\",\"assets/img/unsplash-404.png\"],[11,\"alt\",\"Network error\"],[9],[10],[0,\"\\n                                \"],[7,\"h4\"],[9],[1,[25,[\"unsplash\",\"error\"]],false],[0,\" (\"],[7,\"a\"],[11,\"href\",\"#\"],[9],[0,\"retry\"],[3,\"action\",[[24,0,[]],\"retry\"]],[10],[0,\")\"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"unsplash\",\"isLoading\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"gh-unsplash-loading h-100 flex items-center justify-center pb30\"],[9],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"gh-loading-spinner\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                    \"],[1,[29,\"gh-scroll-trigger\",null,[[\"enter\",\"triggerOffset\"],[[29,\"action\",[[24,0,[]],\"loadNextPage\"],null],1000]]],false],[0,\"\\n                \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"zoomedPhoto\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"absolute flex justify-center top-0 right-0 bottom-0 left-0 pr20 pb10 pl20 bg-white overflow-hidden\"],[9],[0,\"\\n                        \"],[1,[29,\"gh-unsplash-photo\",null,[[\"photo\",\"zoomed\",\"zoom\",\"select\"],[[25,[\"zoomedPhoto\"]],true,[29,\"action\",[[24,0,[]],\"closeZoom\"],null],[29,\"action\",[[24,0,[]],\"select\"],null]]]],false],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"closeZoom\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-unsplash.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-uploader", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "ph8PnULG",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1,[[29,\"hash\",null,[[\"cancel\",\"errors\",\"files\",\"isUploading\",\"progressBar\",\"setFiles\"],[[29,\"action\",[[24,0,[]],\"cancel\"],null],[25,[\"errors\"]],[25,[\"files\"]],[25,[\"_uploadFiles\",\"isRunning\"]],[29,\"component\",[\"gh-progress-bar\"],[[\"percentage\"],[[25,[\"uploadPercentage\"]]]]],[29,\"action\",[[24,0,[]],\"setFiles\"],null]]]]]],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-uploader.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-url-preview", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "VbzoJSwU",
    "block": "{\"symbols\":[],\"statements\":[[1,[23,\"url\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-url-preview.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-user-active", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "CVaiI2oE",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1,[[24,0,[]]]],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-user-active.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-user-list-item", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "wrSJiuBy",
    "block": "{\"symbols\":[\"role\"],\"statements\":[[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-user-id\",\"route\",\"model\"],[[25,[\"user\",\"id\"]],\"staff.user\",[25,[\"user\",\"slug\"]]]],{\"statements\":[[0,\"    \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n            \"],[7,\"span\"],[11,\"class\",\"user-list-item-figure\"],[12,\"style\",[29,\"background-image-style\",[[25,[\"user\",\"profileImageUrl\"]]],null]],[9],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Photo of \"],[1,[25,[\"user\",\"name\"]],false],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[1,[25,[\"user\",\"name\"]],false],[10],[0,\"\\n                \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[0,\"Last seen: \"],[1,[25,[\"component\",\"lastLoginUTC\"]],false],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-card-right\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"user\",\"isLocked\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"span\"],[11,\"class\",\"gh-badge author\"],[9],[0,\"Locked\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"unless\",[[25,[\"session\",\"user\",\"isAuthorOrContributor\"]]],null,{\"statements\":[[4,\"each\",[[25,[\"user\",\"roles\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"span\"],[12,\"class\",[30,[\"gh-badge \",[24,1,[\"lowerCaseName\"]]]]],[11,\"data-test-role-name\",\"\"],[9],[1,[24,1,[\"name\"]],false],[10],[0,\"\\n\"]],\"parameters\":[1]},null]],\"parameters\":[]},null],[0,\"                \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-user-list-item.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/gh-view-title", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "aNiv4VyV",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[7,\"button\"],[11,\"class\",\"gh-mobilemenu-button\"],[11,\"role\",\"presentation\"],[9],[1,[29,\"svg-jar\",[\"icon\"],[[\"class\"],[\"icon-gh\"]]],false],[7,\"span\"],[11,\"class\",\"sr-only\"],[9],[0,\"Menu\"],[10],[3,\"action\",[[24,0,[]],\"openMobileMenu\"],[[\"target\"],[[25,[\"ui\"]]]]],[10],[0,\"\\n\"],[15,1],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/gh-view-title.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/infinity-loader", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "FC8nphvF",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[26,1]],null,{\"statements\":[[0,\"  \"],[15,1,[[25,[\"infinityModelContent\"]]]],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"isDoneLoading\"]]],null,{\"statements\":[[0,\"    \"],[7,\"span\"],[9],[1,[23,\"loadedText\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[7,\"span\"],[9],[1,[23,\"loadingText\"],false],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/infinity-loader.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-all", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "rYbaEIFw",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Would you really like to delete all content from your blog?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"This is permanent! No backups, no restores, no magic undo button. We warned you, k?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete\"],[[\"successText\",\"task\",\"class\"],[\"Deleted\",[25,[\"deleteAll\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-all.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-integration", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "QC27peN5",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        Deleting this integration will remove all webhooks and api keys associated with it.\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete Integration\"],[[\"successText\",\"task\",\"class\"],[\"Deleted\",[25,[\"deleteIntegration\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-integration.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-member", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "rbbdkIyW",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to delete this member?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        You're about to delete \\\"\"],[7,\"strong\"],[9],[1,[29,\"or\",[[25,[\"member\",\"name\"]],[25,[\"member\",\"email\"]]],null],false],[10],[0,\"\\\". This is permanent! We warned you, k?\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete\"],[[\"successText\",\"task\",\"class\"],[\"Deleted\",[25,[\"deleteMember\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-member.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-post", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "qzhhUH2B",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to delete this \"],[1,[25,[\"post\",\"displayName\"]],false],[0,\"?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        You're about to delete \\\"\"],[7,\"strong\"],[9],[1,[25,[\"post\",\"title\"]],false],[10],[0,\"\\\". This is permanent! We warned you, k?\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete\"],[[\"successText\",\"task\",\"class\"],[\"Deleted\",[25,[\"deletePost\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-post.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-subscriber", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "xBhAahFp",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"delete-subscriber\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"strong\"],[9],[0,\"WARNING:\"],[10],[0,\" All data for this subscriber will be deleted. There is no way to recover this.\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-button\",\"cancel-delete-subscriber\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\"Cancel\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete\"],[[\"successText\",\"task\",\"class\",\"data-test-button\"],[\"Deleted\",[25,[\"deleteSubscriber\"]],\"gh-btn gh-btn-red gh-btn-icon\",\"confirm-delete-subscriber\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-subscriber.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-tag", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "nwr7JblZ",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to delete this tag?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"tag\",\"post_count\"]]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"red\"],[9],[0,\"This tag is attached to \"],[1,[25,[\"tag\",\"count\",\"posts\"]],false],[0,\" \"],[1,[23,\"postInflection\"],false],[0,\".\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    You're about to delete \\\"\"],[7,\"strong\"],[9],[1,[25,[\"tag\",\"name\"]],false],[10],[0,\"\\\". This is permanent! We warned you, k?\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete\"],[[\"successText\",\"task\",\"class\"],[\"Deleted\",[25,[\"deleteTag\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-tag.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-theme", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "PimU+69s",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-delete-theme-modal\",\"\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to delete this\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"You're about to delete \\\"\"],[7,\"strong\"],[9],[1,[25,[\"theme\",\"label\"]],false],[10],[0,\"\\\". This is permanent! We warned you, k? Maybe \"],[7,\"a\"],[11,\"href\",\"#\"],[9],[0,\"Download your theme before continuing\"],[3,\"action\",[[24,0,[]],[25,[\"download\"]]]],[10],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-cancel-button\",\"\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete\"],[[\"successText\",\"task\",\"class\",\"data-test-delete-button\"],[\"Deleted\",[25,[\"deleteTheme\"]],\"gh-btn gh-btn-red gh-btn-icon\",true]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-theme.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-user", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "rJkH5ATl",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"delete-user\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to delete this user?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[7,\"strong\"],[9],[0,\"WARNING:\"],[10],[0,\" You are about to delete the user '\"],[7,\"strong\"],[9],[1,[25,[\"user\",\"name\"]],false],[10],[0,\"'. There is no way to recover\\n        this.\\n    \"],[10],[0,\"\\n    \"],[7,\"ul\"],[9],[0,\"\\n        \"],[7,\"li\"],[9],[0,\"The user will not have access to this blog anymore\"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"user\",\"count\",\"posts\"]]],null,{\"statements\":[[0,\"            \"],[7,\"li\"],[9],[7,\"strong\"],[11,\"data-test-text\",\"user-post-count\"],[9],[1,[29,\"pluralize\",[[25,[\"user\",\"count\",\"posts\"]],\"post\"],null],false],[10],[0,\" created by this user will be deleted\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[7,\"li\"],[9],[0,\"All other user data will be deleted\"],[10],[0,\"\\n        \"],[7,\"li\"],[9],[0,\"All contributors will be transfered to you\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-button\",\"cancel-delete-user\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\"Cancel\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[[29,\"if\",[[25,[\"user\",\"count\",\"posts\"]],\"Delete user and their posts\",\"Delete user\"],null]],[[\"successText\",\"task\",\"class\",\"data-test-button\"],[\"Deleted\",[25,[\"deleteUser\"]],\"gh-btn gh-btn-red gh-btn-icon\",\"confirm-delete-user\"]]],false],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-user.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-delete-webhook", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "sxd2vkBl",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        Deleting this webhook may prevent the integration from functioning.\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Delete Webhook\"],[[\"successText\",\"task\",\"class\"],[\"Deleted\",[25,[\"deleteWebhook\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-delete-webhook.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-import-subscribers", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "iUQa6DQ4",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"import-subscribers\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"response\"]]],null,{\"statements\":[[0,\"            Import Successful\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"            Import Subscribers\\n\"]],\"parameters\":[]}],[0,\"    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"response\"]]],null,{\"statements\":[[0,\"        \"],[7,\"table\"],[11,\"class\",\"subscribers-import-results\"],[9],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Imported:\"],[10],[0,\"\\n                \"],[7,\"td\"],[11,\"align\",\"left\"],[11,\"data-test-text\",\"import-subscribers-imported\"],[9],[1,[25,[\"response\",\"imported\"]],false],[10],[0,\"\\n            \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"response\",\"duplicates\"]]],null,{\"statements\":[[0,\"                \"],[7,\"tr\"],[9],[0,\"\\n                    \"],[7,\"td\"],[9],[0,\"Duplicates:\"],[10],[0,\"\\n                    \"],[7,\"td\"],[11,\"align\",\"left\"],[11,\"data-test-text\",\"import-subscribers-duplicates\"],[9],[1,[25,[\"response\",\"duplicates\"]],false],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[25,[\"response\",\"invalid\"]]],null,{\"statements\":[[0,\"                \"],[7,\"tr\"],[9],[0,\"\\n                    \"],[7,\"td\"],[9],[0,\"Invalid:\"],[10],[0,\"\\n                    \"],[7,\"td\"],[11,\"align\",\"left\"],[11,\"data-test-text\",\"import-subscribers-invalid\"],[9],[1,[25,[\"response\",\"invalid\"]],false],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        \"],[1,[29,\"gh-file-uploader\",null,[[\"url\",\"paramName\",\"labelText\",\"uploadStarted\",\"uploadFinished\",\"uploadSuccess\"],[[25,[\"uploadUrl\"]],\"subscribersfile\",\"Select or drag-and-drop a CSV file.\",[29,\"action\",[[24,0,[]],\"uploadStarted\"],null],[29,\"action\",[[24,0,[]],\"uploadFinished\"],null],[29,\"action\",[[24,0,[]],\"uploadSuccess\"],null]]]],false],[0,\"\\n\"]],\"parameters\":[]}],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[12,\"disabled\",[23,\"closeDisabled\"]],[11,\"class\",\"gh-btn\"],[11,\"data-test-button\",\"close-import-subscribers\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[4,\"if\",[[25,[\"response\"]]],null,{\"statements\":[[0,\"Close\"]],\"parameters\":[]},{\"statements\":[[0,\"Cancel\"]],\"parameters\":[]}],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-import-subscribers.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-invite-new-user", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "PUlN++Sa",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Add Users\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"errors\"]],[25,[\"hasValidated\"]],\"username\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"new-user-name\"],[9],[0,\"Username\"],[10],[0,\"\\n            \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"placeholder\",\"name\",\"autofocus\",\"autocapitalize\",\"autocorrect\",\"value\",\"input\",\"keyEvents\",\"focus-out\"],[\"username\",\"new-user-name\",\"Username\",\"username\",\"autofocus\",\"off\",\"off\",[29,\"readonly\",[[25,[\"username\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"username\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"hash\",null,[[\"Enter\"],[[29,\"action\",[[24,0,[]],\"confirm\"],null]]]],[29,\"action\",[[24,0,[]],\"validate\",\"username\"],null]]]],false],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"errors\"]],\"username\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\\n\"],[4,\"gh-form-group\",null,[[\"class\",\"errors\",\"hasValidated\",\"property\"],[\"for-select\",[25,[\"errors\"]],[25,[\"hasValidated\"]],\"role\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"new-user-role\"],[9],[0,\"Role\"],[10],[0,\"\\n            \"],[7,\"span\"],[11,\"class\",\"gh-select\"],[9],[0,\"\\n                \"],[1,[29,\"one-way-select\",null,[[\"id\",\"name\",\"options\",\"optionValuePath\",\"optionLabelPath\",\"value\",\"update\"],[\"new-user-role\",\"role\",[29,\"readonly\",[[25,[\"roles\"]]],null],\"id\",\"name\",[25,[\"role\"]],[29,\"action\",[[24,0,[]],\"setRole\"],null]]]],false],[0,\"\\n                \"],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n            \"],[10],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"errors\"]],\"role\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Add User\"],[[\"successText\",\"task\",\"class\",\"disabled\",\"disableMouseDown\"],[\"Sent\",[25,[\"sendInvitation\"]],\"gh-btn gh-btn-green gh-btn-icon\",[25,[\"fetchRoles\",\"isRunning\"]],true]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-invite-new-user.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-leave-editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "wxarB9p9",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to leave this page?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        Hey there! It looks like you're in the middle of writing something and\\n        you haven't saved all of your content.\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"p\"],[9],[0,\"Save before you go!\"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Stay\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-red\"],[9],[7,\"span\"],[9],[0,\"Leave\"],[10],[3,\"action\",[[24,0,[]],\"confirm\"]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-leave-editor.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-leave-settings", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "AHZvDWXI",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"unsaved-settings\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to leave this page?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        Hey there! It looks like you didn't save the changes you made.\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"p\"],[9],[0,\"Save before you go!\"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-stay-button\",\"\"],[9],[7,\"span\"],[9],[0,\"Stay\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-red\"],[11,\"data-test-leave-button\",\"\"],[9],[7,\"span\"],[9],[0,\"Leave\"],[10],[3,\"action\",[[24,0,[]],\"confirm\"]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-leave-settings.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-markdown-help", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "hdxlhtgw",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Markdown Help\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"markdown-help-container\"],[9],[0,\"\\n        \"],[7,\"table\"],[11,\"class\",\"modal-markdown-help-table\"],[9],[0,\"\\n            \"],[7,\"thead\"],[9],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"th\"],[9],[0,\"Markdown\"],[10],[0,\"\\n                \"],[7,\"th\"],[9],[0,\"Result\"],[10],[0,\"\\n                \"],[7,\"th\"],[9],[0,\"Shortcut\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tbody\"],[9],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"**text**\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[7,\"strong\"],[9],[0,\"Bold\"],[10],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + B\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"*text*\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[7,\"em\"],[9],[0,\"Emphasize\"],[10],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + I\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"~~text~~\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[7,\"del\"],[9],[0,\"Strike-through\"],[10],[10],[0,\"\\n                \"],[7,\"td\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"[title](http://)\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[7,\"a\"],[11,\"href\",\"#\"],[9],[0,\"Link\"],[10],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + K\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"`code`\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[7,\"code\"],[9],[0,\"Inline Code\"],[10],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + Alt + C\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"![alt](http://)\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Image\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + Shift + I\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"* item\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"List\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + L\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"1. item\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ordered List\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + Alt + L\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"> quote\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Blockquote\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + '\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"==Highlight==\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[7,\"mark\"],[9],[0,\"Highlight\"],[10],[10],[0,\"\\n                \"],[7,\"td\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"# Heading\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"H1\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"## Heading\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"H2\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + H\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tr\"],[9],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"### Heading\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"H3\"],[10],[0,\"\\n                \"],[7,\"td\"],[9],[0,\"Ctrl/⌘ + H (x2)\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        For further Markdown syntax reference: \"],[7,\"a\"],[11,\"href\",\"https://docs.ghost.org/faq/using-the-editor/#using-markdown\"],[11,\"target\",\"_blank\"],[9],[0,\"Markdown Documentation\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-markdown-help.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-new-integration", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "3VRkfVql",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"new-integration\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"New custom integration\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"button\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"integration\",\"errors\"]],[25,[\"integration\",\"hasValidated\"]],\"name\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"new-integration-name\"],[11,\"class\",\"fw6\"],[9],[0,\"Name\"],[10],[0,\"\\n            \"],[7,\"input\"],[12,\"value\",[25,[\"integration\",\"name\"]]],[12,\"oninput\",[29,\"action\",[[24,0,[]],\"updateName\"],[[\"value\"],[\"target.value\"]]]],[11,\"id\",\"new-integration-name\"],[11,\"class\",\"gh-input mt1\"],[11,\"placeholder\",\"Integration name...\"],[11,\"name\",\"integration-name\"],[11,\"autofocus\",\"autofocus\"],[11,\"autocapitalize\",\"off\"],[11,\"autocorrect\",\"off\"],[11,\"data-test-input\",\"new-integration-name\"],[11,\"type\",\"text\"],[9],[10],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"integration\",\"errors\"]],\"name\",\"new-integration-name\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-button\",\"cancel-new-integration\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\"Cancel\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Create\"],[[\"successText\",\"task\",\"class\",\"data-test-button\"],[\"Created\",[25,[\"createIntegration\"]],\"gh-btn gh-btn-green gh-btn-icon\",\"create-integration\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-new-integration.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-new-subscriber", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "0oVRBVNa",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"new-subscriber\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Add a Subscriber\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"subscriber\",\"errors\"]],[25,[\"subscriber\",\"hasValidated\"]],\"email\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"new-subscriber-email\"],[9],[0,\"Email Address\"],[10],[0,\"\\n            \"],[7,\"input\"],[12,\"value\",[25,[\"subscriber\",\"email\"]]],[12,\"oninput\",[29,\"action\",[[24,0,[]],\"updateEmail\"],[[\"value\"],[\"target.value\"]]]],[11,\"id\",\"new-subscriber-email\"],[11,\"class\",\"gh-input email\"],[11,\"placeholder\",\"Email Address\"],[11,\"name\",\"email\"],[11,\"autofocus\",\"autofocus\"],[11,\"autocapitalize\",\"off\"],[11,\"autocorrect\",\"off\"],[11,\"data-test-input\",\"new-subscriber-email\"],[11,\"type\",\"email\"],[9],[10],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"subscriber\",\"errors\"]],\"email\",\"new-subscriber-email\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-button\",\"cancel-new-subscriber\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\"Cancel\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Add\"],[[\"successText\",\"task\",\"class\",\"data-test-button\"],[\"Added\",[25,[\"addSubscriber\"]],\"gh-btn gh-btn-green gh-btn-icon\",\"create-subscriber\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-new-subscriber.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-suspend-user", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "Dgq/Qnwf",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to suspend this user?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"strong\"],[9],[0,\"WARNING:\"],[10],[0,\" This user will no longer be able to log in but their posts will be kept.\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Suspend\"],[[\"successText\",\"task\",\"class\",\"data-test-modal-confirm\"],[\"Suspended\",[25,[\"suspendUser\"]],\"gh-btn gh-btn-red gh-btn-icon\",true]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-suspend-user.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-theme-warnings", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "YtROaoKi",
    "block": "{\"symbols\":[\"error\",\"error\",\"error\"],\"statements\":[[7,\"div\"],[11,\"class\",\"theme-validation-container\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n        \"],[7,\"h1\"],[11,\"data-test-theme-warnings-title\",\"\"],[9],[0,\"\\n\"],[4,\"unless\",[[24,0,[\"canActivate\"]]],null,{\"statements\":[[0,\"                \"],[1,[24,0,[\"title\"]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                \"],[1,[24,0,[\"title\"]],false],[0,\" with \"],[4,\"if\",[[24,0,[\"errors\"]]],null,{\"statements\":[[0,\"errors\"]],\"parameters\":[]},{\"statements\":[[0,\"warnings\"]],\"parameters\":[]}],[0,\"\\n\"]],\"parameters\":[]}],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n    \"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"#\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n\"],[4,\"if\",[[24,0,[\"fatalErrors\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[9],[0,\"\\n                \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6 red\"],[9],[0,\"Fatal Errors\"],[10],[0,\"\\n                \"],[7,\"p\"],[11,\"class\",\"mb2 red\"],[9],[0,\"Must-fix to activate theme\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[11,\"data-test-theme-fatal-errors\",\"\"],[9],[0,\"\\n\"],[4,\"each\",[[24,0,[\"fatalErrors\"]]],null,{\"statements\":[[0,\"                \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-fatal-error\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,3,[]]]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[3]},null],[0,\"            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[24,0,[\"errors\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[9],[0,\"\\n                \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6\"],[9],[0,\"Errors\"],[10],[0,\"\\n                \"],[7,\"p\"],[11,\"class\",\"mb2\"],[9],[0,\"Highly recommended to fix, functionality \"],[7,\"span\"],[9],[0,\"could\"],[10],[0,\" be restricted\"],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n            \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[11,\"data-test-theme-errors\",\"\"],[9],[0,\"\\n\"],[4,\"each\",[[24,0,[\"errors\"]]],null,{\"statements\":[[0,\"                \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-error\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,2,[]]]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[29,\"and\",[[24,0,[\"warnings\"]],[29,\"or\",[[24,0,[\"fatalErrors\"]],[24,0,[\"errors\"]]],null]],null]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[9],[0,\"\\n                \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6\"],[9],[0,\"Warnings\"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[24,0,[\"warnings\"]]],null,{\"statements\":[[0,\"            \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[11,\"data-test-theme-warnings\",\"\"],[9],[0,\"\\n\"],[4,\"each\",[[24,0,[\"warnings\"]]],null,{\"statements\":[[0,\"                \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-warning\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,1,[]]]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-modal-close-button\",\"\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\"Ok\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-theme-warnings.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-transfer-owner", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "AE90ZGxF",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Transfer Ownership\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"p\"],[9],[0,\"\\n        Are you sure you want to transfer the ownership of this blog?\\n        You will not be able to undo this action.\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Yep - I'm sure\"],[[\"task\",\"class\"],[[25,[\"transferOwnership\"]],\"gh-btn gh-btn-red gh-btn-icon\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-transfer-owner.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-unsuspend-user", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "PEzrMRaK",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[9],[0,\"\\n    \"],[7,\"h1\"],[9],[0,\"Are you sure you want to un-suspend this user?\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"strong\"],[9],[0,\"WARNING:\"],[10],[0,\" This user will be able to log in again and will have the same permissions they had previously.\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[\"Un-suspend\"],[[\"successText\",\"task\",\"class\",\"data-test-modal-confirm\"],[\"Suspended\",[25,[\"unsuspendUser\"]],\"gh-btn gh-btn-red gh-btn-icon\",true]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-unsuspend-user.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-upload-image", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "zzUexbJ/",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"url\"]]],null,{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"gh-image-uploader -with-image\"],[9],[0,\"\\n            \"],[7,\"div\"],[9],[7,\"img\"],[12,\"src\",[23,\"url\"]],[9],[10],[10],[0,\"\\n            \"],[7,\"a\"],[11,\"class\",\"image-cancel\"],[11,\"title\",\"Delete\"],[9],[0,\"\\n                \"],[1,[29,\"svg-jar\",[\"trash\"],null],false],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Delete\"],[10],[0,\"\\n            \"],[3,\"action\",[[24,0,[]],\"removeImage\"]],[10],[0,\"\\n        \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        \"],[1,[29,\"gh-image-uploader\",null,[[\"image\",\"saveButton\",\"update\",\"uploadStarted\",\"uploadFinished\",\"accept\",\"extensions\",\"uploadUrl\",\"paramsHash\"],[[25,[\"newUrl\"]],false,[29,\"action\",[[24,0,[]],\"fileUploaded\"],null],[29,\"action\",[[24,0,[]],\"isUploading\"],null],[29,\"action\",[[24,0,[]],\"isUploading\"],null],[25,[\"model\",\"accept\"]],[25,[\"model\",\"extensions\"]],[25,[\"model\",\"uploadUrl\"]],[25,[\"model\",\"paramsHas\"]]]]],false],[0,\"\\n\"]],\"parameters\":[]}],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[9],[7,\"span\"],[9],[0,\"Cancel\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"_isUploading\"]]],null,{\"statements\":[[0,\"        \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-blue right gh-btn-icon disabled\"],[9],[7,\"span\"],[9],[0,\"Save\"],[10],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-modal-accept-button\"],[[25,[\"uploadImage\"]],\"gh-btn gh-btn-blue right gh-btn-icon\",true]]],false],[0,\"\\n\"]],\"parameters\":[]}],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-upload-image.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-upload-theme", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "LpSHSqwB",
    "block": "{\"symbols\":[\"error\",\"error\",\"error\",\"error\"],\"statements\":[[7,\"div\"],[11,\"class\",\"theme-validation-container\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"upload-theme\"],[9],[0,\"\\n        \"],[7,\"h1\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"theme\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"hasWarningsOrErrors\"]]],null,{\"statements\":[[0,\"                    Upload successful with \"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"errors\"]],\"parameters\":[]},{\"statements\":[[0,\"warnings\"]],\"parameters\":[]}],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    Upload successful!\\n\"]],\"parameters\":[]}]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[29,\"or\",[[25,[\"validationErrors\"]],[25,[\"fatalValidationErrors\"]]],null]],null,{\"statements\":[[0,\"                Invalid theme\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                Upload a theme\\n            \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n    \"],[7,\"a\"],[11,\"class\",\"close\"],[11,\"href\",\"#\"],[11,\"title\",\"Close\"],[9],[1,[29,\"svg-jar\",[\"close\"],null],false],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"Close\"],[10],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"theme\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"hasWarningsOrErrors\"]]],null,{\"statements\":[[0,\"                \"],[7,\"p\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"canActivateTheme\"]]],null,{\"statements\":[[0,\"                        The theme \"],[7,\"strong\"],[9],[0,\"\\\"\"],[1,[23,\"themeName\"],false],[0,\"\\\"\"],[10],[0,\" was uploaded successfully but we detected some \"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"errors\"]],\"parameters\":[]},{\"statements\":[[0,\"warnings\"]],\"parameters\":[]}],[0,\". You are still able to activate and use the theme but it is recommended to fix these \"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"errors\"]],\"parameters\":[]},{\"statements\":[[0,\"warnings\"]],\"parameters\":[]}],[0,\" before you do so.\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        The theme \"],[7,\"strong\"],[9],[0,\"\\\"\"],[1,[23,\"themeName\"],false],[0,\"\\\"\"],[10],[0,\" was uploaded successfully but we detected some\\n                        \"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"errors\"]],\"parameters\":[]},{\"statements\":[[0,\"warnings\"]],\"parameters\":[]}],[0,\".\\n\"]],\"parameters\":[]}],[0,\"                \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[9],[0,\"\\n                        \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6\"],[9],[0,\"Errors\"],[10],[0,\"\\n                        \"],[7,\"p\"],[11,\"class\",\"mb2\"],[9],[0,\"Highly recommended to fix, functionality \"],[7,\"strong\"],[9],[0,\"could\"],[10],[0,\" be restricted\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-error\"],[9],[0,\"\\n                            \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,4,[]]]]],false],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[4]},null],[0,\"                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"validationWarnings\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[9],[0,\"\\n                        \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6\"],[9],[0,\"Warnings\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"validationWarnings\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-warning\"],[9],[0,\"\\n                            \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,3,[]]]]],false],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[3]},null],[0,\"                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},{\"statements\":[[0,\"                \"],[7,\"p\"],[9],[0,\"\\n                    \\\"\"],[1,[23,\"themeName\"],false],[0,\"\\\" uploaded successfully.\\n                    \"],[4,\"if\",[[25,[\"canActivateTheme\"]]],null,{\"statements\":[[0,\"Do you want to activate it now?\"]],\"parameters\":[]},null],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"displayOverwriteWarning\"]]],null,{\"statements\":[[0,\"            \"],[7,\"p\"],[9],[0,\"\\n                The theme folder \"],[7,\"strong\"],[9],[0,\"\\\"\"],[1,[23,\"fileThemeName\"],false],[0,\"\\\"\"],[10],[0,\" already exists. Do you want to overwrite it?\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[29,\"or\",[[25,[\"validationErrors\"]],[25,[\"fatalValidationErrors\"]]],null]],null,{\"statements\":[[0,\"\\n            \"],[7,\"p\"],[9],[0,\"\\n                This theme is invalid and cannot be activated. Fix the following errors and re-upload the theme.\\n            \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"fatalValidationErrors\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[9],[0,\"\\n                    \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6\"],[9],[0,\"Fatal Errors\"],[10],[0,\"\\n                    \"],[7,\"p\"],[11,\"class\",\"mb2\"],[9],[0,\"Must-fix to activate theme\"],[10],[0,\"\\n                \"],[10],[0,\"\\n\\n                \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"fatalValidationErrors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-fatal-error\"],[9],[0,\"\\n                        \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,2,[]]]]],false],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[9],[0,\"\\n                    \"],[7,\"h2\"],[11,\"class\",\"mb0 mt4 f5 fw6\"],[9],[0,\"Errors\"],[10],[0,\"\\n                    \"],[7,\"p\"],[11,\"class\",\"mb2\"],[9],[0,\"Highly recommended to fix, functionality \"],[7,\"strong\"],[9],[0,\"could\"],[10],[0,\" be restricted\"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"ul\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"li\"],[11,\"class\",\"theme-validation-item theme-error\"],[9],[0,\"\\n                        \"],[1,[29,\"gh-theme-error-li\",null,[[\"error\"],[[24,1,[]]]]],false],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},{\"statements\":[[0,\"            \"],[1,[29,\"gh-file-uploader\",null,[[\"url\",\"paramName\",\"accept\",\"labelText\",\"validate\",\"uploadStarted\",\"uploadFinished\",\"uploadSuccess\",\"uploadFailed\",\"listenTo\"],[[25,[\"uploadUrl\"]],\"file\",[25,[\"accept\"]],\"Click to select or drag-and-drop your theme zip file here.\",[29,\"action\",[[24,0,[]],\"validateTheme\"],null],[29,\"action\",[[24,0,[]],\"uploadStarted\"],null],[29,\"action\",[[24,0,[]],\"uploadFinished\"],null],[29,\"action\",[[24,0,[]],\"uploadSuccess\"],null],[29,\"action\",[[24,0,[]],\"uploadFailed\"],null],\"themeUploader\"]]],false],[0,\"\\n        \"]],\"parameters\":[]}]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"div\"],[12,\"class\",[30,[\"flex items-center justify-between \",[29,\"if\",[[29,\"or\",[[25,[\"displayOverwriteWarning\"]],[25,[\"canActivateTheme\"]],[25,[\"validationErrors\"]],[25,[\"fatalValidationErrors\"]]],null],\"flex-auto\"],null]]]],[9],[0,\"\\n        \"],[7,\"button\"],[12,\"disabled\",[23,\"closeDisabled\"]],[11,\"class\",\"gh-btn\"],[11,\"data-test-close-button\",\"\"],[9],[0,\"\\n            \"],[7,\"span\"],[9],[4,\"if\",[[25,[\"theme\"]]],null,{\"statements\":[[0,\"Close\"]],\"parameters\":[]},{\"statements\":[[0,\"Cancel\"]],\"parameters\":[]}],[10],[0,\"\\n        \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"flex items-center\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"displayOverwriteWarning\"]]],null,{\"statements\":[[0,\"                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-red\"],[11,\"data-test-overwrite-button\",\"\"],[9],[0,\"\\n                    \"],[7,\"span\"],[9],[0,\"Overwrite\"],[10],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],\"confirmOverwrite\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[25,[\"canActivateTheme\"]]],null,{\"statements\":[[0,\"                \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-activate-now-button\",\"\"],[9],[0,\"\\n                    \"],[7,\"span\"],[9],[0,\"Activate\"],[4,\"if\",[[25,[\"validationErrors\"]]],null,{\"statements\":[[0,\" with errors\"]],\"parameters\":[]},null],[10],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],\"activate\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[29,\"or\",[[25,[\"validationErrors\"]],[25,[\"fatalValidationErrors\"]]],null]],null,{\"statements\":[[0,\"                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-blue ml2\"],[11,\"data-test-try-again-button\",\"\"],[9],[0,\"\\n                    \"],[7,\"span\"],[9],[0,\"Retry\"],[10],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],\"reset\"]],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-upload-theme.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/modal-webhook-form", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "sxhbmMY7",
    "block": "{\"symbols\":[],\"statements\":[[7,\"header\"],[11,\"class\",\"modal-header\"],[11,\"data-test-modal\",\"webhook-form\"],[9],[0,\"\\n    \"],[7,\"h1\"],[11,\"data-test-text\",\"title\"],[9],[1,[29,\"if\",[[25,[\"webhook\",\"isNew\"]],\"New\",\"Edit\"],null],false],[0,\" webhook\"],[10],[0,\"\\n\"],[10],[0,\"\\n\"],[7,\"button\"],[11,\"class\",\"close\"],[11,\"href\",\"\"],[11,\"title\",\"Close\"],[9],[0,\"\\n    \"],[1,[29,\"svg-jar\",[\"close\"],null],false],[0,\"\\n\"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-body\"],[9],[0,\"\\n    \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"webhook\",\"errors\"]],[25,[\"webhook\",\"hasValidated\"]],\"name\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"webhook-name\"],[11,\"class\",\"fw6\"],[9],[0,\"Name\"],[10],[0,\"\\n            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"id\",\"name\",\"class\",\"placeholder\",\"autofocus\",\"autocapitalize\",\"autocorrect\",\"data-test-input\"],[[29,\"readonly\",[[25,[\"webhook\",\"name\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"webhook\",\"name\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"name\"],[[\"target\"],[[25,[\"webhook\"]]]]],\"webhook-name\",\"name\",\"gh-input mt1\",\"Webhook name...\",\"autofocus\",\"off\",\"off\",\"webhook-name\"]]],false],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"webhook\",\"errors\"]],\"name\",\"webhook-name\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n    \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"webhook\",\"errors\"]],[25,[\"webhook\",\"hasValidated\"]],\"event\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"webhook-event\"],[11,\"class\",\"fw6\"],[9],[0,\"Event\"],[10],[0,\"\\n            \"],[7,\"span\"],[11,\"class\",\"gh-select\"],[9],[0,\"\\n                \"],[1,[29,\"one-way-select\",[[25,[\"webhook\",\"event\"]]],[[\"options\",\"optionValuePath\",\"optionLabelPath\",\"optionTargetPath\",\"groupLabelPath\",\"class\",\"includeBlank\",\"prompt\",\"update\",\"id\",\"name\",\"data-test-select\"],[[25,[\"availableEvents\"]],\"event\",\"name\",\"event\",\"group\",\"mt1\",true,\"Select an event\",[29,\"action\",[[24,0,[]],\"selectEvent\"],null],\"webhook-event\",\"event\",\"webhook-event\"]]],false],[0,\"\\n                \"],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n            \"],[10],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"webhook\",\"errors\"]],\"event\",\"webhook-event\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n    \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"webhook\",\"errors\"]],[25,[\"webhook\",\"hasValidated\"]],\"targetUrl\"]],{\"statements\":[[0,\"            \"],[7,\"label\"],[11,\"for\",\"webhook-targetUrl\"],[11,\"class\",\"fw6\"],[9],[0,\"Target URL\"],[10],[0,\"\\n            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"id\",\"name\",\"class\",\"placeholder\",\"autofocus\",\"autocapitalize\",\"autocorrect\",\"data-test-input\"],[[29,\"readonly\",[[25,[\"webhook\",\"targetUrl\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"webhook\",\"targetUrl\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"targetUrl\"],[[\"target\"],[[25,[\"webhook\"]]]]],\"webhook-targetUrl\",\"targetUrl\",\"gh-input mt1\",\"Webhook target URL...\",\"autofocus\",\"off\",\"off\",\"webhook-targetUrl\"]]],false],[0,\"\\n            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"webhook\",\"errors\"]],\"targetUrl\",\"webhook-targetUrl\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"config\",\"enableDeveloperExperiments\"]]],null,{\"statements\":[[0,\"        \"],[7,\"fieldset\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"webhook\",\"errors\"]],[25,[\"webhook\",\"hasValidated\"]],\"secret\"]],{\"statements\":[[0,\"                \"],[7,\"label\"],[11,\"for\",\"webhook-secret\"],[11,\"class\",\"fw6\"],[9],[0,\"Secret\"],[10],[0,\"\\n                \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"oninput\",\"focus-out\",\"id\",\"name\",\"class\",\"placeholder\",\"autofocus\",\"autocapitalize\",\"autocorrect\",\"data-test-input\"],[[29,\"readonly\",[[25,[\"webhook\",\"secret\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"webhook\",\"secret\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"secret\"],[[\"target\"],[[25,[\"webhook\"]]]]],\"webhook-secret\",\"secret\",\"gh-input mt1\",\"Webhook secret...\",\"autofocus\",\"off\",\"off\",\"webhook-secret\"]]],false],[0,\"\\n                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"webhook\",\"errors\"]],\"secret\",\"webhook-secret\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[25,[\"error\"]]],null,{\"statements\":[[0,\"        \"],[7,\"p\"],[11,\"class\",\"red\"],[9],[1,[23,\"error\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\\n\"],[7,\"div\"],[11,\"class\",\"modal-footer\"],[9],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[11,\"data-test-button\",\"cancel-webhook\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\"Cancel\"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"closeModal\"]],[3,\"action\",[[24,0,[]],[29,\"optional\",[[25,[\"noop\"]]],null]],[[\"on\"],[\"mouseDown\"]]],[10],[0,\"\\n    \"],[1,[29,\"gh-task-button\",[[25,[\"buttonText\"]]],[[\"successText\",\"task\",\"class\",\"data-test-button\"],[[25,[\"successText\"]],[25,[\"saveWebhook\"]],\"gh-btn gh-btn-green gh-btn-icon\",\"save-webhook\"]]],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/modal-webhook-form.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/object-bin", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "/tukjaeb",
    "block": "{\"symbols\":[\"obj\",\"&default\"],\"statements\":[[4,\"draggable-object-target\",null,[[\"action\"],[\"handleObjectDropped\"]],{\"statements\":[[0,\"  \"],[7,\"div\"],[11,\"class\",\"object-bin-title\"],[9],[1,[23,\"name\"],false],[10],[0,\"\\n  \"],[7,\"br\"],[9],[10],[0,\"\\n\"],[4,\"each\",[[25,[\"model\"]]],null,{\"statements\":[[4,\"draggable-object\",null,[[\"action\",\"content\"],[\"handleObjectDragged\",[24,1,[]]]],{\"statements\":[[0,\"      \"],[15,2,[[24,1,[]]]],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[1]},null]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/object-bin.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/power-select-vertical-collection-options", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "PidKtE1a",
    "block": "{\"symbols\":[\"opt\",\"index\",\"&default\"],\"statements\":[[4,\"if\",[[25,[\"select\",\"loading\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"loadingMessage\"]]],null,{\"statements\":[[0,\"        \"],[7,\"li\"],[11,\"class\",\"ember-power-select-option ember-power-select-option--loading-message\"],[11,\"role\",\"option\"],[9],[1,[23,\"loadingMessage\"],false],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"vertical-collection\",[[25,[\"options\"]]],[[\"minHeight\",\"estimateHeight\",\"bufferSize\"],[30,6,10]],{\"statements\":[[0,\"    \"],[7,\"li\"],[11,\"class\",\"ember-power-select-option\"],[12,\"aria-selected\",[30,[[29,\"ember-power-select-is-selected\",[[24,1,[]],[25,[\"select\",\"selected\"]]],null]]]],[12,\"aria-disabled\",[29,\"ember-power-select-true-string-if-present\",[[24,1,[\"disabled\"]]],null]],[12,\"aria-current\",[30,[[29,\"eq\",[[24,1,[]],[25,[\"select\",\"highlighted\"]]],null]]]],[12,\"data-option-index\",[30,[[23,\"groupIndex\"],[24,2,[]]]]],[11,\"role\",\"option\"],[9],[0,\"\\n        \"],[15,3,[[24,1,[]],[25,[\"select\"]]]],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[1,2]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/power-select-vertical-collection-options.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/power-select/trigger", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "cZRFMCq9",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[4,\"if\",[[25,[\"select\",\"selected\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"selectedItemComponent\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"component\",[[25,[\"selectedItemComponent\"]]],[[\"option\",\"select\"],[[29,\"readonly\",[[25,[\"select\",\"selected\"]]],null],[29,\"readonly\",[[25,[\"select\"]]],null]]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"ember-power-select-selected-item\"],[9],[15,1,[[25,[\"select\",\"selected\"]],[25,[\"select\"]]]],[10],[0,\"\\n\"]],\"parameters\":[]}],[4,\"if\",[[29,\"and\",[[25,[\"allowClear\"]],[29,\"not\",[[25,[\"select\",\"disabled\"]]],null]],null]],null,{\"statements\":[[0,\"        \"],[7,\"span\"],[11,\"class\",\"ember-power-select-clear-btn\"],[12,\"onmousedown\",[29,\"action\",[[24,0,[]],\"clear\"],null]],[12,\"ontouchstart\",[29,\"action\",[[24,0,[]],\"clear\"],null]],[9],[0,\"×\"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},{\"statements\":[[0,\"    \"],[1,[29,\"component\",[[25,[\"placeholderComponent\"]]],[[\"placeholder\"],[[25,[\"placeholder\"]]]]],false],[0,\"\\n\"]],\"parameters\":[]}],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/power-select/trigger.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/components/sortable-objects", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "tIDf66AL",
    "block": "{\"symbols\":[\"&default\"],\"statements\":[[15,1]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/components/sortable-objects.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/editor", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "h2chH9UP",
    "block": "{\"symbols\":[\"editor\",\"post\",\"countdown\"],\"statements\":[[4,\"if\",[[25,[\"post\"]]],null,{\"statements\":[[4,\"gh-editor\",null,[[\"tagName\",\"class\"],[\"section\",\"gh-editor gh-view\"]],{\"statements\":[[0,\"        \"],[7,\"header\"],[12,\"class\",[30,[\"gh-editor-header br2 pe-none \",[24,1,[\"headerClass\"]],\" \",[29,\"if\",[[25,[\"infoMessage\"]],\"bg-white\"],null]]]],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"flex items-center pe-auto\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"ui\",\"isFullScreen\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[12,\"class\",[30,[[29,\"ui-text\",[\"ts\"],null],\" h9 br b--lightgrey pl3 pr4 flex items-center br2 br--left \",[29,\"unless\",[[25,[\"infoMessage\"]],\"bg-white\"],null]]]],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"classNames\",\"data-test-link\",\"route\"],[\"blue link fw4 flex items-center\",[29,\"pluralize\",[[25,[\"post\",\"displayName\"]]],null],[29,\"pluralize\",[[25,[\"post\",\"displayName\"]]],null]]],{\"statements\":[[0,\"                            \"],[1,[29,\"svg-jar\",[\"arrow-left\"],[[\"class\"],[\"w3 fill-blue mr1 nudge-right--2\"]]],false],[0,\"\\n                            \"],[1,[29,\"capitalize\",[[29,\"pluralize\",[[25,[\"post\",\"displayName\"]]],null]],null],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[7,\"div\"],[12,\"class\",[30,[\"flex items-center pl4 pr4 f8 nudge-left--1 h9 br2 br--right \",[29,\"unless\",[[25,[\"infoMessage\"]],\"bg-white\"],null]]]],[9],[0,\"\\n                    \"],[7,\"span\"],[11,\"class\",\"fw3 midgrey-l2\"],[9],[0,\"\\n                        \"],[1,[29,\"gh-editor-post-status\",null,[[\"post\",\"isSaving\"],[[25,[\"post\"]],[29,\"or\",[[25,[\"autosave\",\"isRunning\"]],[25,[\"saveTasks\",\"isRunning\"]]],null]]]],false],[0,\"\\n                    \"],[10],[0,\"\\n\"],[4,\"gh-scheduled-post-countdown\",null,[[\"post\"],[[25,[\"post\"]]]],{\"statements\":[[0,\"                    \"],[7,\"time\"],[12,\"datetime\",[30,[[24,2,[\"publishedAtUTC\"]]]]],[11,\"class\",\"green f8 ml5\"],[11,\"data-test-schedule-countdown\",\"\"],[9],[0,\"\\n                        \"],[1,[29,\"capitalize\",[[24,2,[\"displayName\"]]],null],false],[0,\" will go live \"],[1,[24,3,[]],false],[0,\".\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[2,3]},null],[0,\"                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"infoMessage\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-koenig-info miw88 flex flex-auto justify-center\"],[11,\"style\",\"pointer-events: auto\"],[9],[0,\"\\n                \"],[7,\"div\"],[12,\"class\",[30,[\"midgrey inline-flex tracked-1 center pa1 pl4 pr4 br3 justify-center items-start \",[29,\"if\",[[29,\"eq\",[[24,1,[\"headerClass\"]],\"gh-editor-header-small\"],null],\"bg-white\"],null]]]],[9],[0,\"\\n                    \"],[1,[29,\"svg-jar\",[\"idea\"],[[\"class\"],[\"w7 h7 fill-blue\"]]],false],[0,\"\\n                    \"],[7,\"p\"],[11,\"class\",\"ma0 pa0 ml1 nudge-top--2\"],[9],[0,\"\\n                        \"],[1,[23,\"infoMessage\"],false],[0,\"\\n                        \"],[7,\"button\"],[11,\"class\",\"blue fw5\"],[11,\"type\",\"button\"],[9],[0,\"Got it\"],[3,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"infoMessage\"]],\"\"],null]]],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"section\"],[12,\"class\",[30,[\"view-actions br2 \",[29,\"unless\",[[25,[\"infoMessage\"]],\"bg-white\"],null]]]],[11,\"style\",\"pointer-events: auto\"],[9],[0,\"\\n\"],[4,\"unless\",[[25,[\"post\",\"isNew\"]]],null,{\"statements\":[[4,\"if\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[0,\"                        \"],[1,[29,\"gh-task-button\",[\"Save\"],[[\"task\",\"runningText\",\"class\",\"data-test-contributor-save\"],[[25,[\"save\"]],\"Saving\",\"gh-btn gh-btn-blue gh-btn-icon contributor-save-button\",true]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[1,[29,\"gh-publishmenu\",null,[[\"post\",\"postStatus\",\"saveTask\",\"setSaveType\",\"onOpen\"],[[25,[\"post\"]],[25,[\"post\",\"status\"]],[25,[\"save\"]],[29,\"action\",[[24,0,[]],\"setSaveType\"],null],[29,\"action\",[[24,0,[]],\"cancelAutosave\"],null]]]],false],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]},null],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"post-settings\"],[11,\"title\",\"Settings\"],[11,\"data-test-psm-trigger\",\"\"],[11,\"type\",\"button\"],[9],[0,\"\\n                    \"],[1,[29,\"svg-jar\",[\"settings\"],null],false],[0,\"\\n                \"],[3,\"action\",[[24,0,[]],\"openSettingsMenu\"],[[\"target\"],[[25,[\"ui\"]]]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n\"],[0,\"        \"],[1,[29,\"gh-koenig-editor\",null,[[\"title\",\"titlePlaceholder\",\"onTitleChange\",\"onTitleBlur\",\"body\",\"bodyPlaceholder\",\"bodyAutofocus\",\"onBodyChange\",\"headerOffset\",\"scrollContainerSelector\",\"scrollOffsetTopSelector\",\"scrollOffsetBottomSelector\",\"onEditorCreated\",\"onWordCountChange\"],[[29,\"readonly\",[[25,[\"post\",\"titleScratch\"]]],null],[29,\"concat\",[[29,\"capitalize\",[[25,[\"post\",\"displayName\"]]],null],\" Title\"],null],[29,\"action\",[[24,0,[]],\"updateTitleScratch\"],null],[29,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"saveTitle\"]]],null]],null],[29,\"readonly\",[[25,[\"post\",\"scratch\"]]],null],[29,\"concat\",[\"Begin writing your \",[25,[\"post\",\"displayName\"]],\"...\"],null],[25,[\"shouldFocusEditor\"]],[29,\"action\",[[24,0,[]],\"updateScratch\"],null],[24,1,[\"headerHeight\"]],\".gh-koenig-editor\",\".gh-editor-header-small\",\".gh-mobile-nav-bar\",[29,\"action\",[[24,0,[]],\"setKoenigEditor\"],null],[29,\"action\",[[24,0,[]],\"updateWordCount\"],null]]]],false],[0,\"\\n\\n        \"],[7,\"div\"],[12,\"class\",[30,[\"absolute flex items-center br3 bg-white \",[29,\"if\",[[24,1,[\"headerClass\"]],\"right-4 bottom-4\",\"right-6 bottom-6\"],null]]]],[9],[0,\"\\n            \"],[7,\"div\"],[12,\"class\",[30,[\"midgrey-l2 \",[29,\"if\",[[24,1,[\"headerClass\"]],\"f-supersmall pl2 pr2\",\"f8 pl4 pr3\"],null],\" fw3\"]]],[9],[0,\"\\n                \"],[1,[29,\"pluralize\",[[25,[\"wordCount\",\"wordCount\"]],\"word\"],null],false],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"a\"],[11,\"href\",\"https://docs.ghost.org/faq/using-the-editor/\"],[12,\"class\",[30,[\"flex \",[29,\"if\",[[24,1,[\"headerClass\"]],\"pa2\",\"pa3\"],null]]]],[11,\"target\",\"_blank\"],[9],[1,[29,\"svg-jar\",[\"help\"],[[\"class\"],[\"w4 h4 stroke-midgrey-l2\"]]],false],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n\"]],\"parameters\":[1]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showDeletePostModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-post\"],[[\"model\",\"close\",\"modifier\"],[[29,\"hash\",null,[[\"post\",\"onSuccess\"],[[25,[\"post\"]],[29,\"route-action\",[\"redirectToContentScreen\"],null]]]],[29,\"action\",[[24,0,[]],\"toggleDeletePostModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showLeaveEditorModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-editor\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveEditor\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveEditorModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"liquid-wormhole\",null,null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-post-settings-menu\",null,[[\"post\",\"showSettingsMenu\",\"deletePost\",\"updateSlug\",\"savePost\"],[[25,[\"post\"]],[25,[\"ui\",\"showSettingsMenu\"]],[29,\"action\",[[24,0,[]],\"toggleDeletePostModal\"],null],[25,[\"updateSlug\"]],[25,[\"savePost\"]]]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"\\n\"],[1,[23,\"outlet\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/editor.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/editor/edit-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "ZohFfshl",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-view\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/editor/edit-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/error", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "KD645lPH",
    "block": "{\"symbols\":[\"item\"],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-view\"],[9],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"flex flex-column items-center flex-grow justify-center h-100 nt10\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"absolute error-background nudge-right--5\"],[9],[0,\"\\n            \"],[1,[29,\"svg-jar\",[\"desert\"],[[\"class\"],[\"error-background absolute\"]]],false],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"traveler-1\"],[9],[1,[29,\"svg-jar\",[\"tumbleweed\"],[[\"class\"],[\"w6 h6 absolute bouncer-1\"]]],false],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"traveler-2\"],[9],[1,[29,\"svg-jar\",[\"tumbleweed\"],[[\"class\"],[\"w11 h11 absolute bouncer-2\"]]],false],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"absolute mt50 tc\"],[9],[0,\"\\n            \"],[7,\"h1\"],[11,\"class\",\"midlightgrey error-code-size fw6\"],[9],[1,[23,\"code\"],false],[10],[0,\"\\n            \"],[7,\"h2\"],[11,\"class\",\"midlightgrey f4 fw3\"],[9],[1,[23,\"message\"],false],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"stack\"]]],null,{\"statements\":[[0,\"        \"],[7,\"section\"],[11,\"class\",\"error-stack\"],[9],[0,\"\\n            \"],[7,\"h3\"],[9],[0,\"Stack Trace\"],[10],[0,\"\\n            \"],[7,\"p\"],[9],[7,\"strong\"],[9],[1,[23,\"message\"],false],[10],[10],[0,\"\\n            \"],[7,\"ul\"],[11,\"class\",\"error-stack-list\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"stack\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"li\"],[9],[0,\"\\n                        at\\n                        \"],[4,\"if\",[[24,1,[\"function\"]]],null,{\"statements\":[[7,\"em\"],[11,\"class\",\"error-stack-function\"],[9],[1,[24,1,[\"function\"]],false],[10]],\"parameters\":[]},null],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"error-stack-file\"],[9],[0,\"(\"],[1,[24,1,[\"at\"]],false],[0,\")\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/error.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/member", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "lEV0URZc",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"members\",\"members\"]],{\"statements\":[[0,\"Members\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            \"],[1,[25,[\"member\",\"name\"]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"flex flex-row nl2 mt10\"],[9],[0,\"\\n            \"],[5,\"gh-member-avatar\",[[13,\"class\",\"w20 h20 mr4\"]],[[\"@initialsClass\",\"@member\"],[\"f-subheadline\",[23,\"member\"]]]],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"flex flex-column justify-center\"],[9],[0,\"\\n                \"],[7,\"h3\"],[11,\"class\",\"ma0 pa0\"],[9],[1,[25,[\"member\",\"name\"]],false],[10],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"db\"],[9],[0,\"\\n                    \"],[7,\"a\"],[11,\"class\",\"midlightgrey\"],[12,\"href\",[30,[\"mailto:\",[25,[\"member\",\"email\"]]]]],[9],[0,\"\\n                        \"],[1,[25,[\"member\",\"email\"]],false],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"pt1 mt10\"],[9],[0,\"\\n            \"],[7,\"h2\"],[11,\"class\",\"f7 fw4 midgrey\"],[9],[0,\"Overview\"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"flex flex-row ba b--whitegrey br4 bg-whitegrey-l2 mt2\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"flex flex-column flex-grow-1 pa5 br b--whitegrey w-50\"],[9],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"db ttu f8 midlightgrey\"],[9],[0,\"Member since\"],[10],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"db f5\"],[9],[1,[29,\"moment-format\",[[25,[\"member\",\"createdAt\"]],\"MMMM Do\"],null],false],[10],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"db f8 midlightgrey\"],[9],[0,\"(\"],[1,[29,\"moment-to-now\",[[25,[\"member\",\"createdAt\"]]],[[\"hideAffix\"],[true]]],false],[0,\")\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"flex flex-column flex-grow-1 pa5 w-50\"],[9],[0,\"\\n                \"],[7,\"span\"],[11,\"class\",\"db ttu f8 midlightgrey\"],[9],[0,\"Current plan\"],[10],[0,\"\\n                \"],[7,\"span\"],[12,\"class\",[30,[\"db f5 \",[29,\"if\",[[29,\"eq\",[[25,[\"subscription\",\"plan\"]],\"...\"],null],\"midlightgrey\"],null]]]],[9],[1,[25,[\"subscription\",\"plan\"]],false],[10],[0,\"\\n\"],[4,\"unless\",[[29,\"eq\",[[25,[\"subscription\",\"amount\"]],\"...\"],null]],null,{\"statements\":[[0,\"                    \"],[7,\"span\"],[11,\"class\",\"db f8 midlightgrey\"],[9],[1,[25,[\"subscription\",\"amount\"]],false],[0,\" USD/month\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"h2\"],[11,\"class\",\"pb1 bb b--whitegrey f7 fw4 midgrey mt10 pb2\"],[9],[0,\"Danger zone\"],[10],[0,\"\\n        \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-red gh-btn-icon mt3\"],[11,\"data-test-button\",\"delete-member\"],[11,\"type\",\"button\"],[9],[0,\"\\n            \"],[7,\"span\"],[9],[0,\"Delete member\"],[10],[0,\"\\n        \"],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"showDeleteMemberModal\",[24,0,[]]],null]]],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDeleteMemberModal\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-member\"],[[\"model\",\"close\",\"modifier\"],[[29,\"hash\",null,[[\"member\",\"onSuccess\"],[[25,[\"member\"]],[29,\"action\",[[24,0,[]],\"finaliseDeletion\"],null]]]],[29,\"action\",[[24,0,[]],[29,\"toggle\",[\"showDeleteMemberModal\",[24,0,[]]],null]],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/member.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/members", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "mLvfx2AT",
    "block": "{\"symbols\":[\"member\",\"index\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Members\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"view-container h-100\"],[9],[0,\"\\n\\n        \"],[7,\"div\"],[9],[0,\"\\n            \"],[5,\"gh-text-input\",[[13,\"placeholder\",\"Search members...\"]],[[\"@value\",\"@input\"],[[24,0,[\"searchText\"]],[29,\"action\",[[24,0,[]],[29,\"mut\",[[24,0,[\"searchText\"]]],null]],[[\"value\"],[\"target.value\"]]]]]],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"flex justify-between items-center pt1 mt7\"],[9],[0,\"\\n            \"],[7,\"h2\"],[11,\"class\",\"f7 fw4 midgrey\"],[9],[0,\"\\n\"],[4,\"if\",[[24,0,[\"searchText\"]]],null,{\"statements\":[[0,\"                    Search result\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    All members\\n\"],[4,\"if\",[[24,0,[\"fetchMembers\",\"lastSuccessful\"]]],null,{\"statements\":[[0,\"                        (\"],[1,[24,0,[\"meta\",\"pagination\",\"total\"]],false],[0,\")\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        (Loading...)\\n\"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[24,0,[\"filteredMembers\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"flex flex-row flex-wrap items-start ba br3 b--whitegrey mt2 mb10\"],[9],[0,\"\\n                \"],[5,\"vertical-collection\",[],[[\"@items\",\"@key\",\"@containerSelector\",\"@estimateHeight\"],[[24,0,[\"filteredMembers\"]],\"id\",\".gh-main\",\"76\"]],{\"statements\":[[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"flex-grow-1 flex-shrink-1\"],[11,\"style\",\"flex-basis: 100%\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"classNames\",\"route\",\"model\"],[\"link whitegrey\",\"member\",[24,1,[]]]],{\"statements\":[[0,\"                            \"],[7,\"article\"],[12,\"class\",[30,[\"flex items-center justify-between pa4 \",[29,\"if\",[[24,2,[]],\"bt\"],null],\" hover-bg-table-cell\"]]],[9],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"flex items-center\"],[9],[0,\"\\n                                    \"],[5,\"gh-member-avatar\",[[13,\"class\",\"w10 h10 mr4\"]],[[\"@member\"],[[24,1,[]]]]],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"flex flex-column\"],[9],[0,\"\\n                                        \"],[7,\"h3\"],[11,\"class\",\"darkgrey ma0 f5 fw7\"],[9],[1,[24,1,[\"name\"]],false],[10],[0,\"\\n                                        \"],[7,\"p\"],[11,\"class\",\"midgrey ma0 pa0 f7\"],[9],[1,[24,1,[\"email\"]],false],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"flex items-center\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],[[\"class\"],[\"w4 h4 fill-midlightgrey\"]]],false],[10],[0,\"\\n                            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n                \"]],\"parameters\":[1,2]}],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[24,0,[\"fetchMembers\",\"isRunning\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n                    \"],[5,\"gh-loading-spinner\",[],[[],[]]],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                \"],[7,\"p\"],[11,\"class\",\"mt2 pt10 bt b--whitegrey tc midlightgrey\"],[9],[0,\"No members found.\"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/members.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/pages-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "/Lq4oU6s",
    "block": "{\"symbols\":[\"order\",\"tag\",\"type\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Pages\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"data-test-new-page-button\",\"route\",\"model\"],[\"gh-btn gh-btn-green\",true,\"editor.new\",\"page\"]],{\"statements\":[[7,\"span\"],[9],[0,\"New page\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-left\"],[9],[0,\"\\n\"],[4,\"power-select\",null,[[\"placeholder\",\"selected\",\"options\",\"searchField\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"matchTriggerWidth\",\"data-test-type-select\"],[\"All pages\",[25,[\"selectedType\"]],[25,[\"availableTypes\"]],\"name\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"k\"]]],null]],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-type\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,3,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[3]},null],[0,\"\\n\"],[4,\"power-select\",null,[[\"placeholder\",\"selected\",\"options\",\"searchField\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"searchPlaceholder\",\"matchTriggerWidth\",\"data-test-tag-select\"],[\"All tags\",[25,[\"selectedTag\"]],[25,[\"availableTags\"]],\"name\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"k\"]]],null]],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-tag\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"Search tags\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,2,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-right\"],[9],[0,\"\\n            Sort by:\\n\"],[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"horizontalPosition\",\"matchTriggerWidth\",\"data-test-order-select\"],[[25,[\"selectedOrder\"]],[25,[\"availableOrders\"]],false,[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"k\"]]],null]],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-sort\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"right\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,1,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/pages-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/pages", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "Km2+4PK6",
    "block": "{\"symbols\":[\"page\",\"order\",\"tag\",\"type\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Pages\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"data-test-new-page-button\",\"route\",\"model\"],[\"gh-btn gh-btn-green\",true,\"editor.new\",\"page\"]],{\"statements\":[[7,\"span\"],[9],[0,\"New page\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-left\"],[9],[0,\"\\n\"],[4,\"unless\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"matchTriggerWidth\",\"data-test-type-select\"],[[25,[\"selectedType\"]],[25,[\"availableTypes\"]],false,[29,\"action\",[[24,0,[]],\"changeType\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-type\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,4,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[4]},null]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"unless\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[4,\"power-select\",null,[[\"selected\",\"options\",\"searchField\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"searchPlaceholder\",\"matchTriggerWidth\",\"optionsComponent\",\"data-test-tag-select\"],[[25,[\"selectedTag\"]],[25,[\"availableTags\"]],\"name\",[29,\"action\",[[24,0,[]],\"changeTag\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-tag\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"Search tags\",false,\"power-select-vertical-collection-options\",true]],{\"statements\":[[0,\"                \"],[1,[24,3,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[3]},null]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-right\"],[9],[0,\"\\n            Sort by:\\n\"],[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"horizontalPosition\",\"matchTriggerWidth\",\"data-test-order-select\"],[[25,[\"selectedOrder\"]],[25,[\"availableOrders\"]],false,[29,\"action\",[[24,0,[]],\"changeOrder\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-sort\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"right\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,2,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"content-list\"],[9],[0,\"\\n        \"],[7,\"ol\"],[11,\"class\",\"posts-list\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"postsInfinityModel\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-posts-list-item\",null,[[\"post\",\"currentUserId\",\"onDoubleClick\",\"data-test-page-id\"],[[24,1,[]],[25,[\"currentUserId\"]],[29,\"action\",[[24,0,[]],\"openEditor\"],null],[24,1,[\"id\"]]]]],false],[0,\"\\n\"]],\"parameters\":[1]},{\"statements\":[[0,\"                \"],[7,\"li\"],[11,\"class\",\"no-posts-box\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"no-posts\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"showingAll\"]]],null,{\"statements\":[[0,\"                            \"],[1,[29,\"svg-jar\",[\"pages-placeholder\"],[[\"class\"],[\"gh-pages-placeholder\"]]],false],[0,\"\\n                            \"],[7,\"h3\"],[9],[0,\"You haven't created any pages yet!\"],[10],[0,\"\\n                            \"],[4,\"link-to\",null,[[\"route\",\"model\"],[\"editor.new\",\"page\"]],{\"statements\":[[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-green gh-btn-lg\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Create a new page\"],[10],[10]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                            \"],[7,\"h3\"],[9],[0,\"No pages match the current filter\"],[10],[0,\"\\n                            \"],[4,\"link-to\",null,[[\"query\",\"route\"],[[29,\"hash\",null,[[\"type\",\"author\",\"tag\"],[null,null,null]]],\"pages.index\"]],{\"statements\":[[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-lg\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Show all pages\"],[10],[10]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"        \"],[10],[0,\"\\n\\n        \"],[1,[29,\"gh-infinity-loader\",null,[[\"infinityModel\",\"scrollable\",\"triggerOffset\"],[[25,[\"postsInfinityModel\"]],\".gh-main\",1000]]],false],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[1,[23,\"outlet\"],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/pages.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/posts-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "k1tIsul0",
    "block": "{\"symbols\":[\"order\",\"tag\",\"type\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Your posts\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"data-test-new-post-button\",\"route\",\"model\"],[\"gh-btn gh-btn-green\",true,\"editor.new\",\"post\"]],{\"statements\":[[7,\"span\"],[9],[0,\"New post\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-left\"],[9],[0,\"\\n\"],[4,\"power-select\",null,[[\"placeholder\",\"selected\",\"options\",\"searchField\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"matchTriggerWidth\",\"data-test-type-select\"],[\"All posts\",[25,[\"selectedType\"]],[25,[\"availableTypes\"]],\"name\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"k\"]]],null]],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-type\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,3,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[3]},null],[0,\"\\n\"],[4,\"power-select\",null,[[\"placeholder\",\"selected\",\"options\",\"searchField\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"searchPlaceholder\",\"matchTriggerWidth\",\"data-test-tag-select\"],[\"All tags\",[25,[\"selectedTag\"]],[25,[\"availableTags\"]],\"name\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"k\"]]],null]],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-tag\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"Search tags\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,2,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-right\"],[9],[0,\"\\n            Sort by:\\n\"],[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"horizontalPosition\",\"matchTriggerWidth\",\"data-test-order-select\"],[[25,[\"selectedOrder\"]],[25,[\"availableOrders\"]],false,[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"k\"]]],null]],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-sort\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"right\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,1,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/posts-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/posts", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "3+Lz57LR",
    "block": "{\"symbols\":[\"post\",\"order\",\"tag\",\"type\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Posts\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"data-test-new-post-button\",\"route\",\"model\"],[\"gh-btn gh-btn-green\",true,\"editor.new\",\"post\"]],{\"statements\":[[7,\"span\"],[9],[0,\"New post\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-left\"],[9],[0,\"\\n\"],[4,\"unless\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"matchTriggerWidth\",\"data-test-type-select\"],[[25,[\"selectedType\"]],[25,[\"availableTypes\"]],false,[29,\"action\",[[24,0,[]],\"changeType\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-type\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,4,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[4]},null]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"unless\",[[25,[\"session\",\"user\",\"isContributor\"]]],null,{\"statements\":[[4,\"power-select\",null,[[\"selected\",\"options\",\"searchField\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"searchPlaceholder\",\"matchTriggerWidth\",\"optionsComponent\",\"data-test-tag-select\"],[[25,[\"selectedTag\"]],[25,[\"availableTags\"]],\"name\",[29,\"action\",[[24,0,[]],\"changeTag\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-tag\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"Search tags\",false,\"power-select-vertical-collection-options\",true]],{\"statements\":[[0,\"                \"],[1,[24,3,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[3]},null]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-right\"],[9],[0,\"\\n            Sort by:\\n\"],[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"horizontalPosition\",\"matchTriggerWidth\",\"data-test-order-select\"],[[25,[\"selectedOrder\"]],[25,[\"availableOrders\"]],false,[29,\"action\",[[24,0,[]],\"changeOrder\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-sort\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",\"right\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,2,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"content-list\"],[9],[0,\"\\n        \"],[7,\"ol\"],[11,\"class\",\"posts-list\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"postsInfinityModel\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-posts-list-item\",null,[[\"post\",\"currentUserId\",\"onDoubleClick\",\"data-test-post-id\"],[[24,1,[]],[25,[\"currentUserId\"]],[29,\"action\",[[24,0,[]],\"openEditor\"],null],[24,1,[\"id\"]]]]],false],[0,\"\\n\"]],\"parameters\":[1]},{\"statements\":[[0,\"                \"],[7,\"li\"],[11,\"class\",\"no-posts-box\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"no-posts\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"showingAll\"]]],null,{\"statements\":[[0,\"                            \"],[1,[29,\"svg-jar\",[\"posts-placeholder\"],[[\"class\"],[\"gh-posts-placeholder\"]]],false],[0,\"\\n                            \"],[7,\"h3\"],[9],[0,\"You haven't written any posts yet!\"],[10],[0,\"\\n                            \"],[4,\"link-to\",null,[[\"route\",\"model\"],[\"editor.new\",\"post\"]],{\"statements\":[[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-green gh-btn-lg\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Write a new post\"],[10],[10]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                            \"],[7,\"h3\"],[9],[0,\"No posts match the current filter\"],[10],[0,\"\\n                            \"],[4,\"link-to\",null,[[\"query\",\"route\"],[[29,\"hash\",null,[[\"type\",\"author\",\"tag\"],[null,null,null]]],\"posts.index\"]],{\"statements\":[[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-lg\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Show all posts\"],[10],[10]],\"parameters\":[]},null],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"        \"],[10],[0,\"\\n\\n        \"],[1,[29,\"gh-infinity-loader\",null,[[\"infinityModel\",\"scrollable\",\"triggerOffset\"],[[25,[\"postsInfinityModel\"]],\".gh-main\",1000]]],false],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[1,[23,\"outlet\"],false],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/posts.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/code-injection-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "bX0ysOId",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Code injection\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"disabled\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true,true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/code-injection-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/code-injection", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "ookKbSBd",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Code injection\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-continer\"],[9],[0,\"\\n        \"],[7,\"form\"],[11,\"id\",\"settings-code\"],[11,\"novalidate\",\"novalidate\"],[9],[0,\"\\n            \"],[7,\"fieldset\"],[9],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"\\n                    Ghost allows you to inject code into the top and bottom of your theme files without editing them. This allows for quick modifications to insert useful things like tracking codes and meta tags.\\n                \"],[10],[0,\"\\n\\n                \"],[7,\"div\"],[11,\"class\",\"form-group settings-code\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"for\",\"ghost-head\"],[9],[0,\"Site Header\"],[10],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"Code here will be injected into the \"],[7,\"code\"],[9],[0,\"{{ghost_head}}\"],[10],[0,\" tag on every page of the site\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-cm-editor\",[[25,[\"settings\",\"codeinjectionHead\"]]],[[\"id\",\"class\",\"name\",\"type\",\"update\"],[\"ghost-head\",\"gh-input settings-code-editor\",\"codeInjection[ghost_head]\",\"text\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"codeinjectionHead\"]]],null]],null]]]],false],[0,\"\\n                \"],[10],[0,\"\\n\\n                \"],[7,\"div\"],[11,\"class\",\"form-group settings-code\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"for\",\"ghost-foot\"],[9],[0,\"Site Footer\"],[10],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"Code here will be injected into the \"],[7,\"code\"],[9],[0,\"{{ghost_foot}}\"],[10],[0,\" tag on every page of the site\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-cm-editor\",[[25,[\"settings\",\"codeinjectionFoot\"]]],[[\"id\",\"class\",\"name\",\"type\",\"update\"],[\"ghost-foot\",\"gh-input settings-code-editor\",\"codeInjection[ghost_foot]\",\"text\",[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"codeinjectionFoot\"]]],null]],null]]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/code-injection.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/design-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "qw5p9kTs",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Design\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"disabled\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/design-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/design", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "e+RZh7n2",
    "block": "{\"symbols\":[\"navItem\",\"index\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Design\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Navigation\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-blognav-container\"],[9],[0,\"\\n            \"],[7,\"form\"],[11,\"id\",\"settings-navigation\"],[11,\"class\",\"gh-blognav\"],[11,\"novalidate\",\"novalidate\"],[9],[0,\"\\n\"],[4,\"sortable-objects\",null,[[\"sortableObjectList\",\"useSwap\"],[[25,[\"settings\",\"navigation\"]],false]],{\"statements\":[[4,\"each\",[[25,[\"settings\",\"navigation\"]]],null,{\"statements\":[[4,\"draggable-object\",null,[[\"content\",\"dragHandle\",\"isSortable\"],[[24,1,[]],\".gh-blognav-grab\",true]],{\"statements\":[[0,\"                            \"],[1,[29,\"gh-navitem\",null,[[\"navItem\",\"baseUrl\",\"addItem\",\"deleteItem\",\"updateUrl\",\"updateLabel\",\"data-test-navitem\"],[[24,1,[]],[25,[\"blogUrl\"]],[29,\"action\",[[24,0,[]],\"addNavItem\"],null],[29,\"action\",[[24,0,[]],\"deleteNavItem\"],null],[29,\"action\",[[24,0,[]],\"updateUrl\"],null],[29,\"action\",[[24,0,[]],\"updateLabel\"],null],[24,2,[]]]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[1,2]},null]],\"parameters\":[]},null],[0,\"                \"],[1,[29,\"gh-navitem\",null,[[\"navItem\",\"baseUrl\",\"addItem\",\"updateUrl\",\"data-test-navitem\"],[[25,[\"newNavItem\"]],[25,[\"blogUrl\"]],[29,\"action\",[[24,0,[]],\"addNavItem\"],null],[29,\"action\",[[24,0,[]],\"updateUrl\"],null],\"new\"]]],false],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Theme Directory\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-theme-directory-container\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"theme-directory\"],[9],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-item\"],[11,\"href\",\"https://github.com/TryGhost/Massively\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-screenshot\"],[9],[0,\"\\n                        \"],[7,\"img\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/themes/massively.jpg\"],[11,\"alt\",\"Massively Theme\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-desc\"],[9],[0,\"\\n                        \"],[7,\"div\"],[9],[0,\"Massively \"],[7,\"span\"],[9],[0,\"— Free\"],[10],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-item\"],[11,\"href\",\"https://themeforest.net/item/nurui-multipurpose-ghost-blog-theme/22243886\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-screenshot\"],[9],[0,\"\\n                        \"],[7,\"img\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/themes/nurui.jpg\"],[11,\"alt\",\"Nurui Theme\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-desc\"],[9],[0,\"\\n                        \"],[7,\"div\"],[9],[0,\"Nurui \"],[7,\"span\"],[9],[0,\"— Premium\"],[10],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-item\"],[11,\"href\",\"https://themeforest.net/item/pacific-big-bold-photographydriven-theme/19774541\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-screenshot\"],[9],[0,\"\\n                        \"],[7,\"img\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/themes/pacific.jpg\"],[11,\"alt\",\"Pacific Theme\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-desc\"],[9],[0,\"\\n                        \"],[7,\"div\"],[9],[0,\"Pacific \"],[7,\"span\"],[9],[0,\"— Premium\"],[10],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-item\"],[11,\"href\",\"https://www.hauntedthemes.com/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-screenshot\"],[9],[0,\"\\n                        \"],[7,\"img\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/themes/farafra.jpg\"],[11,\"alt\",\"Farafra Theme\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-desc\"],[9],[0,\"\\n                        \"],[7,\"div\"],[9],[0,\"Farafra \"],[7,\"span\"],[9],[0,\"— Premium\"],[10],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-item\"],[11,\"href\",\"https://themeforest.net/item/valkyrie-a-highly-visual-ghost-blog/22576630\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-screenshot\"],[9],[0,\"\\n                        \"],[7,\"img\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/themes/valkyrie.jpg\"],[11,\"alt\",\"Valkyrie Theme\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-desc\"],[9],[0,\"\\n                        \"],[7,\"div\"],[9],[0,\"Valkyrie \"],[7,\"span\"],[9],[0,\"— Premium\"],[10],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-item\"],[11,\"href\",\"https://themeforest.net/item/sente-magazine-ghost-blog-theme/21019644\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-screenshot\"],[9],[0,\"\\n                        \"],[7,\"img\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/themes/sente.jpg\"],[11,\"alt\",\"Sente Theme\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-item-desc\"],[9],[0,\"\\n                        \"],[7,\"div\"],[9],[0,\"Sente \"],[7,\"span\"],[9],[0,\"— Premium\"],[10],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n            \"],[7,\"div\"],[11,\"class\",\"td-cta\"],[9],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-cta-box td-cta-marketplace\"],[11,\"href\",\"https://ghost.org/marketplace/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-cta-icon\"],[9],[1,[29,\"svg-jar\",[\"store\"],null],false],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-cta-content-wrapper\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"td-cta-content\"],[9],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Theme Marketplace\"],[10],[0,\"\\n                            \"],[7,\"p\"],[9],[0,\"Explore a huge range of free and premium themes for Ghost with a range of design and layout options\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"td-cta-arrow\"],[9],[0,\"\\n                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"a\"],[11,\"class\",\"td-cta-box td-cta-docs\"],[11,\"href\",\"https://docs.ghost.org/api/handlebars-themes/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-cta-icon\"],[9],[1,[29,\"svg-jar\",[\"book-open\"],null],false],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"td-cta-content-wrapper\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"td-cta-content\"],[9],[0,\"\\n                            \"],[7,\"h4\"],[9],[0,\"Developer Docs\"],[10],[0,\"\\n                            \"],[7,\"p\"],[9],[0,\"Build your own custom Ghost theme from scratch using our comprehensive Handlebars.js SDK\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"td-cta-arrow\"],[9],[0,\"\\n                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Installed Themes\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-themes-container\"],[9],[0,\"\\n\\n            \"],[1,[29,\"gh-theme-table\",null,[[\"themes\",\"activateTheme\",\"downloadTheme\",\"deleteTheme\"],[[25,[\"themes\"]],[29,\"action\",[[24,0,[]],\"activateTheme\"],null],[29,\"action\",[[24,0,[]],\"downloadTheme\"],null],[29,\"action\",[[24,0,[]],\"deleteTheme\"],null]]]],false],[0,\"\\n\\n\"],[4,\"link-to\",null,[[\"class\",\"data-test-upload-theme-button\",\"route\"],[\"gh-btn gh-btn-green gh-themes-uploadbtn\",true,\"settings.design.uploadtheme\"]],{\"statements\":[[0,\"                \"],[7,\"span\"],[9],[0,\"Upload a theme\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDeleteThemeModal\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-theme\"],[[\"model\",\"close\",\"confirm\",\"modifier\"],[[29,\"hash\",null,[[\"theme\",\"download\"],[[25,[\"themeToDelete\"]],[29,\"action\",[[24,0,[]],\"downloadTheme\",[25,[\"themeToDelete\"]]],null]]]],[29,\"action\",[[24,0,[]],\"hideDeleteThemeModal\"],null],[29,\"action\",[[24,0,[]],\"deleteTheme\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showThemeWarningsModal\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-fullscreen-modal\",[\"theme-warnings\"],[[\"model\",\"close\",\"modifier\"],[[29,\"hash\",null,[[\"title\",\"warnings\",\"errors\",\"canActivate\"],[\"Activation successful\",[25,[\"themeWarnings\"]],[25,[\"themeErrors\"]],true]]],[29,\"action\",[[24,0,[]],\"hideThemeWarningsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showThemeErrorsModal\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-fullscreen-modal\",[\"theme-warnings\"],[[\"model\",\"close\",\"modifier\"],[[29,\"hash\",null,[[\"title\",\"errors\",\"fatalErrors\",\"canActivate\"],[\"Activation failed\",[25,[\"themeErrors\"]],[25,[\"themeFatalErrors\"]],false]]],[29,\"action\",[[24,0,[]],\"hideThemeWarningsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[1,[23,\"outlet\"],false],[0,\"\\n\\n\"],[1,[29,\"gh-tour-item\",[\"upload-a-theme\"],[[\"target\",\"throbberAttachment\",\"popoverTriangleClass\"],[\".gh-themes-uploadbtn\",\"top middle\",\"bottom\"]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/design.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/design/uploadtheme", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "67zdLXUb",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-fullscreen-modal\",[\"upload-theme\"],[[\"model\",\"close\",\"modifier\"],[[29,\"hash\",null,[[\"themes\",\"activate\"],[[25,[\"themes\"]],[29,\"route-action\",[\"activateTheme\"],null]]]],[29,\"route-action\",[\"cancel\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/design/uploadtheme.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/general-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "f5hD8DI4",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"General\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",[\"Save settings\"],[[\"task\",\"class\",\"disabled\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true,true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/general-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/general", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "jqqzP9HU",
    "block": "{\"symbols\":[\"uploader\",\"error\",\"uploader\",\"error\",\"uploader\",\"error\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"General\"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",[\"Save settings\"],[[\"task\",\"class\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Publication info\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Title & description\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"The details used to identify your publication around the web\"],[10],[0,\"\\n\"],[4,\"liquid-if\",[[25,[\"pubInfoOpen\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"title\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-title-input\"],[[29,\"readonly\",[[25,[\"settings\",\"title\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"title\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"title\"],[[\"target\"],[[25,[\"settings\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"settings\",\"errors\"]],\"title\"]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"The name of your site\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\",\"class\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"description\",\"description-container\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-description-input\"],[[29,\"readonly\",[[25,[\"settings\",\"description\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"description\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"description\"],[[\"target\"],[[25,[\"settings\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"settings\",\"errors\"]],\"description\"]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Used in your theme, meta data and search results\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"data-test-toggle-pub-info\",\"\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"if\",[[25,[\"pubInfoOpen\"]],\"Close\",\"Expand\"],null],false],[10],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"pubInfoOpen\",[24,0,[]]],null]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Site timezone\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Set the time and date of your publication, used for all published posts\"],[10],[0,\"\\n\"],[4,\"liquid-if\",[[25,[\"timezoneOpen\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-timezone-select\",null,[[\"activeTimezone\",\"availableTimezones\",\"update\"],[[25,[\"settings\",\"activeTimezone\"]],[25,[\"availableTimezones\"]],[29,\"action\",[[24,0,[]],\"setTimezone\"],null]]]],false],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"data-test-toggle-timezone\",\"\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"if\",[[25,[\"timezoneOpen\"]],\"Close\",\"Expand\"],null],false],[10],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"timezoneOpen\",[24,0,[]]],null]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Publication Language\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Set the language/locale which is used on your site\"],[10],[0,\"\\n\"],[4,\"liquid-if\",[[25,[\"defaultLocaleOpen\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"defaultLocale\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-default-locale-input\"],[[29,\"readonly\",[[25,[\"settings\",\"defaultLocale\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"defaultLocale\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"defaultLocale\"],[[\"target\"],[[25,[\"settings\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"settings\",\"errors\"]],\"defaultLocale\"]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Default: English (\"],[7,\"strong\"],[9],[0,\"en\"],[10],[0,\"); you can add translation files to your theme for \"],[7,\"a\"],[11,\"href\",\"https://docs.ghost.org/api/handlebars-themes/helpers/translate/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener\"],[9],[0,\"any language\"],[10],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"data-test-toggle-default-locale\",\"\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"if\",[[25,[\"defaultLocaleOpen\"]],\"Close\",\"Expand\"],null],false],[10],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"defaultLocaleOpen\",[24,0,[]]],null]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Publication identity\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[11,\"data-test-setting\",\"icon\"],[9],[0,\"\\n\"],[4,\"gh-uploader\",null,[[\"extensions\",\"paramsHash\",\"onComplete\"],[[25,[\"iconExtensions\"]],[29,\"hash\",null,[[\"purpose\"],[\"icon\"]]],[29,\"action\",[[24,0,[]],\"imageUploaded\",\"icon\"],null]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Publication icon\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"A square, social icon used in the UI of your publication, at least 60x60px\"],[10],[0,\"\\n\"],[4,\"each\",[[24,5,[\"errors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-error\"],[11,\"data-test-error\",\"icon\"],[9],[1,[29,\"or\",[[24,6,[\"context\"]],[24,6,[\"message\"]]],null],false],[10],[0,\"\\n\"]],\"parameters\":[6]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action gh-setting-action-smallimg\"],[9],[0,\"\\n\"],[4,\"if\",[[24,5,[\"isUploading\"]]],null,{\"statements\":[[0,\"                    \"],[1,[24,5,[\"progressBar\"]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"settings\",\"icon\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"img\"],[11,\"class\",\"blog-icon\"],[12,\"src\",[30,[[25,[\"settings\",\"icon\"]]]]],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"alt\",\"icon\"],[11,\"data-test-icon-img\",\"\"],[9],[10],[0,\"\\n                    \"],[7,\"button\"],[11,\"class\",\"gh-setting-action-smallimg-delete\"],[11,\"data-test-delete-image\",\"icon\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"delete\"],[10],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"removeImage\",\"icon\"]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"data-test-image-upload-btn\",\"icon\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"Upload Image\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"                \"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-file-input\",null,[[\"multiple\",\"action\",\"accept\",\"data-test-file-input\"],[false,[24,5,[\"setFiles\"]],[25,[\"iconMimeTypes\"]],\"icon\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[5]},null],[0,\"        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[11,\"data-test-setting\",\"logo\"],[9],[0,\"\\n\"],[4,\"gh-uploader\",null,[[\"extensions\",\"onComplete\"],[[25,[\"imageExtensions\"]],[29,\"action\",[[24,0,[]],\"imageUploaded\",\"logo\"],null]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Publication logo\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"The primary logo for your brand displayed across your theme, should be transparent and at least 600px x 72px\"],[10],[0,\"\\n\"],[4,\"each\",[[24,3,[\"errors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-error\"],[11,\"data-test-error\",\"logo\"],[9],[1,[29,\"or\",[[24,4,[\"context\"]],[24,4,[\"message\"]]],null],false],[10],[0,\"\\n\"]],\"parameters\":[4]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action gh-setting-action-smallimg\"],[9],[0,\"\\n\"],[4,\"if\",[[24,3,[\"isUploading\"]]],null,{\"statements\":[[0,\"                    \"],[1,[24,3,[\"progressBar\"]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"settings\",\"logo\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"img\"],[11,\"class\",\"blog-logo\"],[12,\"src\",[30,[[25,[\"settings\",\"logo\"]]]]],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"alt\",\"logo\"],[11,\"data-test-logo-img\",\"\"],[9],[10],[0,\"\\n                    \"],[7,\"button\"],[11,\"class\",\"gh-setting-action-smallimg-delete\"],[11,\"data-test-delete-image\",\"logo\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"delete\"],[10],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"removeImage\",\"logo\"]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"data-test-image-upload-btn\",\"logo\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"Upload Image\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"                \"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-file-input\",null,[[\"multiple\",\"action\",\"accept\",\"data-test-file-input\"],[false,[24,3,[\"setFiles\"]],[25,[\"imageMimeTypes\"]],\"logo\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[3]},null],[0,\"        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[11,\"data-test-setting\",\"coverImage\"],[9],[0,\"\\n\"],[4,\"gh-uploader\",null,[[\"extensions\",\"onComplete\"],[[25,[\"imageExtensions\"]],[29,\"action\",[[24,0,[]],\"imageUploaded\",\"coverImage\"],null]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Publication cover\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"An optional large background image for your site\"],[10],[0,\"\\n\"],[4,\"each\",[[24,1,[\"errors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-error\"],[11,\"data-test-error\",\"coverImage\"],[9],[1,[29,\"or\",[[24,2,[\"context\"]],[24,2,[\"message\"]]],null],false],[10],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action gh-setting-action-largeimg\"],[9],[0,\"\\n\"],[4,\"if\",[[24,1,[\"isUploading\"]]],null,{\"statements\":[[0,\"                    \"],[1,[24,1,[\"progressBar\"]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"settings\",\"coverImage\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"img\"],[11,\"class\",\"blog-cover\"],[12,\"src\",[30,[[25,[\"settings\",\"coverImage\"]]]]],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"alt\",\"cover photo\"],[11,\"data-test-cover-img\",\"\"],[9],[10],[0,\"\\n                    \"],[7,\"button\"],[11,\"class\",\"gh-setting-action-largeimg-delete\"],[11,\"data-test-delete-image\",\"coverImage\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"delete\"],[10],[0,\"\\n                    \"],[3,\"action\",[[24,0,[]],\"removeImage\",\"coverImage\"]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[7,\"button\"],[11,\"class\",\"gh-btn\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"data-test-image-upload-btn\",\"coverImage\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"Upload Image\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"                \"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-file-input\",null,[[\"multiple\",\"action\",\"accept\",\"data-test-file-input\"],[false,[24,1,[\"setFiles\"]],[25,[\"imageMimeTypes\"]],\"coverImage\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Rocket-Chat\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Rocket.Chat server Url\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Set the url where rocket chat server is running\"],[10],[0,\"\\n\"],[4,\"liquid-if\",[[25,[\"chatServerUrl\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"serverUrl\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-server-url-input\"],[[29,\"readonly\",[[25,[\"settings\",\"serverUrl\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"serverUrl\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\"],[[\"target\"],[[25,[\"settings\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"settings\",\"errors\"]],\"serverUrl\"]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"All the api call will use this url. Verify it before changing.\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"data-test-toggle-server-url\",\"\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"if\",[[25,[\"chatServerUrl\"]],\"Close\",\"Expand\"],null],false],[10],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"chatServerUrl\",[24,0,[]]],null]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Tokens\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Tokens which are used for announcing posts\"],[10],[0,\"\\n\"],[4,\"liquid-if\",[[25,[\"tokenOpen\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"announce-token\"]],{\"statements\":[[0,\"                    \"],[7,\"p\"],[9],[0,\"Announcement Token\"],[10],[0,\"\\n                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-announce-token-input\"],[[29,\"readonly\",[[25,[\"settings\",\"announceToken\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"announceToken\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"announceToken\"],[[\"target\"],[[25,[\"settings\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"settings\",\"errors\"]],\"announceToken\"]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"All the outgoing call will use this token. Verify it before changing.\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"setting-token\"]],{\"statements\":[[0,\"                    \"],[7,\"p\"],[9],[0,\"Settings Token\"],[10],[0,\"\\n                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-settings-token-input\"],[[29,\"readonly\",[[25,[\"settings\",\"settingsToken\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"settingsToken\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"settingsToken\"],[[\"target\"],[[25,[\"settings\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"settings\",\"errors\"]],\"settingsToken\"]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"All the incoming call will use this token. Verify it before changing.\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"data-test-toggle-pub-info\",\"\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"if\",[[25,[\"tokenOpen\"]],\"Close\",\"Expand\"],null],false],[10],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"tokenOpen\",[24,0,[]]],null]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Article Announcement\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"\\n                    Allow the announcement of articles when publish.\\n                \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"settings\",\"isAnnounced\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"room\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"focus-out\",\"data-test-room-input\"],[[29,\"readonly\",[[25,[\"settings\",\"roomName\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"_scratchRoom\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validateRoom\"],null],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-room-error\"],[[25,[\"settings\",\"errors\"]],\"room\",true]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"Channel name for announcement\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"settings-announced\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[25,[\"settings\",\"isAnnounced\"]]],[11,\"id\",\"settings-announced\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"toggleIsAnnounced\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-announced-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Authors Room for announcement\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"\\n                    Allow authors to change the room for announcing their articles.\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"settings-authors-rooms\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"disabled\",[29,\"not\",[[25,[\"settings\",\"isAnnounced\"]]],null]],[12,\"checked\",[25,[\"settings\",\"isAuthorsRooms\"]]],[11,\"id\",\"settings-authors-rooms\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"toggleIsAuthorsRooms\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-authors-rooms-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Allow Collaboration from Room\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"\\n                    Allow author to add others as authors from announcing room.\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"settings-collaboration\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"disabled\",[29,\"not\",[[25,[\"settings\",\"isAnnounced\"]]],null]],[12,\"checked\",[25,[\"settings\",\"canCollaborate\"]]],[11,\"id\",\"settings-collaboration\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"toggleCanCollaborate\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-collaboration-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Only Owner, Admin can add users\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"\\n                    If enabled, then only invited users can access.\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"settings-invite-only\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[25,[\"settings\",\"inviteOnly\"]]],[11,\"id\",\"settings-invite-only\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"toggleInviteOnly\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-invite-only-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Comments on Articles\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"\\n                    Create a Discussion in RC on Articles when published, and show on footer.\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"settings-comments\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[25,[\"settings\",\"isComments\"]]],[11,\"id\",\"settings-comments\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"toggleIsComments\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-comments-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Social accounts\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Social accounts\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Link your social accounts for full structured data and rich card support\"],[10],[0,\"\\n\"],[4,\"liquid-if\",[[25,[\"socialOpen\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"facebook\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"type\",\"placeholder\",\"autocorrect\",\"value\",\"input\",\"focus-out\",\"data-test-facebook-input\"],[\"url\",\"https://www.facebook.com/ghost\",\"off\",[29,\"readonly\",[[25,[\"settings\",\"facebook\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"_scratchFacebook\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validateFacebookUrl\"],null],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-facebook-error\"],[[25,[\"settings\",\"errors\"]],\"facebook\",true]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"URL of your publication's Facebook Page\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"twitter\"]],{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"type\",\"placeholder\",\"autocorrect\",\"value\",\"input\",\"focus-out\",\"data-test-twitter-input\"],[\"url\",\"https://twitter.com/ghost\",\"off\",[29,\"readonly\",[[25,[\"settings\",\"twitter\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"_scratchTwitter\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validateTwitterUrl\"],null],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-twitter-error\"],[[25,[\"settings\",\"errors\"]],\"twitter\",true]]],false],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"URL of your publication's Twitter profile\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"data-test-toggle-social\",\"\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[1,[29,\"if\",[[25,[\"socialOpen\"]],\"Close\",\"Expand\"],null],false],[10],[3,\"action\",[[24,0,[]],[29,\"toggle\",[\"socialOpen\",[24,0,[]]],null]]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Advanced settings\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Make this site private\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"\\n                    Enable protection with simple shared password. All search engine optimization and social features will be disabled.\\n\\n\"],[4,\"if\",[[25,[\"settings\",\"isPrivate\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"span\"],[11,\"class\",\"avoid-break-out\"],[9],[0,\"\\n                            \"],[7,\"br\"],[9],[10],[7,\"br\"],[9],[10],[0,\"\\n                            A private RSS feed is available at\\n                            \"],[7,\"a\"],[12,\"href\",[30,[[23,\"privateRSSUrl\"]]]],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener\"],[9],[1,[23,\"privateRSSUrl\"],false],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"settings\",\"isPrivate\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"settings\",\"errors\"]],[25,[\"settings\",\"hasValidated\"]],\"password\"]],{\"statements\":[[0,\"                            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"name\",\"focus-out\",\"input\",\"data-test-password-input\"],[[29,\"readonly\",[[25,[\"settings\",\"password\"]]],null],\"general[password]\",[29,\"action\",[[24,0,[]],\"validate\",\"password\"],[[\"target\"],[[25,[\"settings\"]]]]],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"settings\",\"password\"]]],null]],[[\"value\"],[\"target.value\"]]],true]]],false],[0,\"\\n                            \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-password-error\"],[[25,[\"settings\",\"errors\"]],\"password\",true]]],false],[0,\"\\n                            \"],[7,\"p\"],[9],[0,\"Set the password for this site\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"class\",\"checkbox\"],[11,\"for\",\"settings-private\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[25,[\"settings\",\"isPrivate\"]]],[11,\"id\",\"settings-private\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"toggleIsPrivate\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-private-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[1,[23,\"outlet\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/general.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integration-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "FfhF1kfp",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"integrations-back\",\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integration-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integration", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "0H68K/M8",
    "block": "{\"symbols\":[\"webhook\",\"uploader\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"form\"],[11,\"class\",\"mb15\"],[9],[0,\"\\n        \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n            \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n                \"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"integrations-back\",\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n                \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n                \"],[1,[25,[\"integration\",\"name\"]],false],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n                \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",\"save\"]]],false],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"flex\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"flex flex-column mr8 items-start\"],[9],[0,\"\\n                \"],[7,\"label\"],[9],[0,\"Icon\"],[10],[0,\"\\n                \"],[7,\"figure\"],[11,\"class\",\"relative flex items-center h-100 ma0 br4 hide-child ba b--whitegrey-d2 pa8 mt1\"],[12,\"style\",[23,\"iconImageStyle\"]],[9],[0,\"\\n\"],[4,\"aspect-ratio-box\",null,[[\"class\",\"ratio\",\"base\"],[\"flex items-center h-100\",\"1/1\",\"height\"]],{\"statements\":[[4,\"unless\",[[25,[\"integration\",\"iconImage\"]]],null,{\"statements\":[[0,\"                            \"],[1,[29,\"svg-jar\",[\"integration\"],[[\"class\"],[\"w14 h14\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-uploader\",null,[[\"extensions\",\"onComplete\"],[[25,[\"imageExtensions\"]],[29,\"action\",[[24,0,[]],\"setIconImage\"],null]]],{\"statements\":[[4,\"if\",[[24,2,[\"isUploading\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"div\"],[11,\"class\",\"absolute top-0 left-0 w-100 h-100 br4 bg-black-70 flex items-center\"],[9],[0,\"\\n                                \"],[1,[24,2,[\"progressBar\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                            \"],[7,\"button\"],[11,\"class\",\"child absolute top-0 left-0 w-100 h-100 br4 b white text-center bg-black-70 f8\"],[11,\"type\",\"button\"],[9],[0,\"\\n                                Upload\\n                            \"],[3,\"action\",[[24,0,[]],\"triggerIconFileDialog\"]],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                        \"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n                            \"],[1,[29,\"gh-file-input\",null,[[\"name\",\"multiple\",\"action\",\"accept\",\"data-test-file-input\"],[\"iconImage\",false,[24,2,[\"setFiles\"]],[25,[\"imageMimeTypes\"]],\"icon\"]]],false],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"flex-auto\"],[9],[0,\"\\n\"],[4,\"gh-validation-status-container\",null,[[\"class\",\"errors\",\"hasValidated\",\"property\"],[\"flex flex-column w-100 mr3\",[25,[\"integration\",\"errors\"]],[25,[\"integration\",\"hasValidated\"]],\"name\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"integration_name\"],[9],[0,\"Name\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"id\",\"class\",\"type\",\"value\",\"input\",\"focus-out\",\"data-test-input\"],[\"integration_name\",\"gh-input mt1 mb1\",\"text\",[29,\"readonly\",[[25,[\"integration\",\"name\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"integration\",\"name\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"name\"],[[\"target\"],[[25,[\"integration\"]]]]],\"name\"]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\",\"class\"],[[25,[\"integration\",\"errors\"]],\"name\",\"name\",\"ma0\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-validation-status-container\",null,[[\"class\",\"errors\",\"hasValidated\",\"property\"],[\"flex flex-column w-100 mr3\",[25,[\"integration\",\"errors\"]],[25,[\"integration\",\"hasValidated\"]],\"decription\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"integration_description\"],[11,\"class\",\"mt3\"],[9],[0,\"Description\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"id\",\"class\",\"type\",\"value\",\"input\",\"focus-out\",\"data-test-input\"],[\"integration_description\",\"gh-input mt1\",\"text\",[29,\"readonly\",[[25,[\"integration\",\"description\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"integration\",\"description\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"description\"],[[\"target\"],[[25,[\"integration\"]]]]],\"description\"]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\",\"class\"],[[25,[\"integration\",\"errors\"]],\"description\",\"description\",\"ma0\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[\"save\"],null]],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"m15 ba br3 b--lightgrey mt4\"],[9],[0,\"\\n        \"],[7,\"table\"],[11,\"class\",\"ma0\"],[11,\"style\",\"table-layout: fixed\"],[9],[0,\"\\n            \"],[7,\"tbody\"],[9],[0,\"\\n                \"],[7,\"tr\"],[11,\"class\",\"bb b--lightgrey\"],[9],[0,\"\\n                    \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"Content API Key\"],[10],[0,\"\\n                    \"],[7,\"td\"],[11,\"class\",\"pa0 truncate\"],[9],[0,\"\\n                        \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyContentKey\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                            \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"content-key\"],[9],[0,\"\\n                                \"],[1,[24,0,[\"integration\",\"contentKey\",\"secret\"]],false],[0,\"\\n                            \"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-1\"],[9],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--5\"],[9],[0,\"\\n                                    \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyContentKey\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                            \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                            Copy\\n\"]],\"parameters\":[]}],[0,\"                                    \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyContentKey\"]]],null]]],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"tr\"],[11,\"class\",\"bb b--lightgrey\"],[9],[0,\"\\n                    \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"Admin API Key\"],[10],[0,\"\\n                    \"],[7,\"td\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n                        \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyAdminKey\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                            \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"admin-key\"],[9],[0,\"\\n                                \"],[1,[24,0,[\"integration\",\"adminKey\",\"secret\"]],false],[0,\"\\n                            \"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-2\"],[9],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--1\"],[9],[0,\"\\n                                    \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyAdminKey\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                            \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                            Copy\\n\"]],\"parameters\":[]}],[0,\"                                    \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyAdminKey\"]]],null]]],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"tr\"],[9],[0,\"\\n                    \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"API URL\"],[10],[0,\"\\n                    \"],[7,\"td\"],[11,\"class\",\"pa0 truncate\"],[9],[0,\"\\n                        \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyApiUrl\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                            \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"api-url\"],[9],[0,\"\\n                                \"],[1,[24,0,[\"apiUrl\"]],false],[0,\"\\n                            \"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-2\"],[9],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--1\"],[9],[0,\"\\n                                    \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyApiUrl\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                            \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                            Copy\\n\"]],\"parameters\":[]}],[0,\"                                    \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyApiUrl\"]]],null]]],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"h4\"],[11,\"class\",\"mt15 midgrey f7 fw4\"],[9],[0,\"Webhooks\"],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"ba br3 b--lightgrey mt4\"],[9],[0,\"\\n        \"],[7,\"table\"],[11,\"class\",\"ma0 w-100\"],[9],[0,\"\\n            \"],[7,\"thead\"],[9],[0,\"\\n                \"],[7,\"tr\"],[9],[0,\"\\n                    \"],[7,\"th\"],[11,\"class\",\"pa2 pl3 midlightgrey fw4 f8 bg-whitegrey-l2 br3 br--top br--left fw6\"],[9],[0,\"Name\"],[10],[0,\"\\n                    \"],[7,\"th\"],[11,\"class\",\"pa2 pl3 midlightgrey fw4 f8 bg-whitegrey-l2 br3 fw6\"],[9],[0,\"Event\"],[10],[0,\"\\n                    \"],[7,\"th\"],[11,\"class\",\"pa2 pl3 midlightgrey fw4 f8 bg-whitegrey-l2 br3 fw6\"],[9],[0,\"URL\"],[10],[0,\"\\n                    \"],[7,\"th\"],[11,\"class\",\"pa2 pl3 midlightgrey fw4 f8 bg-whitegrey-l2 br3 fw6\"],[9],[0,\"Last triggered\"],[10],[0,\"\\n                    \"],[7,\"th\"],[11,\"class\",\"pa2 pl3 bg-whitegrey-l2 br3 br--top br--right\"],[9],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"tbody\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"filteredWebhooks\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"tr\"],[11,\"class\",\"hide-child bt b--whitegrey f7\"],[12,\"data-test-webhook-row\",[30,[[24,1,[\"id\"]]]]],[9],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa2 pl3\"],[11,\"data-test-text\",\"name\"],[9],[1,[24,1,[\"name\"]],false],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa2 pl3\"],[11,\"data-test-text\",\"event\"],[9],[1,[29,\"event-name\",[[24,1,[\"event\"]]],null],false],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa2 pl3\"],[11,\"data-test-text\",\"targetUrl\"],[9],[1,[24,1,[\"targetUrl\"]],false],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa2 pl3\"],[11,\"data-test-text\",\"last-triggered\"],[9],[1,[29,\"or\",[[24,1,[\"lastTriggeredAtUTC\"]],\"Not triggered\"],null],false],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"w1 pa2 pl3 nowrap\"],[9],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"child flex items-center\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-link\",\"route\",\"models\"],[\"edit-webhook\",\"settings.integration.webhooks.edit\",[29,\"array\",[[25,[\"integration\"]],[24,1,[]]],null]]],{\"statements\":[[0,\"                                    \"],[1,[29,\"svg-jar\",[\"pen\"],[[\"class\"],[\"w6 h6 fill-midgrey pa1 mr1\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                \"],[7,\"button\"],[11,\"data-test-button\",\"delete-webhook\"],[9],[0,\"\\n                                    \"],[1,[29,\"svg-jar\",[\"trash\"],[[\"class\"],[\"w6 fill-red pa1\"]]],false],[0,\"\\n                                \"],[3,\"action\",[[24,0,[]],\"confirmWebhookDeletion\",[24,1,[]]]],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[1]},{\"statements\":[[0,\"                \"],[7,\"tr\"],[11,\"class\",\"bt b--whitegrey\"],[11,\"data-test-webhooks-blank-slate\",\"\"],[9],[0,\"\\n                    \"],[7,\"td\"],[11,\"colspan\",\"5\"],[11,\"class\",\"pa5 pt15 pb15 tc midgrey f7\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"flex flex-column items-center\"],[9],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"ma0 pa0 tc midgrey lh-title mt2\"],[9],[0,\"\\n                                No webhooks configured\\n                            \"],[10],[0,\"\\n\"],[4,\"link-to\",null,[[\"classNames\",\"data-test-link\",\"route\",\"model\"],[\"flex items-center\",\"add-webhook\",\"settings.integration.webhooks.new\",[25,[\"integration\"]]]],{\"statements\":[[0,\"                            \"],[7,\"div\"],[11,\"class\",\"flex items-center pa2 pt1\"],[9],[0,\"\\n                                \"],[1,[29,\"svg-jar\",[\"add\"],[[\"class\"],[\"w3 h3 fill-blue-d1\"]]],false],[0,\"\\n                                \"],[7,\"span\"],[11,\"class\",\"ml1 blue\"],[9],[0,\"Add webhook\"],[10],[0,\"\\n                            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"            \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"filteredWebhooks\"]]],null,{\"statements\":[[0,\"            \"],[7,\"tfoot\"],[11,\"class\",\"bt b--lightgrey\"],[9],[0,\"\\n                \"],[7,\"tr\"],[11,\"class\",\"new-webhook-cell\"],[9],[0,\"\\n                    \"],[7,\"td\"],[11,\"colspan\",\"5\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"classNames\",\"data-test-link\",\"route\",\"model\"],[\"flex items-center\",\"add-webhook\",\"settings.integration.webhooks.new\",[25,[\"integration\"]]]],{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"pa3 f7\"],[9],[0,\"\\n                            \"],[1,[29,\"svg-jar\",[\"add\"],[[\"class\"],[\"w3 h3 fill-blue-d1\"]]],false],[0,\"\\n                            \"],[7,\"span\"],[11,\"class\",\"ml1 blue\"],[9],[0,\"Add webhook\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n    \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-red gh-btn-icon mb15 mt15\"],[9],[0,\"\\n        \"],[7,\"span\"],[9],[0,\" Delete Integration \"],[10],[0,\"\\n    \"],[3,\"action\",[[24,0,[]],\"confirmIntegrationDeletion\"]],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showUnsavedChangesModal\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveScreen\"],null],[29,\"action\",[[24,0,[]],\"toggleUnsavedChangesModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showDeleteIntegrationModal\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-integration\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"deleteIntegration\"],null],[29,\"action\",[[24,0,[]],\"cancelIntegrationDeletion\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"webhookToDelete\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-webhook\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"deleteWebhook\"],null],[29,\"action\",[[24,0,[]],\"cancelWebhookDeletion\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[1,[23,\"outlet\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integration.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integration/webhooks/edit", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "LWwkFSpA",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-fullscreen-modal\",[\"webhook-form\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"webhook\"]],[29,\"action\",[[24,0,[]],\"save\"],null],[29,\"action\",[[24,0,[]],\"cancel\"],null],\"action wide\"]]],false]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integration/webhooks/edit.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integration/webhooks/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "sLyEnS/E",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-fullscreen-modal\",[\"webhook-form\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"webhook\"]],[29,\"action\",[[24,0,[]],\"save\"],null],[29,\"action\",[[24,0,[]],\"cancel\"],null],\"action wide\"]]],false]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integration/webhooks/new.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "qwkBrBq3",
    "block": "{\"symbols\":[\"integration\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Integrations\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"integrations-directory\"],[9],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/disqus/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-disqus\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/disqus.svg\"],[11,\"alt\",\"Disqus Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"Disqus\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/google/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-analytics\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;padding:1px 0;\"],[11,\"src\",\"assets/img/google-analytics.png\"],[11,\"alt\",\"Google Analytics Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"Analytics\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/mailchimp/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-mailchimp\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/mailchimp.svg\"],[11,\"alt\",\"Mailchimp Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"Mailchimp\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/typeform/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-typeform\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/typeform.svg\"],[11,\"alt\",\"Typeform Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"Typeform\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/youtube/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-youtube\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;\"],[11,\"src\",\"assets/img/youtube.png\"],[11,\"alt\",\"YouTube Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"YouTube\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/buffer/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-buffer\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;padding:1px 0;\"],[11,\"src\",\"assets/img/buffer.png\"],[11,\"alt\",\"Buffer Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"Buffer\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/revue/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-revue\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;padding:1px 0;\"],[11,\"src\",\"assets/img/revue.svg\"],[11,\"alt\",\"Revue Icon\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"Revue\"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"a\"],[11,\"class\",\"id-item\"],[11,\"href\",\"https://docs.ghost.org/integrations/\"],[11,\"target\",\"_blank\"],[11,\"rel\",\"noopener noreferrer\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"id-item-logo id-more\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"w-100 h-100\"],[11,\"style\",\"object-fit:contain;padding:0 8px;\"],[11,\"src\",\"assets/img/more.png\"],[11,\"alt\",\"Three dots\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"f8 mt1\"],[9],[0,\"More\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"apps-grid-container\"],[9],[0,\"\\n\\n\"],[0,\"        \"],[7,\"span\"],[11,\"class\",\"apps-grid-title pb2\"],[9],[0,\"Built-in integrations\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-grid\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[11,\"data-test-app\",\"zapier\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"zapier\",\"settings.integrations.zapier\"]],{\"statements\":[[0,\"                \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                        \"],[7,\"figure\"],[11,\"class\",\"apps-card-app-icon\"],[11,\"style\",\"background-image:url(assets/img/zapiericon.png);background-size:45px;\"],[9],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                            \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[0,\"Zapier\"],[10],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[0,\"Automation for your favourite apps\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-card-right\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n                            \"],[7,\"span\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Configure\"],[10],[0,\"\\n                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n\\n            \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[11,\"data-test-app\",\"slack\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"slack\",\"settings.integrations.slack\"]],{\"statements\":[[0,\"                \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                        \"],[7,\"figure\"],[11,\"class\",\"apps-card-app-icon\"],[11,\"style\",\"background-image:url(assets/img/slackicon.png)\"],[9],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                            \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[0,\"Slack\"],[10],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[0,\"A messaging app for teams\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-card-right\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"settings\",\"slack\",\"isActive\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"span\"],[11,\"class\",\"green\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Active\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                \"],[7,\"span\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Configure\"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n\\n            \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[11,\"data-test-app\",\"amp\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"amp\",\"settings.integrations.amp\"]],{\"statements\":[[0,\"                \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                        \"],[7,\"figure\"],[11,\"class\",\"apps-card-app-icon\"],[11,\"style\",\"background-image:url(assets/img/ampicon.png)\"],[9],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                            \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[0,\"AMP\"],[10],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[0,\"Google Accelerated Mobile Pages\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-card-right\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"settings\",\"amp\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"span\"],[11,\"class\",\"green\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Active\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                \"],[7,\"span\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Configure\"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n\\n            \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[11,\"data-test-app\",\"unsplash\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"unsplash\",\"settings.integrations.unsplash\"]],{\"statements\":[[0,\"                \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                        \"],[7,\"figure\"],[11,\"class\",\"apps-card-app-icon\"],[11,\"style\",\"background-image:url(assets/icons/unsplash.svg);background-size:38px;\"],[9],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                            \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[0,\"Unsplash\"],[10],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[0,\"Beautiful, free photos\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-card-right\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"settings\",\"unsplash\",\"isActive\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"span\"],[11,\"class\",\"green\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Active\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                \"],[7,\"span\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Configure\"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n\\n            \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[11,\"data-test-app\",\"search\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-link\",\"route\"],[\"search\",\"settings.integrations.search\"]],{\"statements\":[[0,\"                \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                        \"],[7,\"figure\"],[11,\"class\",\"apps-card-app-icon\"],[11,\"style\",\"background-image:url(assets/img/magnifier.png);background-size:45px;\"],[9],[10],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                            \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[9],[0,\"Search\"],[10],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[9],[0,\"This integration will help the Theme to Search Posts\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-card-right\"],[9],[0,\"\\n                        \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n                            \"],[7,\"span\"],[11,\"data-test-app-status\",\"\"],[9],[0,\"Configure\"],[10],[0,\"\\n                            \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n\\n            \\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"apps-grid-container pt6\"],[9],[0,\"\\n\\n        \"],[7,\"span\"],[11,\"class\",\"apps-grid-title pb2\"],[9],[0,\"Custom integrations\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-grid\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"integrations\"]]],null,{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell\"],[11,\"data-test-custom-integration\",\"\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"data-test-integration\",\"route\",\"model\"],[[24,1,[\"id\"]],\"settings.integration\",[24,1,[]]]],{\"statements\":[[0,\"                        \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"apps-card-left\"],[9],[0,\"\\n                                \"],[7,\"figure\"],[11,\"class\",\"apps-card-app-icon flex items-center\"],[12,\"style\",[29,\"integration-icon-style\",[[24,1,[]]],null]],[9],[0,\"\\n\"],[4,\"unless\",[[24,1,[\"iconImage\"]]],null,{\"statements\":[[0,\"                                        \"],[1,[29,\"svg-jar\",[\"integration\"],[[\"class\"],[\"w-100 stroke-darkgrey\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                \"],[10],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"apps-card-meta\"],[9],[0,\"\\n                                    \"],[7,\"h3\"],[11,\"class\",\"apps-card-app-title\"],[11,\"data-test-text\",\"name\"],[9],[0,\"\\n                                        \"],[1,[24,1,[\"name\"]],false],[0,\"\\n                                    \"],[10],[0,\"\\n                                    \"],[7,\"p\"],[11,\"class\",\"apps-card-app-desc\"],[11,\"data-test-text\",\"description\"],[9],[0,\"\\n                                        \"],[1,[24,1,[\"description\"]],false],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"gh-card-right\"],[9],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"apps-configured\"],[9],[0,\"\\n                                    \"],[7,\"span\"],[9],[0,\"Configure\"],[10],[0,\"\\n                                    \"],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[1]},{\"statements\":[[0,\"                \"],[7,\"div\"],[11,\"class\",\"flex flex-column justify-center items-center mih30 miw-100\"],[11,\"data-test-blank\",\"custom-integrations\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"fetchIntegrations\",\"isRunning\"]]],null,{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"gh-loading-spinner\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"div\"],[11,\"class\",\"flex flex-column items-center pa5 pt15 pb15\"],[9],[0,\"\\n                            \"],[7,\"p\"],[11,\"class\",\"ma0 pa0 tc midgrey lh-title mt2\"],[9],[0,\"\\n                                Create your own custom Ghost integrations with dedicated API keys & webhooks\\n                            \"],[10],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"data-test-button\",\"route\"],[\"\",\"new-integration\",\"settings.integrations.new\"]],{\"statements\":[[0,\"                                \"],[7,\"div\"],[11,\"class\",\"flex items-center pa2 pt1\"],[9],[0,\"\\n                                    \"],[1,[29,\"svg-jar\",[\"add\"],[[\"class\"],[\"w3 h3 fill-blue-d1\"]]],false],[0,\"\\n                                    \"],[7,\"span\"],[11,\"class\",\"db ml1 blue nudge-bottom--1\"],[9],[0,\"Add custom integration\"],[10],[0,\"\\n                                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                        \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"\\n\"],[4,\"if\",[[25,[\"integrations\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"apps-grid-cell new-integration-cell\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"data-test-button\",\"route\"],[\"\",\"new-integration\",\"settings.integrations.new\"]],{\"statements\":[[0,\"                \"],[7,\"article\"],[11,\"class\",\"apps-card-app\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"flex items-center\"],[9],[0,\"\\n                        \"],[1,[29,\"svg-jar\",[\"add\"],[[\"class\"],[\"w3 h3 fill-blue-d1\"]]],false],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"db ml1 blue nudge-bottom--1 fw4\"],[9],[0,\"Add custom integration\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[1,[23,\"outlet\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/amp-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "PJvrxyox",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            AMP\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/amp-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/amp", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "DQqRYQ5o",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            AMP\\n        \"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"br\"],[9],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"app-grid\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"app-icon\"],[11,\"src\",\"assets/img/ampicon.png\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"h3\"],[9],[0,\"AMP\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Accelerated Mobile Pages\"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"AMP configuration\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Enable AMP\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Enable \"],[7,\"a\"],[11,\"href\",\"https://ampproject.org\"],[11,\"target\",\"_blank\"],[9],[0,\"Google Accelerated Mobile Pages\"],[10],[0,\" for your posts\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                    \"],[7,\"label\"],[11,\"for\",\"amp\"],[11,\"class\",\"checkbox\"],[9],[0,\"\\n                        \"],[7,\"input\"],[12,\"checked\",[23,\"ampSettings\"]],[11,\"id\",\"amp\"],[11,\"name\",\"amp\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"update\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-amp-checkbox\",\"\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/amp.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "MKQ/NLWf",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-fullscreen-modal\",[\"new-integration\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"integration\"]],[29,\"action\",[[24,0,[]],\"save\"],null],[29,\"action\",[[24,0,[]],\"cancel\"],null],\"action wide\"]]],false]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/new.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/search", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "egXxDaGM",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            Search\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"br\"],[9],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"app-grid\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"app-icon\"],[11,\"src\",\"assets/img/magnifier.png\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"h3\"],[9],[0,\"Search\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"This integration will help the Theme to Search Posts\"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"m15 ba br3 b--lightgrey mt4\"],[9],[0,\"\\n            \"],[7,\"table\"],[11,\"class\",\"ma0\"],[11,\"style\",\"table-layout: fixed\"],[9],[0,\"\\n                \"],[7,\"tbody\"],[9],[0,\"\\n                    \"],[7,\"tr\"],[11,\"class\",\"bb b--lightgrey\"],[9],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"Content API Key\"],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n                            \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyContentKey\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                                \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"admin-key\"],[9],[0,\"\\n                                    \"],[1,[24,0,[\"integration\",\"contentKey\",\"secret\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-2\"],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--1\"],[9],[0,\"\\n                                        \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyContentKey\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                                \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                                Copy\\n\"]],\"parameters\":[]}],[0,\"                                        \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyContentKey\"]]],null]]],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"tr\"],[9],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"API URL\"],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa0 truncate\"],[9],[0,\"\\n                            \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyApiUrl\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                                \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"api-url\"],[9],[0,\"\\n                                    \"],[1,[24,0,[\"apiUrl\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-2\"],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--1\"],[9],[0,\"\\n                                        \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyApiUrl\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                                \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                                Copy\\n\"]],\"parameters\":[]}],[0,\"                                        \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyApiUrl\"]]],null]]],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/search.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/slack-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "9nUbF896",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            Slack\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/slack-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/slack", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "g6ZApxGI",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            Slack\\n        \"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"br\"],[9],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"app-grid\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"app-icon\"],[11,\"src\",\"assets/img/slackicon.png\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"h3\"],[9],[0,\"Slack\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"A messaging app for teams\"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"form\"],[11,\"class\",\"app-config-form\"],[11,\"id\",\"slack-settings\"],[11,\"novalidate\",\"novalidate\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Slack configuration\"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content gh-setting-content--no-action\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Webhook URL\"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Automatically send newly published posts to a channel in Slack or any Slack-compatible service like Discord or Mattermost.\"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"slackSettings\",\"errors\"]],[25,[\"slackSettings\",\"hasValidated\"]],\"url\"]],{\"statements\":[[0,\"                            \"],[1,[29,\"gh-text-input\",null,[[\"placeholder\",\"name\",\"value\",\"input\",\"keyEvents\",\"focus-out\",\"data-test-slack-url-input\"],[\"https://hooks.slack.com/services/...\",\"slack[url]\",[29,\"readonly\",[[25,[\"slackSettings\",\"url\"]]],null],[29,\"action\",[[24,0,[]],\"updateURL\"],[[\"value\"],[\"target.value\"]]],[29,\"hash\",null,[[\"Enter\"],[[29,\"action\",[[24,0,[]],\"save\"],null]]]],[29,\"action\",[[24,0,[]],\"triggerDirtyState\"],null],true]]],false],[0,\"\\n\"],[4,\"unless\",[[25,[\"slackSettings\",\"errors\",\"url\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"p\"],[9],[0,\"Set up a new incoming webhook \"],[7,\"a\"],[11,\"href\",\"https://my.slack.com/apps/new/A0F7XDUAZ-incoming-webhooks\"],[11,\"target\",\"_blank\"],[9],[0,\"here\"],[10],[0,\", and grab the URL.\"],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"slackSettings\",\"errors\"]],\"url\",\"slack-url\"]]],false],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content gh-setting-content--no-action\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Username\"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"The username to display messages from\"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"slackSettings\",\"errors\"]],[25,[\"slackSettings\",\"hasValidated\"]],\"username\"]],{\"statements\":[[0,\"                            \"],[1,[29,\"gh-text-input\",null,[[\"placeholder\",\"name\",\"value\",\"input\",\"keyEvents\",\"focus-out\",\"data-test-slack-username-input\"],[\"Ghost\",\"slack[username]\",[29,\"readonly\",[[25,[\"slackSettings\",\"username\"]]],null],[29,\"action\",[[24,0,[]],\"updateUsername\"],[[\"value\"],[\"target.value\"]]],[29,\"hash\",null,[[\"Enter\"],[[29,\"action\",[[24,0,[]],\"save\"],null]]]],[29,\"action\",[[24,0,[]],\"triggerDirtyState\"],null],true]]],false],[0,\"\\n\"],[4,\"if\",[[25,[\"slackSettings\",\"errors\",\"username\"]]],null,{\"statements\":[[0,\"                                \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\"],[[25,[\"slackSettings\",\"errors\"]],\"username\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[3,\"action\",[[24,0,[]],\"save\"],[[\"on\"],[\"submit\"]]],[10],[0,\"\\n\\n        \"],[1,[29,\"gh-task-button\",[\"Send test notification\"],[[\"task\",\"successText\",\"class\",\"disabled\",\"data-test-send-notification-button\"],[[25,[\"sendTestNotification\"]],\"Sent\",\"gh-btn gh-btn-green gh-btn-icon\",[25,[\"testNotificationDisabled\"]],true]]],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/slack.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/unsplash-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "L9VR5wG+",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            Unsplash\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/unsplash-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/unsplash", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "7YuxXED1",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            Unsplash\\n        \"],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"task\",\"class\",\"data-test-save-button\"],[[25,[\"save\"]],\"gh-btn gh-btn-blue gh-btn-icon\",true]]],false],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"br\"],[9],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"app-grid\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"app-icon\"],[11,\"src\",\"assets/icons/unsplash.svg\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"h3\"],[9],[0,\"Unsplash\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Beautiful, free photos\"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Unsplash configuration\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[11,\"id\",\"unsplash-toggle\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Enable Unsplash\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Enable \"],[7,\"a\"],[11,\"href\",\"https://unsplash.com\"],[11,\"target\",\"_blank\"],[9],[0,\"Unsplash\"],[10],[0,\" image integration for your posts\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"form-group right\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[0,\"\\n                        \"],[7,\"label\"],[11,\"for\",\"isActive\"],[11,\"class\",\"checkbox\"],[9],[0,\"\\n                                \"],[7,\"input\"],[12,\"checked\",[25,[\"unsplashSettings\",\"isActive\"]]],[11,\"id\",\"isActive\"],[11,\"name\",\"isActive\"],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"update\"],[[\"value\"],[\"target.checked\"]]]],[11,\"data-test-checkbox\",\"unsplash\"],[11,\"type\",\"checkbox\"],[9],[10],[0,\"\\n                                \"],[7,\"span\"],[11,\"class\",\"input-toggle-component\"],[9],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/unsplash.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/integrations/zapier", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "rU2LgftM",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"route\"],[\"settings.integrations\"]],{\"statements\":[[0,\"Integrations\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            Zapier\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"br\"],[9],[10],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"app-grid\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"class\",\"app-icon\"],[11,\"src\",\"assets/img/zapiericon.png\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"app-cell\"],[9],[0,\"\\n                \"],[7,\"h3\"],[9],[0,\"Zapier\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Automation for your favourite apps\"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"m15 ba br3 b--lightgrey mt4\"],[9],[0,\"\\n            \"],[7,\"table\"],[11,\"class\",\"ma0\"],[11,\"style\",\"table-layout: fixed\"],[9],[0,\"\\n                \"],[7,\"tbody\"],[9],[0,\"\\n                    \"],[7,\"tr\"],[11,\"class\",\"bb b--lightgrey\"],[9],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"Admin API Key\"],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa0\"],[9],[0,\"\\n                            \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyAdminKey\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                                \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"admin-key\"],[9],[0,\"\\n                                    \"],[1,[24,0,[\"integration\",\"adminKey\",\"secret\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-2\"],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--1\"],[9],[0,\"\\n                                        \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyAdminKey\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                                \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                                Copy\\n\"]],\"parameters\":[]}],[0,\"                                        \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyAdminKey\"]]],null]]],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"tr\"],[9],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa3 w50 fw7\"],[9],[0,\"API URL\"],[10],[0,\"\\n                        \"],[7,\"td\"],[11,\"class\",\"pa0 truncate\"],[9],[0,\"\\n                            \"],[7,\"div\"],[12,\"class\",[30,[\"pa3 relative truncate \",[29,\"unless\",[[25,[\"copyApiUrl\",\"isRunning\"]],\"hide-child-instant\"],null]]]],[9],[0,\"\\n                                \"],[7,\"span\"],[11,\"class\",\"midgrey\"],[11,\"data-test-text\",\"api-url\"],[9],[0,\"\\n                                    \"],[1,[24,0,[\"apiUrl\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"absolute top-1 right-2\"],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"pt1 pr3 pb1 pl3 bg-black-70 child br3 f8 nudge-top--4 nudge-right--1\"],[9],[0,\"\\n                                        \"],[7,\"button\"],[11,\"class\",\"white fw4 flex items-center\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"copyApiUrl\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                                \"],[1,[29,\"svg-jar\",[\"check-circle\"],[[\"class\"],[\"w3 v-mid mr2\"]]],false],[0,\" Copied\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                                Copy\\n\"]],\"parameters\":[]}],[0,\"                                        \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"copyApiUrl\"]]],null]]],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"mt12\"],[11,\"id\",\"zapier-toggle\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content gh-setting-content--no-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Zapier Templates\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc mt1\"],[9],[0,\"Explore pre-built templates for common automation tasks\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content-extended\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"id\",\"zapier-templates\"],[9],[0,\"\\n\"],[4,\"unless\",[[25,[\"isTesting\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"script\"],[11,\"src\",\"https://zapier.com/apps/embed/widget.js?services=ghost&container=true&limit=10&html_id=zapier-templates\"],[9],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/integrations/zapier.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/labs-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "alilk4KH",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Labs\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/labs-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/labs", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "Jq9VzGlQ",
    "block": "{\"symbols\":[\"uploader\",\"error\",\"uploader\",\"error\",\"error\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Labs\"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"view-container settings-debug\"],[9],[0,\"\\n        \"],[7,\"p\"],[11,\"class\",\"gh-box gh-box-info\"],[9],[1,[29,\"svg-jar\",[\"idea\"],null],false],[0,\"This is a testing ground for experimental features which aren't quite ready for primetime. They may change, break or inexplicably disappear at any time.\"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Migration options\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Import content\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Import posts from another Ghost installation\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"form\"],[11,\"id\",\"settings-import\"],[11,\"enctype\",\"multipart/form-data\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-file-upload\",null,[[\"id\",\"classNames\",\"uploadButtonText\",\"onUpload\",\"acceptEncoding\",\"data-test-file-input\"],[\"importfile\",\"flex\",[25,[\"uploadButtonText\"]],[29,\"action\",[[24,0,[]],\"onUpload\"],null],[25,[\"importMimeType\"]],\"import\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"importErrors\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[12,\"class\",[30,[\"gh-import-errors \",[29,\"if\",[[25,[\"importSuccessful\"]],\"gh-import-errors-warning\"],null]]]],[11,\"data-test-import-errors\",\"\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-import-errors-title\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"importSuccessful\"]]],null,{\"statements\":[[0,\"                        Import successful with warnings\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        Import failed\\n\"]],\"parameters\":[]}],[0,\"                \"],[10],[0,\"\\n\\n\"],[4,\"each\",[[25,[\"importErrors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-import-error\"],[11,\"data-test-import-error\",\"\"],[9],[0,\"\\n                        \"],[7,\"p\"],[11,\"class\",\"gh-import-error-message\"],[11,\"data-test-import-error-message\",\"\"],[9],[0,\"\\n                            \"],[4,\"if\",[[24,5,[\"help\"]]],null,{\"statements\":[[1,[24,5,[\"help\"]],false],[0,\": \"]],\"parameters\":[]},null],[1,[24,5,[\"message\"]],false],[0,\"\\n                        \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[24,5,[\"context\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"div\"],[11,\"class\",\"gh-import-error-entry\"],[11,\"data-test-import-error-context\",\"\"],[9],[0,\"\\n                                \"],[7,\"pre\"],[9],[1,[24,5,[\"context\"]],false],[10],[0,\"                            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                    \"],[10],[0,\"\\n\"]],\"parameters\":[5]},null],[0,\"            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Export your content\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Download all of your posts and settings in a single, glorious JSON file\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-blue\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Export\"],[10],[3,\"action\",[[24,0,[]],\"downloadFile\",\"db\"]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Delete all content\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Permanently delete all posts and tags from the database, a hard reset\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-hover-red js-delete\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Delete\"],[10],[3,\"action\",[[24,0,[]],\"toggleDeleteAllModal\"]],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Testing tools\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Test email configuration\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Send yourself a test email to make sure everything is working\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[1,[29,\"gh-task-button\",[\"Send\"],[[\"successText\",\"task\",\"class\"],[\"Sent\",[25,[\"sendTestEmail\"]],\"gh-btn gh-btn-hover-blue gh-btn-icon\"]]],false],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting-header\"],[9],[0,\"Beta features\"],[10],[0,\"\\n\"],[4,\"if\",[[29,\"not-eq\",[[25,[\"feature\",\"labs\",\"publicAPI\"]],[28]],null]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Public API (deprecated)\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"⚠️ Please use the Content API instead, more info in \"],[7,\"a\"],[11,\"href\",\"https://docs.ghost.org/api/content/\"],[9],[0,\"the docs\"],[10],[10],[0,\"\\n                \"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[1,[29,\"gh-feature-flag\",[\"publicAPI\"],null],false],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Night shift\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Swap Ghost admin's colours to a dark background which is easy on the eyes\"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[1,[29,\"gh-feature-flag\",[\"nightShift\"],null],false],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[12,\"class\",[30,[\"gh-setting \",[29,\"if\",[[25,[\"feature\",\"members\"]],\"gh-labs-disabled\"],null]]]],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Subscribers\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Collect email addresses from your readers, more info in \"],[7,\"a\"],[11,\"href\",\"https://docs.ghost.org/faq/enable-subscribers-feature/\"],[9],[0,\"the docs\"],[10],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"feature\",\"members\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[1,[29,\"gh-feature-flag\",[\"subscribers\",\"disabled\"],null],false],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[1,[29,\"gh-feature-flag\",[\"subscribers\"],null],false],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"config\",\"enableDeveloperExperiments\"]]],null,{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Members\"],[10],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Enable free or paid member registration. Restrict content by using \"],[7,\"span\"],[11,\"class\",\"dib blue ba br2 b--blue pa1 pt0 pb0 tag-token--internal\"],[9],[0,\"#members\"],[10],[0,\" hashtag on posts\"],[10],[0,\"\\n\\n\"],[4,\"liquid-if\",[[25,[\"feature\",\"labs\",\"members\"]]],[[\"class\"],[\"nr20\"]],{\"statements\":[[0,\"\\n                        \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-labs-toggle\"],[11,\"type\",\"button\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"showMemberConfig\"]]],null,{\"statements\":[[0,\"                                \"],[1,[29,\"svg-jar\",[\"arrow-down-small\"],null],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                \"],[1,[29,\"svg-jar\",[\"arrow-right-small\"],null],false],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                            Configure\\n                        \"],[3,\"action\",[[24,0,[]],\"toggleMemberConfig\",\"\"]],[10],[0,\"\\n\\n\"],[4,\"liquid-if\",[[25,[\"showMemberConfig\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"div\"],[11,\"class\",\"flex nl2 nr2 mt5\"],[9],[0,\"\\n                                \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"subscriptionSettings\",\"isPaid\"]],false],null],\"active\"],null],\" flex-auto w-50 ba br4 b--whitegrey pa5 gh-labs-members-radio\"]]],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[9],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label f3\"],[9],[7,\"span\"],[11,\"class\",\"fw6 f7 darkgrey\"],[9],[0,\"Free\"],[10],[10],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Access to members-only posts require free user registration\"],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[3,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"isPaid\",false],[[\"on\"],[\"click\"]]],[10],[0,\"\\n                                \"],[7,\"div\"],[12,\"class\",[30,[\"gh-publishmenu-radio \",[29,\"if\",[[29,\"eq\",[[25,[\"subscriptionSettings\",\"isPaid\"]],true],null],\"active\"],null],\" flex-auto w-50 ba br4 b--whitegrey pa5 ml2 gh-labs-members-radio\"]]],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-button\"],[9],[3,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"isPaid\",true],[[\"on\"],[\"click\"]]],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-content\"],[9],[0,\"\\n                                        \"],[7,\"div\"],[9],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-label\"],[9],[7,\"span\"],[11,\"class\",\"fw6 f7 darkgrey\"],[9],[0,\"Paid\"],[10],[10],[0,\"\\n                                            \"],[7,\"div\"],[11,\"class\",\"gh-publishmenu-radio-desc\"],[9],[0,\"Set up paid subscriptions using Stripe\"],[10],[0,\"\\n                                        \"],[3,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"isPaid\",true],[[\"on\"],[\"click\"]]],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n\\n\"],[4,\"liquid-if\",[[29,\"eq\",[[25,[\"subscriptionSettings\",\"isPaid\"]],true],null]],null,{\"statements\":[[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"ba br4 b--whitegrey pa5 mt5\"],[9],[0,\"\\n                                \"],[7,\"div\"],[11,\"class\",\"flex flex-column\"],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"flex\"],[9],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"w-50 mr2\"],[9],[0,\"\\n                                            \"],[7,\"label\"],[11,\"class\",\"fw6\"],[9],[0,\"Stripe publishable API key\"],[10],[0,\"\\n                                            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"class\",\"placeholder\"],[[29,\"readonly\",[[25,[\"subscriptionSettings\",\"stripeConfig\",\"public_token\"]]],null],[29,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"public_token\"],null],\"mt1\",\"pk_...\"]]],false],[0,\"\\n                                        \"],[10],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"w-50 ml2\"],[9],[0,\"\\n                                            \"],[7,\"label\"],[11,\"class\",\"fw6\"],[9],[0,\"Stripe secret API key\"],[10],[0,\"\\n                                            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"input\",\"class\",\"placeholder\"],[[29,\"readonly\",[[25,[\"subscriptionSettings\",\"stripeConfig\",\"secret_token\"]]],null],[29,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"secret_token\"],null],\"mt1\",\"sk_...\"]]],false],[0,\"\\n                                        \"],[10],[0,\"\\n                                    \"],[10],[0,\"\\n                                    \"],[7,\"a\"],[11,\"href\",\"https://dashboard.stripe.com/account/apikeys\"],[11,\"target\",\"_blank\"],[11,\"class\",\"mt1 self-end fw3\"],[9],[0,\"Where to find Stripe API keys\"],[10],[0,\"\\n                                \"],[10],[0,\"\\n\\n                                \"],[7,\"div\"],[11,\"class\",\"mt4 flex nb5\"],[9],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"w-50 mr2\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,null,{\"statements\":[[0,\"                                        \"],[7,\"label\"],[11,\"class\",\"fw6\"],[9],[0,\"Monthly price\"],[10],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"mt1 relative gh-labs-price-label gh-labs-monthly-price\"],[9],[0,\"\\n                                            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"type\",\"input\"],[[29,\"readonly\",[[25,[\"subscriptionSettings\",\"stripeConfig\",\"plans\",\"monthly\",\"dollarAmount\"]]],null],\"number\",[29,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"month\"],null]]]],false],[0,\"\\n                                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                    \"],[10],[0,\"\\n                                    \"],[7,\"div\"],[11,\"class\",\"w-50 ml2\"],[9],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"class\"],[\"description-container\"]],{\"statements\":[[0,\"                                        \"],[7,\"label\"],[11,\"class\",\"fw6\"],[9],[0,\"Yearly price\"],[10],[0,\"\\n                                        \"],[7,\"div\"],[11,\"class\",\"mt1 relative gh-labs-price-label gh-labs-yearly-price\"],[9],[0,\"\\n                                            \"],[1,[29,\"gh-text-input\",null,[[\"value\",\"type\",\"input\"],[[29,\"readonly\",[[25,[\"subscriptionSettings\",\"stripeConfig\",\"plans\",\"yearly\",\"dollarAmount\"]]],null],\"number\",[29,\"action\",[[24,0,[]],\"setSubscriptionSettings\",\"year\"],null]]]],false],[0,\"\\n                                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                                    \"],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                            \"],[7,\"div\"],[11,\"class\",\"mb5 mt5\"],[9],[0,\"\\n                                \"],[1,[29,\"gh-task-button\",[\"Save\"],[[\"task\",\"successText\",\"runningText\",\"class\"],[[25,[\"saveSettings\"]],\"Saved\",\"Saving\",\"gh-btn gh-btn-blue gh-btn-icon\"]]],false],[0,\"\\n                            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[9],[0,\"\\n                    \"],[7,\"div\"],[11,\"class\",\"for-checkbox\"],[9],[1,[29,\"gh-feature-flag\",[\"members\"],null],false],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n\"],[4,\"gh-uploader\",null,[[\"extensions\",\"uploadUrl\",\"paramName\",\"onUploadSuccess\",\"onUploadFailure\"],[[25,[\"jsonExtension\"]],\"/redirects/json/\",\"redirects\",[29,\"perform\",[[25,[\"redirectUploadResult\"]],true],null],[29,\"perform\",[[25,[\"redirectUploadResult\"]],false],null]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Redirects\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Configure redirects for old or moved content, more info in \"],[7,\"a\"],[11,\"href\",\"https://docs.ghost.org/tutorials/implementing-redirects/\"],[9],[0,\"the docs\"],[10],[10],[0,\"\\n\"],[4,\"each\",[[24,3,[\"errors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-error\"],[11,\"data-test-error\",\"redirects\"],[9],[1,[29,\"or\",[[24,4,[\"context\"]],[24,4,[\"message\"]]],null],false],[10],[0,\"\\n\"]],\"parameters\":[4]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[11,\"style\",\"display: flex; flex-direction: column\"],[9],[0,\"\\n\"],[4,\"if\",[[24,3,[\"isUploading\"]]],null,{\"statements\":[[0,\"                    \"],[1,[24,3,[\"progressBar\"]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[7,\"button\"],[12,\"class\",[30,[\"gh-btn gh-btn-icon \",[29,\"if\",[[25,[\"redirectSuccess\"]],\"gh-btn-green\"],null],\" \",[29,\"if\",[[25,[\"redirectFailure\"]],\"gh-btn-red\"],null]]]],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"data-test-button\",\"upload-redirects\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"redirectSuccess\"]]],null,{\"statements\":[[0,\"                                \"],[1,[29,\"svg-jar\",[\"check-circle\"],null],false],[0,\" Uploaded\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"redirectFailure\"]]],null,{\"statements\":[[0,\"                                \"],[1,[29,\"svg-jar\",[\"retry\"],null],false],[0,\" Upload Failed\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                Upload redirects JSON\\n                            \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"span\"],[9],[7,\"a\"],[11,\"href\",\"#\"],[11,\"data-test-link\",\"download-redirects\"],[9],[0,\"Download current redirects\"],[3,\"action\",[[24,0,[]],\"downloadFile\",\"redirects/json\"]],[10],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"\\n                \"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-file-input\",null,[[\"multiple\",\"action\",\"accept\",\"data-test-file-input\"],[false,[24,3,[\"setFiles\"]],[25,[\"jsonMimeType\"]],\"redirects\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[3]},null],[0,\"        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-setting\"],[9],[0,\"\\n\"],[4,\"gh-uploader\",null,[[\"extensions\",\"uploadUrl\",\"paramName\",\"onUploadSuccess\",\"onUploadFailure\"],[[25,[\"yamlExtension\"]],\"/settings/routes/yaml/\",\"routes\",[29,\"perform\",[[25,[\"routesUploadResult\"]],true],null],[29,\"perform\",[[25,[\"routesUploadResult\"]],false],null]]],{\"statements\":[[0,\"            \"],[7,\"div\"],[11,\"class\",\"gh-setting-content\"],[9],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-title\"],[9],[0,\"Routes\"],[10],[0,\"\\n                \"],[7,\"div\"],[11,\"class\",\"gh-setting-desc\"],[9],[0,\"Configure dynamic routing by modifying the routes.yaml file\"],[10],[0,\"\\n\"],[4,\"each\",[[24,1,[\"errors\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"gh-setting-error\"],[11,\"data-test-error\",\"routes\"],[9],[1,[29,\"or\",[[24,2,[\"context\"]],[24,2,[\"message\"]]],null],false],[10],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"            \"],[10],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-setting-action\"],[11,\"style\",\"display: flex; flex-direction: column\"],[9],[0,\"\\n\"],[4,\"if\",[[24,1,[\"isUploading\"]]],null,{\"statements\":[[0,\"                    \"],[1,[24,1,[\"progressBar\"]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    \"],[7,\"button\"],[12,\"class\",[30,[\"gh-btn gh-btn-icon \",[29,\"if\",[[25,[\"routesSuccess\"]],\"gh-btn-green\"],null],\" \",[29,\"if\",[[25,[\"routesFailure\"]],\"gh-btn-red\"],null]]]],[12,\"onclick\",[29,\"action\",[[24,0,[]],\"triggerFileDialog\"],null]],[11,\"data-test-button\",\"upload-routes\"],[11,\"type\",\"button\"],[9],[0,\"\\n                        \"],[7,\"span\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"routesSuccess\"]]],null,{\"statements\":[[0,\"                                \"],[1,[29,\"svg-jar\",[\"check-circle\"],null],false],[0,\" Uploaded\\n\"]],\"parameters\":[]},{\"statements\":[[4,\"if\",[[25,[\"routesFailure\"]]],null,{\"statements\":[[0,\"                                \"],[1,[29,\"svg-jar\",[\"retry\"],null],false],[0,\" Upload Failed\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                Upload routes YAML\\n                            \"]],\"parameters\":[]}]],\"parameters\":[]}],[0,\"                        \"],[10],[0,\"\\n                    \"],[10],[0,\"\\n                    \"],[7,\"span\"],[9],[7,\"a\"],[11,\"href\",\"#\"],[11,\"data-test-link\",\"download-routes\"],[9],[0,\"Download current routes.yaml\"],[3,\"action\",[[24,0,[]],\"downloadFile\",\"settings/routes/yaml\"]],[10],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"\\n                \"],[7,\"div\"],[11,\"style\",\"display:none\"],[9],[0,\"\\n                    \"],[1,[29,\"gh-file-input\",null,[[\"multiple\",\"action\",\"accept\",\"data-test-file-input\"],[false,[24,1,[\"setFiles\"]],[25,[\"yamlMimeType\"]],\"routes\"]]],false],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDeleteAllModal\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-all\"],[[\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"toggleDeleteAllModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/labs.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/tags-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "YxDmABPJ",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-view\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"view-header\"],[9],[0,\"\\n        \"],[4,\"gh-view-title\",null,null,{\"statements\":[[7,\"span\"],[9],[0,\"Tags\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"route\"],[\"gh-btn gh-btn-green\",\"settings.tags.new\"]],{\"statements\":[[7,\"span\"],[9],[0,\"New tag\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/tags-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/tags", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "7OnAbThd",
    "block": "{\"symbols\":[\"container\",\"tag\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-view\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"view-header\"],[9],[0,\"\\n        \"],[4,\"gh-view-title\",null,null,{\"statements\":[[7,\"span\"],[9],[0,\"Tags\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"route\"],[\"gh-btn gh-btn-green\",\"settings.tags.new\"]],{\"statements\":[[7,\"span\"],[9],[0,\"New tag\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"gh-tags-management-container\",null,[[\"tags\",\"selectedTag\",\"enteredMobile\",\"leftMobile\"],[[25,[\"tags\"]],[25,[\"selectedTag\"]],\"enteredMobile\",[29,\"action\",[[24,0,[]],\"leftMobile\"],null]]],{\"statements\":[[0,\"        \"],[7,\"div\"],[11,\"class\",\"tag-list\"],[9],[0,\"\\n            \"],[7,\"section\"],[12,\"class\",[30,[\"tag-list-content settings-tags \",[29,\"if\",[[25,[\"tagListFocused\"]],\"keyboard-focused\"],null]]]],[9],[0,\"\\n\"],[4,\"vertical-collection\",[[25,[\"sortedTags\"]]],[[\"estimateHeight\",\"minHeight\",\"bufferSize\",\"containerSelector\"],[16,67,5,\".tag-list\"]],{\"statements\":[[0,\"                    \"],[1,[29,\"gh-tag\",null,[[\"tag\"],[[24,2,[]]]]],false],[0,\"\\n\"]],\"parameters\":[2]},null],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"section\"],[12,\"class\",[30,[\"settings-menu-container tag-settings \",[29,\"if\",[[25,[\"tagContentFocused\"]],\"keyboard-focused\"],null],\" \",[29,\"if\",[[24,1,[\"displaySettingsPane\"]],\"tag-settings-in\"],null]]]],[9],[0,\"\\n            \"],[1,[23,\"outlet\"],false],[0,\"\\n        \"],[10],[0,\"\\n\"]],\"parameters\":[1]},null],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/tags.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/tags/index", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "XJda+xVN",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"no-posts-box\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"no-posts\"],[9],[0,\"\\n        \"],[7,\"h3\"],[9],[0,\"You haven't added any tags yet!\"],[10],[0,\"\\n        \"],[4,\"link-to\",null,[[\"route\"],[\"settings.tags.new\"]],{\"statements\":[[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-green btn-lg\"],[11,\"title\",\"New tag\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Add a tag\"],[10],[10]],\"parameters\":[]},null],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/tags/index.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/settings/tags/tag", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "TtBtgO6U",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-tag-settings-form\",null,[[\"tag\",\"setProperty\",\"showDeleteTagModal\"],[[25,[\"tag\"]],[29,\"action\",[[24,0,[]],\"setProperty\"],null],[29,\"action\",[[24,0,[]],\"toggleDeleteTagModal\"],null]]]],false],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDeleteTagModal\"]]],null,{\"statements\":[[0,\"    \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-tag\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"tag\"]],[29,\"action\",[[24,0,[]],\"deleteTag\"],null],[29,\"action\",[[24,0,[]],\"toggleDeleteTagModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/settings/tags/tag.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/setup", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "7FzPymfv",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-flow\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-flow-content-wrap\"],[9],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"gh-flow-content\"],[9],[0,\"\\n            \"],[7,\"header\"],[9],[0,\"\\n                \"],[7,\"h1\"],[9],[0,\"Welcome to \"],[7,\"strong\"],[9],[0,\"Ghost\"],[10],[0,\"!\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"All over the world, people have started \"],[7,\"em\"],[9],[1,[23,\"gh-download-count\"],false],[10],[0,\" incredible sites with Ghost. Today,\\n                    we’re starting yours.\"],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n            \"],[7,\"figure\"],[11,\"class\",\"gh-flow-screenshot\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"src\",\"assets/img/install-welcome.png\"],[11,\"alt\",\"Ghost screenshot\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n            \"],[7,\"p\"],[9],[0,\"Setup Ghost from Admin Panel\"],[10],[0,\"\\n\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/setup.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/signin", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "GBhpJ3Qe",
    "block": "{\"symbols\":[],\"statements\":[[7,\"div\"],[11,\"class\",\"gh-flow\"],[9],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-flow-content-wrap\"],[9],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"gh-flow-content\"],[9],[0,\"\\n            \"],[7,\"header\"],[9],[0,\"\\n                \"],[7,\"h1\"],[9],[0,\"Welcome to \"],[7,\"strong\"],[9],[0,\"Ghost\"],[10],[0,\"!\"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Seems you are not logged in RC, or you are not the part of Ghost.\"],[10],[0,\"\\n            \"],[10],[0,\"\\n\\n            \"],[7,\"figure\"],[11,\"class\",\"gh-flow-screenshot\"],[9],[0,\"\\n                \"],[7,\"img\"],[11,\"src\",\"assets/img/login_icon.png\"],[11,\"alt\",\"Ghost screenshot\"],[9],[10],[0,\"\\n            \"],[10],[0,\"\\n                \"],[7,\"p\"],[9],[0,\"Click on Article icon to enter Ghost.\"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/signin.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/site", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "vx7U2VZv",
    "block": "{\"symbols\":[],\"statements\":[[5,\"gh-site-iframe\",[],[[\"@guid\"],[[24,0,[\"guid\"]]]],{\"statements\":[],\"parameters\":[]}]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/site.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/staff/index", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "yP4Gyu/4",
    "block": "{\"symbols\":[\"user\",\"component\",\"user\",\"component\",\"role\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"Users\"],[10],[0,\"\\n\"],[4,\"unless\",[[25,[\"currentUser\",\"isAuthorOrContributor\"]]],null,{\"statements\":[[0,\"            \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n                \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-green\"],[9],[7,\"span\"],[9],[0,\"Invite people\"],[10],[3,\"action\",[[24,0,[]],\"toggleInviteUserModal\"]],[10],[0,\"\\n            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"    \"],[10],[0,\"\\n    \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"gh-contentfilter-left\"],[9],[0,\"\\n\"],[4,\"power-select\",null,[[\"selected\",\"options\",\"searchEnabled\",\"onchange\",\"tagName\",\"classNames\",\"triggerClass\",\"dropdownClass\",\"matchTriggerWidth\",\"data-test-type-select\"],[[25,[\"selectedRole\"]],[25,[\"availableRoles\"]],false,[29,\"action\",[[24,0,[]],\"changeRole\"],null],\"div\",\"gh-contentfilter-menu gh-contentfilter-type\",\"gh-contentfilter-menu-trigger\",\"gh-contentfilter-menu-dropdown\",false,true]],{\"statements\":[[0,\"                \"],[1,[24,5,[\"name\"]],false],[0,\"\\n\"]],\"parameters\":[5]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showInviteUserModal\"]]],null,{\"statements\":[[0,\"        \"],[1,[29,\"gh-fullscreen-modal\",[\"invite-new-user\"],[[\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"toggleInviteUserModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"gh-team\"],[9],[0,\"\\n    \"],[7,\"section\"],[11,\"class\",\"apps-grid-container gh-active-users\"],[11,\"data-test-active-users\",\"\"],[9],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"apps-grid-title\"],[9],[0,\"Users\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-grid\"],[9],[0,\"\\n\"],[4,\"vertical-collection\",[[25,[\"sortedActiveUsers\"]]],[[\"key\",\"containerSelector\",\"estimateHeight\"],[\"id\",\".gh-main\",75]],{\"statements\":[[4,\"gh-user-active\",null,[[\"user\"],[[24,3,[]]]],{\"statements\":[[0,\"                    \"],[1,[29,\"gh-user-list-item\",null,[[\"user\",\"component\"],[[24,3,[]],[24,4,[]]]]],false],[0,\"\\n\"]],\"parameters\":[4]},null]],\"parameters\":[3]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[29,\"and\",[[25,[\"suspendedUsers\"]],[29,\"not\",[[25,[\"currentUser\",\"isAuthorOrContributor\"]]],null]],null]],null,{\"statements\":[[0,\"    \"],[7,\"section\"],[11,\"class\",\"apps-grid-container gh-active-users\"],[11,\"data-test-suspended-users\",\"\"],[9],[0,\"\\n        \"],[7,\"span\"],[11,\"class\",\"apps-grid-title\"],[9],[0,\"Suspended users\"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"apps-grid\"],[9],[0,\"\\n\"],[4,\"each\",[[25,[\"sortedSuspendedUsers\"]]],[[\"key\"],[\"id\"]],{\"statements\":[[4,\"gh-user-active\",null,[[\"user\"],[[24,1,[]]]],{\"statements\":[[0,\"                    \"],[1,[29,\"gh-user-list-item\",null,[[\"user\",\"component\"],[[24,1,[]],[24,2,[]]]]],false],[0,\"\\n\"]],\"parameters\":[2]},null]],\"parameters\":[1]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/staff/index.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/staff/user-loading", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "uWexLHAc",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"data-test-staff-link\",\"route\"],[true,\"staff\"]],{\"statements\":[[0,\"Staff\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            \"],[1,[25,[\"user\",\"name\"]],false],[0,\"\\n        \"],[10],[0,\"\\n\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[7,\"div\"],[11,\"class\",\"gh-btn gh-btn-blue\"],[9],[7,\"span\"],[9],[0,\"Save\"],[10],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gh-content\"],[9],[0,\"\\n        \"],[1,[23,\"gh-loading-spinner\"],false],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/staff/user-loading.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/staff/user", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "e128coHW",
    "block": "{\"symbols\":[],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"data-test-staff-link\",\"route\"],[true,\"staff\"]],{\"statements\":[[0,\"Staff\"]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"arrow-right\"],null],false],[10],[0,\"\\n            \"],[1,[25,[\"user\",\"name\"]],false],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"user\",\"isSuspended\"]]],null,{\"statements\":[[0,\"            \"],[7,\"span\"],[11,\"class\",\"gh-badge suspended\"],[11,\"data-test-suspended-badge\",\"\"],[9],[0,\"Suspended\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showLeaveSettingsModal\"]]],null,{\"statements\":[[0,\"            \"],[1,[29,\"gh-fullscreen-modal\",[\"leave-settings\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"leaveSettings\"],null],[29,\"action\",[[24,0,[]],\"toggleLeaveSettingsModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n        \"],[7,\"section\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n\"],[4,\"if\",[[25,[\"userActionsAreVisible\"]]],null,{\"statements\":[[0,\"                \"],[7,\"span\"],[11,\"class\",\"dropdown\"],[9],[0,\"\\n\"],[4,\"gh-dropdown-button\",null,[[\"dropdownName\",\"classNames\",\"title\",\"data-test-user-actions\"],[\"user-actions-menu\",\"gh-btn gh-btn-default gh-btn-icon only-has-icon user-actions-cog\",\"User Actions\",true]],{\"statements\":[[0,\"                        \"],[7,\"span\"],[9],[0,\"\\n                            \"],[1,[29,\"svg-jar\",[\"settings\"],null],false],[0,\"\\n                            \"],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[0,\"User Settings\"],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"gh-dropdown\",null,[[\"name\",\"tagName\",\"classNames\"],[\"user-actions-menu\",\"ul\",\"user-actions-menu dropdown-menu dropdown-triangle-top-right\"]],{\"statements\":[[4,\"if\",[[25,[\"canMakeOwner\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"li\"],[9],[0,\"\\n                                \"],[7,\"button\"],[9],[0,\"\\n                                    Make Owner\\n                                \"],[3,\"action\",[[24,0,[]],\"toggleTransferOwnerModal\"]],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"showTransferOwnerModal\"]]],null,{\"statements\":[[0,\"                                    \"],[1,[29,\"gh-fullscreen-modal\",[\"transfer-owner\"],[[\"confirm\",\"close\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"transferOwnership\"],null],[29,\"action\",[[24,0,[]],\"toggleTransferOwnerModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"                            \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[25,[\"deleteUserActionIsVisible\"]]],null,{\"statements\":[[0,\"                            \"],[7,\"li\"],[9],[0,\"\\n                                \"],[7,\"button\"],[11,\"class\",\"delete\"],[11,\"data-test-delete-button\",\"\"],[9],[0,\"\\n                                    Delete User\\n                                \"],[3,\"action\",[[24,0,[]],\"toggleDeleteUserModal\"]],[10],[0,\"\\n                            \"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"user\",\"isActive\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"li\"],[9],[0,\"\\n                                    \"],[7,\"button\"],[11,\"class\",\"suspend\"],[11,\"data-test-suspend-button\",\"\"],[9],[0,\"\\n                                        Suspend User\\n                                    \"],[3,\"action\",[[24,0,[]],\"toggleSuspendUserModal\"]],[10],[0,\"\\n                                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[4,\"if\",[[25,[\"user\",\"isSuspended\"]]],null,{\"statements\":[[0,\"                                \"],[7,\"li\"],[9],[0,\"\\n                                    \"],[7,\"button\"],[11,\"class\",\"unsuspend\"],[11,\"data-test-unsuspend-button\",\"\"],[9],[0,\"\\n                                        Un-suspend User\\n                                    \"],[3,\"action\",[[24,0,[]],\"toggleUnsuspendUserModal\"]],[10],[0,\"\\n                                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"                \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n            \"],[1,[29,\"gh-task-button\",null,[[\"class\",\"task\",\"data-test-save-button\"],[\"gh-btn gh-btn-blue gh-btn-icon\",[25,[\"save\"]],true]]],false],[0,\"\\n\\n\"],[4,\"if\",[[25,[\"showDeleteUserModal\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-fullscreen-modal\",[\"delete-user\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"user\"]],[29,\"action\",[[24,0,[]],\"deleteUser\"],null],[29,\"action\",[[24,0,[]],\"toggleDeleteUserModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showSuspendUserModal\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-fullscreen-modal\",[\"suspend-user\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"user\"]],[29,\"action\",[[24,0,[]],\"suspendUser\"],null],[29,\"action\",[[24,0,[]],\"toggleSuspendUserModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"showUnsuspendUserModal\"]]],null,{\"statements\":[[0,\"                \"],[1,[29,\"gh-fullscreen-modal\",[\"unsuspend-user\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"user\"]],[29,\"action\",[[24,0,[]],\"unsuspendUser\"],null],[29,\"action\",[[24,0,[]],\"toggleUnsuspendUserModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"div\"],[11,\"class\",\"gm-main view-container settings-user\"],[9],[0,\"\\n        \"],[7,\"form\"],[11,\"class\",\"user-profile\"],[11,\"novalidate\",\"novalidate\"],[11,\"autocomplete\",\"off\"],[9],[0,\"\\n\\n          \"],[7,\"figure\"],[11,\"class\",\"user-cover\"],[12,\"style\",[29,\"background-image-style\",[[25,[\"user\",\"coverImageUrl\"]]],null]],[9],[0,\"\\n              \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-default user-cover-edit\"],[11,\"type\",\"button\"],[9],[7,\"span\"],[9],[0,\"Change Cover\"],[10],[3,\"action\",[[24,0,[]],\"toggleUploadCoverModal\"]],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"showUploadCoverModal\"]]],null,{\"statements\":[[0,\"                  \"],[1,[29,\"gh-fullscreen-modal\",[\"upload-image\"],[[\"model\",\"close\",\"modifier\"],[[29,\"hash\",null,[[\"model\",\"imageProperty\"],[[25,[\"user\"]],\"coverImage\"]]],[29,\"action\",[[24,0,[]],\"toggleUploadCoverModal\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"          \"],[10],[0,\"\\n\\n          \"],[7,\"figure\"],[11,\"class\",\"user-image bg-whitegrey\"],[9],[0,\"\\n              \"],[7,\"div\"],[11,\"id\",\"user-image\"],[11,\"class\",\"img\"],[12,\"style\",[29,\"background-image-style\",[[25,[\"user\",\"profileImageUrl\"]]],null]],[9],[7,\"span\"],[11,\"class\",\"hidden\"],[9],[1,[25,[\"user\",\"name\"]],false],[0,\"\\\"s Picture\"],[10],[10],[0,\"\\n          \"],[10],[0,\"\\n\\n            \"],[7,\"fieldset\"],[11,\"class\",\"user-details-bottom\"],[9],[0,\"\\n\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\",\"class\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"name\",\"first-form-group\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-name\"],[9],[0,\"Full Name\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"id\",\"class\",\"readonly\",\"placeholder\",\"autocorrect\",\"value\",\"input\",\"focus-out\",\"data-test-name-input\"],[\"user-name\",\"user-name\",true,\"Full Name\",\"off\",[29,\"readonly\",[[25,[\"user\",\"name\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"user\",\"name\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"name\"],[[\"target\"],[[25,[\"user\"]]]]],true]]],false],[0,\"\\n\"],[4,\"if\",[[25,[\"user\",\"errors\",\"name\"]]],null,{\"statements\":[[0,\"                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"name\",\"user-name\"]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"p\"],[9],[0,\"Use your real name so people can recognise you\"],[10],[0,\"\\n\"]],\"parameters\":[]}]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"slug\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-slug\"],[9],[0,\"Slug\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"class\",\"id\",\"name\",\"placeholder\",\"readonly\",\"selectOnClick\",\"autocorrect\",\"value\",\"input\",\"focus-out\",\"data-test-slug-input\"],[\"user-name\",\"user-slug\",\"user\",\"Slug\",true,\"true\",\"off\",[29,\"readonly\",[[25,[\"slugValue\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"slugValue\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"updateSlug\"]],[25,[\"slugValue\"]]],null]],null],true]]],false],[0,\"\\n                    \"],[7,\"p\"],[9],[1,[23,\"gh-blog-url\"],false],[0,\"/author/\"],[1,[23,\"slugValue\"],false],[10],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"slug\",\"user-slug\"]]],false],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"email\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-email\"],[9],[0,\"Email\"],[10],[0,\"\\n\"],[4,\"if\",[[25,[\"canChangeEmail\"]]],null,{\"statements\":[[0,\"                        \"],[1,[29,\"gh-text-input\",null,[[\"type\",\"id\",\"name\",\"readonly\",\"placeholder\",\"autocapitalize\",\"autocorrect\",\"autocomplete\",\"value\",\"input\",\"focus-out\",\"data-test-email-input\"],[\"email\",\"user-email\",\"email\",true,\"Email Address\",\"off\",\"off\",\"off\",[29,\"readonly\",[[25,[\"user\",\"email\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"user\",\"email\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"email\"],[[\"target\"],[[25,[\"user\"]]]]],true]]],false],[0,\"\\n                        \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"email\",\"user-email\"]]],false],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"span\"],[9],[1,[25,[\"user\",\"email\"]],false],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                    \"],[7,\"p\"],[9],[0,\"Used for notifications\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"user\",\"isContributor\"]]],null,{\"statements\":[[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"email\"]],{\"statements\":[[0,\"                        \"],[7,\"label\"],[11,\"for\",\"user-parent\"],[9],[0,\"Parent Editor\"],[10],[0,\"\\n                        \"],[7,\"span\"],[9],[1,[25,[\"parentUser\",\"name\"]],false],[10],[0,\"\\n\"]],\"parameters\":[]},null]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"if\",[[25,[\"rolesDropdownIsVisible\"]]],null,{\"statements\":[[0,\"                    \"],[7,\"div\"],[11,\"class\",\"form-group\"],[9],[0,\"\\n                        \"],[7,\"label\"],[11,\"for\",\"user-role\"],[9],[0,\"Role\"],[10],[0,\"\\n                        \"],[7,\"span\"],[11,\"class\",\"gh-select\"],[11,\"tabindex\",\"0\"],[9],[0,\"\\n                            \"],[1,[29,\"one-way-select\",null,[[\"id\",\"options\",\"optionValuePath\",\"optionLabelPath\",\"value\",\"update\"],[\"new-user-role\",[25,[\"roles\"]],\"id\",\"name\",[25,[\"user\",\"role\"]],[29,\"action\",[[24,0,[]],\"changeRole\"],null]]]],false],[0,\"\\n                        \"],[10],[0,\"\\n                        \"],[7,\"p\"],[9],[0,\"What permissions should this user have?\"],[10],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"location\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-location\"],[9],[0,\"Location\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"id\",\"value\",\"input\",\"focus-out\",\"data-test-location-input\"],[\"user-location\",[29,\"readonly\",[[25,[\"user\",\"location\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"user\",\"location\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"location\"],[[\"target\"],[[25,[\"user\"]]]]],true]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"location\",\"user-location\"]]],false],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"Where in the world do you live?\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"website\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-website\"],[9],[0,\"Website\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"type\",\"id\",\"autocapitalize\",\"autocorrect\",\"autocomplete\",\"value\",\"input\",\"focus-out\",\"data-test-website-input\"],[\"url\",\"user-website\",\"off\",\"off\",\"off\",[29,\"readonly\",[[25,[\"user\",\"website\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"user\",\"website\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"website\"],[[\"target\"],[[25,[\"user\"]]]]],true]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"website\",\"user-website\"]]],false],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"Have a website or blog other than this one? Link it!\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"facebook\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-facebook\"],[9],[0,\"Facebook Profile\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"type\",\"placeholder\",\"autocorrect\",\"id\",\"name\",\"value\",\"input\",\"focus-out\",\"data-test-facebook-input\"],[\"url\",\"https://www.facebook.com/username\",\"off\",\"user-facebook\",\"user[facebook]\",[29,\"readonly\",[[25,[\"user\",\"facebook\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"_scratchFacebook\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validateFacebookUrl\"],null],true]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"facebook\",\"user-facebook\"]]],false],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"URL of your personal Facebook Profile\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"twitter\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-twitter\"],[9],[0,\"Twitter Profile\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-text-input\",null,[[\"type\",\"placeholder\",\"autocorrect\",\"id\",\"name\",\"value\",\"input\",\"focus-out\",\"data-test-twitter-input\"],[\"url\",\"https://twitter.com/username\",\"off\",\"user-twitter\",\"user[twitter]\",[29,\"readonly\",[[25,[\"user\",\"twitter\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"_scratchTwitter\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validateTwitterUrl\"],null],true]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"twitter\",\"user-twitter\"]]],false],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"URL of your personal Twitter profile\"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[4,\"gh-form-group\",null,[[\"errors\",\"hasValidated\",\"property\",\"class\"],[[25,[\"user\",\"errors\"]],[25,[\"user\",\"hasValidated\"]],\"bio\",\"bio-container\"]],{\"statements\":[[0,\"                    \"],[7,\"label\"],[11,\"for\",\"user-bio\"],[9],[0,\"Bio\"],[10],[0,\"\\n                    \"],[1,[29,\"gh-textarea\",null,[[\"id\",\"value\",\"input\",\"focus-out\",\"data-test-bio-input\"],[\"user-bio\",[29,\"readonly\",[[25,[\"user\",\"bio\"]]],null],[29,\"action\",[[24,0,[]],[29,\"mut\",[[25,[\"user\",\"bio\"]]],null]],[[\"value\"],[\"target.value\"]]],[29,\"action\",[[24,0,[]],\"validate\",\"bio\"],[[\"target\"],[[25,[\"user\"]]]]],true]]],false],[0,\"\\n                    \"],[1,[29,\"gh-error-message\",null,[[\"errors\",\"property\",\"data-test-error\"],[[25,[\"user\",\"errors\"]],\"bio\",\"user-bio\"]]],false],[0,\"\\n                    \"],[7,\"p\"],[9],[0,\"\\n                        Write about you, in 200 characters or less.\\n                        \"],[1,[29,\"gh-count-characters\",[[25,[\"user\",\"bio\"]]],null],false],[0,\"\\n                    \"],[10],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n                \"],[7,\"hr\"],[9],[10],[0,\"\\n\\n            \"],[10],[0,\"\\n\\n        \"],[3,\"action\",[[24,0,[]],[29,\"perform\",[[25,[\"save\"]]],null]],[[\"on\"],[\"submit\"]]],[10],[0,\" \"],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/staff/user.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/subscribers", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "V+9KxH9i",
    "block": "{\"symbols\":[\"subscriber\"],\"statements\":[[7,\"section\"],[11,\"class\",\"gh-canvas\"],[9],[0,\"\\n    \"],[7,\"header\"],[11,\"class\",\"gh-canvas-header\"],[9],[0,\"\\n        \"],[7,\"h2\"],[11,\"class\",\"gh-canvas-title\"],[11,\"data-test-screen-title\",\"\"],[9],[0,\"\\n            Subscribers\\n            \"],[7,\"span\"],[11,\"style\",\"font-weight:200;margin-left:10px;display:inline-block;\"],[11,\"data-test-total-subscribers\",\"\"],[9],[0,\"\\n\"],[4,\"if\",[[24,0,[\"fetchSubscribers\",\"lastSuccessful\"]]],null,{\"statements\":[[0,\"                    (\"],[1,[24,0,[\"filteredSubscribers\",\"length\"]],false],[0,\")\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                    (Loading...)\\n\"]],\"parameters\":[]}],[0,\"            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"view-actions\"],[9],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"data-test-link\",\"route\"],[\"gh-btn gh-btn-hover-green mr2\",\"import-csv\",\"subscribers.import\"]],{\"statements\":[[7,\"span\"],[9],[0,\"Import CSV\"],[10]],\"parameters\":[]},null],[0,\"\\n            \"],[7,\"a\"],[11,\"href\",\"#\"],[11,\"class\",\"gh-btn gh-btn-hover-blue mr2\"],[9],[7,\"span\"],[9],[0,\"Export CSV\"],[10],[3,\"action\",[[24,0,[]],\"exportData\"]],[10],[0,\"\\n            \"],[4,\"link-to\",null,[[\"class\",\"data-test-link\",\"route\"],[\"gh-btn gh-btn-green\",\"add-subscriber\",\"subscribers.new\"]],{\"statements\":[[7,\"span\"],[9],[0,\"Add Subscriber\"],[10]],\"parameters\":[]},null],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\\n    \"],[7,\"section\"],[11,\"class\",\"view-container\"],[9],[0,\"\\n        \"],[7,\"div\"],[11,\"class\",\"subscribers-table\"],[9],[0,\"\\n            \"],[7,\"table\"],[9],[0,\"\\n                \"],[7,\"thead\"],[9],[0,\"\\n                    \"],[7,\"tr\"],[9],[0,\"\\n                        \"],[7,\"th\"],[11,\"class\",\"gh-subscribers-table-email-cell\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"params\"],[\"inline-flex items-center darkgrey\",[29,\"subscribers-query-params\",[\"email\",[24,0,[\"order\"]],[24,0,[\"direction\"]]],null]]],{\"statements\":[[0,\"                                \"],[1,[29,\"subscribers-sort-icon\",[\"email\",[24,0,[\"order\"]],[24,0,[\"direction\"]]],null],false],[0,\"\\n                                Email Address\\n\"]],\"parameters\":[]},null],[0,\"                        \"],[10],[0,\"\\n                        \"],[7,\"th\"],[11,\"class\",\"gh-subscribers-table-date-cell\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"params\"],[\"inline-flex items-center darkgrey\",[29,\"subscribers-query-params\",[\"created_at\",[24,0,[\"order\"]],[24,0,[\"direction\"]]],null]]],{\"statements\":[[0,\"                                \"],[1,[29,\"subscribers-sort-icon\",[\"created_at\",[24,0,[\"order\"]],[24,0,[\"direction\"]]],null],false],[0,\"\\n                                Subscription Date\\n\"]],\"parameters\":[]},null],[0,\"                        \"],[10],[0,\"\\n                        \"],[7,\"th\"],[11,\"class\",\"gh-subscribers-table-status-cell\"],[9],[0,\"\\n\"],[4,\"link-to\",null,[[\"class\",\"params\"],[\"inline-flex items-center darkgrey\",[29,\"subscribers-query-params\",[\"status\",[24,0,[\"order\"]],[24,0,[\"direction\"]]],null]]],{\"statements\":[[0,\"                                \"],[1,[29,\"subscribers-sort-icon\",[\"status\",[24,0,[\"order\"]],[24,0,[\"direction\"]]],null],false],[0,\"\\n                                Status\\n\"]],\"parameters\":[]},null],[0,\"                        \"],[10],[0,\"\\n                        \"],[7,\"th\"],[11,\"class\",\"gh-subscribers-table-delete-cell\"],[9],[10],[0,\"\\n                    \"],[10],[0,\"\\n                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n            \"],[7,\"table\"],[9],[0,\"\\n                \"],[7,\"tbody\"],[9],[0,\"\\n\"],[4,\"if\",[[24,0,[\"sortedSubscribers\"]]],null,{\"statements\":[[0,\"                        \"],[5,\"vertical-collection\",[],[[\"@items\",\"@key\",\"@containerSelector\",\"@estimateHeight\",\"@bufferSize\"],[[24,0,[\"sortedSubscribers\"]],\"id\",\".gh-main\",\"34\",\"30\"]],{\"statements\":[[0,\"\\n                            \"],[7,\"tr\"],[9],[0,\"\\n                                \"],[7,\"td\"],[11,\"class\",\"gh-subscribers-table-email-cell\"],[9],[0,\"\\n                                    \"],[1,[24,1,[\"email\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"td\"],[11,\"class\",\"gh-subscribers-table-date-cell\"],[9],[0,\"\\n                                    \"],[1,[29,\"moment-format\",[[24,1,[\"createdAtUTC\"]],\"MMMM DD, YYYY\"],null],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"td\"],[11,\"class\",\"gh-subscribers-table-status-cell\"],[9],[0,\"\\n                                    \"],[1,[24,1,[\"status\"]],false],[0,\"\\n                                \"],[10],[0,\"\\n                                \"],[7,\"td\"],[11,\"class\",\"gh-subscribers-table-delete-cell\"],[9],[0,\"\\n                                    \"],[7,\"button\"],[11,\"class\",\"gh-btn gh-btn-link gh-btn-sm\"],[9],[7,\"span\"],[9],[1,[29,\"svg-jar\",[\"trash\"],null],false],[10],[3,\"action\",[[24,0,[]],\"deleteSubscriber\",[24,1,[]]]],[10],[0,\"\\n                                \"],[10],[0,\"\\n                            \"],[10],[0,\"\\n                        \"]],\"parameters\":[1]}],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                        \"],[7,\"tr\"],[9],[0,\"\\n                            \"],[7,\"td\"],[11,\"colspan\",\"4\"],[9],[0,\"\\n\"],[4,\"if\",[[24,0,[\"fetchSubscribers\",\"isRunning\"]]],null,{\"statements\":[[0,\"                                    \"],[7,\"div\"],[11,\"class\",\"relative h50\"],[9],[5,\"gh-loading-spinner\",[],[[],[]]],[10],[0,\"\\n\"]],\"parameters\":[]},{\"statements\":[[0,\"                                    \"],[7,\"span\"],[11,\"class\",\"dib\"],[11,\"style\",\"line-height: 33px\"],[9],[0,\"No subscribers found.\"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                            \"],[10],[0,\"\\n                        \"],[10],[0,\"\\n\"]],\"parameters\":[]}],[0,\"                \"],[10],[0,\"\\n            \"],[10],[0,\"\\n        \"],[10],[0,\"\\n    \"],[10],[0,\"\\n\"],[10],[0,\"\\n\\n\"],[4,\"if\",[[24,0,[\"subscriberToDelete\"]]],null,{\"statements\":[[0,\"    \"],[5,\"gh-fullscreen-modal\",[],[[\"@modal\",\"@model\",\"@confirm\",\"@close\",\"@modifier\"],[\"delete-subscriber\",[24,0,[\"subscriberToDelete\"]],[29,\"action\",[[24,0,[]],\"confirmDeleteSubscriber\"],null],[29,\"action\",[[24,0,[]],\"cancelDeleteSubscriber\"],null],\"action wide\"]]],[0,\"\\n\"]],\"parameters\":[]},null],[0,\"\\n\"],[1,[23,\"outlet\"],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/subscribers.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/subscribers/import", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "aqXEbcPi",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-fullscreen-modal\",[\"import-subscribers\"],[[\"close\",\"confirm\",\"modifier\"],[[29,\"action\",[[24,0,[]],\"close\"],null],[29,\"action\",[[24,0,[]],\"fetchNewSubscribers\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/subscribers/import.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/templates/subscribers/new", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.HTMLBars.template({
    "id": "5I21Buxq",
    "block": "{\"symbols\":[],\"statements\":[[1,[29,\"gh-fullscreen-modal\",[\"new-subscriber\"],[[\"model\",\"confirm\",\"close\",\"modifier\"],[[25,[\"subscriber\"]],[29,\"route-action\",[\"save\"],null],[29,\"route-action\",[\"cancel\"],null],\"action wide\"]]],false],[0,\"\\n\"]],\"hasEval\":false}",
    "meta": {
      "moduleName": "ghost-admin/templates/subscribers/new.hbs"
    }
  });

  _exports.default = _default;
});
;define("ghost-admin/tests/mirage/mirage.lint-test", [], function () {
  "use strict";

  describe('ESLint | mirage', function () {
    it('mirage/config.js', function () {// test passed
    });
    it('mirage/config/api-keys.js', function () {// test passed
    });
    it('mirage/config/authentication.js', function () {// test passed
    });
    it('mirage/config/config.js', function () {// test passed
    });
    it('mirage/config/integrations.js', function () {// test passed
    });
    it('mirage/config/members.js', function () {// test passed
    });
    it('mirage/config/pages.js', function () {// test passed
    });
    it('mirage/config/posts.js', function () {// test passed
    });
    it('mirage/config/roles.js', function () {// test passed
    });
    it('mirage/config/settings.js', function () {// test passed
    });
    it('mirage/config/site.js', function () {// test passed
    });
    it('mirage/config/slugs.js', function () {// test passed
    });
    it('mirage/config/subscribers.js', function () {// test passed
    });
    it('mirage/config/tags.js', function () {// test passed
    });
    it('mirage/config/themes.js', function () {// test passed
    });
    it('mirage/config/uploads.js', function () {// test passed
    });
    it('mirage/config/users.js', function () {// test passed
    });
    it('mirage/config/webhooks.js', function () {// test passed
    });
    it('mirage/factories/api-key.js', function () {// test passed
    });
    it('mirage/factories/integration.js', function () {// test passed
    });
    it('mirage/factories/invite.js', function () {// test passed
    });
    it('mirage/factories/member.js', function () {// test passed
    });
    it('mirage/factories/notification.js', function () {// test passed
    });
    it('mirage/factories/post.js', function () {// test passed
    });
    it('mirage/factories/role.js', function () {// test passed
    });
    it('mirage/factories/subscriber.js', function () {// test passed
    });
    it('mirage/factories/tag.js', function () {// test passed
    });
    it('mirage/factories/user.js', function () {// test passed
    });
    it('mirage/factories/webhook.js', function () {// test passed
    });
    it('mirage/fixtures/configs.js', function () {// test passed
    });
    it('mirage/fixtures/roles.js', function () {// test passed
    });
    it('mirage/fixtures/settings.js', function () {// test passed
    });
    it('mirage/fixtures/sites.js', function () {// test passed
    });
    it('mirage/fixtures/themes.js', function () {// test passed
    });
    it('mirage/fixtures/timezones.js', function () {// test passed
    });
    it('mirage/models/api-key.js', function () {// test passed
    });
    it('mirage/models/config.js', function () {// test passed
    });
    it('mirage/models/integration.js', function () {// test passed
    });
    it('mirage/models/invite.js', function () {// test passed
    });
    it('mirage/models/member.js', function () {// test passed
    });
    it('mirage/models/notification.js', function () {// test passed
    });
    it('mirage/models/page.js', function () {// test passed
    });
    it('mirage/models/post.js', function () {// test passed
    });
    it('mirage/models/role.js', function () {// test passed
    });
    it('mirage/models/site.js', function () {// test passed
    });
    it('mirage/models/subscriber.js', function () {// test passed
    });
    it('mirage/models/tag.js', function () {// test passed
    });
    it('mirage/models/theme.js', function () {// test passed
    });
    it('mirage/models/user.js', function () {// test passed
    });
    it('mirage/models/webhook.js', function () {// test passed
    });
    it('mirage/scenarios/default.js', function () {// test passed
    });
    it('mirage/serializers/application.js', function () {// test passed
    });
    it('mirage/serializers/integration.js', function () {// test passed
    });
    it('mirage/serializers/page.js', function () {// test passed
    });
    it('mirage/serializers/post.js', function () {// test passed
    });
    it('mirage/serializers/tag.js', function () {// test passed
    });
    it('mirage/serializers/user.js', function () {// test passed
    });
    it('mirage/utils.js', function () {// test passed
    });
  });
});
;define("ghost-admin/transforms/facebook-url-user", ["exports", "ember-data/transform"], function (_exports, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      if (serialized) {
        let [, user] = serialized.match(/(\S+)/);
        return "https://www.facebook.com/".concat(user);
      }

      return serialized;
    },

    serialize(deserialized) {
      if (deserialized) {
        let [, user] = deserialized.match(/(?:https:\/\/)(?:www\.)(?:facebook\.com)\/(?:#!\/)?(\w+\/?\S+)/mi);
        return user;
      }

      return deserialized;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/json-string", ["exports", "ember-data/transform"], function (_exports, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      let _serialized = serialized === '' ? null : serialized;

      return JSON.parse(_serialized);
    },

    serialize(deserialized) {
      return deserialized ? JSON.stringify(deserialized) : null;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/member-subscription", ["exports", "ghost-admin/models/member-subscription", "ember-data/transform"], function (_exports, _memberSubscription, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      let subscriptions, subscriptionArray;
      subscriptionArray = serialized || [];
      subscriptions = subscriptionArray.map(itemDetails => _memberSubscription.default.create(itemDetails));
      return Ember.A(subscriptions);
    },

    serialize(deserialized) {
      let subscriptionArray;

      if (Ember.isArray(deserialized)) {
        subscriptionArray = deserialized.map(item => {
          let adapter = item.get('adapter').trim();
          let amount = item.get('amount');
          let plan = item.get('plan').trim();
          let status = item.get('status').trim();
          let validUntil = item.get('validUntil');
          return {
            adapter,
            amount,
            plan,
            status,
            validUntil
          };
        }).compact();
      } else {
        subscriptionArray = [];
      }

      return subscriptionArray;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/moment-date", ["exports", "ember-data/transform", "moment"], function (_exports, _transform, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      if (serialized) {
        return (0, _moment.default)(serialized);
      }

      return serialized;
    },

    serialize(deserialized) {
      if (deserialized) {
        return (0, _moment.default)(deserialized).toDate();
      }

      return deserialized;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/moment-utc", ["exports", "ember-data/transform", "moment"], function (_exports, _transform, _moment) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      if (serialized) {
        return _moment.default.utc(serialized);
      }

      return serialized;
    },

    serialize(deserialized) {
      if (deserialized) {
        try {
          return deserialized.toJSON();
        } catch (e) {
          return deserialized;
        }
      }

      return deserialized;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/navigation-settings", ["exports", "ghost-admin/models/navigation-item", "ember-data/transform"], function (_exports, _navigationItem, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      let navItems, settingsArray;

      try {
        settingsArray = JSON.parse(serialized) || [];
      } catch (e) {
        settingsArray = [];
      }

      navItems = settingsArray.map(itemDetails => _navigationItem.default.create(itemDetails));
      return Ember.A(navItems);
    },

    serialize(deserialized) {
      let settingsArray;

      if (Ember.isArray(deserialized)) {
        settingsArray = deserialized.map(item => {
          let label = item.get('label').trim();
          let url = item.get('url').trim();
          return {
            label,
            url
          };
        }).compact();
      } else {
        settingsArray = [];
      }

      return JSON.stringify(settingsArray);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/raw", ["exports", "ember-data/transform"], function (_exports, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      return serialized;
    },

    serialize(deserialized) {
      return deserialized;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/slack-settings", ["exports", "ghost-admin/models/slack-integration", "ember-data/transform"], function (_exports, _slackIntegration, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  var _default = _transform.default.extend({
    deserialize(serialized) {
      let settingsArray;

      try {
        settingsArray = JSON.parse(serialized) || [];
      } catch (e) {
        settingsArray = [];
      }

      if (Ember.isEmpty(settingsArray)) {
        settingsArray.push({
          url: '',
          username: ''
        });
      }

      let slackObjs = settingsArray.map(itemDetails => _slackIntegration.default.create(itemDetails));
      return slackObjs;
    },

    serialize(deserialized) {
      let settingsArray;

      if (Ember.isArray(deserialized)) {
        settingsArray = deserialized.map(item => {
          let url = (item.get('url') || '').trim();
          let username = (item.get('username') || '').trim();

          if (url) {
            return {
              url,
              username
            };
          }
        }).compact();
      } else {
        settingsArray = [];
      }

      return JSON.stringify(settingsArray);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/twitter-url-user", ["exports", "ember-data/transform"], function (_exports, _transform) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _transform.default.extend({
    deserialize(serialized) {
      if (serialized) {
        let [, user] = serialized.match(/@?([^/]*)/);
        return "https://twitter.com/".concat(user);
      }

      return serialized;
    },

    serialize(deserialized) {
      if (deserialized) {
        let [, user] = deserialized.match(/(?:https:\/\/)(?:twitter\.com)\/(?:#!\/)?@?([^/]*)/);
        return "@".concat(user);
      }

      return deserialized;
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transforms/unsplash-settings", ["exports", "ember-data/transform", "ghost-admin/models/unsplash-integration"], function (_exports, _transform, _unsplashIntegration) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /* eslint-disable camelcase */
  const DEFAULT_SETTINGS = {
    isActive: true
  };

  var _default = _transform.default.extend({
    deserialize(serialized) {
      if (serialized) {
        let settingsObject;

        try {
          settingsObject = JSON.parse(serialized) || DEFAULT_SETTINGS;
        } catch (e) {
          settingsObject = DEFAULT_SETTINGS;
        }

        return _unsplashIntegration.default.create(settingsObject);
      }

      return DEFAULT_SETTINGS;
    },

    serialize(deserialized) {
      return deserialized ? JSON.stringify(deserialized) : JSON.stringify(DEFAULT_SETTINGS);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/transitions", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  function _default() {
    this.transition(this.hasClass('fullscreen-modal-container'), this.toValue(true), this.use('fade', {
      duration: 150
    }), this.reverse('fade', {
      duration: 150
    }));
    this.transition(this.hasClass('fade-transition'), this.use('crossFade', {
      duration: 100
    }));
    this.transition(this.hasClass('tour-container'), this.toValue(true), this.use('fade', {
      duration: 150
    }), this.reverse('fade', {
      duration: 150
    }));
    this.transition(this.hasClass('tour'), this.toValue(true), this.use('fade', {
      duration: 300
    }), this.reverse('fade', {
      duration: 300
    })); // TODO: Maybe animate with explode. gh-unsplash-window should ideally slide in from bottom to top of screen
    // this.transition(
    //     this.hasClass('gh-unsplash-window'),
    //     this.toValue(true),
    //     this.use('toUp', {duration: 500}),
    //     this.reverse('toDown', {duration: 500})
    // );
  }
});
;define("ghost-admin/transitions/cross-fade", ["exports", "liquid-fire/transitions/cross-fade"], function (_exports, _crossFade) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _crossFade.default;
    }
  });
});
;define("ghost-admin/transitions/default", ["exports", "liquid-fire/transitions/default"], function (_exports, _default) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _default.default;
    }
  });
});
;define("ghost-admin/transitions/explode", ["exports", "liquid-fire/transitions/explode"], function (_exports, _explode) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _explode.default;
    }
  });
});
;define("ghost-admin/transitions/fade", ["exports", "liquid-fire/transitions/fade"], function (_exports, _fade) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _fade.default;
    }
  });
});
;define("ghost-admin/transitions/flex-grow", ["exports", "liquid-fire/transitions/flex-grow"], function (_exports, _flexGrow) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _flexGrow.default;
    }
  });
});
;define("ghost-admin/transitions/fly-to", ["exports", "liquid-fire/transitions/fly-to"], function (_exports, _flyTo) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _flyTo.default;
    }
  });
});
;define("ghost-admin/transitions/move-over", ["exports", "liquid-fire/transitions/move-over"], function (_exports, _moveOver) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _moveOver.default;
    }
  });
});
;define("ghost-admin/transitions/scale", ["exports", "liquid-fire/transitions/scale"], function (_exports, _scale) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _scale.default;
    }
  });
});
;define("ghost-admin/transitions/scroll-then", ["exports", "liquid-fire/transitions/scroll-then"], function (_exports, _scrollThen) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _scrollThen.default;
    }
  });
});
;define("ghost-admin/transitions/to-down", ["exports", "liquid-fire/transitions/to-down"], function (_exports, _toDown) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _toDown.default;
    }
  });
});
;define("ghost-admin/transitions/to-left", ["exports", "liquid-fire/transitions/to-left"], function (_exports, _toLeft) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _toLeft.default;
    }
  });
});
;define("ghost-admin/transitions/to-right", ["exports", "liquid-fire/transitions/to-right"], function (_exports, _toRight) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _toRight.default;
    }
  });
});
;define("ghost-admin/transitions/to-up", ["exports", "liquid-fire/transitions/to-up"], function (_exports, _toUp) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _toUp.default;
    }
  });
});
;define("ghost-admin/transitions/wait", ["exports", "liquid-fire/transitions/wait"], function (_exports, _wait) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  Object.defineProperty(_exports, "default", {
    enumerable: true,
    get: function get() {
      return _wait.default;
    }
  });
});
;define("ghost-admin/transitions/wormhole", ["exports", "liquid-wormhole/transitions/wormhole"], function (_exports, _wormhole) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  // override liquid-wormhole's default `wormhole` transition to focus the first
  // autofocus element after the transition and element swap happens
  function _default() {
    let newWormholeElement;

    if (this.newElement) {
      newWormholeElement = this.newElement.find('.liquid-wormhole-element:last-child');
    }

    return _wormhole.default.apply(this, arguments).finally(() => {
      if (this.newElement && newWormholeElement) {
        let autofocusElem = newWormholeElement[0].querySelector('[autofocus]');

        if (autofocusElem) {
          autofocusElem.focus();
        }
      }
    });
  }
});
;define("ghost-admin/utils/bound-one-way", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  /**
   * Defines a property similarly to `Ember.computed.oneway`,
   * save that while a `oneway` loses its binding upon being set,
   * the `BoundOneWay` will continue to listen for upstream changes.
   *
   * This is an ideal tool for working with values inside of {{input}}
   * elements.
   * @param {*} upstream
   * @param {function} transform a function to transform the **upstream** value.
   */
  function _default(upstream, transform) {
    if (typeof transform !== 'function') {
      // default to the identity function
      transform = function transform(value) {
        return value;
      };
    }

    return Ember.computed(upstream, {
      get() {
        return transform(this.get(upstream));
      },

      set(key, value) {
        return value;
      }

    });
  }
});
;define("ghost-admin/utils/caja-sanitizers", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * google-caja uses url() and id() to verify if the values are allowed.
   */

  /**
   * Check if URL is allowed
   * URLs are allowed if they start with http://, https://, or /.
   * NOTE: # urls are not allowed as clicking them will break the editor when clicked
   */
  let url = function url(_url) {
    _url = _url.toString().replace(/['"]+/g, '');

    if (/^https?:\/\//.test(_url) || /^\//.test(_url)) {
      return _url;
    }
  };
  /**
   * Check if ID is allowed
   * All ids are allowed at the moment.
   */


  let id = function id(_id) {
    return _id;
  };

  var _default = {
    url,
    id
  };
  _exports.default = _default;
});
;define("ghost-admin/utils/copy-text-to-clipboard", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = copyTextToClipboard;

  function copyTextToClipboard(text) {
    let textarea = document.createElement('textarea');
    textarea.value = text;
    textarea.setAttribute('readonly', '');
    textarea.style.position = 'absolute';
    textarea.style.left = '-9999px';
    document.body.appendChild(textarea);
    textarea.select();
    document.execCommand('copy');
    document.body.removeChild(textarea);
  }
});
;define("ghost-admin/utils/ctrl-or-cmd", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = navigator.userAgent.indexOf('Mac') !== -1 ? 'command' : 'ctrl';

  _exports.default = _default;
});
;define("ghost-admin/utils/format-markdown", ["exports", "markdown-it", "markdown-it-footnote", "markdown-it-lazy-headers", "markdown-it-mark", "koenig-editor/helpers/sanitize-html"], function (_exports, _markdownIt, _markdownItFootnote, _markdownItLazyHeaders, _markdownItMark, _sanitizeHtml) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = formatMarkdown;

  let slugify = function slugify(inputString, usedHeaders) {
    let slug = inputString.replace(/[^\w]/g, '').toLowerCase();

    if (usedHeaders[slug]) {
      usedHeaders[slug] += 1;
      slug += usedHeaders[slug];
    }

    return slug;
  }; // originally from https://github.com/leff/markdown-it-named-headers
  // moved here to avoid pulling in http://stringjs.com dependency


  let markdownitNamedHeaders = function markdownitNamedHeaders(md) {
    let originalHeadingOpen = md.renderer.rules.heading_open; // eslint-disable-next-line camelcase

    md.renderer.rules.heading_open = function (tokens, idx, options, env, self) {
      let usedHeaders = {};
      tokens[idx].attrs = tokens[idx].attrs || [];
      let title = tokens[idx + 1].children.reduce(function (acc, t) {
        return acc + t.content;
      }, '');
      let slug = slugify(title, usedHeaders);
      tokens[idx].attrs.push(['id', slug]);

      if (originalHeadingOpen) {
        return originalHeadingOpen.apply(this, arguments);
      } else {
        return self.renderToken(...arguments);
      }
    };
  };

  let md = (0, _markdownIt.default)({
    html: true,
    breaks: true,
    linkify: true
  }).use(_markdownItFootnote.default).use(_markdownItLazyHeaders.default).use(_markdownItMark.default).use(markdownitNamedHeaders); // configure linkify-it

  md.linkify.set({
    fuzzyLink: false
  });

  function formatMarkdown(_markdown) {
    let replaceJS = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    let markdown = _markdown || '';
    let escapedhtml = ''; // convert markdown to HTML

    escapedhtml = md.render(markdown);
    return (0, _sanitizeHtml.sanitizeHtml)(escapedhtml, {
      replaceJS
    });
  }
});
;define("ghost-admin/utils/ghost-paths", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  let makeRoute = function makeRoute(root, args) {
    let slashAtStart = /^\//;
    let slashAtEnd = /\/$/;
    let parts = Array.prototype.slice.call(args, 0);
    let route = root.replace(slashAtEnd, '');
    parts.forEach(part => {
      if (part) {
        route = [route, part.replace(slashAtStart, '').replace(slashAtEnd, '')].join('/');
      }
    });
    return route += '/';
  };

  function _default() {
    let path = window.location.pathname;
    let subdir = path.substr(0, path.search('/ghost/'));
    let adminRoot = "".concat(subdir, "/ghost/");
    let assetRoot = "".concat(subdir, "/ghost/assets/");
    let apiRoot = "".concat(subdir, "/ghost/api/v2/admin");

    function assetUrl(src) {
      return subdir + src;
    }

    return {
      adminRoot,
      assetRoot,
      apiRoot,
      subdir,
      blogRoot: "".concat(subdir, "/"),
      count: 'https://count.ghost.org/',
      url: {
        admin() {
          return makeRoute(adminRoot, arguments);
        },

        api() {
          return makeRoute(apiRoot, arguments);
        },

        join() {
          if (arguments.length > 1) {
            return makeRoute(arguments[0], Array.prototype.slice.call(arguments, 1));
          } else if (arguments.length === 1) {
            let [arg] = arguments;
            return arg.slice(-1) === '/' ? arg : "".concat(arg, "/");
          }

          return '/';
        },

        asset: assetUrl
      }
    };
  }
});
;define("ghost-admin/utils/isFinite", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  /* globals window */
  // isFinite function from lodash
  function _default(value) {
    return window.isFinite(value) && !window.isNaN(parseFloat(value));
  }
});
;define("ghost-admin/utils/isNumber", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;
  // isNumber function from lodash
  const {
    toString
  } = Object.prototype;

  function _default(value) {
    return typeof value === 'number' || value && typeof value === 'object' && toString.call(value) === '[object Number]' || false;
  }
});
;define("ghost-admin/utils/link-component", [], function () {
  "use strict";

  Ember.LinkComponent.reopen({
    active: Ember.computed('attrs.params', '_routing.currentState', function () {
      let isActive = this._super(...arguments);

      if (typeof this.alternateActive === 'function') {
        this.alternateActive(isActive);
      }

      return isActive;
    }),
    activeClass: Ember.computed('tagName', function () {
      return this.tagName === 'button' ? '' : 'active';
    })
  });
});
;define("ghost-admin/utils/random-password", ["exports", "password-generator"], function (_exports, _passwordGenerator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;

  function _default() {
    let word = (0, _passwordGenerator.default)(6);
    let randomN = Math.floor(Math.random() * 1000);
    return word + randomN;
  }
});
;define("ghost-admin/utils/route", [], function () {
  "use strict";

  Ember.Route.reopen({
    actions: {
      willTransition(transition) {
        if (this.get('upgradeStatus.isRequired')) {
          transition.abort();
          this.upgradeStatus.requireUpgrade();
          return false;
        } else {
          return true;
        }
      }

    }
  });
});
;define("ghost-admin/utils/titleize", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = _default;
  const lowerWords = ['of', 'a', 'the', 'and', 'an', 'or', 'nor', 'but', 'is', 'if', 'then', 'else', 'when', 'at', 'from', 'by', 'on', 'off', 'for', 'in', 'out', 'over', 'to', 'into', 'with'];

  function _default(input) {
    let words = input.split(' ').map((word, index) => {
      if (index === 0 || lowerWords.indexOf(word) === -1) {
        word = Ember.String.capitalize(word);
      }

      return word;
    });
    return words.join(' ');
  }
});
;define("ghost-admin/utils/window-proxy", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  var _default = {
    changeLocation(url) {
      window.location = url;
    },

    replaceLocation(url) {
      window.location.replace(url);
    },

    replaceState(params, title, url) {
      window.history.replaceState(params, title, url);
    }

  };
  _exports.default = _default;
});
;define("ghost-admin/validators/base", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  /**
   * Base validator that all validators should extend
   * Handles checking of individual properties or the entire model
   */
  var _default = Ember.Object.extend({
    passed: false,

    init() {
      this._super(...arguments);

      this.properties = this.properties || [];
    },

    /**
     * When passed a model and (optionally) a property name,
     * checks it against a list of validation functions
     * @param  {Ember.Object} model Model to validate
     * @param  {string} prop  Property name to check
     * @return {boolean}      True if the model passed all (or one) validation(s),
     *                        false if not
     */
    check(model, prop) {
      this.set('passed', true);

      if (prop && this[prop]) {
        this[prop](model);
      } else {
        this.properties.forEach(property => {
          if (this[property]) {
            this[property](model);
          }
        });
      }

      return this.passed;
    },

    invalidate() {
      this.set('passed', false);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/integration", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['name'],

    name(model) {
      if (Ember.isBlank(model.name)) {
        model.errors.add('name', 'Please enter a name');
        model.hasValidated.pushObject('name');
        this.invalidate();
      } else if (!_validator.default.isLength(model.name, 0, 191)) {
        model.errors.add('name', 'Name is too long, max 191 chars');
        model.hasValidated.pushObject('name');
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/invite-user", ["exports", "ghost-admin/validators/base"], function (_exports, _base) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['username', 'role'],

    username(model) {
      let username = model.get('username');

      if (Ember.isBlank(username)) {
        model.get('errors').add('username', 'Please enter a username.');
        this.invalidate();
      }
    },

    role(model) {
      let role = model.get('role');

      if (Ember.isBlank(role)) {
        model.get('errors').add('role', 'Please select a role.');
        model.get('hasValidated').pushObject('role');
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/mixins/password", ["exports", "validator"], function (_exports, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;
  const BAD_PASSWORDS = ['1234567890', 'qwertyuiop', 'qwertzuiop', 'asdfghjkl;', 'abcdefghij', '0987654321', '1q2w3e4r5t', '12345asdfg'];
  const DISALLOWED_PASSWORDS = ['ghost', 'password', 'passw0rd'];

  var _default = Ember.Mixin.create({
    /**
    * Counts repeated characters if a string. When 50% or more characters are the same,
    * we return false and therefore invalidate the string.
    * @param {String} stringToTest The password string to check.
    * @return {Boolean}
    */
    _characterOccurance(stringToTest) {
      let chars = {};
      let allowedOccurancy;
      let valid = true;
      allowedOccurancy = stringToTest.length / 2; // Loop through string and accumulate character counts

      for (let i = 0; i < stringToTest.length; i += 1) {
        if (!chars[stringToTest[i]]) {
          chars[stringToTest[i]] = 1;
        } else {
          chars[stringToTest[i]] += 1;
        }
      } // check if any of the accumulated chars exceed the allowed occurancy
      // of 50% of the words' length.


      for (let charCount in chars) {
        if (chars[charCount] >= allowedOccurancy) {
          valid = false;
          return valid;
        }
      }

      return valid;
    },

    passwordValidation(model, password, errorTarget) {
      let blogUrl = model.get('config.blogUrl') || window.location.host;
      let blogTitle = model.get('blogTitle') || model.get('config.blogTitle');
      let blogUrlWithSlash; // the password that needs to be validated can differ from the password in the
      // passed model, e. g. for password changes or reset.

      password = password || model.get('password');
      errorTarget = errorTarget || 'password';
      blogUrl = blogUrl.replace(/^http(s?):\/\//, '');
      blogUrlWithSlash = blogUrl.match(/\/$/) ? blogUrl : "".concat(blogUrl, "/");
      blogTitle = blogTitle ? blogTitle.trim().toLowerCase() : blogTitle; // password must be longer than 10 characters

      if (!_validator.default.isLength(password || '', 10)) {
        model.get('errors').add(errorTarget, 'Password must be at least 10 characters long');
        return this.invalidate();
      }

      password = password.toString(); // dissallow password from badPasswords list (e. g. '1234567890')

      BAD_PASSWORDS.map(badPassword => {
        if (badPassword === password) {
          model.get('errors').add(errorTarget, 'Sorry, you cannot use an insecure password');
          this.invalidate();
        }
      }); // password must not match with users' email

      if (password.toLowerCase() === model.get('email').toLowerCase()) {
        model.get('errors').add(errorTarget, 'Sorry, you cannot use an insecure password');
        this.invalidate();
      } // password must not contain the words 'ghost', 'password', or 'passw0rd'


      DISALLOWED_PASSWORDS.map(disallowedPassword => {
        if (password.toLowerCase().indexOf(disallowedPassword) >= 0) {
          model.get('errors').add(errorTarget, 'Sorry, you cannot use an insecure password');
          this.invalidate();
        }
      }); // password must not match with blog title

      if (password.toLowerCase() === blogTitle) {
        model.get('errors').add(errorTarget, 'Sorry, you cannot use an insecure password');
        this.invalidate();
      } // password must not match with blog URL (without protocol, with or without trailing slash)


      if (password.toLowerCase() === blogUrl || password.toLowerCase() === blogUrlWithSlash) {
        model.get('errors').add(errorTarget, 'Sorry, you cannot use an insecure password');
        this.invalidate();
      } // dissallow passwords where 50% or more of characters are the same


      if (!this._characterOccurance(password)) {
        model.get('errors').add(errorTarget, 'Sorry, you cannot use an insecure password');
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/nav-item", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['label', 'url'],

    label(model) {
      let label = model.get('label');
      let hasValidated = model.get('hasValidated');

      if (Ember.isBlank(label)) {
        model.get('errors').add('label', 'You must specify a label');
        this.invalidate();
      }

      hasValidated.addObject('label');
    },

    url(model) {
      let url = model.get('url');
      let hasValidated = model.get('hasValidated');
      /* eslint-disable camelcase */

      let validatorOptions = {
        require_protocol: true
      };
      /* eslint-enable camelcase */

      let urlRegex = new RegExp(/^(\/|#|[a-zA-Z0-9-]+:)/);

      if (Ember.isBlank(url)) {
        model.get('errors').add('url', 'You must specify a URL or relative path');
        this.invalidate();
      } else if (url.match(/\s/) || !_validator.default.isURL(url, validatorOptions) && !url.match(urlRegex)) {
        model.get('errors').add('url', 'You must specify a valid URL or relative path');
        this.invalidate();
      }

      hasValidated.addObject('url');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/new-user", ["exports", "ghost-admin/validators/base", "ghost-admin/validators/mixins/password", "validator"], function (_exports, _base, _password, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.extend(_password.default, {
    init() {
      this.properties = this.properties || ['name', 'email', 'password'];

      this._super(...arguments);
    },

    name(model) {
      let name = model.get('name');

      if (!_validator.default.isLength(name || '', 1)) {
        model.get('errors').add('name', 'Please enter a name.');
        model.get('hasValidated').addObject('email');
        this.invalidate();
      }
    },

    email(model) {
      let email = model.get('email');

      if (Ember.isBlank(email)) {
        model.get('errors').add('email', 'Please enter an email.');
        this.invalidate();
      } else if (!_validator.default.isEmail(email)) {
        model.get('errors').add('email', 'Invalid Email.');
        this.invalidate();
      }

      model.get('hasValidated').addObject('email');
    },

    password(model) {
      this.passwordValidation(model);
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/post", ["exports", "ghost-admin/validators/base", "moment", "validator"], function (_exports, _base, _moment, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['title', 'authors', 'roomName', 'customExcerpt', 'canonicalUrl', 'codeinjectionHead', 'codeinjectionFoot', 'metaTitle', 'metaDescription', 'ogtitle', 'ogDescription', 'twitterTitle', 'twitterDescription', 'rcTitle', 'rcDescription', 'publishedAtBlogTime', 'publishedAtBlogDate'],

    title(model) {
      if (Ember.isBlank(model.title)) {
        model.errors.add('title', 'You must specify a title for the post.');
        this.invalidate();
      }

      if (!_validator.default.isLength(model.title || '', 0, 255)) {
        model.errors.add('title', 'Title cannot be longer than 255 characters.');
        this.invalidate();
      }
    },

    authors(model) {
      if (Ember.isEmpty(model.authors)) {
        model.errors.add('authors', 'At least one author is required.');
        this.invalidate();
      }
    },

    canonicalUrl(model) {
      let validatorOptions = {
        require_protocol: true
      };
      let urlRegex = new RegExp(/^(\/|[a-zA-Z0-9-]+:)/);
      let url = model.canonicalUrl;

      if (Ember.isBlank(url)) {
        return;
      }

      if (url.match(/\s/) || !_validator.default.isURL(url, validatorOptions) && !url.match(urlRegex)) {
        model.errors.add('canonicalUrl', 'Please enter a valid URL');
        this.invalidate();
      } else if (!_validator.default.isLength(model.canonicalUrl, 0, 2000)) {
        model.errors.add('canonicalUrl', 'Canonical URL is too long, max 2000 chars');
        this.invalidate();
      }
    },

    customExcerpt(model) {
      if (!_validator.default.isLength(model.customExcerpt || '', 0, 300)) {
        model.errors.add('customExcerpt', 'Excerpt cannot be longer than 300 characters.');
        this.invalidate();
      }
    },

    codeinjectionFoot(model) {
      if (!_validator.default.isLength(model.codeinjectionFoot || '', 0, 65535)) {
        model.errors.add('codeinjectionFoot', 'Footer code cannot be longer than 65535 characters.');
        this.invalidate();
      }
    },

    codeinjectionHead(model) {
      if (!_validator.default.isLength(model.codeinjectionHead || '', 0, 65535)) {
        model.errors.add('codeinjectionHead', 'Header code cannot be longer than 65535 characters.');
        this.invalidate();
      }
    },

    metaTitle(model) {
      if (!_validator.default.isLength(model.metaTitle || '', 0, 300)) {
        model.errors.add('metaTitle', 'Meta Title cannot be longer than 300 characters.');
        this.invalidate();
      }
    },

    metaDescription(model) {
      if (!_validator.default.isLength(model.metaDescription || '', 0, 500)) {
        model.errors.add('metaDescription', 'Meta Description cannot be longer than 500 characters.');
        this.invalidate();
      }
    },

    ogTitle(model) {
      if (!_validator.default.isLength(model.ogTitle || '', 0, 300)) {
        model.errors.add('ogTitle', 'Facebook Title cannot be longer than 300 characters.');
        this.invalidate();
      }
    },

    ogDescription(model) {
      if (!_validator.default.isLength(model.ogDescription || '', 0, 500)) {
        model.errors.add('ogDescription', 'Facebook Description cannot be longer than 500 characters.');
        this.invalidate();
      }
    },

    twitterTitle(model) {
      if (!_validator.default.isLength(model.twitterTitle || '', 0, 300)) {
        model.errors.add('twitterTitle', 'Twitter Title cannot be longer than 300 characters.');
        this.invalidate();
      }
    },

    twitterDescription(model) {
      if (!_validator.default.isLength(model.twitterDescription || '', 0, 500)) {
        model.errors.add('twitterDescription', 'Twitter Description cannot be longer than 500 characters.');
        this.invalidate();
      }
    },

    rcTitle(model) {
      if (!_validator.default.isLength(model.rcTitle || '', 0, 300)) {
        model.errors.add('rcTitle', 'Message Title cannot be longer than 300 characters.');
        this.invalidate();
      }
    },

    rcDescription(model) {
      if (!_validator.default.isLength(model.rcDescription || '', 0, 500)) {
        model.errors.add('rcDescription', 'Message Description cannot be longer than 500 characters.');
        this.invalidate();
      }
    },

    // for posts which haven't been published before and where the blog date/time
    // is blank we should ignore the validation
    _shouldValidatePublishedAtBlog(model) {
      return Ember.isPresent(model.publishedAtUTC) || Ember.isPresent(model.publishedAtBlogDate) || Ember.isPresent(model.publishedAtBlogTime);
    },

    // convenience method as .validate({property: 'x'}) doesn't accept multiple properties
    publishedAtBlog(model) {
      this.publishedAtBlogTime(model);
      this.publishedAtBlogDate(model);
    },

    publishedAtBlogTime(model) {
      let timeRegex = /^(([0-1]?[0-9])|([2][0-3])):([0-5][0-9])$/;

      if (!timeRegex.test(model.publishedAtBlogTime) && this._shouldValidatePublishedAtBlog(model)) {
        model.errors.add('publishedAtBlogTime', 'Must be in format: "15:00"');
        this.invalidate();
      }
    },

    publishedAtBlogDate(model) {
      let publishedAtBlogDate = model.publishedAtBlogDate;
      let publishedAtBlogTime = model.publishedAtBlogTime;

      if (!this._shouldValidatePublishedAtBlog(model)) {
        return;
      } // we have a time string but no date string


      if (Ember.isBlank(publishedAtBlogDate) && !Ember.isBlank(publishedAtBlogTime)) {
        model.errors.add('publishedAtBlogDate', 'Can\'t be blank');
        return this.invalidate();
      } // don't validate the date if the time format is incorrect


      if (Ember.isEmpty(model.errors.errorsFor('publishedAtBlogTime'))) {
        let status = model.statusScratch || model.status;
        let now = (0, _moment.default)();
        let publishedAtUTC = model.publishedAtUTC;
        let publishedAtBlogTZ = model.publishedAtBlogTZ;
        let matchesExisting = publishedAtUTC && publishedAtBlogTZ.isSame(publishedAtUTC);
        let isInFuture = publishedAtBlogTZ.isSameOrAfter(now.add(2, 'minutes')); // draft/published must be in past

        if ((status === 'draft' || status === 'published') && publishedAtBlogTZ.isSameOrAfter(now)) {
          model.errors.add('publishedAtBlogDate', 'Must be in the past');
          this.invalidate(); // scheduled must be at least 2 mins in the future
          // ignore if it matches publishedAtUTC as that is likely an update of a scheduled post
        } else if (status === 'scheduled' && !matchesExisting && !isInFuture) {
          model.errors.add('publishedAtBlogDate', 'Must be at least 2 mins in the future');
          this.invalidate();
        }
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/setting", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['title', 'description', 'password'],

    title(model) {
      let title = model.get('title');

      if (!_validator.default.isLength(title || '', 0, 150)) {
        model.get('errors').add('title', 'Title is too long');
        this.invalidate();
      }
    },

    description(model) {
      let desc = model.get('description');

      if (!_validator.default.isLength(desc || '', 0, 200)) {
        model.get('errors').add('description', 'Description is too long');
        this.invalidate();
      }
    },

    password(model) {
      let isPrivate = model.get('isPrivate');
      let password = model.get('password');

      if (isPrivate && password === '') {
        model.get('errors').add('password', 'Password must be supplied');
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/slack-integration", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['url'],

    url(model) {
      let url = model.get('url');
      let hasValidated = model.get('hasValidated'); // eslint-disable-next-line camelcase

      if (!Ember.isBlank(url) && !_validator.default.isURL(url, {
        require_protocol: true
      })) {
        model.get('errors').add('url', 'The URL must be in a format like https://hooks.slack.com/services/<your personal key>');
        this.invalidate();
      }

      hasValidated.addObject('url');
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/subscriber", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['email'],

    email(model) {
      let email = model.get('email');

      if (Ember.isBlank(email)) {
        model.get('errors').add('email', 'Please enter an email.');
        model.get('hasValidated').pushObject('email');
        this.invalidate();
      } else if (!_validator.default.isEmail(email)) {
        model.get('errors').add('email', 'Invalid email.');
        model.get('hasValidated').pushObject('email');
        this.invalidate();
      } else if (!_validator.default.isLength(email, 0, 191)) {
        model.get('errors').add('email', 'Email is too long');
        model.get('hasValidated').pushObject('email');
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/tag-settings", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['name', 'slug', 'description', 'metaTitle', 'metaDescription'],

    name(model) {
      let name = model.get('name');

      if (Ember.isBlank(name)) {
        model.get('errors').add('name', 'You must specify a name for the tag.');
        this.invalidate();
      } else if (name.match(/^,/)) {
        model.get('errors').add('name', 'Tag names can\'t start with commas.');
        this.invalidate();
      } else if (!_validator.default.isLength(name, 0, 191)) {
        model.get('errors').add('name', 'Tag names cannot be longer than 191 characters.');
        this.invalidate();
      }
    },

    slug(model) {
      let slug = model.get('slug');

      if (!_validator.default.isLength(slug || '', 0, 191)) {
        model.get('errors').add('slug', 'URL cannot be longer than 191 characters.');
        this.invalidate();
      }
    },

    description(model) {
      let description = model.get('description');

      if (!_validator.default.isLength(description || '', 0, 500)) {
        model.get('errors').add('description', 'Description cannot be longer than 500 characters.');
        this.invalidate();
      }
    },

    metaTitle(model) {
      let metaTitle = model.get('metaTitle');

      if (!_validator.default.isLength(metaTitle || '', 0, 300)) {
        model.get('errors').add('metaTitle', 'Meta Title cannot be longer than 300 characters.');
        this.invalidate();
      }
    },

    metaDescription(model) {
      let metaDescription = model.get('metaDescription');

      if (!_validator.default.isLength(metaDescription || '', 0, 500)) {
        model.get('errors').add('metaDescription', 'Meta Description cannot be longer than 500 characters.');
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/validators/user", ["exports", "ghost-admin/validators/base", "ghost-admin/validators/mixins/password", "validator"], function (_exports, _base, _password, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  const userValidator = _base.default.extend(_password.default, {
    init() {
      this.properties = this.properties || ['name', 'bio', 'email', 'location', 'website', 'roles'];

      this._super(...arguments);
    },

    isActive(model) {
      return model.get('status') === 'active';
    },

    name(model) {
      let name = model.get('name');

      if (this.isActive(model)) {
        if (Ember.isBlank(name)) {
          model.get('errors').add('name', 'Please enter a name.');
          this.invalidate();
        } else if (!_validator.default.isLength(name, 0, 191)) {
          model.get('errors').add('name', 'Name is too long');
          this.invalidate();
        }
      }
    },

    bio(model) {
      let bio = model.get('bio');

      if (this.isActive(model)) {
        if (!_validator.default.isLength(bio || '', 0, 200)) {
          model.get('errors').add('bio', 'Bio is too long');
          this.invalidate();
        }
      }
    },

    email(model) {
      let email = model.get('email');

      if (!_validator.default.isEmail(email || '')) {
        model.get('errors').add('email', 'Please supply a valid email address');
        this.invalidate();
      }

      if (!_validator.default.isLength(email || '', 0, 191)) {
        model.get('errors').add('email', 'Email is too long');
        this.invalidate();
      }
    },

    location(model) {
      let location = model.get('location');

      if (this.isActive(model)) {
        if (!_validator.default.isLength(location || '', 0, 150)) {
          model.get('errors').add('location', 'Location is too long');
          this.invalidate();
        }
      }
    },

    website(model) {
      let website = model.get('website'); // eslint-disable-next-line camelcase

      let isInvalidWebsite = !_validator.default.isURL(website || '', {
        require_protocol: false
      }) || !_validator.default.isLength(website || '', 0, 2000);

      if (this.isActive(model)) {
        if (!Ember.isBlank(website) && isInvalidWebsite) {
          model.get('errors').add('website', 'Website is not a valid url');
          this.invalidate();
        }
      }
    },

    roles(model) {
      if (!this.isActive(model)) {
        let roles = model.get('roles');

        if (roles.length < 1) {
          model.get('errors').add('role', 'Please select a role');
          this.invalidate();
        }
      }
    },

    passwordChange(model) {
      let newPassword = model.get('newPassword');
      let ne2Password = model.get('ne2Password'); // validation only marks the requested property as validated so we
      // have to add properties manually

      model.get('hasValidated').addObject('newPassword');
      model.get('hasValidated').addObject('ne2Password');

      if (Ember.isBlank(newPassword) && Ember.isBlank(ne2Password)) {
        model.get('errors').add('newPassword', 'Sorry, passwords can\'t be blank');
        this.invalidate();
      } else {
        if (!_validator.default.equals(newPassword, ne2Password || '')) {
          model.get('errors').add('ne2Password', 'Your new passwords do not match');
          this.invalidate();
        }

        this.passwordValidation(model, newPassword, 'newPassword');
      }
    },

    ownPasswordChange(model) {
      let oldPassword = model.get('password');
      this.passwordChange(model); // validation only marks the requested property as validated so we
      // have to add properties manually

      model.get('hasValidated').addObject('password');

      if (Ember.isBlank(oldPassword)) {
        model.get('errors').add('password', 'Your current password is required to set a new one');
        this.invalidate();
      }
    }

  });

  var _default = userValidator.create();

  _exports.default = _default;
});
;define("ghost-admin/validators/webhook", ["exports", "ghost-admin/validators/base", "validator"], function (_exports, _base, _validator) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = _base.default.create({
    properties: ['name', 'event', 'targetUrl'],

    name(model) {
      if (Ember.isBlank(model.name)) {
        model.errors.add('name', 'Please enter a name');
        model.hasValidated.pushObject('name');
        this.invalidate();
      } else if (!_validator.default.isLength(model.name, 0, 191)) {
        model.errors.add('name', 'Name is too long, max 191 chars');
        model.hasValidated.pushObject('name');
        this.invalidate();
      }
    },

    event(model) {
      if (Ember.isBlank(model.event)) {
        model.errors.add('event', 'Please select an event');
        model.hasValidated.pushObject('event');
        this.invalidate();
      }
    },

    targetUrl(model) {
      if (Ember.isBlank(model.targetUrl)) {
        model.errors.add('targetUrl', 'Please enter a target URL');
      } else if (!_validator.default.isURL(model.targetUrl || '', {
        require_protocol: false
      })) {
        model.errors.add('targetUrl', 'Please enter a valid URL');
      } else if (!_validator.default.isLength(model.targetUrl, 0, 2000)) {
        model.errors.add('targetUrl', 'Target URL is too long, max 2000 chars');
      }

      model.hasValidated.pushObject('targetUrl');

      if (model.errors.has('targetUrl')) {
        this.invalidate();
      }
    }

  });

  _exports.default = _default;
});
;define("ghost-admin/views/application", ["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _default = Ember.Component.extend({});

  _exports.default = _default;
});
;

;define('ghost-admin/config/environment', [], function() {
  var prefix = 'ghost-admin';
try {
  var metaName = prefix + '/config/environment';
  var rawConfig = document.querySelector('meta[name="' + metaName + '"]').getAttribute('content');
  var config = JSON.parse(decodeURIComponent(rawConfig));

  var exports = { 'default': config };

  Object.defineProperty(exports, '__esModule', { value: true });

  return exports;
}
catch(err) {
  throw new Error('Could not read config from meta tag with name "' + metaName + '".');
}

});

;
          if (!runningTests) {
            require("ghost-admin/app")["default"].create({"version":"2.23","LOG_ACTIVE_GENERATION":true,"LOG_TRANSITIONS":true,"LOG_TRANSITIONS_INTERNAL":true,"LOG_VIEW_LOOKUPS":true,"name":"ghost-admin"});
          }
        
//# sourceMappingURL=ghost.map
